###########################################################################
#
#  Copyright 2020 Google LLC
#
#  Licensed under the Apache License, Version 2.0 (the "License"  "");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
###########################################################################

"""Object used to generate BigQuery table for mapping US geography.

This object in table form allows joins and expansion of gographic
regions within the US. Unfortunately some Postal Codes map to multiple
DMAs in this implementation.

See: https://blog.cubitplanning.com/2015/05/current-census-demographics-by-dma/

"""

US_GEOGRAPHY_SCHEMA = [
  {"name":"Postal_Code", "type":"INTEGER", "mode":"REQUIRED"},
  {"name":"FIPS", "type":"STRING", "mode":"REQUIRED"},
  {"name":"County", "type":"STRING", "mode":"REQUIRED"},
  {"name":"State", "type":"STRING", "mode":"REQUIRED"},
  {"name":"DMA", "type":"RECORD", "mode":"REQUIRED", "fields":[
    {"name":"Code", "type":"INTEGER", "mode":"NULLABLE"},
    {"name":"Name", "type":"STRING", "mode":"NULLABLE"}
  ]}
]

US_GEOGRAPHY_DATA = [
  {"Postal_Code": "36003", "FIPS": "01001", "County": "Autauga", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36006", "FIPS": "01001", "County": "Autauga", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36008", "FIPS": "01001", "County": "Autauga", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36051", "FIPS": "01001", "County": "Autauga", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36066", "FIPS": "01001", "County": "Autauga", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36067", "FIPS": "01001", "County": "Autauga", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36068", "FIPS": "01001", "County": "Autauga", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36749", "FIPS": "01001", "County": "Autauga", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36507", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36511", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36526", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36527", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36530", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36532", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36533", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36535", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36536", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36542", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36547", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36549", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36550", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36551", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36555", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36559", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36561", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36562", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36564", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36567", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36574", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36576", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36577", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36578", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36579", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36580", "FIPS": "01003", "County": "Baldwin", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36016", "FIPS": "01005", "County": "Barbour", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36017", "FIPS": "01005", "County": "Barbour", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36027", "FIPS": "01005", "County": "Barbour", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36048", "FIPS": "01005", "County": "Barbour", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36053", "FIPS": "01005", "County": "Barbour", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36072", "FIPS": "01005", "County": "Barbour", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "35034", "FIPS": "01007", "County": "Bibb", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35035", "FIPS": "01007", "County": "Bibb", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35042", "FIPS": "01007", "County": "Bibb", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35074", "FIPS": "01007", "County": "Bibb", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35184", "FIPS": "01007", "County": "Bibb", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35188", "FIPS": "01007", "County": "Bibb", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36792", "FIPS": "01007", "County": "Bibb", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36793", "FIPS": "01007", "County": "Bibb", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35013", "FIPS": "01009", "County": "Blount", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35031", "FIPS": "01009", "County": "Blount", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35049", "FIPS": "01009", "County": "Blount", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35079", "FIPS": "01009", "County": "Blount", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35097", "FIPS": "01009", "County": "Blount", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35121", "FIPS": "01009", "County": "Blount", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35133", "FIPS": "01009", "County": "Blount", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35172", "FIPS": "01009", "County": "Blount", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36029", "FIPS": "01011", "County": "Bullock", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36061", "FIPS": "01011", "County": "Bullock", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36089", "FIPS": "01011", "County": "Bullock", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36015", "FIPS": "01013", "County": "Butler", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36030", "FIPS": "01013", "County": "Butler", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36033", "FIPS": "01013", "County": "Butler", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36037", "FIPS": "01013", "County": "Butler", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36456", "FIPS": "01013", "County": "Butler", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36201", "FIPS": "01015", "County": "Calhoun", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36202", "FIPS": "01015", "County": "Calhoun", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36203", "FIPS": "01015", "County": "Calhoun", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36204", "FIPS": "01015", "County": "Calhoun", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36205", "FIPS": "01015", "County": "Calhoun", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36206", "FIPS": "01015", "County": "Calhoun", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36207", "FIPS": "01015", "County": "Calhoun", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36210", "FIPS": "01015", "County": "Calhoun", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36250", "FIPS": "01015", "County": "Calhoun", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36253", "FIPS": "01015", "County": "Calhoun", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36254", "FIPS": "01015", "County": "Calhoun", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36257", "FIPS": "01015", "County": "Calhoun", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36260", "FIPS": "01015", "County": "Calhoun", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36265", "FIPS": "01015", "County": "Calhoun", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36271", "FIPS": "01015", "County": "Calhoun", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36272", "FIPS": "01015", "County": "Calhoun", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36277", "FIPS": "01015", "County": "Calhoun", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36279", "FIPS": "01015", "County": "Calhoun", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36854", "FIPS": "01017", "County": "Chambers", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36855", "FIPS": "01017", "County": "Chambers", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36862", "FIPS": "01017", "County": "Chambers", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36863", "FIPS": "01017", "County": "Chambers", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "35959", "FIPS": "01019", "County": "Cherokee", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35960", "FIPS": "01019", "County": "Cherokee", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35973", "FIPS": "01019", "County": "Cherokee", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35983", "FIPS": "01019", "County": "Cherokee", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36275", "FIPS": "01019", "County": "Cherokee", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35045", "FIPS": "01021", "County": "Chilton", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35046", "FIPS": "01021", "County": "Chilton", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35085", "FIPS": "01021", "County": "Chilton", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35171", "FIPS": "01021", "County": "Chilton", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36091", "FIPS": "01021", "County": "Chilton", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36750", "FIPS": "01021", "County": "Chilton", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36790", "FIPS": "01021", "County": "Chilton", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36904", "FIPS": "01023", "County": "Choctaw", "State": "AL", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "36908", "FIPS": "01023", "County": "Choctaw", "State": "AL", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "36910", "FIPS": "01023", "County": "Choctaw", "State": "AL", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "36912", "FIPS": "01023", "County": "Choctaw", "State": "AL", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "36913", "FIPS": "01023", "County": "Choctaw", "State": "AL", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "36915", "FIPS": "01023", "County": "Choctaw", "State": "AL", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "36916", "FIPS": "01023", "County": "Choctaw", "State": "AL", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "36919", "FIPS": "01023", "County": "Choctaw", "State": "AL", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "36921", "FIPS": "01023", "County": "Choctaw", "State": "AL", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "36922", "FIPS": "01023", "County": "Choctaw", "State": "AL", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "36436", "FIPS": "01025", "County": "Clarke", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36446", "FIPS": "01025", "County": "Clarke", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36451", "FIPS": "01025", "County": "Clarke", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36482", "FIPS": "01025", "County": "Clarke", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36501", "FIPS": "01025", "County": "Clarke", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36515", "FIPS": "01025", "County": "Clarke", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36524", "FIPS": "01025", "County": "Clarke", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36540", "FIPS": "01025", "County": "Clarke", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36545", "FIPS": "01025", "County": "Clarke", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36727", "FIPS": "01025", "County": "Clarke", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36762", "FIPS": "01025", "County": "Clarke", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36784", "FIPS": "01025", "County": "Clarke", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "35082", "FIPS": "01027", "County": "Clay", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36251", "FIPS": "01027", "County": "Clay", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36255", "FIPS": "01027", "County": "Clay", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36258", "FIPS": "01027", "County": "Clay", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36266", "FIPS": "01027", "County": "Clay", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36267", "FIPS": "01027", "County": "Clay", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36261", "FIPS": "01029", "County": "Cleburne", "State": "AL", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "36262", "FIPS": "01029", "County": "Cleburne", "State": "AL", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "36264", "FIPS": "01029", "County": "Cleburne", "State": "AL", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "36269", "FIPS": "01029", "County": "Cleburne", "State": "AL", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "36273", "FIPS": "01029", "County": "Cleburne", "State": "AL", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "36323", "FIPS": "01031", "County": "Coffee", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36330", "FIPS": "01031", "County": "Coffee", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36331", "FIPS": "01031", "County": "Coffee", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36346", "FIPS": "01031", "County": "Coffee", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36351", "FIPS": "01031", "County": "Coffee", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36453", "FIPS": "01031", "County": "Coffee", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "35616", "FIPS": "01033", "County": "Colbert", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35646", "FIPS": "01033", "County": "Colbert", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35660", "FIPS": "01033", "County": "Colbert", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35661", "FIPS": "01033", "County": "Colbert", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35662", "FIPS": "01033", "County": "Colbert", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35674", "FIPS": "01033", "County": "Colbert", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "36401", "FIPS": "01035", "County": "Conecuh", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36429", "FIPS": "01035", "County": "Conecuh", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36432", "FIPS": "01035", "County": "Conecuh", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36454", "FIPS": "01035", "County": "Conecuh", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36473", "FIPS": "01035", "County": "Conecuh", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36475", "FIPS": "01035", "County": "Conecuh", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "35072", "FIPS": "01037", "County": "Coosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35089", "FIPS": "01037", "County": "Coosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35136", "FIPS": "01037", "County": "Coosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35183", "FIPS": "01037", "County": "Coosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36026", "FIPS": "01037", "County": "Coosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36038", "FIPS": "01039", "County": "Covington", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36420", "FIPS": "01039", "County": "Covington", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36421", "FIPS": "01039", "County": "Covington", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36442", "FIPS": "01039", "County": "Covington", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36455", "FIPS": "01039", "County": "Covington", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36467", "FIPS": "01039", "County": "Covington", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36474", "FIPS": "01039", "County": "Covington", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36476", "FIPS": "01039", "County": "Covington", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36483", "FIPS": "01039", "County": "Covington", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36009", "FIPS": "01041", "County": "Crenshaw", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36028", "FIPS": "01041", "County": "Crenshaw", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36034", "FIPS": "01041", "County": "Crenshaw", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36041", "FIPS": "01041", "County": "Crenshaw", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36042", "FIPS": "01041", "County": "Crenshaw", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36049", "FIPS": "01041", "County": "Crenshaw", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36062", "FIPS": "01041", "County": "Crenshaw", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36071", "FIPS": "01041", "County": "Crenshaw", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "35019", "FIPS": "01043", "County": "Cullman", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35033", "FIPS": "01043", "County": "Cullman", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35053", "FIPS": "01043", "County": "Cullman", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35055", "FIPS": "01043", "County": "Cullman", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35056", "FIPS": "01043", "County": "Cullman", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35057", "FIPS": "01043", "County": "Cullman", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35058", "FIPS": "01043", "County": "Cullman", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35070", "FIPS": "01043", "County": "Cullman", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35077", "FIPS": "01043", "County": "Cullman", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35083", "FIPS": "01043", "County": "Cullman", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35087", "FIPS": "01043", "County": "Cullman", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35098", "FIPS": "01043", "County": "Cullman", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35179", "FIPS": "01043", "County": "Cullman", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36311", "FIPS": "01045", "County": "Dale", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36322", "FIPS": "01045", "County": "Dale", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36350", "FIPS": "01045", "County": "Dale", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36352", "FIPS": "01045", "County": "Dale", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36360", "FIPS": "01045", "County": "Dale", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36361", "FIPS": "01045", "County": "Dale", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36362", "FIPS": "01045", "County": "Dale", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36371", "FIPS": "01045", "County": "Dale", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36374", "FIPS": "01045", "County": "Dale", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36701", "FIPS": "01047", "County": "Dallas", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36702", "FIPS": "01047", "County": "Dallas", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36703", "FIPS": "01047", "County": "Dallas", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36758", "FIPS": "01047", "County": "Dallas", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36759", "FIPS": "01047", "County": "Dallas", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36761", "FIPS": "01047", "County": "Dallas", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36767", "FIPS": "01047", "County": "Dallas", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36773", "FIPS": "01047", "County": "Dallas", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36775", "FIPS": "01047", "County": "Dallas", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "35961", "FIPS": "01049", "County": "De Kalb", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35962", "FIPS": "01049", "County": "De Kalb", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35963", "FIPS": "01049", "County": "De Kalb", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35967", "FIPS": "01049", "County": "De Kalb", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35968", "FIPS": "01049", "County": "De Kalb", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35971", "FIPS": "01049", "County": "De Kalb", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35974", "FIPS": "01049", "County": "De Kalb", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35975", "FIPS": "01049", "County": "De Kalb", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35978", "FIPS": "01049", "County": "De Kalb", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35981", "FIPS": "01049", "County": "De Kalb", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35984", "FIPS": "01049", "County": "De Kalb", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35986", "FIPS": "01049", "County": "De Kalb", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35988", "FIPS": "01049", "County": "De Kalb", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35989", "FIPS": "01049", "County": "De Kalb", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "36020", "FIPS": "01051", "County": "Elmore", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36022", "FIPS": "01051", "County": "Elmore", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36024", "FIPS": "01051", "County": "Elmore", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36025", "FIPS": "01051", "County": "Elmore", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36045", "FIPS": "01051", "County": "Elmore", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36054", "FIPS": "01051", "County": "Elmore", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36078", "FIPS": "01051", "County": "Elmore", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36080", "FIPS": "01051", "County": "Elmore", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36092", "FIPS": "01051", "County": "Elmore", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36093", "FIPS": "01051", "County": "Elmore", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36426", "FIPS": "01053", "County": "Escambia", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36427", "FIPS": "01053", "County": "Escambia", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36441", "FIPS": "01053", "County": "Escambia", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36502", "FIPS": "01053", "County": "Escambia", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36503", "FIPS": "01053", "County": "Escambia", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36504", "FIPS": "01053", "County": "Escambia", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36543", "FIPS": "01053", "County": "Escambia", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "35901", "FIPS": "01055", "County": "Etowah", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35902", "FIPS": "01055", "County": "Etowah", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35903", "FIPS": "01055", "County": "Etowah", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35904", "FIPS": "01055", "County": "Etowah", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35905", "FIPS": "01055", "County": "Etowah", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35906", "FIPS": "01055", "County": "Etowah", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35907", "FIPS": "01055", "County": "Etowah", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35952", "FIPS": "01055", "County": "Etowah", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35954", "FIPS": "01055", "County": "Etowah", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35956", "FIPS": "01055", "County": "Etowah", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35972", "FIPS": "01055", "County": "Etowah", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35990", "FIPS": "01055", "County": "Etowah", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35542", "FIPS": "01057", "County": "Fayette", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35545", "FIPS": "01057", "County": "Fayette", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35546", "FIPS": "01057", "County": "Fayette", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35555", "FIPS": "01057", "County": "Fayette", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35559", "FIPS": "01057", "County": "Fayette", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35571", "FIPS": "01059", "County": "Franklin", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35581", "FIPS": "01059", "County": "Franklin", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35582", "FIPS": "01059", "County": "Franklin", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35585", "FIPS": "01059", "County": "Franklin", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35593", "FIPS": "01059", "County": "Franklin", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35653", "FIPS": "01059", "County": "Franklin", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35654", "FIPS": "01059", "County": "Franklin", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "36313", "FIPS": "01061", "County": "Geneva", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36314", "FIPS": "01061", "County": "Geneva", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36316", "FIPS": "01061", "County": "Geneva", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36318", "FIPS": "01061", "County": "Geneva", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36340", "FIPS": "01061", "County": "Geneva", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36344", "FIPS": "01061", "County": "Geneva", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36349", "FIPS": "01061", "County": "Geneva", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36375", "FIPS": "01061", "County": "Geneva", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36477", "FIPS": "01061", "County": "Geneva", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "35443", "FIPS": "01063", "County": "Greene", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35448", "FIPS": "01063", "County": "Greene", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35462", "FIPS": "01063", "County": "Greene", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35469", "FIPS": "01063", "County": "Greene", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35491", "FIPS": "01063", "County": "Greene", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36740", "FIPS": "01063", "County": "Greene", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35441", "FIPS": "01065", "County": "Hale", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35474", "FIPS": "01065", "County": "Hale", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36744", "FIPS": "01065", "County": "Hale", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36765", "FIPS": "01065", "County": "Hale", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36776", "FIPS": "01065", "County": "Hale", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36310", "FIPS": "01067", "County": "Henry", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36317", "FIPS": "01067", "County": "Henry", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36345", "FIPS": "01067", "County": "Henry", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36353", "FIPS": "01067", "County": "Henry", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36373", "FIPS": "01067", "County": "Henry", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36301", "FIPS": "01069", "County": "Houston", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36302", "FIPS": "01069", "County": "Houston", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36303", "FIPS": "01069", "County": "Houston", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36304", "FIPS": "01069", "County": "Houston", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36305", "FIPS": "01069", "County": "Houston", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36312", "FIPS": "01069", "County": "Houston", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36319", "FIPS": "01069", "County": "Houston", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36320", "FIPS": "01069", "County": "Houston", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36321", "FIPS": "01069", "County": "Houston", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36343", "FIPS": "01069", "County": "Houston", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36370", "FIPS": "01069", "County": "Houston", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "36376", "FIPS": "01069", "County": "Houston", "State": "AL", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "35740", "FIPS": "01071", "County": "Jackson", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35744", "FIPS": "01071", "County": "Jackson", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35745", "FIPS": "01071", "County": "Jackson", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35746", "FIPS": "01071", "County": "Jackson", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35751", "FIPS": "01071", "County": "Jackson", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35752", "FIPS": "01071", "County": "Jackson", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35755", "FIPS": "01071", "County": "Jackson", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35764", "FIPS": "01071", "County": "Jackson", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35765", "FIPS": "01071", "County": "Jackson", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35766", "FIPS": "01071", "County": "Jackson", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35768", "FIPS": "01071", "County": "Jackson", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35769", "FIPS": "01071", "County": "Jackson", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35771", "FIPS": "01071", "County": "Jackson", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35772", "FIPS": "01071", "County": "Jackson", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35774", "FIPS": "01071", "County": "Jackson", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35776", "FIPS": "01071", "County": "Jackson", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35958", "FIPS": "01071", "County": "Jackson", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35966", "FIPS": "01071", "County": "Jackson", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35979", "FIPS": "01071", "County": "Jackson", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35005", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35006", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35015", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35020", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35021", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35022", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35023", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35036", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35041", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35048", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35060", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35061", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35062", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35064", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35068", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35071", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35073", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35091", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35094", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35111", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35116", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35117", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35118", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35119", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35123", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35126", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35127", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35139", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35142", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35173", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35180", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35181", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35201", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35202", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35203", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35204", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35205", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35206", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35207", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35208", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35209", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35210", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35211", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35212", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35213", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35214", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35215", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35216", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35217", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35218", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35219", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35220", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35221", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35222", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35223", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35224", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35225", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35226", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35228", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35229", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35230", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35231", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35232", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35233", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35234", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35235", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35236", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35237", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35238", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35240", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35243", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35244", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35245", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35246", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35249", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35253", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35254", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35255", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35259", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35260", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35261", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35263", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35266", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35277", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35278", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35279", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35280", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35281", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35282", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35283", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35285", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35286", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35287", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35288", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35289", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35290", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35291", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35292", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35293", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35294", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35295", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35296", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35297", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35298", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35299", "FIPS": "01073", "County": "Jefferson", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35544", "FIPS": "01075", "County": "Lamar", "State": "AL", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "35552", "FIPS": "01075", "County": "Lamar", "State": "AL", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "35574", "FIPS": "01075", "County": "Lamar", "State": "AL", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "35576", "FIPS": "01075", "County": "Lamar", "State": "AL", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "35586", "FIPS": "01075", "County": "Lamar", "State": "AL", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "35592", "FIPS": "01075", "County": "Lamar", "State": "AL", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "35610", "FIPS": "01077", "County": "Lauderdale", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35617", "FIPS": "01077", "County": "Lauderdale", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35630", "FIPS": "01077", "County": "Lauderdale", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35631", "FIPS": "01077", "County": "Lauderdale", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35632", "FIPS": "01077", "County": "Lauderdale", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35633", "FIPS": "01077", "County": "Lauderdale", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35634", "FIPS": "01077", "County": "Lauderdale", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35645", "FIPS": "01077", "County": "Lauderdale", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35648", "FIPS": "01077", "County": "Lauderdale", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35652", "FIPS": "01077", "County": "Lauderdale", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35677", "FIPS": "01077", "County": "Lauderdale", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35618", "FIPS": "01079", "County": "Lawrence", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35643", "FIPS": "01079", "County": "Lawrence", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35650", "FIPS": "01079", "County": "Lawrence", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35651", "FIPS": "01079", "County": "Lawrence", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35672", "FIPS": "01079", "County": "Lawrence", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "36801", "FIPS": "01081", "County": "Lee", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36802", "FIPS": "01081", "County": "Lee", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36803", "FIPS": "01081", "County": "Lee", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36804", "FIPS": "01081", "County": "Lee", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36830", "FIPS": "01081", "County": "Lee", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36831", "FIPS": "01081", "County": "Lee", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36832", "FIPS": "01081", "County": "Lee", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36849", "FIPS": "01081", "County": "Lee", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36852", "FIPS": "01081", "County": "Lee", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36865", "FIPS": "01081", "County": "Lee", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36870", "FIPS": "01081", "County": "Lee", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36872", "FIPS": "01081", "County": "Lee", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36874", "FIPS": "01081", "County": "Lee", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36877", "FIPS": "01081", "County": "Lee", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36879", "FIPS": "01081", "County": "Lee", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "35611", "FIPS": "01083", "County": "Limestone", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35612", "FIPS": "01083", "County": "Limestone", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35613", "FIPS": "01083", "County": "Limestone", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35614", "FIPS": "01083", "County": "Limestone", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35615", "FIPS": "01083", "County": "Limestone", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35620", "FIPS": "01083", "County": "Limestone", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35647", "FIPS": "01083", "County": "Limestone", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35649", "FIPS": "01083", "County": "Limestone", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35671", "FIPS": "01083", "County": "Limestone", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35739", "FIPS": "01083", "County": "Limestone", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35742", "FIPS": "01083", "County": "Limestone", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35756", "FIPS": "01083", "County": "Limestone", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "36032", "FIPS": "01085", "County": "Lowndes", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36040", "FIPS": "01085", "County": "Lowndes", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36047", "FIPS": "01085", "County": "Lowndes", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36752", "FIPS": "01085", "County": "Lowndes", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36785", "FIPS": "01085", "County": "Lowndes", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36031", "FIPS": "01087", "County": "Macon", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36039", "FIPS": "01087", "County": "Macon", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36075", "FIPS": "01087", "County": "Macon", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36083", "FIPS": "01087", "County": "Macon", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36087", "FIPS": "01087", "County": "Macon", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36088", "FIPS": "01087", "County": "Macon", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36866", "FIPS": "01087", "County": "Macon", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "35741", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35748", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35749", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35750", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35757", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35758", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35759", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35760", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35761", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35762", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35763", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35767", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35773", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35801", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35802", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35803", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35804", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35805", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35806", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35807", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35808", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35809", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35810", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35811", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35812", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35813", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35814", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35815", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35816", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35824", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35893", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35894", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35895", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35896", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35897", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35898", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35899", "FIPS": "01089", "County": "Madison", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "36732", "FIPS": "01091", "County": "Marengo", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36736", "FIPS": "01091", "County": "Marengo", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36738", "FIPS": "01091", "County": "Marengo", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36742", "FIPS": "01091", "County": "Marengo", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36745", "FIPS": "01091", "County": "Marengo", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36748", "FIPS": "01091", "County": "Marengo", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36754", "FIPS": "01091", "County": "Marengo", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36763", "FIPS": "01091", "County": "Marengo", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36764", "FIPS": "01091", "County": "Marengo", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36782", "FIPS": "01091", "County": "Marengo", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36783", "FIPS": "01091", "County": "Marengo", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "35543", "FIPS": "01093", "County": "Marion", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35548", "FIPS": "01093", "County": "Marion", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35563", "FIPS": "01093", "County": "Marion", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35564", "FIPS": "01093", "County": "Marion", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35570", "FIPS": "01093", "County": "Marion", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35594", "FIPS": "01093", "County": "Marion", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35016", "FIPS": "01095", "County": "Marshall", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35175", "FIPS": "01095", "County": "Marshall", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35747", "FIPS": "01095", "County": "Marshall", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35950", "FIPS": "01095", "County": "Marshall", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35951", "FIPS": "01095", "County": "Marshall", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35957", "FIPS": "01095", "County": "Marshall", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35964", "FIPS": "01095", "County": "Marshall", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35976", "FIPS": "01095", "County": "Marshall", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35980", "FIPS": "01095", "County": "Marshall", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "36505", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36509", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36512", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36521", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36522", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36523", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36525", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36528", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36541", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36544", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36560", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36568", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36571", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36572", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36575", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36582", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36587", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36590", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36601", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36602", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36603", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36604", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36605", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36606", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36607", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36608", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36609", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36610", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36611", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36612", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36613", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36615", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36616", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36617", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36618", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36619", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36621", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36622", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36625", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36628", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36630", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36633", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36640", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36641", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36644", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36652", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36660", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36663", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36670", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36671", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36675", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36685", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36688", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36689", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36690", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36691", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36693", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36695", "FIPS": "01097", "County": "Mobile", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36425", "FIPS": "01099", "County": "Monroe", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36439", "FIPS": "01099", "County": "Monroe", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36444", "FIPS": "01099", "County": "Monroe", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36445", "FIPS": "01099", "County": "Monroe", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36449", "FIPS": "01099", "County": "Monroe", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36457", "FIPS": "01099", "County": "Monroe", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36458", "FIPS": "01099", "County": "Monroe", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36460", "FIPS": "01099", "County": "Monroe", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36461", "FIPS": "01099", "County": "Monroe", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36462", "FIPS": "01099", "County": "Monroe", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36470", "FIPS": "01099", "County": "Monroe", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36471", "FIPS": "01099", "County": "Monroe", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36480", "FIPS": "01099", "County": "Monroe", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36481", "FIPS": "01099", "County": "Monroe", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36013", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36036", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36043", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36046", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36052", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36057", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36064", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36065", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36069", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36101", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36102", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36103", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36104", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36105", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36106", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36107", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36108", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36109", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36110", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36111", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36112", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36113", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36114", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36115", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36116", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36117", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36118", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36119", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36120", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36121", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36123", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36124", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36125", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36130", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36131", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36132", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36133", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36134", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36135", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36140", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36141", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36142", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36177", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36191", "FIPS": "01101", "County": "Montgomery", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "35601", "FIPS": "01103", "County": "Morgan", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35602", "FIPS": "01103", "County": "Morgan", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35603", "FIPS": "01103", "County": "Morgan", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35609", "FIPS": "01103", "County": "Morgan", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35619", "FIPS": "01103", "County": "Morgan", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35621", "FIPS": "01103", "County": "Morgan", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35622", "FIPS": "01103", "County": "Morgan", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35640", "FIPS": "01103", "County": "Morgan", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35670", "FIPS": "01103", "County": "Morgan", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35673", "FIPS": "01103", "County": "Morgan", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35699", "FIPS": "01103", "County": "Morgan", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35754", "FIPS": "01103", "County": "Morgan", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "35775", "FIPS": "01103", "County": "Morgan", "State": "AL", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "36756", "FIPS": "01105", "County": "Perry", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36786", "FIPS": "01105", "County": "Perry", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "35442", "FIPS": "01107", "County": "Pickens", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35447", "FIPS": "01107", "County": "Pickens", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35461", "FIPS": "01107", "County": "Pickens", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35466", "FIPS": "01107", "County": "Pickens", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35471", "FIPS": "01107", "County": "Pickens", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35481", "FIPS": "01107", "County": "Pickens", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36005", "FIPS": "01109", "County": "Pike", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36010", "FIPS": "01109", "County": "Pike", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36035", "FIPS": "01109", "County": "Pike", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36079", "FIPS": "01109", "County": "Pike", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36081", "FIPS": "01109", "County": "Pike", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36082", "FIPS": "01109", "County": "Pike", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36263", "FIPS": "01111", "County": "Randolph", "State": "AL", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "36274", "FIPS": "01111", "County": "Randolph", "State": "AL", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "36276", "FIPS": "01111", "County": "Randolph", "State": "AL", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "36278", "FIPS": "01111", "County": "Randolph", "State": "AL", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "36280", "FIPS": "01111", "County": "Randolph", "State": "AL", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "36851", "FIPS": "01113", "County": "Russell", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36856", "FIPS": "01113", "County": "Russell", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36858", "FIPS": "01113", "County": "Russell", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36859", "FIPS": "01113", "County": "Russell", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36860", "FIPS": "01113", "County": "Russell", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36867", "FIPS": "01113", "County": "Russell", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36868", "FIPS": "01113", "County": "Russell", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36869", "FIPS": "01113", "County": "Russell", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36871", "FIPS": "01113", "County": "Russell", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "36875", "FIPS": "01113", "County": "Russell", "State": "AL", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "35004", "FIPS": "01115", "County": "Saint Clair", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35052", "FIPS": "01115", "County": "Saint Clair", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35054", "FIPS": "01115", "County": "Saint Clair", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35112", "FIPS": "01115", "County": "Saint Clair", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35120", "FIPS": "01115", "County": "Saint Clair", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35125", "FIPS": "01115", "County": "Saint Clair", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35128", "FIPS": "01115", "County": "Saint Clair", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35131", "FIPS": "01115", "County": "Saint Clair", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35135", "FIPS": "01115", "County": "Saint Clair", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35146", "FIPS": "01115", "County": "Saint Clair", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35182", "FIPS": "01115", "County": "Saint Clair", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35953", "FIPS": "01115", "County": "Saint Clair", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35987", "FIPS": "01115", "County": "Saint Clair", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35007", "FIPS": "01117", "County": "Shelby", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35040", "FIPS": "01117", "County": "Shelby", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35043", "FIPS": "01117", "County": "Shelby", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35051", "FIPS": "01117", "County": "Shelby", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35078", "FIPS": "01117", "County": "Shelby", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35080", "FIPS": "01117", "County": "Shelby", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35114", "FIPS": "01117", "County": "Shelby", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35115", "FIPS": "01117", "County": "Shelby", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35124", "FIPS": "01117", "County": "Shelby", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35137", "FIPS": "01117", "County": "Shelby", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35143", "FIPS": "01117", "County": "Shelby", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35144", "FIPS": "01117", "County": "Shelby", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35147", "FIPS": "01117", "County": "Shelby", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35176", "FIPS": "01117", "County": "Shelby", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35178", "FIPS": "01117", "County": "Shelby", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35185", "FIPS": "01117", "County": "Shelby", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35186", "FIPS": "01117", "County": "Shelby", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35187", "FIPS": "01117", "County": "Shelby", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35242", "FIPS": "01117", "County": "Shelby", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35459", "FIPS": "01119", "County": "Sumter", "State": "AL", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "35460", "FIPS": "01119", "County": "Sumter", "State": "AL", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "35464", "FIPS": "01119", "County": "Sumter", "State": "AL", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "35470", "FIPS": "01119", "County": "Sumter", "State": "AL", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "35477", "FIPS": "01119", "County": "Sumter", "State": "AL", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "36901", "FIPS": "01119", "County": "Sumter", "State": "AL", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "36907", "FIPS": "01119", "County": "Sumter", "State": "AL", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "36925", "FIPS": "01119", "County": "Sumter", "State": "AL", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "35014", "FIPS": "01121", "County": "Talladega", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35032", "FIPS": "01121", "County": "Talladega", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35044", "FIPS": "01121", "County": "Talladega", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35096", "FIPS": "01121", "County": "Talladega", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35149", "FIPS": "01121", "County": "Talladega", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35150", "FIPS": "01121", "County": "Talladega", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35151", "FIPS": "01121", "County": "Talladega", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35160", "FIPS": "01121", "County": "Talladega", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35161", "FIPS": "01121", "County": "Talladega", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36268", "FIPS": "01121", "County": "Talladega", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35010", "FIPS": "01123", "County": "Tallapoosa", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "35011", "FIPS": "01123", "County": "Tallapoosa", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36023", "FIPS": "01123", "County": "Tallapoosa", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36256", "FIPS": "01123", "County": "Tallapoosa", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36850", "FIPS": "01123", "County": "Tallapoosa", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36853", "FIPS": "01123", "County": "Tallapoosa", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36861", "FIPS": "01123", "County": "Tallapoosa", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "35401", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35402", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35403", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35404", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35405", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35406", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35407", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35440", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35444", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35446", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35449", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35452", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35453", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35456", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35457", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35458", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35463", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35468", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35473", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35475", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35476", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35478", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35480", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35482", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35485", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35486", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35487", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35490", "FIPS": "01125", "County": "Tuscaloosa", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35038", "FIPS": "01127", "County": "Walker", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35063", "FIPS": "01127", "County": "Walker", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35130", "FIPS": "01127", "County": "Walker", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35148", "FIPS": "01127", "County": "Walker", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35501", "FIPS": "01127", "County": "Walker", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35502", "FIPS": "01127", "County": "Walker", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35503", "FIPS": "01127", "County": "Walker", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35504", "FIPS": "01127", "County": "Walker", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35549", "FIPS": "01127", "County": "Walker", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35550", "FIPS": "01127", "County": "Walker", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35554", "FIPS": "01127", "County": "Walker", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35560", "FIPS": "01127", "County": "Walker", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35573", "FIPS": "01127", "County": "Walker", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35578", "FIPS": "01127", "County": "Walker", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35579", "FIPS": "01127", "County": "Walker", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35580", "FIPS": "01127", "County": "Walker", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35584", "FIPS": "01127", "County": "Walker", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35587", "FIPS": "01127", "County": "Walker", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "36513", "FIPS": "01129", "County": "Washington", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36518", "FIPS": "01129", "County": "Washington", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36529", "FIPS": "01129", "County": "Washington", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36538", "FIPS": "01129", "County": "Washington", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36539", "FIPS": "01129", "County": "Washington", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36548", "FIPS": "01129", "County": "Washington", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36553", "FIPS": "01129", "County": "Washington", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36556", "FIPS": "01129", "County": "Washington", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36558", "FIPS": "01129", "County": "Washington", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36569", "FIPS": "01129", "County": "Washington", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36581", "FIPS": "01129", "County": "Washington", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36583", "FIPS": "01129", "County": "Washington", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36584", "FIPS": "01129", "County": "Washington", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36585", "FIPS": "01129", "County": "Washington", "State": "AL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "36435", "FIPS": "01131", "County": "Wilcox", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36720", "FIPS": "01131", "County": "Wilcox", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36721", "FIPS": "01131", "County": "Wilcox", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36722", "FIPS": "01131", "County": "Wilcox", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36723", "FIPS": "01131", "County": "Wilcox", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36726", "FIPS": "01131", "County": "Wilcox", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36728", "FIPS": "01131", "County": "Wilcox", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36741", "FIPS": "01131", "County": "Wilcox", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36751", "FIPS": "01131", "County": "Wilcox", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36753", "FIPS": "01131", "County": "Wilcox", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36766", "FIPS": "01131", "County": "Wilcox", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36768", "FIPS": "01131", "County": "Wilcox", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "36769", "FIPS": "01131", "County": "Wilcox", "State": "AL", "DMA": {"Code": 698, "Name": "MONTGOMERY"}},
  {"Postal_Code": "35540", "FIPS": "01133", "County": "Winston", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35541", "FIPS": "01133", "County": "Winston", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35551", "FIPS": "01133", "County": "Winston", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35553", "FIPS": "01133", "County": "Winston", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35565", "FIPS": "01133", "County": "Winston", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35572", "FIPS": "01133", "County": "Winston", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35575", "FIPS": "01133", "County": "Winston", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "35577", "FIPS": "01133", "County": "Winston", "State": "AL", "DMA": {"Code": 630, "Name": "BIRMINGHAM"}},
  {"Postal_Code": "99553", "FIPS": "02013", "County": "Aleutians East", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99571", "FIPS": "02013", "County": "Aleutians East", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99583", "FIPS": "02013", "County": "Aleutians East", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99612", "FIPS": "02013", "County": "Aleutians East", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99661", "FIPS": "02013", "County": "Aleutians East", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99546", "FIPS": "02016", "County": "Aleutians West", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99547", "FIPS": "02016", "County": "Aleutians West", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99591", "FIPS": "02016", "County": "Aleutians West", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99638", "FIPS": "02016", "County": "Aleutians West", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99660", "FIPS": "02016", "County": "Aleutians West", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99685", "FIPS": "02016", "County": "Aleutians West", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99692", "FIPS": "02016", "County": "Aleutians West", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99501", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99502", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99503", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99504", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99505", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99506", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99507", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99508", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99509", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99510", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99511", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99513", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99514", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99515", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99516", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99517", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99518", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99519", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99520", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99521", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99522", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99523", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99524", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99529", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99530", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99540", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99567", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99577", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99587", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99599", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99695", "FIPS": "02020", "County": "Anchorage", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99545", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99551", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99552", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99557", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99559", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99561", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99575", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99578", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99589", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99607", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99609", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99614", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99621", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99622", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99626", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99630", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99634", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99637", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99641", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99651", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99655", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99656", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99668", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99679", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99680", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99681", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99690", "FIPS": "02050", "County": "Bethel", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99613", "FIPS": "02060", "County": "Bristol Bay", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99633", "FIPS": "02060", "County": "Bristol Bay", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99670", "FIPS": "02060", "County": "Bristol Bay", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99704", "FIPS": "02068", "County": "Denali", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99729", "FIPS": "02068", "County": "Denali", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99743", "FIPS": "02068", "County": "Denali", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99744", "FIPS": "02068", "County": "Denali", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99755", "FIPS": "02068", "County": "Denali", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99555", "FIPS": "02070", "County": "Dillingham", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99569", "FIPS": "02070", "County": "Dillingham", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99576", "FIPS": "02070", "County": "Dillingham", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99580", "FIPS": "02070", "County": "Dillingham", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99628", "FIPS": "02070", "County": "Dillingham", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99636", "FIPS": "02070", "County": "Dillingham", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99678", "FIPS": "02070", "County": "Dillingham", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99701", "FIPS": "02090", "County": "Fairbanks North Star", "State": "AK", "DMA": {"Code": 745, "Name": "FAIRBANKS"}},
  {"Postal_Code": "99702", "FIPS": "02090", "County": "Fairbanks North Star", "State": "AK", "DMA": {"Code": 745, "Name": "FAIRBANKS"}},
  {"Postal_Code": "99703", "FIPS": "02090", "County": "Fairbanks North Star", "State": "AK", "DMA": {"Code": 745, "Name": "FAIRBANKS"}},
  {"Postal_Code": "99705", "FIPS": "02090", "County": "Fairbanks North Star", "State": "AK", "DMA": {"Code": 745, "Name": "FAIRBANKS"}},
  {"Postal_Code": "99706", "FIPS": "02090", "County": "Fairbanks North Star", "State": "AK", "DMA": {"Code": 745, "Name": "FAIRBANKS"}},
  {"Postal_Code": "99707", "FIPS": "02090", "County": "Fairbanks North Star", "State": "AK", "DMA": {"Code": 745, "Name": "FAIRBANKS"}},
  {"Postal_Code": "99708", "FIPS": "02090", "County": "Fairbanks North Star", "State": "AK", "DMA": {"Code": 745, "Name": "FAIRBANKS"}},
  {"Postal_Code": "99709", "FIPS": "02090", "County": "Fairbanks North Star", "State": "AK", "DMA": {"Code": 745, "Name": "FAIRBANKS"}},
  {"Postal_Code": "99710", "FIPS": "02090", "County": "Fairbanks North Star", "State": "AK", "DMA": {"Code": 745, "Name": "FAIRBANKS"}},
  {"Postal_Code": "99711", "FIPS": "02090", "County": "Fairbanks North Star", "State": "AK", "DMA": {"Code": 745, "Name": "FAIRBANKS"}},
  {"Postal_Code": "99712", "FIPS": "02090", "County": "Fairbanks North Star", "State": "AK", "DMA": {"Code": 745, "Name": "FAIRBANKS"}},
  {"Postal_Code": "99714", "FIPS": "02090", "County": "Fairbanks North Star", "State": "AK", "DMA": {"Code": 745, "Name": "FAIRBANKS"}},
  {"Postal_Code": "99716", "FIPS": "02090", "County": "Fairbanks North Star", "State": "AK", "DMA": {"Code": 745, "Name": "FAIRBANKS"}},
  {"Postal_Code": "99725", "FIPS": "02090", "County": "Fairbanks North Star", "State": "AK", "DMA": {"Code": 745, "Name": "FAIRBANKS"}},
  {"Postal_Code": "99775", "FIPS": "02090", "County": "Fairbanks North Star", "State": "AK", "DMA": {"Code": 745, "Name": "FAIRBANKS"}},
  {"Postal_Code": "99790", "FIPS": "02090", "County": "Fairbanks North Star", "State": "AK", "DMA": {"Code": 745, "Name": "FAIRBANKS"}},
  {"Postal_Code": "99827", "FIPS": "02100", "County": "Haines", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99801", "FIPS": "02110", "County": "Juneau", "State": "AK", "DMA": {"Code": 747, "Name": "JUNEAU"}},
  {"Postal_Code": "99802", "FIPS": "02110", "County": "Juneau", "State": "AK", "DMA": {"Code": 747, "Name": "JUNEAU"}},
  {"Postal_Code": "99803", "FIPS": "02110", "County": "Juneau", "State": "AK", "DMA": {"Code": 747, "Name": "JUNEAU"}},
  {"Postal_Code": "99811", "FIPS": "02110", "County": "Juneau", "State": "AK", "DMA": {"Code": 747, "Name": "JUNEAU"}},
  {"Postal_Code": "99812", "FIPS": "02110", "County": "Juneau", "State": "AK", "DMA": {"Code": 747, "Name": "JUNEAU"}},
  {"Postal_Code": "99821", "FIPS": "02110", "County": "Juneau", "State": "AK", "DMA": {"Code": 747, "Name": "JUNEAU"}},
  {"Postal_Code": "99824", "FIPS": "02110", "County": "Juneau", "State": "AK", "DMA": {"Code": 747, "Name": "JUNEAU"}},
  {"Postal_Code": "99850", "FIPS": "02110", "County": "Juneau", "State": "AK", "DMA": {"Code": 747, "Name": "JUNEAU"}},
  {"Postal_Code": "99556", "FIPS": "02122", "County": "Kenai Peninsula", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99568", "FIPS": "02122", "County": "Kenai Peninsula", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99572", "FIPS": "02122", "County": "Kenai Peninsula", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99603", "FIPS": "02122", "County": "Kenai Peninsula", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99605", "FIPS": "02122", "County": "Kenai Peninsula", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99610", "FIPS": "02122", "County": "Kenai Peninsula", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99611", "FIPS": "02122", "County": "Kenai Peninsula", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99631", "FIPS": "02122", "County": "Kenai Peninsula", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99635", "FIPS": "02122", "County": "Kenai Peninsula", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99639", "FIPS": "02122", "County": "Kenai Peninsula", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99663", "FIPS": "02122", "County": "Kenai Peninsula", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99664", "FIPS": "02122", "County": "Kenai Peninsula", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99669", "FIPS": "02122", "County": "Kenai Peninsula", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99672", "FIPS": "02122", "County": "Kenai Peninsula", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99682", "FIPS": "02122", "County": "Kenai Peninsula", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99901", "FIPS": "02130", "County": "Ketchikan Gateway", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99928", "FIPS": "02130", "County": "Ketchikan Gateway", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99950", "FIPS": "02130", "County": "Ketchikan Gateway", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99550", "FIPS": "02150", "County": "Kodiak Island", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99608", "FIPS": "02150", "County": "Kodiak Island", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99615", "FIPS": "02150", "County": "Kodiak Island", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99619", "FIPS": "02150", "County": "Kodiak Island", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99624", "FIPS": "02150", "County": "Kodiak Island", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99643", "FIPS": "02150", "County": "Kodiak Island", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99644", "FIPS": "02150", "County": "Kodiak Island", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99697", "FIPS": "02150", "County": "Kodiak Island", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99548", "FIPS": "02164", "County": "Lake and Peninsula", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99549", "FIPS": "02164", "County": "Lake and Peninsula", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99564", "FIPS": "02164", "County": "Lake and Peninsula", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99565", "FIPS": "02164", "County": "Lake and Peninsula", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99579", "FIPS": "02164", "County": "Lake and Peninsula", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99606", "FIPS": "02164", "County": "Lake and Peninsula", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99625", "FIPS": "02164", "County": "Lake and Peninsula", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99640", "FIPS": "02164", "County": "Lake and Peninsula", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99647", "FIPS": "02164", "County": "Lake and Peninsula", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99648", "FIPS": "02164", "County": "Lake and Peninsula", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99649", "FIPS": "02164", "County": "Lake and Peninsula", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99653", "FIPS": "02164", "County": "Lake and Peninsula", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99629", "FIPS": "02170", "County": "Matanuska Susitna", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99645", "FIPS": "02170", "County": "Matanuska Susitna", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99652", "FIPS": "02170", "County": "Matanuska Susitna", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99654", "FIPS": "02170", "County": "Matanuska Susitna", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99667", "FIPS": "02170", "County": "Matanuska Susitna", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99674", "FIPS": "02170", "County": "Matanuska Susitna", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99676", "FIPS": "02170", "County": "Matanuska Susitna", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99683", "FIPS": "02170", "County": "Matanuska Susitna", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99687", "FIPS": "02170", "County": "Matanuska Susitna", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99688", "FIPS": "02170", "County": "Matanuska Susitna", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99694", "FIPS": "02170", "County": "Matanuska Susitna", "State": "AK", "DMA": {"Code": 743, "Name": "ANCHORAGE"}},
  {"Postal_Code": "99659", "FIPS": "02180", "County": "Nome", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99671", "FIPS": "02180", "County": "Nome", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99684", "FIPS": "02180", "County": "Nome", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99739", "FIPS": "02180", "County": "Nome", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99742", "FIPS": "02180", "County": "Nome", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99753", "FIPS": "02180", "County": "Nome", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99762", "FIPS": "02180", "County": "Nome", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99769", "FIPS": "02180", "County": "Nome", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99771", "FIPS": "02180", "County": "Nome", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99772", "FIPS": "02180", "County": "Nome", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99778", "FIPS": "02180", "County": "Nome", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99783", "FIPS": "02180", "County": "Nome", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99784", "FIPS": "02180", "County": "Nome", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99785", "FIPS": "02180", "County": "Nome", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99721", "FIPS": "02185", "County": "North Slope", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99723", "FIPS": "02185", "County": "North Slope", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99734", "FIPS": "02185", "County": "North Slope", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99747", "FIPS": "02185", "County": "North Slope", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99759", "FIPS": "02185", "County": "North Slope", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99766", "FIPS": "02185", "County": "North Slope", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99782", "FIPS": "02185", "County": "North Slope", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99789", "FIPS": "02185", "County": "North Slope", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99791", "FIPS": "02185", "County": "North Slope", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99727", "FIPS": "02188", "County": "Northwest Arctic", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99736", "FIPS": "02188", "County": "Northwest Arctic", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99749", "FIPS": "02188", "County": "Northwest Arctic", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99750", "FIPS": "02188", "County": "Northwest Arctic", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99751", "FIPS": "02188", "County": "Northwest Arctic", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99752", "FIPS": "02188", "County": "Northwest Arctic", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99761", "FIPS": "02188", "County": "Northwest Arctic", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99763", "FIPS": "02188", "County": "Northwest Arctic", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99770", "FIPS": "02188", "County": "Northwest Arctic", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99773", "FIPS": "02188", "County": "Northwest Arctic", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99786", "FIPS": "02188", "County": "Northwest Arctic", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99903", "FIPS": "02201", "County": "Prince Wales Ketchikan", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99918", "FIPS": "02201", "County": "Prince Wales Ketchikan", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99919", "FIPS": "02201", "County": "Prince Wales Ketchikan", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99921", "FIPS": "02201", "County": "Prince Wales Ketchikan", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99922", "FIPS": "02201", "County": "Prince Wales Ketchikan", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99923", "FIPS": "02201", "County": "Prince Wales Ketchikan", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99925", "FIPS": "02201", "County": "Prince Wales Ketchikan", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99926", "FIPS": "02201", "County": "Prince Wales Ketchikan", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99927", "FIPS": "02201", "County": "Prince Wales Ketchikan", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99835", "FIPS": "02220", "County": "Sitka", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99820", "FIPS": "02232", "County": "Skagway Hoonah Angoon", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99825", "FIPS": "02232", "County": "Skagway Hoonah Angoon", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99826", "FIPS": "02232", "County": "Skagway Hoonah Angoon", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99829", "FIPS": "02232", "County": "Skagway Hoonah Angoon", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99832", "FIPS": "02232", "County": "Skagway Hoonah Angoon", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99840", "FIPS": "02232", "County": "Skagway Hoonah Angoon", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99841", "FIPS": "02232", "County": "Skagway Hoonah Angoon", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99731", "FIPS": "02240", "County": "Southeast Fairbanks", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99732", "FIPS": "02240", "County": "Southeast Fairbanks", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99737", "FIPS": "02240", "County": "Southeast Fairbanks", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99738", "FIPS": "02240", "County": "Southeast Fairbanks", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99764", "FIPS": "02240", "County": "Southeast Fairbanks", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99776", "FIPS": "02240", "County": "Southeast Fairbanks", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99779", "FIPS": "02240", "County": "Southeast Fairbanks", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99780", "FIPS": "02240", "County": "Southeast Fairbanks", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99566", "FIPS": "02261", "County": "Valdez Cordova", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99573", "FIPS": "02261", "County": "Valdez Cordova", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99574", "FIPS": "02261", "County": "Valdez Cordova", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99586", "FIPS": "02261", "County": "Valdez Cordova", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99588", "FIPS": "02261", "County": "Valdez Cordova", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99677", "FIPS": "02261", "County": "Valdez Cordova", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99686", "FIPS": "02261", "County": "Valdez Cordova", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99693", "FIPS": "02261", "County": "Valdez Cordova", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99554", "FIPS": "02270", "County": "Wade Hampton", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99563", "FIPS": "02270", "County": "Wade Hampton", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99581", "FIPS": "02270", "County": "Wade Hampton", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99585", "FIPS": "02270", "County": "Wade Hampton", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99604", "FIPS": "02270", "County": "Wade Hampton", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99620", "FIPS": "02270", "County": "Wade Hampton", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99632", "FIPS": "02270", "County": "Wade Hampton", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99650", "FIPS": "02270", "County": "Wade Hampton", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99657", "FIPS": "02270", "County": "Wade Hampton", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99658", "FIPS": "02270", "County": "Wade Hampton", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99662", "FIPS": "02270", "County": "Wade Hampton", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99666", "FIPS": "02270", "County": "Wade Hampton", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99830", "FIPS": "02280", "County": "Wrangell Petersburg", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99833", "FIPS": "02280", "County": "Wrangell Petersburg", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99836", "FIPS": "02280", "County": "Wrangell Petersburg", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99929", "FIPS": "02280", "County": "Wrangell Petersburg", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99689", "FIPS": "02282", "County": "Yakutat", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99558", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99590", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99602", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99627", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99665", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99675", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99691", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99720", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99722", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99724", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99726", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99730", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99733", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99740", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99741", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99745", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99746", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99748", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99754", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99756", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99757", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99758", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99760", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99765", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99767", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99768", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99774", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99777", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99781", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "99788", "FIPS": "02290", "County": "Yukon Koyukuk", "State": "AK", "DMA": {"Code": None, "Name": None}},
  {"Postal_Code": "85920", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "85924", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "85925", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "85927", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "85930", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "85932", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "85936", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "85938", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "85940", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "86028", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "86502", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "86503", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "86504", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "86505", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "86506", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "86507", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "86508", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "86511", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "86512", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "86514", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "86515", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "86535", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "86538", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "86540", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "86544", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "86545", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "86547", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "86556", "FIPS": "04001", "County": "Apache", "State": "AZ", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "85602", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85603", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85605", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85606", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85607", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85608", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85609", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85610", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85613", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85615", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85616", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85617", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85620", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85625", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85626", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85627", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85630", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85632", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85635", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85636", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85638", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85643", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85644", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85650", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85655", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85670", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85671", "FIPS": "04003", "County": "Cochise", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85931", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86001", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86002", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86003", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86004", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86011", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86015", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86016", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86017", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86018", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86020", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86022", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86023", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86024", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86035", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86036", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86038", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86040", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86044", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86045", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86046", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86052", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86053", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86339", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86435", "FIPS": "04005", "County": "Coconino", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85235", "FIPS": "04007", "County": "Gila", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85292", "FIPS": "04007", "County": "Gila", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85501", "FIPS": "04007", "County": "Gila", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85502", "FIPS": "04007", "County": "Gila", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85532", "FIPS": "04007", "County": "Gila", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85539", "FIPS": "04007", "County": "Gila", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85541", "FIPS": "04007", "County": "Gila", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85542", "FIPS": "04007", "County": "Gila", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85544", "FIPS": "04007", "County": "Gila", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85545", "FIPS": "04007", "County": "Gila", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85547", "FIPS": "04007", "County": "Gila", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85550", "FIPS": "04007", "County": "Gila", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85553", "FIPS": "04007", "County": "Gila", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85554", "FIPS": "04007", "County": "Gila", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85530", "FIPS": "04009", "County": "Graham", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85531", "FIPS": "04009", "County": "Graham", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85535", "FIPS": "04009", "County": "Graham", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85536", "FIPS": "04009", "County": "Graham", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85543", "FIPS": "04009", "County": "Graham", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85546", "FIPS": "04009", "County": "Graham", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85548", "FIPS": "04009", "County": "Graham", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85551", "FIPS": "04009", "County": "Graham", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85552", "FIPS": "04009", "County": "Graham", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85533", "FIPS": "04011", "County": "Greenlee", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85534", "FIPS": "04011", "County": "Greenlee", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85540", "FIPS": "04011", "County": "Greenlee", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85922", "FIPS": "04011", "County": "Greenlee", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85325", "FIPS": "04012", "County": "La Paz", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85328", "FIPS": "04012", "County": "La Paz", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85334", "FIPS": "04012", "County": "La Paz", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85344", "FIPS": "04012", "County": "La Paz", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85346", "FIPS": "04012", "County": "La Paz", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85348", "FIPS": "04012", "County": "La Paz", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85357", "FIPS": "04012", "County": "La Paz", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85359", "FIPS": "04012", "County": "La Paz", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85371", "FIPS": "04012", "County": "La Paz", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85001", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85002", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85003", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85004", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85005", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85006", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85007", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85008", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85009", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85010", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85011", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85012", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85013", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85014", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85015", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85016", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85017", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85018", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85019", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85020", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85021", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85022", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85023", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85024", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85025", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85026", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85027", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85028", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85029", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85030", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85031", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85032", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85033", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85034", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85035", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85036", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85037", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85038", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85039", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85040", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85041", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85042", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85043", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85044", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85045", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85046", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85048", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85050", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85051", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85053", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85054", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85055", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85060", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85061", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85062", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85063", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85064", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85065", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85066", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85067", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85068", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85069", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85070", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85071", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85072", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85073", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85074", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85075", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85076", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85077", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85078", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85079", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85080", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85082", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85085", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85086", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85087", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85098", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85099", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85201", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85202", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85203", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85204", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85205", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85206", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85207", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85208", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85209", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85210", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85211", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85212", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85213", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85214", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85215", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85216", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85224", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85225", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85226", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85227", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85233", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85234", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85236", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85242", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85244", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85246", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85248", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85249", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85250", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85251", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85252", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85253", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85254", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85255", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85256", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85257", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85258", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85259", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85260", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85261", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85262", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85263", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85264", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85266", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85267", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85268", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85269", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85271", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85274", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85275", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85277", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85280", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85281", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85282", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85283", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85284", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85285", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85287", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85289", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85290", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85296", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85297", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85299", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85301", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85302", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85303", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85304", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85305", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85306", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85307", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85308", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85309", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85310", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85311", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85312", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85313", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85318", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85320", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85322", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85323", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85326", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85327", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85329", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85331", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85335", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85337", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85338", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85339", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85340", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85342", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85343", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85345", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85351", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85353", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85354", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85355", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85358", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85361", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85363", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85372", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85373", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85374", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85375", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85376", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85377", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85378", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85379", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85380", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85381", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85382", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85383", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85385", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85387", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85388", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85390", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85396", "FIPS": "04013", "County": "Maricopa", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85360", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86021", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86401", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86402", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86403", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86404", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86405", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86406", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86409", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86411", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86412", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86413", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86426", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86427", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86429", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86430", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86431", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86432", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86433", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86434", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86436", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86437", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86438", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86439", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86440", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86441", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86442", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86443", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86444", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86445", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86446", "FIPS": "04015", "County": "Mohave", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85901", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85902", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85911", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85912", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85923", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85926", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85928", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85929", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85933", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85934", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85935", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85937", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85939", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85941", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85942", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86025", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86029", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86030", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86031", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86032", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86033", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86034", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86039", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86042", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86043", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86047", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86054", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86510", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86520", "FIPS": "04017", "County": "Navajo", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85321", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85341", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85601", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85614", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85619", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85622", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85629", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85633", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85634", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85639", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85641", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85652", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85653", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85654", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85701", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85702", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85703", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85704", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85705", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85706", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85707", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85708", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85709", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85710", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85711", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85712", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85713", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85714", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85715", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85716", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85717", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85718", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85719", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85720", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85721", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85722", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85723", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85724", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85725", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85726", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85728", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85730", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85731", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85732", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85733", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85734", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85735", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85736", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85737", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85738", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85739", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85740", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85741", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85742", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85743", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85744", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85745", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85746", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85747", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85748", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85749", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85750", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85751", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85752", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85754", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85755", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85757", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85775", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85777", "FIPS": "04019", "County": "Pima", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85217", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85218", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85219", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85220", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85221", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85222", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85223", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85228", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85230", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85231", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85232", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85237", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85239", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85241", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85243", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85245", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85247", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85272", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85273", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85278", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85279", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85291", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85618", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85623", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85631", "FIPS": "04021", "County": "Pinal", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85611", "FIPS": "04023", "County": "Santa Cruz", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85621", "FIPS": "04023", "County": "Santa Cruz", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85624", "FIPS": "04023", "County": "Santa Cruz", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85628", "FIPS": "04023", "County": "Santa Cruz", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85637", "FIPS": "04023", "County": "Santa Cruz", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85640", "FIPS": "04023", "County": "Santa Cruz", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85645", "FIPS": "04023", "County": "Santa Cruz", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85646", "FIPS": "04023", "County": "Santa Cruz", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85648", "FIPS": "04023", "County": "Santa Cruz", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85662", "FIPS": "04023", "County": "Santa Cruz", "State": "AZ", "DMA": {"Code": 789, "Name": "TUCSON-SIERRA VISTA"}},
  {"Postal_Code": "85324", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85332", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85362", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86301", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86302", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86303", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86304", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86305", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86312", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86313", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86314", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86320", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86321", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86322", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86323", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86324", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86325", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86326", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86327", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86329", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86330", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86331", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86332", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86333", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86334", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86335", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86336", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86337", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86338", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86340", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86341", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86342", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86343", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "86351", "FIPS": "04025", "County": "Yavapai", "State": "AZ", "DMA": {"Code": 753, "Name": "PHOENIX"}},
  {"Postal_Code": "85333", "FIPS": "04027", "County": "Yuma", "State": "AZ", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "85336", "FIPS": "04027", "County": "Yuma", "State": "AZ", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "85347", "FIPS": "04027", "County": "Yuma", "State": "AZ", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "85349", "FIPS": "04027", "County": "Yuma", "State": "AZ", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "85350", "FIPS": "04027", "County": "Yuma", "State": "AZ", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "85352", "FIPS": "04027", "County": "Yuma", "State": "AZ", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "85356", "FIPS": "04027", "County": "Yuma", "State": "AZ", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "85364", "FIPS": "04027", "County": "Yuma", "State": "AZ", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "85365", "FIPS": "04027", "County": "Yuma", "State": "AZ", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "85366", "FIPS": "04027", "County": "Yuma", "State": "AZ", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "85367", "FIPS": "04027", "County": "Yuma", "State": "AZ", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "85369", "FIPS": "04027", "County": "Yuma", "State": "AZ", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "72003", "FIPS": "05001", "County": "Arkansas", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72026", "FIPS": "05001", "County": "Arkansas", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72038", "FIPS": "05001", "County": "Arkansas", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72042", "FIPS": "05001", "County": "Arkansas", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72048", "FIPS": "05001", "County": "Arkansas", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72055", "FIPS": "05001", "County": "Arkansas", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72073", "FIPS": "05001", "County": "Arkansas", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72140", "FIPS": "05001", "County": "Arkansas", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72160", "FIPS": "05001", "County": "Arkansas", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72166", "FIPS": "05001", "County": "Arkansas", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71635", "FIPS": "05003", "County": "Ashley", "State": "AR", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71642", "FIPS": "05003", "County": "Ashley", "State": "AR", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71646", "FIPS": "05003", "County": "Ashley", "State": "AR", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71658", "FIPS": "05003", "County": "Ashley", "State": "AR", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71661", "FIPS": "05003", "County": "Ashley", "State": "AR", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71663", "FIPS": "05003", "County": "Ashley", "State": "AR", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71676", "FIPS": "05003", "County": "Ashley", "State": "AR", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "72537", "FIPS": "05005", "County": "Baxter", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72544", "FIPS": "05005", "County": "Baxter", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72617", "FIPS": "05005", "County": "Baxter", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72623", "FIPS": "05005", "County": "Baxter", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72626", "FIPS": "05005", "County": "Baxter", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72635", "FIPS": "05005", "County": "Baxter", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72642", "FIPS": "05005", "County": "Baxter", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72651", "FIPS": "05005", "County": "Baxter", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72653", "FIPS": "05005", "County": "Baxter", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72654", "FIPS": "05005", "County": "Baxter", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72658", "FIPS": "05005", "County": "Baxter", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72659", "FIPS": "05005", "County": "Baxter", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72711", "FIPS": "05007", "County": "Benton", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72712", "FIPS": "05007", "County": "Benton", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72714", "FIPS": "05007", "County": "Benton", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72715", "FIPS": "05007", "County": "Benton", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72716", "FIPS": "05007", "County": "Benton", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72718", "FIPS": "05007", "County": "Benton", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72719", "FIPS": "05007", "County": "Benton", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72722", "FIPS": "05007", "County": "Benton", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72732", "FIPS": "05007", "County": "Benton", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72733", "FIPS": "05007", "County": "Benton", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72734", "FIPS": "05007", "County": "Benton", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72736", "FIPS": "05007", "County": "Benton", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72739", "FIPS": "05007", "County": "Benton", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72745", "FIPS": "05007", "County": "Benton", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72747", "FIPS": "05007", "County": "Benton", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72751", "FIPS": "05007", "County": "Benton", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72756", "FIPS": "05007", "County": "Benton", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72757", "FIPS": "05007", "County": "Benton", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72758", "FIPS": "05007", "County": "Benton", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72761", "FIPS": "05007", "County": "Benton", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72768", "FIPS": "05007", "County": "Benton", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72601", "FIPS": "05009", "County": "Boone", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72602", "FIPS": "05009", "County": "Boone", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72611", "FIPS": "05009", "County": "Boone", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72615", "FIPS": "05009", "County": "Boone", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72630", "FIPS": "05009", "County": "Boone", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72633", "FIPS": "05009", "County": "Boone", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72644", "FIPS": "05009", "County": "Boone", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72662", "FIPS": "05009", "County": "Boone", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72682", "FIPS": "05009", "County": "Boone", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "71631", "FIPS": "05011", "County": "Bradley", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71647", "FIPS": "05011", "County": "Bradley", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71651", "FIPS": "05011", "County": "Bradley", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71671", "FIPS": "05011", "County": "Bradley", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71744", "FIPS": "05013", "County": "Calhoun", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71745", "FIPS": "05013", "County": "Calhoun", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71766", "FIPS": "05013", "County": "Calhoun", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72613", "FIPS": "05015", "County": "Carroll", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72616", "FIPS": "05015", "County": "Carroll", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72631", "FIPS": "05015", "County": "Carroll", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72632", "FIPS": "05015", "County": "Carroll", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72638", "FIPS": "05015", "County": "Carroll", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72660", "FIPS": "05015", "County": "Carroll", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "71638", "FIPS": "05017", "County": "Chicot", "State": "AR", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "71640", "FIPS": "05017", "County": "Chicot", "State": "AR", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "71653", "FIPS": "05017", "County": "Chicot", "State": "AR", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "71721", "FIPS": "05019", "County": "Clark", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71728", "FIPS": "05019", "County": "Clark", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71743", "FIPS": "05019", "County": "Clark", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71772", "FIPS": "05019", "County": "Clark", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71920", "FIPS": "05019", "County": "Clark", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71921", "FIPS": "05019", "County": "Clark", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71923", "FIPS": "05019", "County": "Clark", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71962", "FIPS": "05019", "County": "Clark", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71998", "FIPS": "05019", "County": "Clark", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71999", "FIPS": "05019", "County": "Clark", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72422", "FIPS": "05021", "County": "Clay", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72424", "FIPS": "05021", "County": "Clay", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72430", "FIPS": "05021", "County": "Clay", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72435", "FIPS": "05021", "County": "Clay", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72441", "FIPS": "05021", "County": "Clay", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72453", "FIPS": "05021", "County": "Clay", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72454", "FIPS": "05021", "County": "Clay", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72456", "FIPS": "05021", "County": "Clay", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72461", "FIPS": "05021", "County": "Clay", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72464", "FIPS": "05021", "County": "Clay", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72470", "FIPS": "05021", "County": "Clay", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72044", "FIPS": "05023", "County": "Cleburne", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72067", "FIPS": "05023", "County": "Cleburne", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72130", "FIPS": "05023", "County": "Cleburne", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72131", "FIPS": "05023", "County": "Cleburne", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72179", "FIPS": "05023", "County": "Cleburne", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72523", "FIPS": "05023", "County": "Cleburne", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72530", "FIPS": "05023", "County": "Cleburne", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72543", "FIPS": "05023", "County": "Cleburne", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72545", "FIPS": "05023", "County": "Cleburne", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72546", "FIPS": "05023", "County": "Cleburne", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72581", "FIPS": "05023", "County": "Cleburne", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71652", "FIPS": "05025", "County": "Cleveland", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71660", "FIPS": "05025", "County": "Cleveland", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71665", "FIPS": "05025", "County": "Cleveland", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71740", "FIPS": "05027", "County": "Columbia", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71752", "FIPS": "05027", "County": "Columbia", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71753", "FIPS": "05027", "County": "Columbia", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71754", "FIPS": "05027", "County": "Columbia", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71770", "FIPS": "05027", "County": "Columbia", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71861", "FIPS": "05027", "County": "Columbia", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "72027", "FIPS": "05029", "County": "Conway", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72030", "FIPS": "05029", "County": "Conway", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72063", "FIPS": "05029", "County": "Conway", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72080", "FIPS": "05029", "County": "Conway", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72107", "FIPS": "05029", "County": "Conway", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72110", "FIPS": "05029", "County": "Conway", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72127", "FIPS": "05029", "County": "Conway", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72156", "FIPS": "05029", "County": "Conway", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72157", "FIPS": "05029", "County": "Conway", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72401", "FIPS": "05031", "County": "Craighead", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72402", "FIPS": "05031", "County": "Craighead", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72403", "FIPS": "05031", "County": "Craighead", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72404", "FIPS": "05031", "County": "Craighead", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72411", "FIPS": "05031", "County": "Craighead", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72414", "FIPS": "05031", "County": "Craighead", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72416", "FIPS": "05031", "County": "Craighead", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72417", "FIPS": "05031", "County": "Craighead", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72419", "FIPS": "05031", "County": "Craighead", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72421", "FIPS": "05031", "County": "Craighead", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72427", "FIPS": "05031", "County": "Craighead", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72437", "FIPS": "05031", "County": "Craighead", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72447", "FIPS": "05031", "County": "Craighead", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72467", "FIPS": "05031", "County": "Craighead", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72921", "FIPS": "05033", "County": "Crawford", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72932", "FIPS": "05033", "County": "Crawford", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72934", "FIPS": "05033", "County": "Crawford", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72935", "FIPS": "05033", "County": "Crawford", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72946", "FIPS": "05033", "County": "Crawford", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72947", "FIPS": "05033", "County": "Crawford", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72948", "FIPS": "05033", "County": "Crawford", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72952", "FIPS": "05033", "County": "Crawford", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72955", "FIPS": "05033", "County": "Crawford", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72956", "FIPS": "05033", "County": "Crawford", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72957", "FIPS": "05033", "County": "Crawford", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72301", "FIPS": "05035", "County": "Crittenden", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72303", "FIPS": "05035", "County": "Crittenden", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72325", "FIPS": "05035", "County": "Crittenden", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72327", "FIPS": "05035", "County": "Crittenden", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72331", "FIPS": "05035", "County": "Crittenden", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72332", "FIPS": "05035", "County": "Crittenden", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72339", "FIPS": "05035", "County": "Crittenden", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72364", "FIPS": "05035", "County": "Crittenden", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72376", "FIPS": "05035", "County": "Crittenden", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72384", "FIPS": "05035", "County": "Crittenden", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72324", "FIPS": "05037", "County": "Cross", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72347", "FIPS": "05037", "County": "Cross", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72373", "FIPS": "05037", "County": "Cross", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72387", "FIPS": "05037", "County": "Cross", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72396", "FIPS": "05037", "County": "Cross", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "71725", "FIPS": "05039", "County": "Dallas", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71742", "FIPS": "05039", "County": "Dallas", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71748", "FIPS": "05039", "County": "Dallas", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71763", "FIPS": "05039", "County": "Dallas", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71630", "FIPS": "05041", "County": "Desha", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71639", "FIPS": "05041", "County": "Desha", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71654", "FIPS": "05041", "County": "Desha", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71662", "FIPS": "05041", "County": "Desha", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71666", "FIPS": "05041", "County": "Desha", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71670", "FIPS": "05041", "County": "Desha", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71674", "FIPS": "05041", "County": "Desha", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72379", "FIPS": "05041", "County": "Desha", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71655", "FIPS": "05043", "County": "Drew", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71656", "FIPS": "05043", "County": "Drew", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71657", "FIPS": "05043", "County": "Drew", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71675", "FIPS": "05043", "County": "Drew", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71677", "FIPS": "05043", "County": "Drew", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72032", "FIPS": "05045", "County": "Faulkner", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72033", "FIPS": "05045", "County": "Faulkner", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72034", "FIPS": "05045", "County": "Faulkner", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72035", "FIPS": "05045", "County": "Faulkner", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72039", "FIPS": "05045", "County": "Faulkner", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72047", "FIPS": "05045", "County": "Faulkner", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72058", "FIPS": "05045", "County": "Faulkner", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72061", "FIPS": "05045", "County": "Faulkner", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72106", "FIPS": "05045", "County": "Faulkner", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72111", "FIPS": "05045", "County": "Faulkner", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72173", "FIPS": "05045", "County": "Faulkner", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72181", "FIPS": "05045", "County": "Faulkner", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72820", "FIPS": "05047", "County": "Franklin", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72821", "FIPS": "05047", "County": "Franklin", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72928", "FIPS": "05047", "County": "Franklin", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72930", "FIPS": "05047", "County": "Franklin", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72933", "FIPS": "05047", "County": "Franklin", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72949", "FIPS": "05047", "County": "Franklin", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72515", "FIPS": "05049", "County": "Fulton", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72520", "FIPS": "05049", "County": "Fulton", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72531", "FIPS": "05049", "County": "Fulton", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72538", "FIPS": "05049", "County": "Fulton", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72539", "FIPS": "05049", "County": "Fulton", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72554", "FIPS": "05049", "County": "Fulton", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72576", "FIPS": "05049", "County": "Fulton", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72578", "FIPS": "05049", "County": "Fulton", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72583", "FIPS": "05049", "County": "Fulton", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "71901", "FIPS": "05051", "County": "Garland", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71902", "FIPS": "05051", "County": "Garland", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71903", "FIPS": "05051", "County": "Garland", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71909", "FIPS": "05051", "County": "Garland", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71910", "FIPS": "05051", "County": "Garland", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71913", "FIPS": "05051", "County": "Garland", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71914", "FIPS": "05051", "County": "Garland", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71949", "FIPS": "05051", "County": "Garland", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71951", "FIPS": "05051", "County": "Garland", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71956", "FIPS": "05051", "County": "Garland", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71964", "FIPS": "05051", "County": "Garland", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71968", "FIPS": "05051", "County": "Garland", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72087", "FIPS": "05051", "County": "Garland", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72057", "FIPS": "05053", "County": "Grant", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72084", "FIPS": "05053", "County": "Grant", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72128", "FIPS": "05053", "County": "Grant", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72129", "FIPS": "05053", "County": "Grant", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72150", "FIPS": "05053", "County": "Grant", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72412", "FIPS": "05055", "County": "Greene", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72425", "FIPS": "05055", "County": "Greene", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72436", "FIPS": "05055", "County": "Greene", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72439", "FIPS": "05055", "County": "Greene", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72443", "FIPS": "05055", "County": "Greene", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72450", "FIPS": "05055", "County": "Greene", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72451", "FIPS": "05055", "County": "Greene", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72474", "FIPS": "05055", "County": "Greene", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "71801", "FIPS": "05057", "County": "Hempstead", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71802", "FIPS": "05057", "County": "Hempstead", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71825", "FIPS": "05057", "County": "Hempstead", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71831", "FIPS": "05057", "County": "Hempstead", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71838", "FIPS": "05057", "County": "Hempstead", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71847", "FIPS": "05057", "County": "Hempstead", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71855", "FIPS": "05057", "County": "Hempstead", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71862", "FIPS": "05057", "County": "Hempstead", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71929", "FIPS": "05059", "County": "Hot Spring", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71933", "FIPS": "05059", "County": "Hot Spring", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71941", "FIPS": "05059", "County": "Hot Spring", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71942", "FIPS": "05059", "County": "Hot Spring", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72104", "FIPS": "05059", "County": "Hot Spring", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72105", "FIPS": "05059", "County": "Hot Spring", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71833", "FIPS": "05061", "County": "Howard", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71851", "FIPS": "05061", "County": "Howard", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71852", "FIPS": "05061", "County": "Howard", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71859", "FIPS": "05061", "County": "Howard", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71971", "FIPS": "05061", "County": "Howard", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "72165", "FIPS": "05063", "County": "Independence", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72501", "FIPS": "05063", "County": "Independence", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72503", "FIPS": "05063", "County": "Independence", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72522", "FIPS": "05063", "County": "Independence", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72524", "FIPS": "05063", "County": "Independence", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72526", "FIPS": "05063", "County": "Independence", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72527", "FIPS": "05063", "County": "Independence", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72534", "FIPS": "05063", "County": "Independence", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72550", "FIPS": "05063", "County": "Independence", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72553", "FIPS": "05063", "County": "Independence", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72562", "FIPS": "05063", "County": "Independence", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72564", "FIPS": "05063", "County": "Independence", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72568", "FIPS": "05063", "County": "Independence", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72571", "FIPS": "05063", "County": "Independence", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72575", "FIPS": "05063", "County": "Independence", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72579", "FIPS": "05063", "County": "Independence", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72512", "FIPS": "05065", "County": "Izard", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72517", "FIPS": "05065", "County": "Izard", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72519", "FIPS": "05065", "County": "Izard", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72528", "FIPS": "05065", "County": "Izard", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72536", "FIPS": "05065", "County": "Izard", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72540", "FIPS": "05065", "County": "Izard", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72556", "FIPS": "05065", "County": "Izard", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72561", "FIPS": "05065", "County": "Izard", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72565", "FIPS": "05065", "County": "Izard", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72566", "FIPS": "05065", "County": "Izard", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72573", "FIPS": "05065", "County": "Izard", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72584", "FIPS": "05065", "County": "Izard", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72585", "FIPS": "05065", "County": "Izard", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72587", "FIPS": "05065", "County": "Izard", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72005", "FIPS": "05067", "County": "Jackson", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72014", "FIPS": "05067", "County": "Jackson", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72043", "FIPS": "05067", "County": "Jackson", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72075", "FIPS": "05067", "County": "Jackson", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72112", "FIPS": "05067", "County": "Jackson", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72169", "FIPS": "05067", "County": "Jackson", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72431", "FIPS": "05067", "County": "Jackson", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72471", "FIPS": "05067", "County": "Jackson", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72473", "FIPS": "05067", "County": "Jackson", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71601", "FIPS": "05069", "County": "Jefferson", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71602", "FIPS": "05069", "County": "Jefferson", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71603", "FIPS": "05069", "County": "Jefferson", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71611", "FIPS": "05069", "County": "Jefferson", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71612", "FIPS": "05069", "County": "Jefferson", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71613", "FIPS": "05069", "County": "Jefferson", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71659", "FIPS": "05069", "County": "Jefferson", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72004", "FIPS": "05069", "County": "Jefferson", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72079", "FIPS": "05069", "County": "Jefferson", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72132", "FIPS": "05069", "County": "Jefferson", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72133", "FIPS": "05069", "County": "Jefferson", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72152", "FIPS": "05069", "County": "Jefferson", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72168", "FIPS": "05069", "County": "Jefferson", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72175", "FIPS": "05069", "County": "Jefferson", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72182", "FIPS": "05069", "County": "Jefferson", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72830", "FIPS": "05071", "County": "Johnson", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72832", "FIPS": "05071", "County": "Johnson", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72839", "FIPS": "05071", "County": "Johnson", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72840", "FIPS": "05071", "County": "Johnson", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72845", "FIPS": "05071", "County": "Johnson", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72846", "FIPS": "05071", "County": "Johnson", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72852", "FIPS": "05071", "County": "Johnson", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72854", "FIPS": "05071", "County": "Johnson", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "71826", "FIPS": "05073", "County": "Lafayette", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71827", "FIPS": "05073", "County": "Lafayette", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71845", "FIPS": "05073", "County": "Lafayette", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71860", "FIPS": "05073", "County": "Lafayette", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "72410", "FIPS": "05075", "County": "Lawrence", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72415", "FIPS": "05075", "County": "Lawrence", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72433", "FIPS": "05075", "County": "Lawrence", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72434", "FIPS": "05075", "County": "Lawrence", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72440", "FIPS": "05075", "County": "Lawrence", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72445", "FIPS": "05075", "County": "Lawrence", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72457", "FIPS": "05075", "County": "Lawrence", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72458", "FIPS": "05075", "County": "Lawrence", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72459", "FIPS": "05075", "County": "Lawrence", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72465", "FIPS": "05075", "County": "Lawrence", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72466", "FIPS": "05075", "County": "Lawrence", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72469", "FIPS": "05075", "County": "Lawrence", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72476", "FIPS": "05075", "County": "Lawrence", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72572", "FIPS": "05075", "County": "Lawrence", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72311", "FIPS": "05077", "County": "Lee", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72320", "FIPS": "05077", "County": "Lee", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72341", "FIPS": "05077", "County": "Lee", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72352", "FIPS": "05077", "County": "Lee", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72360", "FIPS": "05077", "County": "Lee", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72368", "FIPS": "05077", "County": "Lee", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "71643", "FIPS": "05079", "County": "Lincoln", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71644", "FIPS": "05079", "County": "Lincoln", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71667", "FIPS": "05079", "County": "Lincoln", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71678", "FIPS": "05079", "County": "Lincoln", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71820", "FIPS": "05081", "County": "Little River", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71822", "FIPS": "05081", "County": "Little River", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71836", "FIPS": "05081", "County": "Little River", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71853", "FIPS": "05081", "County": "Little River", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71865", "FIPS": "05081", "County": "Little River", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71866", "FIPS": "05081", "County": "Little River", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "72826", "FIPS": "05083", "County": "Logan", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72835", "FIPS": "05083", "County": "Logan", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72851", "FIPS": "05083", "County": "Logan", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72855", "FIPS": "05083", "County": "Logan", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72863", "FIPS": "05083", "County": "Logan", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72865", "FIPS": "05083", "County": "Logan", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72927", "FIPS": "05083", "County": "Logan", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72943", "FIPS": "05083", "County": "Logan", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72951", "FIPS": "05083", "County": "Logan", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72007", "FIPS": "05085", "County": "Lonoke", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72023", "FIPS": "05085", "County": "Lonoke", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72024", "FIPS": "05085", "County": "Lonoke", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72037", "FIPS": "05085", "County": "Lonoke", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72046", "FIPS": "05085", "County": "Lonoke", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72072", "FIPS": "05085", "County": "Lonoke", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72083", "FIPS": "05085", "County": "Lonoke", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72086", "FIPS": "05085", "County": "Lonoke", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72176", "FIPS": "05085", "County": "Lonoke", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72721", "FIPS": "05087", "County": "Madison", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72738", "FIPS": "05087", "County": "Madison", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72740", "FIPS": "05087", "County": "Madison", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72742", "FIPS": "05087", "County": "Madison", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72752", "FIPS": "05087", "County": "Madison", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72760", "FIPS": "05087", "County": "Madison", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72773", "FIPS": "05087", "County": "Madison", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72776", "FIPS": "05087", "County": "Madison", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72619", "FIPS": "05089", "County": "Marion", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72634", "FIPS": "05089", "County": "Marion", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72661", "FIPS": "05089", "County": "Marion", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72668", "FIPS": "05089", "County": "Marion", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72672", "FIPS": "05089", "County": "Marion", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72677", "FIPS": "05089", "County": "Marion", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72687", "FIPS": "05089", "County": "Marion", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "71834", "FIPS": "05091", "County": "Miller", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71837", "FIPS": "05091", "County": "Miller", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71839", "FIPS": "05091", "County": "Miller", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71840", "FIPS": "05091", "County": "Miller", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71854", "FIPS": "05091", "County": "Miller", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "72310", "FIPS": "05093", "County": "Mississippi", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72313", "FIPS": "05093", "County": "Mississippi", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72315", "FIPS": "05093", "County": "Mississippi", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72316", "FIPS": "05093", "County": "Mississippi", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72319", "FIPS": "05093", "County": "Mississippi", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72321", "FIPS": "05093", "County": "Mississippi", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72329", "FIPS": "05093", "County": "Mississippi", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72330", "FIPS": "05093", "County": "Mississippi", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72338", "FIPS": "05093", "County": "Mississippi", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72350", "FIPS": "05093", "County": "Mississippi", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72351", "FIPS": "05093", "County": "Mississippi", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72358", "FIPS": "05093", "County": "Mississippi", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72370", "FIPS": "05093", "County": "Mississippi", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72391", "FIPS": "05093", "County": "Mississippi", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72395", "FIPS": "05093", "County": "Mississippi", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72426", "FIPS": "05093", "County": "Mississippi", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72428", "FIPS": "05093", "County": "Mississippi", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72438", "FIPS": "05093", "County": "Mississippi", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72442", "FIPS": "05093", "County": "Mississippi", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72021", "FIPS": "05095", "County": "Monroe", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72029", "FIPS": "05095", "County": "Monroe", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72069", "FIPS": "05095", "County": "Monroe", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72108", "FIPS": "05095", "County": "Monroe", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72134", "FIPS": "05095", "County": "Monroe", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71935", "FIPS": "05097", "County": "Montgomery", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71957", "FIPS": "05097", "County": "Montgomery", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71960", "FIPS": "05097", "County": "Montgomery", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71961", "FIPS": "05097", "County": "Montgomery", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71965", "FIPS": "05097", "County": "Montgomery", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71966", "FIPS": "05097", "County": "Montgomery", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71969", "FIPS": "05097", "County": "Montgomery", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71970", "FIPS": "05097", "County": "Montgomery", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71722", "FIPS": "05099", "County": "Nevada", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71828", "FIPS": "05099", "County": "Nevada", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71835", "FIPS": "05099", "County": "Nevada", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71844", "FIPS": "05099", "County": "Nevada", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71857", "FIPS": "05099", "County": "Nevada", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71858", "FIPS": "05099", "County": "Nevada", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71864", "FIPS": "05099", "County": "Nevada", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72624", "FIPS": "05101", "County": "Newton", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72628", "FIPS": "05101", "County": "Newton", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72640", "FIPS": "05101", "County": "Newton", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72641", "FIPS": "05101", "County": "Newton", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72648", "FIPS": "05101", "County": "Newton", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72655", "FIPS": "05101", "County": "Newton", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72666", "FIPS": "05101", "County": "Newton", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72670", "FIPS": "05101", "County": "Newton", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72683", "FIPS": "05101", "County": "Newton", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72685", "FIPS": "05101", "County": "Newton", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "72856", "FIPS": "05101", "County": "Newton", "State": "AR", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "71701", "FIPS": "05103", "County": "Ouachita", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71711", "FIPS": "05103", "County": "Ouachita", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71720", "FIPS": "05103", "County": "Ouachita", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71726", "FIPS": "05103", "County": "Ouachita", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71751", "FIPS": "05103", "County": "Ouachita", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71764", "FIPS": "05103", "County": "Ouachita", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72001", "FIPS": "05105", "County": "Perry", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72016", "FIPS": "05105", "County": "Perry", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72025", "FIPS": "05105", "County": "Perry", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72070", "FIPS": "05105", "County": "Perry", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72125", "FIPS": "05105", "County": "Perry", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72126", "FIPS": "05105", "County": "Perry", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72312", "FIPS": "05107", "County": "Phillips", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72328", "FIPS": "05107", "County": "Phillips", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72333", "FIPS": "05107", "County": "Phillips", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72342", "FIPS": "05107", "County": "Phillips", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72353", "FIPS": "05107", "County": "Phillips", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72355", "FIPS": "05107", "County": "Phillips", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72366", "FIPS": "05107", "County": "Phillips", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72367", "FIPS": "05107", "County": "Phillips", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72369", "FIPS": "05107", "County": "Phillips", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72374", "FIPS": "05107", "County": "Phillips", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72383", "FIPS": "05107", "County": "Phillips", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72389", "FIPS": "05107", "County": "Phillips", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72390", "FIPS": "05107", "County": "Phillips", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "71922", "FIPS": "05109", "County": "Pike", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71940", "FIPS": "05109", "County": "Pike", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71943", "FIPS": "05109", "County": "Pike", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71950", "FIPS": "05109", "County": "Pike", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71952", "FIPS": "05109", "County": "Pike", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71958", "FIPS": "05109", "County": "Pike", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71959", "FIPS": "05109", "County": "Pike", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72354", "FIPS": "05111", "County": "Poinsett", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72365", "FIPS": "05111", "County": "Poinsett", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72377", "FIPS": "05111", "County": "Poinsett", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72386", "FIPS": "05111", "County": "Poinsett", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72429", "FIPS": "05111", "County": "Poinsett", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72432", "FIPS": "05111", "County": "Poinsett", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72472", "FIPS": "05111", "County": "Poinsett", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72475", "FIPS": "05111", "County": "Poinsett", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72479", "FIPS": "05111", "County": "Poinsett", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "71932", "FIPS": "05113", "County": "Polk", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71937", "FIPS": "05113", "County": "Polk", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71944", "FIPS": "05113", "County": "Polk", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71945", "FIPS": "05113", "County": "Polk", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71953", "FIPS": "05113", "County": "Polk", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71972", "FIPS": "05113", "County": "Polk", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71973", "FIPS": "05113", "County": "Polk", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72679", "FIPS": "05115", "County": "Pope", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72801", "FIPS": "05115", "County": "Pope", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72802", "FIPS": "05115", "County": "Pope", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72811", "FIPS": "05115", "County": "Pope", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72812", "FIPS": "05115", "County": "Pope", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72823", "FIPS": "05115", "County": "Pope", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72837", "FIPS": "05115", "County": "Pope", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72843", "FIPS": "05115", "County": "Pope", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72847", "FIPS": "05115", "County": "Pope", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72858", "FIPS": "05115", "County": "Pope", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72017", "FIPS": "05117", "County": "Prairie", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72040", "FIPS": "05117", "County": "Prairie", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72041", "FIPS": "05117", "County": "Prairie", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72064", "FIPS": "05117", "County": "Prairie", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72066", "FIPS": "05117", "County": "Prairie", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72170", "FIPS": "05117", "County": "Prairie", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72053", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72076", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72078", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72099", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72113", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72114", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72115", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72116", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72117", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72118", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72119", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72120", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72124", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72135", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72142", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72164", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72180", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72183", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72190", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72198", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72199", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72201", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72202", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72203", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72204", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72205", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72206", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72207", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72209", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72210", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72211", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72212", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72214", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72215", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72216", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72217", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72219", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72221", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72222", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72223", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72225", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72227", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72231", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72260", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72295", "FIPS": "05119", "County": "Pulaski", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72413", "FIPS": "05121", "County": "Randolph", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72444", "FIPS": "05121", "County": "Randolph", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72449", "FIPS": "05121", "County": "Randolph", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72455", "FIPS": "05121", "County": "Randolph", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72460", "FIPS": "05121", "County": "Randolph", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72462", "FIPS": "05121", "County": "Randolph", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72478", "FIPS": "05121", "County": "Randolph", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72322", "FIPS": "05123", "County": "Saint Francis", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72326", "FIPS": "05123", "County": "Saint Francis", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72335", "FIPS": "05123", "County": "Saint Francis", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72336", "FIPS": "05123", "County": "Saint Francis", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72340", "FIPS": "05123", "County": "Saint Francis", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72346", "FIPS": "05123", "County": "Saint Francis", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72348", "FIPS": "05123", "County": "Saint Francis", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72359", "FIPS": "05123", "County": "Saint Francis", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72372", "FIPS": "05123", "County": "Saint Francis", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72392", "FIPS": "05123", "County": "Saint Francis", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72394", "FIPS": "05123", "County": "Saint Francis", "State": "AR", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "72002", "FIPS": "05125", "County": "Saline", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72011", "FIPS": "05125", "County": "Saline", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72015", "FIPS": "05125", "County": "Saline", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72018", "FIPS": "05125", "County": "Saline", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72019", "FIPS": "05125", "County": "Saline", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72022", "FIPS": "05125", "County": "Saline", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72065", "FIPS": "05125", "County": "Saline", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72089", "FIPS": "05125", "County": "Saline", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72103", "FIPS": "05125", "County": "Saline", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72122", "FIPS": "05125", "County": "Saline", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72158", "FIPS": "05125", "County": "Saline", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72167", "FIPS": "05125", "County": "Saline", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72841", "FIPS": "05127", "County": "Scott", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72926", "FIPS": "05127", "County": "Scott", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72944", "FIPS": "05127", "County": "Scott", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72950", "FIPS": "05127", "County": "Scott", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72958", "FIPS": "05127", "County": "Scott", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72636", "FIPS": "05129", "County": "Searcy", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72639", "FIPS": "05129", "County": "Searcy", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72645", "FIPS": "05129", "County": "Searcy", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72650", "FIPS": "05129", "County": "Searcy", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72669", "FIPS": "05129", "County": "Searcy", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72675", "FIPS": "05129", "County": "Searcy", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72686", "FIPS": "05129", "County": "Searcy", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72901", "FIPS": "05131", "County": "Sebastian", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72902", "FIPS": "05131", "County": "Sebastian", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72903", "FIPS": "05131", "County": "Sebastian", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72904", "FIPS": "05131", "County": "Sebastian", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72905", "FIPS": "05131", "County": "Sebastian", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72906", "FIPS": "05131", "County": "Sebastian", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72908", "FIPS": "05131", "County": "Sebastian", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72913", "FIPS": "05131", "County": "Sebastian", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72914", "FIPS": "05131", "County": "Sebastian", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72916", "FIPS": "05131", "County": "Sebastian", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72917", "FIPS": "05131", "County": "Sebastian", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72918", "FIPS": "05131", "County": "Sebastian", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72919", "FIPS": "05131", "County": "Sebastian", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72923", "FIPS": "05131", "County": "Sebastian", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72936", "FIPS": "05131", "County": "Sebastian", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72937", "FIPS": "05131", "County": "Sebastian", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72938", "FIPS": "05131", "County": "Sebastian", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72940", "FIPS": "05131", "County": "Sebastian", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72941", "FIPS": "05131", "County": "Sebastian", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72945", "FIPS": "05131", "County": "Sebastian", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "71823", "FIPS": "05133", "County": "Sevier", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71832", "FIPS": "05133", "County": "Sevier", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71841", "FIPS": "05133", "County": "Sevier", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71842", "FIPS": "05133", "County": "Sevier", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71846", "FIPS": "05133", "County": "Sevier", "State": "AR", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "72482", "FIPS": "05135", "County": "Sharp", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72513", "FIPS": "05135", "County": "Sharp", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72521", "FIPS": "05135", "County": "Sharp", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72525", "FIPS": "05135", "County": "Sharp", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72529", "FIPS": "05135", "County": "Sharp", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72532", "FIPS": "05135", "County": "Sharp", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72542", "FIPS": "05135", "County": "Sharp", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72569", "FIPS": "05135", "County": "Sharp", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72577", "FIPS": "05135", "County": "Sharp", "State": "AR", "DMA": {"Code": 734, "Name": "JONESBORO"}},
  {"Postal_Code": "72051", "FIPS": "05137", "County": "Stone", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72533", "FIPS": "05137", "County": "Stone", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72555", "FIPS": "05137", "County": "Stone", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72560", "FIPS": "05137", "County": "Stone", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72567", "FIPS": "05137", "County": "Stone", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72657", "FIPS": "05137", "County": "Stone", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72663", "FIPS": "05137", "County": "Stone", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72680", "FIPS": "05137", "County": "Stone", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "71724", "FIPS": "05139", "County": "Union", "State": "AR", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71730", "FIPS": "05139", "County": "Union", "State": "AR", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71731", "FIPS": "05139", "County": "Union", "State": "AR", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71747", "FIPS": "05139", "County": "Union", "State": "AR", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71749", "FIPS": "05139", "County": "Union", "State": "AR", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71750", "FIPS": "05139", "County": "Union", "State": "AR", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71758", "FIPS": "05139", "County": "Union", "State": "AR", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71759", "FIPS": "05139", "County": "Union", "State": "AR", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71762", "FIPS": "05139", "County": "Union", "State": "AR", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71765", "FIPS": "05139", "County": "Union", "State": "AR", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71768", "FIPS": "05139", "County": "Union", "State": "AR", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "72013", "FIPS": "05141", "County": "Van Buren", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72028", "FIPS": "05141", "County": "Van Buren", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72031", "FIPS": "05141", "County": "Van Buren", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72088", "FIPS": "05141", "County": "Van Buren", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72141", "FIPS": "05141", "County": "Van Buren", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72153", "FIPS": "05141", "County": "Van Buren", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72629", "FIPS": "05141", "County": "Van Buren", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72701", "FIPS": "05143", "County": "Washington", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72702", "FIPS": "05143", "County": "Washington", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72703", "FIPS": "05143", "County": "Washington", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72704", "FIPS": "05143", "County": "Washington", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72717", "FIPS": "05143", "County": "Washington", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72727", "FIPS": "05143", "County": "Washington", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72728", "FIPS": "05143", "County": "Washington", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72729", "FIPS": "05143", "County": "Washington", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72730", "FIPS": "05143", "County": "Washington", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72735", "FIPS": "05143", "County": "Washington", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72737", "FIPS": "05143", "County": "Washington", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72741", "FIPS": "05143", "County": "Washington", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72744", "FIPS": "05143", "County": "Washington", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72749", "FIPS": "05143", "County": "Washington", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72753", "FIPS": "05143", "County": "Washington", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72762", "FIPS": "05143", "County": "Washington", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72764", "FIPS": "05143", "County": "Washington", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72765", "FIPS": "05143", "County": "Washington", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72766", "FIPS": "05143", "County": "Washington", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72769", "FIPS": "05143", "County": "Washington", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72770", "FIPS": "05143", "County": "Washington", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72774", "FIPS": "05143", "County": "Washington", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72959", "FIPS": "05143", "County": "Washington", "State": "AR", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "72010", "FIPS": "05145", "County": "White", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72012", "FIPS": "05145", "County": "White", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72020", "FIPS": "05145", "County": "White", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72045", "FIPS": "05145", "County": "White", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72052", "FIPS": "05145", "County": "White", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72060", "FIPS": "05145", "County": "White", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72068", "FIPS": "05145", "County": "White", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72081", "FIPS": "05145", "County": "White", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72082", "FIPS": "05145", "County": "White", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72085", "FIPS": "05145", "County": "White", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72102", "FIPS": "05145", "County": "White", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72121", "FIPS": "05145", "County": "White", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72136", "FIPS": "05145", "County": "White", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72137", "FIPS": "05145", "County": "White", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72139", "FIPS": "05145", "County": "White", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72143", "FIPS": "05145", "County": "White", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72145", "FIPS": "05145", "County": "White", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72149", "FIPS": "05145", "County": "White", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72178", "FIPS": "05145", "County": "White", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72006", "FIPS": "05147", "County": "Woodruff", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72036", "FIPS": "05147", "County": "Woodruff", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72059", "FIPS": "05147", "County": "Woodruff", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72074", "FIPS": "05147", "County": "Woodruff", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72101", "FIPS": "05147", "County": "Woodruff", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72123", "FIPS": "05147", "County": "Woodruff", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72189", "FIPS": "05147", "County": "Woodruff", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72824", "FIPS": "05149", "County": "Yell", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72827", "FIPS": "05149", "County": "Yell", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72828", "FIPS": "05149", "County": "Yell", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72829", "FIPS": "05149", "County": "Yell", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72833", "FIPS": "05149", "County": "Yell", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72834", "FIPS": "05149", "County": "Yell", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72838", "FIPS": "05149", "County": "Yell", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72842", "FIPS": "05149", "County": "Yell", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72853", "FIPS": "05149", "County": "Yell", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72857", "FIPS": "05149", "County": "Yell", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "72860", "FIPS": "05149", "County": "Yell", "State": "AR", "DMA": {"Code": 693, "Name": "LITTLE ROCK-PINE BLUFF"}},
  {"Postal_Code": "94501", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94502", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94536", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94537", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94538", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94539", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94540", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94541", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94542", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94543", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94544", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94545", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94546", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94550", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94551", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94552", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94555", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94557", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94560", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94566", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94568", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94577", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94578", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94579", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94580", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94586", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94587", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94588", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94601", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94602", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94603", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94604", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94605", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94606", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94607", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94608", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94609", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94610", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94611", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94612", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94613", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94614", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94615", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94617", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94618", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94619", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94620", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94621", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94622", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94623", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94624", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94625", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94649", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94659", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94660", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94661", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94662", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94666", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94701", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94702", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94703", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94704", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94705", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94706", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94707", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94708", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94709", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94710", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94712", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94720", "FIPS": "06001", "County": "Alameda", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95646", "FIPS": "06003", "County": "Alpine", "State": "CA", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "96120", "FIPS": "06003", "County": "Alpine", "State": "CA", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "95601", "FIPS": "06005", "County": "Amador", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95629", "FIPS": "06005", "County": "Amador", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95640", "FIPS": "06005", "County": "Amador", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95642", "FIPS": "06005", "County": "Amador", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95644", "FIPS": "06005", "County": "Amador", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95654", "FIPS": "06005", "County": "Amador", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95665", "FIPS": "06005", "County": "Amador", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95666", "FIPS": "06005", "County": "Amador", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95669", "FIPS": "06005", "County": "Amador", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95675", "FIPS": "06005", "County": "Amador", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95685", "FIPS": "06005", "County": "Amador", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95689", "FIPS": "06005", "County": "Amador", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95699", "FIPS": "06005", "County": "Amador", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95914", "FIPS": "06007", "County": "Butte", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95916", "FIPS": "06007", "County": "Butte", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95917", "FIPS": "06007", "County": "Butte", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95926", "FIPS": "06007", "County": "Butte", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95927", "FIPS": "06007", "County": "Butte", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95928", "FIPS": "06007", "County": "Butte", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95929", "FIPS": "06007", "County": "Butte", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95930", "FIPS": "06007", "County": "Butte", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95938", "FIPS": "06007", "County": "Butte", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95940", "FIPS": "06007", "County": "Butte", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95941", "FIPS": "06007", "County": "Butte", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95942", "FIPS": "06007", "County": "Butte", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95948", "FIPS": "06007", "County": "Butte", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95954", "FIPS": "06007", "County": "Butte", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95958", "FIPS": "06007", "County": "Butte", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95965", "FIPS": "06007", "County": "Butte", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95966", "FIPS": "06007", "County": "Butte", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95967", "FIPS": "06007", "County": "Butte", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95968", "FIPS": "06007", "County": "Butte", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95969", "FIPS": "06007", "County": "Butte", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95973", "FIPS": "06007", "County": "Butte", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95974", "FIPS": "06007", "County": "Butte", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95976", "FIPS": "06007", "County": "Butte", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95978", "FIPS": "06007", "County": "Butte", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95221", "FIPS": "06009", "County": "Calaveras", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95222", "FIPS": "06009", "County": "Calaveras", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95223", "FIPS": "06009", "County": "Calaveras", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95224", "FIPS": "06009", "County": "Calaveras", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95225", "FIPS": "06009", "County": "Calaveras", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95226", "FIPS": "06009", "County": "Calaveras", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95228", "FIPS": "06009", "County": "Calaveras", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95229", "FIPS": "06009", "County": "Calaveras", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95232", "FIPS": "06009", "County": "Calaveras", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95233", "FIPS": "06009", "County": "Calaveras", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95245", "FIPS": "06009", "County": "Calaveras", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95246", "FIPS": "06009", "County": "Calaveras", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95247", "FIPS": "06009", "County": "Calaveras", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95248", "FIPS": "06009", "County": "Calaveras", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95249", "FIPS": "06009", "County": "Calaveras", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95250", "FIPS": "06009", "County": "Calaveras", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95251", "FIPS": "06009", "County": "Calaveras", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95252", "FIPS": "06009", "County": "Calaveras", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95254", "FIPS": "06009", "County": "Calaveras", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95255", "FIPS": "06009", "County": "Calaveras", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95257", "FIPS": "06009", "County": "Calaveras", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95912", "FIPS": "06011", "County": "Colusa", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95932", "FIPS": "06011", "County": "Colusa", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95950", "FIPS": "06011", "County": "Colusa", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95955", "FIPS": "06011", "County": "Colusa", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95970", "FIPS": "06011", "County": "Colusa", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95979", "FIPS": "06011", "County": "Colusa", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95987", "FIPS": "06011", "County": "Colusa", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94506", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94507", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94509", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94511", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94513", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94514", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94516", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94517", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94518", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94519", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94520", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94521", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94522", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94523", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94524", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94525", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94526", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94527", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94528", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94529", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94530", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94531", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94547", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94548", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94549", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94553", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94556", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94561", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94563", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94564", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94565", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94569", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94570", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94572", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94575", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94582", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94583", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94595", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94596", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94597", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94598", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94801", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94802", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94803", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94804", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94805", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94806", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94807", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94808", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94820", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94850", "FIPS": "06013", "County": "Contra Costa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95531", "FIPS": "06015", "County": "Del Norte", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95532", "FIPS": "06015", "County": "Del Norte", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95538", "FIPS": "06015", "County": "Del Norte", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95543", "FIPS": "06015", "County": "Del Norte", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95548", "FIPS": "06015", "County": "Del Norte", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95567", "FIPS": "06015", "County": "Del Norte", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95613", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95614", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95619", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95623", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95633", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95634", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95635", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95636", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95651", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95656", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95664", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95667", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95672", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95682", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95684", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95709", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95720", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95721", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95726", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95735", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95762", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96142", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96150", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96151", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96152", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96154", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96155", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96156", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96157", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96158", "FIPS": "06017", "County": "El Dorado", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "93210", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93234", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93242", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93602", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93605", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93606", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93607", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93608", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93609", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93611", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93612", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93613", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93616", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93619", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93621", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93622", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93624", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93625", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93626", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93627", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93628", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93630", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93631", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93634", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93640", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93641", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93642", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93646", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93648", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93649", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93650", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93651", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93652", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93654", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93656", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93657", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93660", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93662", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93664", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93667", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93668", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93675", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93701", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93702", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93703", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93704", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93705", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93706", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93707", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93708", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93709", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93710", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93711", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93712", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93714", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93715", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93716", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93717", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93718", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93720", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93721", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93722", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93723", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93724", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93725", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93726", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93727", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93728", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93729", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93730", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93740", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93741", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93744", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93745", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93747", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93750", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93755", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93760", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93761", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93764", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93765", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93771", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93772", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93773", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93774", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93775", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93776", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93777", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93778", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93779", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93780", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93784", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93786", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93790", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93791", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93792", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93793", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93794", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93844", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93888", "FIPS": "06019", "County": "Fresno", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95913", "FIPS": "06021", "County": "Glenn", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95920", "FIPS": "06021", "County": "Glenn", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95939", "FIPS": "06021", "County": "Glenn", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95943", "FIPS": "06021", "County": "Glenn", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95951", "FIPS": "06021", "County": "Glenn", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95963", "FIPS": "06021", "County": "Glenn", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95988", "FIPS": "06021", "County": "Glenn", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95501", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95502", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95503", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95511", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95514", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95518", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95519", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95521", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95524", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95525", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95526", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95528", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95534", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95536", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95537", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95540", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95542", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95545", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95546", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95547", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95549", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95550", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95551", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95553", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95554", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95555", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95556", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95558", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95559", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95560", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95562", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95564", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95565", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95569", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95570", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95571", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95573", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "95589", "FIPS": "06023", "County": "Humboldt", "State": "CA", "DMA": {"Code": 802, "Name": "EUREKA"}},
  {"Postal_Code": "92222", "FIPS": "06025", "County": "Imperial", "State": "CA", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "92227", "FIPS": "06025", "County": "Imperial", "State": "CA", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "92231", "FIPS": "06025", "County": "Imperial", "State": "CA", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "92232", "FIPS": "06025", "County": "Imperial", "State": "CA", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "92233", "FIPS": "06025", "County": "Imperial", "State": "CA", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "92243", "FIPS": "06025", "County": "Imperial", "State": "CA", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "92244", "FIPS": "06025", "County": "Imperial", "State": "CA", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "92249", "FIPS": "06025", "County": "Imperial", "State": "CA", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "92250", "FIPS": "06025", "County": "Imperial", "State": "CA", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "92251", "FIPS": "06025", "County": "Imperial", "State": "CA", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "92257", "FIPS": "06025", "County": "Imperial", "State": "CA", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "92259", "FIPS": "06025", "County": "Imperial", "State": "CA", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "92266", "FIPS": "06025", "County": "Imperial", "State": "CA", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "92273", "FIPS": "06025", "County": "Imperial", "State": "CA", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "92275", "FIPS": "06025", "County": "Imperial", "State": "CA", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "92281", "FIPS": "06025", "County": "Imperial", "State": "CA", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "92283", "FIPS": "06025", "County": "Imperial", "State": "CA", "DMA": {"Code": 771, "Name": "YUMA-EL CENTRO"}},
  {"Postal_Code": "92328", "FIPS": "06027", "County": "Inyo", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92384", "FIPS": "06027", "County": "Inyo", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92389", "FIPS": "06027", "County": "Inyo", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93513", "FIPS": "06027", "County": "Inyo", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93514", "FIPS": "06027", "County": "Inyo", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93515", "FIPS": "06027", "County": "Inyo", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93522", "FIPS": "06027", "County": "Inyo", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93526", "FIPS": "06027", "County": "Inyo", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93530", "FIPS": "06027", "County": "Inyo", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93542", "FIPS": "06027", "County": "Inyo", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93545", "FIPS": "06027", "County": "Inyo", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93549", "FIPS": "06027", "County": "Inyo", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93203", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93205", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93206", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93215", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93216", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93220", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93222", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93224", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93225", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93226", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93238", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93240", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93241", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93243", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93249", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93250", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93251", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93252", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93255", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93263", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93268", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93276", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93280", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93283", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93285", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93287", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93301", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93302", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93303", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93304", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93305", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93306", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93307", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93308", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93309", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93311", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93312", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93313", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93314", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93380", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93381", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93382", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93383", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93384", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93385", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93386", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93387", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93388", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93389", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93390", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93501", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93502", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93504", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93505", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93516", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93518", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93519", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93523", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93524", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93527", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93528", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93531", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93554", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93555", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93556", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93560", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93561", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93581", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93596", "FIPS": "06029", "County": "Kern", "State": "CA", "DMA": {"Code": 800, "Name": "BAKERSFIELD"}},
  {"Postal_Code": "93202", "FIPS": "06031", "County": "Kings", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93204", "FIPS": "06031", "County": "Kings", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93212", "FIPS": "06031", "County": "Kings", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93230", "FIPS": "06031", "County": "Kings", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93232", "FIPS": "06031", "County": "Kings", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93239", "FIPS": "06031", "County": "Kings", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93245", "FIPS": "06031", "County": "Kings", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93246", "FIPS": "06031", "County": "Kings", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93266", "FIPS": "06031", "County": "Kings", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95422", "FIPS": "06033", "County": "Lake", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95423", "FIPS": "06033", "County": "Lake", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95424", "FIPS": "06033", "County": "Lake", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95426", "FIPS": "06033", "County": "Lake", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95435", "FIPS": "06033", "County": "Lake", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95443", "FIPS": "06033", "County": "Lake", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95451", "FIPS": "06033", "County": "Lake", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95453", "FIPS": "06033", "County": "Lake", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95457", "FIPS": "06033", "County": "Lake", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95458", "FIPS": "06033", "County": "Lake", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95461", "FIPS": "06033", "County": "Lake", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95464", "FIPS": "06033", "County": "Lake", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95467", "FIPS": "06033", "County": "Lake", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95485", "FIPS": "06033", "County": "Lake", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95493", "FIPS": "06033", "County": "Lake", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "96009", "FIPS": "06035", "County": "Lassen", "State": "CA", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "96056", "FIPS": "06035", "County": "Lassen", "State": "CA", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "96068", "FIPS": "06035", "County": "Lassen", "State": "CA", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "96109", "FIPS": "06035", "County": "Lassen", "State": "CA", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "96113", "FIPS": "06035", "County": "Lassen", "State": "CA", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "96114", "FIPS": "06035", "County": "Lassen", "State": "CA", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "96117", "FIPS": "06035", "County": "Lassen", "State": "CA", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "96119", "FIPS": "06035", "County": "Lassen", "State": "CA", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "96121", "FIPS": "06035", "County": "Lassen", "State": "CA", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "96123", "FIPS": "06035", "County": "Lassen", "State": "CA", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "96127", "FIPS": "06035", "County": "Lassen", "State": "CA", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "96128", "FIPS": "06035", "County": "Lassen", "State": "CA", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "96130", "FIPS": "06035", "County": "Lassen", "State": "CA", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "96132", "FIPS": "06035", "County": "Lassen", "State": "CA", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "96136", "FIPS": "06035", "County": "Lassen", "State": "CA", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "90001", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90002", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90003", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90004", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90005", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90006", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90007", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90008", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90009", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90010", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90011", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90012", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90013", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90014", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90015", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90016", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90017", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90018", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90019", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90020", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90021", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90022", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90023", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90024", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90025", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90026", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90027", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90028", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90029", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90030", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90031", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90032", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90033", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90034", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90035", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90036", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90037", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90038", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90039", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90040", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90041", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90042", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90043", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90044", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90045", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90046", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90047", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90048", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90049", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90050", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90051", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90052", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90053", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90054", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90055", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90056", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90057", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90058", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90059", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90060", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90061", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90062", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90063", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90064", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90065", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90066", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90067", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90068", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90069", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90070", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90071", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90072", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90073", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90074", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90075", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90076", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90077", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90078", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90079", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90080", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90081", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90082", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90083", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90084", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90086", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90087", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90088", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90089", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90091", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90093", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90094", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90095", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90096", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90099", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90101", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90102", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90103", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90189", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90201", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90202", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90209", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90210", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90211", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90212", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90213", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90220", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90221", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90222", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90223", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90224", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90230", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90231", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90232", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90233", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90239", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90240", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90241", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90242", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90245", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90247", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90248", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90249", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90250", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90251", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90254", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90255", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90260", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90261", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90262", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90263", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90264", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90265", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90266", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90267", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90270", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90272", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90274", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90275", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90277", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90278", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90280", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90290", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90291", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90292", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90293", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90294", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90295", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90296", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90301", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90302", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90303", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90304", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90305", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90306", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90307", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90308", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90309", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90310", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90311", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90312", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90313", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90397", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90398", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90401", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90402", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90403", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90404", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90405", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90406", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90407", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90408", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90409", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90410", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90411", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90501", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90502", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90503", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90504", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90505", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90506", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90507", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90508", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90509", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90510", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90601", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90602", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90603", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90604", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90605", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90606", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90607", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90608", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90609", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90610", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90612", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90637", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90638", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90639", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90640", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90650", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90651", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90652", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90659", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90660", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90661", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90662", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90670", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90671", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90701", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90702", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90703", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90704", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90706", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90707", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90710", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90711", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90712", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90713", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90714", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90715", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90716", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90717", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90723", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90731", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90732", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90733", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90734", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90744", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90745", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90746", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90747", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90748", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90749", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90755", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90801", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90802", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90803", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90804", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90805", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90806", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90807", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90808", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90809", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90810", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90813", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90814", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90815", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90822", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90831", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90832", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90833", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90834", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90835", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90840", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90842", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90844", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90845", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90846", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90847", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90848", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90853", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90888", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90895", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90899", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91001", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91003", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91006", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91007", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91009", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91010", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91011", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91012", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91016", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91017", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91020", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91021", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91023", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91024", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91025", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91030", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91031", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91040", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91041", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91042", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91043", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91046", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91066", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91077", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91101", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91102", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91103", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91104", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91105", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91106", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91107", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91108", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91109", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91110", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91114", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91115", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91116", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91117", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91118", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91121", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91123", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91124", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91125", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91126", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91129", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91131", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91182", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91184", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91185", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91188", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91189", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91191", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91199", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91201", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91202", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91203", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91204", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91205", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91206", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91207", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91208", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91209", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91210", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91214", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91221", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91222", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91224", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91225", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91226", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91301", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91302", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91303", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91304", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91305", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91306", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91307", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91308", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91309", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91310", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91311", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91313", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91316", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91321", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91322", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91324", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91325", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91326", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91327", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91328", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91329", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91330", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91331", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91333", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91334", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91335", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91337", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91340", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91341", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91342", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91343", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91344", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91345", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91346", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91350", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91351", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91352", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91353", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91354", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91355", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91356", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91357", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91363", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91364", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91365", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91367", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91371", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91372", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91376", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91380", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91381", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91382", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91383", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91384", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91385", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91386", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91387", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91388", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91390", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91392", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91393", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91394", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91395", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91396", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91399", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91401", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91402", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91403", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91404", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91405", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91406", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91407", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91408", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91409", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91410", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91411", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91412", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91413", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91416", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91423", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91426", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91436", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91470", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91482", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91495", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91496", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91497", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91499", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91501", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91502", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91503", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91504", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91505", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91506", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91507", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91508", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91510", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91521", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91522", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91523", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91526", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91601", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91602", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91603", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91604", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91605", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91606", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91607", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91608", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91609", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91610", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91611", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91612", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91614", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91615", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91616", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91617", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91618", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91702", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91706", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91711", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91714", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91715", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91716", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91722", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91723", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91724", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91731", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91732", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91733", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91734", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91735", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91740", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91741", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91744", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91745", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91746", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91747", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91748", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91749", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91750", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91754", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91755", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91756", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91759", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91765", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91766", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91767", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91768", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91769", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91770", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91771", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91772", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91773", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91775", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91776", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91778", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91780", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91788", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91789", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91790", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91791", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91792", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91793", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91795", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91797", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91799", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91801", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91802", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91803", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91804", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91841", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91896", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91899", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93510", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93532", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93534", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93535", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93536", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93539", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93543", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93544", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93550", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93551", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93552", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93553", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93563", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93584", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93586", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93590", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93591", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93599", "FIPS": "06037", "County": "Los Angeles", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93601", "FIPS": "06039", "County": "Madera", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93604", "FIPS": "06039", "County": "Madera", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93610", "FIPS": "06039", "County": "Madera", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93614", "FIPS": "06039", "County": "Madera", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93636", "FIPS": "06039", "County": "Madera", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93637", "FIPS": "06039", "County": "Madera", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93638", "FIPS": "06039", "County": "Madera", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93639", "FIPS": "06039", "County": "Madera", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93643", "FIPS": "06039", "County": "Madera", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93644", "FIPS": "06039", "County": "Madera", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93645", "FIPS": "06039", "County": "Madera", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93653", "FIPS": "06039", "County": "Madera", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93669", "FIPS": "06039", "County": "Madera", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "94901", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94903", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94904", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94912", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94913", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94914", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94915", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94920", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94924", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94925", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94929", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94930", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94933", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94937", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94938", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94939", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94940", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94941", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94942", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94945", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94946", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94947", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94948", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94949", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94950", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94956", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94957", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94960", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94963", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94964", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94965", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94966", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94970", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94971", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94973", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94974", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94976", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94977", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94978", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94979", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94998", "FIPS": "06041", "County": "Marin", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "93623", "FIPS": "06043", "County": "Mariposa", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95306", "FIPS": "06043", "County": "Mariposa", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95311", "FIPS": "06043", "County": "Mariposa", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95318", "FIPS": "06043", "County": "Mariposa", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95325", "FIPS": "06043", "County": "Mariposa", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95338", "FIPS": "06043", "County": "Mariposa", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95345", "FIPS": "06043", "County": "Mariposa", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95389", "FIPS": "06043", "County": "Mariposa", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95410", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95415", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95417", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95418", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95420", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95427", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95428", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95429", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95432", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95437", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95445", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95449", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95454", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95456", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95459", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95460", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95463", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95466", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95468", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95469", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95470", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95481", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95482", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95488", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95490", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95494", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95585", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95587", "FIPS": "06045", "County": "Mendocino", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "93620", "FIPS": "06047", "County": "Merced", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93635", "FIPS": "06047", "County": "Merced", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93661", "FIPS": "06047", "County": "Merced", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93665", "FIPS": "06047", "County": "Merced", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95301", "FIPS": "06047", "County": "Merced", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95303", "FIPS": "06047", "County": "Merced", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95312", "FIPS": "06047", "County": "Merced", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95315", "FIPS": "06047", "County": "Merced", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95317", "FIPS": "06047", "County": "Merced", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95322", "FIPS": "06047", "County": "Merced", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95324", "FIPS": "06047", "County": "Merced", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95333", "FIPS": "06047", "County": "Merced", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95334", "FIPS": "06047", "County": "Merced", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95340", "FIPS": "06047", "County": "Merced", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95341", "FIPS": "06047", "County": "Merced", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95343", "FIPS": "06047", "County": "Merced", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95344", "FIPS": "06047", "County": "Merced", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95348", "FIPS": "06047", "County": "Merced", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95365", "FIPS": "06047", "County": "Merced", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95369", "FIPS": "06047", "County": "Merced", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95374", "FIPS": "06047", "County": "Merced", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95388", "FIPS": "06047", "County": "Merced", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "96006", "FIPS": "06049", "County": "Modoc", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96015", "FIPS": "06049", "County": "Modoc", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96054", "FIPS": "06049", "County": "Modoc", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96101", "FIPS": "06049", "County": "Modoc", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96104", "FIPS": "06049", "County": "Modoc", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96108", "FIPS": "06049", "County": "Modoc", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96110", "FIPS": "06049", "County": "Modoc", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96112", "FIPS": "06049", "County": "Modoc", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96115", "FIPS": "06049", "County": "Modoc", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96116", "FIPS": "06049", "County": "Modoc", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "93512", "FIPS": "06051", "County": "Mono", "State": "CA", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "93517", "FIPS": "06051", "County": "Mono", "State": "CA", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "93529", "FIPS": "06051", "County": "Mono", "State": "CA", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "93541", "FIPS": "06051", "County": "Mono", "State": "CA", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "93546", "FIPS": "06051", "County": "Mono", "State": "CA", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "96107", "FIPS": "06051", "County": "Mono", "State": "CA", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "96133", "FIPS": "06051", "County": "Mono", "State": "CA", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "93426", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93450", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93901", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93902", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93905", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93906", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93907", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93908", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93912", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93915", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93920", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93921", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93922", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93923", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93924", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93925", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93926", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93927", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93928", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93930", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93932", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93933", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93940", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93942", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93943", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93944", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93950", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93953", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93954", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93955", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93960", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "93962", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95004", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95012", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95039", "FIPS": "06053", "County": "Monterey", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "94503", "FIPS": "06055", "County": "Napa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94508", "FIPS": "06055", "County": "Napa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94515", "FIPS": "06055", "County": "Napa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94558", "FIPS": "06055", "County": "Napa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94559", "FIPS": "06055", "County": "Napa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94562", "FIPS": "06055", "County": "Napa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94567", "FIPS": "06055", "County": "Napa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94573", "FIPS": "06055", "County": "Napa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94574", "FIPS": "06055", "County": "Napa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94576", "FIPS": "06055", "County": "Napa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94581", "FIPS": "06055", "County": "Napa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94599", "FIPS": "06055", "County": "Napa", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95712", "FIPS": "06057", "County": "Nevada", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95724", "FIPS": "06057", "County": "Nevada", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95728", "FIPS": "06057", "County": "Nevada", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95924", "FIPS": "06057", "County": "Nevada", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95945", "FIPS": "06057", "County": "Nevada", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95946", "FIPS": "06057", "County": "Nevada", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95949", "FIPS": "06057", "County": "Nevada", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95959", "FIPS": "06057", "County": "Nevada", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95960", "FIPS": "06057", "County": "Nevada", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95975", "FIPS": "06057", "County": "Nevada", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95977", "FIPS": "06057", "County": "Nevada", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95986", "FIPS": "06057", "County": "Nevada", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96111", "FIPS": "06057", "County": "Nevada", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96160", "FIPS": "06057", "County": "Nevada", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96161", "FIPS": "06057", "County": "Nevada", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96162", "FIPS": "06057", "County": "Nevada", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "90620", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90621", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90622", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90623", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90624", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90630", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90631", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90632", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90633", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90680", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90720", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90721", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90740", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90742", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "90743", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92602", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92603", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92604", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92605", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92606", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92607", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92609", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92610", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92612", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92614", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92615", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92616", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92617", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92618", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92619", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92620", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92623", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92624", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92625", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92626", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92627", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92628", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92629", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92630", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92637", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92646", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92647", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92648", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92649", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92650", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92651", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92652", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92653", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92654", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92655", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92656", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92657", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92658", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92659", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92660", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92661", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92662", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92663", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92672", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92673", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92674", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92675", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92676", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92677", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92678", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92679", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92683", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92684", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92685", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92688", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92690", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92691", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92692", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92693", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92694", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92697", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92698", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92701", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92702", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92703", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92704", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92705", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92706", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92707", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92708", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92709", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92710", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92711", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92712", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92725", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92728", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92735", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92780", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92781", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92782", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92799", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92801", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92802", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92803", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92804", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92805", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92806", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92807", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92808", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92809", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92811", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92812", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92814", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92815", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92816", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92817", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92821", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92822", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92823", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92825", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92831", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92832", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92833", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92834", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92835", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92836", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92837", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92838", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92840", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92841", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92842", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92843", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92844", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92845", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92846", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92850", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92856", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92857", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92859", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92861", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92862", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92863", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92864", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92865", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92866", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92867", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92868", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92869", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92870", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92871", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92885", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92886", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92887", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92899", "FIPS": "06059", "County": "Orange", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "95602", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95603", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95604", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95631", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95648", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95650", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95658", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95661", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95663", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95677", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95678", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95681", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95701", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95703", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95713", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95714", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95715", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95717", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95722", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95736", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95746", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95747", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95765", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96140", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96141", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96143", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96145", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96146", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96148", "FIPS": "06061", "County": "Placer", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95915", "FIPS": "06063", "County": "Plumas", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95923", "FIPS": "06063", "County": "Plumas", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95934", "FIPS": "06063", "County": "Plumas", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95947", "FIPS": "06063", "County": "Plumas", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95956", "FIPS": "06063", "County": "Plumas", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95971", "FIPS": "06063", "County": "Plumas", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95980", "FIPS": "06063", "County": "Plumas", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95983", "FIPS": "06063", "County": "Plumas", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95984", "FIPS": "06063", "County": "Plumas", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96020", "FIPS": "06063", "County": "Plumas", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96103", "FIPS": "06063", "County": "Plumas", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96105", "FIPS": "06063", "County": "Plumas", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96106", "FIPS": "06063", "County": "Plumas", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96122", "FIPS": "06063", "County": "Plumas", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96129", "FIPS": "06063", "County": "Plumas", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96135", "FIPS": "06063", "County": "Plumas", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96137", "FIPS": "06063", "County": "Plumas", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "91752", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92201", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92202", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92203", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92210", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92211", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92220", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92223", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92225", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92226", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92230", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92234", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92235", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92236", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92239", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92240", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92241", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92247", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92248", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92253", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92254", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92255", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92258", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92260", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92261", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92262", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92263", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92264", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92270", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92274", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92276", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92282", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92292", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92320", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92501", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92502", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92503", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92504", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92505", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92506", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92507", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92508", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92509", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92513", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92514", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92515", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92516", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92517", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92518", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92519", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92521", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92522", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92530", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92531", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92532", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92536", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92539", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92543", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92544", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92545", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92546", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92548", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92549", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92551", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92552", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92553", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92554", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92555", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92556", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92557", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92561", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92562", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92563", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92564", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92567", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92570", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92571", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92572", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92581", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92582", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92583", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92584", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92585", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92586", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92587", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92589", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92590", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92591", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92592", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92593", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92595", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92596", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92599", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92860", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92877", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92878", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92879", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92880", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92881", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92882", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92883", "FIPS": "06065", "County": "Riverside", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "94203", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94204", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94205", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94206", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94207", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94208", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94209", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94211", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94229", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94230", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94232", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94234", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94235", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94236", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94237", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94239", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94240", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94244", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94245", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94246", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94247", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94248", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94249", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94250", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94252", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94254", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94256", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94257", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94258", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94259", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94261", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94262", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94263", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94267", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94268", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94269", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94271", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94273", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94274", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94277", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94278", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94279", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94280", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94282", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94283", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94284", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94285", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94286", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94287", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94288", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94289", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94290", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94291", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94293", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94294", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94295", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94296", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94297", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94298", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94299", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95608", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95609", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95610", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95611", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95615", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95621", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95624", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95626", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95628", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95630", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95632", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95638", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95639", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95641", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95652", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95655", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95660", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95662", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95670", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95671", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95673", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95680", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95683", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95690", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95693", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95741", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95742", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95757", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95758", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95759", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95763", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95812", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95813", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95814", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95815", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95816", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95817", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95818", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95819", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95820", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95821", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95822", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95823", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95824", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95825", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95826", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95827", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95828", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95829", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95830", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95831", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95832", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95833", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95834", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95835", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95836", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95837", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95838", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95840", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95841", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95842", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95843", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95851", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95852", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95853", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95860", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95864", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95865", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95866", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95867", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95887", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95894", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95899", "FIPS": "06067", "County": "Sacramento", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95023", "FIPS": "06069", "County": "San Benito", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95024", "FIPS": "06069", "County": "San Benito", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95043", "FIPS": "06069", "County": "San Benito", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95045", "FIPS": "06069", "County": "San Benito", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95075", "FIPS": "06069", "County": "San Benito", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "91701", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91708", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91709", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91710", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91729", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91730", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91737", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91739", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91743", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91758", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91761", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91762", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91763", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91764", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91784", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91785", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91786", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91798", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92242", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92252", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92256", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92267", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92268", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92277", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92278", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92280", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92284", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92285", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92286", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92301", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92304", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92305", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92307", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92308", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92309", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92310", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92311", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92312", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92313", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92314", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92315", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92316", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92317", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92318", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92321", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92322", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92323", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92324", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92325", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92326", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92327", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92329", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92331", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92332", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92333", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92334", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92335", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92336", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92337", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92338", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92339", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92340", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92341", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92342", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92344", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92345", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92346", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92347", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92350", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92352", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92354", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92356", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92357", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92358", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92359", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92363", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92364", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92365", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92366", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92368", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92369", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92371", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92372", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92373", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92374", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92375", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92376", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92377", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92378", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92382", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92385", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92386", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92391", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92392", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92393", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92394", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92395", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92397", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92398", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92399", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92401", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92402", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92403", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92404", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92405", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92406", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92407", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92408", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92410", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92411", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92412", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92413", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92414", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92415", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92418", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92423", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92424", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "92427", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93558", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93562", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93592", "FIPS": "06071", "County": "San Bernardino", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91901", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91902", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91903", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91905", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91906", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91908", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91909", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91910", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91911", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91912", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91913", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91914", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91915", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91916", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91917", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91921", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91931", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91932", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91933", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91934", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91935", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91941", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91942", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91943", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91944", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91945", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91946", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91947", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91948", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91950", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91951", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91962", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91963", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91976", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91977", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91978", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91979", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91980", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91987", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "91990", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92003", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92004", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92007", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92008", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92009", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92010", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92011", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92013", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92014", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92018", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92019", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92020", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92021", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92022", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92023", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92024", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92025", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92026", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92027", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92028", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92029", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92030", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92033", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92036", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92037", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92038", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92039", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92040", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92046", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92049", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92051", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92052", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92054", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92055", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92056", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92057", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92059", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92060", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92061", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92064", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92065", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92066", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92067", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92068", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92069", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92070", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92071", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92072", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92074", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92075", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92078", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92079", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92081", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92082", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92083", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92084", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92085", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92086", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92088", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92090", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92091", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92092", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92093", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92096", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92101", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92102", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92103", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92104", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92105", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92106", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92107", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92108", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92109", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92110", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92111", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92112", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92113", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92114", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92115", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92116", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92117", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92118", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92119", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92120", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92121", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92122", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92123", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92124", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92126", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92127", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92128", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92129", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92130", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92131", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92132", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92133", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92134", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92135", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92136", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92137", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92138", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92139", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92140", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92142", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92143", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92145", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92147", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92149", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92150", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92152", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92153", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92154", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92155", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92158", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92159", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92160", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92161", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92162", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92163", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92164", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92165", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92166", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92167", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92168", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92169", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92170", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92171", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92172", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92173", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92174", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92175", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92176", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92177", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92178", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92179", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92182", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92184", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92186", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92187", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92190", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92191", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92192", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92193", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92194", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92195", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92196", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92197", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92198", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "92199", "FIPS": "06073", "County": "San Diego", "State": "CA", "DMA": {"Code": 825, "Name": "SAN DIEGO"}},
  {"Postal_Code": "94101", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94102", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94103", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94104", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94105", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94106", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94107", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94108", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94109", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94110", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94111", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94112", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94114", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94115", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94116", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94117", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94118", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94119", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94120", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94121", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94122", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94123", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94124", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94125", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94126", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94127", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94129", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94130", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94131", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94132", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94133", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94134", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94135", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94136", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94137", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94138", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94139", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94140", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94141", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94142", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94143", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94144", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94145", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94146", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94147", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94150", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94151", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94152", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94153", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94154", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94155", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94156", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94157", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94158", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94159", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94160", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94161", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94162", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94163", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94164", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94165", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94166", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94167", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94168", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94169", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94170", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94171", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94172", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94175", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94177", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94188", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94199", "FIPS": "06075", "County": "San Francisco", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95201", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95202", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95203", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95204", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95205", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95206", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95207", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95208", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95209", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95210", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95211", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95212", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95213", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95215", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95219", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95220", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95227", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95230", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95231", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95234", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95236", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95237", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95240", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95241", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95242", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95253", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95258", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95267", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95269", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95296", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95297", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95304", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95320", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95330", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95336", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95337", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95366", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95376", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95377", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95378", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95385", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95391", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95686", "FIPS": "06077", "County": "San Joaquin", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "93401", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93402", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93403", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93405", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93406", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93407", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93408", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93409", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93410", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93412", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93420", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93421", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93422", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93423", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93424", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93428", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93430", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93432", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93433", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93435", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93442", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93443", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93444", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93445", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93446", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93447", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93448", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93449", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93451", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93452", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93453", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93461", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93465", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93475", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93483", "FIPS": "06079", "County": "San Luis Obispo", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "94002", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94005", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94010", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94011", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94013", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94014", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94015", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94016", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94017", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94018", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94019", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94020", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94021", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94025", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94026", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94027", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94028", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94030", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94037", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94038", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94044", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94060", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94061", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94062", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94063", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94064", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94065", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94066", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94070", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94074", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94080", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94083", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94096", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94098", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94099", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94128", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94401", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94402", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94403", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94404", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94497", "FIPS": "06081", "County": "San Mateo", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "93013", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93014", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93067", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93101", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93102", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93103", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93105", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93106", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93107", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93108", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93109", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93110", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93111", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93116", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93117", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93118", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93120", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93121", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93130", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93140", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93150", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93160", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93190", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93199", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93254", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93427", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93429", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93434", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93436", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93437", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93438", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93440", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93441", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93454", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93455", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93456", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93457", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93458", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93460", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93463", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "93464", "FIPS": "06083", "County": "Santa Barbara", "State": "CA", "DMA": {"Code": 855, "Name": "SANTA BARBARA-SAN MAR-SAN LUIS OBISPO"}},
  {"Postal_Code": "94022", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94023", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94024", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94035", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94039", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94040", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94041", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94042", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94043", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94085", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94086", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94087", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94088", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94089", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94301", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94302", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94303", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94304", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94305", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94306", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94309", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95002", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95008", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95009", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95011", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95013", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95014", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95015", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95020", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95021", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95026", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95030", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95031", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95032", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95035", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95036", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95037", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95038", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95042", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95044", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95046", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95050", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95051", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95052", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95053", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95054", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95055", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95056", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95070", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95071", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95101", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95103", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95106", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95108", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95109", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95110", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95111", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95112", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95113", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95115", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95116", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95117", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95118", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95119", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95120", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95121", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95122", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95123", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95124", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95125", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95126", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95127", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95128", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95129", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95130", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95131", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95132", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95133", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95134", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95135", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95136", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95138", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95139", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95140", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95141", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95148", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95150", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95151", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95152", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95153", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95154", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95155", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95156", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95157", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95158", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95159", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95160", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95161", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95164", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95170", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95172", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95173", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95190", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95191", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95192", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95193", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95194", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95196", "FIPS": "06085", "County": "Santa Clara", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95001", "FIPS": "06087", "County": "Santa Cruz", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95003", "FIPS": "06087", "County": "Santa Cruz", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95005", "FIPS": "06087", "County": "Santa Cruz", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95006", "FIPS": "06087", "County": "Santa Cruz", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95007", "FIPS": "06087", "County": "Santa Cruz", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95010", "FIPS": "06087", "County": "Santa Cruz", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95017", "FIPS": "06087", "County": "Santa Cruz", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95018", "FIPS": "06087", "County": "Santa Cruz", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95019", "FIPS": "06087", "County": "Santa Cruz", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95033", "FIPS": "06087", "County": "Santa Cruz", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95041", "FIPS": "06087", "County": "Santa Cruz", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95060", "FIPS": "06087", "County": "Santa Cruz", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95061", "FIPS": "06087", "County": "Santa Cruz", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95062", "FIPS": "06087", "County": "Santa Cruz", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95063", "FIPS": "06087", "County": "Santa Cruz", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95064", "FIPS": "06087", "County": "Santa Cruz", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95065", "FIPS": "06087", "County": "Santa Cruz", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95066", "FIPS": "06087", "County": "Santa Cruz", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95067", "FIPS": "06087", "County": "Santa Cruz", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95073", "FIPS": "06087", "County": "Santa Cruz", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95076", "FIPS": "06087", "County": "Santa Cruz", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "95077", "FIPS": "06087", "County": "Santa Cruz", "State": "CA", "DMA": {"Code": 828, "Name": "MONTEREY-SALINAS"}},
  {"Postal_Code": "96001", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96002", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96003", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96007", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96008", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96011", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96013", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96016", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96017", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96019", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96022", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96028", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96033", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96040", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96047", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96049", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96051", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96062", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96065", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96069", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96070", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96071", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96073", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96076", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96079", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96084", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96087", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96088", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96089", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96095", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96096", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96099", "FIPS": "06089", "County": "Shasta", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95910", "FIPS": "06091", "County": "Sierra", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95936", "FIPS": "06091", "County": "Sierra", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95944", "FIPS": "06091", "County": "Sierra", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96118", "FIPS": "06091", "County": "Sierra", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96124", "FIPS": "06091", "County": "Sierra", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96125", "FIPS": "06091", "County": "Sierra", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96126", "FIPS": "06091", "County": "Sierra", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95568", "FIPS": "06093", "County": "Siskiyou", "State": "CA", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "96014", "FIPS": "06093", "County": "Siskiyou", "State": "CA", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "96023", "FIPS": "06093", "County": "Siskiyou", "State": "CA", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "96025", "FIPS": "06093", "County": "Siskiyou", "State": "CA", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "96027", "FIPS": "06093", "County": "Siskiyou", "State": "CA", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "96031", "FIPS": "06093", "County": "Siskiyou", "State": "CA", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "96032", "FIPS": "06093", "County": "Siskiyou", "State": "CA", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "96034", "FIPS": "06093", "County": "Siskiyou", "State": "CA", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "96037", "FIPS": "06093", "County": "Siskiyou", "State": "CA", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "96038", "FIPS": "06093", "County": "Siskiyou", "State": "CA", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "96039", "FIPS": "06093", "County": "Siskiyou", "State": "CA", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "96044", "FIPS": "06093", "County": "Siskiyou", "State": "CA", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "96050", "FIPS": "06093", "County": "Siskiyou", "State": "CA", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "96057", "FIPS": "06093", "County": "Siskiyou", "State": "CA", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "96058", "FIPS": "06093", "County": "Siskiyou", "State": "CA", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "96064", "FIPS": "06093", "County": "Siskiyou", "State": "CA", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "96067", "FIPS": "06093", "County": "Siskiyou", "State": "CA", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "96085", "FIPS": "06093", "County": "Siskiyou", "State": "CA", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "96086", "FIPS": "06093", "County": "Siskiyou", "State": "CA", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "96094", "FIPS": "06093", "County": "Siskiyou", "State": "CA", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "96097", "FIPS": "06093", "County": "Siskiyou", "State": "CA", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "96134", "FIPS": "06093", "County": "Siskiyou", "State": "CA", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "94510", "FIPS": "06095", "County": "Solano", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94512", "FIPS": "06095", "County": "Solano", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94533", "FIPS": "06095", "County": "Solano", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94534", "FIPS": "06095", "County": "Solano", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94535", "FIPS": "06095", "County": "Solano", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94571", "FIPS": "06095", "County": "Solano", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94585", "FIPS": "06095", "County": "Solano", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94589", "FIPS": "06095", "County": "Solano", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94590", "FIPS": "06095", "County": "Solano", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94591", "FIPS": "06095", "County": "Solano", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94592", "FIPS": "06095", "County": "Solano", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95620", "FIPS": "06095", "County": "Solano", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95625", "FIPS": "06095", "County": "Solano", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95687", "FIPS": "06095", "County": "Solano", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95688", "FIPS": "06095", "County": "Solano", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95696", "FIPS": "06095", "County": "Solano", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "94922", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94923", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94926", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94927", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94928", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94931", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94951", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94952", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94953", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94954", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94955", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94972", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94975", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "94999", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95401", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95402", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95403", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95404", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95405", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95406", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95407", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95408", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95409", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95412", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95416", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95419", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95421", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95425", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95430", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95431", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95433", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95436", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95439", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95441", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95442", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95444", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95446", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95448", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95450", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95452", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95462", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95465", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95471", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95472", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95473", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95476", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95480", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95486", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95487", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95492", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95497", "FIPS": "06097", "County": "Sonoma", "State": "CA", "DMA": {"Code": 807, "Name": "SAN FRANCISCO-OAKLAND-SAN JOSE"}},
  {"Postal_Code": "95307", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95313", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95316", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95319", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95323", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95326", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95328", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95329", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95350", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95351", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95352", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95353", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95354", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95355", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95356", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95357", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95358", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95360", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95361", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95363", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95367", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95368", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95380", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95381", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95382", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95386", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95387", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95397", "FIPS": "06099", "County": "Stanislaus", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95659", "FIPS": "06101", "County": "Sutter", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95668", "FIPS": "06101", "County": "Sutter", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95674", "FIPS": "06101", "County": "Sutter", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95676", "FIPS": "06101", "County": "Sutter", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95953", "FIPS": "06101", "County": "Sutter", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95957", "FIPS": "06101", "County": "Sutter", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95982", "FIPS": "06101", "County": "Sutter", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95991", "FIPS": "06101", "County": "Sutter", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95992", "FIPS": "06101", "County": "Sutter", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95993", "FIPS": "06101", "County": "Sutter", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "96021", "FIPS": "06103", "County": "Tehama", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96029", "FIPS": "06103", "County": "Tehama", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96035", "FIPS": "06103", "County": "Tehama", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96055", "FIPS": "06103", "County": "Tehama", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96059", "FIPS": "06103", "County": "Tehama", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96061", "FIPS": "06103", "County": "Tehama", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96063", "FIPS": "06103", "County": "Tehama", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96074", "FIPS": "06103", "County": "Tehama", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96075", "FIPS": "06103", "County": "Tehama", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96078", "FIPS": "06103", "County": "Tehama", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96080", "FIPS": "06103", "County": "Tehama", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96090", "FIPS": "06103", "County": "Tehama", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96092", "FIPS": "06103", "County": "Tehama", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95527", "FIPS": "06105", "County": "Trinity", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95552", "FIPS": "06105", "County": "Trinity", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95563", "FIPS": "06105", "County": "Trinity", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "95595", "FIPS": "06105", "County": "Trinity", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96010", "FIPS": "06105", "County": "Trinity", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96024", "FIPS": "06105", "County": "Trinity", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96041", "FIPS": "06105", "County": "Trinity", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96046", "FIPS": "06105", "County": "Trinity", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96048", "FIPS": "06105", "County": "Trinity", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96052", "FIPS": "06105", "County": "Trinity", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96091", "FIPS": "06105", "County": "Trinity", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "96093", "FIPS": "06105", "County": "Trinity", "State": "CA", "DMA": {"Code": 868, "Name": "CHICO-REDDING"}},
  {"Postal_Code": "93201", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93207", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93208", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93218", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93219", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93221", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93223", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93227", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93235", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93237", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93244", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93247", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93256", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93257", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93258", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93260", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93261", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93262", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93265", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93267", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93270", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93271", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93272", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93274", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93275", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93277", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93278", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93279", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93282", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93286", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93290", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93291", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93292", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93603", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93615", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93618", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93633", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93647", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93666", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93670", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "93673", "FIPS": "06107", "County": "Tulare", "State": "CA", "DMA": {"Code": 866, "Name": "FRESNO-VISALIA"}},
  {"Postal_Code": "95305", "FIPS": "06109", "County": "Tuolumne", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95309", "FIPS": "06109", "County": "Tuolumne", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95310", "FIPS": "06109", "County": "Tuolumne", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95314", "FIPS": "06109", "County": "Tuolumne", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95321", "FIPS": "06109", "County": "Tuolumne", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95327", "FIPS": "06109", "County": "Tuolumne", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95335", "FIPS": "06109", "County": "Tuolumne", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95346", "FIPS": "06109", "County": "Tuolumne", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95347", "FIPS": "06109", "County": "Tuolumne", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95364", "FIPS": "06109", "County": "Tuolumne", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95370", "FIPS": "06109", "County": "Tuolumne", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95372", "FIPS": "06109", "County": "Tuolumne", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95373", "FIPS": "06109", "County": "Tuolumne", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95375", "FIPS": "06109", "County": "Tuolumne", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95379", "FIPS": "06109", "County": "Tuolumne", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95383", "FIPS": "06109", "County": "Tuolumne", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "91319", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91320", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91358", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91359", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91360", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91361", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91362", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "91377", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93001", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93002", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93003", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93004", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93005", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93006", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93007", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93009", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93010", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93011", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93012", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93015", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93016", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93020", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93021", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93022", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93023", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93024", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93030", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93031", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93032", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93033", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93034", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93035", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93036", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93040", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93041", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93042", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93043", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93044", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93060", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93061", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93062", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93063", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93064", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93065", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93066", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93093", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93094", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "93099", "FIPS": "06111", "County": "Ventura", "State": "CA", "DMA": {"Code": 803, "Name": "LOS ANGELES"}},
  {"Postal_Code": "95605", "FIPS": "06113", "County": "Yolo", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95606", "FIPS": "06113", "County": "Yolo", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95607", "FIPS": "06113", "County": "Yolo", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95612", "FIPS": "06113", "County": "Yolo", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95616", "FIPS": "06113", "County": "Yolo", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95617", "FIPS": "06113", "County": "Yolo", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95618", "FIPS": "06113", "County": "Yolo", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95627", "FIPS": "06113", "County": "Yolo", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95637", "FIPS": "06113", "County": "Yolo", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95645", "FIPS": "06113", "County": "Yolo", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95653", "FIPS": "06113", "County": "Yolo", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95679", "FIPS": "06113", "County": "Yolo", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95691", "FIPS": "06113", "County": "Yolo", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95694", "FIPS": "06113", "County": "Yolo", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95695", "FIPS": "06113", "County": "Yolo", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95697", "FIPS": "06113", "County": "Yolo", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95698", "FIPS": "06113", "County": "Yolo", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95776", "FIPS": "06113", "County": "Yolo", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95798", "FIPS": "06113", "County": "Yolo", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95799", "FIPS": "06113", "County": "Yolo", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95937", "FIPS": "06113", "County": "Yolo", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95692", "FIPS": "06115", "County": "Yuba", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95901", "FIPS": "06115", "County": "Yuba", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95903", "FIPS": "06115", "County": "Yuba", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95918", "FIPS": "06115", "County": "Yuba", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95919", "FIPS": "06115", "County": "Yuba", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95922", "FIPS": "06115", "County": "Yuba", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95925", "FIPS": "06115", "County": "Yuba", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95935", "FIPS": "06115", "County": "Yuba", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95961", "FIPS": "06115", "County": "Yuba", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95962", "FIPS": "06115", "County": "Yuba", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95972", "FIPS": "06115", "County": "Yuba", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "95981", "FIPS": "06115", "County": "Yuba", "State": "CA", "DMA": {"Code": 862, "Name": "SACRAMENTO-STOCKTON-MODESTO"}},
  {"Postal_Code": "80011", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80019", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80022", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80024", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80030", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80031", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80035", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80036", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80037", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80040", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80042", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80045", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80102", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80136", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80137", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80221", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80229", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80233", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80234", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80241", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80260", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80601", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80602", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80614", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80640", "FIPS": "08001", "County": "Adams", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81101", "FIPS": "08003", "County": "Alamosa", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81102", "FIPS": "08003", "County": "Alamosa", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81136", "FIPS": "08003", "County": "Alamosa", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81146", "FIPS": "08003", "County": "Alamosa", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80010", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80012", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80013", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80014", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80015", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80016", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80017", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80018", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80041", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80044", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80046", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80047", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80103", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80105", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80110", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80111", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80112", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80113", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80120", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80121", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80122", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80150", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80151", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80155", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80160", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80161", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80165", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80166", "FIPS": "08005", "County": "Arapahoe", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81121", "FIPS": "08007", "County": "Archuleta", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81127", "FIPS": "08007", "County": "Archuleta", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81128", "FIPS": "08007", "County": "Archuleta", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81147", "FIPS": "08007", "County": "Archuleta", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81157", "FIPS": "08007", "County": "Archuleta", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81029", "FIPS": "08009", "County": "Baca", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81064", "FIPS": "08009", "County": "Baca", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81073", "FIPS": "08009", "County": "Baca", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81084", "FIPS": "08009", "County": "Baca", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81087", "FIPS": "08009", "County": "Baca", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81090", "FIPS": "08009", "County": "Baca", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81038", "FIPS": "08011", "County": "Bent", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81044", "FIPS": "08011", "County": "Bent", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81054", "FIPS": "08011", "County": "Bent", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81057", "FIPS": "08011", "County": "Bent", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80025", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80026", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80027", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80028", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80301", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80302", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80303", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80304", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80305", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80306", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80307", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80308", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80309", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80310", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80314", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80321", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80322", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80323", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80328", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80329", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80455", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80466", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80471", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80481", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80501", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80502", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80503", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80510", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80533", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80540", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80544", "FIPS": "08013", "County": "Boulder", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80020", "FIPS": "08014", "County": "Broomfield", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80021", "FIPS": "08014", "County": "Broomfield", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80038", "FIPS": "08014", "County": "Broomfield", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81201", "FIPS": "08015", "County": "Chaffee", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81211", "FIPS": "08015", "County": "Chaffee", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81227", "FIPS": "08015", "County": "Chaffee", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81228", "FIPS": "08015", "County": "Chaffee", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81236", "FIPS": "08015", "County": "Chaffee", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81242", "FIPS": "08015", "County": "Chaffee", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80802", "FIPS": "08017", "County": "Cheyenne", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80810", "FIPS": "08017", "County": "Cheyenne", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80825", "FIPS": "08017", "County": "Cheyenne", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80862", "FIPS": "08017", "County": "Cheyenne", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80436", "FIPS": "08019", "County": "Clear Creek", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80438", "FIPS": "08019", "County": "Clear Creek", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80444", "FIPS": "08019", "County": "Clear Creek", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80452", "FIPS": "08019", "County": "Clear Creek", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80476", "FIPS": "08019", "County": "Clear Creek", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81120", "FIPS": "08021", "County": "Conejos", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81124", "FIPS": "08021", "County": "Conejos", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81129", "FIPS": "08021", "County": "Conejos", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81140", "FIPS": "08021", "County": "Conejos", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81141", "FIPS": "08021", "County": "Conejos", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81148", "FIPS": "08021", "County": "Conejos", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81151", "FIPS": "08021", "County": "Conejos", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81123", "FIPS": "08023", "County": "Costilla", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81126", "FIPS": "08023", "County": "Costilla", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81133", "FIPS": "08023", "County": "Costilla", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81134", "FIPS": "08023", "County": "Costilla", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81138", "FIPS": "08023", "County": "Costilla", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81152", "FIPS": "08023", "County": "Costilla", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81153", "FIPS": "08023", "County": "Costilla", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81033", "FIPS": "08025", "County": "Crowley", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81034", "FIPS": "08025", "County": "Crowley", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81062", "FIPS": "08025", "County": "Crowley", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81063", "FIPS": "08025", "County": "Crowley", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81076", "FIPS": "08025", "County": "Crowley", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81252", "FIPS": "08027", "County": "Custer", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81253", "FIPS": "08027", "County": "Custer", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81410", "FIPS": "08029", "County": "Delta", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81413", "FIPS": "08029", "County": "Delta", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81414", "FIPS": "08029", "County": "Delta", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81415", "FIPS": "08029", "County": "Delta", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81416", "FIPS": "08029", "County": "Delta", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81418", "FIPS": "08029", "County": "Delta", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81419", "FIPS": "08029", "County": "Delta", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81420", "FIPS": "08029", "County": "Delta", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81428", "FIPS": "08029", "County": "Delta", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80201", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80202", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80203", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80204", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80205", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80206", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80207", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80208", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80209", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80210", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80211", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80212", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80216", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80217", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80218", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80219", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80220", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80222", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80223", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80224", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80230", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80231", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80236", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80237", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80238", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80239", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80243", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80244", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80246", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80247", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80248", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80249", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80250", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80251", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80252", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80256", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80257", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80259", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80261", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80262", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80263", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80264", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80265", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80266", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80271", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80273", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80274", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80279", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80280", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80281", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80290", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80291", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80293", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80294", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80295", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80299", "FIPS": "08031", "County": "Denver", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81320", "FIPS": "08033", "County": "Dolores", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81324", "FIPS": "08033", "County": "Dolores", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81332", "FIPS": "08033", "County": "Dolores", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80104", "FIPS": "08035", "County": "Douglas", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80108", "FIPS": "08035", "County": "Douglas", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80109", "FIPS": "08035", "County": "Douglas", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80116", "FIPS": "08035", "County": "Douglas", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80118", "FIPS": "08035", "County": "Douglas", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80124", "FIPS": "08035", "County": "Douglas", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80125", "FIPS": "08035", "County": "Douglas", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80126", "FIPS": "08035", "County": "Douglas", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80129", "FIPS": "08035", "County": "Douglas", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80130", "FIPS": "08035", "County": "Douglas", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80131", "FIPS": "08035", "County": "Douglas", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80134", "FIPS": "08035", "County": "Douglas", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80135", "FIPS": "08035", "County": "Douglas", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80138", "FIPS": "08035", "County": "Douglas", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80163", "FIPS": "08035", "County": "Douglas", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80423", "FIPS": "08037", "County": "Eagle", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80426", "FIPS": "08037", "County": "Eagle", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80463", "FIPS": "08037", "County": "Eagle", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81620", "FIPS": "08037", "County": "Eagle", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81621", "FIPS": "08037", "County": "Eagle", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81631", "FIPS": "08037", "County": "Eagle", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81632", "FIPS": "08037", "County": "Eagle", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81637", "FIPS": "08037", "County": "Eagle", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81645", "FIPS": "08037", "County": "Eagle", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81649", "FIPS": "08037", "County": "Eagle", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81655", "FIPS": "08037", "County": "Eagle", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81657", "FIPS": "08037", "County": "Eagle", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81658", "FIPS": "08037", "County": "Eagle", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80101", "FIPS": "08039", "County": "Elbert", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80107", "FIPS": "08039", "County": "Elbert", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80117", "FIPS": "08039", "County": "Elbert", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80830", "FIPS": "08039", "County": "Elbert", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80835", "FIPS": "08039", "County": "Elbert", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80106", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80132", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80133", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80808", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80809", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80817", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80819", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80829", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80831", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80832", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80833", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80840", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80841", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80864", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80901", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80903", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80904", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80905", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80906", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80907", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80908", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80909", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80910", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80911", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80912", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80913", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80914", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80915", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80916", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80917", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80918", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80919", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80920", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80921", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80922", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80923", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80924", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80925", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80926", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80927", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80928", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80929", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80930", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80931", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80932", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80933", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80934", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80935", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80936", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80937", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80938", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80939", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80940", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80941", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80942", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80943", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80944", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80945", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80946", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80947", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80949", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80950", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80951", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80960", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80962", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80970", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80977", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80995", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80997", "FIPS": "08041", "County": "El Paso", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81212", "FIPS": "08043", "County": "Fremont", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81215", "FIPS": "08043", "County": "Fremont", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81221", "FIPS": "08043", "County": "Fremont", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81222", "FIPS": "08043", "County": "Fremont", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81223", "FIPS": "08043", "County": "Fremont", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81226", "FIPS": "08043", "County": "Fremont", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81232", "FIPS": "08043", "County": "Fremont", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81233", "FIPS": "08043", "County": "Fremont", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81240", "FIPS": "08043", "County": "Fremont", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81244", "FIPS": "08043", "County": "Fremont", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81290", "FIPS": "08043", "County": "Fremont", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81601", "FIPS": "08045", "County": "Garfield", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81602", "FIPS": "08045", "County": "Garfield", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81623", "FIPS": "08045", "County": "Garfield", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81635", "FIPS": "08045", "County": "Garfield", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81636", "FIPS": "08045", "County": "Garfield", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81647", "FIPS": "08045", "County": "Garfield", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81650", "FIPS": "08045", "County": "Garfield", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81652", "FIPS": "08045", "County": "Garfield", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80422", "FIPS": "08047", "County": "Gilpin", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80427", "FIPS": "08047", "County": "Gilpin", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80474", "FIPS": "08047", "County": "Gilpin", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80442", "FIPS": "08049", "County": "Grand", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80446", "FIPS": "08049", "County": "Grand", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80447", "FIPS": "08049", "County": "Grand", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80451", "FIPS": "08049", "County": "Grand", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80459", "FIPS": "08049", "County": "Grand", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80468", "FIPS": "08049", "County": "Grand", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80478", "FIPS": "08049", "County": "Grand", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80482", "FIPS": "08049", "County": "Grand", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81210", "FIPS": "08051", "County": "Gunnison", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81224", "FIPS": "08051", "County": "Gunnison", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81225", "FIPS": "08051", "County": "Gunnison", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81230", "FIPS": "08051", "County": "Gunnison", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81231", "FIPS": "08051", "County": "Gunnison", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81237", "FIPS": "08051", "County": "Gunnison", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81239", "FIPS": "08051", "County": "Gunnison", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81241", "FIPS": "08051", "County": "Gunnison", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81243", "FIPS": "08051", "County": "Gunnison", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81247", "FIPS": "08051", "County": "Gunnison", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81434", "FIPS": "08051", "County": "Gunnison", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81235", "FIPS": "08053", "County": "Hinsdale", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81040", "FIPS": "08055", "County": "Huerfano", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81055", "FIPS": "08055", "County": "Huerfano", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81089", "FIPS": "08055", "County": "Huerfano", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80430", "FIPS": "08057", "County": "Jackson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80434", "FIPS": "08057", "County": "Jackson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80473", "FIPS": "08057", "County": "Jackson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80480", "FIPS": "08057", "County": "Jackson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80001", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80002", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80003", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80004", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80005", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80006", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80007", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80033", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80034", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80123", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80127", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80128", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80162", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80214", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80215", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80225", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80226", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80227", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80228", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80232", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80235", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80401", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80402", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80403", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80419", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80425", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80433", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80437", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80439", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80453", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80454", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80457", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80465", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80470", "FIPS": "08059", "County": "Jefferson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81021", "FIPS": "08061", "County": "Kiowa", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81036", "FIPS": "08061", "County": "Kiowa", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81045", "FIPS": "08061", "County": "Kiowa", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81071", "FIPS": "08061", "County": "Kiowa", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80805", "FIPS": "08063", "County": "Kit Carson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80807", "FIPS": "08063", "County": "Kit Carson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80815", "FIPS": "08063", "County": "Kit Carson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80834", "FIPS": "08063", "County": "Kit Carson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80836", "FIPS": "08063", "County": "Kit Carson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80861", "FIPS": "08063", "County": "Kit Carson", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80429", "FIPS": "08065", "County": "Lake", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80461", "FIPS": "08065", "County": "Lake", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81251", "FIPS": "08065", "County": "Lake", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81122", "FIPS": "08067", "County": "La Plata", "State": "CO", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "81137", "FIPS": "08067", "County": "La Plata", "State": "CO", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "81301", "FIPS": "08067", "County": "La Plata", "State": "CO", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "81302", "FIPS": "08067", "County": "La Plata", "State": "CO", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "81303", "FIPS": "08067", "County": "La Plata", "State": "CO", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "81326", "FIPS": "08067", "County": "La Plata", "State": "CO", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "81329", "FIPS": "08067", "County": "La Plata", "State": "CO", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "80511", "FIPS": "08069", "County": "Larimer", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80512", "FIPS": "08069", "County": "Larimer", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80513", "FIPS": "08069", "County": "Larimer", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80515", "FIPS": "08069", "County": "Larimer", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80517", "FIPS": "08069", "County": "Larimer", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80521", "FIPS": "08069", "County": "Larimer", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80522", "FIPS": "08069", "County": "Larimer", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80523", "FIPS": "08069", "County": "Larimer", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80524", "FIPS": "08069", "County": "Larimer", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80525", "FIPS": "08069", "County": "Larimer", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80526", "FIPS": "08069", "County": "Larimer", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80527", "FIPS": "08069", "County": "Larimer", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80528", "FIPS": "08069", "County": "Larimer", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80532", "FIPS": "08069", "County": "Larimer", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80535", "FIPS": "08069", "County": "Larimer", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80536", "FIPS": "08069", "County": "Larimer", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80537", "FIPS": "08069", "County": "Larimer", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80538", "FIPS": "08069", "County": "Larimer", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80539", "FIPS": "08069", "County": "Larimer", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80541", "FIPS": "08069", "County": "Larimer", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80545", "FIPS": "08069", "County": "Larimer", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80547", "FIPS": "08069", "County": "Larimer", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80549", "FIPS": "08069", "County": "Larimer", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80553", "FIPS": "08069", "County": "Larimer", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81020", "FIPS": "08071", "County": "Las Animas", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81024", "FIPS": "08071", "County": "Las Animas", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81027", "FIPS": "08071", "County": "Las Animas", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81046", "FIPS": "08071", "County": "Las Animas", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81049", "FIPS": "08071", "County": "Las Animas", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81059", "FIPS": "08071", "County": "Las Animas", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81081", "FIPS": "08071", "County": "Las Animas", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81082", "FIPS": "08071", "County": "Las Animas", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81091", "FIPS": "08071", "County": "Las Animas", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80804", "FIPS": "08073", "County": "Lincoln", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80818", "FIPS": "08073", "County": "Lincoln", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80821", "FIPS": "08073", "County": "Lincoln", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80823", "FIPS": "08073", "County": "Lincoln", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80826", "FIPS": "08073", "County": "Lincoln", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80828", "FIPS": "08073", "County": "Lincoln", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80722", "FIPS": "08075", "County": "Logan", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80726", "FIPS": "08075", "County": "Logan", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80728", "FIPS": "08075", "County": "Logan", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80736", "FIPS": "08075", "County": "Logan", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80741", "FIPS": "08075", "County": "Logan", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80745", "FIPS": "08075", "County": "Logan", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80747", "FIPS": "08075", "County": "Logan", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80751", "FIPS": "08075", "County": "Logan", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81501", "FIPS": "08077", "County": "Mesa", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81502", "FIPS": "08077", "County": "Mesa", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81503", "FIPS": "08077", "County": "Mesa", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81504", "FIPS": "08077", "County": "Mesa", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81505", "FIPS": "08077", "County": "Mesa", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81506", "FIPS": "08077", "County": "Mesa", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81520", "FIPS": "08077", "County": "Mesa", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81521", "FIPS": "08077", "County": "Mesa", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81522", "FIPS": "08077", "County": "Mesa", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81523", "FIPS": "08077", "County": "Mesa", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81524", "FIPS": "08077", "County": "Mesa", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81525", "FIPS": "08077", "County": "Mesa", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81526", "FIPS": "08077", "County": "Mesa", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81527", "FIPS": "08077", "County": "Mesa", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81624", "FIPS": "08077", "County": "Mesa", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81630", "FIPS": "08077", "County": "Mesa", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81643", "FIPS": "08077", "County": "Mesa", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81646", "FIPS": "08077", "County": "Mesa", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81130", "FIPS": "08079", "County": "Mineral", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81610", "FIPS": "08081", "County": "Moffat", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81625", "FIPS": "08081", "County": "Moffat", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81626", "FIPS": "08081", "County": "Moffat", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81633", "FIPS": "08081", "County": "Moffat", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81638", "FIPS": "08081", "County": "Moffat", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81640", "FIPS": "08081", "County": "Moffat", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81653", "FIPS": "08081", "County": "Moffat", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81321", "FIPS": "08083", "County": "Montezuma", "State": "CO", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "81323", "FIPS": "08083", "County": "Montezuma", "State": "CO", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "81327", "FIPS": "08083", "County": "Montezuma", "State": "CO", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "81328", "FIPS": "08083", "County": "Montezuma", "State": "CO", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "81330", "FIPS": "08083", "County": "Montezuma", "State": "CO", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "81331", "FIPS": "08083", "County": "Montezuma", "State": "CO", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "81334", "FIPS": "08083", "County": "Montezuma", "State": "CO", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "81335", "FIPS": "08083", "County": "Montezuma", "State": "CO", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "81220", "FIPS": "08085", "County": "Montrose", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81401", "FIPS": "08085", "County": "Montrose", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81402", "FIPS": "08085", "County": "Montrose", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81411", "FIPS": "08085", "County": "Montrose", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81422", "FIPS": "08085", "County": "Montrose", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81424", "FIPS": "08085", "County": "Montrose", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81425", "FIPS": "08085", "County": "Montrose", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81429", "FIPS": "08085", "County": "Montrose", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "81431", "FIPS": "08085", "County": "Montrose", "State": "CO", "DMA": {"Code": 773, "Name": "GRAND JUNCTION-MONTROSE"}},
  {"Postal_Code": "80649", "FIPS": "08087", "County": "Morgan", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80653", "FIPS": "08087", "County": "Morgan", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80654", "FIPS": "08087", "County": "Morgan", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80701", "FIPS": "08087", "County": "Morgan", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80705", "FIPS": "08087", "County": "Morgan", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80723", "FIPS": "08087", "County": "Morgan", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80733", "FIPS": "08087", "County": "Morgan", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80750", "FIPS": "08087", "County": "Morgan", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81030", "FIPS": "08089", "County": "Otero", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81039", "FIPS": "08089", "County": "Otero", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81050", "FIPS": "08089", "County": "Otero", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81058", "FIPS": "08089", "County": "Otero", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81067", "FIPS": "08089", "County": "Otero", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81077", "FIPS": "08089", "County": "Otero", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81427", "FIPS": "08091", "County": "Ouray", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81432", "FIPS": "08091", "County": "Ouray", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80420", "FIPS": "08093", "County": "Park", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80421", "FIPS": "08093", "County": "Park", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80432", "FIPS": "08093", "County": "Park", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80440", "FIPS": "08093", "County": "Park", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80448", "FIPS": "08093", "County": "Park", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80449", "FIPS": "08093", "County": "Park", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80456", "FIPS": "08093", "County": "Park", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80475", "FIPS": "08093", "County": "Park", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80820", "FIPS": "08093", "County": "Park", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80827", "FIPS": "08093", "County": "Park", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80721", "FIPS": "08095", "County": "Phillips", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80731", "FIPS": "08095", "County": "Phillips", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80734", "FIPS": "08095", "County": "Phillips", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80746", "FIPS": "08095", "County": "Phillips", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81611", "FIPS": "08097", "County": "Pitkin", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81612", "FIPS": "08097", "County": "Pitkin", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81615", "FIPS": "08097", "County": "Pitkin", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81642", "FIPS": "08097", "County": "Pitkin", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81654", "FIPS": "08097", "County": "Pitkin", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81656", "FIPS": "08097", "County": "Pitkin", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81041", "FIPS": "08099", "County": "Prowers", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81043", "FIPS": "08099", "County": "Prowers", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81047", "FIPS": "08099", "County": "Prowers", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81052", "FIPS": "08099", "County": "Prowers", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81092", "FIPS": "08099", "County": "Prowers", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81001", "FIPS": "08101", "County": "Pueblo", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81002", "FIPS": "08101", "County": "Pueblo", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81003", "FIPS": "08101", "County": "Pueblo", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81004", "FIPS": "08101", "County": "Pueblo", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81005", "FIPS": "08101", "County": "Pueblo", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81006", "FIPS": "08101", "County": "Pueblo", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81007", "FIPS": "08101", "County": "Pueblo", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81008", "FIPS": "08101", "County": "Pueblo", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81009", "FIPS": "08101", "County": "Pueblo", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81010", "FIPS": "08101", "County": "Pueblo", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81011", "FIPS": "08101", "County": "Pueblo", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81012", "FIPS": "08101", "County": "Pueblo", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81019", "FIPS": "08101", "County": "Pueblo", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81022", "FIPS": "08101", "County": "Pueblo", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81023", "FIPS": "08101", "County": "Pueblo", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81025", "FIPS": "08101", "County": "Pueblo", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81069", "FIPS": "08101", "County": "Pueblo", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "81641", "FIPS": "08103", "County": "Rio Blanco", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81648", "FIPS": "08103", "County": "Rio Blanco", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81125", "FIPS": "08105", "County": "Rio Grande", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81132", "FIPS": "08105", "County": "Rio Grande", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81135", "FIPS": "08105", "County": "Rio Grande", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81144", "FIPS": "08105", "County": "Rio Grande", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81154", "FIPS": "08105", "County": "Rio Grande", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80428", "FIPS": "08107", "County": "Routt", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80467", "FIPS": "08107", "County": "Routt", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80469", "FIPS": "08107", "County": "Routt", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80477", "FIPS": "08107", "County": "Routt", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80479", "FIPS": "08107", "County": "Routt", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80483", "FIPS": "08107", "County": "Routt", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80487", "FIPS": "08107", "County": "Routt", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80488", "FIPS": "08107", "County": "Routt", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81639", "FIPS": "08107", "County": "Routt", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81131", "FIPS": "08109", "County": "Saguache", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81143", "FIPS": "08109", "County": "Saguache", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81149", "FIPS": "08109", "County": "Saguache", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81155", "FIPS": "08109", "County": "Saguache", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81248", "FIPS": "08109", "County": "Saguache", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81433", "FIPS": "08111", "County": "San Juan", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81325", "FIPS": "08113", "County": "San Miguel", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81423", "FIPS": "08113", "County": "San Miguel", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81426", "FIPS": "08113", "County": "San Miguel", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81430", "FIPS": "08113", "County": "San Miguel", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "81435", "FIPS": "08113", "County": "San Miguel", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80737", "FIPS": "08115", "County": "Sedgwick", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80744", "FIPS": "08115", "County": "Sedgwick", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80749", "FIPS": "08115", "County": "Sedgwick", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80424", "FIPS": "08117", "County": "Summit", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80435", "FIPS": "08117", "County": "Summit", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80443", "FIPS": "08117", "County": "Summit", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80497", "FIPS": "08117", "County": "Summit", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80498", "FIPS": "08117", "County": "Summit", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80813", "FIPS": "08119", "County": "Teller", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80814", "FIPS": "08119", "County": "Teller", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80816", "FIPS": "08119", "County": "Teller", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80860", "FIPS": "08119", "County": "Teller", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80863", "FIPS": "08119", "County": "Teller", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80866", "FIPS": "08119", "County": "Teller", "State": "CO", "DMA": {"Code": 752, "Name": "COLORADO SPRINGS-PUEBLO"}},
  {"Postal_Code": "80720", "FIPS": "08121", "County": "Washington", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80740", "FIPS": "08121", "County": "Washington", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80743", "FIPS": "08121", "County": "Washington", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80757", "FIPS": "08121", "County": "Washington", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80801", "FIPS": "08121", "County": "Washington", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80812", "FIPS": "08121", "County": "Washington", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80504", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80514", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80516", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80520", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80530", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80534", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80542", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80543", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80546", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80550", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80551", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80603", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80610", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80611", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80612", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80615", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80620", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80621", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80622", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80623", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80624", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80631", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80632", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80633", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80634", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80638", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80639", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80642", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80643", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80644", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80645", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80646", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80648", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80650", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80651", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80652", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80729", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80732", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80742", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80754", "FIPS": "08123", "County": "Weld", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80727", "FIPS": "08125", "County": "Yuma", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80735", "FIPS": "08125", "County": "Yuma", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80755", "FIPS": "08125", "County": "Yuma", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80758", "FIPS": "08125", "County": "Yuma", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80759", "FIPS": "08125", "County": "Yuma", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80822", "FIPS": "08125", "County": "Yuma", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "80824", "FIPS": "08125", "County": "Yuma", "State": "CO", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "06404", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06440", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06468", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06470", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06482", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06484", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06491", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06497", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06601", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06602", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06604", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06605", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06606", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06607", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06608", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06610", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06611", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06612", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06614", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06615", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06650", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06673", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06699", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06784", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06801", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06804", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06807", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06810", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06811", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06812", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06813", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06814", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06816", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06817", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06820", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06824", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06825", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06828", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06829", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06830", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06831", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06832", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06836", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06838", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06840", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06842", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06850", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06851", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06852", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06853", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06854", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06855", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06856", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06857", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06858", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06859", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06860", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06870", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06875", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06876", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06877", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06878", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06879", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06880", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06881", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06883", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06888", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06889", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06890", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06896", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06897", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06901", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06902", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06903", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06904", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06905", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06906", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06907", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06910", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06911", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06912", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06913", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06914", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06920", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06921", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06922", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06925", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06926", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06927", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06928", "FIPS": "09001", "County": "Fairfield", "State": "CT", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06001", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06002", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06006", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06010", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06011", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06013", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06016", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06019", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06020", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06022", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06023", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06025", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06026", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06027", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06028", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06030", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06032", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06033", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06034", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06035", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06037", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06040", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06041", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06042", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06045", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06050", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06051", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06052", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06053", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06059", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06060", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06062", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06064", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06067", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06070", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06073", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06074", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06078", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06080", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06081", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06082", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06083", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06085", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06087", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06088", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06089", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06090", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06091", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06092", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06093", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06095", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06096", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06101", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06102", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06103", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06104", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06105", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06106", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06107", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06108", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06109", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06110", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06111", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06112", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06114", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06115", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06117", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06118", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06119", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06120", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06123", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06126", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06127", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06128", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06129", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06131", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06132", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06133", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06134", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06137", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06138", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06140", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06141", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06142", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06143", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06144", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06145", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06146", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06147", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06150", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06151", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06152", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06153", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06154", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06155", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06156", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06160", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06161", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06167", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06176", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06180", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06183", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06199", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06444", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06447", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06467", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06479", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06489", "FIPS": "09003", "County": "Hartford", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06018", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06021", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06024", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06031", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06039", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06057", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06058", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06061", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06063", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06065", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06068", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06069", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06079", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06094", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06098", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06750", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06751", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06752", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06753", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06754", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06755", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06756", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06757", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06758", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06759", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06763", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06776", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06777", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06778", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06779", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06781", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06782", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06783", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06785", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06786", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06787", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06790", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06791", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06792", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06793", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06794", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06795", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06796", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06798", "FIPS": "09005", "County": "Litchfield", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06409", "FIPS": "09007", "County": "Middlesex", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06412", "FIPS": "09007", "County": "Middlesex", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06413", "FIPS": "09007", "County": "Middlesex", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06414", "FIPS": "09007", "County": "Middlesex", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06416", "FIPS": "09007", "County": "Middlesex", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06417", "FIPS": "09007", "County": "Middlesex", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06419", "FIPS": "09007", "County": "Middlesex", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06422", "FIPS": "09007", "County": "Middlesex", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06423", "FIPS": "09007", "County": "Middlesex", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06424", "FIPS": "09007", "County": "Middlesex", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06426", "FIPS": "09007", "County": "Middlesex", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06438", "FIPS": "09007", "County": "Middlesex", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06441", "FIPS": "09007", "County": "Middlesex", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06442", "FIPS": "09007", "County": "Middlesex", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06455", "FIPS": "09007", "County": "Middlesex", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06456", "FIPS": "09007", "County": "Middlesex", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06457", "FIPS": "09007", "County": "Middlesex", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06459", "FIPS": "09007", "County": "Middlesex", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06469", "FIPS": "09007", "County": "Middlesex", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06475", "FIPS": "09007", "County": "Middlesex", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06480", "FIPS": "09007", "County": "Middlesex", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06481", "FIPS": "09007", "County": "Middlesex", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06498", "FIPS": "09007", "County": "Middlesex", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06401", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06403", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06405", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06408", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06410", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06411", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06418", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06437", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06443", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06450", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06451", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06454", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06460", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06461", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06471", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06472", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06473", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06477", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06478", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06483", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06487", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06488", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06492", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06493", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06494", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06495", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06501", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06502", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06503", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06504", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06505", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06506", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06507", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06508", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06509", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06510", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06511", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06512", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06513", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06514", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06515", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06516", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06517", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06518", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06519", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06520", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06521", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06524", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06525", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06530", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06531", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06532", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06533", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06534", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06535", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06536", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06537", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06538", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06540", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06701", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06702", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06703", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06704", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06705", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06706", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06708", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06710", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06712", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06716", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06720", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06721", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06722", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06723", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06724", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06725", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06726", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06749", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06762", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06770", "FIPS": "09009", "County": "New Haven", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06249", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06254", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06320", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06330", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06333", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06334", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06335", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06336", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06338", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06339", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06340", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06349", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06350", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06351", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06353", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06355", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06357", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06359", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06360", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06365", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06370", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06371", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06372", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06375", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06376", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06378", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06379", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06380", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06382", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06383", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06384", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06385", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06386", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06388", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06389", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06415", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06420", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06439", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06474", "FIPS": "09011", "County": "New London", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06029", "FIPS": "09013", "County": "Tolland", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06043", "FIPS": "09013", "County": "Tolland", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06066", "FIPS": "09013", "County": "Tolland", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06071", "FIPS": "09013", "County": "Tolland", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06072", "FIPS": "09013", "County": "Tolland", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06075", "FIPS": "09013", "County": "Tolland", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06076", "FIPS": "09013", "County": "Tolland", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06077", "FIPS": "09013", "County": "Tolland", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06084", "FIPS": "09013", "County": "Tolland", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06231", "FIPS": "09013", "County": "Tolland", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06232", "FIPS": "09013", "County": "Tolland", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06237", "FIPS": "09013", "County": "Tolland", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06238", "FIPS": "09013", "County": "Tolland", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06248", "FIPS": "09013", "County": "Tolland", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06250", "FIPS": "09013", "County": "Tolland", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06251", "FIPS": "09013", "County": "Tolland", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06265", "FIPS": "09013", "County": "Tolland", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06268", "FIPS": "09013", "County": "Tolland", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06269", "FIPS": "09013", "County": "Tolland", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06279", "FIPS": "09013", "County": "Tolland", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06226", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06230", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06233", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06234", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06235", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06239", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06241", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06242", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06243", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06244", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06245", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06246", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06247", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06255", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06256", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06258", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06259", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06260", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06262", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06263", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06264", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06266", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06267", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06277", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06278", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06280", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06281", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06282", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06331", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06332", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06354", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06373", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06374", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06377", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "06387", "FIPS": "09015", "County": "Windham", "State": "CT", "DMA": {"Code": 533, "Name": "HARTFORD-NEW HAVEN"}},
  {"Postal_Code": "19901", "FIPS": "10001", "County": "Kent", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19902", "FIPS": "10001", "County": "Kent", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19903", "FIPS": "10001", "County": "Kent", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19904", "FIPS": "10001", "County": "Kent", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19905", "FIPS": "10001", "County": "Kent", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19906", "FIPS": "10001", "County": "Kent", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19934", "FIPS": "10001", "County": "Kent", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19936", "FIPS": "10001", "County": "Kent", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19938", "FIPS": "10001", "County": "Kent", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19943", "FIPS": "10001", "County": "Kent", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19946", "FIPS": "10001", "County": "Kent", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19952", "FIPS": "10001", "County": "Kent", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19953", "FIPS": "10001", "County": "Kent", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19954", "FIPS": "10001", "County": "Kent", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19955", "FIPS": "10001", "County": "Kent", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19961", "FIPS": "10001", "County": "Kent", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19962", "FIPS": "10001", "County": "Kent", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19964", "FIPS": "10001", "County": "Kent", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19977", "FIPS": "10001", "County": "Kent", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19979", "FIPS": "10001", "County": "Kent", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19980", "FIPS": "10001", "County": "Kent", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19701", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19702", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19703", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19706", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19707", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19708", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19709", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19710", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19711", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19712", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19713", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19714", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19715", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19716", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19717", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19718", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19720", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19721", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19725", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19726", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19730", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19731", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19732", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19733", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19734", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19735", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19736", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19801", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19802", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19803", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19804", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19805", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19806", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19807", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19808", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19809", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19810", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19850", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19880", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19884", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19885", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19886", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19887", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19889", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19890", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19891", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19892", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19893", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19894", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19895", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19896", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19897", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19898", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19899", "FIPS": "10003", "County": "New Castle", "State": "DE", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19930", "FIPS": "10005", "County": "Sussex", "State": "DE", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "19931", "FIPS": "10005", "County": "Sussex", "State": "DE", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "19933", "FIPS": "10005", "County": "Sussex", "State": "DE", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "19939", "FIPS": "10005", "County": "Sussex", "State": "DE", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "19940", "FIPS": "10005", "County": "Sussex", "State": "DE", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "19941", "FIPS": "10005", "County": "Sussex", "State": "DE", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "19944", "FIPS": "10005", "County": "Sussex", "State": "DE", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "19945", "FIPS": "10005", "County": "Sussex", "State": "DE", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "19947", "FIPS": "10005", "County": "Sussex", "State": "DE", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "19950", "FIPS": "10005", "County": "Sussex", "State": "DE", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "19951", "FIPS": "10005", "County": "Sussex", "State": "DE", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "19956", "FIPS": "10005", "County": "Sussex", "State": "DE", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "19958", "FIPS": "10005", "County": "Sussex", "State": "DE", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "19960", "FIPS": "10005", "County": "Sussex", "State": "DE", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "19963", "FIPS": "10005", "County": "Sussex", "State": "DE", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "19966", "FIPS": "10005", "County": "Sussex", "State": "DE", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "19967", "FIPS": "10005", "County": "Sussex", "State": "DE", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "19968", "FIPS": "10005", "County": "Sussex", "State": "DE", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "19969", "FIPS": "10005", "County": "Sussex", "State": "DE", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "19970", "FIPS": "10005", "County": "Sussex", "State": "DE", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "19971", "FIPS": "10005", "County": "Sussex", "State": "DE", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "19973", "FIPS": "10005", "County": "Sussex", "State": "DE", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "19975", "FIPS": "10005", "County": "Sussex", "State": "DE", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "20001", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20002", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20003", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20004", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20005", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20006", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20007", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20008", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20009", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20010", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20011", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20012", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20013", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20015", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20016", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20017", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20018", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20019", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20020", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20022", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20023", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20024", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20026", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20027", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20029", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20030", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20032", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20033", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20035", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20036", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20037", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20038", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20039", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20040", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20041", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20042", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20043", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20044", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20045", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20046", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20047", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20049", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20050", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20051", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20052", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20053", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20055", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20056", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20057", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20058", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20059", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20060", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20061", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20062", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20063", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20064", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20065", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20066", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20067", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20068", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20069", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20070", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20071", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20073", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20074", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20075", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20076", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20077", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20078", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20080", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20081", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20082", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20088", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20090", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20091", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20097", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20098", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20201", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20202", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20203", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20204", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20206", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20207", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20208", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20210", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20211", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20212", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20213", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20214", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20215", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20216", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20217", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20218", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20219", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20220", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20221", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20222", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20223", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20224", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20226", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20227", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20228", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20229", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20230", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20232", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20233", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20235", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20237", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20238", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20239", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20240", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20241", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20242", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20244", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20245", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20250", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20251", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20254", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20260", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20261", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20262", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20265", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20266", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20268", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20270", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20277", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20289", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20299", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20301", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20303", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20306", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20307", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20310", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20314", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20317", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20318", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20319", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20330", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20340", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20350", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20355", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20370", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20372", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20373", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20374", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20375", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20376", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20380", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20388", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20389", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20390", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20391", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20392", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20393", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20394", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20395", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20398", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20401", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20402", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20403", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20404", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20405", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20406", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20407", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20408", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20409", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20410", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20411", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20412", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20413", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20414", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20415", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20416", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20418", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20419", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20420", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20421", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20422", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20423", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20424", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20425", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20426", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20427", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20428", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20429", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20431", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20433", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20434", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20435", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20436", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20437", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20439", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20440", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20441", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20442", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20444", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20447", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20451", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20453", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20456", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20460", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20463", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20468", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20469", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20470", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20472", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20500", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20501", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20502", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20503", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20504", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20505", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20506", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20507", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20508", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20509", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20510", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20511", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20515", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20520", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20521", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20522", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20523", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20524", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20525", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20526", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20527", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20528", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20529", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20530", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20531", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20532", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20533", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20534", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20535", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20536", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20537", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20538", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20539", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20540", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20541", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20542", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20543", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20544", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20546", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20547", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20548", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20549", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20551", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20552", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20553", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20554", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20555", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20557", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20558", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20559", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20560", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20565", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20566", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20570", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20571", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20572", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20573", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20575", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20576", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20577", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20578", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20579", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20580", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20581", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20585", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20586", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20590", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20591", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20593", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20594", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20597", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20599", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "56901", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "56915", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "56920", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "56933", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "56944", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "56972", "FIPS": "11001", "County": "District of Columbia", "State": "DC", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "32601", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32602", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32603", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32604", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32605", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32606", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32607", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32608", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32609", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32610", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32611", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32612", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32613", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32614", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32615", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32616", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32618", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32627", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32631", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32633", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32635", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32640", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32641", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32643", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32653", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32654", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32655", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32658", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32662", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32667", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32669", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32694", "FIPS": "12001", "County": "Alachua", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32040", "FIPS": "12003", "County": "Baker", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32063", "FIPS": "12003", "County": "Baker", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32072", "FIPS": "12003", "County": "Baker", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32087", "FIPS": "12003", "County": "Baker", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32401", "FIPS": "12005", "County": "Bay", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32402", "FIPS": "12005", "County": "Bay", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32403", "FIPS": "12005", "County": "Bay", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32404", "FIPS": "12005", "County": "Bay", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32405", "FIPS": "12005", "County": "Bay", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32406", "FIPS": "12005", "County": "Bay", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32407", "FIPS": "12005", "County": "Bay", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32408", "FIPS": "12005", "County": "Bay", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32409", "FIPS": "12005", "County": "Bay", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32410", "FIPS": "12005", "County": "Bay", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32411", "FIPS": "12005", "County": "Bay", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32412", "FIPS": "12005", "County": "Bay", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32413", "FIPS": "12005", "County": "Bay", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32417", "FIPS": "12005", "County": "Bay", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32438", "FIPS": "12005", "County": "Bay", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32444", "FIPS": "12005", "County": "Bay", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32466", "FIPS": "12005", "County": "Bay", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32042", "FIPS": "12007", "County": "Bradford", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32044", "FIPS": "12007", "County": "Bradford", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32058", "FIPS": "12007", "County": "Bradford", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32091", "FIPS": "12007", "County": "Bradford", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32622", "FIPS": "12007", "County": "Bradford", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32754", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32775", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32780", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32781", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32782", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32783", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32796", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32815", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32899", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32901", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32902", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32903", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32904", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32905", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32906", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32907", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32908", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32909", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32910", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32911", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32912", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32919", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32920", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32922", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32923", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32924", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32925", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32926", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32927", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32931", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32932", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32934", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32935", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32936", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32937", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32940", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32941", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32949", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32950", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32951", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32952", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32953", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32954", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32955", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32956", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32959", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32976", "FIPS": "12009", "County": "Brevard", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "33004", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33008", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33009", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33019", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33020", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33021", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33022", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33023", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33024", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33025", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33026", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33027", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33028", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33029", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33060", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33061", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33062", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33063", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33064", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33065", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33066", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33067", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33068", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33069", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33071", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33072", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33073", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33074", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33075", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33076", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33077", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33081", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33082", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33083", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33084", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33093", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33097", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33301", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33302", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33303", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33304", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33305", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33306", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33307", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33308", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33309", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33310", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33311", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33312", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33313", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33314", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33315", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33316", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33317", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33318", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33319", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33320", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33321", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33322", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33323", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33324", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33325", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33326", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33327", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33328", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33329", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33330", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33331", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33332", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33334", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33335", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33336", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33337", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33338", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33339", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33340", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33345", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33346", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33348", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33349", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33351", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33355", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33359", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33388", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33394", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33441", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33442", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33443", "FIPS": "12011", "County": "Broward", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "32421", "FIPS": "12013", "County": "Calhoun", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32424", "FIPS": "12013", "County": "Calhoun", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32430", "FIPS": "12013", "County": "Calhoun", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32449", "FIPS": "12013", "County": "Calhoun", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "33927", "FIPS": "12015", "County": "Charlotte", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33938", "FIPS": "12015", "County": "Charlotte", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33946", "FIPS": "12015", "County": "Charlotte", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33947", "FIPS": "12015", "County": "Charlotte", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33948", "FIPS": "12015", "County": "Charlotte", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33949", "FIPS": "12015", "County": "Charlotte", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33950", "FIPS": "12015", "County": "Charlotte", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33951", "FIPS": "12015", "County": "Charlotte", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33952", "FIPS": "12015", "County": "Charlotte", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33953", "FIPS": "12015", "County": "Charlotte", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33954", "FIPS": "12015", "County": "Charlotte", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33955", "FIPS": "12015", "County": "Charlotte", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33980", "FIPS": "12015", "County": "Charlotte", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33981", "FIPS": "12015", "County": "Charlotte", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33982", "FIPS": "12015", "County": "Charlotte", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33983", "FIPS": "12015", "County": "Charlotte", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34224", "FIPS": "12015", "County": "Charlotte", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34423", "FIPS": "12017", "County": "Citrus", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34428", "FIPS": "12017", "County": "Citrus", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34429", "FIPS": "12017", "County": "Citrus", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34433", "FIPS": "12017", "County": "Citrus", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34434", "FIPS": "12017", "County": "Citrus", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34436", "FIPS": "12017", "County": "Citrus", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34442", "FIPS": "12017", "County": "Citrus", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34445", "FIPS": "12017", "County": "Citrus", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34446", "FIPS": "12017", "County": "Citrus", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34447", "FIPS": "12017", "County": "Citrus", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34448", "FIPS": "12017", "County": "Citrus", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34450", "FIPS": "12017", "County": "Citrus", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34451", "FIPS": "12017", "County": "Citrus", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34452", "FIPS": "12017", "County": "Citrus", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34453", "FIPS": "12017", "County": "Citrus", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34460", "FIPS": "12017", "County": "Citrus", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34461", "FIPS": "12017", "County": "Citrus", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34464", "FIPS": "12017", "County": "Citrus", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34465", "FIPS": "12017", "County": "Citrus", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34487", "FIPS": "12017", "County": "Citrus", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "32003", "FIPS": "12019", "County": "Clay", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32006", "FIPS": "12019", "County": "Clay", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32030", "FIPS": "12019", "County": "Clay", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32043", "FIPS": "12019", "County": "Clay", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32050", "FIPS": "12019", "County": "Clay", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32065", "FIPS": "12019", "County": "Clay", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32067", "FIPS": "12019", "County": "Clay", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32068", "FIPS": "12019", "County": "Clay", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32073", "FIPS": "12019", "County": "Clay", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32079", "FIPS": "12019", "County": "Clay", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32160", "FIPS": "12019", "County": "Clay", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32656", "FIPS": "12019", "County": "Clay", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "34101", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34102", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34103", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34104", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34105", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34106", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34107", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34108", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34109", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34110", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34112", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34113", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34114", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34116", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34117", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34119", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34120", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34137", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34138", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34139", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34140", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34141", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34142", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34143", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34145", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34146", "FIPS": "12021", "County": "Collier", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "32024", "FIPS": "12023", "County": "Columbia", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32025", "FIPS": "12023", "County": "Columbia", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32038", "FIPS": "12023", "County": "Columbia", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32055", "FIPS": "12023", "County": "Columbia", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32056", "FIPS": "12023", "County": "Columbia", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32061", "FIPS": "12023", "County": "Columbia", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "34265", "FIPS": "12027", "County": "De Soto", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34266", "FIPS": "12027", "County": "De Soto", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34267", "FIPS": "12027", "County": "De Soto", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34268", "FIPS": "12027", "County": "De Soto", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34269", "FIPS": "12027", "County": "De Soto", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "32628", "FIPS": "12029", "County": "Dixie", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32648", "FIPS": "12029", "County": "Dixie", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32680", "FIPS": "12029", "County": "Dixie", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32692", "FIPS": "12029", "County": "Dixie", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32099", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32201", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32202", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32203", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32204", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32205", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32206", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32207", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32208", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32209", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32210", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32211", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32212", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32214", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32215", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32216", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32217", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32218", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32219", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32220", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32221", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32222", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32223", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32224", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32225", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32226", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32227", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32228", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32229", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32230", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32231", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32232", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32233", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32234", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32235", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32236", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32237", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32238", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32239", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32240", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32241", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32244", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32245", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32246", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32247", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32250", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32254", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32255", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32256", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32257", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32258", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32266", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32267", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32277", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32290", "FIPS": "12031", "County": "Duval", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32501", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32502", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32503", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32504", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32505", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32506", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32507", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32508", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32509", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32511", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32512", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32513", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32514", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32516", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32520", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32521", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32522", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32523", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32524", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32526", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32533", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32534", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32535", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32559", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32560", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32568", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32577", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32590", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32591", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32592", "FIPS": "12033", "County": "Escambia", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32110", "FIPS": "12035", "County": "Flagler", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32135", "FIPS": "12035", "County": "Flagler", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32136", "FIPS": "12035", "County": "Flagler", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32137", "FIPS": "12035", "County": "Flagler", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32142", "FIPS": "12035", "County": "Flagler", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32164", "FIPS": "12035", "County": "Flagler", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32320", "FIPS": "12037", "County": "Franklin", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32322", "FIPS": "12037", "County": "Franklin", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32323", "FIPS": "12037", "County": "Franklin", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32328", "FIPS": "12037", "County": "Franklin", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32329", "FIPS": "12037", "County": "Franklin", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32324", "FIPS": "12039", "County": "Gadsden", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32330", "FIPS": "12039", "County": "Gadsden", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32332", "FIPS": "12039", "County": "Gadsden", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32333", "FIPS": "12039", "County": "Gadsden", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32343", "FIPS": "12039", "County": "Gadsden", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32351", "FIPS": "12039", "County": "Gadsden", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32352", "FIPS": "12039", "County": "Gadsden", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32353", "FIPS": "12039", "County": "Gadsden", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32619", "FIPS": "12041", "County": "Gilchrist", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32693", "FIPS": "12041", "County": "Gilchrist", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "33471", "FIPS": "12043", "County": "Glades", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33944", "FIPS": "12043", "County": "Glades", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "32456", "FIPS": "12045", "County": "Gulf", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32457", "FIPS": "12045", "County": "Gulf", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32465", "FIPS": "12045", "County": "Gulf", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32052", "FIPS": "12047", "County": "Hamilton", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32053", "FIPS": "12047", "County": "Hamilton", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32096", "FIPS": "12047", "County": "Hamilton", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "33834", "FIPS": "12049", "County": "Hardee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33865", "FIPS": "12049", "County": "Hardee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33873", "FIPS": "12049", "County": "Hardee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33890", "FIPS": "12049", "County": "Hardee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33440", "FIPS": "12051", "County": "Hendry", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33930", "FIPS": "12051", "County": "Hendry", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33935", "FIPS": "12051", "County": "Hendry", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33975", "FIPS": "12051", "County": "Hendry", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34601", "FIPS": "12053", "County": "Hernando", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34602", "FIPS": "12053", "County": "Hernando", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34603", "FIPS": "12053", "County": "Hernando", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34604", "FIPS": "12053", "County": "Hernando", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34605", "FIPS": "12053", "County": "Hernando", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34606", "FIPS": "12053", "County": "Hernando", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34607", "FIPS": "12053", "County": "Hernando", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34608", "FIPS": "12053", "County": "Hernando", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34609", "FIPS": "12053", "County": "Hernando", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34611", "FIPS": "12053", "County": "Hernando", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34613", "FIPS": "12053", "County": "Hernando", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34614", "FIPS": "12053", "County": "Hernando", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34636", "FIPS": "12053", "County": "Hernando", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34661", "FIPS": "12053", "County": "Hernando", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33825", "FIPS": "12055", "County": "Highlands", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33826", "FIPS": "12055", "County": "Highlands", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33852", "FIPS": "12055", "County": "Highlands", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33857", "FIPS": "12055", "County": "Highlands", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33862", "FIPS": "12055", "County": "Highlands", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33870", "FIPS": "12055", "County": "Highlands", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33871", "FIPS": "12055", "County": "Highlands", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33872", "FIPS": "12055", "County": "Highlands", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33875", "FIPS": "12055", "County": "Highlands", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33876", "FIPS": "12055", "County": "Highlands", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33960", "FIPS": "12055", "County": "Highlands", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33503", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33508", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33509", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33510", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33511", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33527", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33530", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33534", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33547", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33548", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33549", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33550", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33556", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33558", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33559", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33563", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33564", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33565", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33566", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33567", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33568", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33569", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33570", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33571", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33572", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33573", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33575", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33583", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33584", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33586", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33587", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33592", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33594", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33595", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33598", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33601", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33602", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33603", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33604", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33605", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33606", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33607", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33608", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33609", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33610", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33611", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33612", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33613", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33614", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33615", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33616", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33617", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33618", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33619", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33620", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33621", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33622", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33623", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33624", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33625", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33626", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33629", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33630", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33631", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33633", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33634", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33635", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33637", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33647", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33650", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33651", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33655", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33660", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33661", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33662", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33663", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33664", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33672", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33673", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33674", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33675", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33677", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33679", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33680", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33681", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33682", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33684", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33685", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33686", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33687", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33688", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33689", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33690", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33694", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33697", "FIPS": "12057", "County": "Hillsborough", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "32425", "FIPS": "12059", "County": "Holmes", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32452", "FIPS": "12059", "County": "Holmes", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32455", "FIPS": "12059", "County": "Holmes", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32464", "FIPS": "12059", "County": "Holmes", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32948", "FIPS": "12061", "County": "Indian River", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "32957", "FIPS": "12061", "County": "Indian River", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "32958", "FIPS": "12061", "County": "Indian River", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "32960", "FIPS": "12061", "County": "Indian River", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "32961", "FIPS": "12061", "County": "Indian River", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "32962", "FIPS": "12061", "County": "Indian River", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "32963", "FIPS": "12061", "County": "Indian River", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "32964", "FIPS": "12061", "County": "Indian River", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "32965", "FIPS": "12061", "County": "Indian River", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "32966", "FIPS": "12061", "County": "Indian River", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "32967", "FIPS": "12061", "County": "Indian River", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "32968", "FIPS": "12061", "County": "Indian River", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "32969", "FIPS": "12061", "County": "Indian River", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "32970", "FIPS": "12061", "County": "Indian River", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "32971", "FIPS": "12061", "County": "Indian River", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "32978", "FIPS": "12061", "County": "Indian River", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "32420", "FIPS": "12063", "County": "Jackson", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32423", "FIPS": "12063", "County": "Jackson", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32426", "FIPS": "12063", "County": "Jackson", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32431", "FIPS": "12063", "County": "Jackson", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32432", "FIPS": "12063", "County": "Jackson", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32440", "FIPS": "12063", "County": "Jackson", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32442", "FIPS": "12063", "County": "Jackson", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32443", "FIPS": "12063", "County": "Jackson", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32445", "FIPS": "12063", "County": "Jackson", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32446", "FIPS": "12063", "County": "Jackson", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32447", "FIPS": "12063", "County": "Jackson", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32448", "FIPS": "12063", "County": "Jackson", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32460", "FIPS": "12063", "County": "Jackson", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32336", "FIPS": "12065", "County": "Jefferson", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32337", "FIPS": "12065", "County": "Jefferson", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32344", "FIPS": "12065", "County": "Jefferson", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32345", "FIPS": "12065", "County": "Jefferson", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32361", "FIPS": "12065", "County": "Jefferson", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32013", "FIPS": "12067", "County": "Lafayette", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32066", "FIPS": "12067", "County": "Lafayette", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32102", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32158", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32159", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32702", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32726", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32727", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32735", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32736", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32756", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32757", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32767", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32776", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32778", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32784", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34705", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34711", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34712", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34713", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34714", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34715", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34729", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34731", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34736", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34737", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34748", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34749", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34753", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34755", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34756", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34762", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34788", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34789", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34797", "FIPS": "12069", "County": "Lake", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "33901", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33902", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33903", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33904", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33905", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33906", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33907", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33908", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33909", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33910", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33911", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33912", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33913", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33914", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33915", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33916", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33917", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33918", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33919", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33920", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33921", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33922", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33924", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33928", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33931", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33932", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33936", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33945", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33956", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33957", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33965", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33966", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33967", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33970", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33971", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33972", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33990", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33991", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33993", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "33994", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34133", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34134", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34135", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "34136", "FIPS": "12071", "County": "Lee", "State": "FL", "DMA": {"Code": 571, "Name": "FORT MYERS-NAPLES"}},
  {"Postal_Code": "32301", "FIPS": "12073", "County": "Leon", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32302", "FIPS": "12073", "County": "Leon", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32303", "FIPS": "12073", "County": "Leon", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32304", "FIPS": "12073", "County": "Leon", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32305", "FIPS": "12073", "County": "Leon", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32306", "FIPS": "12073", "County": "Leon", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32307", "FIPS": "12073", "County": "Leon", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32308", "FIPS": "12073", "County": "Leon", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32309", "FIPS": "12073", "County": "Leon", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32310", "FIPS": "12073", "County": "Leon", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32311", "FIPS": "12073", "County": "Leon", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32312", "FIPS": "12073", "County": "Leon", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32313", "FIPS": "12073", "County": "Leon", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32314", "FIPS": "12073", "County": "Leon", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32315", "FIPS": "12073", "County": "Leon", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32316", "FIPS": "12073", "County": "Leon", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32317", "FIPS": "12073", "County": "Leon", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32318", "FIPS": "12073", "County": "Leon", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32362", "FIPS": "12073", "County": "Leon", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32395", "FIPS": "12073", "County": "Leon", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32399", "FIPS": "12073", "County": "Leon", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32621", "FIPS": "12075", "County": "Levy", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32625", "FIPS": "12075", "County": "Levy", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32626", "FIPS": "12075", "County": "Levy", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32639", "FIPS": "12075", "County": "Levy", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32644", "FIPS": "12075", "County": "Levy", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32668", "FIPS": "12075", "County": "Levy", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32683", "FIPS": "12075", "County": "Levy", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32696", "FIPS": "12075", "County": "Levy", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "34449", "FIPS": "12075", "County": "Levy", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "34498", "FIPS": "12075", "County": "Levy", "State": "FL", "DMA": {"Code": 592, "Name": "GAINESVILLE"}},
  {"Postal_Code": "32321", "FIPS": "12077", "County": "Liberty", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32334", "FIPS": "12077", "County": "Liberty", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32335", "FIPS": "12077", "County": "Liberty", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32360", "FIPS": "12077", "County": "Liberty", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32059", "FIPS": "12079", "County": "Madison", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32331", "FIPS": "12079", "County": "Madison", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32340", "FIPS": "12079", "County": "Madison", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32341", "FIPS": "12079", "County": "Madison", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32350", "FIPS": "12079", "County": "Madison", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "34201", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34202", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34203", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34204", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34205", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34206", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34207", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34208", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34209", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34210", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34211", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34212", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34215", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34216", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34217", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34218", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34219", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34220", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34221", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34222", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34228", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34243", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34250", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34251", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34260", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34264", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34270", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34280", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34281", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34282", "FIPS": "12081", "County": "Manatee", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "32111", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32113", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32133", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32134", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32179", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32182", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32183", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32192", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32195", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32617", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32634", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32663", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32664", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32681", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32686", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34420", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34421", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34430", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34431", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34432", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34470", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34471", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34472", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34473", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34474", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34475", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34476", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34477", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34478", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34479", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34480", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34481", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34482", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34483", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34488", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34489", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34491", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34492", "FIPS": "12083", "County": "Marion", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "33455", "FIPS": "12085", "County": "Martin", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33475", "FIPS": "12085", "County": "Martin", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34956", "FIPS": "12085", "County": "Martin", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34957", "FIPS": "12085", "County": "Martin", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34958", "FIPS": "12085", "County": "Martin", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34990", "FIPS": "12085", "County": "Martin", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34991", "FIPS": "12085", "County": "Martin", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34992", "FIPS": "12085", "County": "Martin", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34994", "FIPS": "12085", "County": "Martin", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34995", "FIPS": "12085", "County": "Martin", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34996", "FIPS": "12085", "County": "Martin", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34997", "FIPS": "12085", "County": "Martin", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33002", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33010", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33011", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33012", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33013", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33014", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33015", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33016", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33017", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33018", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33030", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33031", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33032", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33033", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33034", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33035", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33039", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33054", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33055", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33056", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33090", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33092", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33101", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33102", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33107", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33109", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33110", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33111", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33112", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33114", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33116", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33119", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33121", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33122", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33124", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33125", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33126", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33127", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33128", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33129", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33130", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33131", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33132", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33133", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33134", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33135", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33136", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33137", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33138", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33139", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33140", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33141", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33142", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33143", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33144", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33145", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33146", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33147", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33148", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33149", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33150", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33151", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33152", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33153", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33154", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33155", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33156", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33157", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33158", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33159", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33160", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33161", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33162", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33163", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33164", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33165", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33166", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33167", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33168", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33169", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33170", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33172", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33173", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33174", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33175", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33176", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33177", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33178", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33179", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33180", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33181", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33182", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33183", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33184", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33185", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33186", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33187", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33188", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33189", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33190", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33193", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33194", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33195", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33196", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33197", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33199", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33231", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33233", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33234", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33238", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33239", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33242", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33243", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33245", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33247", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33255", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33256", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33257", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33261", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33265", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33266", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33269", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33280", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33283", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33296", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33299", "FIPS": "12086", "County": "Miami-Dade", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33001", "FIPS": "12087", "County": "Monroe", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33036", "FIPS": "12087", "County": "Monroe", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33037", "FIPS": "12087", "County": "Monroe", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33040", "FIPS": "12087", "County": "Monroe", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33041", "FIPS": "12087", "County": "Monroe", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33042", "FIPS": "12087", "County": "Monroe", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33043", "FIPS": "12087", "County": "Monroe", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33045", "FIPS": "12087", "County": "Monroe", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33050", "FIPS": "12087", "County": "Monroe", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33051", "FIPS": "12087", "County": "Monroe", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33052", "FIPS": "12087", "County": "Monroe", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "33070", "FIPS": "12087", "County": "Monroe", "State": "FL", "DMA": {"Code": 528, "Name": "MIAMI-FORT LAUDERDALE"}},
  {"Postal_Code": "32009", "FIPS": "12089", "County": "Nassau", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32011", "FIPS": "12089", "County": "Nassau", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32034", "FIPS": "12089", "County": "Nassau", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32035", "FIPS": "12089", "County": "Nassau", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32041", "FIPS": "12089", "County": "Nassau", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32046", "FIPS": "12089", "County": "Nassau", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32097", "FIPS": "12089", "County": "Nassau", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32531", "FIPS": "12091", "County": "Okaloosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32536", "FIPS": "12091", "County": "Okaloosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32537", "FIPS": "12091", "County": "Okaloosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32539", "FIPS": "12091", "County": "Okaloosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32540", "FIPS": "12091", "County": "Okaloosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32541", "FIPS": "12091", "County": "Okaloosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32542", "FIPS": "12091", "County": "Okaloosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32544", "FIPS": "12091", "County": "Okaloosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32547", "FIPS": "12091", "County": "Okaloosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32548", "FIPS": "12091", "County": "Okaloosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32549", "FIPS": "12091", "County": "Okaloosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32564", "FIPS": "12091", "County": "Okaloosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32567", "FIPS": "12091", "County": "Okaloosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32569", "FIPS": "12091", "County": "Okaloosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32578", "FIPS": "12091", "County": "Okaloosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32579", "FIPS": "12091", "County": "Okaloosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32580", "FIPS": "12091", "County": "Okaloosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32588", "FIPS": "12091", "County": "Okaloosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "34972", "FIPS": "12093", "County": "Okeechobee", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34973", "FIPS": "12093", "County": "Okeechobee", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34974", "FIPS": "12093", "County": "Okeechobee", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "32703", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32704", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32709", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32710", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32712", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32751", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32768", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32777", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32789", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32790", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32792", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32793", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32794", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32798", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32801", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32802", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32803", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32804", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32805", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32806", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32807", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32808", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32809", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32810", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32811", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32812", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32814", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32816", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32817", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32818", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32819", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32820", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32821", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32822", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32824", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32825", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32826", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32827", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32828", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32829", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32830", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32831", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32832", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32833", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32834", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32835", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32836", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32837", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32839", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32853", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32854", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32855", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32856", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32857", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32858", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32859", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32860", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32861", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32862", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32867", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32868", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32869", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32872", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32877", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32878", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32885", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32886", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32887", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32890", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32891", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32893", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32896", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32897", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32898", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34734", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34740", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34760", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34761", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34777", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34778", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34786", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34787", "FIPS": "12095", "County": "Orange", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "33848", "FIPS": "12097", "County": "Osceola", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34739", "FIPS": "12097", "County": "Osceola", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34741", "FIPS": "12097", "County": "Osceola", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34742", "FIPS": "12097", "County": "Osceola", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34743", "FIPS": "12097", "County": "Osceola", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34744", "FIPS": "12097", "County": "Osceola", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34745", "FIPS": "12097", "County": "Osceola", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34746", "FIPS": "12097", "County": "Osceola", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34747", "FIPS": "12097", "County": "Osceola", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34758", "FIPS": "12097", "County": "Osceola", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34769", "FIPS": "12097", "County": "Osceola", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34770", "FIPS": "12097", "County": "Osceola", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34771", "FIPS": "12097", "County": "Osceola", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34772", "FIPS": "12097", "County": "Osceola", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34773", "FIPS": "12097", "County": "Osceola", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "33401", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33402", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33403", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33404", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33405", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33406", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33407", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33408", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33409", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33410", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33411", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33412", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33413", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33414", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33415", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33416", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33417", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33418", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33419", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33420", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33421", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33422", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33424", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33425", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33426", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33427", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33428", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33429", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33430", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33431", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33432", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33433", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33434", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33435", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33436", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33437", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33438", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33439", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33444", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33445", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33446", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33447", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33448", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33454", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33458", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33459", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33460", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33461", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33462", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33463", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33464", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33465", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33466", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33467", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33468", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33469", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33470", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33474", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33476", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33477", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33478", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33480", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33481", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33482", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33483", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33484", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33486", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33487", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33488", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33493", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33496", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33497", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33498", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33499", "FIPS": "12099", "County": "Palm Beach", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "33523", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33524", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33525", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33526", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33537", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33539", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33540", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33541", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33542", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33543", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33544", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33574", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33576", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33593", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34610", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34637", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34638", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34639", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34652", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34653", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34654", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34655", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34656", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34667", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34668", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34669", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34673", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34674", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34679", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34680", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34690", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34691", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34692", "FIPS": "12101", "County": "Pasco", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33701", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33702", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33703", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33704", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33705", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33706", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33707", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33708", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33709", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33710", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33711", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33712", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33713", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33714", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33715", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33716", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33729", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33730", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33731", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33732", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33733", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33734", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33736", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33737", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33738", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33740", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33741", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33742", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33743", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33744", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33747", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33755", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33756", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33757", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33758", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33759", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33760", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33761", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33762", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33763", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33764", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33765", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33766", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33767", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33769", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33770", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33771", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33772", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33773", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33774", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33775", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33776", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33777", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33778", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33779", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33780", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33781", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33782", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33784", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33785", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33786", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34660", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34677", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34681", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34682", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34683", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34684", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34685", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34688", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34689", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34695", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34697", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34698", "FIPS": "12103", "County": "Pinellas", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33801", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33802", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33803", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33804", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33805", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33806", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33807", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33809", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33810", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33811", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33812", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33813", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33815", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33820", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33823", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33827", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33830", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33831", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33835", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33836", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33837", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33838", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33839", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33840", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33841", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33843", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33844", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33845", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33846", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33847", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33849", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33850", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33851", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33853", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33854", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33855", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33856", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33858", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33859", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33860", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33863", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33867", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33868", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33877", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33880", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33881", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33882", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33883", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33884", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33885", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33888", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33896", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33897", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "33898", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34759", "FIPS": "12105", "County": "Polk", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "32007", "FIPS": "12107", "County": "Putnam", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32112", "FIPS": "12107", "County": "Putnam", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32131", "FIPS": "12107", "County": "Putnam", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32138", "FIPS": "12107", "County": "Putnam", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32139", "FIPS": "12107", "County": "Putnam", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32140", "FIPS": "12107", "County": "Putnam", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32147", "FIPS": "12107", "County": "Putnam", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32148", "FIPS": "12107", "County": "Putnam", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32149", "FIPS": "12107", "County": "Putnam", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32157", "FIPS": "12107", "County": "Putnam", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32177", "FIPS": "12107", "County": "Putnam", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32178", "FIPS": "12107", "County": "Putnam", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32181", "FIPS": "12107", "County": "Putnam", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32185", "FIPS": "12107", "County": "Putnam", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32187", "FIPS": "12107", "County": "Putnam", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32189", "FIPS": "12107", "County": "Putnam", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32193", "FIPS": "12107", "County": "Putnam", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32666", "FIPS": "12107", "County": "Putnam", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32004", "FIPS": "12109", "County": "Saint Johns", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32033", "FIPS": "12109", "County": "Saint Johns", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32080", "FIPS": "12109", "County": "Saint Johns", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32081", "FIPS": "12109", "County": "Saint Johns", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32082", "FIPS": "12109", "County": "Saint Johns", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32084", "FIPS": "12109", "County": "Saint Johns", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32085", "FIPS": "12109", "County": "Saint Johns", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32086", "FIPS": "12109", "County": "Saint Johns", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32092", "FIPS": "12109", "County": "Saint Johns", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32095", "FIPS": "12109", "County": "Saint Johns", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32145", "FIPS": "12109", "County": "Saint Johns", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32259", "FIPS": "12109", "County": "Saint Johns", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32260", "FIPS": "12109", "County": "Saint Johns", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "34945", "FIPS": "12111", "County": "Saint Lucie", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34946", "FIPS": "12111", "County": "Saint Lucie", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34947", "FIPS": "12111", "County": "Saint Lucie", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34948", "FIPS": "12111", "County": "Saint Lucie", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34949", "FIPS": "12111", "County": "Saint Lucie", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34950", "FIPS": "12111", "County": "Saint Lucie", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34951", "FIPS": "12111", "County": "Saint Lucie", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34952", "FIPS": "12111", "County": "Saint Lucie", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34953", "FIPS": "12111", "County": "Saint Lucie", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34954", "FIPS": "12111", "County": "Saint Lucie", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34979", "FIPS": "12111", "County": "Saint Lucie", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34981", "FIPS": "12111", "County": "Saint Lucie", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34982", "FIPS": "12111", "County": "Saint Lucie", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34983", "FIPS": "12111", "County": "Saint Lucie", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34984", "FIPS": "12111", "County": "Saint Lucie", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34985", "FIPS": "12111", "County": "Saint Lucie", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34986", "FIPS": "12111", "County": "Saint Lucie", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34987", "FIPS": "12111", "County": "Saint Lucie", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "34988", "FIPS": "12111", "County": "Saint Lucie", "State": "FL", "DMA": {"Code": 548, "Name": "WEST PALM BEACH-FT PIERCE"}},
  {"Postal_Code": "32530", "FIPS": "12113", "County": "Santa Rosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32561", "FIPS": "12113", "County": "Santa Rosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32562", "FIPS": "12113", "County": "Santa Rosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32563", "FIPS": "12113", "County": "Santa Rosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32565", "FIPS": "12113", "County": "Santa Rosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32566", "FIPS": "12113", "County": "Santa Rosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32570", "FIPS": "12113", "County": "Santa Rosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32571", "FIPS": "12113", "County": "Santa Rosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32572", "FIPS": "12113", "County": "Santa Rosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "32583", "FIPS": "12113", "County": "Santa Rosa", "State": "FL", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "34223", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34229", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34230", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34231", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34232", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34233", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34234", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34235", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34236", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34237", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34238", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34239", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34240", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34241", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34242", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34272", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34274", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34275", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34276", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34277", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34278", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34284", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34285", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34286", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34287", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34288", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34289", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34292", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34293", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "34295", "FIPS": "12115", "County": "Sarasota", "State": "FL", "DMA": {"Code": 539, "Name": "TAMPA-ST PETERSBURG-SARASOTA"}},
  {"Postal_Code": "32701", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32707", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32708", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32714", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32715", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32716", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32718", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32719", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32730", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32732", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32733", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32745", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32746", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32747", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32750", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32752", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32762", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32765", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32766", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32771", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32772", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32773", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32779", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32791", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32795", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32799", "FIPS": "12117", "County": "Seminole", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32162", "FIPS": "12119", "County": "Sumter", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "33513", "FIPS": "12119", "County": "Sumter", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "33514", "FIPS": "12119", "County": "Sumter", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "33521", "FIPS": "12119", "County": "Sumter", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "33538", "FIPS": "12119", "County": "Sumter", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "33585", "FIPS": "12119", "County": "Sumter", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "33597", "FIPS": "12119", "County": "Sumter", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34484", "FIPS": "12119", "County": "Sumter", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "34785", "FIPS": "12119", "County": "Sumter", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32008", "FIPS": "12121", "County": "Suwannee", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32060", "FIPS": "12121", "County": "Suwannee", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32062", "FIPS": "12121", "County": "Suwannee", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32064", "FIPS": "12121", "County": "Suwannee", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32071", "FIPS": "12121", "County": "Suwannee", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32094", "FIPS": "12121", "County": "Suwannee", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32347", "FIPS": "12123", "County": "Taylor", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32348", "FIPS": "12123", "County": "Taylor", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32356", "FIPS": "12123", "County": "Taylor", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32357", "FIPS": "12123", "County": "Taylor", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32359", "FIPS": "12123", "County": "Taylor", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32026", "FIPS": "12125", "County": "Union", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32054", "FIPS": "12125", "County": "Union", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32083", "FIPS": "12125", "County": "Union", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32697", "FIPS": "12125", "County": "Union", "State": "FL", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "32105", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32114", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32115", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32116", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32117", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32118", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32119", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32120", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32121", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32122", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32123", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32124", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32125", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32126", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32127", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32128", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32129", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32130", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32132", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32141", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32168", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32169", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32170", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32173", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32174", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32175", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32176", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32180", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32190", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32198", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32706", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32713", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32720", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32721", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32722", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32723", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32724", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32725", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32728", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32738", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32739", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32744", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32753", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32759", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32763", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32764", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32774", "FIPS": "12127", "County": "Volusia", "State": "FL", "DMA": {"Code": 534, "Name": "ORLANDO-DAYTONA BEACH-MELBOURNE"}},
  {"Postal_Code": "32326", "FIPS": "12129", "County": "Wakulla", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32327", "FIPS": "12129", "County": "Wakulla", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32346", "FIPS": "12129", "County": "Wakulla", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32355", "FIPS": "12129", "County": "Wakulla", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32358", "FIPS": "12129", "County": "Wakulla", "State": "FL", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "32422", "FIPS": "12131", "County": "Walton", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32433", "FIPS": "12131", "County": "Walton", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32434", "FIPS": "12131", "County": "Walton", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32435", "FIPS": "12131", "County": "Walton", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32439", "FIPS": "12131", "County": "Walton", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32454", "FIPS": "12131", "County": "Walton", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32459", "FIPS": "12131", "County": "Walton", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32461", "FIPS": "12131", "County": "Walton", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32538", "FIPS": "12131", "County": "Walton", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32550", "FIPS": "12131", "County": "Walton", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32427", "FIPS": "12133", "County": "Washington", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32428", "FIPS": "12133", "County": "Washington", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32437", "FIPS": "12133", "County": "Washington", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32462", "FIPS": "12133", "County": "Washington", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "32463", "FIPS": "12133", "County": "Washington", "State": "FL", "DMA": {"Code": 656, "Name": "PANAMA CITY"}},
  {"Postal_Code": "31513", "FIPS": "13001", "County": "Appling", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31515", "FIPS": "13001", "County": "Appling", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31563", "FIPS": "13001", "County": "Appling", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31624", "FIPS": "13003", "County": "Atkinson", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31642", "FIPS": "13003", "County": "Atkinson", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31650", "FIPS": "13003", "County": "Atkinson", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31510", "FIPS": "13005", "County": "Bacon", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "39870", "FIPS": "13007", "County": "Baker", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31034", "FIPS": "13009", "County": "Baldwin", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31059", "FIPS": "13009", "County": "Baldwin", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31061", "FIPS": "13009", "County": "Baldwin", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31062", "FIPS": "13009", "County": "Baldwin", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "30511", "FIPS": "13011", "County": "Banks", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30530", "FIPS": "13011", "County": "Banks", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30547", "FIPS": "13011", "County": "Banks", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30558", "FIPS": "13011", "County": "Banks", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30011", "FIPS": "13013", "County": "Barrow", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30620", "FIPS": "13013", "County": "Barrow", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30666", "FIPS": "13013", "County": "Barrow", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30680", "FIPS": "13013", "County": "Barrow", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30103", "FIPS": "13015", "County": "Bartow", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30120", "FIPS": "13015", "County": "Bartow", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30121", "FIPS": "13015", "County": "Bartow", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30123", "FIPS": "13015", "County": "Bartow", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30137", "FIPS": "13015", "County": "Bartow", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30145", "FIPS": "13015", "County": "Bartow", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30171", "FIPS": "13015", "County": "Bartow", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30178", "FIPS": "13015", "County": "Bartow", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30184", "FIPS": "13015", "County": "Bartow", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31750", "FIPS": "13017", "County": "Ben Hill", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31622", "FIPS": "13019", "County": "Berrien", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31639", "FIPS": "13019", "County": "Berrien", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31645", "FIPS": "13019", "County": "Berrien", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31749", "FIPS": "13019", "County": "Berrien", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31052", "FIPS": "13021", "County": "Bibb", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31201", "FIPS": "13021", "County": "Bibb", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31202", "FIPS": "13021", "County": "Bibb", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31203", "FIPS": "13021", "County": "Bibb", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31204", "FIPS": "13021", "County": "Bibb", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31205", "FIPS": "13021", "County": "Bibb", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31206", "FIPS": "13021", "County": "Bibb", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31207", "FIPS": "13021", "County": "Bibb", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31208", "FIPS": "13021", "County": "Bibb", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31209", "FIPS": "13021", "County": "Bibb", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31210", "FIPS": "13021", "County": "Bibb", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31211", "FIPS": "13021", "County": "Bibb", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31212", "FIPS": "13021", "County": "Bibb", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31213", "FIPS": "13021", "County": "Bibb", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31216", "FIPS": "13021", "County": "Bibb", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31217", "FIPS": "13021", "County": "Bibb", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31220", "FIPS": "13021", "County": "Bibb", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31221", "FIPS": "13021", "County": "Bibb", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31294", "FIPS": "13021", "County": "Bibb", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31295", "FIPS": "13021", "County": "Bibb", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31296", "FIPS": "13021", "County": "Bibb", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31297", "FIPS": "13021", "County": "Bibb", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31014", "FIPS": "13023", "County": "Bleckley", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31542", "FIPS": "13025", "County": "Brantley", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31543", "FIPS": "13025", "County": "Brantley", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31553", "FIPS": "13025", "County": "Brantley", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31566", "FIPS": "13025", "County": "Brantley", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31625", "FIPS": "13027", "County": "Brooks", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31629", "FIPS": "13027", "County": "Brooks", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31638", "FIPS": "13027", "County": "Brooks", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31643", "FIPS": "13027", "County": "Brooks", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31720", "FIPS": "13027", "County": "Brooks", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31308", "FIPS": "13029", "County": "Bryan", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31321", "FIPS": "13029", "County": "Bryan", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31324", "FIPS": "13029", "County": "Bryan", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30415", "FIPS": "13031", "County": "Bulloch", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30450", "FIPS": "13031", "County": "Bulloch", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30452", "FIPS": "13031", "County": "Bulloch", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30458", "FIPS": "13031", "County": "Bulloch", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30459", "FIPS": "13031", "County": "Bulloch", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30460", "FIPS": "13031", "County": "Bulloch", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30461", "FIPS": "13031", "County": "Bulloch", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30426", "FIPS": "13033", "County": "Burke", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30441", "FIPS": "13033", "County": "Burke", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30456", "FIPS": "13033", "County": "Burke", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30811", "FIPS": "13033", "County": "Burke", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30816", "FIPS": "13033", "County": "Burke", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30830", "FIPS": "13033", "County": "Burke", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30216", "FIPS": "13035", "County": "Butts", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30233", "FIPS": "13035", "County": "Butts", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30234", "FIPS": "13035", "County": "Butts", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "39813", "FIPS": "13037", "County": "Calhoun", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "39846", "FIPS": "13037", "County": "Calhoun", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "39862", "FIPS": "13037", "County": "Calhoun", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "39866", "FIPS": "13037", "County": "Calhoun", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31547", "FIPS": "13039", "County": "Camden", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31548", "FIPS": "13039", "County": "Camden", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31558", "FIPS": "13039", "County": "Camden", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31565", "FIPS": "13039", "County": "Camden", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31568", "FIPS": "13039", "County": "Camden", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31569", "FIPS": "13039", "County": "Camden", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "30439", "FIPS": "13043", "County": "Candler", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30451", "FIPS": "13043", "County": "Candler", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30108", "FIPS": "13045", "County": "Carroll", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30109", "FIPS": "13045", "County": "Carroll", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30112", "FIPS": "13045", "County": "Carroll", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30116", "FIPS": "13045", "County": "Carroll", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30117", "FIPS": "13045", "County": "Carroll", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30118", "FIPS": "13045", "County": "Carroll", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30119", "FIPS": "13045", "County": "Carroll", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30150", "FIPS": "13045", "County": "Carroll", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30170", "FIPS": "13045", "County": "Carroll", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30179", "FIPS": "13045", "County": "Carroll", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30180", "FIPS": "13045", "County": "Carroll", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30185", "FIPS": "13045", "County": "Carroll", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30726", "FIPS": "13047", "County": "Catoosa", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "30736", "FIPS": "13047", "County": "Catoosa", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "30742", "FIPS": "13047", "County": "Catoosa", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "31537", "FIPS": "13049", "County": "Charlton", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31562", "FIPS": "13049", "County": "Charlton", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31302", "FIPS": "13051", "County": "Chatham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31322", "FIPS": "13051", "County": "Chatham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31328", "FIPS": "13051", "County": "Chatham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31401", "FIPS": "13051", "County": "Chatham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31402", "FIPS": "13051", "County": "Chatham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31403", "FIPS": "13051", "County": "Chatham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31404", "FIPS": "13051", "County": "Chatham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31405", "FIPS": "13051", "County": "Chatham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31406", "FIPS": "13051", "County": "Chatham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31407", "FIPS": "13051", "County": "Chatham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31408", "FIPS": "13051", "County": "Chatham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31409", "FIPS": "13051", "County": "Chatham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31410", "FIPS": "13051", "County": "Chatham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31411", "FIPS": "13051", "County": "Chatham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31412", "FIPS": "13051", "County": "Chatham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31414", "FIPS": "13051", "County": "Chatham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31415", "FIPS": "13051", "County": "Chatham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31416", "FIPS": "13051", "County": "Chatham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31418", "FIPS": "13051", "County": "Chatham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31419", "FIPS": "13051", "County": "Chatham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31420", "FIPS": "13051", "County": "Chatham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31421", "FIPS": "13051", "County": "Chatham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31805", "FIPS": "13053", "County": "Chattahoochee", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "30730", "FIPS": "13055", "County": "Chattooga", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30731", "FIPS": "13055", "County": "Chattooga", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30747", "FIPS": "13055", "County": "Chattooga", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30753", "FIPS": "13055", "County": "Chattooga", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30102", "FIPS": "13057", "County": "Cherokee", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30107", "FIPS": "13057", "County": "Cherokee", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30114", "FIPS": "13057", "County": "Cherokee", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30115", "FIPS": "13057", "County": "Cherokee", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30142", "FIPS": "13057", "County": "Cherokee", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30146", "FIPS": "13057", "County": "Cherokee", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30151", "FIPS": "13057", "County": "Cherokee", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30169", "FIPS": "13057", "County": "Cherokee", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30183", "FIPS": "13057", "County": "Cherokee", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30188", "FIPS": "13057", "County": "Cherokee", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30189", "FIPS": "13057", "County": "Cherokee", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30601", "FIPS": "13059", "County": "Clarke", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30602", "FIPS": "13059", "County": "Clarke", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30603", "FIPS": "13059", "County": "Clarke", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30604", "FIPS": "13059", "County": "Clarke", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30605", "FIPS": "13059", "County": "Clarke", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30606", "FIPS": "13059", "County": "Clarke", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30607", "FIPS": "13059", "County": "Clarke", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30608", "FIPS": "13059", "County": "Clarke", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30609", "FIPS": "13059", "County": "Clarke", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30612", "FIPS": "13059", "County": "Clarke", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30683", "FIPS": "13059", "County": "Clarke", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "39824", "FIPS": "13061", "County": "Clay", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "39851", "FIPS": "13061", "County": "Clay", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "30236", "FIPS": "13063", "County": "Clayton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30237", "FIPS": "13063", "County": "Clayton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30238", "FIPS": "13063", "County": "Clayton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30250", "FIPS": "13063", "County": "Clayton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30260", "FIPS": "13063", "County": "Clayton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30273", "FIPS": "13063", "County": "Clayton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30274", "FIPS": "13063", "County": "Clayton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30287", "FIPS": "13063", "County": "Clayton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30288", "FIPS": "13063", "County": "Clayton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30296", "FIPS": "13063", "County": "Clayton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30297", "FIPS": "13063", "County": "Clayton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30298", "FIPS": "13063", "County": "Clayton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31623", "FIPS": "13065", "County": "Clinch", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31630", "FIPS": "13065", "County": "Clinch", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31631", "FIPS": "13065", "County": "Clinch", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31634", "FIPS": "13065", "County": "Clinch", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "30006", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30007", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30008", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30060", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30061", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30062", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30063", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30064", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30065", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30066", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30067", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30068", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30069", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30080", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30081", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30082", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30090", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30101", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30106", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30111", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30126", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30127", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30144", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30152", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30156", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30160", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30168", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30339", "FIPS": "13067", "County": "Cobb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31512", "FIPS": "13069", "County": "Coffee", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31519", "FIPS": "13069", "County": "Coffee", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31533", "FIPS": "13069", "County": "Coffee", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31534", "FIPS": "13069", "County": "Coffee", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31535", "FIPS": "13069", "County": "Coffee", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31554", "FIPS": "13069", "County": "Coffee", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31567", "FIPS": "13069", "County": "Coffee", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31722", "FIPS": "13071", "County": "Colquitt", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31744", "FIPS": "13071", "County": "Colquitt", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31747", "FIPS": "13071", "County": "Colquitt", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31753", "FIPS": "13071", "County": "Colquitt", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31756", "FIPS": "13071", "County": "Colquitt", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31768", "FIPS": "13071", "County": "Colquitt", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31771", "FIPS": "13071", "County": "Colquitt", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31776", "FIPS": "13071", "County": "Colquitt", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31788", "FIPS": "13071", "County": "Colquitt", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "30802", "FIPS": "13073", "County": "Columbia", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30809", "FIPS": "13073", "County": "Columbia", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30813", "FIPS": "13073", "County": "Columbia", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30814", "FIPS": "13073", "County": "Columbia", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30907", "FIPS": "13073", "County": "Columbia", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30917", "FIPS": "13073", "County": "Columbia", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "31620", "FIPS": "13075", "County": "Cook", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31627", "FIPS": "13075", "County": "Cook", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31637", "FIPS": "13075", "County": "Cook", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31647", "FIPS": "13075", "County": "Cook", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "30220", "FIPS": "13077", "County": "Coweta", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30229", "FIPS": "13077", "County": "Coweta", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30259", "FIPS": "13077", "County": "Coweta", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30263", "FIPS": "13077", "County": "Coweta", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30264", "FIPS": "13077", "County": "Coweta", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30265", "FIPS": "13077", "County": "Coweta", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30271", "FIPS": "13077", "County": "Coweta", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30275", "FIPS": "13077", "County": "Coweta", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30276", "FIPS": "13077", "County": "Coweta", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30277", "FIPS": "13077", "County": "Coweta", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30289", "FIPS": "13077", "County": "Coweta", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31050", "FIPS": "13079", "County": "Crawford", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31066", "FIPS": "13079", "County": "Crawford", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31078", "FIPS": "13079", "County": "Crawford", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31010", "FIPS": "13081", "County": "Crisp", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31015", "FIPS": "13081", "County": "Crisp", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31712", "FIPS": "13081", "County": "Crisp", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "30738", "FIPS": "13083", "County": "Dade", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "30752", "FIPS": "13083", "County": "Dade", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "30757", "FIPS": "13083", "County": "Dade", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "30534", "FIPS": "13085", "County": "Dawson", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "39815", "FIPS": "13087", "County": "Decatur", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "39817", "FIPS": "13087", "County": "Decatur", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "39818", "FIPS": "13087", "County": "Decatur", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "39819", "FIPS": "13087", "County": "Decatur", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "39825", "FIPS": "13087", "County": "Decatur", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "39834", "FIPS": "13087", "County": "Decatur", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "39852", "FIPS": "13087", "County": "Decatur", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "30002", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30021", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30030", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30031", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30032", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30033", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30034", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30035", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30036", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30037", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30038", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30058", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30072", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30073", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30074", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30079", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30083", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30084", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30085", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30086", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30087", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30088", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30294", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30317", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30319", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30322", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30329", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30333", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30338", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30340", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30341", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30345", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30346", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30356", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30359", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30360", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30362", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30366", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31119", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31120", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31141", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31145", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31146", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "39901", "FIPS": "13089", "County": "Dekalb", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31011", "FIPS": "13091", "County": "Dodge", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31012", "FIPS": "13091", "County": "Dodge", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31023", "FIPS": "13091", "County": "Dodge", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31077", "FIPS": "13091", "County": "Dodge", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31007", "FIPS": "13093", "County": "Dooly", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31051", "FIPS": "13093", "County": "Dooly", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31070", "FIPS": "13093", "County": "Dooly", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31091", "FIPS": "13093", "County": "Dooly", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31092", "FIPS": "13093", "County": "Dooly", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31701", "FIPS": "13095", "County": "Dougherty", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31702", "FIPS": "13095", "County": "Dougherty", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31703", "FIPS": "13095", "County": "Dougherty", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31704", "FIPS": "13095", "County": "Dougherty", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31705", "FIPS": "13095", "County": "Dougherty", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31706", "FIPS": "13095", "County": "Dougherty", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31707", "FIPS": "13095", "County": "Dougherty", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31708", "FIPS": "13095", "County": "Dougherty", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31721", "FIPS": "13095", "County": "Dougherty", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31782", "FIPS": "13095", "County": "Dougherty", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "30122", "FIPS": "13097", "County": "Douglas", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30133", "FIPS": "13097", "County": "Douglas", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30134", "FIPS": "13097", "County": "Douglas", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30135", "FIPS": "13097", "County": "Douglas", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30154", "FIPS": "13097", "County": "Douglas", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30187", "FIPS": "13097", "County": "Douglas", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "39823", "FIPS": "13099", "County": "Early", "State": "GA", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "39832", "FIPS": "13099", "County": "Early", "State": "GA", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "39841", "FIPS": "13099", "County": "Early", "State": "GA", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "39861", "FIPS": "13099", "County": "Early", "State": "GA", "DMA": {"Code": 606, "Name": "DOTHAN"}},
  {"Postal_Code": "31648", "FIPS": "13101", "County": "Echols", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31303", "FIPS": "13103", "County": "Effingham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31307", "FIPS": "13103", "County": "Effingham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31312", "FIPS": "13103", "County": "Effingham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31318", "FIPS": "13103", "County": "Effingham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31326", "FIPS": "13103", "County": "Effingham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31329", "FIPS": "13103", "County": "Effingham", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30624", "FIPS": "13105", "County": "Elbert", "State": "GA", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "30634", "FIPS": "13105", "County": "Elbert", "State": "GA", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "30635", "FIPS": "13105", "County": "Elbert", "State": "GA", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "30401", "FIPS": "13107", "County": "Emanuel", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30425", "FIPS": "13107", "County": "Emanuel", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30447", "FIPS": "13107", "County": "Emanuel", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30448", "FIPS": "13107", "County": "Emanuel", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30464", "FIPS": "13107", "County": "Emanuel", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30471", "FIPS": "13107", "County": "Emanuel", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "31002", "FIPS": "13107", "County": "Emanuel", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30414", "FIPS": "13109", "County": "Evans", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30417", "FIPS": "13109", "County": "Evans", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30423", "FIPS": "13109", "County": "Evans", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30429", "FIPS": "13109", "County": "Evans", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30513", "FIPS": "13111", "County": "Fannin", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30541", "FIPS": "13111", "County": "Fannin", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30555", "FIPS": "13111", "County": "Fannin", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30559", "FIPS": "13111", "County": "Fannin", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30560", "FIPS": "13111", "County": "Fannin", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30205", "FIPS": "13113", "County": "Fayette", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30214", "FIPS": "13113", "County": "Fayette", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30215", "FIPS": "13113", "County": "Fayette", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30269", "FIPS": "13113", "County": "Fayette", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30270", "FIPS": "13113", "County": "Fayette", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30290", "FIPS": "13113", "County": "Fayette", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30105", "FIPS": "13115", "County": "Floyd", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30124", "FIPS": "13115", "County": "Floyd", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30129", "FIPS": "13115", "County": "Floyd", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30147", "FIPS": "13115", "County": "Floyd", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30149", "FIPS": "13115", "County": "Floyd", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30161", "FIPS": "13115", "County": "Floyd", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30162", "FIPS": "13115", "County": "Floyd", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30163", "FIPS": "13115", "County": "Floyd", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30164", "FIPS": "13115", "County": "Floyd", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30165", "FIPS": "13115", "County": "Floyd", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30172", "FIPS": "13115", "County": "Floyd", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30173", "FIPS": "13115", "County": "Floyd", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30028", "FIPS": "13117", "County": "Forsyth", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30040", "FIPS": "13117", "County": "Forsyth", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30041", "FIPS": "13117", "County": "Forsyth", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30520", "FIPS": "13119", "County": "Franklin", "State": "GA", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "30521", "FIPS": "13119", "County": "Franklin", "State": "GA", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "30553", "FIPS": "13119", "County": "Franklin", "State": "GA", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "30639", "FIPS": "13119", "County": "Franklin", "State": "GA", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "30662", "FIPS": "13119", "County": "Franklin", "State": "GA", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "30004", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30005", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30009", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30022", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30023", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30075", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30076", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30077", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30213", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30268", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30272", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30291", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30301", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30302", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30303", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30304", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30305", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30306", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30307", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30308", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30309", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30310", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30311", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30312", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30313", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30314", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30315", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30316", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30318", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30320", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30321", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30324", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30325", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30326", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30327", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30328", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30330", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30331", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30332", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30334", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30336", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30337", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30342", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30343", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30344", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30347", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30348", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30349", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30350", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30353", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30354", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30355", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30357", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30358", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30361", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30363", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30364", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30368", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30369", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30370", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30371", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30374", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30375", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30376", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30377", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30378", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30379", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30380", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30384", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30385", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30386", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30387", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30388", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30389", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30390", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30392", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30394", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30396", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30398", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30399", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31106", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31107", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31126", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31131", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31136", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31139", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31150", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31156", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31191", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31192", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31193", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31195", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31196", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31197", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31198", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31199", "FIPS": "13121", "County": "Fulton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30522", "FIPS": "13123", "County": "Gilmer", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30536", "FIPS": "13123", "County": "Gilmer", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30539", "FIPS": "13123", "County": "Gilmer", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30540", "FIPS": "13123", "County": "Gilmer", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30810", "FIPS": "13125", "County": "Glascock", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30820", "FIPS": "13125", "County": "Glascock", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "31520", "FIPS": "13127", "County": "Glynn", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31521", "FIPS": "13127", "County": "Glynn", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31522", "FIPS": "13127", "County": "Glynn", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31523", "FIPS": "13127", "County": "Glynn", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31524", "FIPS": "13127", "County": "Glynn", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31525", "FIPS": "13127", "County": "Glynn", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31527", "FIPS": "13127", "County": "Glynn", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31561", "FIPS": "13127", "County": "Glynn", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "30139", "FIPS": "13129", "County": "Gordon", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30701", "FIPS": "13129", "County": "Gordon", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30703", "FIPS": "13129", "County": "Gordon", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30732", "FIPS": "13129", "County": "Gordon", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30733", "FIPS": "13129", "County": "Gordon", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30734", "FIPS": "13129", "County": "Gordon", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30735", "FIPS": "13129", "County": "Gordon", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30746", "FIPS": "13129", "County": "Gordon", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "39827", "FIPS": "13131", "County": "Grady", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "39828", "FIPS": "13131", "County": "Grady", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "39829", "FIPS": "13131", "County": "Grady", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "39897", "FIPS": "13131", "County": "Grady", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "30642", "FIPS": "13133", "County": "Greene", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30665", "FIPS": "13133", "County": "Greene", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30669", "FIPS": "13133", "County": "Greene", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30678", "FIPS": "13133", "County": "Greene", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30003", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30010", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30017", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30019", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30024", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30026", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30029", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30039", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30042", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30043", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30044", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30045", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30046", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30047", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30048", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30049", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30071", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30078", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30091", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30092", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30093", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30095", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30096", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30097", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30098", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30099", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30515", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30518", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30519", "FIPS": "13135", "County": "Gwinnett", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30510", "FIPS": "13137", "County": "Habersham", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30523", "FIPS": "13137", "County": "Habersham", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30531", "FIPS": "13137", "County": "Habersham", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30535", "FIPS": "13137", "County": "Habersham", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30544", "FIPS": "13137", "County": "Habersham", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30563", "FIPS": "13137", "County": "Habersham", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30580", "FIPS": "13137", "County": "Habersham", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30596", "FIPS": "13137", "County": "Habersham", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30501", "FIPS": "13139", "County": "Hall", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30502", "FIPS": "13139", "County": "Hall", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30503", "FIPS": "13139", "County": "Hall", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30504", "FIPS": "13139", "County": "Hall", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30506", "FIPS": "13139", "County": "Hall", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30507", "FIPS": "13139", "County": "Hall", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30527", "FIPS": "13139", "County": "Hall", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30542", "FIPS": "13139", "County": "Hall", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30543", "FIPS": "13139", "County": "Hall", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30554", "FIPS": "13139", "County": "Hall", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30564", "FIPS": "13139", "County": "Hall", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30566", "FIPS": "13139", "County": "Hall", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31087", "FIPS": "13141", "County": "Hancock", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "30110", "FIPS": "13143", "County": "Haralson", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30113", "FIPS": "13143", "County": "Haralson", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30140", "FIPS": "13143", "County": "Haralson", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30176", "FIPS": "13143", "County": "Haralson", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30182", "FIPS": "13143", "County": "Haralson", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31804", "FIPS": "13145", "County": "Harris", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31807", "FIPS": "13145", "County": "Harris", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31811", "FIPS": "13145", "County": "Harris", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31822", "FIPS": "13145", "County": "Harris", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31823", "FIPS": "13145", "County": "Harris", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31826", "FIPS": "13145", "County": "Harris", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31831", "FIPS": "13145", "County": "Harris", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "30516", "FIPS": "13147", "County": "Hart", "State": "GA", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "30643", "FIPS": "13147", "County": "Hart", "State": "GA", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "30217", "FIPS": "13149", "County": "Heard", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30219", "FIPS": "13149", "County": "Heard", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30228", "FIPS": "13151", "County": "Henry", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30248", "FIPS": "13151", "County": "Henry", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30252", "FIPS": "13151", "County": "Henry", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30253", "FIPS": "13151", "County": "Henry", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30281", "FIPS": "13151", "County": "Henry", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31005", "FIPS": "13153", "County": "Houston", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31013", "FIPS": "13153", "County": "Houston", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31025", "FIPS": "13153", "County": "Houston", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31028", "FIPS": "13153", "County": "Houston", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31047", "FIPS": "13153", "County": "Houston", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31069", "FIPS": "13153", "County": "Houston", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31088", "FIPS": "13153", "County": "Houston", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31093", "FIPS": "13153", "County": "Houston", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31095", "FIPS": "13153", "County": "Houston", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31098", "FIPS": "13153", "County": "Houston", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31099", "FIPS": "13153", "County": "Houston", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31760", "FIPS": "13155", "County": "Irwin", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31769", "FIPS": "13155", "County": "Irwin", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31774", "FIPS": "13155", "County": "Irwin", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31798", "FIPS": "13155", "County": "Irwin", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "30517", "FIPS": "13157", "County": "Jackson", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30529", "FIPS": "13157", "County": "Jackson", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30548", "FIPS": "13157", "County": "Jackson", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30549", "FIPS": "13157", "County": "Jackson", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30565", "FIPS": "13157", "County": "Jackson", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30567", "FIPS": "13157", "County": "Jackson", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30575", "FIPS": "13157", "County": "Jackson", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30599", "FIPS": "13157", "County": "Jackson", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30055", "FIPS": "13159", "County": "Jasper", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31038", "FIPS": "13159", "County": "Jasper", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31064", "FIPS": "13159", "County": "Jasper", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31085", "FIPS": "13159", "County": "Jasper", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31532", "FIPS": "13161", "County": "Jeff Davis", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31539", "FIPS": "13161", "County": "Jeff Davis", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30413", "FIPS": "13163", "County": "Jefferson", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30434", "FIPS": "13163", "County": "Jefferson", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30477", "FIPS": "13163", "County": "Jefferson", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30803", "FIPS": "13163", "County": "Jefferson", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30818", "FIPS": "13163", "County": "Jefferson", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30823", "FIPS": "13163", "County": "Jefferson", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30833", "FIPS": "13163", "County": "Jefferson", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30442", "FIPS": "13165", "County": "Jenkins", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30822", "FIPS": "13165", "County": "Jenkins", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "31049", "FIPS": "13167", "County": "Johnson", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31096", "FIPS": "13167", "County": "Johnson", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31032", "FIPS": "13169", "County": "Jones", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31033", "FIPS": "13169", "County": "Jones", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "30204", "FIPS": "13171", "County": "Lamar", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30257", "FIPS": "13171", "County": "Lamar", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31635", "FIPS": "13173", "County": "Lanier", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31649", "FIPS": "13173", "County": "Lanier", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "30454", "FIPS": "13175", "County": "Laurens", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31009", "FIPS": "13175", "County": "Laurens", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31019", "FIPS": "13175", "County": "Laurens", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31021", "FIPS": "13175", "County": "Laurens", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31022", "FIPS": "13175", "County": "Laurens", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31027", "FIPS": "13175", "County": "Laurens", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31040", "FIPS": "13175", "County": "Laurens", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31065", "FIPS": "13175", "County": "Laurens", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31075", "FIPS": "13175", "County": "Laurens", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31763", "FIPS": "13177", "County": "Lee", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31787", "FIPS": "13177", "County": "Lee", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31301", "FIPS": "13179", "County": "Liberty", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31309", "FIPS": "13179", "County": "Liberty", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31310", "FIPS": "13179", "County": "Liberty", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31313", "FIPS": "13179", "County": "Liberty", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31314", "FIPS": "13179", "County": "Liberty", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31315", "FIPS": "13179", "County": "Liberty", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31320", "FIPS": "13179", "County": "Liberty", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31323", "FIPS": "13179", "County": "Liberty", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31333", "FIPS": "13179", "County": "Liberty", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30817", "FIPS": "13181", "County": "Lincoln", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "31316", "FIPS": "13183", "County": "Long", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31601", "FIPS": "13185", "County": "Lowndes", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31602", "FIPS": "13185", "County": "Lowndes", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31603", "FIPS": "13185", "County": "Lowndes", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31604", "FIPS": "13185", "County": "Lowndes", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31605", "FIPS": "13185", "County": "Lowndes", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31606", "FIPS": "13185", "County": "Lowndes", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31632", "FIPS": "13185", "County": "Lowndes", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31636", "FIPS": "13185", "County": "Lowndes", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31641", "FIPS": "13185", "County": "Lowndes", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31698", "FIPS": "13185", "County": "Lowndes", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31699", "FIPS": "13185", "County": "Lowndes", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "30533", "FIPS": "13187", "County": "Lumpkin", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30597", "FIPS": "13187", "County": "Lumpkin", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30806", "FIPS": "13189", "County": "McDuffie", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30808", "FIPS": "13189", "County": "McDuffie", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30824", "FIPS": "13189", "County": "McDuffie", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "31304", "FIPS": "13191", "County": "McIntosh", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31305", "FIPS": "13191", "County": "McIntosh", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31319", "FIPS": "13191", "County": "McIntosh", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31327", "FIPS": "13191", "County": "McIntosh", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31331", "FIPS": "13191", "County": "McIntosh", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31041", "FIPS": "13193", "County": "Macon", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31057", "FIPS": "13193", "County": "Macon", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31063", "FIPS": "13193", "County": "Macon", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31068", "FIPS": "13193", "County": "Macon", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "30627", "FIPS": "13195", "County": "Madison", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30628", "FIPS": "13195", "County": "Madison", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30629", "FIPS": "13195", "County": "Madison", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30633", "FIPS": "13195", "County": "Madison", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30646", "FIPS": "13195", "County": "Madison", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30647", "FIPS": "13195", "County": "Madison", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31058", "FIPS": "13197", "County": "Marion", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31803", "FIPS": "13197", "County": "Marion", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "30218", "FIPS": "13199", "County": "Meriwether", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30222", "FIPS": "13199", "County": "Meriwether", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30251", "FIPS": "13199", "County": "Meriwether", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30293", "FIPS": "13199", "County": "Meriwether", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31816", "FIPS": "13199", "County": "Meriwether", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31830", "FIPS": "13199", "County": "Meriwether", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "39837", "FIPS": "13201", "County": "Miller", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31716", "FIPS": "13205", "County": "Mitchell", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31730", "FIPS": "13205", "County": "Mitchell", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31739", "FIPS": "13205", "County": "Mitchell", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31779", "FIPS": "13205", "County": "Mitchell", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31784", "FIPS": "13205", "County": "Mitchell", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31004", "FIPS": "13207", "County": "Monroe", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31016", "FIPS": "13207", "County": "Monroe", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31029", "FIPS": "13207", "County": "Monroe", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31046", "FIPS": "13207", "County": "Monroe", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31086", "FIPS": "13207", "County": "Monroe", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "30410", "FIPS": "13209", "County": "Montgomery", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30412", "FIPS": "13209", "County": "Montgomery", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30445", "FIPS": "13209", "County": "Montgomery", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30470", "FIPS": "13209", "County": "Montgomery", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30473", "FIPS": "13209", "County": "Montgomery", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30623", "FIPS": "13211", "County": "Morgan", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30625", "FIPS": "13211", "County": "Morgan", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30645", "FIPS": "13211", "County": "Morgan", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30650", "FIPS": "13211", "County": "Morgan", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30663", "FIPS": "13211", "County": "Morgan", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30705", "FIPS": "13213", "County": "Murray", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "30708", "FIPS": "13213", "County": "Murray", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "30711", "FIPS": "13213", "County": "Murray", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "30724", "FIPS": "13213", "County": "Murray", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "30751", "FIPS": "13213", "County": "Murray", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "31808", "FIPS": "13215", "County": "Muscogee", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31820", "FIPS": "13215", "County": "Muscogee", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31829", "FIPS": "13215", "County": "Muscogee", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31901", "FIPS": "13215", "County": "Muscogee", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31902", "FIPS": "13215", "County": "Muscogee", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31903", "FIPS": "13215", "County": "Muscogee", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31904", "FIPS": "13215", "County": "Muscogee", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31905", "FIPS": "13215", "County": "Muscogee", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31906", "FIPS": "13215", "County": "Muscogee", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31907", "FIPS": "13215", "County": "Muscogee", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31908", "FIPS": "13215", "County": "Muscogee", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31909", "FIPS": "13215", "County": "Muscogee", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31914", "FIPS": "13215", "County": "Muscogee", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31917", "FIPS": "13215", "County": "Muscogee", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31993", "FIPS": "13215", "County": "Muscogee", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31995", "FIPS": "13215", "County": "Muscogee", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31997", "FIPS": "13215", "County": "Muscogee", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31998", "FIPS": "13215", "County": "Muscogee", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31999", "FIPS": "13215", "County": "Muscogee", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "30014", "FIPS": "13217", "County": "Newton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30015", "FIPS": "13217", "County": "Newton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30016", "FIPS": "13217", "County": "Newton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30054", "FIPS": "13217", "County": "Newton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30056", "FIPS": "13217", "County": "Newton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30070", "FIPS": "13217", "County": "Newton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30621", "FIPS": "13219", "County": "Oconee", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30622", "FIPS": "13219", "County": "Oconee", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30638", "FIPS": "13219", "County": "Oconee", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30677", "FIPS": "13219", "County": "Oconee", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30619", "FIPS": "13221", "County": "Oglethorpe", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30630", "FIPS": "13221", "County": "Oglethorpe", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30648", "FIPS": "13221", "County": "Oglethorpe", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30667", "FIPS": "13221", "County": "Oglethorpe", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30671", "FIPS": "13221", "County": "Oglethorpe", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30132", "FIPS": "13223", "County": "Paulding", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30141", "FIPS": "13223", "County": "Paulding", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30157", "FIPS": "13223", "County": "Paulding", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31008", "FIPS": "13225", "County": "Peach", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31030", "FIPS": "13225", "County": "Peach", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "30143", "FIPS": "13227", "County": "Pickens", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30148", "FIPS": "13227", "County": "Pickens", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30175", "FIPS": "13227", "County": "Pickens", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30177", "FIPS": "13227", "County": "Pickens", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31516", "FIPS": "13229", "County": "Pierce", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31518", "FIPS": "13229", "County": "Pierce", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31551", "FIPS": "13229", "County": "Pierce", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31556", "FIPS": "13229", "County": "Pierce", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31557", "FIPS": "13229", "County": "Pierce", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "30206", "FIPS": "13231", "County": "Pike", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30256", "FIPS": "13231", "County": "Pike", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30258", "FIPS": "13231", "County": "Pike", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30292", "FIPS": "13231", "County": "Pike", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30295", "FIPS": "13231", "County": "Pike", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30104", "FIPS": "13233", "County": "Polk", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30125", "FIPS": "13233", "County": "Polk", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30138", "FIPS": "13233", "County": "Polk", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30153", "FIPS": "13233", "County": "Polk", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31036", "FIPS": "13235", "County": "Pulaski", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31024", "FIPS": "13237", "County": "Putnam", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31026", "FIPS": "13237", "County": "Putnam", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "39854", "FIPS": "13239", "County": "Quitman", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "39867", "FIPS": "13239", "County": "Quitman", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "30525", "FIPS": "13241", "County": "Rabun", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30537", "FIPS": "13241", "County": "Rabun", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30552", "FIPS": "13241", "County": "Rabun", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30562", "FIPS": "13241", "County": "Rabun", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30568", "FIPS": "13241", "County": "Rabun", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30573", "FIPS": "13241", "County": "Rabun", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30576", "FIPS": "13241", "County": "Rabun", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30581", "FIPS": "13241", "County": "Rabun", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "39836", "FIPS": "13243", "County": "Randolph", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "39840", "FIPS": "13243", "County": "Randolph", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "39886", "FIPS": "13243", "County": "Randolph", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "30805", "FIPS": "13245", "County": "Richmond", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30812", "FIPS": "13245", "County": "Richmond", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30815", "FIPS": "13245", "County": "Richmond", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30901", "FIPS": "13245", "County": "Richmond", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30903", "FIPS": "13245", "County": "Richmond", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30904", "FIPS": "13245", "County": "Richmond", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30905", "FIPS": "13245", "County": "Richmond", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30906", "FIPS": "13245", "County": "Richmond", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30909", "FIPS": "13245", "County": "Richmond", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30911", "FIPS": "13245", "County": "Richmond", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30912", "FIPS": "13245", "County": "Richmond", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30913", "FIPS": "13245", "County": "Richmond", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30914", "FIPS": "13245", "County": "Richmond", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30916", "FIPS": "13245", "County": "Richmond", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30919", "FIPS": "13245", "County": "Richmond", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30999", "FIPS": "13245", "County": "Richmond", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30012", "FIPS": "13247", "County": "Rockdale", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30013", "FIPS": "13247", "County": "Rockdale", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30094", "FIPS": "13247", "County": "Rockdale", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31806", "FIPS": "13249", "County": "Schley", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "30424", "FIPS": "13251", "County": "Screven", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30446", "FIPS": "13251", "County": "Screven", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30449", "FIPS": "13251", "County": "Screven", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30455", "FIPS": "13251", "County": "Screven", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30467", "FIPS": "13251", "County": "Screven", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "39845", "FIPS": "13253", "County": "Seminole", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "39859", "FIPS": "13253", "County": "Seminole", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "30212", "FIPS": "13255", "County": "Spalding", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30223", "FIPS": "13255", "County": "Spalding", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30224", "FIPS": "13255", "County": "Spalding", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30266", "FIPS": "13255", "County": "Spalding", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30284", "FIPS": "13255", "County": "Spalding", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30538", "FIPS": "13257", "County": "Stephens", "State": "GA", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "30557", "FIPS": "13257", "County": "Stephens", "State": "GA", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "30577", "FIPS": "13257", "County": "Stephens", "State": "GA", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "30598", "FIPS": "13257", "County": "Stephens", "State": "GA", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "31814", "FIPS": "13259", "County": "Stewart", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31815", "FIPS": "13259", "County": "Stewart", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31821", "FIPS": "13259", "County": "Stewart", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31825", "FIPS": "13259", "County": "Stewart", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31709", "FIPS": "13261", "County": "Sumter", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31710", "FIPS": "13261", "County": "Sumter", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31711", "FIPS": "13261", "County": "Sumter", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31719", "FIPS": "13261", "County": "Sumter", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31735", "FIPS": "13261", "County": "Sumter", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31743", "FIPS": "13261", "County": "Sumter", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31764", "FIPS": "13261", "County": "Sumter", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31780", "FIPS": "13261", "County": "Sumter", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31801", "FIPS": "13263", "County": "Talbot", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31810", "FIPS": "13263", "County": "Talbot", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31812", "FIPS": "13263", "County": "Talbot", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31827", "FIPS": "13263", "County": "Talbot", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31836", "FIPS": "13263", "County": "Talbot", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "30631", "FIPS": "13265", "County": "Taliaferro", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30664", "FIPS": "13265", "County": "Taliaferro", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30420", "FIPS": "13267", "County": "Tattnall", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30421", "FIPS": "13267", "County": "Tattnall", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30427", "FIPS": "13267", "County": "Tattnall", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30438", "FIPS": "13267", "County": "Tattnall", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30453", "FIPS": "13267", "County": "Tattnall", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30499", "FIPS": "13267", "County": "Tattnall", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31006", "FIPS": "13269", "County": "Taylor", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31039", "FIPS": "13269", "County": "Taylor", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31076", "FIPS": "13269", "County": "Taylor", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31081", "FIPS": "13269", "County": "Taylor", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31037", "FIPS": "13271", "County": "Telfair", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31055", "FIPS": "13271", "County": "Telfair", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31060", "FIPS": "13271", "County": "Telfair", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31083", "FIPS": "13271", "County": "Telfair", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31544", "FIPS": "13271", "County": "Telfair", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31549", "FIPS": "13271", "County": "Telfair", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "39826", "FIPS": "13273", "County": "Terrell", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "39842", "FIPS": "13273", "County": "Terrell", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "39877", "FIPS": "13273", "County": "Terrell", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "39885", "FIPS": "13273", "County": "Terrell", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31626", "FIPS": "13275", "County": "Thomas", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31738", "FIPS": "13275", "County": "Thomas", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31757", "FIPS": "13275", "County": "Thomas", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31758", "FIPS": "13275", "County": "Thomas", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31765", "FIPS": "13275", "County": "Thomas", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31773", "FIPS": "13275", "County": "Thomas", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31778", "FIPS": "13275", "County": "Thomas", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31792", "FIPS": "13275", "County": "Thomas", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31799", "FIPS": "13275", "County": "Thomas", "State": "GA", "DMA": {"Code": 530, "Name": "TALLAHASSEE-THOMASVILLE"}},
  {"Postal_Code": "31727", "FIPS": "13277", "County": "Tift", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31733", "FIPS": "13277", "County": "Tift", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31775", "FIPS": "13277", "County": "Tift", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31793", "FIPS": "13277", "County": "Tift", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31794", "FIPS": "13277", "County": "Tift", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31795", "FIPS": "13277", "County": "Tift", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "30436", "FIPS": "13279", "County": "Toombs", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30474", "FIPS": "13279", "County": "Toombs", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30475", "FIPS": "13279", "County": "Toombs", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "30546", "FIPS": "13281", "County": "Towns", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30582", "FIPS": "13281", "County": "Towns", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30457", "FIPS": "13283", "County": "Treutlen", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "30230", "FIPS": "13285", "County": "Troup", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30240", "FIPS": "13285", "County": "Troup", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30241", "FIPS": "13285", "County": "Troup", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30261", "FIPS": "13285", "County": "Troup", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31833", "FIPS": "13285", "County": "Troup", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31714", "FIPS": "13287", "County": "Turner", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31783", "FIPS": "13287", "County": "Turner", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31790", "FIPS": "13287", "County": "Turner", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31017", "FIPS": "13289", "County": "Twiggs", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31020", "FIPS": "13289", "County": "Twiggs", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31044", "FIPS": "13289", "County": "Twiggs", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "30512", "FIPS": "13291", "County": "Union", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30514", "FIPS": "13291", "County": "Union", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30572", "FIPS": "13291", "County": "Union", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30285", "FIPS": "13293", "County": "Upson", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30286", "FIPS": "13293", "County": "Upson", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31097", "FIPS": "13293", "County": "Upson", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30707", "FIPS": "13295", "County": "Walker", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "30725", "FIPS": "13295", "County": "Walker", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "30728", "FIPS": "13295", "County": "Walker", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "30739", "FIPS": "13295", "County": "Walker", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "30741", "FIPS": "13295", "County": "Walker", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "30750", "FIPS": "13295", "County": "Walker", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "30018", "FIPS": "13297", "County": "Walton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30025", "FIPS": "13297", "County": "Walton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30052", "FIPS": "13297", "County": "Walton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30641", "FIPS": "13297", "County": "Walton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30655", "FIPS": "13297", "County": "Walton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30656", "FIPS": "13297", "County": "Walton", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "31501", "FIPS": "13299", "County": "Ware", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31502", "FIPS": "13299", "County": "Ware", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31503", "FIPS": "13299", "County": "Ware", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31550", "FIPS": "13299", "County": "Ware", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31552", "FIPS": "13299", "County": "Ware", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "31564", "FIPS": "13299", "County": "Ware", "State": "GA", "DMA": {"Code": 561, "Name": "JACKSONVILLE"}},
  {"Postal_Code": "30807", "FIPS": "13301", "County": "Warren", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30819", "FIPS": "13301", "County": "Warren", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30821", "FIPS": "13301", "County": "Warren", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30828", "FIPS": "13301", "County": "Warren", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "31045", "FIPS": "13301", "County": "Warren", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "31018", "FIPS": "13303", "County": "Washington", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31035", "FIPS": "13303", "County": "Washington", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31067", "FIPS": "13303", "County": "Washington", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31082", "FIPS": "13303", "County": "Washington", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31089", "FIPS": "13303", "County": "Washington", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31094", "FIPS": "13303", "County": "Washington", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31545", "FIPS": "13305", "County": "Wayne", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31546", "FIPS": "13305", "County": "Wayne", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31555", "FIPS": "13305", "County": "Wayne", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31560", "FIPS": "13305", "County": "Wayne", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31598", "FIPS": "13305", "County": "Wayne", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31599", "FIPS": "13305", "County": "Wayne", "State": "GA", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "31824", "FIPS": "13307", "County": "Webster", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "31832", "FIPS": "13307", "County": "Webster", "State": "GA", "DMA": {"Code": 522, "Name": "COLUMBUS, GA"}},
  {"Postal_Code": "30411", "FIPS": "13309", "County": "Wheeler", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "30428", "FIPS": "13309", "County": "Wheeler", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "30528", "FIPS": "13311", "County": "White", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30545", "FIPS": "13311", "County": "White", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30571", "FIPS": "13311", "County": "White", "State": "GA", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "30710", "FIPS": "13313", "County": "Whitfield", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "30719", "FIPS": "13313", "County": "Whitfield", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "30720", "FIPS": "13313", "County": "Whitfield", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "30721", "FIPS": "13313", "County": "Whitfield", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "30722", "FIPS": "13313", "County": "Whitfield", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "30740", "FIPS": "13313", "County": "Whitfield", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "30755", "FIPS": "13313", "County": "Whitfield", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "30756", "FIPS": "13313", "County": "Whitfield", "State": "GA", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "31001", "FIPS": "13315", "County": "Wilcox", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31071", "FIPS": "13315", "County": "Wilcox", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31072", "FIPS": "13315", "County": "Wilcox", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31079", "FIPS": "13315", "County": "Wilcox", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31084", "FIPS": "13315", "County": "Wilcox", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "30660", "FIPS": "13317", "County": "Wilkes", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30668", "FIPS": "13317", "County": "Wilkes", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "30673", "FIPS": "13317", "County": "Wilkes", "State": "GA", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "31003", "FIPS": "13319", "County": "Wilkinson", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31031", "FIPS": "13319", "County": "Wilkinson", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31042", "FIPS": "13319", "County": "Wilkinson", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31054", "FIPS": "13319", "County": "Wilkinson", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31090", "FIPS": "13319", "County": "Wilkinson", "State": "GA", "DMA": {"Code": 503, "Name": "MACON"}},
  {"Postal_Code": "31772", "FIPS": "13321", "County": "Worth", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31781", "FIPS": "13321", "County": "Worth", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31789", "FIPS": "13321", "County": "Worth", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31791", "FIPS": "13321", "County": "Worth", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "31796", "FIPS": "13321", "County": "Worth", "State": "GA", "DMA": {"Code": 525, "Name": "ALBANY, GA"}},
  {"Postal_Code": "96704", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96710", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96718", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96719", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96720", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96721", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96725", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96726", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96727", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96728", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96737", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96738", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96739", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96740", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96743", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96745", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96749", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96750", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96755", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96760", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96764", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96771", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96772", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96773", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96774", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96776", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96777", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96778", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96780", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96781", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96783", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96785", "FIPS": "15001", "County": "Hawaii", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96701", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96706", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96707", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96709", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96712", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96717", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96730", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96731", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96734", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96744", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96759", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96762", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96782", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96786", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96789", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96791", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96792", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96795", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96797", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96801", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96802", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96803", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96804", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96805", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96806", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96807", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96808", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96809", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96810", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96811", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96812", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96813", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96814", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96815", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96816", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96817", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96818", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96819", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96820", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96821", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96822", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96823", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96824", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96825", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96826", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96827", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96828", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96830", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96835", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96836", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96837", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96838", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96839", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96840", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96841", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96843", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96844", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96846", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96847", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96848", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96849", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96850", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96853", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96854", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96857", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96858", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96859", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96860", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96861", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96863", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96898", "FIPS": "15003", "County": "Honolulu", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96742", "FIPS": "15005", "County": "Kalawao", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96703", "FIPS": "15007", "County": "Kauai", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96705", "FIPS": "15007", "County": "Kauai", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96714", "FIPS": "15007", "County": "Kauai", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96715", "FIPS": "15007", "County": "Kauai", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96716", "FIPS": "15007", "County": "Kauai", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96722", "FIPS": "15007", "County": "Kauai", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96741", "FIPS": "15007", "County": "Kauai", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96746", "FIPS": "15007", "County": "Kauai", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96747", "FIPS": "15007", "County": "Kauai", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96751", "FIPS": "15007", "County": "Kauai", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96752", "FIPS": "15007", "County": "Kauai", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96754", "FIPS": "15007", "County": "Kauai", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96756", "FIPS": "15007", "County": "Kauai", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96765", "FIPS": "15007", "County": "Kauai", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96766", "FIPS": "15007", "County": "Kauai", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96769", "FIPS": "15007", "County": "Kauai", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96796", "FIPS": "15007", "County": "Kauai", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96708", "FIPS": "15009", "County": "Maui", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96713", "FIPS": "15009", "County": "Maui", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96729", "FIPS": "15009", "County": "Maui", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96732", "FIPS": "15009", "County": "Maui", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96733", "FIPS": "15009", "County": "Maui", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96748", "FIPS": "15009", "County": "Maui", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96753", "FIPS": "15009", "County": "Maui", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96757", "FIPS": "15009", "County": "Maui", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96761", "FIPS": "15009", "County": "Maui", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96763", "FIPS": "15009", "County": "Maui", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96767", "FIPS": "15009", "County": "Maui", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96768", "FIPS": "15009", "County": "Maui", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96770", "FIPS": "15009", "County": "Maui", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96779", "FIPS": "15009", "County": "Maui", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96784", "FIPS": "15009", "County": "Maui", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96788", "FIPS": "15009", "County": "Maui", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96790", "FIPS": "15009", "County": "Maui", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "96793", "FIPS": "15009", "County": "Maui", "State": "HI", "DMA": {"Code": 744, "Name": "HONOLULU"}},
  {"Postal_Code": "83616", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83634", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83642", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83646", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83669", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83680", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83701", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83702", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83703", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83704", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83705", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83706", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83707", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83708", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83709", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83711", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83712", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83713", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83714", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83715", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83716", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83717", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83719", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83720", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83721", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83722", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83724", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83725", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83726", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83727", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83728", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83729", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83730", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83731", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83732", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83733", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83735", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83756", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83757", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83799", "FIPS": "16001", "County": "Ada", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83612", "FIPS": "16003", "County": "Adams", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83632", "FIPS": "16003", "County": "Adams", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83643", "FIPS": "16003", "County": "Adams", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83654", "FIPS": "16003", "County": "Adams", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83201", "FIPS": "16005", "County": "Bannock", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83202", "FIPS": "16005", "County": "Bannock", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83204", "FIPS": "16005", "County": "Bannock", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83205", "FIPS": "16005", "County": "Bannock", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83206", "FIPS": "16005", "County": "Bannock", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83209", "FIPS": "16005", "County": "Bannock", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83214", "FIPS": "16005", "County": "Bannock", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83234", "FIPS": "16005", "County": "Bannock", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83245", "FIPS": "16005", "County": "Bannock", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83246", "FIPS": "16005", "County": "Bannock", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83250", "FIPS": "16005", "County": "Bannock", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83281", "FIPS": "16005", "County": "Bannock", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83220", "FIPS": "16007", "County": "Bear Lake", "State": "ID", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83223", "FIPS": "16007", "County": "Bear Lake", "State": "ID", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83233", "FIPS": "16007", "County": "Bear Lake", "State": "ID", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83238", "FIPS": "16007", "County": "Bear Lake", "State": "ID", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83239", "FIPS": "16007", "County": "Bear Lake", "State": "ID", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83254", "FIPS": "16007", "County": "Bear Lake", "State": "ID", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83261", "FIPS": "16007", "County": "Bear Lake", "State": "ID", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83272", "FIPS": "16007", "County": "Bear Lake", "State": "ID", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83287", "FIPS": "16007", "County": "Bear Lake", "State": "ID", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83824", "FIPS": "16009", "County": "Benewah", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83830", "FIPS": "16009", "County": "Benewah", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83851", "FIPS": "16009", "County": "Benewah", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83861", "FIPS": "16009", "County": "Benewah", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83866", "FIPS": "16009", "County": "Benewah", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83870", "FIPS": "16009", "County": "Benewah", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83203", "FIPS": "16011", "County": "Bingham", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83210", "FIPS": "16011", "County": "Bingham", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83215", "FIPS": "16011", "County": "Bingham", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83218", "FIPS": "16011", "County": "Bingham", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83221", "FIPS": "16011", "County": "Bingham", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83236", "FIPS": "16011", "County": "Bingham", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83256", "FIPS": "16011", "County": "Bingham", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83262", "FIPS": "16011", "County": "Bingham", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83274", "FIPS": "16011", "County": "Bingham", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83277", "FIPS": "16011", "County": "Bingham", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83313", "FIPS": "16013", "County": "Blaine", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83320", "FIPS": "16013", "County": "Blaine", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83333", "FIPS": "16013", "County": "Blaine", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83340", "FIPS": "16013", "County": "Blaine", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83348", "FIPS": "16013", "County": "Blaine", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83353", "FIPS": "16013", "County": "Blaine", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83354", "FIPS": "16013", "County": "Blaine", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83602", "FIPS": "16015", "County": "Boise", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83622", "FIPS": "16015", "County": "Boise", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83629", "FIPS": "16015", "County": "Boise", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83631", "FIPS": "16015", "County": "Boise", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83637", "FIPS": "16015", "County": "Boise", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83666", "FIPS": "16015", "County": "Boise", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83804", "FIPS": "16017", "County": "Bonner", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83809", "FIPS": "16017", "County": "Bonner", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83811", "FIPS": "16017", "County": "Bonner", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83813", "FIPS": "16017", "County": "Bonner", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83821", "FIPS": "16017", "County": "Bonner", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83822", "FIPS": "16017", "County": "Bonner", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83825", "FIPS": "16017", "County": "Bonner", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83836", "FIPS": "16017", "County": "Bonner", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83840", "FIPS": "16017", "County": "Bonner", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83841", "FIPS": "16017", "County": "Bonner", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83848", "FIPS": "16017", "County": "Bonner", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83852", "FIPS": "16017", "County": "Bonner", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83856", "FIPS": "16017", "County": "Bonner", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83860", "FIPS": "16017", "County": "Bonner", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83864", "FIPS": "16017", "County": "Bonner", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83865", "FIPS": "16017", "County": "Bonner", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83401", "FIPS": "16019", "County": "Bonneville", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83402", "FIPS": "16019", "County": "Bonneville", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83403", "FIPS": "16019", "County": "Bonneville", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83404", "FIPS": "16019", "County": "Bonneville", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83405", "FIPS": "16019", "County": "Bonneville", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83406", "FIPS": "16019", "County": "Bonneville", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83415", "FIPS": "16019", "County": "Bonneville", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83427", "FIPS": "16019", "County": "Bonneville", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83428", "FIPS": "16019", "County": "Bonneville", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83449", "FIPS": "16019", "County": "Bonneville", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83454", "FIPS": "16019", "County": "Bonneville", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83805", "FIPS": "16021", "County": "Boundary", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83826", "FIPS": "16021", "County": "Boundary", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83845", "FIPS": "16021", "County": "Boundary", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83847", "FIPS": "16021", "County": "Boundary", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83853", "FIPS": "16021", "County": "Boundary", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83213", "FIPS": "16023", "County": "Butte", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83244", "FIPS": "16023", "County": "Butte", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83255", "FIPS": "16023", "County": "Butte", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83322", "FIPS": "16025", "County": "Camas", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83327", "FIPS": "16025", "County": "Camas", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83337", "FIPS": "16025", "County": "Camas", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83605", "FIPS": "16027", "County": "Canyon", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83606", "FIPS": "16027", "County": "Canyon", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83607", "FIPS": "16027", "County": "Canyon", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83626", "FIPS": "16027", "County": "Canyon", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83630", "FIPS": "16027", "County": "Canyon", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83641", "FIPS": "16027", "County": "Canyon", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83644", "FIPS": "16027", "County": "Canyon", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83651", "FIPS": "16027", "County": "Canyon", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83652", "FIPS": "16027", "County": "Canyon", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83653", "FIPS": "16027", "County": "Canyon", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83656", "FIPS": "16027", "County": "Canyon", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83660", "FIPS": "16027", "County": "Canyon", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83676", "FIPS": "16027", "County": "Canyon", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83686", "FIPS": "16027", "County": "Canyon", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83687", "FIPS": "16027", "County": "Canyon", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83217", "FIPS": "16029", "County": "Caribou", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83230", "FIPS": "16029", "County": "Caribou", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83241", "FIPS": "16029", "County": "Caribou", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83276", "FIPS": "16029", "County": "Caribou", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83285", "FIPS": "16029", "County": "Caribou", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83311", "FIPS": "16031", "County": "Cassia", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83312", "FIPS": "16031", "County": "Cassia", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83318", "FIPS": "16031", "County": "Cassia", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83323", "FIPS": "16031", "County": "Cassia", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83342", "FIPS": "16031", "County": "Cassia", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83346", "FIPS": "16031", "County": "Cassia", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83423", "FIPS": "16033", "County": "Clark", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83446", "FIPS": "16033", "County": "Clark", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83520", "FIPS": "16035", "County": "Clearwater", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83541", "FIPS": "16035", "County": "Clearwater", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83544", "FIPS": "16035", "County": "Clearwater", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83546", "FIPS": "16035", "County": "Clearwater", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83553", "FIPS": "16035", "County": "Clearwater", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83827", "FIPS": "16035", "County": "Clearwater", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83226", "FIPS": "16037", "County": "Custer", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83227", "FIPS": "16037", "County": "Custer", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83235", "FIPS": "16037", "County": "Custer", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83251", "FIPS": "16037", "County": "Custer", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83278", "FIPS": "16037", "County": "Custer", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83601", "FIPS": "16039", "County": "Elmore", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83623", "FIPS": "16039", "County": "Elmore", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83627", "FIPS": "16039", "County": "Elmore", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83633", "FIPS": "16039", "County": "Elmore", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83647", "FIPS": "16039", "County": "Elmore", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83648", "FIPS": "16039", "County": "Elmore", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83228", "FIPS": "16041", "County": "Franklin", "State": "ID", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83232", "FIPS": "16041", "County": "Franklin", "State": "ID", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83237", "FIPS": "16041", "County": "Franklin", "State": "ID", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83263", "FIPS": "16041", "County": "Franklin", "State": "ID", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83283", "FIPS": "16041", "County": "Franklin", "State": "ID", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83286", "FIPS": "16041", "County": "Franklin", "State": "ID", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83420", "FIPS": "16043", "County": "Fremont", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83421", "FIPS": "16043", "County": "Fremont", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83429", "FIPS": "16043", "County": "Fremont", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83433", "FIPS": "16043", "County": "Fremont", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83436", "FIPS": "16043", "County": "Fremont", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83438", "FIPS": "16043", "County": "Fremont", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83445", "FIPS": "16043", "County": "Fremont", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83451", "FIPS": "16043", "County": "Fremont", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83617", "FIPS": "16045", "County": "Gem", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83636", "FIPS": "16045", "County": "Gem", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83657", "FIPS": "16045", "County": "Gem", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83670", "FIPS": "16045", "County": "Gem", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83314", "FIPS": "16047", "County": "Gooding", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83330", "FIPS": "16047", "County": "Gooding", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83332", "FIPS": "16047", "County": "Gooding", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83355", "FIPS": "16047", "County": "Gooding", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83522", "FIPS": "16049", "County": "Idaho", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83525", "FIPS": "16049", "County": "Idaho", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83526", "FIPS": "16049", "County": "Idaho", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83530", "FIPS": "16049", "County": "Idaho", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83531", "FIPS": "16049", "County": "Idaho", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83533", "FIPS": "16049", "County": "Idaho", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83539", "FIPS": "16049", "County": "Idaho", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83542", "FIPS": "16049", "County": "Idaho", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83547", "FIPS": "16049", "County": "Idaho", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83549", "FIPS": "16049", "County": "Idaho", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83552", "FIPS": "16049", "County": "Idaho", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83554", "FIPS": "16049", "County": "Idaho", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83671", "FIPS": "16049", "County": "Idaho", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83425", "FIPS": "16051", "County": "Jefferson", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83431", "FIPS": "16051", "County": "Jefferson", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83434", "FIPS": "16051", "County": "Jefferson", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83435", "FIPS": "16051", "County": "Jefferson", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83442", "FIPS": "16051", "County": "Jefferson", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83443", "FIPS": "16051", "County": "Jefferson", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83444", "FIPS": "16051", "County": "Jefferson", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83450", "FIPS": "16051", "County": "Jefferson", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83325", "FIPS": "16053", "County": "Jerome", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83335", "FIPS": "16053", "County": "Jerome", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83338", "FIPS": "16053", "County": "Jerome", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83801", "FIPS": "16055", "County": "Kootenai", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83803", "FIPS": "16055", "County": "Kootenai", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83810", "FIPS": "16055", "County": "Kootenai", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83814", "FIPS": "16055", "County": "Kootenai", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83815", "FIPS": "16055", "County": "Kootenai", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83816", "FIPS": "16055", "County": "Kootenai", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83833", "FIPS": "16055", "County": "Kootenai", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83835", "FIPS": "16055", "County": "Kootenai", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83842", "FIPS": "16055", "County": "Kootenai", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83854", "FIPS": "16055", "County": "Kootenai", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83858", "FIPS": "16055", "County": "Kootenai", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83869", "FIPS": "16055", "County": "Kootenai", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83876", "FIPS": "16055", "County": "Kootenai", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83877", "FIPS": "16055", "County": "Kootenai", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83535", "FIPS": "16057", "County": "Latah", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83537", "FIPS": "16057", "County": "Latah", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83806", "FIPS": "16057", "County": "Latah", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83823", "FIPS": "16057", "County": "Latah", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83832", "FIPS": "16057", "County": "Latah", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83834", "FIPS": "16057", "County": "Latah", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83843", "FIPS": "16057", "County": "Latah", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83844", "FIPS": "16057", "County": "Latah", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83855", "FIPS": "16057", "County": "Latah", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83857", "FIPS": "16057", "County": "Latah", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83871", "FIPS": "16057", "County": "Latah", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83872", "FIPS": "16057", "County": "Latah", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83229", "FIPS": "16059", "County": "Lemhi", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83253", "FIPS": "16059", "County": "Lemhi", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83462", "FIPS": "16059", "County": "Lemhi", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83463", "FIPS": "16059", "County": "Lemhi", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83464", "FIPS": "16059", "County": "Lemhi", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83465", "FIPS": "16059", "County": "Lemhi", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83466", "FIPS": "16059", "County": "Lemhi", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83467", "FIPS": "16059", "County": "Lemhi", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83468", "FIPS": "16059", "County": "Lemhi", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83469", "FIPS": "16059", "County": "Lemhi", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83523", "FIPS": "16061", "County": "Lewis", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83536", "FIPS": "16061", "County": "Lewis", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83543", "FIPS": "16061", "County": "Lewis", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83548", "FIPS": "16061", "County": "Lewis", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83555", "FIPS": "16061", "County": "Lewis", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83324", "FIPS": "16063", "County": "Lincoln", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83349", "FIPS": "16063", "County": "Lincoln", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83352", "FIPS": "16063", "County": "Lincoln", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83440", "FIPS": "16065", "County": "Madison", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83441", "FIPS": "16065", "County": "Madison", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83448", "FIPS": "16065", "County": "Madison", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83460", "FIPS": "16065", "County": "Madison", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83336", "FIPS": "16067", "County": "Minidoka", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83343", "FIPS": "16067", "County": "Minidoka", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83347", "FIPS": "16067", "County": "Minidoka", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83350", "FIPS": "16067", "County": "Minidoka", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83501", "FIPS": "16069", "County": "Nez Perce", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83524", "FIPS": "16069", "County": "Nez Perce", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83540", "FIPS": "16069", "County": "Nez Perce", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83545", "FIPS": "16069", "County": "Nez Perce", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83243", "FIPS": "16071", "County": "Oneida", "State": "ID", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83252", "FIPS": "16071", "County": "Oneida", "State": "ID", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83604", "FIPS": "16073", "County": "Owyhee", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83624", "FIPS": "16073", "County": "Owyhee", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83628", "FIPS": "16073", "County": "Owyhee", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83639", "FIPS": "16073", "County": "Owyhee", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83650", "FIPS": "16073", "County": "Owyhee", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83619", "FIPS": "16075", "County": "Payette", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83655", "FIPS": "16075", "County": "Payette", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83661", "FIPS": "16075", "County": "Payette", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83211", "FIPS": "16077", "County": "Power", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83212", "FIPS": "16077", "County": "Power", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83271", "FIPS": "16077", "County": "Power", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83802", "FIPS": "16079", "County": "Shoshone", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83808", "FIPS": "16079", "County": "Shoshone", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83812", "FIPS": "16079", "County": "Shoshone", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83837", "FIPS": "16079", "County": "Shoshone", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83839", "FIPS": "16079", "County": "Shoshone", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83846", "FIPS": "16079", "County": "Shoshone", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83849", "FIPS": "16079", "County": "Shoshone", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83850", "FIPS": "16079", "County": "Shoshone", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83867", "FIPS": "16079", "County": "Shoshone", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83868", "FIPS": "16079", "County": "Shoshone", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83873", "FIPS": "16079", "County": "Shoshone", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83874", "FIPS": "16079", "County": "Shoshone", "State": "ID", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "83422", "FIPS": "16081", "County": "Teton", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83424", "FIPS": "16081", "County": "Teton", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83452", "FIPS": "16081", "County": "Teton", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83455", "FIPS": "16081", "County": "Teton", "State": "ID", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83301", "FIPS": "16083", "County": "Twin Falls", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83302", "FIPS": "16083", "County": "Twin Falls", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83303", "FIPS": "16083", "County": "Twin Falls", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83316", "FIPS": "16083", "County": "Twin Falls", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83321", "FIPS": "16083", "County": "Twin Falls", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83328", "FIPS": "16083", "County": "Twin Falls", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83334", "FIPS": "16083", "County": "Twin Falls", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83341", "FIPS": "16083", "County": "Twin Falls", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83344", "FIPS": "16083", "County": "Twin Falls", "State": "ID", "DMA": {"Code": 760, "Name": "TWIN FALLS"}},
  {"Postal_Code": "83611", "FIPS": "16085", "County": "Valley", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83615", "FIPS": "16085", "County": "Valley", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83635", "FIPS": "16085", "County": "Valley", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83638", "FIPS": "16085", "County": "Valley", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83677", "FIPS": "16085", "County": "Valley", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83610", "FIPS": "16087", "County": "Washington", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83645", "FIPS": "16087", "County": "Washington", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "83672", "FIPS": "16087", "County": "Washington", "State": "ID", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "62301", "FIPS": "17001", "County": "Adams", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62305", "FIPS": "17001", "County": "Adams", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62306", "FIPS": "17001", "County": "Adams", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62320", "FIPS": "17001", "County": "Adams", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62324", "FIPS": "17001", "County": "Adams", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62325", "FIPS": "17001", "County": "Adams", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62338", "FIPS": "17001", "County": "Adams", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62339", "FIPS": "17001", "County": "Adams", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62346", "FIPS": "17001", "County": "Adams", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62347", "FIPS": "17001", "County": "Adams", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62348", "FIPS": "17001", "County": "Adams", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62349", "FIPS": "17001", "County": "Adams", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62351", "FIPS": "17001", "County": "Adams", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62359", "FIPS": "17001", "County": "Adams", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62360", "FIPS": "17001", "County": "Adams", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62365", "FIPS": "17001", "County": "Adams", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62376", "FIPS": "17001", "County": "Adams", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62914", "FIPS": "17003", "County": "Alexander", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62957", "FIPS": "17003", "County": "Alexander", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62962", "FIPS": "17003", "County": "Alexander", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62969", "FIPS": "17003", "County": "Alexander", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62988", "FIPS": "17003", "County": "Alexander", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62990", "FIPS": "17003", "County": "Alexander", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62993", "FIPS": "17003", "County": "Alexander", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62086", "FIPS": "17005", "County": "Bond", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62246", "FIPS": "17005", "County": "Bond", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62262", "FIPS": "17005", "County": "Bond", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62273", "FIPS": "17005", "County": "Bond", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62275", "FIPS": "17005", "County": "Bond", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62284", "FIPS": "17005", "County": "Bond", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "61008", "FIPS": "17007", "County": "Boone", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61011", "FIPS": "17007", "County": "Boone", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61012", "FIPS": "17007", "County": "Boone", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61038", "FIPS": "17007", "County": "Boone", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61065", "FIPS": "17007", "County": "Boone", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "62353", "FIPS": "17009", "County": "Brown", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62375", "FIPS": "17009", "County": "Brown", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62378", "FIPS": "17009", "County": "Brown", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "61312", "FIPS": "17011", "County": "Bureau", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61314", "FIPS": "17011", "County": "Bureau", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61315", "FIPS": "17011", "County": "Bureau", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61317", "FIPS": "17011", "County": "Bureau", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61320", "FIPS": "17011", "County": "Bureau", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61322", "FIPS": "17011", "County": "Bureau", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61323", "FIPS": "17011", "County": "Bureau", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61328", "FIPS": "17011", "County": "Bureau", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61329", "FIPS": "17011", "County": "Bureau", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61330", "FIPS": "17011", "County": "Bureau", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61337", "FIPS": "17011", "County": "Bureau", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61338", "FIPS": "17011", "County": "Bureau", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61344", "FIPS": "17011", "County": "Bureau", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61345", "FIPS": "17011", "County": "Bureau", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61346", "FIPS": "17011", "County": "Bureau", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61349", "FIPS": "17011", "County": "Bureau", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61356", "FIPS": "17011", "County": "Bureau", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61359", "FIPS": "17011", "County": "Bureau", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61361", "FIPS": "17011", "County": "Bureau", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61362", "FIPS": "17011", "County": "Bureau", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61368", "FIPS": "17011", "County": "Bureau", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61374", "FIPS": "17011", "County": "Bureau", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61376", "FIPS": "17011", "County": "Bureau", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61379", "FIPS": "17011", "County": "Bureau", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "62006", "FIPS": "17013", "County": "Calhoun", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62013", "FIPS": "17013", "County": "Calhoun", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62036", "FIPS": "17013", "County": "Calhoun", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62045", "FIPS": "17013", "County": "Calhoun", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62047", "FIPS": "17013", "County": "Calhoun", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62053", "FIPS": "17013", "County": "Calhoun", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62065", "FIPS": "17013", "County": "Calhoun", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62070", "FIPS": "17013", "County": "Calhoun", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "61014", "FIPS": "17015", "County": "Carroll", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61046", "FIPS": "17015", "County": "Carroll", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61051", "FIPS": "17015", "County": "Carroll", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61053", "FIPS": "17015", "County": "Carroll", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61074", "FIPS": "17015", "County": "Carroll", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61078", "FIPS": "17015", "County": "Carroll", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61285", "FIPS": "17015", "County": "Carroll", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "62611", "FIPS": "17017", "County": "Cass", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62612", "FIPS": "17017", "County": "Cass", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62618", "FIPS": "17017", "County": "Cass", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62622", "FIPS": "17017", "County": "Cass", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62627", "FIPS": "17017", "County": "Cass", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62691", "FIPS": "17017", "County": "Cass", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60949", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61801", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61802", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61803", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61815", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61816", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61820", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61821", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61822", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61824", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61825", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61826", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61840", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61843", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61845", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61847", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61849", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61851", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61852", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61853", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61859", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61862", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61863", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61864", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61866", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61871", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61872", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61873", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61874", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61875", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61877", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61878", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61880", "FIPS": "17019", "County": "Champaign", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62083", "FIPS": "17021", "County": "Christian", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62510", "FIPS": "17021", "County": "Christian", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62517", "FIPS": "17021", "County": "Christian", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62531", "FIPS": "17021", "County": "Christian", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62540", "FIPS": "17021", "County": "Christian", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62546", "FIPS": "17021", "County": "Christian", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62547", "FIPS": "17021", "County": "Christian", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62550", "FIPS": "17021", "County": "Christian", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62555", "FIPS": "17021", "County": "Christian", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62556", "FIPS": "17021", "County": "Christian", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62557", "FIPS": "17021", "County": "Christian", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62567", "FIPS": "17021", "County": "Christian", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62568", "FIPS": "17021", "County": "Christian", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62570", "FIPS": "17021", "County": "Christian", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62420", "FIPS": "17023", "County": "Clark", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62423", "FIPS": "17023", "County": "Clark", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62441", "FIPS": "17023", "County": "Clark", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62442", "FIPS": "17023", "County": "Clark", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62474", "FIPS": "17023", "County": "Clark", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62477", "FIPS": "17023", "County": "Clark", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62434", "FIPS": "17025", "County": "Clay", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62824", "FIPS": "17025", "County": "Clay", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62839", "FIPS": "17025", "County": "Clay", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62858", "FIPS": "17025", "County": "Clay", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62879", "FIPS": "17025", "County": "Clay", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62899", "FIPS": "17025", "County": "Clay", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62215", "FIPS": "17027", "County": "Clinton", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62216", "FIPS": "17027", "County": "Clinton", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62218", "FIPS": "17027", "County": "Clinton", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62219", "FIPS": "17027", "County": "Clinton", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62230", "FIPS": "17027", "County": "Clinton", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62231", "FIPS": "17027", "County": "Clinton", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62245", "FIPS": "17027", "County": "Clinton", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62250", "FIPS": "17027", "County": "Clinton", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62252", "FIPS": "17027", "County": "Clinton", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62253", "FIPS": "17027", "County": "Clinton", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62265", "FIPS": "17027", "County": "Clinton", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62266", "FIPS": "17027", "County": "Clinton", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62293", "FIPS": "17027", "County": "Clinton", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "61912", "FIPS": "17029", "County": "Coles", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61920", "FIPS": "17029", "County": "Coles", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61931", "FIPS": "17029", "County": "Coles", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61938", "FIPS": "17029", "County": "Coles", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61943", "FIPS": "17029", "County": "Coles", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62440", "FIPS": "17029", "County": "Coles", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62469", "FIPS": "17029", "County": "Coles", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60004", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60005", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60006", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60007", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60008", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60009", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60016", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60017", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60018", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60019", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60022", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60025", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60026", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60029", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60038", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60043", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60053", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60055", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60056", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60062", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60065", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60067", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60068", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60070", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60074", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60076", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60077", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60078", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60082", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60090", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60091", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60093", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60094", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60095", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60104", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60107", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60130", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60131", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60133", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60141", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60153", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60154", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60155", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60159", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60160", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60161", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60162", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60163", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60164", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60165", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60168", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60169", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60171", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60173", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60176", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60179", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60192", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60193", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60194", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60195", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60196", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60201", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60202", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60203", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60204", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60208", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60209", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60296", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60297", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60301", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60302", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60303", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60304", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60305", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60402", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60406", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60409", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60411", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60412", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60415", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60419", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60422", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60425", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60426", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60428", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60429", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60430", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60438", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60439", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60443", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60445", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60452", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60453", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60454", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60455", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60456", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60457", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60458", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60459", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60461", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60462", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60463", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60464", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60465", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60466", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60467", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60469", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60471", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60472", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60473", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60475", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60476", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60477", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60478", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60480", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60482", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60483", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60487", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60499", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60501", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60513", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60525", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60526", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60534", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60546", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60558", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60601", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60602", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60603", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60604", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60605", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60606", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60607", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60608", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60609", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60610", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60611", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60612", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60613", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60614", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60615", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60616", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60617", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60618", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60619", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60620", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60621", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60622", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60623", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60624", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60625", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60626", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60628", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60629", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60630", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60631", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60632", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60633", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60634", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60636", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60637", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60638", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60639", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60640", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60641", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60643", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60644", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60645", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60646", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60647", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60649", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60651", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60652", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60653", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60654", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60655", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60656", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60657", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60659", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60660", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60661", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60663", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60664", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60666", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60668", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60669", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60670", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60673", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60674", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60675", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60677", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60678", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60679", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60680", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60681", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60682", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60684", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60685", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60686", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60687", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60688", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60689", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60690", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60691", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60693", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60694", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60695", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60696", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60697", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60699", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60701", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60706", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60707", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60712", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60714", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60803", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60804", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60805", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60827", "FIPS": "17031", "County": "Cook", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "62413", "FIPS": "17033", "County": "Crawford", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62427", "FIPS": "17033", "County": "Crawford", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62433", "FIPS": "17033", "County": "Crawford", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62449", "FIPS": "17033", "County": "Crawford", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62451", "FIPS": "17033", "County": "Crawford", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62454", "FIPS": "17033", "County": "Crawford", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62464", "FIPS": "17033", "County": "Crawford", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62478", "FIPS": "17033", "County": "Crawford", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62428", "FIPS": "17035", "County": "Cumberland", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62435", "FIPS": "17035", "County": "Cumberland", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62436", "FIPS": "17035", "County": "Cumberland", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62447", "FIPS": "17035", "County": "Cumberland", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62468", "FIPS": "17035", "County": "Cumberland", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60111", "FIPS": "17037", "County": "DeKalb", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60112", "FIPS": "17037", "County": "DeKalb", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60115", "FIPS": "17037", "County": "DeKalb", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60129", "FIPS": "17037", "County": "DeKalb", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60135", "FIPS": "17037", "County": "DeKalb", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60145", "FIPS": "17037", "County": "DeKalb", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60146", "FIPS": "17037", "County": "DeKalb", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60150", "FIPS": "17037", "County": "DeKalb", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60178", "FIPS": "17037", "County": "DeKalb", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60520", "FIPS": "17037", "County": "DeKalb", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60548", "FIPS": "17037", "County": "DeKalb", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60550", "FIPS": "17037", "County": "DeKalb", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60552", "FIPS": "17037", "County": "DeKalb", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60556", "FIPS": "17037", "County": "DeKalb", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "61727", "FIPS": "17039", "County": "Dewitt", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61735", "FIPS": "17039", "County": "Dewitt", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61749", "FIPS": "17039", "County": "Dewitt", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61750", "FIPS": "17039", "County": "Dewitt", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61777", "FIPS": "17039", "County": "Dewitt", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61778", "FIPS": "17039", "County": "Dewitt", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61842", "FIPS": "17039", "County": "Dewitt", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61882", "FIPS": "17039", "County": "Dewitt", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61910", "FIPS": "17041", "County": "Douglas", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61911", "FIPS": "17041", "County": "Douglas", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61913", "FIPS": "17041", "County": "Douglas", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61919", "FIPS": "17041", "County": "Douglas", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61930", "FIPS": "17041", "County": "Douglas", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61941", "FIPS": "17041", "County": "Douglas", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61942", "FIPS": "17041", "County": "Douglas", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61953", "FIPS": "17041", "County": "Douglas", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61956", "FIPS": "17041", "County": "Douglas", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60101", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60103", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60105", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60106", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60108", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60116", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60117", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60122", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60125", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60126", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60128", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60132", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60137", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60138", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60139", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60143", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60148", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60157", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60172", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60181", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60184", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60185", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60186", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60187", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60188", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60189", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60190", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60191", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60197", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60199", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60399", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60502", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60503", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60504", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60514", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60515", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60516", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60517", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60519", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60521", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60522", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60523", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60527", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60532", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60540", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60555", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60559", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60561", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60563", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60565", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60566", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60567", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60570", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60572", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60597", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60598", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60599", "FIPS": "17043", "County": "DuPage", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "61917", "FIPS": "17045", "County": "Edgar", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61924", "FIPS": "17045", "County": "Edgar", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61932", "FIPS": "17045", "County": "Edgar", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61933", "FIPS": "17045", "County": "Edgar", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61940", "FIPS": "17045", "County": "Edgar", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61944", "FIPS": "17045", "County": "Edgar", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61949", "FIPS": "17045", "County": "Edgar", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61955", "FIPS": "17045", "County": "Edgar", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62476", "FIPS": "17047", "County": "Edwards", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62806", "FIPS": "17047", "County": "Edwards", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62815", "FIPS": "17047", "County": "Edwards", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62818", "FIPS": "17047", "County": "Edwards", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62833", "FIPS": "17047", "County": "Edwards", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62401", "FIPS": "17049", "County": "Effingham", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62411", "FIPS": "17049", "County": "Effingham", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62414", "FIPS": "17049", "County": "Effingham", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62424", "FIPS": "17049", "County": "Effingham", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62426", "FIPS": "17049", "County": "Effingham", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62443", "FIPS": "17049", "County": "Effingham", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62445", "FIPS": "17049", "County": "Effingham", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62461", "FIPS": "17049", "County": "Effingham", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62467", "FIPS": "17049", "County": "Effingham", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62473", "FIPS": "17049", "County": "Effingham", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62011", "FIPS": "17051", "County": "Fayette", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62080", "FIPS": "17051", "County": "Fayette", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62247", "FIPS": "17051", "County": "Fayette", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62418", "FIPS": "17051", "County": "Fayette", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62458", "FIPS": "17051", "County": "Fayette", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62471", "FIPS": "17051", "County": "Fayette", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62838", "FIPS": "17051", "County": "Fayette", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62857", "FIPS": "17051", "County": "Fayette", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62880", "FIPS": "17051", "County": "Fayette", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62885", "FIPS": "17051", "County": "Fayette", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "60919", "FIPS": "17053", "County": "Ford", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60933", "FIPS": "17053", "County": "Ford", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60936", "FIPS": "17053", "County": "Ford", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60946", "FIPS": "17053", "County": "Ford", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60952", "FIPS": "17053", "County": "Ford", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60957", "FIPS": "17053", "County": "Ford", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60959", "FIPS": "17053", "County": "Ford", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60962", "FIPS": "17053", "County": "Ford", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61773", "FIPS": "17053", "County": "Ford", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62805", "FIPS": "17055", "County": "Franklin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62812", "FIPS": "17055", "County": "Franklin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62819", "FIPS": "17055", "County": "Franklin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62822", "FIPS": "17055", "County": "Franklin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62825", "FIPS": "17055", "County": "Franklin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62836", "FIPS": "17055", "County": "Franklin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62840", "FIPS": "17055", "County": "Franklin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62856", "FIPS": "17055", "County": "Franklin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62865", "FIPS": "17055", "County": "Franklin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62874", "FIPS": "17055", "County": "Franklin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62884", "FIPS": "17055", "County": "Franklin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62890", "FIPS": "17055", "County": "Franklin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62891", "FIPS": "17055", "County": "Franklin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62896", "FIPS": "17055", "County": "Franklin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62897", "FIPS": "17055", "County": "Franklin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62983", "FIPS": "17055", "County": "Franklin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62999", "FIPS": "17055", "County": "Franklin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "61415", "FIPS": "17057", "County": "Fulton", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61427", "FIPS": "17057", "County": "Fulton", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61431", "FIPS": "17057", "County": "Fulton", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61432", "FIPS": "17057", "County": "Fulton", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61433", "FIPS": "17057", "County": "Fulton", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61441", "FIPS": "17057", "County": "Fulton", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61459", "FIPS": "17057", "County": "Fulton", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61477", "FIPS": "17057", "County": "Fulton", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61482", "FIPS": "17057", "County": "Fulton", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61484", "FIPS": "17057", "County": "Fulton", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61501", "FIPS": "17057", "County": "Fulton", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61519", "FIPS": "17057", "County": "Fulton", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61520", "FIPS": "17057", "County": "Fulton", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61524", "FIPS": "17057", "County": "Fulton", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61531", "FIPS": "17057", "County": "Fulton", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61542", "FIPS": "17057", "County": "Fulton", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61543", "FIPS": "17057", "County": "Fulton", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61544", "FIPS": "17057", "County": "Fulton", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61553", "FIPS": "17057", "County": "Fulton", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61563", "FIPS": "17057", "County": "Fulton", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "62867", "FIPS": "17059", "County": "Gallatin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62871", "FIPS": "17059", "County": "Gallatin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62934", "FIPS": "17059", "County": "Gallatin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62954", "FIPS": "17059", "County": "Gallatin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62979", "FIPS": "17059", "County": "Gallatin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62984", "FIPS": "17059", "County": "Gallatin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62016", "FIPS": "17061", "County": "Greene", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62027", "FIPS": "17061", "County": "Greene", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62044", "FIPS": "17061", "County": "Greene", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62050", "FIPS": "17061", "County": "Greene", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62054", "FIPS": "17061", "County": "Greene", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62078", "FIPS": "17061", "County": "Greene", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62081", "FIPS": "17061", "County": "Greene", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62082", "FIPS": "17061", "County": "Greene", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62092", "FIPS": "17061", "County": "Greene", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62098", "FIPS": "17061", "County": "Greene", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "60407", "FIPS": "17063", "County": "Grundy", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60416", "FIPS": "17063", "County": "Grundy", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60424", "FIPS": "17063", "County": "Grundy", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60437", "FIPS": "17063", "County": "Grundy", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60444", "FIPS": "17063", "County": "Grundy", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60447", "FIPS": "17063", "County": "Grundy", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60450", "FIPS": "17063", "County": "Grundy", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60474", "FIPS": "17063", "County": "Grundy", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60479", "FIPS": "17063", "County": "Grundy", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "62817", "FIPS": "17065", "County": "Hamilton", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62828", "FIPS": "17065", "County": "Hamilton", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62829", "FIPS": "17065", "County": "Hamilton", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62859", "FIPS": "17065", "County": "Hamilton", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62860", "FIPS": "17065", "County": "Hamilton", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "61450", "FIPS": "17067", "County": "Hancock", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62311", "FIPS": "17067", "County": "Hancock", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62313", "FIPS": "17067", "County": "Hancock", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62316", "FIPS": "17067", "County": "Hancock", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62321", "FIPS": "17067", "County": "Hancock", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62329", "FIPS": "17067", "County": "Hancock", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62330", "FIPS": "17067", "County": "Hancock", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62334", "FIPS": "17067", "County": "Hancock", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62336", "FIPS": "17067", "County": "Hancock", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62341", "FIPS": "17067", "County": "Hancock", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62354", "FIPS": "17067", "County": "Hancock", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62358", "FIPS": "17067", "County": "Hancock", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62367", "FIPS": "17067", "County": "Hancock", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62373", "FIPS": "17067", "County": "Hancock", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62379", "FIPS": "17067", "County": "Hancock", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62380", "FIPS": "17067", "County": "Hancock", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62919", "FIPS": "17069", "County": "Hardin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62931", "FIPS": "17069", "County": "Hardin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62955", "FIPS": "17069", "County": "Hardin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62982", "FIPS": "17069", "County": "Hardin", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "61418", "FIPS": "17071", "County": "Henderson", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61425", "FIPS": "17071", "County": "Henderson", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61437", "FIPS": "17071", "County": "Henderson", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61454", "FIPS": "17071", "County": "Henderson", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61460", "FIPS": "17071", "County": "Henderson", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61469", "FIPS": "17071", "County": "Henderson", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61471", "FIPS": "17071", "County": "Henderson", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61480", "FIPS": "17071", "County": "Henderson", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61233", "FIPS": "17073", "County": "Henry", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61234", "FIPS": "17073", "County": "Henry", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61235", "FIPS": "17073", "County": "Henry", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61238", "FIPS": "17073", "County": "Henry", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61241", "FIPS": "17073", "County": "Henry", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61254", "FIPS": "17073", "County": "Henry", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61258", "FIPS": "17073", "County": "Henry", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61262", "FIPS": "17073", "County": "Henry", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61273", "FIPS": "17073", "County": "Henry", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61274", "FIPS": "17073", "County": "Henry", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61413", "FIPS": "17073", "County": "Henry", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61419", "FIPS": "17073", "County": "Henry", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61434", "FIPS": "17073", "County": "Henry", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61443", "FIPS": "17073", "County": "Henry", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61468", "FIPS": "17073", "County": "Henry", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61490", "FIPS": "17073", "County": "Henry", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "60911", "FIPS": "17075", "County": "Iroquois", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60912", "FIPS": "17075", "County": "Iroquois", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60918", "FIPS": "17075", "County": "Iroquois", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60922", "FIPS": "17075", "County": "Iroquois", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60924", "FIPS": "17075", "County": "Iroquois", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60926", "FIPS": "17075", "County": "Iroquois", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60927", "FIPS": "17075", "County": "Iroquois", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60928", "FIPS": "17075", "County": "Iroquois", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60930", "FIPS": "17075", "County": "Iroquois", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60931", "FIPS": "17075", "County": "Iroquois", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60938", "FIPS": "17075", "County": "Iroquois", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60939", "FIPS": "17075", "County": "Iroquois", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60945", "FIPS": "17075", "County": "Iroquois", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60948", "FIPS": "17075", "County": "Iroquois", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60951", "FIPS": "17075", "County": "Iroquois", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60953", "FIPS": "17075", "County": "Iroquois", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60955", "FIPS": "17075", "County": "Iroquois", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60956", "FIPS": "17075", "County": "Iroquois", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60966", "FIPS": "17075", "County": "Iroquois", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60967", "FIPS": "17075", "County": "Iroquois", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60968", "FIPS": "17075", "County": "Iroquois", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60970", "FIPS": "17075", "County": "Iroquois", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60973", "FIPS": "17075", "County": "Iroquois", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60974", "FIPS": "17075", "County": "Iroquois", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62901", "FIPS": "17077", "County": "Jackson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62902", "FIPS": "17077", "County": "Jackson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62903", "FIPS": "17077", "County": "Jackson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62907", "FIPS": "17077", "County": "Jackson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62916", "FIPS": "17077", "County": "Jackson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62924", "FIPS": "17077", "County": "Jackson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62927", "FIPS": "17077", "County": "Jackson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62932", "FIPS": "17077", "County": "Jackson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62940", "FIPS": "17077", "County": "Jackson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62942", "FIPS": "17077", "County": "Jackson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62950", "FIPS": "17077", "County": "Jackson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62958", "FIPS": "17077", "County": "Jackson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62966", "FIPS": "17077", "County": "Jackson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62971", "FIPS": "17077", "County": "Jackson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62975", "FIPS": "17077", "County": "Jackson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62994", "FIPS": "17077", "County": "Jackson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62432", "FIPS": "17079", "County": "Jasper", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62448", "FIPS": "17079", "County": "Jasper", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62459", "FIPS": "17079", "County": "Jasper", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62475", "FIPS": "17079", "County": "Jasper", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62479", "FIPS": "17079", "County": "Jasper", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62480", "FIPS": "17079", "County": "Jasper", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62481", "FIPS": "17079", "County": "Jasper", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62810", "FIPS": "17081", "County": "Jefferson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62814", "FIPS": "17081", "County": "Jefferson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62816", "FIPS": "17081", "County": "Jefferson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62830", "FIPS": "17081", "County": "Jefferson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62846", "FIPS": "17081", "County": "Jefferson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62864", "FIPS": "17081", "County": "Jefferson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62866", "FIPS": "17081", "County": "Jefferson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62872", "FIPS": "17081", "County": "Jefferson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62883", "FIPS": "17081", "County": "Jefferson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62889", "FIPS": "17081", "County": "Jefferson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62894", "FIPS": "17081", "County": "Jefferson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62898", "FIPS": "17081", "County": "Jefferson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62022", "FIPS": "17083", "County": "Jersey", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62028", "FIPS": "17083", "County": "Jersey", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62030", "FIPS": "17083", "County": "Jersey", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62031", "FIPS": "17083", "County": "Jersey", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62037", "FIPS": "17083", "County": "Jersey", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62052", "FIPS": "17083", "County": "Jersey", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62063", "FIPS": "17083", "County": "Jersey", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "61001", "FIPS": "17085", "County": "Jo Daviess", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61025", "FIPS": "17085", "County": "Jo Daviess", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61028", "FIPS": "17085", "County": "Jo Daviess", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61036", "FIPS": "17085", "County": "Jo Daviess", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61041", "FIPS": "17085", "County": "Jo Daviess", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61059", "FIPS": "17085", "County": "Jo Daviess", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61075", "FIPS": "17085", "County": "Jo Daviess", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61085", "FIPS": "17085", "County": "Jo Daviess", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61087", "FIPS": "17085", "County": "Jo Daviess", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "62908", "FIPS": "17087", "County": "Johnson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62909", "FIPS": "17087", "County": "Johnson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62912", "FIPS": "17087", "County": "Johnson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62923", "FIPS": "17087", "County": "Johnson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62939", "FIPS": "17087", "County": "Johnson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62943", "FIPS": "17087", "County": "Johnson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62967", "FIPS": "17087", "County": "Johnson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62972", "FIPS": "17087", "County": "Johnson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62985", "FIPS": "17087", "County": "Johnson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62995", "FIPS": "17087", "County": "Johnson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "60109", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60110", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60118", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60119", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60120", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60121", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60123", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60124", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60134", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60136", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60140", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60144", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60147", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60151", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60170", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60174", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60175", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60177", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60183", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60505", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60506", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60507", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60510", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60511", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60539", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60542", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60554", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60568", "FIPS": "17089", "County": "Kane", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60901", "FIPS": "17091", "County": "Kankakee", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60910", "FIPS": "17091", "County": "Kankakee", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60913", "FIPS": "17091", "County": "Kankakee", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60914", "FIPS": "17091", "County": "Kankakee", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60915", "FIPS": "17091", "County": "Kankakee", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60917", "FIPS": "17091", "County": "Kankakee", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60935", "FIPS": "17091", "County": "Kankakee", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60940", "FIPS": "17091", "County": "Kankakee", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60941", "FIPS": "17091", "County": "Kankakee", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60944", "FIPS": "17091", "County": "Kankakee", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60950", "FIPS": "17091", "County": "Kankakee", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60954", "FIPS": "17091", "County": "Kankakee", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60961", "FIPS": "17091", "County": "Kankakee", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60964", "FIPS": "17091", "County": "Kankakee", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60969", "FIPS": "17091", "County": "Kankakee", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60512", "FIPS": "17093", "County": "Kendall", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60536", "FIPS": "17093", "County": "Kendall", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60537", "FIPS": "17093", "County": "Kendall", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60538", "FIPS": "17093", "County": "Kendall", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60541", "FIPS": "17093", "County": "Kendall", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60543", "FIPS": "17093", "County": "Kendall", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60545", "FIPS": "17093", "County": "Kendall", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60560", "FIPS": "17093", "County": "Kendall", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "61401", "FIPS": "17095", "County": "Knox", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61402", "FIPS": "17095", "County": "Knox", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61410", "FIPS": "17095", "County": "Knox", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61414", "FIPS": "17095", "County": "Knox", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61428", "FIPS": "17095", "County": "Knox", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61430", "FIPS": "17095", "County": "Knox", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61436", "FIPS": "17095", "County": "Knox", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61439", "FIPS": "17095", "County": "Knox", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61448", "FIPS": "17095", "County": "Knox", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61458", "FIPS": "17095", "County": "Knox", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61467", "FIPS": "17095", "County": "Knox", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61472", "FIPS": "17095", "County": "Knox", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61474", "FIPS": "17095", "County": "Knox", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61485", "FIPS": "17095", "County": "Knox", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61488", "FIPS": "17095", "County": "Knox", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61489", "FIPS": "17095", "County": "Knox", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61572", "FIPS": "17095", "County": "Knox", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "60002", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60010", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60011", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60015", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60020", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60030", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60031", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60035", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60037", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60040", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60041", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60042", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60044", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60045", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60046", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60047", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60048", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60049", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60060", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60061", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60064", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60069", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60073", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60075", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60079", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60083", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60084", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60085", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60086", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60087", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60088", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60089", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60092", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60096", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60099", "FIPS": "17097", "County": "Lake", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60470", "FIPS": "17099", "County": "La Salle", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60518", "FIPS": "17099", "County": "La Salle", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60531", "FIPS": "17099", "County": "La Salle", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60549", "FIPS": "17099", "County": "La Salle", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60551", "FIPS": "17099", "County": "La Salle", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60557", "FIPS": "17099", "County": "La Salle", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "61301", "FIPS": "17099", "County": "La Salle", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "61316", "FIPS": "17099", "County": "La Salle", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "61321", "FIPS": "17099", "County": "La Salle", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "61325", "FIPS": "17099", "County": "La Salle", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "61332", "FIPS": "17099", "County": "La Salle", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "61334", "FIPS": "17099", "County": "La Salle", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "61341", "FIPS": "17099", "County": "La Salle", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "61342", "FIPS": "17099", "County": "La Salle", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "61348", "FIPS": "17099", "County": "La Salle", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "61350", "FIPS": "17099", "County": "La Salle", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "61354", "FIPS": "17099", "County": "La Salle", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "61358", "FIPS": "17099", "County": "La Salle", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "61360", "FIPS": "17099", "County": "La Salle", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "61364", "FIPS": "17099", "County": "La Salle", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "61370", "FIPS": "17099", "County": "La Salle", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "61371", "FIPS": "17099", "County": "La Salle", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "61372", "FIPS": "17099", "County": "La Salle", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "61373", "FIPS": "17099", "County": "La Salle", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "62417", "FIPS": "17101", "County": "Lawrence", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62439", "FIPS": "17101", "County": "Lawrence", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62460", "FIPS": "17101", "County": "Lawrence", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62466", "FIPS": "17101", "County": "Lawrence", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "60530", "FIPS": "17103", "County": "Lee", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "60553", "FIPS": "17103", "County": "Lee", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61006", "FIPS": "17103", "County": "Lee", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61021", "FIPS": "17103", "County": "Lee", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61031", "FIPS": "17103", "County": "Lee", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61042", "FIPS": "17103", "County": "Lee", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61057", "FIPS": "17103", "County": "Lee", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61058", "FIPS": "17103", "County": "Lee", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61310", "FIPS": "17103", "County": "Lee", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61318", "FIPS": "17103", "County": "Lee", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61324", "FIPS": "17103", "County": "Lee", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61331", "FIPS": "17103", "County": "Lee", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61353", "FIPS": "17103", "County": "Lee", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61367", "FIPS": "17103", "County": "Lee", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61378", "FIPS": "17103", "County": "Lee", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "60420", "FIPS": "17105", "County": "Livingston", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "60460", "FIPS": "17105", "County": "Livingston", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "60920", "FIPS": "17105", "County": "Livingston", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "60921", "FIPS": "17105", "County": "Livingston", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "60929", "FIPS": "17105", "County": "Livingston", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "60934", "FIPS": "17105", "County": "Livingston", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61311", "FIPS": "17105", "County": "Livingston", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61313", "FIPS": "17105", "County": "Livingston", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61319", "FIPS": "17105", "County": "Livingston", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61333", "FIPS": "17105", "County": "Livingston", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61739", "FIPS": "17105", "County": "Livingston", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61740", "FIPS": "17105", "County": "Livingston", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61741", "FIPS": "17105", "County": "Livingston", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61743", "FIPS": "17105", "County": "Livingston", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61764", "FIPS": "17105", "County": "Livingston", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61769", "FIPS": "17105", "County": "Livingston", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61775", "FIPS": "17105", "County": "Livingston", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61723", "FIPS": "17107", "County": "Logan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61751", "FIPS": "17107", "County": "Logan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62512", "FIPS": "17107", "County": "Logan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62518", "FIPS": "17107", "County": "Logan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62519", "FIPS": "17107", "County": "Logan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62541", "FIPS": "17107", "County": "Logan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62543", "FIPS": "17107", "County": "Logan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62548", "FIPS": "17107", "County": "Logan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62634", "FIPS": "17107", "County": "Logan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62635", "FIPS": "17107", "County": "Logan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62643", "FIPS": "17107", "County": "Logan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62656", "FIPS": "17107", "County": "Logan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62666", "FIPS": "17107", "County": "Logan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62671", "FIPS": "17107", "County": "Logan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61411", "FIPS": "17109", "County": "McDonough", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "61416", "FIPS": "17109", "County": "McDonough", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "61420", "FIPS": "17109", "County": "McDonough", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "61422", "FIPS": "17109", "County": "McDonough", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "61438", "FIPS": "17109", "County": "McDonough", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "61440", "FIPS": "17109", "County": "McDonough", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "61455", "FIPS": "17109", "County": "McDonough", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "61470", "FIPS": "17109", "County": "McDonough", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "61475", "FIPS": "17109", "County": "McDonough", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62326", "FIPS": "17109", "County": "McDonough", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62374", "FIPS": "17109", "County": "McDonough", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "60001", "FIPS": "17111", "County": "McHenry", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60012", "FIPS": "17111", "County": "McHenry", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60013", "FIPS": "17111", "County": "McHenry", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60014", "FIPS": "17111", "County": "McHenry", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60021", "FIPS": "17111", "County": "McHenry", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60033", "FIPS": "17111", "County": "McHenry", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60034", "FIPS": "17111", "County": "McHenry", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60039", "FIPS": "17111", "County": "McHenry", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60050", "FIPS": "17111", "County": "McHenry", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60051", "FIPS": "17111", "County": "McHenry", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60071", "FIPS": "17111", "County": "McHenry", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60072", "FIPS": "17111", "County": "McHenry", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60081", "FIPS": "17111", "County": "McHenry", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60097", "FIPS": "17111", "County": "McHenry", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60098", "FIPS": "17111", "County": "McHenry", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60102", "FIPS": "17111", "County": "McHenry", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60142", "FIPS": "17111", "County": "McHenry", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60152", "FIPS": "17111", "County": "McHenry", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60156", "FIPS": "17111", "County": "McHenry", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60180", "FIPS": "17111", "County": "McHenry", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "61701", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61702", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61704", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61709", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61710", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61720", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61722", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61724", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61725", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61726", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61728", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61730", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61731", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61732", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61736", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61737", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61744", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61745", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61748", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61752", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61753", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61754", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61758", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61761", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61770", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61772", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61774", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61776", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61790", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61791", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61799", "FIPS": "17113", "County": "McLean", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61756", "FIPS": "17115", "County": "Macon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62501", "FIPS": "17115", "County": "Macon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62513", "FIPS": "17115", "County": "Macon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62514", "FIPS": "17115", "County": "Macon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62521", "FIPS": "17115", "County": "Macon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62522", "FIPS": "17115", "County": "Macon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62523", "FIPS": "17115", "County": "Macon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62524", "FIPS": "17115", "County": "Macon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62525", "FIPS": "17115", "County": "Macon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62526", "FIPS": "17115", "County": "Macon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62532", "FIPS": "17115", "County": "Macon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62535", "FIPS": "17115", "County": "Macon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62537", "FIPS": "17115", "County": "Macon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62544", "FIPS": "17115", "County": "Macon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62549", "FIPS": "17115", "County": "Macon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62551", "FIPS": "17115", "County": "Macon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62554", "FIPS": "17115", "County": "Macon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62573", "FIPS": "17115", "County": "Macon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62009", "FIPS": "17117", "County": "Macoupin", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62012", "FIPS": "17117", "County": "Macoupin", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62014", "FIPS": "17117", "County": "Macoupin", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62023", "FIPS": "17117", "County": "Macoupin", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62033", "FIPS": "17117", "County": "Macoupin", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62069", "FIPS": "17117", "County": "Macoupin", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62079", "FIPS": "17117", "County": "Macoupin", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62085", "FIPS": "17117", "County": "Macoupin", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62088", "FIPS": "17117", "County": "Macoupin", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62093", "FIPS": "17117", "County": "Macoupin", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62626", "FIPS": "17117", "County": "Macoupin", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62630", "FIPS": "17117", "County": "Macoupin", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62640", "FIPS": "17117", "County": "Macoupin", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62649", "FIPS": "17117", "County": "Macoupin", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62667", "FIPS": "17117", "County": "Macoupin", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62672", "FIPS": "17117", "County": "Macoupin", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62674", "FIPS": "17117", "County": "Macoupin", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62683", "FIPS": "17117", "County": "Macoupin", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62685", "FIPS": "17117", "County": "Macoupin", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62690", "FIPS": "17117", "County": "Macoupin", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62001", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62002", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62010", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62018", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62021", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62024", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62025", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62026", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62034", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62035", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62040", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62046", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62048", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62058", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62060", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62061", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62062", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62067", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62074", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62084", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62087", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62090", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62095", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62097", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62234", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62249", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62281", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62294", "FIPS": "17119", "County": "Madison", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62801", "FIPS": "17121", "County": "Marion", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62807", "FIPS": "17121", "County": "Marion", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62849", "FIPS": "17121", "County": "Marion", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62853", "FIPS": "17121", "County": "Marion", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62854", "FIPS": "17121", "County": "Marion", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62870", "FIPS": "17121", "County": "Marion", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62875", "FIPS": "17121", "County": "Marion", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62881", "FIPS": "17121", "County": "Marion", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62882", "FIPS": "17121", "County": "Marion", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62892", "FIPS": "17121", "County": "Marion", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62893", "FIPS": "17121", "County": "Marion", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "61369", "FIPS": "17123", "County": "Marshall", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61375", "FIPS": "17123", "County": "Marshall", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61377", "FIPS": "17123", "County": "Marshall", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61424", "FIPS": "17123", "County": "Marshall", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61537", "FIPS": "17123", "County": "Marshall", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61540", "FIPS": "17123", "County": "Marshall", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61541", "FIPS": "17123", "County": "Marshall", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61565", "FIPS": "17123", "County": "Marshall", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61570", "FIPS": "17123", "County": "Marshall", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61532", "FIPS": "17125", "County": "Mason", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61546", "FIPS": "17125", "County": "Mason", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61567", "FIPS": "17125", "County": "Mason", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "62617", "FIPS": "17125", "County": "Mason", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "62633", "FIPS": "17125", "County": "Mason", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "62644", "FIPS": "17125", "County": "Mason", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "62655", "FIPS": "17125", "County": "Mason", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "62664", "FIPS": "17125", "County": "Mason", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "62682", "FIPS": "17125", "County": "Mason", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "62910", "FIPS": "17127", "County": "Massac", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62953", "FIPS": "17127", "County": "Massac", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62960", "FIPS": "17127", "County": "Massac", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62613", "FIPS": "17129", "County": "Menard", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62642", "FIPS": "17129", "County": "Menard", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62659", "FIPS": "17129", "County": "Menard", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62673", "FIPS": "17129", "County": "Menard", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62675", "FIPS": "17129", "County": "Menard", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62688", "FIPS": "17129", "County": "Menard", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61231", "FIPS": "17131", "County": "Mercer", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61260", "FIPS": "17131", "County": "Mercer", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61263", "FIPS": "17131", "County": "Mercer", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61272", "FIPS": "17131", "County": "Mercer", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61276", "FIPS": "17131", "County": "Mercer", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61281", "FIPS": "17131", "County": "Mercer", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61412", "FIPS": "17131", "County": "Mercer", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61442", "FIPS": "17131", "County": "Mercer", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61465", "FIPS": "17131", "County": "Mercer", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61466", "FIPS": "17131", "County": "Mercer", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61476", "FIPS": "17131", "County": "Mercer", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61486", "FIPS": "17131", "County": "Mercer", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "62236", "FIPS": "17133", "County": "Monroe", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62244", "FIPS": "17133", "County": "Monroe", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62248", "FIPS": "17133", "County": "Monroe", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62256", "FIPS": "17133", "County": "Monroe", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62279", "FIPS": "17133", "County": "Monroe", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62295", "FIPS": "17133", "County": "Monroe", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62298", "FIPS": "17133", "County": "Monroe", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62015", "FIPS": "17135", "County": "Montgomery", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62017", "FIPS": "17135", "County": "Montgomery", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62019", "FIPS": "17135", "County": "Montgomery", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62032", "FIPS": "17135", "County": "Montgomery", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62049", "FIPS": "17135", "County": "Montgomery", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62051", "FIPS": "17135", "County": "Montgomery", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62056", "FIPS": "17135", "County": "Montgomery", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62075", "FIPS": "17135", "County": "Montgomery", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62076", "FIPS": "17135", "County": "Montgomery", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62077", "FIPS": "17135", "County": "Montgomery", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62089", "FIPS": "17135", "County": "Montgomery", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62091", "FIPS": "17135", "County": "Montgomery", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62094", "FIPS": "17135", "County": "Montgomery", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62533", "FIPS": "17135", "County": "Montgomery", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62538", "FIPS": "17135", "County": "Montgomery", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62560", "FIPS": "17135", "County": "Montgomery", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62572", "FIPS": "17135", "County": "Montgomery", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62601", "FIPS": "17137", "County": "Morgan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62628", "FIPS": "17137", "County": "Morgan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62631", "FIPS": "17137", "County": "Morgan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62638", "FIPS": "17137", "County": "Morgan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62650", "FIPS": "17137", "County": "Morgan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62651", "FIPS": "17137", "County": "Morgan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62660", "FIPS": "17137", "County": "Morgan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62665", "FIPS": "17137", "County": "Morgan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62668", "FIPS": "17137", "County": "Morgan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62692", "FIPS": "17137", "County": "Morgan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62695", "FIPS": "17137", "County": "Morgan", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61914", "FIPS": "17139", "County": "Moultrie", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61925", "FIPS": "17139", "County": "Moultrie", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61928", "FIPS": "17139", "County": "Moultrie", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61937", "FIPS": "17139", "County": "Moultrie", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61951", "FIPS": "17139", "County": "Moultrie", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60113", "FIPS": "17141", "County": "Ogle", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61007", "FIPS": "17141", "County": "Ogle", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61010", "FIPS": "17141", "County": "Ogle", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61015", "FIPS": "17141", "County": "Ogle", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61020", "FIPS": "17141", "County": "Ogle", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61030", "FIPS": "17141", "County": "Ogle", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61043", "FIPS": "17141", "County": "Ogle", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61047", "FIPS": "17141", "County": "Ogle", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61049", "FIPS": "17141", "County": "Ogle", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61052", "FIPS": "17141", "County": "Ogle", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61054", "FIPS": "17141", "County": "Ogle", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61061", "FIPS": "17141", "County": "Ogle", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61064", "FIPS": "17141", "County": "Ogle", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61068", "FIPS": "17141", "County": "Ogle", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61084", "FIPS": "17141", "County": "Ogle", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61091", "FIPS": "17141", "County": "Ogle", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61451", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61517", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61523", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61525", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61526", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61528", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61529", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61533", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61536", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61539", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61547", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61552", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61559", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61562", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61569", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61601", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61602", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61603", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61604", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61605", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61606", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61607", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61612", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61613", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61614", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61615", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61616", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61625", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61629", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61630", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61633", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61634", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61635", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61636", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61637", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61638", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61639", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61641", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61643", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61650", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61651", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61652", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61653", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61654", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61655", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61656", "FIPS": "17143", "County": "Peoria", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "62238", "FIPS": "17145", "County": "Perry", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62274", "FIPS": "17145", "County": "Perry", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62832", "FIPS": "17145", "County": "Perry", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62888", "FIPS": "17145", "County": "Perry", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62997", "FIPS": "17145", "County": "Perry", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "61813", "FIPS": "17147", "County": "Piatt", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61818", "FIPS": "17147", "County": "Piatt", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61830", "FIPS": "17147", "County": "Piatt", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61839", "FIPS": "17147", "County": "Piatt", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61854", "FIPS": "17147", "County": "Piatt", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61855", "FIPS": "17147", "County": "Piatt", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61856", "FIPS": "17147", "County": "Piatt", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61884", "FIPS": "17147", "County": "Piatt", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61929", "FIPS": "17147", "County": "Piatt", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61936", "FIPS": "17147", "County": "Piatt", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62312", "FIPS": "17149", "County": "Pike", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62314", "FIPS": "17149", "County": "Pike", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62323", "FIPS": "17149", "County": "Pike", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62340", "FIPS": "17149", "County": "Pike", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62343", "FIPS": "17149", "County": "Pike", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62345", "FIPS": "17149", "County": "Pike", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62352", "FIPS": "17149", "County": "Pike", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62355", "FIPS": "17149", "County": "Pike", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62356", "FIPS": "17149", "County": "Pike", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62357", "FIPS": "17149", "County": "Pike", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62361", "FIPS": "17149", "County": "Pike", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62362", "FIPS": "17149", "County": "Pike", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62363", "FIPS": "17149", "County": "Pike", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62366", "FIPS": "17149", "County": "Pike", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62370", "FIPS": "17149", "County": "Pike", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62928", "FIPS": "17151", "County": "Pope", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62938", "FIPS": "17151", "County": "Pope", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62947", "FIPS": "17151", "County": "Pope", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62941", "FIPS": "17153", "County": "Pulaski", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62956", "FIPS": "17153", "County": "Pulaski", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62963", "FIPS": "17153", "County": "Pulaski", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62964", "FIPS": "17153", "County": "Pulaski", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62970", "FIPS": "17153", "County": "Pulaski", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62973", "FIPS": "17153", "County": "Pulaski", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62976", "FIPS": "17153", "County": "Pulaski", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62992", "FIPS": "17153", "County": "Pulaski", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62996", "FIPS": "17153", "County": "Pulaski", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "61326", "FIPS": "17155", "County": "Putnam", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61327", "FIPS": "17155", "County": "Putnam", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61335", "FIPS": "17155", "County": "Putnam", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61336", "FIPS": "17155", "County": "Putnam", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61340", "FIPS": "17155", "County": "Putnam", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61363", "FIPS": "17155", "County": "Putnam", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61560", "FIPS": "17155", "County": "Putnam", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "62217", "FIPS": "17157", "County": "Randolph", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62233", "FIPS": "17157", "County": "Randolph", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62237", "FIPS": "17157", "County": "Randolph", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62241", "FIPS": "17157", "County": "Randolph", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62242", "FIPS": "17157", "County": "Randolph", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62259", "FIPS": "17157", "County": "Randolph", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62261", "FIPS": "17157", "County": "Randolph", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62272", "FIPS": "17157", "County": "Randolph", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62277", "FIPS": "17157", "County": "Randolph", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62278", "FIPS": "17157", "County": "Randolph", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62280", "FIPS": "17157", "County": "Randolph", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62286", "FIPS": "17157", "County": "Randolph", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62288", "FIPS": "17157", "County": "Randolph", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62292", "FIPS": "17157", "County": "Randolph", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62297", "FIPS": "17157", "County": "Randolph", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62419", "FIPS": "17159", "County": "Richland", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62421", "FIPS": "17159", "County": "Richland", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62425", "FIPS": "17159", "County": "Richland", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62450", "FIPS": "17159", "County": "Richland", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62452", "FIPS": "17159", "County": "Richland", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "62868", "FIPS": "17159", "County": "Richland", "State": "IL", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "61201", "FIPS": "17161", "County": "Rock Island", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61204", "FIPS": "17161", "County": "Rock Island", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61232", "FIPS": "17161", "County": "Rock Island", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61236", "FIPS": "17161", "County": "Rock Island", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61237", "FIPS": "17161", "County": "Rock Island", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61239", "FIPS": "17161", "County": "Rock Island", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61240", "FIPS": "17161", "County": "Rock Island", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61242", "FIPS": "17161", "County": "Rock Island", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61244", "FIPS": "17161", "County": "Rock Island", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61256", "FIPS": "17161", "County": "Rock Island", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61257", "FIPS": "17161", "County": "Rock Island", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61259", "FIPS": "17161", "County": "Rock Island", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61264", "FIPS": "17161", "County": "Rock Island", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61265", "FIPS": "17161", "County": "Rock Island", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61266", "FIPS": "17161", "County": "Rock Island", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61275", "FIPS": "17161", "County": "Rock Island", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61278", "FIPS": "17161", "County": "Rock Island", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61279", "FIPS": "17161", "County": "Rock Island", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61282", "FIPS": "17161", "County": "Rock Island", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61284", "FIPS": "17161", "County": "Rock Island", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61299", "FIPS": "17161", "County": "Rock Island", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "62059", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62071", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62201", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62202", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62203", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62204", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62205", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62206", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62207", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62208", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62220", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62221", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62222", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62223", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62224", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62225", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62226", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62232", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62239", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62240", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62243", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62254", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62255", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62257", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62258", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62260", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62264", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62269", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62282", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62285", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62289", "FIPS": "17163", "County": "Saint Clair", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62917", "FIPS": "17165", "County": "Saline", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62930", "FIPS": "17165", "County": "Saline", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62935", "FIPS": "17165", "County": "Saline", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62946", "FIPS": "17165", "County": "Saline", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62965", "FIPS": "17165", "County": "Saline", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62977", "FIPS": "17165", "County": "Saline", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62987", "FIPS": "17165", "County": "Saline", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62515", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62520", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62530", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62536", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62539", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62545", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62558", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62561", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62563", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62615", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62625", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62629", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62661", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62662", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62670", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62677", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62684", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62689", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62693", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62701", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62702", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62703", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62704", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62705", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62706", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62707", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62708", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62711", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62712", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62713", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62715", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62716", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62719", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62721", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62722", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62723", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62726", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62736", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62739", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62746", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62756", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62757", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62761", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62762", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62763", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62764", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62765", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62766", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62767", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62769", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62776", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62777", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62781", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62786", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62791", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62794", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62796", "FIPS": "17167", "County": "Sangamon", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61452", "FIPS": "17169", "County": "Schuyler", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62319", "FIPS": "17169", "County": "Schuyler", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62344", "FIPS": "17169", "County": "Schuyler", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62624", "FIPS": "17169", "County": "Schuyler", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62639", "FIPS": "17169", "County": "Schuyler", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62681", "FIPS": "17169", "County": "Schuyler", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62610", "FIPS": "17171", "County": "Scott", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62621", "FIPS": "17171", "County": "Scott", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62663", "FIPS": "17171", "County": "Scott", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "62694", "FIPS": "17171", "County": "Scott", "State": "IL", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "61957", "FIPS": "17173", "County": "Shelby", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62422", "FIPS": "17173", "County": "Shelby", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62431", "FIPS": "17173", "County": "Shelby", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62438", "FIPS": "17173", "County": "Shelby", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62444", "FIPS": "17173", "County": "Shelby", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62462", "FIPS": "17173", "County": "Shelby", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62463", "FIPS": "17173", "County": "Shelby", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62465", "FIPS": "17173", "County": "Shelby", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62534", "FIPS": "17173", "County": "Shelby", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62553", "FIPS": "17173", "County": "Shelby", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62565", "FIPS": "17173", "County": "Shelby", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62571", "FIPS": "17173", "County": "Shelby", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61421", "FIPS": "17175", "County": "Stark", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61426", "FIPS": "17175", "County": "Stark", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61449", "FIPS": "17175", "County": "Stark", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61479", "FIPS": "17175", "County": "Stark", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61483", "FIPS": "17175", "County": "Stark", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61491", "FIPS": "17175", "County": "Stark", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61013", "FIPS": "17177", "County": "Stephenson", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61018", "FIPS": "17177", "County": "Stephenson", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61019", "FIPS": "17177", "County": "Stephenson", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61027", "FIPS": "17177", "County": "Stephenson", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61032", "FIPS": "17177", "County": "Stephenson", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61039", "FIPS": "17177", "County": "Stephenson", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61044", "FIPS": "17177", "County": "Stephenson", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61048", "FIPS": "17177", "County": "Stephenson", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61050", "FIPS": "17177", "County": "Stephenson", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61060", "FIPS": "17177", "County": "Stephenson", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61062", "FIPS": "17177", "County": "Stephenson", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61067", "FIPS": "17177", "County": "Stephenson", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61070", "FIPS": "17177", "County": "Stephenson", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61089", "FIPS": "17177", "County": "Stephenson", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61534", "FIPS": "17179", "County": "Tazewell", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61535", "FIPS": "17179", "County": "Tazewell", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61550", "FIPS": "17179", "County": "Tazewell", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61554", "FIPS": "17179", "County": "Tazewell", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61555", "FIPS": "17179", "County": "Tazewell", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61558", "FIPS": "17179", "County": "Tazewell", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61564", "FIPS": "17179", "County": "Tazewell", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61568", "FIPS": "17179", "County": "Tazewell", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61571", "FIPS": "17179", "County": "Tazewell", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61610", "FIPS": "17179", "County": "Tazewell", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61611", "FIPS": "17179", "County": "Tazewell", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61721", "FIPS": "17179", "County": "Tazewell", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61733", "FIPS": "17179", "County": "Tazewell", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61734", "FIPS": "17179", "County": "Tazewell", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61747", "FIPS": "17179", "County": "Tazewell", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61755", "FIPS": "17179", "County": "Tazewell", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61759", "FIPS": "17179", "County": "Tazewell", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "62905", "FIPS": "17181", "County": "Union", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62906", "FIPS": "17181", "County": "Union", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62920", "FIPS": "17181", "County": "Union", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62926", "FIPS": "17181", "County": "Union", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62952", "FIPS": "17181", "County": "Union", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62961", "FIPS": "17181", "County": "Union", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62998", "FIPS": "17181", "County": "Union", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "60932", "FIPS": "17183", "County": "Vermilion", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60942", "FIPS": "17183", "County": "Vermilion", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60960", "FIPS": "17183", "County": "Vermilion", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "60963", "FIPS": "17183", "County": "Vermilion", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61810", "FIPS": "17183", "County": "Vermilion", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61811", "FIPS": "17183", "County": "Vermilion", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61812", "FIPS": "17183", "County": "Vermilion", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61814", "FIPS": "17183", "County": "Vermilion", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61817", "FIPS": "17183", "County": "Vermilion", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61831", "FIPS": "17183", "County": "Vermilion", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61832", "FIPS": "17183", "County": "Vermilion", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61833", "FIPS": "17183", "County": "Vermilion", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61834", "FIPS": "17183", "County": "Vermilion", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61841", "FIPS": "17183", "County": "Vermilion", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61844", "FIPS": "17183", "County": "Vermilion", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61846", "FIPS": "17183", "County": "Vermilion", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61848", "FIPS": "17183", "County": "Vermilion", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61850", "FIPS": "17183", "County": "Vermilion", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61857", "FIPS": "17183", "County": "Vermilion", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61858", "FIPS": "17183", "County": "Vermilion", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61865", "FIPS": "17183", "County": "Vermilion", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61870", "FIPS": "17183", "County": "Vermilion", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61876", "FIPS": "17183", "County": "Vermilion", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "61883", "FIPS": "17183", "County": "Vermilion", "State": "IL", "DMA": {"Code": 648, "Name": "CHAMPAIGN-SPRINGFIELD-DECATUR"}},
  {"Postal_Code": "62410", "FIPS": "17185", "County": "Wabash", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62811", "FIPS": "17185", "County": "Wabash", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62852", "FIPS": "17185", "County": "Wabash", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62855", "FIPS": "17185", "County": "Wabash", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62863", "FIPS": "17185", "County": "Wabash", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "61417", "FIPS": "17187", "County": "Warren", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61423", "FIPS": "17187", "County": "Warren", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61435", "FIPS": "17187", "County": "Warren", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61447", "FIPS": "17187", "County": "Warren", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61453", "FIPS": "17187", "County": "Warren", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61462", "FIPS": "17187", "County": "Warren", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61473", "FIPS": "17187", "County": "Warren", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61478", "FIPS": "17187", "County": "Warren", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "62214", "FIPS": "17189", "County": "Washington", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62263", "FIPS": "17189", "County": "Washington", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62268", "FIPS": "17189", "County": "Washington", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62271", "FIPS": "17189", "County": "Washington", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62803", "FIPS": "17189", "County": "Washington", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62808", "FIPS": "17189", "County": "Washington", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62831", "FIPS": "17189", "County": "Washington", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62848", "FIPS": "17189", "County": "Washington", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62876", "FIPS": "17189", "County": "Washington", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62877", "FIPS": "17189", "County": "Washington", "State": "IL", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "62446", "FIPS": "17191", "County": "Wayne", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62809", "FIPS": "17191", "County": "Wayne", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62823", "FIPS": "17191", "County": "Wayne", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62837", "FIPS": "17191", "County": "Wayne", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62842", "FIPS": "17191", "County": "Wayne", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62843", "FIPS": "17191", "County": "Wayne", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62850", "FIPS": "17191", "County": "Wayne", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62851", "FIPS": "17191", "County": "Wayne", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62878", "FIPS": "17191", "County": "Wayne", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62886", "FIPS": "17191", "County": "Wayne", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62895", "FIPS": "17191", "County": "Wayne", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62820", "FIPS": "17193", "County": "White", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62821", "FIPS": "17193", "County": "White", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62827", "FIPS": "17193", "County": "White", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62834", "FIPS": "17193", "County": "White", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62835", "FIPS": "17193", "County": "White", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62844", "FIPS": "17193", "County": "White", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62861", "FIPS": "17193", "County": "White", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62862", "FIPS": "17193", "County": "White", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62869", "FIPS": "17193", "County": "White", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "62887", "FIPS": "17193", "County": "White", "State": "IL", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "61037", "FIPS": "17195", "County": "Whiteside", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61071", "FIPS": "17195", "County": "Whiteside", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61081", "FIPS": "17195", "County": "Whiteside", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61230", "FIPS": "17195", "County": "Whiteside", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61243", "FIPS": "17195", "County": "Whiteside", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61250", "FIPS": "17195", "County": "Whiteside", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61251", "FIPS": "17195", "County": "Whiteside", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61252", "FIPS": "17195", "County": "Whiteside", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61261", "FIPS": "17195", "County": "Whiteside", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61270", "FIPS": "17195", "County": "Whiteside", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61277", "FIPS": "17195", "County": "Whiteside", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "61283", "FIPS": "17195", "County": "Whiteside", "State": "IL", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "60401", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60403", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60404", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60408", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60410", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60417", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60421", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60423", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60431", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60432", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60433", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60434", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60435", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60436", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60440", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60441", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60442", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60446", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60448", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60449", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60451", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60468", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60481", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60490", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60491", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60544", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60564", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60585", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "60586", "FIPS": "17197", "County": "Will", "State": "IL", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "62841", "FIPS": "17199", "County": "Williamson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62915", "FIPS": "17199", "County": "Williamson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62918", "FIPS": "17199", "County": "Williamson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62921", "FIPS": "17199", "County": "Williamson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62922", "FIPS": "17199", "County": "Williamson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62933", "FIPS": "17199", "County": "Williamson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62948", "FIPS": "17199", "County": "Williamson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62949", "FIPS": "17199", "County": "Williamson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62951", "FIPS": "17199", "County": "Williamson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62959", "FIPS": "17199", "County": "Williamson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "62974", "FIPS": "17199", "County": "Williamson", "State": "IL", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "61016", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61024", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61063", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61072", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61073", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61077", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61079", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61080", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61088", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61101", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61102", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61103", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61104", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61105", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61106", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61107", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61108", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61109", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61110", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61111", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61112", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61114", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61115", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61125", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61126", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61130", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61131", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61132", "FIPS": "17201", "County": "Winnebago", "State": "IL", "DMA": {"Code": 610, "Name": "ROCKFORD"}},
  {"Postal_Code": "61516", "FIPS": "17203", "County": "Woodford", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61530", "FIPS": "17203", "County": "Woodford", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61545", "FIPS": "17203", "County": "Woodford", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61548", "FIPS": "17203", "County": "Woodford", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61561", "FIPS": "17203", "County": "Woodford", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61729", "FIPS": "17203", "County": "Woodford", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61738", "FIPS": "17203", "County": "Woodford", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61742", "FIPS": "17203", "County": "Woodford", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61760", "FIPS": "17203", "County": "Woodford", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "61771", "FIPS": "17203", "County": "Woodford", "State": "IL", "DMA": {"Code": 675, "Name": "PEORIA-BLOOMINGTON"}},
  {"Postal_Code": "46711", "FIPS": "18001", "County": "Adams", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46733", "FIPS": "18001", "County": "Adams", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46740", "FIPS": "18001", "County": "Adams", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46769", "FIPS": "18001", "County": "Adams", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46772", "FIPS": "18001", "County": "Adams", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46780", "FIPS": "18001", "County": "Adams", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46782", "FIPS": "18001", "County": "Adams", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46704", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46741", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46743", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46745", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46748", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46765", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46773", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46774", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46788", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46797", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46798", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46799", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46801", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46802", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46803", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46804", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46805", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46806", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46807", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46808", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46809", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46814", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46815", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46816", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46818", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46819", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46825", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46835", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46845", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46850", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46851", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46852", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46853", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46854", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46855", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46856", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46857", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46858", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46859", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46860", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46861", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46862", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46863", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46864", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46865", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46866", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46867", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46868", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46869", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46885", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46895", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46896", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46897", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46898", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46899", "FIPS": "18003", "County": "Allen", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "47201", "FIPS": "18005", "County": "Bartholomew", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47202", "FIPS": "18005", "County": "Bartholomew", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47203", "FIPS": "18005", "County": "Bartholomew", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47226", "FIPS": "18005", "County": "Bartholomew", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47232", "FIPS": "18005", "County": "Bartholomew", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47236", "FIPS": "18005", "County": "Bartholomew", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47244", "FIPS": "18005", "County": "Bartholomew", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47246", "FIPS": "18005", "County": "Bartholomew", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47247", "FIPS": "18005", "County": "Bartholomew", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47280", "FIPS": "18005", "County": "Bartholomew", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47917", "FIPS": "18007", "County": "Benton", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47921", "FIPS": "18007", "County": "Benton", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47942", "FIPS": "18007", "County": "Benton", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47944", "FIPS": "18007", "County": "Benton", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47970", "FIPS": "18007", "County": "Benton", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47971", "FIPS": "18007", "County": "Benton", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47984", "FIPS": "18007", "County": "Benton", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47986", "FIPS": "18007", "County": "Benton", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47348", "FIPS": "18009", "County": "Blackford", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47359", "FIPS": "18009", "County": "Blackford", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46052", "FIPS": "18011", "County": "Boone", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46071", "FIPS": "18011", "County": "Boone", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46075", "FIPS": "18011", "County": "Boone", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46077", "FIPS": "18011", "County": "Boone", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46102", "FIPS": "18011", "County": "Boone", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46147", "FIPS": "18011", "County": "Boone", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46160", "FIPS": "18013", "County": "Brown", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47435", "FIPS": "18013", "County": "Brown", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47448", "FIPS": "18013", "County": "Brown", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46913", "FIPS": "18015", "County": "Carroll", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46915", "FIPS": "18015", "County": "Carroll", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46916", "FIPS": "18015", "County": "Carroll", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46917", "FIPS": "18015", "County": "Carroll", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46920", "FIPS": "18015", "County": "Carroll", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46923", "FIPS": "18015", "County": "Carroll", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46929", "FIPS": "18015", "County": "Carroll", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46977", "FIPS": "18015", "County": "Carroll", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47997", "FIPS": "18015", "County": "Carroll", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46932", "FIPS": "18017", "County": "Cass", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46942", "FIPS": "18017", "County": "Cass", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46947", "FIPS": "18017", "County": "Cass", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46950", "FIPS": "18017", "County": "Cass", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46961", "FIPS": "18017", "County": "Cass", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46967", "FIPS": "18017", "County": "Cass", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46978", "FIPS": "18017", "County": "Cass", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46988", "FIPS": "18017", "County": "Cass", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46994", "FIPS": "18017", "County": "Cass", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46998", "FIPS": "18017", "County": "Cass", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47104", "FIPS": "18019", "County": "Clark", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47106", "FIPS": "18019", "County": "Clark", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47111", "FIPS": "18019", "County": "Clark", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47126", "FIPS": "18019", "County": "Clark", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47129", "FIPS": "18019", "County": "Clark", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47130", "FIPS": "18019", "County": "Clark", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47131", "FIPS": "18019", "County": "Clark", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47132", "FIPS": "18019", "County": "Clark", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47133", "FIPS": "18019", "County": "Clark", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47134", "FIPS": "18019", "County": "Clark", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47141", "FIPS": "18019", "County": "Clark", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47143", "FIPS": "18019", "County": "Clark", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47144", "FIPS": "18019", "County": "Clark", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47147", "FIPS": "18019", "County": "Clark", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47162", "FIPS": "18019", "County": "Clark", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47163", "FIPS": "18019", "County": "Clark", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47172", "FIPS": "18019", "County": "Clark", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47190", "FIPS": "18019", "County": "Clark", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47199", "FIPS": "18019", "County": "Clark", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47833", "FIPS": "18021", "County": "Clay", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47834", "FIPS": "18021", "County": "Clay", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47837", "FIPS": "18021", "County": "Clay", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47840", "FIPS": "18021", "County": "Clay", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47841", "FIPS": "18021", "County": "Clay", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47845", "FIPS": "18021", "County": "Clay", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47846", "FIPS": "18021", "County": "Clay", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47853", "FIPS": "18021", "County": "Clay", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47857", "FIPS": "18021", "County": "Clay", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47881", "FIPS": "18021", "County": "Clay", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "46035", "FIPS": "18023", "County": "Clinton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46039", "FIPS": "18023", "County": "Clinton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46041", "FIPS": "18023", "County": "Clinton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46050", "FIPS": "18023", "County": "Clinton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46057", "FIPS": "18023", "County": "Clinton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46058", "FIPS": "18023", "County": "Clinton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46065", "FIPS": "18023", "County": "Clinton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46067", "FIPS": "18023", "County": "Clinton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47116", "FIPS": "18025", "County": "Crawford", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47118", "FIPS": "18025", "County": "Crawford", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47123", "FIPS": "18025", "County": "Crawford", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47137", "FIPS": "18025", "County": "Crawford", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47140", "FIPS": "18025", "County": "Crawford", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47145", "FIPS": "18025", "County": "Crawford", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47174", "FIPS": "18025", "County": "Crawford", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47175", "FIPS": "18025", "County": "Crawford", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47501", "FIPS": "18027", "County": "Daviess", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47519", "FIPS": "18027", "County": "Daviess", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47529", "FIPS": "18027", "County": "Daviess", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47558", "FIPS": "18027", "County": "Daviess", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47562", "FIPS": "18027", "County": "Daviess", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47568", "FIPS": "18027", "County": "Daviess", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47001", "FIPS": "18029", "County": "Dearborn", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47018", "FIPS": "18029", "County": "Dearborn", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47022", "FIPS": "18029", "County": "Dearborn", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47025", "FIPS": "18029", "County": "Dearborn", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47032", "FIPS": "18029", "County": "Dearborn", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47060", "FIPS": "18029", "County": "Dearborn", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47225", "FIPS": "18031", "County": "Decatur", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47240", "FIPS": "18031", "County": "Decatur", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47261", "FIPS": "18031", "County": "Decatur", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47263", "FIPS": "18031", "County": "Decatur", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47272", "FIPS": "18031", "County": "Decatur", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47283", "FIPS": "18031", "County": "Decatur", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46705", "FIPS": "18033", "County": "De Kalb", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46706", "FIPS": "18033", "County": "De Kalb", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46721", "FIPS": "18033", "County": "De Kalb", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46730", "FIPS": "18033", "County": "De Kalb", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46738", "FIPS": "18033", "County": "De Kalb", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46785", "FIPS": "18033", "County": "De Kalb", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46793", "FIPS": "18033", "County": "De Kalb", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "47302", "FIPS": "18035", "County": "Delaware", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47303", "FIPS": "18035", "County": "Delaware", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47304", "FIPS": "18035", "County": "Delaware", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47305", "FIPS": "18035", "County": "Delaware", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47306", "FIPS": "18035", "County": "Delaware", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47307", "FIPS": "18035", "County": "Delaware", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47308", "FIPS": "18035", "County": "Delaware", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47320", "FIPS": "18035", "County": "Delaware", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47334", "FIPS": "18035", "County": "Delaware", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47338", "FIPS": "18035", "County": "Delaware", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47342", "FIPS": "18035", "County": "Delaware", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47367", "FIPS": "18035", "County": "Delaware", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47383", "FIPS": "18035", "County": "Delaware", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47396", "FIPS": "18035", "County": "Delaware", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47513", "FIPS": "18037", "County": "Dubois", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47521", "FIPS": "18037", "County": "Dubois", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47527", "FIPS": "18037", "County": "Dubois", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47532", "FIPS": "18037", "County": "Dubois", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47541", "FIPS": "18037", "County": "Dubois", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47542", "FIPS": "18037", "County": "Dubois", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47545", "FIPS": "18037", "County": "Dubois", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47546", "FIPS": "18037", "County": "Dubois", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47547", "FIPS": "18037", "County": "Dubois", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47549", "FIPS": "18037", "County": "Dubois", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47575", "FIPS": "18037", "County": "Dubois", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47580", "FIPS": "18037", "County": "Dubois", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "46507", "FIPS": "18039", "County": "Elkhart", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46514", "FIPS": "18039", "County": "Elkhart", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46515", "FIPS": "18039", "County": "Elkhart", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46516", "FIPS": "18039", "County": "Elkhart", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46517", "FIPS": "18039", "County": "Elkhart", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46526", "FIPS": "18039", "County": "Elkhart", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46527", "FIPS": "18039", "County": "Elkhart", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46528", "FIPS": "18039", "County": "Elkhart", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46540", "FIPS": "18039", "County": "Elkhart", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46543", "FIPS": "18039", "County": "Elkhart", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46550", "FIPS": "18039", "County": "Elkhart", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46553", "FIPS": "18039", "County": "Elkhart", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46573", "FIPS": "18039", "County": "Elkhart", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46133", "FIPS": "18041", "County": "Fayette", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47322", "FIPS": "18041", "County": "Fayette", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47331", "FIPS": "18041", "County": "Fayette", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47119", "FIPS": "18043", "County": "Floyd", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47122", "FIPS": "18043", "County": "Floyd", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47124", "FIPS": "18043", "County": "Floyd", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47146", "FIPS": "18043", "County": "Floyd", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47150", "FIPS": "18043", "County": "Floyd", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47151", "FIPS": "18043", "County": "Floyd", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47918", "FIPS": "18045", "County": "Fountain", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47932", "FIPS": "18045", "County": "Fountain", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47949", "FIPS": "18045", "County": "Fountain", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47952", "FIPS": "18045", "County": "Fountain", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47958", "FIPS": "18045", "County": "Fountain", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47969", "FIPS": "18045", "County": "Fountain", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47987", "FIPS": "18045", "County": "Fountain", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47988", "FIPS": "18045", "County": "Fountain", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47006", "FIPS": "18047", "County": "Franklin", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47010", "FIPS": "18047", "County": "Franklin", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47012", "FIPS": "18047", "County": "Franklin", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47016", "FIPS": "18047", "County": "Franklin", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47024", "FIPS": "18047", "County": "Franklin", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47030", "FIPS": "18047", "County": "Franklin", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47035", "FIPS": "18047", "County": "Franklin", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47036", "FIPS": "18047", "County": "Franklin", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "46910", "FIPS": "18049", "County": "Fulton", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46912", "FIPS": "18049", "County": "Fulton", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46922", "FIPS": "18049", "County": "Fulton", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46931", "FIPS": "18049", "County": "Fulton", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46935", "FIPS": "18049", "County": "Fulton", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46939", "FIPS": "18049", "County": "Fulton", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46945", "FIPS": "18049", "County": "Fulton", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46975", "FIPS": "18049", "County": "Fulton", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "47639", "FIPS": "18051", "County": "Gibson", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47640", "FIPS": "18051", "County": "Gibson", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47647", "FIPS": "18051", "County": "Gibson", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47648", "FIPS": "18051", "County": "Gibson", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47649", "FIPS": "18051", "County": "Gibson", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47654", "FIPS": "18051", "County": "Gibson", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47660", "FIPS": "18051", "County": "Gibson", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47665", "FIPS": "18051", "County": "Gibson", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47666", "FIPS": "18051", "County": "Gibson", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47670", "FIPS": "18051", "County": "Gibson", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47683", "FIPS": "18051", "County": "Gibson", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "46928", "FIPS": "18053", "County": "Grant", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46930", "FIPS": "18053", "County": "Grant", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46933", "FIPS": "18053", "County": "Grant", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46938", "FIPS": "18053", "County": "Grant", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46952", "FIPS": "18053", "County": "Grant", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46953", "FIPS": "18053", "County": "Grant", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46957", "FIPS": "18053", "County": "Grant", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46986", "FIPS": "18053", "County": "Grant", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46987", "FIPS": "18053", "County": "Grant", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46989", "FIPS": "18053", "County": "Grant", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46991", "FIPS": "18053", "County": "Grant", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47424", "FIPS": "18055", "County": "Greene", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47438", "FIPS": "18055", "County": "Greene", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47439", "FIPS": "18055", "County": "Greene", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47441", "FIPS": "18055", "County": "Greene", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47443", "FIPS": "18055", "County": "Greene", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47445", "FIPS": "18055", "County": "Greene", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47449", "FIPS": "18055", "County": "Greene", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47453", "FIPS": "18055", "County": "Greene", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47457", "FIPS": "18055", "County": "Greene", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47459", "FIPS": "18055", "County": "Greene", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47465", "FIPS": "18055", "County": "Greene", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47471", "FIPS": "18055", "County": "Greene", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "46030", "FIPS": "18057", "County": "Hamilton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46031", "FIPS": "18057", "County": "Hamilton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46032", "FIPS": "18057", "County": "Hamilton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46033", "FIPS": "18057", "County": "Hamilton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46034", "FIPS": "18057", "County": "Hamilton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46037", "FIPS": "18057", "County": "Hamilton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46038", "FIPS": "18057", "County": "Hamilton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46060", "FIPS": "18057", "County": "Hamilton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46061", "FIPS": "18057", "County": "Hamilton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46062", "FIPS": "18057", "County": "Hamilton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46069", "FIPS": "18057", "County": "Hamilton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46074", "FIPS": "18057", "County": "Hamilton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46082", "FIPS": "18057", "County": "Hamilton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46280", "FIPS": "18057", "County": "Hamilton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46290", "FIPS": "18057", "County": "Hamilton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46040", "FIPS": "18059", "County": "Hancock", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46055", "FIPS": "18059", "County": "Hancock", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46117", "FIPS": "18059", "County": "Hancock", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46129", "FIPS": "18059", "County": "Hancock", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46140", "FIPS": "18059", "County": "Hancock", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46154", "FIPS": "18059", "County": "Hancock", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46163", "FIPS": "18059", "County": "Hancock", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46186", "FIPS": "18059", "County": "Hancock", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47107", "FIPS": "18061", "County": "Harrison", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47110", "FIPS": "18061", "County": "Harrison", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47112", "FIPS": "18061", "County": "Harrison", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47114", "FIPS": "18061", "County": "Harrison", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47115", "FIPS": "18061", "County": "Harrison", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47117", "FIPS": "18061", "County": "Harrison", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47135", "FIPS": "18061", "County": "Harrison", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47136", "FIPS": "18061", "County": "Harrison", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47142", "FIPS": "18061", "County": "Harrison", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47160", "FIPS": "18061", "County": "Harrison", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47161", "FIPS": "18061", "County": "Harrison", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47164", "FIPS": "18061", "County": "Harrison", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47166", "FIPS": "18061", "County": "Harrison", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "46103", "FIPS": "18063", "County": "Hendricks", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46112", "FIPS": "18063", "County": "Hendricks", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46118", "FIPS": "18063", "County": "Hendricks", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46122", "FIPS": "18063", "County": "Hendricks", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46123", "FIPS": "18063", "County": "Hendricks", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46149", "FIPS": "18063", "County": "Hendricks", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46165", "FIPS": "18063", "County": "Hendricks", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46167", "FIPS": "18063", "County": "Hendricks", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46168", "FIPS": "18063", "County": "Hendricks", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46180", "FIPS": "18063", "County": "Hendricks", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46148", "FIPS": "18065", "County": "Henry", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47337", "FIPS": "18065", "County": "Henry", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47344", "FIPS": "18065", "County": "Henry", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47351", "FIPS": "18065", "County": "Henry", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47352", "FIPS": "18065", "County": "Henry", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47356", "FIPS": "18065", "County": "Henry", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47360", "FIPS": "18065", "County": "Henry", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47361", "FIPS": "18065", "County": "Henry", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47362", "FIPS": "18065", "County": "Henry", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47366", "FIPS": "18065", "County": "Henry", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47384", "FIPS": "18065", "County": "Henry", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47385", "FIPS": "18065", "County": "Henry", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47386", "FIPS": "18065", "County": "Henry", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47387", "FIPS": "18065", "County": "Henry", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47388", "FIPS": "18065", "County": "Henry", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46901", "FIPS": "18067", "County": "Howard", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46902", "FIPS": "18067", "County": "Howard", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46903", "FIPS": "18067", "County": "Howard", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46904", "FIPS": "18067", "County": "Howard", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46936", "FIPS": "18067", "County": "Howard", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46937", "FIPS": "18067", "County": "Howard", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46965", "FIPS": "18067", "County": "Howard", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46979", "FIPS": "18067", "County": "Howard", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46995", "FIPS": "18067", "County": "Howard", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46702", "FIPS": "18069", "County": "Huntington", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46713", "FIPS": "18069", "County": "Huntington", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46750", "FIPS": "18069", "County": "Huntington", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46783", "FIPS": "18069", "County": "Huntington", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46792", "FIPS": "18069", "County": "Huntington", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "47220", "FIPS": "18071", "County": "Jackson", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47228", "FIPS": "18071", "County": "Jackson", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47229", "FIPS": "18071", "County": "Jackson", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47235", "FIPS": "18071", "County": "Jackson", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47249", "FIPS": "18071", "County": "Jackson", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47260", "FIPS": "18071", "County": "Jackson", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47264", "FIPS": "18071", "County": "Jackson", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47274", "FIPS": "18071", "County": "Jackson", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47281", "FIPS": "18071", "County": "Jackson", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "46310", "FIPS": "18073", "County": "Jasper", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46380", "FIPS": "18073", "County": "Jasper", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46392", "FIPS": "18073", "County": "Jasper", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "47943", "FIPS": "18073", "County": "Jasper", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "47977", "FIPS": "18073", "County": "Jasper", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "47978", "FIPS": "18073", "County": "Jasper", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "47326", "FIPS": "18075", "County": "Jay", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "47336", "FIPS": "18075", "County": "Jay", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "47369", "FIPS": "18075", "County": "Jay", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "47371", "FIPS": "18075", "County": "Jay", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "47373", "FIPS": "18075", "County": "Jay", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "47381", "FIPS": "18075", "County": "Jay", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "47224", "FIPS": "18077", "County": "Jefferson", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47230", "FIPS": "18077", "County": "Jefferson", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47231", "FIPS": "18077", "County": "Jefferson", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47243", "FIPS": "18077", "County": "Jefferson", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47250", "FIPS": "18077", "County": "Jefferson", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47223", "FIPS": "18079", "County": "Jennings", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47227", "FIPS": "18079", "County": "Jennings", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47245", "FIPS": "18079", "County": "Jennings", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47265", "FIPS": "18079", "County": "Jennings", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47270", "FIPS": "18079", "County": "Jennings", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47273", "FIPS": "18079", "County": "Jennings", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47282", "FIPS": "18079", "County": "Jennings", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "46106", "FIPS": "18081", "County": "Johnson", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46124", "FIPS": "18081", "County": "Johnson", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46131", "FIPS": "18081", "County": "Johnson", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46142", "FIPS": "18081", "County": "Johnson", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46143", "FIPS": "18081", "County": "Johnson", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46162", "FIPS": "18081", "County": "Johnson", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46164", "FIPS": "18081", "County": "Johnson", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46181", "FIPS": "18081", "County": "Johnson", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46184", "FIPS": "18081", "County": "Johnson", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47512", "FIPS": "18083", "County": "Knox", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47516", "FIPS": "18083", "County": "Knox", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47524", "FIPS": "18083", "County": "Knox", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47528", "FIPS": "18083", "County": "Knox", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47535", "FIPS": "18083", "County": "Knox", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47557", "FIPS": "18083", "County": "Knox", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47561", "FIPS": "18083", "County": "Knox", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47573", "FIPS": "18083", "County": "Knox", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47578", "FIPS": "18083", "County": "Knox", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47591", "FIPS": "18083", "County": "Knox", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47596", "FIPS": "18083", "County": "Knox", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47597", "FIPS": "18083", "County": "Knox", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "46502", "FIPS": "18085", "County": "Kosciusko", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46508", "FIPS": "18085", "County": "Kosciusko", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46510", "FIPS": "18085", "County": "Kosciusko", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46524", "FIPS": "18085", "County": "Kosciusko", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46538", "FIPS": "18085", "County": "Kosciusko", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46539", "FIPS": "18085", "County": "Kosciusko", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46542", "FIPS": "18085", "County": "Kosciusko", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46555", "FIPS": "18085", "County": "Kosciusko", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46562", "FIPS": "18085", "County": "Kosciusko", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46567", "FIPS": "18085", "County": "Kosciusko", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46580", "FIPS": "18085", "County": "Kosciusko", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46581", "FIPS": "18085", "County": "Kosciusko", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46582", "FIPS": "18085", "County": "Kosciusko", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46590", "FIPS": "18085", "County": "Kosciusko", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46982", "FIPS": "18085", "County": "Kosciusko", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46565", "FIPS": "18087", "County": "Lagrange", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46571", "FIPS": "18087", "County": "Lagrange", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46746", "FIPS": "18087", "County": "Lagrange", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46761", "FIPS": "18087", "County": "Lagrange", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46771", "FIPS": "18087", "County": "Lagrange", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46786", "FIPS": "18087", "County": "Lagrange", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46789", "FIPS": "18087", "County": "Lagrange", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46795", "FIPS": "18087", "County": "Lagrange", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46303", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46307", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46308", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46311", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46312", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46319", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46320", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46321", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46322", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46323", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46324", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46325", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46327", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46342", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46355", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46356", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46373", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46375", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46376", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46377", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46394", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46401", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46402", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46403", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46404", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46405", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46406", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46407", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46408", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46409", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46410", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46411", "FIPS": "18089", "County": "Lake", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46340", "FIPS": "18091", "County": "La Porte", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46345", "FIPS": "18091", "County": "La Porte", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46346", "FIPS": "18091", "County": "La Porte", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46348", "FIPS": "18091", "County": "La Porte", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46350", "FIPS": "18091", "County": "La Porte", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46352", "FIPS": "18091", "County": "La Porte", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46360", "FIPS": "18091", "County": "La Porte", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46361", "FIPS": "18091", "County": "La Porte", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46365", "FIPS": "18091", "County": "La Porte", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46371", "FIPS": "18091", "County": "La Porte", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46382", "FIPS": "18091", "County": "La Porte", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46390", "FIPS": "18091", "County": "La Porte", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46391", "FIPS": "18091", "County": "La Porte", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "47420", "FIPS": "18093", "County": "Lawrence", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47421", "FIPS": "18093", "County": "Lawrence", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47430", "FIPS": "18093", "County": "Lawrence", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47436", "FIPS": "18093", "County": "Lawrence", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47437", "FIPS": "18093", "County": "Lawrence", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47446", "FIPS": "18093", "County": "Lawrence", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47451", "FIPS": "18093", "County": "Lawrence", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47462", "FIPS": "18093", "County": "Lawrence", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47467", "FIPS": "18093", "County": "Lawrence", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47470", "FIPS": "18093", "County": "Lawrence", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46001", "FIPS": "18095", "County": "Madison", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46011", "FIPS": "18095", "County": "Madison", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46012", "FIPS": "18095", "County": "Madison", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46013", "FIPS": "18095", "County": "Madison", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46014", "FIPS": "18095", "County": "Madison", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46015", "FIPS": "18095", "County": "Madison", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46016", "FIPS": "18095", "County": "Madison", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46017", "FIPS": "18095", "County": "Madison", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46018", "FIPS": "18095", "County": "Madison", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46036", "FIPS": "18095", "County": "Madison", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46044", "FIPS": "18095", "County": "Madison", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46048", "FIPS": "18095", "County": "Madison", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46051", "FIPS": "18095", "County": "Madison", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46056", "FIPS": "18095", "County": "Madison", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46063", "FIPS": "18095", "County": "Madison", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46064", "FIPS": "18095", "County": "Madison", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46070", "FIPS": "18095", "County": "Madison", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46107", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46183", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46201", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46202", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46203", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46204", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46205", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46206", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46207", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46208", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46209", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46211", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46214", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46216", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46217", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46218", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46219", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46220", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46221", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46222", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46223", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46224", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46225", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46226", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46227", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46228", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46229", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46230", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46231", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46234", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46235", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46236", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46237", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46239", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46240", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46241", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46242", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46244", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46247", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46249", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46250", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46251", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46253", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46254", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46255", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46256", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46259", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46260", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46266", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46268", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46274", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46275", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46277", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46278", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46282", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46283", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46285", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46291", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46295", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46296", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46298", "FIPS": "18097", "County": "Marion", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46501", "FIPS": "18099", "County": "Marshall", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46504", "FIPS": "18099", "County": "Marshall", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46506", "FIPS": "18099", "County": "Marshall", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46511", "FIPS": "18099", "County": "Marshall", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46513", "FIPS": "18099", "County": "Marshall", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46537", "FIPS": "18099", "County": "Marshall", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46563", "FIPS": "18099", "County": "Marshall", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46570", "FIPS": "18099", "County": "Marshall", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46572", "FIPS": "18099", "County": "Marshall", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "47522", "FIPS": "18101", "County": "Martin", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47553", "FIPS": "18101", "County": "Martin", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47581", "FIPS": "18101", "County": "Martin", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "46911", "FIPS": "18103", "County": "Miami", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46914", "FIPS": "18103", "County": "Miami", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46919", "FIPS": "18103", "County": "Miami", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46921", "FIPS": "18103", "County": "Miami", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46926", "FIPS": "18103", "County": "Miami", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46951", "FIPS": "18103", "County": "Miami", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46958", "FIPS": "18103", "County": "Miami", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46959", "FIPS": "18103", "County": "Miami", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46970", "FIPS": "18103", "County": "Miami", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46971", "FIPS": "18103", "County": "Miami", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47401", "FIPS": "18105", "County": "Monroe", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47402", "FIPS": "18105", "County": "Monroe", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47403", "FIPS": "18105", "County": "Monroe", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47404", "FIPS": "18105", "County": "Monroe", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47405", "FIPS": "18105", "County": "Monroe", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47406", "FIPS": "18105", "County": "Monroe", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47407", "FIPS": "18105", "County": "Monroe", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47408", "FIPS": "18105", "County": "Monroe", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47426", "FIPS": "18105", "County": "Monroe", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47429", "FIPS": "18105", "County": "Monroe", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47434", "FIPS": "18105", "County": "Monroe", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47458", "FIPS": "18105", "County": "Monroe", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47463", "FIPS": "18105", "County": "Monroe", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47464", "FIPS": "18105", "County": "Monroe", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47468", "FIPS": "18105", "County": "Monroe", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47490", "FIPS": "18105", "County": "Monroe", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47916", "FIPS": "18107", "County": "Montgomery", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47933", "FIPS": "18107", "County": "Montgomery", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47934", "FIPS": "18107", "County": "Montgomery", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47935", "FIPS": "18107", "County": "Montgomery", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47936", "FIPS": "18107", "County": "Montgomery", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47937", "FIPS": "18107", "County": "Montgomery", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47938", "FIPS": "18107", "County": "Montgomery", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47939", "FIPS": "18107", "County": "Montgomery", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47940", "FIPS": "18107", "County": "Montgomery", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47954", "FIPS": "18107", "County": "Montgomery", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47955", "FIPS": "18107", "County": "Montgomery", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47965", "FIPS": "18107", "County": "Montgomery", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47967", "FIPS": "18107", "County": "Montgomery", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47968", "FIPS": "18107", "County": "Montgomery", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47989", "FIPS": "18107", "County": "Montgomery", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47990", "FIPS": "18107", "County": "Montgomery", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47994", "FIPS": "18107", "County": "Montgomery", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46111", "FIPS": "18109", "County": "Morgan", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46113", "FIPS": "18109", "County": "Morgan", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46125", "FIPS": "18109", "County": "Morgan", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46151", "FIPS": "18109", "County": "Morgan", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46157", "FIPS": "18109", "County": "Morgan", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46158", "FIPS": "18109", "County": "Morgan", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46166", "FIPS": "18109", "County": "Morgan", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46349", "FIPS": "18111", "County": "Newton", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46372", "FIPS": "18111", "County": "Newton", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46379", "FIPS": "18111", "County": "Newton", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46381", "FIPS": "18111", "County": "Newton", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "47922", "FIPS": "18111", "County": "Newton", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "47948", "FIPS": "18111", "County": "Newton", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "47951", "FIPS": "18111", "County": "Newton", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "47963", "FIPS": "18111", "County": "Newton", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "47964", "FIPS": "18111", "County": "Newton", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46701", "FIPS": "18113", "County": "Noble", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46710", "FIPS": "18113", "County": "Noble", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46732", "FIPS": "18113", "County": "Noble", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46755", "FIPS": "18113", "County": "Noble", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46760", "FIPS": "18113", "County": "Noble", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46763", "FIPS": "18113", "County": "Noble", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46767", "FIPS": "18113", "County": "Noble", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46784", "FIPS": "18113", "County": "Noble", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46794", "FIPS": "18113", "County": "Noble", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46796", "FIPS": "18113", "County": "Noble", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "47040", "FIPS": "18115", "County": "Ohio", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47432", "FIPS": "18117", "County": "Orange", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47452", "FIPS": "18117", "County": "Orange", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47454", "FIPS": "18117", "County": "Orange", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47469", "FIPS": "18117", "County": "Orange", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47427", "FIPS": "18119", "County": "Owen", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47431", "FIPS": "18119", "County": "Owen", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47433", "FIPS": "18119", "County": "Owen", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47455", "FIPS": "18119", "County": "Owen", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47456", "FIPS": "18119", "County": "Owen", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47460", "FIPS": "18119", "County": "Owen", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47868", "FIPS": "18119", "County": "Owen", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47830", "FIPS": "18121", "County": "Parke", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47832", "FIPS": "18121", "County": "Parke", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47836", "FIPS": "18121", "County": "Parke", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47856", "FIPS": "18121", "County": "Parke", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47859", "FIPS": "18121", "County": "Parke", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47860", "FIPS": "18121", "County": "Parke", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47862", "FIPS": "18121", "County": "Parke", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47872", "FIPS": "18121", "County": "Parke", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47874", "FIPS": "18121", "County": "Parke", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47514", "FIPS": "18123", "County": "Perry", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47515", "FIPS": "18123", "County": "Perry", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47520", "FIPS": "18123", "County": "Perry", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47525", "FIPS": "18123", "County": "Perry", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47551", "FIPS": "18123", "County": "Perry", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47574", "FIPS": "18123", "County": "Perry", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47576", "FIPS": "18123", "County": "Perry", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47586", "FIPS": "18123", "County": "Perry", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47564", "FIPS": "18125", "County": "Pike", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47567", "FIPS": "18125", "County": "Pike", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47584", "FIPS": "18125", "County": "Pike", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47585", "FIPS": "18125", "County": "Pike", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47590", "FIPS": "18125", "County": "Pike", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47598", "FIPS": "18125", "County": "Pike", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "46301", "FIPS": "18127", "County": "Porter", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46302", "FIPS": "18127", "County": "Porter", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46304", "FIPS": "18127", "County": "Porter", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46341", "FIPS": "18127", "County": "Porter", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46347", "FIPS": "18127", "County": "Porter", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46368", "FIPS": "18127", "County": "Porter", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46383", "FIPS": "18127", "County": "Porter", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46384", "FIPS": "18127", "County": "Porter", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46385", "FIPS": "18127", "County": "Porter", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "46393", "FIPS": "18127", "County": "Porter", "State": "IN", "DMA": {"Code": 602, "Name": "CHICAGO"}},
  {"Postal_Code": "47612", "FIPS": "18129", "County": "Posey", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47616", "FIPS": "18129", "County": "Posey", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47620", "FIPS": "18129", "County": "Posey", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47631", "FIPS": "18129", "County": "Posey", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47633", "FIPS": "18129", "County": "Posey", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47638", "FIPS": "18129", "County": "Posey", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "46960", "FIPS": "18131", "County": "Pulaski", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46985", "FIPS": "18131", "County": "Pulaski", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46996", "FIPS": "18131", "County": "Pulaski", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "47946", "FIPS": "18131", "County": "Pulaski", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "47957", "FIPS": "18131", "County": "Pulaski", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46105", "FIPS": "18133", "County": "Putnam", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46120", "FIPS": "18133", "County": "Putnam", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46121", "FIPS": "18133", "County": "Putnam", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46128", "FIPS": "18133", "County": "Putnam", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46135", "FIPS": "18133", "County": "Putnam", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46170", "FIPS": "18133", "County": "Putnam", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46171", "FIPS": "18133", "County": "Putnam", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46172", "FIPS": "18133", "County": "Putnam", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46175", "FIPS": "18133", "County": "Putnam", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47340", "FIPS": "18135", "County": "Randolph", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47354", "FIPS": "18135", "County": "Randolph", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47355", "FIPS": "18135", "County": "Randolph", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47358", "FIPS": "18135", "County": "Randolph", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47368", "FIPS": "18135", "County": "Randolph", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47380", "FIPS": "18135", "County": "Randolph", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47382", "FIPS": "18135", "County": "Randolph", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47390", "FIPS": "18135", "County": "Randolph", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47394", "FIPS": "18135", "County": "Randolph", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47017", "FIPS": "18137", "County": "Ripley", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47021", "FIPS": "18137", "County": "Ripley", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47023", "FIPS": "18137", "County": "Ripley", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47031", "FIPS": "18137", "County": "Ripley", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47033", "FIPS": "18137", "County": "Ripley", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47034", "FIPS": "18137", "County": "Ripley", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47037", "FIPS": "18137", "County": "Ripley", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47039", "FIPS": "18137", "County": "Ripley", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47041", "FIPS": "18137", "County": "Ripley", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47042", "FIPS": "18137", "County": "Ripley", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "46104", "FIPS": "18139", "County": "Rush", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46115", "FIPS": "18139", "County": "Rush", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46127", "FIPS": "18139", "County": "Rush", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46146", "FIPS": "18139", "County": "Rush", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46150", "FIPS": "18139", "County": "Rush", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46155", "FIPS": "18139", "County": "Rush", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46156", "FIPS": "18139", "County": "Rush", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46173", "FIPS": "18139", "County": "Rush", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46530", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46536", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46544", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46545", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46546", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46552", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46554", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46556", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46561", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46574", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46595", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46601", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46604", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46613", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46614", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46615", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46616", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46617", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46619", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46620", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46624", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46626", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46628", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46629", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46634", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46635", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46637", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46660", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46680", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46699", "FIPS": "18141", "County": "St Joseph", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "47102", "FIPS": "18143", "County": "Scott", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47138", "FIPS": "18143", "County": "Scott", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47170", "FIPS": "18143", "County": "Scott", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47177", "FIPS": "18143", "County": "Scott", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "46110", "FIPS": "18145", "County": "Shelby", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46126", "FIPS": "18145", "County": "Shelby", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46130", "FIPS": "18145", "County": "Shelby", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46144", "FIPS": "18145", "County": "Shelby", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46161", "FIPS": "18145", "County": "Shelby", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46176", "FIPS": "18145", "County": "Shelby", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46182", "FIPS": "18145", "County": "Shelby", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47234", "FIPS": "18145", "County": "Shelby", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47523", "FIPS": "18147", "County": "Spencer", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47531", "FIPS": "18147", "County": "Spencer", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47536", "FIPS": "18147", "County": "Spencer", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47537", "FIPS": "18147", "County": "Spencer", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47550", "FIPS": "18147", "County": "Spencer", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47552", "FIPS": "18147", "County": "Spencer", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47556", "FIPS": "18147", "County": "Spencer", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47577", "FIPS": "18147", "County": "Spencer", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47579", "FIPS": "18147", "County": "Spencer", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47588", "FIPS": "18147", "County": "Spencer", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47611", "FIPS": "18147", "County": "Spencer", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47615", "FIPS": "18147", "County": "Spencer", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47617", "FIPS": "18147", "County": "Spencer", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47634", "FIPS": "18147", "County": "Spencer", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47635", "FIPS": "18147", "County": "Spencer", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "46366", "FIPS": "18149", "County": "Starke", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46374", "FIPS": "18149", "County": "Starke", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46531", "FIPS": "18149", "County": "Starke", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46532", "FIPS": "18149", "County": "Starke", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46534", "FIPS": "18149", "County": "Starke", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46968", "FIPS": "18149", "County": "Starke", "State": "IN", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "46703", "FIPS": "18151", "County": "Steuben", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46737", "FIPS": "18151", "County": "Steuben", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46742", "FIPS": "18151", "County": "Steuben", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46747", "FIPS": "18151", "County": "Steuben", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46776", "FIPS": "18151", "County": "Steuben", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46779", "FIPS": "18151", "County": "Steuben", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "47838", "FIPS": "18153", "County": "Sullivan", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47848", "FIPS": "18153", "County": "Sullivan", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47849", "FIPS": "18153", "County": "Sullivan", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47850", "FIPS": "18153", "County": "Sullivan", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47852", "FIPS": "18153", "County": "Sullivan", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47855", "FIPS": "18153", "County": "Sullivan", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47861", "FIPS": "18153", "County": "Sullivan", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47864", "FIPS": "18153", "County": "Sullivan", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47865", "FIPS": "18153", "County": "Sullivan", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47879", "FIPS": "18153", "County": "Sullivan", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47882", "FIPS": "18153", "County": "Sullivan", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47011", "FIPS": "18155", "County": "Switzerland", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47019", "FIPS": "18155", "County": "Switzerland", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47020", "FIPS": "18155", "County": "Switzerland", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47038", "FIPS": "18155", "County": "Switzerland", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47043", "FIPS": "18155", "County": "Switzerland", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47901", "FIPS": "18157", "County": "Tippecanoe", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47902", "FIPS": "18157", "County": "Tippecanoe", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47903", "FIPS": "18157", "County": "Tippecanoe", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47904", "FIPS": "18157", "County": "Tippecanoe", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47905", "FIPS": "18157", "County": "Tippecanoe", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47906", "FIPS": "18157", "County": "Tippecanoe", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47907", "FIPS": "18157", "County": "Tippecanoe", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47909", "FIPS": "18157", "County": "Tippecanoe", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47920", "FIPS": "18157", "County": "Tippecanoe", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47924", "FIPS": "18157", "County": "Tippecanoe", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47930", "FIPS": "18157", "County": "Tippecanoe", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47941", "FIPS": "18157", "County": "Tippecanoe", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47962", "FIPS": "18157", "County": "Tippecanoe", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47981", "FIPS": "18157", "County": "Tippecanoe", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47983", "FIPS": "18157", "County": "Tippecanoe", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47992", "FIPS": "18157", "County": "Tippecanoe", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47996", "FIPS": "18157", "County": "Tippecanoe", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "46045", "FIPS": "18159", "County": "Tipton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46047", "FIPS": "18159", "County": "Tipton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46049", "FIPS": "18159", "County": "Tipton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46068", "FIPS": "18159", "County": "Tipton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46072", "FIPS": "18159", "County": "Tipton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46076", "FIPS": "18159", "County": "Tipton", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47003", "FIPS": "18161", "County": "Union", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47325", "FIPS": "18161", "County": "Union", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47353", "FIPS": "18161", "County": "Union", "State": "IN", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "47618", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47701", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47702", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47703", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47704", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47705", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47706", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47708", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47710", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47711", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47712", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47713", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47714", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47715", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47716", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47719", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47720", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47721", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47722", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47724", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47725", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47727", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47728", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47730", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47731", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47732", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47733", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47734", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47735", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47736", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47737", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47739", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47740", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47741", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47744", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47747", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47750", "FIPS": "18163", "County": "Vanderburgh", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47831", "FIPS": "18165", "County": "Vermillion", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47842", "FIPS": "18165", "County": "Vermillion", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47847", "FIPS": "18165", "County": "Vermillion", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47854", "FIPS": "18165", "County": "Vermillion", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47875", "FIPS": "18165", "County": "Vermillion", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47884", "FIPS": "18165", "County": "Vermillion", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47928", "FIPS": "18165", "County": "Vermillion", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47966", "FIPS": "18165", "County": "Vermillion", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47974", "FIPS": "18165", "County": "Vermillion", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47801", "FIPS": "18167", "County": "Vigo", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47802", "FIPS": "18167", "County": "Vigo", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47803", "FIPS": "18167", "County": "Vigo", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47804", "FIPS": "18167", "County": "Vigo", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47805", "FIPS": "18167", "County": "Vigo", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47807", "FIPS": "18167", "County": "Vigo", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47808", "FIPS": "18167", "County": "Vigo", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47809", "FIPS": "18167", "County": "Vigo", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47811", "FIPS": "18167", "County": "Vigo", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47812", "FIPS": "18167", "County": "Vigo", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47813", "FIPS": "18167", "County": "Vigo", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47814", "FIPS": "18167", "County": "Vigo", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47851", "FIPS": "18167", "County": "Vigo", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47858", "FIPS": "18167", "County": "Vigo", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47863", "FIPS": "18167", "County": "Vigo", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47866", "FIPS": "18167", "County": "Vigo", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47869", "FIPS": "18167", "County": "Vigo", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47870", "FIPS": "18167", "County": "Vigo", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47871", "FIPS": "18167", "County": "Vigo", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47876", "FIPS": "18167", "County": "Vigo", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47878", "FIPS": "18167", "County": "Vigo", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47880", "FIPS": "18167", "County": "Vigo", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "47885", "FIPS": "18167", "County": "Vigo", "State": "IN", "DMA": {"Code": 581, "Name": "TERRE HAUTE"}},
  {"Postal_Code": "46940", "FIPS": "18169", "County": "Wabash", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46941", "FIPS": "18169", "County": "Wabash", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46943", "FIPS": "18169", "County": "Wabash", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46946", "FIPS": "18169", "County": "Wabash", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46962", "FIPS": "18169", "County": "Wabash", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46974", "FIPS": "18169", "County": "Wabash", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46980", "FIPS": "18169", "County": "Wabash", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46984", "FIPS": "18169", "County": "Wabash", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46990", "FIPS": "18169", "County": "Wabash", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46992", "FIPS": "18169", "County": "Wabash", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "47975", "FIPS": "18171", "County": "Warren", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47982", "FIPS": "18171", "County": "Warren", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47991", "FIPS": "18171", "County": "Warren", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47993", "FIPS": "18171", "County": "Warren", "State": "IN", "DMA": {"Code": 582, "Name": "LAFAYETTE, IN"}},
  {"Postal_Code": "47601", "FIPS": "18173", "County": "Warrick", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47610", "FIPS": "18173", "County": "Warrick", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47613", "FIPS": "18173", "County": "Warrick", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47614", "FIPS": "18173", "County": "Warrick", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47619", "FIPS": "18173", "County": "Warrick", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47629", "FIPS": "18173", "County": "Warrick", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47630", "FIPS": "18173", "County": "Warrick", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47637", "FIPS": "18173", "County": "Warrick", "State": "IN", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "47108", "FIPS": "18175", "County": "Washington", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47120", "FIPS": "18175", "County": "Washington", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47125", "FIPS": "18175", "County": "Washington", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47139", "FIPS": "18175", "County": "Washington", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47165", "FIPS": "18175", "County": "Washington", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47167", "FIPS": "18175", "County": "Washington", "State": "IN", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "47324", "FIPS": "18177", "County": "Wayne", "State": "IN", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "47327", "FIPS": "18177", "County": "Wayne", "State": "IN", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "47330", "FIPS": "18177", "County": "Wayne", "State": "IN", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "47335", "FIPS": "18177", "County": "Wayne", "State": "IN", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "47339", "FIPS": "18177", "County": "Wayne", "State": "IN", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "47341", "FIPS": "18177", "County": "Wayne", "State": "IN", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "47345", "FIPS": "18177", "County": "Wayne", "State": "IN", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "47346", "FIPS": "18177", "County": "Wayne", "State": "IN", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "47357", "FIPS": "18177", "County": "Wayne", "State": "IN", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "47370", "FIPS": "18177", "County": "Wayne", "State": "IN", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "47374", "FIPS": "18177", "County": "Wayne", "State": "IN", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "47375", "FIPS": "18177", "County": "Wayne", "State": "IN", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "47392", "FIPS": "18177", "County": "Wayne", "State": "IN", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "47393", "FIPS": "18177", "County": "Wayne", "State": "IN", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "46714", "FIPS": "18179", "County": "Wells", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46731", "FIPS": "18179", "County": "Wells", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46759", "FIPS": "18179", "County": "Wells", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46766", "FIPS": "18179", "County": "Wells", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46770", "FIPS": "18179", "County": "Wells", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46777", "FIPS": "18179", "County": "Wells", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46778", "FIPS": "18179", "County": "Wells", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46781", "FIPS": "18179", "County": "Wells", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46791", "FIPS": "18179", "County": "Wells", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "47923", "FIPS": "18181", "County": "White", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47925", "FIPS": "18181", "County": "White", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47926", "FIPS": "18181", "County": "White", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47929", "FIPS": "18181", "County": "White", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47950", "FIPS": "18181", "County": "White", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47959", "FIPS": "18181", "County": "White", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47960", "FIPS": "18181", "County": "White", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47980", "FIPS": "18181", "County": "White", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "47995", "FIPS": "18181", "County": "White", "State": "IN", "DMA": {"Code": 527, "Name": "INDIANAPOLIS"}},
  {"Postal_Code": "46723", "FIPS": "18183", "County": "Whitley", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46725", "FIPS": "18183", "County": "Whitley", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46764", "FIPS": "18183", "County": "Whitley", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "46787", "FIPS": "18183", "County": "Whitley", "State": "IN", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "50002", "FIPS": "19001", "County": "Adair", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50837", "FIPS": "19001", "County": "Adair", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50846", "FIPS": "19001", "County": "Adair", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50849", "FIPS": "19001", "County": "Adair", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50858", "FIPS": "19001", "County": "Adair", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50839", "FIPS": "19003", "County": "Adams", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50841", "FIPS": "19003", "County": "Adams", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50857", "FIPS": "19003", "County": "Adams", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50859", "FIPS": "19003", "County": "Adams", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52140", "FIPS": "19005", "County": "Allamakee", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52146", "FIPS": "19005", "County": "Allamakee", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52151", "FIPS": "19005", "County": "Allamakee", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52160", "FIPS": "19005", "County": "Allamakee", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52162", "FIPS": "19005", "County": "Allamakee", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52170", "FIPS": "19005", "County": "Allamakee", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52172", "FIPS": "19005", "County": "Allamakee", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52544", "FIPS": "19007", "County": "Appanoose", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52549", "FIPS": "19007", "County": "Appanoose", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52555", "FIPS": "19007", "County": "Appanoose", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52571", "FIPS": "19007", "County": "Appanoose", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52572", "FIPS": "19007", "County": "Appanoose", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52574", "FIPS": "19007", "County": "Appanoose", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52581", "FIPS": "19007", "County": "Appanoose", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52593", "FIPS": "19007", "County": "Appanoose", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52594", "FIPS": "19007", "County": "Appanoose", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50025", "FIPS": "19009", "County": "Audubon", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50042", "FIPS": "19009", "County": "Audubon", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50076", "FIPS": "19009", "County": "Audubon", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50110", "FIPS": "19009", "County": "Audubon", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50117", "FIPS": "19009", "County": "Audubon", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "51543", "FIPS": "19009", "County": "Audubon", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52206", "FIPS": "19011", "County": "Benton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52208", "FIPS": "19011", "County": "Benton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52209", "FIPS": "19011", "County": "Benton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52229", "FIPS": "19011", "County": "Benton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52249", "FIPS": "19011", "County": "Benton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52257", "FIPS": "19011", "County": "Benton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52313", "FIPS": "19011", "County": "Benton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52315", "FIPS": "19011", "County": "Benton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52318", "FIPS": "19011", "County": "Benton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52332", "FIPS": "19011", "County": "Benton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52345", "FIPS": "19011", "County": "Benton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52346", "FIPS": "19011", "County": "Benton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52349", "FIPS": "19011", "County": "Benton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52351", "FIPS": "19011", "County": "Benton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52354", "FIPS": "19011", "County": "Benton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50613", "FIPS": "19013", "County": "Black Hawk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50614", "FIPS": "19013", "County": "Black Hawk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50623", "FIPS": "19013", "County": "Black Hawk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50626", "FIPS": "19013", "County": "Black Hawk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50634", "FIPS": "19013", "County": "Black Hawk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50643", "FIPS": "19013", "County": "Black Hawk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50651", "FIPS": "19013", "County": "Black Hawk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50667", "FIPS": "19013", "County": "Black Hawk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50701", "FIPS": "19013", "County": "Black Hawk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50702", "FIPS": "19013", "County": "Black Hawk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50703", "FIPS": "19013", "County": "Black Hawk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50704", "FIPS": "19013", "County": "Black Hawk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50706", "FIPS": "19013", "County": "Black Hawk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50707", "FIPS": "19013", "County": "Black Hawk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50031", "FIPS": "19015", "County": "Boone", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50036", "FIPS": "19015", "County": "Boone", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50037", "FIPS": "19015", "County": "Boone", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50040", "FIPS": "19015", "County": "Boone", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50152", "FIPS": "19015", "County": "Boone", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50156", "FIPS": "19015", "County": "Boone", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50212", "FIPS": "19015", "County": "Boone", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50223", "FIPS": "19015", "County": "Boone", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50622", "FIPS": "19017", "County": "Bremer", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50631", "FIPS": "19017", "County": "Bremer", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50647", "FIPS": "19017", "County": "Bremer", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50666", "FIPS": "19017", "County": "Bremer", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50668", "FIPS": "19017", "County": "Bremer", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50674", "FIPS": "19017", "County": "Bremer", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50676", "FIPS": "19017", "County": "Bremer", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50677", "FIPS": "19017", "County": "Bremer", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50607", "FIPS": "19019", "County": "Buchanan", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50629", "FIPS": "19019", "County": "Buchanan", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50641", "FIPS": "19019", "County": "Buchanan", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50644", "FIPS": "19019", "County": "Buchanan", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50648", "FIPS": "19019", "County": "Buchanan", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50650", "FIPS": "19019", "County": "Buchanan", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50671", "FIPS": "19019", "County": "Buchanan", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50682", "FIPS": "19019", "County": "Buchanan", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52210", "FIPS": "19019", "County": "Buchanan", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52326", "FIPS": "19019", "County": "Buchanan", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52329", "FIPS": "19019", "County": "Buchanan", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50510", "FIPS": "19021", "County": "Buena Vista", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "50565", "FIPS": "19021", "County": "Buena Vista", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "50568", "FIPS": "19021", "County": "Buena Vista", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "50576", "FIPS": "19021", "County": "Buena Vista", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "50585", "FIPS": "19021", "County": "Buena Vista", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "50588", "FIPS": "19021", "County": "Buena Vista", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "50592", "FIPS": "19021", "County": "Buena Vista", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51002", "FIPS": "19021", "County": "Buena Vista", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51033", "FIPS": "19021", "County": "Buena Vista", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "50602", "FIPS": "19023", "County": "Butler", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50604", "FIPS": "19023", "County": "Butler", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50605", "FIPS": "19023", "County": "Butler", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50608", "FIPS": "19023", "County": "Butler", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50611", "FIPS": "19023", "County": "Butler", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50619", "FIPS": "19023", "County": "Butler", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50625", "FIPS": "19023", "County": "Butler", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50636", "FIPS": "19023", "County": "Butler", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50649", "FIPS": "19023", "County": "Butler", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50660", "FIPS": "19023", "County": "Butler", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50665", "FIPS": "19023", "County": "Butler", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50670", "FIPS": "19023", "County": "Butler", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50538", "FIPS": "19025", "County": "Calhoun", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50551", "FIPS": "19025", "County": "Calhoun", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50552", "FIPS": "19025", "County": "Calhoun", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50561", "FIPS": "19025", "County": "Calhoun", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50563", "FIPS": "19025", "County": "Calhoun", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50575", "FIPS": "19025", "County": "Calhoun", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50579", "FIPS": "19025", "County": "Calhoun", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50586", "FIPS": "19025", "County": "Calhoun", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "51449", "FIPS": "19025", "County": "Calhoun", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "51453", "FIPS": "19025", "County": "Calhoun", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50058", "FIPS": "19027", "County": "Carroll", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "51401", "FIPS": "19027", "County": "Carroll", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "51430", "FIPS": "19027", "County": "Carroll", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "51436", "FIPS": "19027", "County": "Carroll", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "51440", "FIPS": "19027", "County": "Carroll", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "51443", "FIPS": "19027", "County": "Carroll", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "51444", "FIPS": "19027", "County": "Carroll", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "51451", "FIPS": "19027", "County": "Carroll", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "51452", "FIPS": "19027", "County": "Carroll", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "51455", "FIPS": "19027", "County": "Carroll", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "51459", "FIPS": "19027", "County": "Carroll", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "51463", "FIPS": "19027", "County": "Carroll", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50020", "FIPS": "19029", "County": "Cass", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "50022", "FIPS": "19029", "County": "Cass", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "50274", "FIPS": "19029", "County": "Cass", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "50843", "FIPS": "19029", "County": "Cass", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "50853", "FIPS": "19029", "County": "Cass", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51535", "FIPS": "19029", "County": "Cass", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51544", "FIPS": "19029", "County": "Cass", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51552", "FIPS": "19029", "County": "Cass", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "52216", "FIPS": "19031", "County": "Cedar", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52255", "FIPS": "19031", "County": "Cedar", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52306", "FIPS": "19031", "County": "Cedar", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52337", "FIPS": "19031", "County": "Cedar", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52358", "FIPS": "19031", "County": "Cedar", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52721", "FIPS": "19031", "County": "Cedar", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52747", "FIPS": "19031", "County": "Cedar", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52772", "FIPS": "19031", "County": "Cedar", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50401", "FIPS": "19033", "County": "Cerro Gordo", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50402", "FIPS": "19033", "County": "Cerro Gordo", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50428", "FIPS": "19033", "County": "Cerro Gordo", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50433", "FIPS": "19033", "County": "Cerro Gordo", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50457", "FIPS": "19033", "County": "Cerro Gordo", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50464", "FIPS": "19033", "County": "Cerro Gordo", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50467", "FIPS": "19033", "County": "Cerro Gordo", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50469", "FIPS": "19033", "County": "Cerro Gordo", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50477", "FIPS": "19033", "County": "Cerro Gordo", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50479", "FIPS": "19033", "County": "Cerro Gordo", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50482", "FIPS": "19033", "County": "Cerro Gordo", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "51005", "FIPS": "19035", "County": "Cherokee", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51012", "FIPS": "19035", "County": "Cherokee", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51014", "FIPS": "19035", "County": "Cherokee", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51029", "FIPS": "19035", "County": "Cherokee", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51035", "FIPS": "19035", "County": "Cherokee", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51037", "FIPS": "19035", "County": "Cherokee", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51049", "FIPS": "19035", "County": "Cherokee", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51061", "FIPS": "19035", "County": "Cherokee", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "50603", "FIPS": "19037", "County": "Chickasaw", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50630", "FIPS": "19037", "County": "Chickasaw", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50645", "FIPS": "19037", "County": "Chickasaw", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50658", "FIPS": "19037", "County": "Chickasaw", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50659", "FIPS": "19037", "County": "Chickasaw", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50661", "FIPS": "19037", "County": "Chickasaw", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52154", "FIPS": "19037", "County": "Chickasaw", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50174", "FIPS": "19039", "County": "Clarke", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50213", "FIPS": "19039", "County": "Clarke", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50275", "FIPS": "19039", "County": "Clarke", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "51047", "FIPS": "19041", "County": "Clay", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51301", "FIPS": "19041", "County": "Clay", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51333", "FIPS": "19041", "County": "Clay", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51338", "FIPS": "19041", "County": "Clay", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51340", "FIPS": "19041", "County": "Clay", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51341", "FIPS": "19041", "County": "Clay", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51343", "FIPS": "19041", "County": "Clay", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51357", "FIPS": "19041", "County": "Clay", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51366", "FIPS": "19041", "County": "Clay", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "52042", "FIPS": "19043", "County": "Clayton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52043", "FIPS": "19043", "County": "Clayton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52044", "FIPS": "19043", "County": "Clayton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52047", "FIPS": "19043", "County": "Clayton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52048", "FIPS": "19043", "County": "Clayton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52049", "FIPS": "19043", "County": "Clayton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52052", "FIPS": "19043", "County": "Clayton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52066", "FIPS": "19043", "County": "Clayton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52072", "FIPS": "19043", "County": "Clayton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52076", "FIPS": "19043", "County": "Clayton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52077", "FIPS": "19043", "County": "Clayton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52156", "FIPS": "19043", "County": "Clayton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52157", "FIPS": "19043", "County": "Clayton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52158", "FIPS": "19043", "County": "Clayton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52159", "FIPS": "19043", "County": "Clayton", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52037", "FIPS": "19045", "County": "Clinton", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52254", "FIPS": "19045", "County": "Clinton", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52701", "FIPS": "19045", "County": "Clinton", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52727", "FIPS": "19045", "County": "Clinton", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52729", "FIPS": "19045", "County": "Clinton", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52730", "FIPS": "19045", "County": "Clinton", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52731", "FIPS": "19045", "County": "Clinton", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52732", "FIPS": "19045", "County": "Clinton", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52733", "FIPS": "19045", "County": "Clinton", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52734", "FIPS": "19045", "County": "Clinton", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52736", "FIPS": "19045", "County": "Clinton", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52742", "FIPS": "19045", "County": "Clinton", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52750", "FIPS": "19045", "County": "Clinton", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52751", "FIPS": "19045", "County": "Clinton", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52757", "FIPS": "19045", "County": "Clinton", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52771", "FIPS": "19045", "County": "Clinton", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52774", "FIPS": "19045", "County": "Clinton", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52777", "FIPS": "19045", "County": "Clinton", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "51432", "FIPS": "19047", "County": "Crawford", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51439", "FIPS": "19047", "County": "Crawford", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51441", "FIPS": "19047", "County": "Crawford", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51442", "FIPS": "19047", "County": "Crawford", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51448", "FIPS": "19047", "County": "Crawford", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51454", "FIPS": "19047", "County": "Crawford", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51460", "FIPS": "19047", "County": "Crawford", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51461", "FIPS": "19047", "County": "Crawford", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51465", "FIPS": "19047", "County": "Crawford", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51467", "FIPS": "19047", "County": "Crawford", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51520", "FIPS": "19047", "County": "Crawford", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51528", "FIPS": "19047", "County": "Crawford", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "50003", "FIPS": "19049", "County": "Dallas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50038", "FIPS": "19049", "County": "Dallas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50039", "FIPS": "19049", "County": "Dallas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50063", "FIPS": "19049", "County": "Dallas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50066", "FIPS": "19049", "County": "Dallas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50069", "FIPS": "19049", "County": "Dallas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50070", "FIPS": "19049", "County": "Dallas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50109", "FIPS": "19049", "County": "Dallas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50146", "FIPS": "19049", "County": "Dallas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50167", "FIPS": "19049", "County": "Dallas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50220", "FIPS": "19049", "County": "Dallas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50233", "FIPS": "19049", "County": "Dallas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50261", "FIPS": "19049", "County": "Dallas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50263", "FIPS": "19049", "County": "Dallas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50276", "FIPS": "19049", "County": "Dallas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52537", "FIPS": "19051", "County": "Davis", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52538", "FIPS": "19051", "County": "Davis", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52552", "FIPS": "19051", "County": "Davis", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52560", "FIPS": "19051", "County": "Davis", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52584", "FIPS": "19051", "County": "Davis", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "50065", "FIPS": "19053", "County": "Decatur", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50067", "FIPS": "19053", "County": "Decatur", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50103", "FIPS": "19053", "County": "Decatur", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50108", "FIPS": "19053", "County": "Decatur", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50140", "FIPS": "19053", "County": "Decatur", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50144", "FIPS": "19053", "County": "Decatur", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50262", "FIPS": "19053", "County": "Decatur", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50264", "FIPS": "19053", "County": "Decatur", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50654", "FIPS": "19055", "County": "Delaware", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52035", "FIPS": "19055", "County": "Delaware", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52036", "FIPS": "19055", "County": "Delaware", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52038", "FIPS": "19055", "County": "Delaware", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52041", "FIPS": "19055", "County": "Delaware", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52050", "FIPS": "19055", "County": "Delaware", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52057", "FIPS": "19055", "County": "Delaware", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52223", "FIPS": "19055", "County": "Delaware", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52237", "FIPS": "19055", "County": "Delaware", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52330", "FIPS": "19055", "County": "Delaware", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52601", "FIPS": "19057", "County": "Des Moines", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52623", "FIPS": "19057", "County": "Des Moines", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52637", "FIPS": "19057", "County": "Des Moines", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52638", "FIPS": "19057", "County": "Des Moines", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52650", "FIPS": "19057", "County": "Des Moines", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52655", "FIPS": "19057", "County": "Des Moines", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52660", "FIPS": "19057", "County": "Des Moines", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "51331", "FIPS": "19059", "County": "Dickinson", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51347", "FIPS": "19059", "County": "Dickinson", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51351", "FIPS": "19059", "County": "Dickinson", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51355", "FIPS": "19059", "County": "Dickinson", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51360", "FIPS": "19059", "County": "Dickinson", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51363", "FIPS": "19059", "County": "Dickinson", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51364", "FIPS": "19059", "County": "Dickinson", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "52001", "FIPS": "19061", "County": "Dubuque", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52002", "FIPS": "19061", "County": "Dubuque", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52003", "FIPS": "19061", "County": "Dubuque", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52004", "FIPS": "19061", "County": "Dubuque", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52032", "FIPS": "19061", "County": "Dubuque", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52033", "FIPS": "19061", "County": "Dubuque", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52039", "FIPS": "19061", "County": "Dubuque", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52040", "FIPS": "19061", "County": "Dubuque", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52045", "FIPS": "19061", "County": "Dubuque", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52046", "FIPS": "19061", "County": "Dubuque", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52053", "FIPS": "19061", "County": "Dubuque", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52056", "FIPS": "19061", "County": "Dubuque", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52065", "FIPS": "19061", "County": "Dubuque", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52068", "FIPS": "19061", "County": "Dubuque", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52073", "FIPS": "19061", "County": "Dubuque", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52078", "FIPS": "19061", "County": "Dubuque", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52079", "FIPS": "19061", "County": "Dubuque", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52099", "FIPS": "19061", "County": "Dubuque", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50514", "FIPS": "19063", "County": "Emmet", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "50531", "FIPS": "19063", "County": "Emmet", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "50578", "FIPS": "19063", "County": "Emmet", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51334", "FIPS": "19063", "County": "Emmet", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51344", "FIPS": "19063", "County": "Emmet", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51365", "FIPS": "19063", "County": "Emmet", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "50606", "FIPS": "19065", "County": "Fayette", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50655", "FIPS": "19065", "County": "Fayette", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50662", "FIPS": "19065", "County": "Fayette", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50664", "FIPS": "19065", "County": "Fayette", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50681", "FIPS": "19065", "County": "Fayette", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52135", "FIPS": "19065", "County": "Fayette", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52141", "FIPS": "19065", "County": "Fayette", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52142", "FIPS": "19065", "County": "Fayette", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52147", "FIPS": "19065", "County": "Fayette", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52164", "FIPS": "19065", "County": "Fayette", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52166", "FIPS": "19065", "County": "Fayette", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52169", "FIPS": "19065", "County": "Fayette", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52171", "FIPS": "19065", "County": "Fayette", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52175", "FIPS": "19065", "County": "Fayette", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50435", "FIPS": "19067", "County": "Floyd", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50458", "FIPS": "19067", "County": "Floyd", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50468", "FIPS": "19067", "County": "Floyd", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50471", "FIPS": "19067", "County": "Floyd", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50616", "FIPS": "19067", "County": "Floyd", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50620", "FIPS": "19067", "County": "Floyd", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50653", "FIPS": "19067", "County": "Floyd", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50041", "FIPS": "19069", "County": "Franklin", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50227", "FIPS": "19069", "County": "Franklin", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50420", "FIPS": "19069", "County": "Franklin", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50427", "FIPS": "19069", "County": "Franklin", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50431", "FIPS": "19069", "County": "Franklin", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50441", "FIPS": "19069", "County": "Franklin", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50452", "FIPS": "19069", "County": "Franklin", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50475", "FIPS": "19069", "County": "Franklin", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50633", "FIPS": "19069", "County": "Franklin", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "51639", "FIPS": "19071", "County": "Fremont", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51640", "FIPS": "19071", "County": "Fremont", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51645", "FIPS": "19071", "County": "Fremont", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51648", "FIPS": "19071", "County": "Fremont", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51649", "FIPS": "19071", "County": "Fremont", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51650", "FIPS": "19071", "County": "Fremont", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51652", "FIPS": "19071", "County": "Fremont", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51653", "FIPS": "19071", "County": "Fremont", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51654", "FIPS": "19071", "County": "Fremont", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "50050", "FIPS": "19073", "County": "Greene", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50059", "FIPS": "19073", "County": "Greene", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50064", "FIPS": "19073", "County": "Greene", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50107", "FIPS": "19073", "County": "Greene", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50129", "FIPS": "19073", "County": "Greene", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50217", "FIPS": "19073", "County": "Greene", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50235", "FIPS": "19073", "County": "Greene", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "51462", "FIPS": "19073", "County": "Greene", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50609", "FIPS": "19075", "County": "Grundy", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50621", "FIPS": "19075", "County": "Grundy", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50624", "FIPS": "19075", "County": "Grundy", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50638", "FIPS": "19075", "County": "Grundy", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50642", "FIPS": "19075", "County": "Grundy", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50657", "FIPS": "19075", "County": "Grundy", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50669", "FIPS": "19075", "County": "Grundy", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50673", "FIPS": "19075", "County": "Grundy", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50680", "FIPS": "19075", "County": "Grundy", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50026", "FIPS": "19077", "County": "Guthrie", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50029", "FIPS": "19077", "County": "Guthrie", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50048", "FIPS": "19077", "County": "Guthrie", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50115", "FIPS": "19077", "County": "Guthrie", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50128", "FIPS": "19077", "County": "Guthrie", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50164", "FIPS": "19077", "County": "Guthrie", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50216", "FIPS": "19077", "County": "Guthrie", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50250", "FIPS": "19077", "County": "Guthrie", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50277", "FIPS": "19077", "County": "Guthrie", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50034", "FIPS": "19079", "County": "Hamilton", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50075", "FIPS": "19079", "County": "Hamilton", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50130", "FIPS": "19079", "County": "Hamilton", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50132", "FIPS": "19079", "County": "Hamilton", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50231", "FIPS": "19079", "County": "Hamilton", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50246", "FIPS": "19079", "County": "Hamilton", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50249", "FIPS": "19079", "County": "Hamilton", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50271", "FIPS": "19079", "County": "Hamilton", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50595", "FIPS": "19079", "County": "Hamilton", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50423", "FIPS": "19081", "County": "Hancock", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50430", "FIPS": "19081", "County": "Hancock", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50432", "FIPS": "19081", "County": "Hancock", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50438", "FIPS": "19081", "County": "Hancock", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50439", "FIPS": "19081", "County": "Hancock", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50447", "FIPS": "19081", "County": "Hancock", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50449", "FIPS": "19081", "County": "Hancock", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50484", "FIPS": "19081", "County": "Hancock", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50006", "FIPS": "19083", "County": "Hardin", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50043", "FIPS": "19083", "County": "Hardin", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50102", "FIPS": "19083", "County": "Hardin", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50122", "FIPS": "19083", "County": "Hardin", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50126", "FIPS": "19083", "County": "Hardin", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50206", "FIPS": "19083", "County": "Hardin", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50230", "FIPS": "19083", "County": "Hardin", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50258", "FIPS": "19083", "County": "Hardin", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50259", "FIPS": "19083", "County": "Hardin", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50269", "FIPS": "19083", "County": "Hardin", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50601", "FIPS": "19083", "County": "Hardin", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50627", "FIPS": "19083", "County": "Hardin", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50672", "FIPS": "19083", "County": "Hardin", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "51529", "FIPS": "19085", "County": "Harrison", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51545", "FIPS": "19085", "County": "Harrison", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51546", "FIPS": "19085", "County": "Harrison", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51550", "FIPS": "19085", "County": "Harrison", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51555", "FIPS": "19085", "County": "Harrison", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51556", "FIPS": "19085", "County": "Harrison", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51557", "FIPS": "19085", "County": "Harrison", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51563", "FIPS": "19085", "County": "Harrison", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51564", "FIPS": "19085", "County": "Harrison", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51579", "FIPS": "19085", "County": "Harrison", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "52630", "FIPS": "19087", "County": "Henry", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52641", "FIPS": "19087", "County": "Henry", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52642", "FIPS": "19087", "County": "Henry", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52644", "FIPS": "19087", "County": "Henry", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52645", "FIPS": "19087", "County": "Henry", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52647", "FIPS": "19087", "County": "Henry", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52649", "FIPS": "19087", "County": "Henry", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52652", "FIPS": "19087", "County": "Henry", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52654", "FIPS": "19087", "County": "Henry", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52659", "FIPS": "19087", "County": "Henry", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "50466", "FIPS": "19089", "County": "Howard", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50628", "FIPS": "19089", "County": "Howard", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "52134", "FIPS": "19089", "County": "Howard", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "52136", "FIPS": "19089", "County": "Howard", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "52155", "FIPS": "19089", "County": "Howard", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "52163", "FIPS": "19089", "County": "Howard", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50519", "FIPS": "19091", "County": "Humboldt", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50520", "FIPS": "19091", "County": "Humboldt", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50529", "FIPS": "19091", "County": "Humboldt", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50541", "FIPS": "19091", "County": "Humboldt", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50545", "FIPS": "19091", "County": "Humboldt", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50548", "FIPS": "19091", "County": "Humboldt", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50558", "FIPS": "19091", "County": "Humboldt", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50570", "FIPS": "19091", "County": "Humboldt", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50577", "FIPS": "19091", "County": "Humboldt", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50582", "FIPS": "19091", "County": "Humboldt", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50591", "FIPS": "19091", "County": "Humboldt", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "51006", "FIPS": "19093", "County": "Ida", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51020", "FIPS": "19093", "County": "Ida", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51025", "FIPS": "19093", "County": "Ida", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51431", "FIPS": "19093", "County": "Ida", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51445", "FIPS": "19093", "County": "Ida", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "52203", "FIPS": "19095", "County": "Iowa", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52204", "FIPS": "19095", "County": "Iowa", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52220", "FIPS": "19095", "County": "Iowa", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52236", "FIPS": "19095", "County": "Iowa", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52251", "FIPS": "19095", "County": "Iowa", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52301", "FIPS": "19095", "County": "Iowa", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52307", "FIPS": "19095", "County": "Iowa", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52308", "FIPS": "19095", "County": "Iowa", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52316", "FIPS": "19095", "County": "Iowa", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52325", "FIPS": "19095", "County": "Iowa", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52334", "FIPS": "19095", "County": "Iowa", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52347", "FIPS": "19095", "County": "Iowa", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52361", "FIPS": "19095", "County": "Iowa", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52030", "FIPS": "19097", "County": "Jackson", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52031", "FIPS": "19097", "County": "Jackson", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52054", "FIPS": "19097", "County": "Jackson", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52060", "FIPS": "19097", "County": "Jackson", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52064", "FIPS": "19097", "County": "Jackson", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52069", "FIPS": "19097", "County": "Jackson", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52070", "FIPS": "19097", "County": "Jackson", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52071", "FIPS": "19097", "County": "Jackson", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52074", "FIPS": "19097", "County": "Jackson", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52075", "FIPS": "19097", "County": "Jackson", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52207", "FIPS": "19097", "County": "Jackson", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52309", "FIPS": "19097", "County": "Jackson", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "50028", "FIPS": "19099", "County": "Jasper", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50054", "FIPS": "19099", "County": "Jasper", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50127", "FIPS": "19099", "County": "Jasper", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50135", "FIPS": "19099", "County": "Jasper", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50137", "FIPS": "19099", "County": "Jasper", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50153", "FIPS": "19099", "County": "Jasper", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50168", "FIPS": "19099", "County": "Jasper", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50170", "FIPS": "19099", "County": "Jasper", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50208", "FIPS": "19099", "County": "Jasper", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50228", "FIPS": "19099", "County": "Jasper", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50232", "FIPS": "19099", "County": "Jasper", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50251", "FIPS": "19099", "County": "Jasper", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52533", "FIPS": "19101", "County": "Jefferson", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52556", "FIPS": "19101", "County": "Jefferson", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52557", "FIPS": "19101", "County": "Jefferson", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52567", "FIPS": "19101", "County": "Jefferson", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52580", "FIPS": "19101", "County": "Jefferson", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52635", "FIPS": "19101", "County": "Jefferson", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52235", "FIPS": "19103", "County": "Johnson", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52240", "FIPS": "19103", "County": "Johnson", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52241", "FIPS": "19103", "County": "Johnson", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52242", "FIPS": "19103", "County": "Johnson", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52243", "FIPS": "19103", "County": "Johnson", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52244", "FIPS": "19103", "County": "Johnson", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52245", "FIPS": "19103", "County": "Johnson", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52246", "FIPS": "19103", "County": "Johnson", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52317", "FIPS": "19103", "County": "Johnson", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52319", "FIPS": "19103", "County": "Johnson", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52322", "FIPS": "19103", "County": "Johnson", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52333", "FIPS": "19103", "County": "Johnson", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52338", "FIPS": "19103", "County": "Johnson", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52340", "FIPS": "19103", "County": "Johnson", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52755", "FIPS": "19103", "County": "Johnson", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52205", "FIPS": "19105", "County": "Jones", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52212", "FIPS": "19105", "County": "Jones", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52252", "FIPS": "19105", "County": "Jones", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52305", "FIPS": "19105", "County": "Jones", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52310", "FIPS": "19105", "County": "Jones", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52312", "FIPS": "19105", "County": "Jones", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52320", "FIPS": "19105", "County": "Jones", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52321", "FIPS": "19105", "County": "Jones", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52323", "FIPS": "19105", "County": "Jones", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52362", "FIPS": "19105", "County": "Jones", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50104", "FIPS": "19107", "County": "Keokuk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50136", "FIPS": "19107", "County": "Keokuk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50255", "FIPS": "19107", "County": "Keokuk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50268", "FIPS": "19107", "County": "Keokuk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52231", "FIPS": "19107", "County": "Keokuk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52248", "FIPS": "19107", "County": "Keokuk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52335", "FIPS": "19107", "County": "Keokuk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52355", "FIPS": "19107", "County": "Keokuk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52550", "FIPS": "19107", "County": "Keokuk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52562", "FIPS": "19107", "County": "Keokuk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52563", "FIPS": "19107", "County": "Keokuk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52568", "FIPS": "19107", "County": "Keokuk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52576", "FIPS": "19107", "County": "Keokuk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52585", "FIPS": "19107", "County": "Keokuk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52591", "FIPS": "19107", "County": "Keokuk", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50451", "FIPS": "19109", "County": "Kossuth", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50480", "FIPS": "19109", "County": "Kossuth", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50483", "FIPS": "19109", "County": "Kossuth", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50511", "FIPS": "19109", "County": "Kossuth", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50517", "FIPS": "19109", "County": "Kossuth", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50522", "FIPS": "19109", "County": "Kossuth", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50539", "FIPS": "19109", "County": "Kossuth", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50556", "FIPS": "19109", "County": "Kossuth", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50559", "FIPS": "19109", "County": "Kossuth", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50560", "FIPS": "19109", "County": "Kossuth", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50590", "FIPS": "19109", "County": "Kossuth", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50598", "FIPS": "19109", "County": "Kossuth", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52619", "FIPS": "19111", "County": "Lee", "State": "IA", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "52624", "FIPS": "19111", "County": "Lee", "State": "IA", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "52625", "FIPS": "19111", "County": "Lee", "State": "IA", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "52627", "FIPS": "19111", "County": "Lee", "State": "IA", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "52631", "FIPS": "19111", "County": "Lee", "State": "IA", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "52632", "FIPS": "19111", "County": "Lee", "State": "IA", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "52639", "FIPS": "19111", "County": "Lee", "State": "IA", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "52648", "FIPS": "19111", "County": "Lee", "State": "IA", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "52656", "FIPS": "19111", "County": "Lee", "State": "IA", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "52657", "FIPS": "19111", "County": "Lee", "State": "IA", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "52658", "FIPS": "19111", "County": "Lee", "State": "IA", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "52202", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52213", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52214", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52218", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52219", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52227", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52228", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52233", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52253", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52302", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52314", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52324", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52328", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52336", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52341", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52344", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52350", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52352", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52401", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52402", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52403", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52404", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52405", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52406", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52407", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52408", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52409", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52410", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52411", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52497", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52498", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52499", "FIPS": "19113", "County": "Linn", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52640", "FIPS": "19115", "County": "Louisa", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52646", "FIPS": "19115", "County": "Louisa", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52653", "FIPS": "19115", "County": "Louisa", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52737", "FIPS": "19115", "County": "Louisa", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52738", "FIPS": "19115", "County": "Louisa", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52752", "FIPS": "19115", "County": "Louisa", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52754", "FIPS": "19115", "County": "Louisa", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "50049", "FIPS": "19117", "County": "Lucas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50068", "FIPS": "19117", "County": "Lucas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50151", "FIPS": "19117", "County": "Lucas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50238", "FIPS": "19117", "County": "Lucas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50272", "FIPS": "19117", "County": "Lucas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "51230", "FIPS": "19119", "County": "Lyon", "State": "IA", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "51235", "FIPS": "19119", "County": "Lyon", "State": "IA", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "51237", "FIPS": "19119", "County": "Lyon", "State": "IA", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "51240", "FIPS": "19119", "County": "Lyon", "State": "IA", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "51241", "FIPS": "19119", "County": "Lyon", "State": "IA", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "51242", "FIPS": "19119", "County": "Lyon", "State": "IA", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "51243", "FIPS": "19119", "County": "Lyon", "State": "IA", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "51246", "FIPS": "19119", "County": "Lyon", "State": "IA", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "50033", "FIPS": "19121", "County": "Madison", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50072", "FIPS": "19121", "County": "Madison", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50155", "FIPS": "19121", "County": "Madison", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50218", "FIPS": "19121", "County": "Madison", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50222", "FIPS": "19121", "County": "Madison", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50240", "FIPS": "19121", "County": "Madison", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50257", "FIPS": "19121", "County": "Madison", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50273", "FIPS": "19121", "County": "Madison", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50027", "FIPS": "19123", "County": "Mahaska", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50143", "FIPS": "19123", "County": "Mahaska", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50207", "FIPS": "19123", "County": "Mahaska", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52534", "FIPS": "19123", "County": "Mahaska", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52543", "FIPS": "19123", "County": "Mahaska", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52561", "FIPS": "19123", "County": "Mahaska", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52577", "FIPS": "19123", "County": "Mahaska", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52586", "FIPS": "19123", "County": "Mahaska", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52595", "FIPS": "19123", "County": "Mahaska", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50044", "FIPS": "19125", "County": "Marion", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50057", "FIPS": "19125", "County": "Marion", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50062", "FIPS": "19125", "County": "Marion", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50116", "FIPS": "19125", "County": "Marion", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50119", "FIPS": "19125", "County": "Marion", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50138", "FIPS": "19125", "County": "Marion", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50163", "FIPS": "19125", "County": "Marion", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50214", "FIPS": "19125", "County": "Marion", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50219", "FIPS": "19125", "County": "Marion", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50225", "FIPS": "19125", "County": "Marion", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50252", "FIPS": "19125", "County": "Marion", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50256", "FIPS": "19125", "County": "Marion", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50005", "FIPS": "19127", "County": "Marshall", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50051", "FIPS": "19127", "County": "Marshall", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50078", "FIPS": "19127", "County": "Marshall", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50106", "FIPS": "19127", "County": "Marshall", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50120", "FIPS": "19127", "County": "Marshall", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50141", "FIPS": "19127", "County": "Marshall", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50142", "FIPS": "19127", "County": "Marshall", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50148", "FIPS": "19127", "County": "Marshall", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50158", "FIPS": "19127", "County": "Marshall", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50162", "FIPS": "19127", "County": "Marshall", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50234", "FIPS": "19127", "County": "Marshall", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50239", "FIPS": "19127", "County": "Marshall", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50247", "FIPS": "19127", "County": "Marshall", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "51533", "FIPS": "19129", "County": "Mills", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51534", "FIPS": "19129", "County": "Mills", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51540", "FIPS": "19129", "County": "Mills", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51541", "FIPS": "19129", "County": "Mills", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51551", "FIPS": "19129", "County": "Mills", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51554", "FIPS": "19129", "County": "Mills", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51561", "FIPS": "19129", "County": "Mills", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51571", "FIPS": "19129", "County": "Mills", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "50426", "FIPS": "19131", "County": "Mitchell", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50454", "FIPS": "19131", "County": "Mitchell", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50455", "FIPS": "19131", "County": "Mitchell", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50460", "FIPS": "19131", "County": "Mitchell", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50461", "FIPS": "19131", "County": "Mitchell", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50472", "FIPS": "19131", "County": "Mitchell", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50476", "FIPS": "19131", "County": "Mitchell", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50481", "FIPS": "19131", "County": "Mitchell", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "51010", "FIPS": "19133", "County": "Monona", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51034", "FIPS": "19133", "County": "Monona", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51040", "FIPS": "19133", "County": "Monona", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51051", "FIPS": "19133", "County": "Monona", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51060", "FIPS": "19133", "County": "Monona", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51063", "FIPS": "19133", "County": "Monona", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51523", "FIPS": "19133", "County": "Monona", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51558", "FIPS": "19133", "County": "Monona", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51572", "FIPS": "19133", "County": "Monona", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "50150", "FIPS": "19135", "County": "Monroe", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52531", "FIPS": "19135", "County": "Monroe", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52569", "FIPS": "19135", "County": "Monroe", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50847", "FIPS": "19137", "County": "Montgomery", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "50864", "FIPS": "19137", "County": "Montgomery", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51532", "FIPS": "19137", "County": "Montgomery", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51566", "FIPS": "19137", "County": "Montgomery", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51573", "FIPS": "19137", "County": "Montgomery", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51591", "FIPS": "19137", "County": "Montgomery", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "52720", "FIPS": "19139", "County": "Muscatine", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52739", "FIPS": "19139", "County": "Muscatine", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52749", "FIPS": "19139", "County": "Muscatine", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52759", "FIPS": "19139", "County": "Muscatine", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52760", "FIPS": "19139", "County": "Muscatine", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52761", "FIPS": "19139", "County": "Muscatine", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52766", "FIPS": "19139", "County": "Muscatine", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52769", "FIPS": "19139", "County": "Muscatine", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52776", "FIPS": "19139", "County": "Muscatine", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52778", "FIPS": "19139", "County": "Muscatine", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "51009", "FIPS": "19141", "County": "Obrien", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51046", "FIPS": "19141", "County": "Obrien", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51058", "FIPS": "19141", "County": "Obrien", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51201", "FIPS": "19141", "County": "Obrien", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51231", "FIPS": "19141", "County": "Obrien", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51245", "FIPS": "19141", "County": "Obrien", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51248", "FIPS": "19141", "County": "Obrien", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51346", "FIPS": "19141", "County": "Obrien", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51232", "FIPS": "19143", "County": "Osceola", "State": "IA", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "51249", "FIPS": "19143", "County": "Osceola", "State": "IA", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "51345", "FIPS": "19143", "County": "Osceola", "State": "IA", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "51350", "FIPS": "19143", "County": "Osceola", "State": "IA", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "51354", "FIPS": "19143", "County": "Osceola", "State": "IA", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "51601", "FIPS": "19145", "County": "Page", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51602", "FIPS": "19145", "County": "Page", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51603", "FIPS": "19145", "County": "Page", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51630", "FIPS": "19145", "County": "Page", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51631", "FIPS": "19145", "County": "Page", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51632", "FIPS": "19145", "County": "Page", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51636", "FIPS": "19145", "County": "Page", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51637", "FIPS": "19145", "County": "Page", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51638", "FIPS": "19145", "County": "Page", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51647", "FIPS": "19145", "County": "Page", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51651", "FIPS": "19145", "County": "Page", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51656", "FIPS": "19145", "County": "Page", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "50515", "FIPS": "19147", "County": "Palo Alto", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "50527", "FIPS": "19147", "County": "Palo Alto", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "50528", "FIPS": "19147", "County": "Palo Alto", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "50536", "FIPS": "19147", "County": "Palo Alto", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "50562", "FIPS": "19147", "County": "Palo Alto", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "50597", "FIPS": "19147", "County": "Palo Alto", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51342", "FIPS": "19147", "County": "Palo Alto", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51358", "FIPS": "19147", "County": "Palo Alto", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51001", "FIPS": "19149", "County": "Plymouth", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51008", "FIPS": "19149", "County": "Plymouth", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51024", "FIPS": "19149", "County": "Plymouth", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51028", "FIPS": "19149", "County": "Plymouth", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51031", "FIPS": "19149", "County": "Plymouth", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51038", "FIPS": "19149", "County": "Plymouth", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51045", "FIPS": "19149", "County": "Plymouth", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51050", "FIPS": "19149", "County": "Plymouth", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51062", "FIPS": "19149", "County": "Plymouth", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "50540", "FIPS": "19151", "County": "Pocahontas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50546", "FIPS": "19151", "County": "Pocahontas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50554", "FIPS": "19151", "County": "Pocahontas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50571", "FIPS": "19151", "County": "Pocahontas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50573", "FIPS": "19151", "County": "Pocahontas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50574", "FIPS": "19151", "County": "Pocahontas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50581", "FIPS": "19151", "County": "Pocahontas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50593", "FIPS": "19151", "County": "Pocahontas", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50007", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50009", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50021", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50023", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50032", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50035", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50073", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50111", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50131", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50169", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50226", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50237", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50243", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50265", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50266", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50301", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50302", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50303", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50304", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50305", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50306", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50307", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50308", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50309", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50310", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50311", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50312", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50313", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50314", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50315", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50316", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50317", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50318", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50319", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50320", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50321", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50322", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50323", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50325", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50327", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50328", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50329", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50330", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50331", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50332", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50333", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50334", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50335", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50336", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50338", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50339", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50340", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50347", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50350", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50359", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50360", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50361", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50362", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50363", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50364", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50367", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50368", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50369", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50380", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50381", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50391", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50392", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50393", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50394", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50395", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50396", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50397", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50398", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50936", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50940", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50947", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50950", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50980", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50981", "FIPS": "19153", "County": "Polk", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "51501", "FIPS": "19155", "County": "Pottawattamie", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51502", "FIPS": "19155", "County": "Pottawattamie", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51503", "FIPS": "19155", "County": "Pottawattamie", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51510", "FIPS": "19155", "County": "Pottawattamie", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51521", "FIPS": "19155", "County": "Pottawattamie", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51525", "FIPS": "19155", "County": "Pottawattamie", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51526", "FIPS": "19155", "County": "Pottawattamie", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51536", "FIPS": "19155", "County": "Pottawattamie", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51542", "FIPS": "19155", "County": "Pottawattamie", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51548", "FIPS": "19155", "County": "Pottawattamie", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51549", "FIPS": "19155", "County": "Pottawattamie", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51553", "FIPS": "19155", "County": "Pottawattamie", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51559", "FIPS": "19155", "County": "Pottawattamie", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51560", "FIPS": "19155", "County": "Pottawattamie", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51570", "FIPS": "19155", "County": "Pottawattamie", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51575", "FIPS": "19155", "County": "Pottawattamie", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51576", "FIPS": "19155", "County": "Pottawattamie", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51577", "FIPS": "19155", "County": "Pottawattamie", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "50112", "FIPS": "19157", "County": "Poweshiek", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50157", "FIPS": "19157", "County": "Poweshiek", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50171", "FIPS": "19157", "County": "Poweshiek", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50242", "FIPS": "19157", "County": "Poweshiek", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52211", "FIPS": "19157", "County": "Poweshiek", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52221", "FIPS": "19157", "County": "Poweshiek", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52222", "FIPS": "19157", "County": "Poweshiek", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52232", "FIPS": "19157", "County": "Poweshiek", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50074", "FIPS": "19159", "County": "Ringgold", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50133", "FIPS": "19159", "County": "Ringgold", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50835", "FIPS": "19159", "County": "Ringgold", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50845", "FIPS": "19159", "County": "Ringgold", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50854", "FIPS": "19159", "County": "Ringgold", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50860", "FIPS": "19159", "County": "Ringgold", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50863", "FIPS": "19159", "County": "Ringgold", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50535", "FIPS": "19161", "County": "Sac", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "50567", "FIPS": "19161", "County": "Sac", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "50583", "FIPS": "19161", "County": "Sac", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51053", "FIPS": "19161", "County": "Sac", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51433", "FIPS": "19161", "County": "Sac", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51450", "FIPS": "19161", "County": "Sac", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51458", "FIPS": "19161", "County": "Sac", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51466", "FIPS": "19161", "County": "Sac", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "52722", "FIPS": "19163", "County": "Scott", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52726", "FIPS": "19163", "County": "Scott", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52728", "FIPS": "19163", "County": "Scott", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52745", "FIPS": "19163", "County": "Scott", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52746", "FIPS": "19163", "County": "Scott", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52748", "FIPS": "19163", "County": "Scott", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52753", "FIPS": "19163", "County": "Scott", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52756", "FIPS": "19163", "County": "Scott", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52758", "FIPS": "19163", "County": "Scott", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52765", "FIPS": "19163", "County": "Scott", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52767", "FIPS": "19163", "County": "Scott", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52768", "FIPS": "19163", "County": "Scott", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52773", "FIPS": "19163", "County": "Scott", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52801", "FIPS": "19163", "County": "Scott", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52802", "FIPS": "19163", "County": "Scott", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52803", "FIPS": "19163", "County": "Scott", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52804", "FIPS": "19163", "County": "Scott", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52805", "FIPS": "19163", "County": "Scott", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52806", "FIPS": "19163", "County": "Scott", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52807", "FIPS": "19163", "County": "Scott", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52808", "FIPS": "19163", "County": "Scott", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "52809", "FIPS": "19163", "County": "Scott", "State": "IA", "DMA": {"Code": 682, "Name": "DAVENPORT-ROCK ISLAND-MOLINE"}},
  {"Postal_Code": "51446", "FIPS": "19165", "County": "Shelby", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51447", "FIPS": "19165", "County": "Shelby", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51527", "FIPS": "19165", "County": "Shelby", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51530", "FIPS": "19165", "County": "Shelby", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51531", "FIPS": "19165", "County": "Shelby", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51537", "FIPS": "19165", "County": "Shelby", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51562", "FIPS": "19165", "County": "Shelby", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51565", "FIPS": "19165", "County": "Shelby", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51578", "FIPS": "19165", "County": "Shelby", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51593", "FIPS": "19165", "County": "Shelby", "State": "IA", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "51003", "FIPS": "19167", "County": "Sioux", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51011", "FIPS": "19167", "County": "Sioux", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51022", "FIPS": "19167", "County": "Sioux", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51023", "FIPS": "19167", "County": "Sioux", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51027", "FIPS": "19167", "County": "Sioux", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51036", "FIPS": "19167", "County": "Sioux", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51041", "FIPS": "19167", "County": "Sioux", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51234", "FIPS": "19167", "County": "Sioux", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51238", "FIPS": "19167", "County": "Sioux", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51239", "FIPS": "19167", "County": "Sioux", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51244", "FIPS": "19167", "County": "Sioux", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51247", "FIPS": "19167", "County": "Sioux", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51250", "FIPS": "19167", "County": "Sioux", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "50010", "FIPS": "19169", "County": "Story", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50011", "FIPS": "19169", "County": "Story", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50012", "FIPS": "19169", "County": "Story", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50013", "FIPS": "19169", "County": "Story", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50014", "FIPS": "19169", "County": "Story", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50046", "FIPS": "19169", "County": "Story", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50055", "FIPS": "19169", "County": "Story", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50056", "FIPS": "19169", "County": "Story", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50105", "FIPS": "19169", "County": "Story", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50124", "FIPS": "19169", "County": "Story", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50134", "FIPS": "19169", "County": "Story", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50154", "FIPS": "19169", "County": "Story", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50161", "FIPS": "19169", "County": "Story", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50201", "FIPS": "19169", "County": "Story", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50236", "FIPS": "19169", "County": "Story", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50244", "FIPS": "19169", "County": "Story", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50248", "FIPS": "19169", "County": "Story", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50278", "FIPS": "19169", "County": "Story", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50173", "FIPS": "19171", "County": "Tama", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50612", "FIPS": "19171", "County": "Tama", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50632", "FIPS": "19171", "County": "Tama", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50635", "FIPS": "19171", "County": "Tama", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50652", "FIPS": "19171", "County": "Tama", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50675", "FIPS": "19171", "County": "Tama", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52215", "FIPS": "19171", "County": "Tama", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52217", "FIPS": "19171", "County": "Tama", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52224", "FIPS": "19171", "County": "Tama", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52225", "FIPS": "19171", "County": "Tama", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52339", "FIPS": "19171", "County": "Tama", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52342", "FIPS": "19171", "County": "Tama", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52348", "FIPS": "19171", "County": "Tama", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50833", "FIPS": "19173", "County": "Taylor", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50836", "FIPS": "19173", "County": "Taylor", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50840", "FIPS": "19173", "County": "Taylor", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50848", "FIPS": "19173", "County": "Taylor", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50851", "FIPS": "19173", "County": "Taylor", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50862", "FIPS": "19173", "County": "Taylor", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "51646", "FIPS": "19173", "County": "Taylor", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50149", "FIPS": "19175", "County": "Union", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50254", "FIPS": "19175", "County": "Union", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50801", "FIPS": "19175", "County": "Union", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50830", "FIPS": "19175", "County": "Union", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50831", "FIPS": "19175", "County": "Union", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50842", "FIPS": "19175", "County": "Union", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50861", "FIPS": "19175", "County": "Union", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52535", "FIPS": "19177", "County": "Van Buren", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52542", "FIPS": "19177", "County": "Van Buren", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52551", "FIPS": "19177", "County": "Van Buren", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52565", "FIPS": "19177", "County": "Van Buren", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52570", "FIPS": "19177", "County": "Van Buren", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52573", "FIPS": "19177", "County": "Van Buren", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52588", "FIPS": "19177", "County": "Van Buren", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52620", "FIPS": "19177", "County": "Van Buren", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52626", "FIPS": "19177", "County": "Van Buren", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52651", "FIPS": "19177", "County": "Van Buren", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52501", "FIPS": "19179", "County": "Wapello", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52530", "FIPS": "19179", "County": "Wapello", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52536", "FIPS": "19179", "County": "Wapello", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52548", "FIPS": "19179", "County": "Wapello", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52553", "FIPS": "19179", "County": "Wapello", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52554", "FIPS": "19179", "County": "Wapello", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "52566", "FIPS": "19179", "County": "Wapello", "State": "IA", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "50001", "FIPS": "19181", "County": "Warren", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50047", "FIPS": "19181", "County": "Warren", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50061", "FIPS": "19181", "County": "Warren", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50118", "FIPS": "19181", "County": "Warren", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50125", "FIPS": "19181", "County": "Warren", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50139", "FIPS": "19181", "County": "Warren", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50145", "FIPS": "19181", "County": "Warren", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50160", "FIPS": "19181", "County": "Warren", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50166", "FIPS": "19181", "County": "Warren", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50210", "FIPS": "19181", "County": "Warren", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50211", "FIPS": "19181", "County": "Warren", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50229", "FIPS": "19181", "County": "Warren", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50241", "FIPS": "19181", "County": "Warren", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52201", "FIPS": "19183", "County": "Washington", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52247", "FIPS": "19183", "County": "Washington", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52327", "FIPS": "19183", "County": "Washington", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52353", "FIPS": "19183", "County": "Washington", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52356", "FIPS": "19183", "County": "Washington", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52359", "FIPS": "19183", "County": "Washington", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52540", "FIPS": "19183", "County": "Washington", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52621", "FIPS": "19183", "County": "Washington", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "50008", "FIPS": "19185", "County": "Wayne", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50052", "FIPS": "19185", "County": "Wayne", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50060", "FIPS": "19185", "County": "Wayne", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50123", "FIPS": "19185", "County": "Wayne", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50147", "FIPS": "19185", "County": "Wayne", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50165", "FIPS": "19185", "County": "Wayne", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52583", "FIPS": "19185", "County": "Wayne", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "52590", "FIPS": "19185", "County": "Wayne", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50501", "FIPS": "19187", "County": "Webster", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50516", "FIPS": "19187", "County": "Webster", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50518", "FIPS": "19187", "County": "Webster", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50521", "FIPS": "19187", "County": "Webster", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50523", "FIPS": "19187", "County": "Webster", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50524", "FIPS": "19187", "County": "Webster", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50530", "FIPS": "19187", "County": "Webster", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50532", "FIPS": "19187", "County": "Webster", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50543", "FIPS": "19187", "County": "Webster", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50544", "FIPS": "19187", "County": "Webster", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50557", "FIPS": "19187", "County": "Webster", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50566", "FIPS": "19187", "County": "Webster", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50569", "FIPS": "19187", "County": "Webster", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50594", "FIPS": "19187", "County": "Webster", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50424", "FIPS": "19189", "County": "Winnebago", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50436", "FIPS": "19189", "County": "Winnebago", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50450", "FIPS": "19189", "County": "Winnebago", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50453", "FIPS": "19189", "County": "Winnebago", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50465", "FIPS": "19189", "County": "Winnebago", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50473", "FIPS": "19189", "County": "Winnebago", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50478", "FIPS": "19189", "County": "Winnebago", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "52101", "FIPS": "19191", "County": "Winneshiek", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52132", "FIPS": "19191", "County": "Winneshiek", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52133", "FIPS": "19191", "County": "Winneshiek", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52144", "FIPS": "19191", "County": "Winneshiek", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52149", "FIPS": "19191", "County": "Winneshiek", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52161", "FIPS": "19191", "County": "Winneshiek", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52165", "FIPS": "19191", "County": "Winneshiek", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "52168", "FIPS": "19191", "County": "Winneshiek", "State": "IA", "DMA": {"Code": 637, "Name": "CEDAR RAPIDS-WATERLOO-DUBUQUE"}},
  {"Postal_Code": "51004", "FIPS": "19193", "County": "Woodbury", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51007", "FIPS": "19193", "County": "Woodbury", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51015", "FIPS": "19193", "County": "Woodbury", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51016", "FIPS": "19193", "County": "Woodbury", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51018", "FIPS": "19193", "County": "Woodbury", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51019", "FIPS": "19193", "County": "Woodbury", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51026", "FIPS": "19193", "County": "Woodbury", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51030", "FIPS": "19193", "County": "Woodbury", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51039", "FIPS": "19193", "County": "Woodbury", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51044", "FIPS": "19193", "County": "Woodbury", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51048", "FIPS": "19193", "County": "Woodbury", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51052", "FIPS": "19193", "County": "Woodbury", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51054", "FIPS": "19193", "County": "Woodbury", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51055", "FIPS": "19193", "County": "Woodbury", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51056", "FIPS": "19193", "County": "Woodbury", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51101", "FIPS": "19193", "County": "Woodbury", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51102", "FIPS": "19193", "County": "Woodbury", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51103", "FIPS": "19193", "County": "Woodbury", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51104", "FIPS": "19193", "County": "Woodbury", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51105", "FIPS": "19193", "County": "Woodbury", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51106", "FIPS": "19193", "County": "Woodbury", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51108", "FIPS": "19193", "County": "Woodbury", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51109", "FIPS": "19193", "County": "Woodbury", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "51111", "FIPS": "19193", "County": "Woodbury", "State": "IA", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "50434", "FIPS": "19195", "County": "Worth", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50440", "FIPS": "19195", "County": "Worth", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50444", "FIPS": "19195", "County": "Worth", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50446", "FIPS": "19195", "County": "Worth", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50448", "FIPS": "19195", "County": "Worth", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50456", "FIPS": "19195", "County": "Worth", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50459", "FIPS": "19195", "County": "Worth", "State": "IA", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "50071", "FIPS": "19197", "County": "Wright", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50101", "FIPS": "19197", "County": "Wright", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50421", "FIPS": "19197", "County": "Wright", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50470", "FIPS": "19197", "County": "Wright", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50525", "FIPS": "19197", "County": "Wright", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50526", "FIPS": "19197", "County": "Wright", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50533", "FIPS": "19197", "County": "Wright", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50542", "FIPS": "19197", "County": "Wright", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "50599", "FIPS": "19197", "County": "Wright", "State": "IA", "DMA": {"Code": 679, "Name": "DES MOINES-AMES"}},
  {"Postal_Code": "66732", "FIPS": "20001", "County": "Allen", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66742", "FIPS": "20001", "County": "Allen", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66748", "FIPS": "20001", "County": "Allen", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66749", "FIPS": "20001", "County": "Allen", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66751", "FIPS": "20001", "County": "Allen", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66755", "FIPS": "20001", "County": "Allen", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66772", "FIPS": "20001", "County": "Allen", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66014", "FIPS": "20003", "County": "Anderson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66015", "FIPS": "20003", "County": "Anderson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66032", "FIPS": "20003", "County": "Anderson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66033", "FIPS": "20003", "County": "Anderson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66039", "FIPS": "20003", "County": "Anderson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66091", "FIPS": "20003", "County": "Anderson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66093", "FIPS": "20003", "County": "Anderson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66002", "FIPS": "20005", "County": "Atchison", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66016", "FIPS": "20005", "County": "Atchison", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66023", "FIPS": "20005", "County": "Atchison", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66041", "FIPS": "20005", "County": "Atchison", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66058", "FIPS": "20005", "County": "Atchison", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66077", "FIPS": "20005", "County": "Atchison", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "67057", "FIPS": "20007", "County": "Barber", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67061", "FIPS": "20007", "County": "Barber", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67065", "FIPS": "20007", "County": "Barber", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67070", "FIPS": "20007", "County": "Barber", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67071", "FIPS": "20007", "County": "Barber", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67104", "FIPS": "20007", "County": "Barber", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67138", "FIPS": "20007", "County": "Barber", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67143", "FIPS": "20007", "County": "Barber", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67511", "FIPS": "20009", "County": "Barton", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67525", "FIPS": "20009", "County": "Barton", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67526", "FIPS": "20009", "County": "Barton", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67530", "FIPS": "20009", "County": "Barton", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67544", "FIPS": "20009", "County": "Barton", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67564", "FIPS": "20009", "County": "Barton", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67567", "FIPS": "20009", "County": "Barton", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66701", "FIPS": "20011", "County": "Bourbon", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66716", "FIPS": "20011", "County": "Bourbon", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66738", "FIPS": "20011", "County": "Bourbon", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66741", "FIPS": "20011", "County": "Bourbon", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66754", "FIPS": "20011", "County": "Bourbon", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66769", "FIPS": "20011", "County": "Bourbon", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66779", "FIPS": "20011", "County": "Bourbon", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66424", "FIPS": "20013", "County": "Brown", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66425", "FIPS": "20013", "County": "Brown", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66434", "FIPS": "20013", "County": "Brown", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66439", "FIPS": "20013", "County": "Brown", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66515", "FIPS": "20013", "County": "Brown", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66527", "FIPS": "20013", "County": "Brown", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66532", "FIPS": "20013", "County": "Brown", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66842", "FIPS": "20015", "County": "Butler", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67002", "FIPS": "20015", "County": "Butler", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67010", "FIPS": "20015", "County": "Butler", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67012", "FIPS": "20015", "County": "Butler", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67017", "FIPS": "20015", "County": "Butler", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67039", "FIPS": "20015", "County": "Butler", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67041", "FIPS": "20015", "County": "Butler", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67042", "FIPS": "20015", "County": "Butler", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67072", "FIPS": "20015", "County": "Butler", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67074", "FIPS": "20015", "County": "Butler", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67123", "FIPS": "20015", "County": "Butler", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67132", "FIPS": "20015", "County": "Butler", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67133", "FIPS": "20015", "County": "Butler", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67144", "FIPS": "20015", "County": "Butler", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67154", "FIPS": "20015", "County": "Butler", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66843", "FIPS": "20017", "County": "Chase", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66845", "FIPS": "20017", "County": "Chase", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66850", "FIPS": "20017", "County": "Chase", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66862", "FIPS": "20017", "County": "Chase", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66869", "FIPS": "20017", "County": "Chase", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67024", "FIPS": "20019", "County": "Chautauqua", "State": "KS", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "67334", "FIPS": "20019", "County": "Chautauqua", "State": "KS", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "67355", "FIPS": "20019", "County": "Chautauqua", "State": "KS", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "67360", "FIPS": "20019", "County": "Chautauqua", "State": "KS", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "67361", "FIPS": "20019", "County": "Chautauqua", "State": "KS", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "66713", "FIPS": "20021", "County": "Cherokee", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66725", "FIPS": "20021", "County": "Cherokee", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66728", "FIPS": "20021", "County": "Cherokee", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66739", "FIPS": "20021", "County": "Cherokee", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66770", "FIPS": "20021", "County": "Cherokee", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66773", "FIPS": "20021", "County": "Cherokee", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66778", "FIPS": "20021", "County": "Cherokee", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66781", "FIPS": "20021", "County": "Cherokee", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66782", "FIPS": "20021", "County": "Cherokee", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "67731", "FIPS": "20023", "County": "Cheyenne", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67756", "FIPS": "20023", "County": "Cheyenne", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67831", "FIPS": "20025", "County": "Clark", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67840", "FIPS": "20025", "County": "Clark", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67865", "FIPS": "20025", "County": "Clark", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67432", "FIPS": "20027", "County": "Clay", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "67447", "FIPS": "20027", "County": "Clay", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "67458", "FIPS": "20027", "County": "Clay", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "67468", "FIPS": "20027", "County": "Clay", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "67487", "FIPS": "20027", "County": "Clay", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66901", "FIPS": "20029", "County": "Cloud", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66938", "FIPS": "20029", "County": "Cloud", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66948", "FIPS": "20029", "County": "Cloud", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "67417", "FIPS": "20029", "County": "Cloud", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "67445", "FIPS": "20029", "County": "Cloud", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "67466", "FIPS": "20029", "County": "Cloud", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66758", "FIPS": "20031", "County": "Coffey", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66839", "FIPS": "20031", "County": "Coffey", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66852", "FIPS": "20031", "County": "Coffey", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66856", "FIPS": "20031", "County": "Coffey", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66857", "FIPS": "20031", "County": "Coffey", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66871", "FIPS": "20031", "County": "Coffey", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "67029", "FIPS": "20033", "County": "Comanche", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67127", "FIPS": "20033", "County": "Comanche", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67155", "FIPS": "20033", "County": "Comanche", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67005", "FIPS": "20035", "County": "Cowley", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67008", "FIPS": "20035", "County": "Cowley", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67019", "FIPS": "20035", "County": "Cowley", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67023", "FIPS": "20035", "County": "Cowley", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67038", "FIPS": "20035", "County": "Cowley", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67102", "FIPS": "20035", "County": "Cowley", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67131", "FIPS": "20035", "County": "Cowley", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67146", "FIPS": "20035", "County": "Cowley", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67156", "FIPS": "20035", "County": "Cowley", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66711", "FIPS": "20037", "County": "Crawford", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66712", "FIPS": "20037", "County": "Crawford", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66724", "FIPS": "20037", "County": "Crawford", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66734", "FIPS": "20037", "County": "Crawford", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66735", "FIPS": "20037", "County": "Crawford", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66743", "FIPS": "20037", "County": "Crawford", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66746", "FIPS": "20037", "County": "Crawford", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66753", "FIPS": "20037", "County": "Crawford", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66756", "FIPS": "20037", "County": "Crawford", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66760", "FIPS": "20037", "County": "Crawford", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66762", "FIPS": "20037", "County": "Crawford", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66763", "FIPS": "20037", "County": "Crawford", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66780", "FIPS": "20037", "County": "Crawford", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "67635", "FIPS": "20039", "County": "Decatur", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67643", "FIPS": "20039", "County": "Decatur", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67653", "FIPS": "20039", "County": "Decatur", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67749", "FIPS": "20039", "County": "Decatur", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67410", "FIPS": "20041", "County": "Dickinson", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67431", "FIPS": "20041", "County": "Dickinson", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67441", "FIPS": "20041", "County": "Dickinson", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67449", "FIPS": "20041", "County": "Dickinson", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67451", "FIPS": "20041", "County": "Dickinson", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67480", "FIPS": "20041", "County": "Dickinson", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67482", "FIPS": "20041", "County": "Dickinson", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67492", "FIPS": "20041", "County": "Dickinson", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66008", "FIPS": "20043", "County": "Doniphan", "State": "KS", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "66017", "FIPS": "20043", "County": "Doniphan", "State": "KS", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "66024", "FIPS": "20043", "County": "Doniphan", "State": "KS", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "66035", "FIPS": "20043", "County": "Doniphan", "State": "KS", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "66087", "FIPS": "20043", "County": "Doniphan", "State": "KS", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "66090", "FIPS": "20043", "County": "Doniphan", "State": "KS", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "66094", "FIPS": "20043", "County": "Doniphan", "State": "KS", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "66006", "FIPS": "20045", "County": "Douglas", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66025", "FIPS": "20045", "County": "Douglas", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66044", "FIPS": "20045", "County": "Douglas", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66045", "FIPS": "20045", "County": "Douglas", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66046", "FIPS": "20045", "County": "Douglas", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66047", "FIPS": "20045", "County": "Douglas", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66049", "FIPS": "20045", "County": "Douglas", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "67519", "FIPS": "20047", "County": "Edwards", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67547", "FIPS": "20047", "County": "Edwards", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67552", "FIPS": "20047", "County": "Edwards", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67563", "FIPS": "20047", "County": "Edwards", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67345", "FIPS": "20049", "County": "Elk", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67346", "FIPS": "20049", "County": "Elk", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67349", "FIPS": "20049", "County": "Elk", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67352", "FIPS": "20049", "County": "Elk", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67353", "FIPS": "20049", "County": "Elk", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67601", "FIPS": "20051", "County": "Ellis", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67627", "FIPS": "20051", "County": "Ellis", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67637", "FIPS": "20051", "County": "Ellis", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67660", "FIPS": "20051", "County": "Ellis", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67667", "FIPS": "20051", "County": "Ellis", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67671", "FIPS": "20051", "County": "Ellis", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67674", "FIPS": "20051", "County": "Ellis", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67439", "FIPS": "20053", "County": "Ellsworth", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67450", "FIPS": "20053", "County": "Ellsworth", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67454", "FIPS": "20053", "County": "Ellsworth", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67459", "FIPS": "20053", "County": "Ellsworth", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67490", "FIPS": "20053", "County": "Ellsworth", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67846", "FIPS": "20055", "County": "Finney", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67851", "FIPS": "20055", "County": "Finney", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67868", "FIPS": "20055", "County": "Finney", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67801", "FIPS": "20057", "County": "Ford", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67834", "FIPS": "20057", "County": "Ford", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67842", "FIPS": "20057", "County": "Ford", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67843", "FIPS": "20057", "County": "Ford", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67876", "FIPS": "20057", "County": "Ford", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67882", "FIPS": "20057", "County": "Ford", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66042", "FIPS": "20059", "County": "Franklin", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66067", "FIPS": "20059", "County": "Franklin", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66076", "FIPS": "20059", "County": "Franklin", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66078", "FIPS": "20059", "County": "Franklin", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66079", "FIPS": "20059", "County": "Franklin", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66080", "FIPS": "20059", "County": "Franklin", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66092", "FIPS": "20059", "County": "Franklin", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66095", "FIPS": "20059", "County": "Franklin", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66441", "FIPS": "20061", "County": "Geary", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66442", "FIPS": "20061", "County": "Geary", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66514", "FIPS": "20061", "County": "Geary", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "67736", "FIPS": "20063", "County": "Gove", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67737", "FIPS": "20063", "County": "Gove", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67738", "FIPS": "20063", "County": "Gove", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67751", "FIPS": "20063", "County": "Gove", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67752", "FIPS": "20063", "County": "Gove", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67625", "FIPS": "20065", "County": "Graham", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67642", "FIPS": "20065", "County": "Graham", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67650", "FIPS": "20065", "County": "Graham", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67659", "FIPS": "20065", "County": "Graham", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67880", "FIPS": "20067", "County": "Grant", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67835", "FIPS": "20069", "County": "Gray", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67837", "FIPS": "20069", "County": "Gray", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67841", "FIPS": "20069", "County": "Gray", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67853", "FIPS": "20069", "County": "Gray", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67867", "FIPS": "20069", "County": "Gray", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67879", "FIPS": "20071", "County": "Greeley", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66853", "FIPS": "20073", "County": "Greenwood", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66855", "FIPS": "20073", "County": "Greenwood", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66860", "FIPS": "20073", "County": "Greenwood", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66863", "FIPS": "20073", "County": "Greenwood", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66870", "FIPS": "20073", "County": "Greenwood", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67045", "FIPS": "20073", "County": "Greenwood", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67047", "FIPS": "20073", "County": "Greenwood", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67122", "FIPS": "20073", "County": "Greenwood", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67137", "FIPS": "20073", "County": "Greenwood", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67836", "FIPS": "20075", "County": "Hamilton", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67857", "FIPS": "20075", "County": "Hamilton", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67878", "FIPS": "20075", "County": "Hamilton", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67003", "FIPS": "20077", "County": "Harper", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67009", "FIPS": "20077", "County": "Harper", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67018", "FIPS": "20077", "County": "Harper", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67036", "FIPS": "20077", "County": "Harper", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67049", "FIPS": "20077", "County": "Harper", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67058", "FIPS": "20077", "County": "Harper", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67150", "FIPS": "20077", "County": "Harper", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67020", "FIPS": "20079", "County": "Harvey", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67056", "FIPS": "20079", "County": "Harvey", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67062", "FIPS": "20079", "County": "Harvey", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67114", "FIPS": "20079", "County": "Harvey", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67117", "FIPS": "20079", "County": "Harvey", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67135", "FIPS": "20079", "County": "Harvey", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67151", "FIPS": "20079", "County": "Harvey", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67870", "FIPS": "20081", "County": "Haskell", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67877", "FIPS": "20081", "County": "Haskell", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67849", "FIPS": "20083", "County": "Hodgeman", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67854", "FIPS": "20083", "County": "Hodgeman", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66416", "FIPS": "20085", "County": "Jackson", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66418", "FIPS": "20085", "County": "Jackson", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66419", "FIPS": "20085", "County": "Jackson", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66436", "FIPS": "20085", "County": "Jackson", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66440", "FIPS": "20085", "County": "Jackson", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66509", "FIPS": "20085", "County": "Jackson", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66516", "FIPS": "20085", "County": "Jackson", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66540", "FIPS": "20085", "County": "Jackson", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66552", "FIPS": "20085", "County": "Jackson", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66054", "FIPS": "20087", "County": "Jefferson", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66060", "FIPS": "20087", "County": "Jefferson", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66066", "FIPS": "20087", "County": "Jefferson", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66070", "FIPS": "20087", "County": "Jefferson", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66073", "FIPS": "20087", "County": "Jefferson", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66088", "FIPS": "20087", "County": "Jefferson", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66097", "FIPS": "20087", "County": "Jefferson", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66429", "FIPS": "20087", "County": "Jefferson", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66512", "FIPS": "20087", "County": "Jefferson", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66936", "FIPS": "20089", "County": "Jewell", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "66941", "FIPS": "20089", "County": "Jewell", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "66942", "FIPS": "20089", "County": "Jewell", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "66949", "FIPS": "20089", "County": "Jewell", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "66956", "FIPS": "20089", "County": "Jewell", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "66963", "FIPS": "20089", "County": "Jewell", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "66970", "FIPS": "20089", "County": "Jewell", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "66018", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66019", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66021", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66030", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66031", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66051", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66061", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66062", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66063", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66083", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66085", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66201", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66202", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66203", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66204", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66205", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66206", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66207", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66208", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66209", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66210", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66211", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66212", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66213", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66214", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66215", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66216", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66217", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66218", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66219", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66220", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66221", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66222", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66223", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66224", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66225", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66226", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66227", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66250", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66251", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66276", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66279", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66282", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66283", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66285", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66286", "FIPS": "20091", "County": "Johnson", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "67838", "FIPS": "20093", "County": "Kearny", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67860", "FIPS": "20093", "County": "Kearny", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67035", "FIPS": "20095", "County": "Kingman", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67068", "FIPS": "20095", "County": "Kingman", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67111", "FIPS": "20095", "County": "Kingman", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67112", "FIPS": "20095", "County": "Kingman", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67118", "FIPS": "20095", "County": "Kingman", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67142", "FIPS": "20095", "County": "Kingman", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67159", "FIPS": "20095", "County": "Kingman", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67054", "FIPS": "20097", "County": "Kiowa", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67059", "FIPS": "20097", "County": "Kiowa", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67109", "FIPS": "20097", "County": "Kiowa", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67330", "FIPS": "20099", "County": "Labette", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "67332", "FIPS": "20099", "County": "Labette", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "67336", "FIPS": "20099", "County": "Labette", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "67341", "FIPS": "20099", "County": "Labette", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "67342", "FIPS": "20099", "County": "Labette", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "67354", "FIPS": "20099", "County": "Labette", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "67356", "FIPS": "20099", "County": "Labette", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "67357", "FIPS": "20099", "County": "Labette", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "67839", "FIPS": "20101", "County": "Lane", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67850", "FIPS": "20101", "County": "Lane", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66007", "FIPS": "20103", "County": "Leavenworth", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66020", "FIPS": "20103", "County": "Leavenworth", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66027", "FIPS": "20103", "County": "Leavenworth", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66043", "FIPS": "20103", "County": "Leavenworth", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66048", "FIPS": "20103", "County": "Leavenworth", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66052", "FIPS": "20103", "County": "Leavenworth", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66086", "FIPS": "20103", "County": "Leavenworth", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "67418", "FIPS": "20105", "County": "Lincoln", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67423", "FIPS": "20105", "County": "Lincoln", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67455", "FIPS": "20105", "County": "Lincoln", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67481", "FIPS": "20105", "County": "Lincoln", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66010", "FIPS": "20107", "County": "Linn", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66040", "FIPS": "20107", "County": "Linn", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66056", "FIPS": "20107", "County": "Linn", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66072", "FIPS": "20107", "County": "Linn", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66075", "FIPS": "20107", "County": "Linn", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66767", "FIPS": "20107", "County": "Linn", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "67747", "FIPS": "20109", "County": "Logan", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67748", "FIPS": "20109", "County": "Logan", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67764", "FIPS": "20109", "County": "Logan", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66801", "FIPS": "20111", "County": "Lyon", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66830", "FIPS": "20111", "County": "Lyon", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66833", "FIPS": "20111", "County": "Lyon", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66835", "FIPS": "20111", "County": "Lyon", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66854", "FIPS": "20111", "County": "Lyon", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66864", "FIPS": "20111", "County": "Lyon", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66865", "FIPS": "20111", "County": "Lyon", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66868", "FIPS": "20111", "County": "Lyon", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "67107", "FIPS": "20113", "County": "McPherson", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67428", "FIPS": "20113", "County": "McPherson", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67443", "FIPS": "20113", "County": "McPherson", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67456", "FIPS": "20113", "County": "McPherson", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67460", "FIPS": "20113", "County": "McPherson", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67464", "FIPS": "20113", "County": "McPherson", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67476", "FIPS": "20113", "County": "McPherson", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67491", "FIPS": "20113", "County": "McPherson", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67546", "FIPS": "20113", "County": "McPherson", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66840", "FIPS": "20115", "County": "Marion", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66851", "FIPS": "20115", "County": "Marion", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66858", "FIPS": "20115", "County": "Marion", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66859", "FIPS": "20115", "County": "Marion", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66861", "FIPS": "20115", "County": "Marion", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66866", "FIPS": "20115", "County": "Marion", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67053", "FIPS": "20115", "County": "Marion", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67063", "FIPS": "20115", "County": "Marion", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67073", "FIPS": "20115", "County": "Marion", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67438", "FIPS": "20115", "County": "Marion", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67475", "FIPS": "20115", "County": "Marion", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67483", "FIPS": "20115", "County": "Marion", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66403", "FIPS": "20117", "County": "Marshall", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66406", "FIPS": "20117", "County": "Marshall", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66411", "FIPS": "20117", "County": "Marshall", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66412", "FIPS": "20117", "County": "Marshall", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66427", "FIPS": "20117", "County": "Marshall", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66438", "FIPS": "20117", "County": "Marshall", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66508", "FIPS": "20117", "County": "Marshall", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66518", "FIPS": "20117", "County": "Marshall", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66541", "FIPS": "20117", "County": "Marshall", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66544", "FIPS": "20117", "County": "Marshall", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66548", "FIPS": "20117", "County": "Marshall", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "67844", "FIPS": "20119", "County": "Meade", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67864", "FIPS": "20119", "County": "Meade", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67869", "FIPS": "20119", "County": "Meade", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66013", "FIPS": "20121", "County": "Miami", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66026", "FIPS": "20121", "County": "Miami", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66036", "FIPS": "20121", "County": "Miami", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66053", "FIPS": "20121", "County": "Miami", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66064", "FIPS": "20121", "County": "Miami", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66071", "FIPS": "20121", "County": "Miami", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "67420", "FIPS": "20123", "County": "Mitchell", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67430", "FIPS": "20123", "County": "Mitchell", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67446", "FIPS": "20123", "County": "Mitchell", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67452", "FIPS": "20123", "County": "Mitchell", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67478", "FIPS": "20123", "County": "Mitchell", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67485", "FIPS": "20123", "County": "Mitchell", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67301", "FIPS": "20125", "County": "Montgomery", "State": "KS", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "67333", "FIPS": "20125", "County": "Montgomery", "State": "KS", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "67335", "FIPS": "20125", "County": "Montgomery", "State": "KS", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "67337", "FIPS": "20125", "County": "Montgomery", "State": "KS", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "67340", "FIPS": "20125", "County": "Montgomery", "State": "KS", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "67344", "FIPS": "20125", "County": "Montgomery", "State": "KS", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "67347", "FIPS": "20125", "County": "Montgomery", "State": "KS", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "67351", "FIPS": "20125", "County": "Montgomery", "State": "KS", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "67363", "FIPS": "20125", "County": "Montgomery", "State": "KS", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "67364", "FIPS": "20125", "County": "Montgomery", "State": "KS", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "66838", "FIPS": "20127", "County": "Morris", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66846", "FIPS": "20127", "County": "Morris", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66849", "FIPS": "20127", "County": "Morris", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66872", "FIPS": "20127", "County": "Morris", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66873", "FIPS": "20127", "County": "Morris", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "67950", "FIPS": "20129", "County": "Morton", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67953", "FIPS": "20129", "County": "Morton", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67954", "FIPS": "20129", "County": "Morton", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66404", "FIPS": "20131", "County": "Nemaha", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66408", "FIPS": "20131", "County": "Nemaha", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66415", "FIPS": "20131", "County": "Nemaha", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66417", "FIPS": "20131", "County": "Nemaha", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66428", "FIPS": "20131", "County": "Nemaha", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66522", "FIPS": "20131", "County": "Nemaha", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66534", "FIPS": "20131", "County": "Nemaha", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66538", "FIPS": "20131", "County": "Nemaha", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66550", "FIPS": "20131", "County": "Nemaha", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66720", "FIPS": "20133", "County": "Neosho", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66733", "FIPS": "20133", "County": "Neosho", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66740", "FIPS": "20133", "County": "Neosho", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66771", "FIPS": "20133", "County": "Neosho", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66775", "FIPS": "20133", "County": "Neosho", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66776", "FIPS": "20133", "County": "Neosho", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "67515", "FIPS": "20135", "County": "Ness", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67516", "FIPS": "20135", "County": "Ness", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67518", "FIPS": "20135", "County": "Ness", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67521", "FIPS": "20135", "County": "Ness", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67560", "FIPS": "20135", "County": "Ness", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67572", "FIPS": "20135", "County": "Ness", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67584", "FIPS": "20135", "County": "Ness", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67622", "FIPS": "20137", "County": "Norton", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67629", "FIPS": "20137", "County": "Norton", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67645", "FIPS": "20137", "County": "Norton", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67654", "FIPS": "20137", "County": "Norton", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66413", "FIPS": "20139", "County": "Osage", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66414", "FIPS": "20139", "County": "Osage", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66451", "FIPS": "20139", "County": "Osage", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66510", "FIPS": "20139", "County": "Osage", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66523", "FIPS": "20139", "County": "Osage", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66524", "FIPS": "20139", "County": "Osage", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66528", "FIPS": "20139", "County": "Osage", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66537", "FIPS": "20139", "County": "Osage", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66543", "FIPS": "20139", "County": "Osage", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "67437", "FIPS": "20141", "County": "Osborne", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67473", "FIPS": "20141", "County": "Osborne", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67474", "FIPS": "20141", "County": "Osborne", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67623", "FIPS": "20141", "County": "Osborne", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67651", "FIPS": "20141", "County": "Osborne", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67422", "FIPS": "20143", "County": "Ottawa", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67436", "FIPS": "20143", "County": "Ottawa", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67467", "FIPS": "20143", "County": "Ottawa", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67484", "FIPS": "20143", "County": "Ottawa", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67523", "FIPS": "20145", "County": "Pawnee", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67529", "FIPS": "20145", "County": "Pawnee", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67550", "FIPS": "20145", "County": "Pawnee", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67574", "FIPS": "20145", "County": "Pawnee", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67621", "FIPS": "20147", "County": "Phillips", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "67639", "FIPS": "20147", "County": "Phillips", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "67644", "FIPS": "20147", "County": "Phillips", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "67646", "FIPS": "20147", "County": "Phillips", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "67647", "FIPS": "20147", "County": "Phillips", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "67661", "FIPS": "20147", "County": "Phillips", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "67664", "FIPS": "20147", "County": "Phillips", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "66407", "FIPS": "20149", "County": "Pottawatomie", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66422", "FIPS": "20149", "County": "Pottawatomie", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66426", "FIPS": "20149", "County": "Pottawatomie", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66432", "FIPS": "20149", "County": "Pottawatomie", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66520", "FIPS": "20149", "County": "Pottawatomie", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66521", "FIPS": "20149", "County": "Pottawatomie", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66535", "FIPS": "20149", "County": "Pottawatomie", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66536", "FIPS": "20149", "County": "Pottawatomie", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66547", "FIPS": "20149", "County": "Pottawatomie", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66549", "FIPS": "20149", "County": "Pottawatomie", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "67021", "FIPS": "20151", "County": "Pratt", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67028", "FIPS": "20151", "County": "Pratt", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67066", "FIPS": "20151", "County": "Pratt", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67124", "FIPS": "20151", "County": "Pratt", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67134", "FIPS": "20151", "County": "Pratt", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67730", "FIPS": "20153", "County": "Rawlins", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67739", "FIPS": "20153", "County": "Rawlins", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67744", "FIPS": "20153", "County": "Rawlins", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67745", "FIPS": "20153", "County": "Rawlins", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67501", "FIPS": "20155", "County": "Reno", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67502", "FIPS": "20155", "County": "Reno", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67504", "FIPS": "20155", "County": "Reno", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67505", "FIPS": "20155", "County": "Reno", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67510", "FIPS": "20155", "County": "Reno", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67514", "FIPS": "20155", "County": "Reno", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67522", "FIPS": "20155", "County": "Reno", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67543", "FIPS": "20155", "County": "Reno", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67561", "FIPS": "20155", "County": "Reno", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67566", "FIPS": "20155", "County": "Reno", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67568", "FIPS": "20155", "County": "Reno", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67570", "FIPS": "20155", "County": "Reno", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67581", "FIPS": "20155", "County": "Reno", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67583", "FIPS": "20155", "County": "Reno", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67585", "FIPS": "20155", "County": "Reno", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66930", "FIPS": "20157", "County": "Republic", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "66935", "FIPS": "20157", "County": "Republic", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "66939", "FIPS": "20157", "County": "Republic", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "66940", "FIPS": "20157", "County": "Republic", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "66959", "FIPS": "20157", "County": "Republic", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "66960", "FIPS": "20157", "County": "Republic", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "66961", "FIPS": "20157", "County": "Republic", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "66964", "FIPS": "20157", "County": "Republic", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "66966", "FIPS": "20157", "County": "Republic", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "67427", "FIPS": "20159", "County": "Rice", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67444", "FIPS": "20159", "County": "Rice", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67457", "FIPS": "20159", "County": "Rice", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67512", "FIPS": "20159", "County": "Rice", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67524", "FIPS": "20159", "County": "Rice", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67554", "FIPS": "20159", "County": "Rice", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67573", "FIPS": "20159", "County": "Rice", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67579", "FIPS": "20159", "County": "Rice", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66449", "FIPS": "20161", "County": "Riley", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66502", "FIPS": "20161", "County": "Riley", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66503", "FIPS": "20161", "County": "Riley", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66505", "FIPS": "20161", "County": "Riley", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66506", "FIPS": "20161", "County": "Riley", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66517", "FIPS": "20161", "County": "Riley", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66531", "FIPS": "20161", "County": "Riley", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66554", "FIPS": "20161", "County": "Riley", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "67632", "FIPS": "20163", "County": "Rooks", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67657", "FIPS": "20163", "County": "Rooks", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67663", "FIPS": "20163", "County": "Rooks", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67669", "FIPS": "20163", "County": "Rooks", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67675", "FIPS": "20163", "County": "Rooks", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67513", "FIPS": "20165", "County": "Rush", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67520", "FIPS": "20165", "County": "Rush", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67548", "FIPS": "20165", "County": "Rush", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67553", "FIPS": "20165", "County": "Rush", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67556", "FIPS": "20165", "County": "Rush", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67559", "FIPS": "20165", "County": "Rush", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67565", "FIPS": "20165", "County": "Rush", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67575", "FIPS": "20165", "County": "Rush", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67626", "FIPS": "20167", "County": "Russell", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67634", "FIPS": "20167", "County": "Russell", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67640", "FIPS": "20167", "County": "Russell", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67648", "FIPS": "20167", "County": "Russell", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67649", "FIPS": "20167", "County": "Russell", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67658", "FIPS": "20167", "County": "Russell", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67665", "FIPS": "20167", "County": "Russell", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67673", "FIPS": "20167", "County": "Russell", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67401", "FIPS": "20169", "County": "Saline", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67402", "FIPS": "20169", "County": "Saline", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67416", "FIPS": "20169", "County": "Saline", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67425", "FIPS": "20169", "County": "Saline", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67442", "FIPS": "20169", "County": "Saline", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67448", "FIPS": "20169", "County": "Saline", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67470", "FIPS": "20169", "County": "Saline", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67871", "FIPS": "20171", "County": "Scott", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67001", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67016", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67025", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67026", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67030", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67037", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67050", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67052", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67055", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67060", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67067", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67101", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67108", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67110", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67120", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67147", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67149", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67201", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67202", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67203", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67204", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67205", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67206", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67207", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67208", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67209", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67210", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67211", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67212", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67213", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67214", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67215", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67216", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67217", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67218", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67219", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67220", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67221", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67223", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67226", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67227", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67228", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67230", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67232", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67235", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67260", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67275", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67276", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67277", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67278", "FIPS": "20173", "County": "Sedgwick", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67859", "FIPS": "20175", "County": "Seward", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67901", "FIPS": "20175", "County": "Seward", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67905", "FIPS": "20175", "County": "Seward", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66050", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66402", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66409", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66420", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66533", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66539", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66542", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66546", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66601", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66603", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66604", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66605", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66606", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66607", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66608", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66609", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66610", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66611", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66612", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66614", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66615", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66616", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66617", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66618", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66619", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66620", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66621", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66622", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66624", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66625", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66626", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66628", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66629", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66636", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66637", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66642", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66647", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66652", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66653", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66667", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66675", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66683", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66692", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66699", "FIPS": "20177", "County": "Shawnee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "67740", "FIPS": "20179", "County": "Sheridan", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67757", "FIPS": "20179", "County": "Sheridan", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67733", "FIPS": "20181", "County": "Sherman", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67735", "FIPS": "20181", "County": "Sherman", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67741", "FIPS": "20181", "County": "Sherman", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66932", "FIPS": "20183", "County": "Smith", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "66951", "FIPS": "20183", "County": "Smith", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "66952", "FIPS": "20183", "County": "Smith", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "66967", "FIPS": "20183", "County": "Smith", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "67628", "FIPS": "20183", "County": "Smith", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "67638", "FIPS": "20183", "County": "Smith", "State": "KS", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "67545", "FIPS": "20185", "County": "Stafford", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67557", "FIPS": "20185", "County": "Stafford", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67576", "FIPS": "20185", "County": "Stafford", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67578", "FIPS": "20185", "County": "Stafford", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67855", "FIPS": "20187", "County": "Stanton", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67862", "FIPS": "20187", "County": "Stanton", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67951", "FIPS": "20189", "County": "Stevens", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67952", "FIPS": "20189", "County": "Stevens", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67004", "FIPS": "20191", "County": "Sumner", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67013", "FIPS": "20191", "County": "Sumner", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67022", "FIPS": "20191", "County": "Sumner", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67031", "FIPS": "20191", "County": "Sumner", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67051", "FIPS": "20191", "County": "Sumner", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67103", "FIPS": "20191", "County": "Sumner", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67105", "FIPS": "20191", "County": "Sumner", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67106", "FIPS": "20191", "County": "Sumner", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67119", "FIPS": "20191", "County": "Sumner", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67140", "FIPS": "20191", "County": "Sumner", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67152", "FIPS": "20191", "County": "Sumner", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67701", "FIPS": "20193", "County": "Thomas", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67732", "FIPS": "20193", "County": "Thomas", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67734", "FIPS": "20193", "County": "Thomas", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67743", "FIPS": "20193", "County": "Thomas", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67753", "FIPS": "20193", "County": "Thomas", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67631", "FIPS": "20195", "County": "Trego", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67656", "FIPS": "20195", "County": "Trego", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67672", "FIPS": "20195", "County": "Trego", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66401", "FIPS": "20197", "County": "Wabaunsee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66423", "FIPS": "20197", "County": "Wabaunsee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66431", "FIPS": "20197", "County": "Wabaunsee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66501", "FIPS": "20197", "County": "Wabaunsee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66507", "FIPS": "20197", "County": "Wabaunsee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66526", "FIPS": "20197", "County": "Wabaunsee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66834", "FIPS": "20197", "County": "Wabaunsee", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "67758", "FIPS": "20199", "County": "Wallace", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67761", "FIPS": "20199", "County": "Wallace", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67762", "FIPS": "20199", "County": "Wallace", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66933", "FIPS": "20201", "County": "Washington", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66937", "FIPS": "20201", "County": "Washington", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66943", "FIPS": "20201", "County": "Washington", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66944", "FIPS": "20201", "County": "Washington", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66945", "FIPS": "20201", "County": "Washington", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66946", "FIPS": "20201", "County": "Washington", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66953", "FIPS": "20201", "County": "Washington", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66955", "FIPS": "20201", "County": "Washington", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66958", "FIPS": "20201", "County": "Washington", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66962", "FIPS": "20201", "County": "Washington", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "66968", "FIPS": "20201", "County": "Washington", "State": "KS", "DMA": {"Code": 605, "Name": "TOPEKA"}},
  {"Postal_Code": "67861", "FIPS": "20203", "County": "Wichita", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "67863", "FIPS": "20203", "County": "Wichita", "State": "KS", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "66710", "FIPS": "20205", "County": "Wilson", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66714", "FIPS": "20205", "County": "Wilson", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66717", "FIPS": "20205", "County": "Wilson", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66736", "FIPS": "20205", "County": "Wilson", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66757", "FIPS": "20205", "County": "Wilson", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66759", "FIPS": "20205", "County": "Wilson", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66761", "FIPS": "20207", "County": "Woodson", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66777", "FIPS": "20207", "County": "Woodson", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66783", "FIPS": "20207", "County": "Woodson", "State": "KS", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "66012", "FIPS": "20209", "County": "Wyandotte", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66101", "FIPS": "20209", "County": "Wyandotte", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66102", "FIPS": "20209", "County": "Wyandotte", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66103", "FIPS": "20209", "County": "Wyandotte", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66104", "FIPS": "20209", "County": "Wyandotte", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66105", "FIPS": "20209", "County": "Wyandotte", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66106", "FIPS": "20209", "County": "Wyandotte", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66109", "FIPS": "20209", "County": "Wyandotte", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66110", "FIPS": "20209", "County": "Wyandotte", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66111", "FIPS": "20209", "County": "Wyandotte", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66112", "FIPS": "20209", "County": "Wyandotte", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66113", "FIPS": "20209", "County": "Wyandotte", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66115", "FIPS": "20209", "County": "Wyandotte", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66117", "FIPS": "20209", "County": "Wyandotte", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66118", "FIPS": "20209", "County": "Wyandotte", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66119", "FIPS": "20209", "County": "Wyandotte", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "66160", "FIPS": "20209", "County": "Wyandotte", "State": "KS", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "42715", "FIPS": "21001", "County": "Adair", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42720", "FIPS": "21001", "County": "Adair", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42728", "FIPS": "21001", "County": "Adair", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42741", "FIPS": "21001", "County": "Adair", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42742", "FIPS": "21001", "County": "Adair", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42753", "FIPS": "21001", "County": "Adair", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42120", "FIPS": "21003", "County": "Allen", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42153", "FIPS": "21003", "County": "Allen", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42164", "FIPS": "21003", "County": "Allen", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "40342", "FIPS": "21005", "County": "Anderson", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42022", "FIPS": "21007", "County": "Ballard", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42024", "FIPS": "21007", "County": "Ballard", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42056", "FIPS": "21007", "County": "Ballard", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42060", "FIPS": "21007", "County": "Ballard", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42087", "FIPS": "21007", "County": "Ballard", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42123", "FIPS": "21009", "County": "Barren", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42127", "FIPS": "21009", "County": "Barren", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42130", "FIPS": "21009", "County": "Barren", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42131", "FIPS": "21009", "County": "Barren", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42141", "FIPS": "21009", "County": "Barren", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42142", "FIPS": "21009", "County": "Barren", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42152", "FIPS": "21009", "County": "Barren", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42156", "FIPS": "21009", "County": "Barren", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42160", "FIPS": "21009", "County": "Barren", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "40334", "FIPS": "21011", "County": "Bath", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40358", "FIPS": "21011", "County": "Bath", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40360", "FIPS": "21011", "County": "Bath", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40366", "FIPS": "21011", "County": "Bath", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40371", "FIPS": "21011", "County": "Bath", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40374", "FIPS": "21011", "County": "Bath", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40813", "FIPS": "21013", "County": "Bell", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40845", "FIPS": "21013", "County": "Bell", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40856", "FIPS": "21013", "County": "Bell", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40902", "FIPS": "21013", "County": "Bell", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40913", "FIPS": "21013", "County": "Bell", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40939", "FIPS": "21013", "County": "Bell", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40940", "FIPS": "21013", "County": "Bell", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40955", "FIPS": "21013", "County": "Bell", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40958", "FIPS": "21013", "County": "Bell", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40965", "FIPS": "21013", "County": "Bell", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40977", "FIPS": "21013", "County": "Bell", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40988", "FIPS": "21013", "County": "Bell", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "41005", "FIPS": "21015", "County": "Boone", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41022", "FIPS": "21015", "County": "Boone", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41042", "FIPS": "21015", "County": "Boone", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41048", "FIPS": "21015", "County": "Boone", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41080", "FIPS": "21015", "County": "Boone", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41091", "FIPS": "21015", "County": "Boone", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41092", "FIPS": "21015", "County": "Boone", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41094", "FIPS": "21015", "County": "Boone", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45275", "FIPS": "21015", "County": "Boone", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "40348", "FIPS": "21017", "County": "Bourbon", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40357", "FIPS": "21017", "County": "Bourbon", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40361", "FIPS": "21017", "County": "Bourbon", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40362", "FIPS": "21017", "County": "Bourbon", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41101", "FIPS": "21019", "County": "Boyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41102", "FIPS": "21019", "County": "Boyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41105", "FIPS": "21019", "County": "Boyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41114", "FIPS": "21019", "County": "Boyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41129", "FIPS": "21019", "County": "Boyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41168", "FIPS": "21019", "County": "Boyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "40422", "FIPS": "21021", "County": "Boyle", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40423", "FIPS": "21021", "County": "Boyle", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40440", "FIPS": "21021", "County": "Boyle", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40452", "FIPS": "21021", "County": "Boyle", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40464", "FIPS": "21021", "County": "Boyle", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40468", "FIPS": "21021", "County": "Boyle", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41002", "FIPS": "21023", "County": "Bracken", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41004", "FIPS": "21023", "County": "Bracken", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41043", "FIPS": "21023", "County": "Bracken", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41044", "FIPS": "21023", "County": "Bracken", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41061", "FIPS": "21023", "County": "Bracken", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41307", "FIPS": "21025", "County": "Breathitt", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41310", "FIPS": "21025", "County": "Breathitt", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41317", "FIPS": "21025", "County": "Breathitt", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41339", "FIPS": "21025", "County": "Breathitt", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41348", "FIPS": "21025", "County": "Breathitt", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41366", "FIPS": "21025", "County": "Breathitt", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41385", "FIPS": "21025", "County": "Breathitt", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41390", "FIPS": "21025", "County": "Breathitt", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40111", "FIPS": "21027", "County": "Breckinridge", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40115", "FIPS": "21027", "County": "Breckinridge", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40140", "FIPS": "21027", "County": "Breckinridge", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40143", "FIPS": "21027", "County": "Breckinridge", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40144", "FIPS": "21027", "County": "Breckinridge", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40145", "FIPS": "21027", "County": "Breckinridge", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40146", "FIPS": "21027", "County": "Breckinridge", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40152", "FIPS": "21027", "County": "Breckinridge", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40153", "FIPS": "21027", "County": "Breckinridge", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40170", "FIPS": "21027", "County": "Breckinridge", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40171", "FIPS": "21027", "County": "Breckinridge", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40176", "FIPS": "21027", "County": "Breckinridge", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40178", "FIPS": "21027", "County": "Breckinridge", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40047", "FIPS": "21029", "County": "Bullitt", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40109", "FIPS": "21029", "County": "Bullitt", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40110", "FIPS": "21029", "County": "Bullitt", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40129", "FIPS": "21029", "County": "Bullitt", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40150", "FIPS": "21029", "County": "Bullitt", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40165", "FIPS": "21029", "County": "Bullitt", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42201", "FIPS": "21031", "County": "Butler", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42219", "FIPS": "21031", "County": "Butler", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42252", "FIPS": "21031", "County": "Butler", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42261", "FIPS": "21031", "County": "Butler", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42273", "FIPS": "21031", "County": "Butler", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42287", "FIPS": "21031", "County": "Butler", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42288", "FIPS": "21031", "County": "Butler", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42411", "FIPS": "21033", "County": "Caldwell", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42445", "FIPS": "21033", "County": "Caldwell", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42020", "FIPS": "21035", "County": "Calloway", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42036", "FIPS": "21035", "County": "Calloway", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42049", "FIPS": "21035", "County": "Calloway", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42054", "FIPS": "21035", "County": "Calloway", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42071", "FIPS": "21035", "County": "Calloway", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42076", "FIPS": "21035", "County": "Calloway", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "41001", "FIPS": "21037", "County": "Campbell", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41007", "FIPS": "21037", "County": "Campbell", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41059", "FIPS": "21037", "County": "Campbell", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41071", "FIPS": "21037", "County": "Campbell", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41072", "FIPS": "21037", "County": "Campbell", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41073", "FIPS": "21037", "County": "Campbell", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41074", "FIPS": "21037", "County": "Campbell", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41075", "FIPS": "21037", "County": "Campbell", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41076", "FIPS": "21037", "County": "Campbell", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41085", "FIPS": "21037", "County": "Campbell", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41099", "FIPS": "21037", "County": "Campbell", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "42021", "FIPS": "21039", "County": "Carlisle", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42023", "FIPS": "21039", "County": "Carlisle", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42035", "FIPS": "21039", "County": "Carlisle", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42070", "FIPS": "21039", "County": "Carlisle", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "41008", "FIPS": "21041", "County": "Carroll", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "41045", "FIPS": "21041", "County": "Carroll", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "41083", "FIPS": "21041", "County": "Carroll", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "41098", "FIPS": "21041", "County": "Carroll", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "41128", "FIPS": "21043", "County": "Carter", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41132", "FIPS": "21043", "County": "Carter", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41142", "FIPS": "21043", "County": "Carter", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41143", "FIPS": "21043", "County": "Carter", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41146", "FIPS": "21043", "County": "Carter", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41164", "FIPS": "21043", "County": "Carter", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41173", "FIPS": "21043", "County": "Carter", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41181", "FIPS": "21043", "County": "Carter", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "42516", "FIPS": "21045", "County": "Casey", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42528", "FIPS": "21045", "County": "Casey", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42539", "FIPS": "21045", "County": "Casey", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42541", "FIPS": "21045", "County": "Casey", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42565", "FIPS": "21045", "County": "Casey", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42566", "FIPS": "21045", "County": "Casey", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42217", "FIPS": "21047", "County": "Christian", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42221", "FIPS": "21047", "County": "Christian", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42223", "FIPS": "21047", "County": "Christian", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42232", "FIPS": "21047", "County": "Christian", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42236", "FIPS": "21047", "County": "Christian", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42240", "FIPS": "21047", "County": "Christian", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42241", "FIPS": "21047", "County": "Christian", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42254", "FIPS": "21047", "County": "Christian", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42262", "FIPS": "21047", "County": "Christian", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42266", "FIPS": "21047", "County": "Christian", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "40391", "FIPS": "21049", "County": "Clark", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40392", "FIPS": "21049", "County": "Clark", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40914", "FIPS": "21051", "County": "Clay", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40931", "FIPS": "21051", "County": "Clay", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40932", "FIPS": "21051", "County": "Clay", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40941", "FIPS": "21051", "County": "Clay", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40944", "FIPS": "21051", "County": "Clay", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40951", "FIPS": "21051", "County": "Clay", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40962", "FIPS": "21051", "County": "Clay", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40972", "FIPS": "21051", "County": "Clay", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40983", "FIPS": "21051", "County": "Clay", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42602", "FIPS": "21053", "County": "Clinton", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42603", "FIPS": "21053", "County": "Clinton", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42033", "FIPS": "21055", "County": "Crittenden", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42037", "FIPS": "21055", "County": "Crittenden", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42064", "FIPS": "21055", "County": "Crittenden", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42084", "FIPS": "21055", "County": "Crittenden", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42717", "FIPS": "21057", "County": "Cumberland", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42731", "FIPS": "21057", "County": "Cumberland", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42759", "FIPS": "21057", "County": "Cumberland", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42301", "FIPS": "21059", "County": "Daviess", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42302", "FIPS": "21059", "County": "Daviess", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42303", "FIPS": "21059", "County": "Daviess", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42304", "FIPS": "21059", "County": "Daviess", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42334", "FIPS": "21059", "County": "Daviess", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42355", "FIPS": "21059", "County": "Daviess", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42356", "FIPS": "21059", "County": "Daviess", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42366", "FIPS": "21059", "County": "Daviess", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42375", "FIPS": "21059", "County": "Daviess", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42376", "FIPS": "21059", "County": "Daviess", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42377", "FIPS": "21059", "County": "Daviess", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42378", "FIPS": "21059", "County": "Daviess", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42163", "FIPS": "21061", "County": "Edmonson", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42207", "FIPS": "21061", "County": "Edmonson", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42210", "FIPS": "21061", "County": "Edmonson", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42259", "FIPS": "21061", "County": "Edmonson", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42275", "FIPS": "21061", "County": "Edmonson", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42285", "FIPS": "21061", "County": "Edmonson", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "41149", "FIPS": "21063", "County": "Elliott", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41171", "FIPS": "21063", "County": "Elliott", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "40336", "FIPS": "21065", "County": "Estill", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40472", "FIPS": "21065", "County": "Estill", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40495", "FIPS": "21065", "County": "Estill", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40502", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40503", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40504", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40505", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40506", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40507", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40508", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40509", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40510", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40511", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40512", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40513", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40514", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40515", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40516", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40517", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40522", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40523", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40524", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40526", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40533", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40536", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40544", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40546", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40550", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40555", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40574", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40575", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40576", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40577", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40578", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40579", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40580", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40581", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40582", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40583", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40588", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40591", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40598", "FIPS": "21067", "County": "Fayette", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41037", "FIPS": "21069", "County": "Fleming", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41039", "FIPS": "21069", "County": "Fleming", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41041", "FIPS": "21069", "County": "Fleming", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41049", "FIPS": "21069", "County": "Fleming", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41065", "FIPS": "21069", "County": "Fleming", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41081", "FIPS": "21069", "County": "Fleming", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41093", "FIPS": "21069", "County": "Fleming", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41601", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41602", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41603", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41604", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41605", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41606", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41607", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41612", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41615", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41616", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41619", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41621", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41622", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41630", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41631", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41635", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41636", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41640", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41642", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41643", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41645", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41647", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41649", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41650", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41651", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41653", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41655", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41659", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41660", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41663", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41666", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41667", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41669", "FIPS": "21071", "County": "Floyd", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "40601", "FIPS": "21073", "County": "Franklin", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40602", "FIPS": "21073", "County": "Franklin", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40603", "FIPS": "21073", "County": "Franklin", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40604", "FIPS": "21073", "County": "Franklin", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40618", "FIPS": "21073", "County": "Franklin", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40619", "FIPS": "21073", "County": "Franklin", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40620", "FIPS": "21073", "County": "Franklin", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40621", "FIPS": "21073", "County": "Franklin", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40622", "FIPS": "21073", "County": "Franklin", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42041", "FIPS": "21075", "County": "Fulton", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42050", "FIPS": "21075", "County": "Fulton", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "41046", "FIPS": "21077", "County": "Gallatin", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41086", "FIPS": "21077", "County": "Gallatin", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41095", "FIPS": "21077", "County": "Gallatin", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "40410", "FIPS": "21079", "County": "Garrard", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40444", "FIPS": "21079", "County": "Garrard", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40446", "FIPS": "21079", "County": "Garrard", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40461", "FIPS": "21079", "County": "Garrard", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41010", "FIPS": "21081", "County": "Grant", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41030", "FIPS": "21081", "County": "Grant", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41035", "FIPS": "21081", "County": "Grant", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41052", "FIPS": "21081", "County": "Grant", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41054", "FIPS": "21081", "County": "Grant", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41097", "FIPS": "21081", "County": "Grant", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "42027", "FIPS": "21083", "County": "Graves", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42039", "FIPS": "21083", "County": "Graves", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42040", "FIPS": "21083", "County": "Graves", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42051", "FIPS": "21083", "County": "Graves", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42061", "FIPS": "21083", "County": "Graves", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42063", "FIPS": "21083", "County": "Graves", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42066", "FIPS": "21083", "County": "Graves", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42069", "FIPS": "21083", "County": "Graves", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42079", "FIPS": "21083", "County": "Graves", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42082", "FIPS": "21083", "County": "Graves", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42085", "FIPS": "21083", "County": "Graves", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42088", "FIPS": "21083", "County": "Graves", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "40119", "FIPS": "21085", "County": "Grayson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42712", "FIPS": "21085", "County": "Grayson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42721", "FIPS": "21085", "County": "Grayson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42726", "FIPS": "21085", "County": "Grayson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42754", "FIPS": "21085", "County": "Grayson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42755", "FIPS": "21085", "County": "Grayson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42762", "FIPS": "21085", "County": "Grayson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42743", "FIPS": "21087", "County": "Green", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42782", "FIPS": "21087", "County": "Green", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "41121", "FIPS": "21089", "County": "Greenup", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41139", "FIPS": "21089", "County": "Greenup", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41144", "FIPS": "21089", "County": "Greenup", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41169", "FIPS": "21089", "County": "Greenup", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41174", "FIPS": "21089", "County": "Greenup", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41175", "FIPS": "21089", "County": "Greenup", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41183", "FIPS": "21089", "County": "Greenup", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "42348", "FIPS": "21091", "County": "Hancock", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42351", "FIPS": "21091", "County": "Hancock", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42364", "FIPS": "21091", "County": "Hancock", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42368", "FIPS": "21091", "County": "Hancock", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "40121", "FIPS": "21093", "County": "Hardin", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40159", "FIPS": "21093", "County": "Hardin", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40160", "FIPS": "21093", "County": "Hardin", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40162", "FIPS": "21093", "County": "Hardin", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40175", "FIPS": "21093", "County": "Hardin", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40177", "FIPS": "21093", "County": "Hardin", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42701", "FIPS": "21093", "County": "Hardin", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42702", "FIPS": "21093", "County": "Hardin", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42724", "FIPS": "21093", "County": "Hardin", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42732", "FIPS": "21093", "County": "Hardin", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42740", "FIPS": "21093", "County": "Hardin", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42776", "FIPS": "21093", "County": "Hardin", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42784", "FIPS": "21093", "County": "Hardin", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42788", "FIPS": "21093", "County": "Hardin", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40801", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40806", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40807", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40808", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40810", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40815", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40818", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40819", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40820", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40823", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40824", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40828", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40829", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40830", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40831", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40843", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40847", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40849", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40854", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40855", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40863", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40865", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40870", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40873", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40927", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "40964", "FIPS": "21095", "County": "Harlan", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "41003", "FIPS": "21097", "County": "Harrison", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41031", "FIPS": "21097", "County": "Harrison", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42713", "FIPS": "21099", "County": "Hart", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42722", "FIPS": "21099", "County": "Hart", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42729", "FIPS": "21099", "County": "Hart", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42746", "FIPS": "21099", "County": "Hart", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42749", "FIPS": "21099", "County": "Hart", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42765", "FIPS": "21099", "County": "Hart", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42402", "FIPS": "21101", "County": "Henderson", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42406", "FIPS": "21101", "County": "Henderson", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42419", "FIPS": "21101", "County": "Henderson", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42420", "FIPS": "21101", "County": "Henderson", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42451", "FIPS": "21101", "County": "Henderson", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42452", "FIPS": "21101", "County": "Henderson", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42457", "FIPS": "21101", "County": "Henderson", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42458", "FIPS": "21101", "County": "Henderson", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "40007", "FIPS": "21103", "County": "Henry", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40011", "FIPS": "21103", "County": "Henry", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40019", "FIPS": "21103", "County": "Henry", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40036", "FIPS": "21103", "County": "Henry", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40050", "FIPS": "21103", "County": "Henry", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40055", "FIPS": "21103", "County": "Henry", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40057", "FIPS": "21103", "County": "Henry", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40058", "FIPS": "21103", "County": "Henry", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40068", "FIPS": "21103", "County": "Henry", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40070", "FIPS": "21103", "County": "Henry", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40075", "FIPS": "21103", "County": "Henry", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42031", "FIPS": "21105", "County": "Hickman", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42032", "FIPS": "21105", "County": "Hickman", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42408", "FIPS": "21107", "County": "Hopkins", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42410", "FIPS": "21107", "County": "Hopkins", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42413", "FIPS": "21107", "County": "Hopkins", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42431", "FIPS": "21107", "County": "Hopkins", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42436", "FIPS": "21107", "County": "Hopkins", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42440", "FIPS": "21107", "County": "Hopkins", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42441", "FIPS": "21107", "County": "Hopkins", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42442", "FIPS": "21107", "County": "Hopkins", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42453", "FIPS": "21107", "County": "Hopkins", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42464", "FIPS": "21107", "County": "Hopkins", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "40402", "FIPS": "21109", "County": "Jackson", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40434", "FIPS": "21109", "County": "Jackson", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40447", "FIPS": "21109", "County": "Jackson", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40481", "FIPS": "21109", "County": "Jackson", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40486", "FIPS": "21109", "County": "Jackson", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40488", "FIPS": "21109", "County": "Jackson", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40018", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40023", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40025", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40027", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40041", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40059", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40118", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40201", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40202", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40203", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40204", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40205", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40206", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40207", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40208", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40209", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40210", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40211", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40212", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40213", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40214", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40215", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40216", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40217", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40218", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40219", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40220", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40221", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40222", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40223", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40224", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40225", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40228", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40229", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40231", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40232", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40233", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40241", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40242", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40243", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40245", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40250", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40251", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40252", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40253", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40255", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40256", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40257", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40258", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40259", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40261", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40266", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40268", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40269", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40270", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40272", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40280", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40281", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40282", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40283", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40285", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40287", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40289", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40290", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40291", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40292", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40293", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40294", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40295", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40296", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40297", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40298", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40299", "FIPS": "21111", "County": "Jefferson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40339", "FIPS": "21113", "County": "Jessamine", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40340", "FIPS": "21113", "County": "Jessamine", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40356", "FIPS": "21113", "County": "Jessamine", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40390", "FIPS": "21113", "County": "Jessamine", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41204", "FIPS": "21115", "County": "Johnson", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41216", "FIPS": "21115", "County": "Johnson", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41219", "FIPS": "21115", "County": "Johnson", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41222", "FIPS": "21115", "County": "Johnson", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41226", "FIPS": "21115", "County": "Johnson", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41234", "FIPS": "21115", "County": "Johnson", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41238", "FIPS": "21115", "County": "Johnson", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41240", "FIPS": "21115", "County": "Johnson", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41254", "FIPS": "21115", "County": "Johnson", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41255", "FIPS": "21115", "County": "Johnson", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41256", "FIPS": "21115", "County": "Johnson", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41257", "FIPS": "21115", "County": "Johnson", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41260", "FIPS": "21115", "County": "Johnson", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41263", "FIPS": "21115", "County": "Johnson", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41265", "FIPS": "21115", "County": "Johnson", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41268", "FIPS": "21115", "County": "Johnson", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41271", "FIPS": "21115", "County": "Johnson", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41274", "FIPS": "21115", "County": "Johnson", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41011", "FIPS": "21117", "County": "Kenton", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41012", "FIPS": "21117", "County": "Kenton", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41014", "FIPS": "21117", "County": "Kenton", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41015", "FIPS": "21117", "County": "Kenton", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41016", "FIPS": "21117", "County": "Kenton", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41017", "FIPS": "21117", "County": "Kenton", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41018", "FIPS": "21117", "County": "Kenton", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41019", "FIPS": "21117", "County": "Kenton", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41051", "FIPS": "21117", "County": "Kenton", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41053", "FIPS": "21117", "County": "Kenton", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41063", "FIPS": "21117", "County": "Kenton", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41725", "FIPS": "21119", "County": "Knott", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41740", "FIPS": "21119", "County": "Knott", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41743", "FIPS": "21119", "County": "Knott", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41759", "FIPS": "21119", "County": "Knott", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41772", "FIPS": "21119", "County": "Knott", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41817", "FIPS": "21119", "County": "Knott", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41822", "FIPS": "21119", "County": "Knott", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41828", "FIPS": "21119", "County": "Knott", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41831", "FIPS": "21119", "County": "Knott", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41834", "FIPS": "21119", "County": "Knott", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41836", "FIPS": "21119", "County": "Knott", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41839", "FIPS": "21119", "County": "Knott", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41843", "FIPS": "21119", "County": "Knott", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41844", "FIPS": "21119", "County": "Knott", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41847", "FIPS": "21119", "County": "Knott", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41859", "FIPS": "21119", "County": "Knott", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41861", "FIPS": "21119", "County": "Knott", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41862", "FIPS": "21119", "County": "Knott", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40734", "FIPS": "21121", "County": "Knox", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40771", "FIPS": "21121", "County": "Knox", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40903", "FIPS": "21121", "County": "Knox", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40906", "FIPS": "21121", "County": "Knox", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40915", "FIPS": "21121", "County": "Knox", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40921", "FIPS": "21121", "County": "Knox", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40923", "FIPS": "21121", "County": "Knox", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40930", "FIPS": "21121", "County": "Knox", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40935", "FIPS": "21121", "County": "Knox", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40943", "FIPS": "21121", "County": "Knox", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40946", "FIPS": "21121", "County": "Knox", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40949", "FIPS": "21121", "County": "Knox", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40953", "FIPS": "21121", "County": "Knox", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40982", "FIPS": "21121", "County": "Knox", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40995", "FIPS": "21121", "County": "Knox", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40997", "FIPS": "21121", "County": "Knox", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40999", "FIPS": "21121", "County": "Knox", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42716", "FIPS": "21123", "County": "Larue", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42748", "FIPS": "21123", "County": "Larue", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42757", "FIPS": "21123", "County": "Larue", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42764", "FIPS": "21123", "County": "Larue", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40724", "FIPS": "21125", "County": "Laurel", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40729", "FIPS": "21125", "County": "Laurel", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40737", "FIPS": "21125", "County": "Laurel", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40740", "FIPS": "21125", "County": "Laurel", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40741", "FIPS": "21125", "County": "Laurel", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40742", "FIPS": "21125", "County": "Laurel", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40743", "FIPS": "21125", "County": "Laurel", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40744", "FIPS": "21125", "County": "Laurel", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40745", "FIPS": "21125", "County": "Laurel", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40755", "FIPS": "21125", "County": "Laurel", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41124", "FIPS": "21127", "County": "Lawrence", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41159", "FIPS": "21127", "County": "Lawrence", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41160", "FIPS": "21127", "County": "Lawrence", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41180", "FIPS": "21127", "County": "Lawrence", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41201", "FIPS": "21127", "County": "Lawrence", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41230", "FIPS": "21127", "County": "Lawrence", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41232", "FIPS": "21127", "County": "Lawrence", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41264", "FIPS": "21127", "County": "Lawrence", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41311", "FIPS": "21129", "County": "Lee", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41333", "FIPS": "21129", "County": "Lee", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41347", "FIPS": "21129", "County": "Lee", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41362", "FIPS": "21129", "County": "Lee", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41368", "FIPS": "21129", "County": "Lee", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41397", "FIPS": "21129", "County": "Lee", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40803", "FIPS": "21131", "County": "Leslie", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "40816", "FIPS": "21131", "County": "Leslie", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "40827", "FIPS": "21131", "County": "Leslie", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "40840", "FIPS": "21131", "County": "Leslie", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "40844", "FIPS": "21131", "County": "Leslie", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "40858", "FIPS": "21131", "County": "Leslie", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "40868", "FIPS": "21131", "County": "Leslie", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "40874", "FIPS": "21131", "County": "Leslie", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "40979", "FIPS": "21131", "County": "Leslie", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41714", "FIPS": "21131", "County": "Leslie", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41749", "FIPS": "21131", "County": "Leslie", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41762", "FIPS": "21131", "County": "Leslie", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41764", "FIPS": "21131", "County": "Leslie", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41766", "FIPS": "21131", "County": "Leslie", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41775", "FIPS": "21131", "County": "Leslie", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41776", "FIPS": "21131", "County": "Leslie", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41777", "FIPS": "21131", "County": "Leslie", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "40826", "FIPS": "21133", "County": "Letcher", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "40862", "FIPS": "21133", "County": "Letcher", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41517", "FIPS": "21133", "County": "Letcher", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41537", "FIPS": "21133", "County": "Letcher", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41804", "FIPS": "21133", "County": "Letcher", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41810", "FIPS": "21133", "County": "Letcher", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41812", "FIPS": "21133", "County": "Letcher", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41815", "FIPS": "21133", "County": "Letcher", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41819", "FIPS": "21133", "County": "Letcher", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41821", "FIPS": "21133", "County": "Letcher", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41824", "FIPS": "21133", "County": "Letcher", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41825", "FIPS": "21133", "County": "Letcher", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41826", "FIPS": "21133", "County": "Letcher", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41832", "FIPS": "21133", "County": "Letcher", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41833", "FIPS": "21133", "County": "Letcher", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41835", "FIPS": "21133", "County": "Letcher", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41837", "FIPS": "21133", "County": "Letcher", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41838", "FIPS": "21133", "County": "Letcher", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41840", "FIPS": "21133", "County": "Letcher", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41845", "FIPS": "21133", "County": "Letcher", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41848", "FIPS": "21133", "County": "Letcher", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41849", "FIPS": "21133", "County": "Letcher", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41855", "FIPS": "21133", "County": "Letcher", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41858", "FIPS": "21133", "County": "Letcher", "State": "KY", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "41135", "FIPS": "21135", "County": "Lewis", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41141", "FIPS": "21135", "County": "Lewis", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41166", "FIPS": "21135", "County": "Lewis", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41179", "FIPS": "21135", "County": "Lewis", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41189", "FIPS": "21135", "County": "Lewis", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "40419", "FIPS": "21137", "County": "Lincoln", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40437", "FIPS": "21137", "County": "Lincoln", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40442", "FIPS": "21137", "County": "Lincoln", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40448", "FIPS": "21137", "County": "Lincoln", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40484", "FIPS": "21137", "County": "Lincoln", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40489", "FIPS": "21137", "County": "Lincoln", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42028", "FIPS": "21139", "County": "Livingston", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42045", "FIPS": "21139", "County": "Livingston", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42047", "FIPS": "21139", "County": "Livingston", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42058", "FIPS": "21139", "County": "Livingston", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42078", "FIPS": "21139", "County": "Livingston", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42081", "FIPS": "21139", "County": "Livingston", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42083", "FIPS": "21139", "County": "Livingston", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42202", "FIPS": "21141", "County": "Logan", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42206", "FIPS": "21141", "County": "Logan", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42256", "FIPS": "21141", "County": "Logan", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42265", "FIPS": "21141", "County": "Logan", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42276", "FIPS": "21141", "County": "Logan", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42283", "FIPS": "21141", "County": "Logan", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42038", "FIPS": "21143", "County": "Lyon", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42055", "FIPS": "21143", "County": "Lyon", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42001", "FIPS": "21145", "County": "McCracken", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42002", "FIPS": "21145", "County": "McCracken", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42003", "FIPS": "21145", "County": "McCracken", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42053", "FIPS": "21145", "County": "McCracken", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42086", "FIPS": "21145", "County": "McCracken", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42631", "FIPS": "21147", "County": "McCreary", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "42634", "FIPS": "21147", "County": "McCreary", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "42635", "FIPS": "21147", "County": "McCreary", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "42638", "FIPS": "21147", "County": "McCreary", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "42647", "FIPS": "21147", "County": "McCreary", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "42649", "FIPS": "21147", "County": "McCreary", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "42653", "FIPS": "21147", "County": "McCreary", "State": "KY", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "42322", "FIPS": "21149", "County": "McLean", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42327", "FIPS": "21149", "County": "McLean", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42350", "FIPS": "21149", "County": "McLean", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42352", "FIPS": "21149", "County": "McLean", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42371", "FIPS": "21149", "County": "McLean", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42372", "FIPS": "21149", "County": "McLean", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "40385", "FIPS": "21151", "County": "Madison", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40403", "FIPS": "21151", "County": "Madison", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40404", "FIPS": "21151", "County": "Madison", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40405", "FIPS": "21151", "County": "Madison", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40475", "FIPS": "21151", "County": "Madison", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40476", "FIPS": "21151", "County": "Madison", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41426", "FIPS": "21153", "County": "Magoffin", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41433", "FIPS": "21153", "County": "Magoffin", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41464", "FIPS": "21153", "County": "Magoffin", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41465", "FIPS": "21153", "County": "Magoffin", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41632", "FIPS": "21153", "County": "Magoffin", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40009", "FIPS": "21155", "County": "Marion", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40033", "FIPS": "21155", "County": "Marion", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40037", "FIPS": "21155", "County": "Marion", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40049", "FIPS": "21155", "County": "Marion", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40060", "FIPS": "21155", "County": "Marion", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40062", "FIPS": "21155", "County": "Marion", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40063", "FIPS": "21155", "County": "Marion", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40328", "FIPS": "21155", "County": "Marion", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42025", "FIPS": "21157", "County": "Marshall", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42029", "FIPS": "21157", "County": "Marshall", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42044", "FIPS": "21157", "County": "Marshall", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "42048", "FIPS": "21157", "County": "Marshall", "State": "KY", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "41203", "FIPS": "21159", "County": "Martin", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41214", "FIPS": "21159", "County": "Martin", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41224", "FIPS": "21159", "County": "Martin", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41231", "FIPS": "21159", "County": "Martin", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41250", "FIPS": "21159", "County": "Martin", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41262", "FIPS": "21159", "County": "Martin", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41267", "FIPS": "21159", "County": "Martin", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41034", "FIPS": "21161", "County": "Mason", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41055", "FIPS": "21161", "County": "Mason", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41056", "FIPS": "21161", "County": "Mason", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41062", "FIPS": "21161", "County": "Mason", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41096", "FIPS": "21161", "County": "Mason", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "40104", "FIPS": "21163", "County": "Meade", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40108", "FIPS": "21163", "County": "Meade", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40117", "FIPS": "21163", "County": "Meade", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40142", "FIPS": "21163", "County": "Meade", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40155", "FIPS": "21163", "County": "Meade", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40157", "FIPS": "21163", "County": "Meade", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40161", "FIPS": "21163", "County": "Meade", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40316", "FIPS": "21165", "County": "Menifee", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40322", "FIPS": "21165", "County": "Menifee", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40346", "FIPS": "21165", "County": "Menifee", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40387", "FIPS": "21165", "County": "Menifee", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40310", "FIPS": "21167", "County": "Mercer", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40330", "FIPS": "21167", "County": "Mercer", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40372", "FIPS": "21167", "County": "Mercer", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42124", "FIPS": "21169", "County": "Metcalfe", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42129", "FIPS": "21169", "County": "Metcalfe", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42154", "FIPS": "21169", "County": "Metcalfe", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42166", "FIPS": "21169", "County": "Metcalfe", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42214", "FIPS": "21169", "County": "Metcalfe", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42133", "FIPS": "21171", "County": "Monroe", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42140", "FIPS": "21171", "County": "Monroe", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42151", "FIPS": "21171", "County": "Monroe", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42157", "FIPS": "21171", "County": "Monroe", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42167", "FIPS": "21171", "County": "Monroe", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "40337", "FIPS": "21173", "County": "Montgomery", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40353", "FIPS": "21173", "County": "Montgomery", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41352", "FIPS": "21175", "County": "Morgan", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41408", "FIPS": "21175", "County": "Morgan", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41413", "FIPS": "21175", "County": "Morgan", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41421", "FIPS": "21175", "County": "Morgan", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41425", "FIPS": "21175", "County": "Morgan", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41451", "FIPS": "21175", "County": "Morgan", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41459", "FIPS": "21175", "County": "Morgan", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41472", "FIPS": "21175", "County": "Morgan", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41477", "FIPS": "21175", "County": "Morgan", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42321", "FIPS": "21177", "County": "Muhlenberg", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42323", "FIPS": "21177", "County": "Muhlenberg", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42324", "FIPS": "21177", "County": "Muhlenberg", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42325", "FIPS": "21177", "County": "Muhlenberg", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42326", "FIPS": "21177", "County": "Muhlenberg", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42330", "FIPS": "21177", "County": "Muhlenberg", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42332", "FIPS": "21177", "County": "Muhlenberg", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42337", "FIPS": "21177", "County": "Muhlenberg", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42339", "FIPS": "21177", "County": "Muhlenberg", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42344", "FIPS": "21177", "County": "Muhlenberg", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42345", "FIPS": "21177", "County": "Muhlenberg", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42367", "FIPS": "21177", "County": "Muhlenberg", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42374", "FIPS": "21177", "County": "Muhlenberg", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "40004", "FIPS": "21179", "County": "Nelson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40008", "FIPS": "21179", "County": "Nelson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40012", "FIPS": "21179", "County": "Nelson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40013", "FIPS": "21179", "County": "Nelson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40020", "FIPS": "21179", "County": "Nelson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40048", "FIPS": "21179", "County": "Nelson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40051", "FIPS": "21179", "County": "Nelson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40052", "FIPS": "21179", "County": "Nelson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40107", "FIPS": "21179", "County": "Nelson", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40311", "FIPS": "21181", "County": "Nicholas", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40350", "FIPS": "21181", "County": "Nicholas", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42320", "FIPS": "21183", "County": "Ohio", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42328", "FIPS": "21183", "County": "Ohio", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42333", "FIPS": "21183", "County": "Ohio", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42338", "FIPS": "21183", "County": "Ohio", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42343", "FIPS": "21183", "County": "Ohio", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42347", "FIPS": "21183", "County": "Ohio", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42349", "FIPS": "21183", "County": "Ohio", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42354", "FIPS": "21183", "County": "Ohio", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42361", "FIPS": "21183", "County": "Ohio", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42369", "FIPS": "21183", "County": "Ohio", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42370", "FIPS": "21183", "County": "Ohio", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "40010", "FIPS": "21185", "County": "Oldham", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40014", "FIPS": "21185", "County": "Oldham", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40026", "FIPS": "21185", "County": "Oldham", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40031", "FIPS": "21185", "County": "Oldham", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40032", "FIPS": "21185", "County": "Oldham", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40056", "FIPS": "21185", "County": "Oldham", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40077", "FIPS": "21185", "County": "Oldham", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40355", "FIPS": "21187", "County": "Owen", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "40359", "FIPS": "21187", "County": "Owen", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "40363", "FIPS": "21187", "County": "Owen", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41314", "FIPS": "21189", "County": "Owsley", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41338", "FIPS": "21189", "County": "Owsley", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41351", "FIPS": "21189", "County": "Owsley", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41364", "FIPS": "21189", "County": "Owsley", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41386", "FIPS": "21189", "County": "Owsley", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41006", "FIPS": "21191", "County": "Pendleton", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41033", "FIPS": "21191", "County": "Pendleton", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "41040", "FIPS": "21191", "County": "Pendleton", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "40981", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41367", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41701", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41702", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41712", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41713", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41719", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41721", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41722", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41723", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41727", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41729", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41731", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41735", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41736", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41739", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41745", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41746", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41747", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41751", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41754", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41760", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41763", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41773", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41774", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41778", "FIPS": "21193", "County": "Perry", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41501", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41502", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41503", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41512", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41513", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41514", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41519", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41520", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41522", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41524", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41526", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41527", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41528", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41531", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41534", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41535", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41538", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41539", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41540", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41542", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41543", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41544", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41547", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41548", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41549", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41553", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41554", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41555", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41557", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41558", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41559", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41560", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41561", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41562", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41563", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41564", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41566", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41567", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41568", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41571", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "41572", "FIPS": "21195", "County": "Pike", "State": "KY", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "40312", "FIPS": "21197", "County": "Powell", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40376", "FIPS": "21197", "County": "Powell", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40380", "FIPS": "21197", "County": "Powell", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42501", "FIPS": "21199", "County": "Pulaski", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42502", "FIPS": "21199", "County": "Pulaski", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42503", "FIPS": "21199", "County": "Pulaski", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42518", "FIPS": "21199", "County": "Pulaski", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42519", "FIPS": "21199", "County": "Pulaski", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42533", "FIPS": "21199", "County": "Pulaski", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42544", "FIPS": "21199", "County": "Pulaski", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42553", "FIPS": "21199", "County": "Pulaski", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42558", "FIPS": "21199", "County": "Pulaski", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42564", "FIPS": "21199", "County": "Pulaski", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42567", "FIPS": "21199", "County": "Pulaski", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41064", "FIPS": "21201", "County": "Robertson", "State": "KY", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "40409", "FIPS": "21203", "County": "Rockcastle", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40445", "FIPS": "21203", "County": "Rockcastle", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40456", "FIPS": "21203", "County": "Rockcastle", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40460", "FIPS": "21203", "County": "Rockcastle", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40473", "FIPS": "21203", "County": "Rockcastle", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40492", "FIPS": "21203", "County": "Rockcastle", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40313", "FIPS": "21205", "County": "Rowan", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40317", "FIPS": "21205", "County": "Rowan", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40319", "FIPS": "21205", "County": "Rowan", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40351", "FIPS": "21205", "County": "Rowan", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42629", "FIPS": "21207", "County": "Russell", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42642", "FIPS": "21207", "County": "Russell", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40324", "FIPS": "21209", "County": "Scott", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40370", "FIPS": "21209", "County": "Scott", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40379", "FIPS": "21209", "County": "Scott", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40003", "FIPS": "21211", "County": "Shelby", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40022", "FIPS": "21211", "County": "Shelby", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40065", "FIPS": "21211", "County": "Shelby", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40066", "FIPS": "21211", "County": "Shelby", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40067", "FIPS": "21211", "County": "Shelby", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40076", "FIPS": "21211", "County": "Shelby", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42134", "FIPS": "21213", "County": "Simpson", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42135", "FIPS": "21213", "County": "Simpson", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "40046", "FIPS": "21215", "County": "Spencer", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40071", "FIPS": "21215", "County": "Spencer", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42718", "FIPS": "21217", "County": "Taylor", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42719", "FIPS": "21217", "County": "Taylor", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42733", "FIPS": "21217", "County": "Taylor", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42758", "FIPS": "21217", "County": "Taylor", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42204", "FIPS": "21219", "County": "Todd", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42216", "FIPS": "21219", "County": "Todd", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42220", "FIPS": "21219", "County": "Todd", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42234", "FIPS": "21219", "County": "Todd", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42280", "FIPS": "21219", "County": "Todd", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42286", "FIPS": "21219", "County": "Todd", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42211", "FIPS": "21221", "County": "Trigg", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "42215", "FIPS": "21221", "County": "Trigg", "State": "KY", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "40006", "FIPS": "21223", "County": "Trimble", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40045", "FIPS": "21223", "County": "Trimble", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42437", "FIPS": "21225", "County": "Union", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42459", "FIPS": "21225", "County": "Union", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42460", "FIPS": "21225", "County": "Union", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42461", "FIPS": "21225", "County": "Union", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42462", "FIPS": "21225", "County": "Union", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42101", "FIPS": "21227", "County": "Warren", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42102", "FIPS": "21227", "County": "Warren", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42103", "FIPS": "21227", "County": "Warren", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42104", "FIPS": "21227", "County": "Warren", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42122", "FIPS": "21227", "County": "Warren", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42128", "FIPS": "21227", "County": "Warren", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42159", "FIPS": "21227", "County": "Warren", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42170", "FIPS": "21227", "County": "Warren", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42171", "FIPS": "21227", "County": "Warren", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "42274", "FIPS": "21227", "County": "Warren", "State": "KY", "DMA": {"Code": 736, "Name": "BOWLING GREEN"}},
  {"Postal_Code": "40040", "FIPS": "21229", "County": "Washington", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40061", "FIPS": "21229", "County": "Washington", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40069", "FIPS": "21229", "County": "Washington", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "40078", "FIPS": "21229", "County": "Washington", "State": "KY", "DMA": {"Code": 529, "Name": "LOUISVILLE"}},
  {"Postal_Code": "42633", "FIPS": "21231", "County": "Wayne", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "42403", "FIPS": "21233", "County": "Webster", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42404", "FIPS": "21233", "County": "Webster", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42409", "FIPS": "21233", "County": "Webster", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42444", "FIPS": "21233", "County": "Webster", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42450", "FIPS": "21233", "County": "Webster", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42455", "FIPS": "21233", "County": "Webster", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42456", "FIPS": "21233", "County": "Webster", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "42463", "FIPS": "21233", "County": "Webster", "State": "KY", "DMA": {"Code": 649, "Name": "EVANSVILLE"}},
  {"Postal_Code": "40701", "FIPS": "21235", "County": "Whitley", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40702", "FIPS": "21235", "County": "Whitley", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40730", "FIPS": "21235", "County": "Whitley", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40754", "FIPS": "21235", "County": "Whitley", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40759", "FIPS": "21235", "County": "Whitley", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40763", "FIPS": "21235", "County": "Whitley", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40769", "FIPS": "21235", "County": "Whitley", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41301", "FIPS": "21237", "County": "Wolfe", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41313", "FIPS": "21237", "County": "Wolfe", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41332", "FIPS": "21237", "County": "Wolfe", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41360", "FIPS": "21237", "County": "Wolfe", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "41365", "FIPS": "21237", "County": "Wolfe", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40347", "FIPS": "21239", "County": "Woodford", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40383", "FIPS": "21239", "County": "Woodford", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40384", "FIPS": "21239", "County": "Woodford", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "40386", "FIPS": "21239", "County": "Woodford", "State": "KY", "DMA": {"Code": 541, "Name": "LEXINGTON"}},
  {"Postal_Code": "70516", "FIPS": "22001", "County": "Acadia", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70525", "FIPS": "22001", "County": "Acadia", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70526", "FIPS": "22001", "County": "Acadia", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70527", "FIPS": "22001", "County": "Acadia", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70531", "FIPS": "22001", "County": "Acadia", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70534", "FIPS": "22001", "County": "Acadia", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70537", "FIPS": "22001", "County": "Acadia", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70543", "FIPS": "22001", "County": "Acadia", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70556", "FIPS": "22001", "County": "Acadia", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70559", "FIPS": "22001", "County": "Acadia", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70578", "FIPS": "22001", "County": "Acadia", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70638", "FIPS": "22003", "County": "Allen", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70644", "FIPS": "22003", "County": "Allen", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70648", "FIPS": "22003", "County": "Allen", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70651", "FIPS": "22003", "County": "Allen", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70654", "FIPS": "22003", "County": "Allen", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70655", "FIPS": "22003", "County": "Allen", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70658", "FIPS": "22003", "County": "Allen", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "71463", "FIPS": "22003", "County": "Allen", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70346", "FIPS": "22005", "County": "Ascension", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70707", "FIPS": "22005", "County": "Ascension", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70718", "FIPS": "22005", "County": "Ascension", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70725", "FIPS": "22005", "County": "Ascension", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70728", "FIPS": "22005", "County": "Ascension", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70734", "FIPS": "22005", "County": "Ascension", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70737", "FIPS": "22005", "County": "Ascension", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70738", "FIPS": "22005", "County": "Ascension", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70769", "FIPS": "22005", "County": "Ascension", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70774", "FIPS": "22005", "County": "Ascension", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70778", "FIPS": "22005", "County": "Ascension", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70339", "FIPS": "22007", "County": "Assumption", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70341", "FIPS": "22007", "County": "Assumption", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70372", "FIPS": "22007", "County": "Assumption", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70390", "FIPS": "22007", "County": "Assumption", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70391", "FIPS": "22007", "County": "Assumption", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70393", "FIPS": "22007", "County": "Assumption", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "71320", "FIPS": "22009", "County": "Avoyelles", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71322", "FIPS": "22009", "County": "Avoyelles", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71323", "FIPS": "22009", "County": "Avoyelles", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71327", "FIPS": "22009", "County": "Avoyelles", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71329", "FIPS": "22009", "County": "Avoyelles", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71331", "FIPS": "22009", "County": "Avoyelles", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71333", "FIPS": "22009", "County": "Avoyelles", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71339", "FIPS": "22009", "County": "Avoyelles", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71341", "FIPS": "22009", "County": "Avoyelles", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71350", "FIPS": "22009", "County": "Avoyelles", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71351", "FIPS": "22009", "County": "Avoyelles", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71355", "FIPS": "22009", "County": "Avoyelles", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71362", "FIPS": "22009", "County": "Avoyelles", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71369", "FIPS": "22009", "County": "Avoyelles", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "70634", "FIPS": "22011", "County": "Beauregard", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70637", "FIPS": "22011", "County": "Beauregard", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70652", "FIPS": "22011", "County": "Beauregard", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70653", "FIPS": "22011", "County": "Beauregard", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70657", "FIPS": "22011", "County": "Beauregard", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70660", "FIPS": "22011", "County": "Beauregard", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70662", "FIPS": "22011", "County": "Beauregard", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "71001", "FIPS": "22013", "County": "Bienville", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71008", "FIPS": "22013", "County": "Bienville", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71016", "FIPS": "22013", "County": "Bienville", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71028", "FIPS": "22013", "County": "Bienville", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71045", "FIPS": "22013", "County": "Bienville", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71068", "FIPS": "22013", "County": "Bienville", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71070", "FIPS": "22013", "County": "Bienville", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71080", "FIPS": "22013", "County": "Bienville", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71006", "FIPS": "22015", "County": "Bossier", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71037", "FIPS": "22015", "County": "Bossier", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71051", "FIPS": "22015", "County": "Bossier", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71064", "FIPS": "22015", "County": "Bossier", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71067", "FIPS": "22015", "County": "Bossier", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71110", "FIPS": "22015", "County": "Bossier", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71111", "FIPS": "22015", "County": "Bossier", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71112", "FIPS": "22015", "County": "Bossier", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71113", "FIPS": "22015", "County": "Bossier", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71171", "FIPS": "22015", "County": "Bossier", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71172", "FIPS": "22015", "County": "Bossier", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71004", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71007", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71009", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71029", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71033", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71043", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71044", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71047", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71060", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71061", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71069", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71082", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71101", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71102", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71103", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71104", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71105", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71106", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71107", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71108", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71109", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71115", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71118", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71119", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71120", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71129", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71130", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71133", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71134", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71135", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71136", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71137", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71138", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71148", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71149", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71150", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71151", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71152", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71153", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71154", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71156", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71161", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71162", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71163", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71164", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71165", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71166", "FIPS": "22017", "County": "Caddo", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "70601", "FIPS": "22019", "County": "Calcasieu", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70602", "FIPS": "22019", "County": "Calcasieu", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70605", "FIPS": "22019", "County": "Calcasieu", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70606", "FIPS": "22019", "County": "Calcasieu", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70607", "FIPS": "22019", "County": "Calcasieu", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70609", "FIPS": "22019", "County": "Calcasieu", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70611", "FIPS": "22019", "County": "Calcasieu", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70612", "FIPS": "22019", "County": "Calcasieu", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70615", "FIPS": "22019", "County": "Calcasieu", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70616", "FIPS": "22019", "County": "Calcasieu", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70629", "FIPS": "22019", "County": "Calcasieu", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70630", "FIPS": "22019", "County": "Calcasieu", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70633", "FIPS": "22019", "County": "Calcasieu", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70646", "FIPS": "22019", "County": "Calcasieu", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70647", "FIPS": "22019", "County": "Calcasieu", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70661", "FIPS": "22019", "County": "Calcasieu", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70663", "FIPS": "22019", "County": "Calcasieu", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70664", "FIPS": "22019", "County": "Calcasieu", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70665", "FIPS": "22019", "County": "Calcasieu", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70668", "FIPS": "22019", "County": "Calcasieu", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70669", "FIPS": "22019", "County": "Calcasieu", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "71415", "FIPS": "22021", "County": "Caldwell", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71418", "FIPS": "22021", "County": "Caldwell", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71435", "FIPS": "22021", "County": "Caldwell", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71441", "FIPS": "22021", "County": "Caldwell", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "70631", "FIPS": "22023", "County": "Cameron", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70632", "FIPS": "22023", "County": "Cameron", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70643", "FIPS": "22023", "County": "Cameron", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "70645", "FIPS": "22023", "County": "Cameron", "State": "LA", "DMA": {"Code": 643, "Name": "LAKE CHARLES"}},
  {"Postal_Code": "71340", "FIPS": "22025", "County": "Catahoula", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71343", "FIPS": "22025", "County": "Catahoula", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71363", "FIPS": "22025", "County": "Catahoula", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71368", "FIPS": "22025", "County": "Catahoula", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71401", "FIPS": "22025", "County": "Catahoula", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71425", "FIPS": "22025", "County": "Catahoula", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71003", "FIPS": "22027", "County": "Claiborne", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71038", "FIPS": "22027", "County": "Claiborne", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71040", "FIPS": "22027", "County": "Claiborne", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71048", "FIPS": "22027", "County": "Claiborne", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71079", "FIPS": "22027", "County": "Claiborne", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71316", "FIPS": "22029", "County": "Concordia", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71326", "FIPS": "22029", "County": "Concordia", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71334", "FIPS": "22029", "County": "Concordia", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71354", "FIPS": "22029", "County": "Concordia", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71373", "FIPS": "22029", "County": "Concordia", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71377", "FIPS": "22029", "County": "Concordia", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71027", "FIPS": "22031", "County": "De Soto", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71030", "FIPS": "22031", "County": "De Soto", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71032", "FIPS": "22031", "County": "De Soto", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71046", "FIPS": "22031", "County": "De Soto", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71049", "FIPS": "22031", "County": "De Soto", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71050", "FIPS": "22031", "County": "De Soto", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71052", "FIPS": "22031", "County": "De Soto", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71063", "FIPS": "22031", "County": "De Soto", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71078", "FIPS": "22031", "County": "De Soto", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "70704", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70714", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70739", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70770", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70791", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70801", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70802", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70803", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70804", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70805", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70806", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70807", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70808", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70809", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70810", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70811", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70812", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70813", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70814", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70815", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70816", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70817", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70818", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70819", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70820", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70821", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70822", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70823", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70825", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70826", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70827", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70831", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70833", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70835", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70836", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70837", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70873", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70874", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70879", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70883", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70884", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70891", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70892", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70893", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70894", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70895", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70896", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70898", "FIPS": "22033", "County": "East Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "71254", "FIPS": "22035", "County": "East Carroll", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71276", "FIPS": "22035", "County": "East Carroll", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71286", "FIPS": "22035", "County": "East Carroll", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "70722", "FIPS": "22037", "County": "East Feliciana", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70730", "FIPS": "22037", "County": "East Feliciana", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70748", "FIPS": "22037", "County": "East Feliciana", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70761", "FIPS": "22037", "County": "East Feliciana", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70777", "FIPS": "22037", "County": "East Feliciana", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70789", "FIPS": "22037", "County": "East Feliciana", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70515", "FIPS": "22039", "County": "Evangeline", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70524", "FIPS": "22039", "County": "Evangeline", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70554", "FIPS": "22039", "County": "Evangeline", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70576", "FIPS": "22039", "County": "Evangeline", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70580", "FIPS": "22039", "County": "Evangeline", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70585", "FIPS": "22039", "County": "Evangeline", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70586", "FIPS": "22039", "County": "Evangeline", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "71367", "FIPS": "22039", "County": "Evangeline", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "71219", "FIPS": "22041", "County": "Franklin", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71230", "FIPS": "22041", "County": "Franklin", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71243", "FIPS": "22041", "County": "Franklin", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71249", "FIPS": "22041", "County": "Franklin", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71295", "FIPS": "22041", "County": "Franklin", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71324", "FIPS": "22041", "County": "Franklin", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71336", "FIPS": "22041", "County": "Franklin", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71378", "FIPS": "22041", "County": "Franklin", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71407", "FIPS": "22043", "County": "Grant", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71417", "FIPS": "22043", "County": "Grant", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71423", "FIPS": "22043", "County": "Grant", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71432", "FIPS": "22043", "County": "Grant", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71454", "FIPS": "22043", "County": "Grant", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71467", "FIPS": "22043", "County": "Grant", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "70513", "FIPS": "22045", "County": "Iberia", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70544", "FIPS": "22045", "County": "Iberia", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70552", "FIPS": "22045", "County": "Iberia", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70560", "FIPS": "22045", "County": "Iberia", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70562", "FIPS": "22045", "County": "Iberia", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70563", "FIPS": "22045", "County": "Iberia", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70569", "FIPS": "22045", "County": "Iberia", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70721", "FIPS": "22047", "County": "Iberville", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70740", "FIPS": "22047", "County": "Iberville", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70757", "FIPS": "22047", "County": "Iberville", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70764", "FIPS": "22047", "County": "Iberville", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70765", "FIPS": "22047", "County": "Iberville", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70772", "FIPS": "22047", "County": "Iberville", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70776", "FIPS": "22047", "County": "Iberville", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70780", "FIPS": "22047", "County": "Iberville", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70788", "FIPS": "22047", "County": "Iberville", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "71226", "FIPS": "22049", "County": "Jackson", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71247", "FIPS": "22049", "County": "Jackson", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71251", "FIPS": "22049", "County": "Jackson", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71268", "FIPS": "22049", "County": "Jackson", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "70001", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70002", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70003", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70004", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70005", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70006", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70009", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70010", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70011", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70033", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70036", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70053", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70054", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70055", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70056", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70058", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70059", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70060", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70062", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70063", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70064", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70065", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70067", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70072", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70073", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70094", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70096", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70097", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70121", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70123", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70141", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70181", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70183", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70358", "FIPS": "22051", "County": "Jefferson", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70532", "FIPS": "22053", "County": "Jefferson Davis", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70546", "FIPS": "22053", "County": "Jefferson Davis", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70549", "FIPS": "22053", "County": "Jefferson Davis", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70581", "FIPS": "22053", "County": "Jefferson Davis", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70591", "FIPS": "22053", "County": "Jefferson Davis", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70640", "FIPS": "22053", "County": "Jefferson Davis", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70650", "FIPS": "22053", "County": "Jefferson Davis", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70501", "FIPS": "22055", "County": "Lafayette", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70502", "FIPS": "22055", "County": "Lafayette", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70503", "FIPS": "22055", "County": "Lafayette", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70504", "FIPS": "22055", "County": "Lafayette", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70505", "FIPS": "22055", "County": "Lafayette", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70506", "FIPS": "22055", "County": "Lafayette", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70507", "FIPS": "22055", "County": "Lafayette", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70508", "FIPS": "22055", "County": "Lafayette", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70509", "FIPS": "22055", "County": "Lafayette", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70518", "FIPS": "22055", "County": "Lafayette", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70520", "FIPS": "22055", "County": "Lafayette", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70529", "FIPS": "22055", "County": "Lafayette", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70558", "FIPS": "22055", "County": "Lafayette", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70583", "FIPS": "22055", "County": "Lafayette", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70592", "FIPS": "22055", "County": "Lafayette", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70593", "FIPS": "22055", "County": "Lafayette", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70595", "FIPS": "22055", "County": "Lafayette", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70596", "FIPS": "22055", "County": "Lafayette", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70598", "FIPS": "22055", "County": "Lafayette", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70301", "FIPS": "22057", "County": "Lafourche", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70302", "FIPS": "22057", "County": "Lafourche", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70310", "FIPS": "22057", "County": "Lafourche", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70345", "FIPS": "22057", "County": "Lafourche", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70354", "FIPS": "22057", "County": "Lafourche", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70355", "FIPS": "22057", "County": "Lafourche", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70357", "FIPS": "22057", "County": "Lafourche", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70371", "FIPS": "22057", "County": "Lafourche", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70373", "FIPS": "22057", "County": "Lafourche", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70374", "FIPS": "22057", "County": "Lafourche", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70375", "FIPS": "22057", "County": "Lafourche", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70394", "FIPS": "22057", "County": "Lafourche", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "71342", "FIPS": "22059", "County": "La Salle", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71371", "FIPS": "22059", "County": "La Salle", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71465", "FIPS": "22059", "County": "La Salle", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71479", "FIPS": "22059", "County": "La Salle", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71480", "FIPS": "22059", "County": "La Salle", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71227", "FIPS": "22061", "County": "Lincoln", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71235", "FIPS": "22061", "County": "Lincoln", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71245", "FIPS": "22061", "County": "Lincoln", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71270", "FIPS": "22061", "County": "Lincoln", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71272", "FIPS": "22061", "County": "Lincoln", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71273", "FIPS": "22061", "County": "Lincoln", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71275", "FIPS": "22061", "County": "Lincoln", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "70449", "FIPS": "22063", "County": "Livingston", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70462", "FIPS": "22063", "County": "Livingston", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70706", "FIPS": "22063", "County": "Livingston", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70711", "FIPS": "22063", "County": "Livingston", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70726", "FIPS": "22063", "County": "Livingston", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70727", "FIPS": "22063", "County": "Livingston", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70733", "FIPS": "22063", "County": "Livingston", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70744", "FIPS": "22063", "County": "Livingston", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70754", "FIPS": "22063", "County": "Livingston", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70785", "FIPS": "22063", "County": "Livingston", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70786", "FIPS": "22063", "County": "Livingston", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "71233", "FIPS": "22065", "County": "Madison", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71282", "FIPS": "22065", "County": "Madison", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71284", "FIPS": "22065", "County": "Madison", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71220", "FIPS": "22067", "County": "Morehouse", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71221", "FIPS": "22067", "County": "Morehouse", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71223", "FIPS": "22067", "County": "Morehouse", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71229", "FIPS": "22067", "County": "Morehouse", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71250", "FIPS": "22067", "County": "Morehouse", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71261", "FIPS": "22067", "County": "Morehouse", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71264", "FIPS": "22067", "County": "Morehouse", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71002", "FIPS": "22069", "County": "Natchitoches", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71031", "FIPS": "22069", "County": "Natchitoches", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71066", "FIPS": "22069", "County": "Natchitoches", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71411", "FIPS": "22069", "County": "Natchitoches", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71414", "FIPS": "22069", "County": "Natchitoches", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71416", "FIPS": "22069", "County": "Natchitoches", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71428", "FIPS": "22069", "County": "Natchitoches", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71434", "FIPS": "22069", "County": "Natchitoches", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71450", "FIPS": "22069", "County": "Natchitoches", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71452", "FIPS": "22069", "County": "Natchitoches", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71456", "FIPS": "22069", "County": "Natchitoches", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71457", "FIPS": "22069", "County": "Natchitoches", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71458", "FIPS": "22069", "County": "Natchitoches", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71468", "FIPS": "22069", "County": "Natchitoches", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71469", "FIPS": "22069", "County": "Natchitoches", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71497", "FIPS": "22069", "County": "Natchitoches", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "70112", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70113", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70114", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70115", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70116", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70117", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70118", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70119", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70122", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70124", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70125", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70126", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70127", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70128", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70129", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70130", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70131", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70139", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70140", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70142", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70143", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70145", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70146", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70148", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70149", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70150", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70151", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70152", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70153", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70154", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70156", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70157", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70158", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70159", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70160", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70161", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70162", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70163", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70164", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70165", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70166", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70167", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70170", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70172", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70174", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70175", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70176", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70177", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70178", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70179", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70182", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70184", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70185", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70186", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70187", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70189", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70190", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70195", "FIPS": "22071", "County": "Orleans", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "71201", "FIPS": "22073", "County": "Ouachita", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71202", "FIPS": "22073", "County": "Ouachita", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71203", "FIPS": "22073", "County": "Ouachita", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71207", "FIPS": "22073", "County": "Ouachita", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71208", "FIPS": "22073", "County": "Ouachita", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71209", "FIPS": "22073", "County": "Ouachita", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71210", "FIPS": "22073", "County": "Ouachita", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71211", "FIPS": "22073", "County": "Ouachita", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71212", "FIPS": "22073", "County": "Ouachita", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71213", "FIPS": "22073", "County": "Ouachita", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71217", "FIPS": "22073", "County": "Ouachita", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71225", "FIPS": "22073", "County": "Ouachita", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71238", "FIPS": "22073", "County": "Ouachita", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71240", "FIPS": "22073", "County": "Ouachita", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71280", "FIPS": "22073", "County": "Ouachita", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71281", "FIPS": "22073", "County": "Ouachita", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71291", "FIPS": "22073", "County": "Ouachita", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71292", "FIPS": "22073", "County": "Ouachita", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71294", "FIPS": "22073", "County": "Ouachita", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "70037", "FIPS": "22075", "County": "Plaquemines", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70038", "FIPS": "22075", "County": "Plaquemines", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70040", "FIPS": "22075", "County": "Plaquemines", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70041", "FIPS": "22075", "County": "Plaquemines", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70050", "FIPS": "22075", "County": "Plaquemines", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70081", "FIPS": "22075", "County": "Plaquemines", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70082", "FIPS": "22075", "County": "Plaquemines", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70083", "FIPS": "22075", "County": "Plaquemines", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70091", "FIPS": "22075", "County": "Plaquemines", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70093", "FIPS": "22075", "County": "Plaquemines", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70715", "FIPS": "22077", "County": "Pointe Coupee", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70732", "FIPS": "22077", "County": "Pointe Coupee", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70736", "FIPS": "22077", "County": "Pointe Coupee", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70747", "FIPS": "22077", "County": "Pointe Coupee", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70749", "FIPS": "22077", "County": "Pointe Coupee", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70752", "FIPS": "22077", "County": "Pointe Coupee", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70753", "FIPS": "22077", "County": "Pointe Coupee", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70755", "FIPS": "22077", "County": "Pointe Coupee", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70756", "FIPS": "22077", "County": "Pointe Coupee", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70759", "FIPS": "22077", "County": "Pointe Coupee", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70760", "FIPS": "22077", "County": "Pointe Coupee", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70762", "FIPS": "22077", "County": "Pointe Coupee", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70773", "FIPS": "22077", "County": "Pointe Coupee", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70783", "FIPS": "22077", "County": "Pointe Coupee", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "71301", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71302", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71303", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71306", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71307", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71309", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71315", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71325", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71328", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71330", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71346", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71348", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71359", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71360", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71361", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71365", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71405", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71409", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71424", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71427", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71430", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71431", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71433", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71438", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71447", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71448", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71455", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71466", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71472", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71477", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71485", "FIPS": "22079", "County": "Rapides", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71019", "FIPS": "22081", "County": "Red River", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71034", "FIPS": "22081", "County": "Red River", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71218", "FIPS": "22083", "County": "Richland", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71232", "FIPS": "22083", "County": "Richland", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71259", "FIPS": "22083", "County": "Richland", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71269", "FIPS": "22083", "County": "Richland", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71279", "FIPS": "22083", "County": "Richland", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71065", "FIPS": "22085", "County": "Sabine", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71406", "FIPS": "22085", "County": "Sabine", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71419", "FIPS": "22085", "County": "Sabine", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71426", "FIPS": "22085", "County": "Sabine", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71429", "FIPS": "22085", "County": "Sabine", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71449", "FIPS": "22085", "County": "Sabine", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71460", "FIPS": "22085", "County": "Sabine", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71462", "FIPS": "22085", "County": "Sabine", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71486", "FIPS": "22085", "County": "Sabine", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "70032", "FIPS": "22087", "County": "Saint Bernard", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70043", "FIPS": "22087", "County": "Saint Bernard", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70044", "FIPS": "22087", "County": "Saint Bernard", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70075", "FIPS": "22087", "County": "Saint Bernard", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70085", "FIPS": "22087", "County": "Saint Bernard", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70092", "FIPS": "22087", "County": "Saint Bernard", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70030", "FIPS": "22089", "County": "Saint Charles", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70031", "FIPS": "22089", "County": "Saint Charles", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70039", "FIPS": "22089", "County": "Saint Charles", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70047", "FIPS": "22089", "County": "Saint Charles", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70057", "FIPS": "22089", "County": "Saint Charles", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70070", "FIPS": "22089", "County": "Saint Charles", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70078", "FIPS": "22089", "County": "Saint Charles", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70079", "FIPS": "22089", "County": "Saint Charles", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70080", "FIPS": "22089", "County": "Saint Charles", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70087", "FIPS": "22089", "County": "Saint Charles", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70441", "FIPS": "22091", "County": "Saint Helena", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70453", "FIPS": "22091", "County": "Saint Helena", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70052", "FIPS": "22093", "County": "Saint James", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70071", "FIPS": "22093", "County": "Saint James", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70086", "FIPS": "22093", "County": "Saint James", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70090", "FIPS": "22093", "County": "Saint James", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70723", "FIPS": "22093", "County": "Saint James", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70743", "FIPS": "22093", "County": "Saint James", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70763", "FIPS": "22093", "County": "Saint James", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70792", "FIPS": "22093", "County": "Saint James", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70049", "FIPS": "22095", "County": "St John the Baptist", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70051", "FIPS": "22095", "County": "St John the Baptist", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70068", "FIPS": "22095", "County": "St John the Baptist", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70069", "FIPS": "22095", "County": "St John the Baptist", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70076", "FIPS": "22095", "County": "St John the Baptist", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70084", "FIPS": "22095", "County": "St John the Baptist", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70512", "FIPS": "22097", "County": "Saint Landry", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70535", "FIPS": "22097", "County": "Saint Landry", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70541", "FIPS": "22097", "County": "Saint Landry", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70550", "FIPS": "22097", "County": "Saint Landry", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70551", "FIPS": "22097", "County": "Saint Landry", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70570", "FIPS": "22097", "County": "Saint Landry", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70571", "FIPS": "22097", "County": "Saint Landry", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70577", "FIPS": "22097", "County": "Saint Landry", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70584", "FIPS": "22097", "County": "Saint Landry", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70589", "FIPS": "22097", "County": "Saint Landry", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70750", "FIPS": "22097", "County": "Saint Landry", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "71345", "FIPS": "22097", "County": "Saint Landry", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "71353", "FIPS": "22097", "County": "Saint Landry", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "71356", "FIPS": "22097", "County": "Saint Landry", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "71358", "FIPS": "22097", "County": "Saint Landry", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70517", "FIPS": "22099", "County": "Saint Martin", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70519", "FIPS": "22099", "County": "Saint Martin", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70521", "FIPS": "22099", "County": "Saint Martin", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70582", "FIPS": "22099", "County": "Saint Martin", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70340", "FIPS": "22101", "County": "Saint Mary", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70342", "FIPS": "22101", "County": "Saint Mary", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70380", "FIPS": "22101", "County": "Saint Mary", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70381", "FIPS": "22101", "County": "Saint Mary", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70392", "FIPS": "22101", "County": "Saint Mary", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70514", "FIPS": "22101", "County": "Saint Mary", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70522", "FIPS": "22101", "County": "Saint Mary", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70523", "FIPS": "22101", "County": "Saint Mary", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70538", "FIPS": "22101", "County": "Saint Mary", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70540", "FIPS": "22101", "County": "Saint Mary", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70420", "FIPS": "22103", "County": "Saint Tammany", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70431", "FIPS": "22103", "County": "Saint Tammany", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70433", "FIPS": "22103", "County": "Saint Tammany", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70434", "FIPS": "22103", "County": "Saint Tammany", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70435", "FIPS": "22103", "County": "Saint Tammany", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70437", "FIPS": "22103", "County": "Saint Tammany", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70445", "FIPS": "22103", "County": "Saint Tammany", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70447", "FIPS": "22103", "County": "Saint Tammany", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70448", "FIPS": "22103", "County": "Saint Tammany", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70452", "FIPS": "22103", "County": "Saint Tammany", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70457", "FIPS": "22103", "County": "Saint Tammany", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70458", "FIPS": "22103", "County": "Saint Tammany", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70459", "FIPS": "22103", "County": "Saint Tammany", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70460", "FIPS": "22103", "County": "Saint Tammany", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70461", "FIPS": "22103", "County": "Saint Tammany", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70463", "FIPS": "22103", "County": "Saint Tammany", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70464", "FIPS": "22103", "County": "Saint Tammany", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70469", "FIPS": "22103", "County": "Saint Tammany", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70470", "FIPS": "22103", "County": "Saint Tammany", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70471", "FIPS": "22103", "County": "Saint Tammany", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70401", "FIPS": "22105", "County": "Tangipahoa", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70402", "FIPS": "22105", "County": "Tangipahoa", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70403", "FIPS": "22105", "County": "Tangipahoa", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70404", "FIPS": "22105", "County": "Tangipahoa", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70421", "FIPS": "22105", "County": "Tangipahoa", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70422", "FIPS": "22105", "County": "Tangipahoa", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70436", "FIPS": "22105", "County": "Tangipahoa", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70442", "FIPS": "22105", "County": "Tangipahoa", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70443", "FIPS": "22105", "County": "Tangipahoa", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70444", "FIPS": "22105", "County": "Tangipahoa", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70446", "FIPS": "22105", "County": "Tangipahoa", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70451", "FIPS": "22105", "County": "Tangipahoa", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70454", "FIPS": "22105", "County": "Tangipahoa", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70455", "FIPS": "22105", "County": "Tangipahoa", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70456", "FIPS": "22105", "County": "Tangipahoa", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70465", "FIPS": "22105", "County": "Tangipahoa", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70466", "FIPS": "22105", "County": "Tangipahoa", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "71357", "FIPS": "22107", "County": "Tensas", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71366", "FIPS": "22107", "County": "Tensas", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71375", "FIPS": "22107", "County": "Tensas", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "70343", "FIPS": "22109", "County": "Terrebonne", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70344", "FIPS": "22109", "County": "Terrebonne", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70352", "FIPS": "22109", "County": "Terrebonne", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70353", "FIPS": "22109", "County": "Terrebonne", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70356", "FIPS": "22109", "County": "Terrebonne", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70359", "FIPS": "22109", "County": "Terrebonne", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70360", "FIPS": "22109", "County": "Terrebonne", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70361", "FIPS": "22109", "County": "Terrebonne", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70363", "FIPS": "22109", "County": "Terrebonne", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70364", "FIPS": "22109", "County": "Terrebonne", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70377", "FIPS": "22109", "County": "Terrebonne", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70395", "FIPS": "22109", "County": "Terrebonne", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70397", "FIPS": "22109", "County": "Terrebonne", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "71222", "FIPS": "22111", "County": "Union", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71234", "FIPS": "22111", "County": "Union", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71241", "FIPS": "22111", "County": "Union", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71256", "FIPS": "22111", "County": "Union", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71260", "FIPS": "22111", "County": "Union", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71277", "FIPS": "22111", "County": "Union", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "70510", "FIPS": "22113", "County": "Vermilion", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70511", "FIPS": "22113", "County": "Vermilion", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70528", "FIPS": "22113", "County": "Vermilion", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70533", "FIPS": "22113", "County": "Vermilion", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70542", "FIPS": "22113", "County": "Vermilion", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70548", "FIPS": "22113", "County": "Vermilion", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70555", "FIPS": "22113", "County": "Vermilion", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70575", "FIPS": "22113", "County": "Vermilion", "State": "LA", "DMA": {"Code": 642, "Name": "LAFAYETTE, LA"}},
  {"Postal_Code": "70639", "FIPS": "22115", "County": "Vernon", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "70656", "FIPS": "22115", "County": "Vernon", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "70659", "FIPS": "22115", "County": "Vernon", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71403", "FIPS": "22115", "County": "Vernon", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71439", "FIPS": "22115", "County": "Vernon", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71443", "FIPS": "22115", "County": "Vernon", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71446", "FIPS": "22115", "County": "Vernon", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71459", "FIPS": "22115", "County": "Vernon", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71461", "FIPS": "22115", "County": "Vernon", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71474", "FIPS": "22115", "County": "Vernon", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71475", "FIPS": "22115", "County": "Vernon", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "71496", "FIPS": "22115", "County": "Vernon", "State": "LA", "DMA": {"Code": 644, "Name": "ALEXANDRIA, LA"}},
  {"Postal_Code": "70426", "FIPS": "22117", "County": "Washington", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70427", "FIPS": "22117", "County": "Washington", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70429", "FIPS": "22117", "County": "Washington", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70438", "FIPS": "22117", "County": "Washington", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70450", "FIPS": "22117", "County": "Washington", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "70467", "FIPS": "22117", "County": "Washington", "State": "LA", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "71018", "FIPS": "22119", "County": "Webster", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71021", "FIPS": "22119", "County": "Webster", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71023", "FIPS": "22119", "County": "Webster", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71024", "FIPS": "22119", "County": "Webster", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71039", "FIPS": "22119", "County": "Webster", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71055", "FIPS": "22119", "County": "Webster", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71058", "FIPS": "22119", "County": "Webster", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71071", "FIPS": "22119", "County": "Webster", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71072", "FIPS": "22119", "County": "Webster", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71073", "FIPS": "22119", "County": "Webster", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "71075", "FIPS": "22119", "County": "Webster", "State": "LA", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "70710", "FIPS": "22121", "County": "West Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70719", "FIPS": "22121", "County": "West Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70729", "FIPS": "22121", "County": "West Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70767", "FIPS": "22121", "County": "West Baton Rouge", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "71237", "FIPS": "22123", "County": "West Carroll", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71242", "FIPS": "22123", "County": "West Carroll", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71253", "FIPS": "22123", "County": "West Carroll", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71263", "FIPS": "22123", "County": "West Carroll", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71266", "FIPS": "22123", "County": "West Carroll", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "70712", "FIPS": "22125", "County": "West Feliciana", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70775", "FIPS": "22125", "County": "West Feliciana", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70782", "FIPS": "22125", "County": "West Feliciana", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70784", "FIPS": "22125", "County": "West Feliciana", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "70787", "FIPS": "22125", "County": "West Feliciana", "State": "LA", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "71404", "FIPS": "22127", "County": "Winn", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71410", "FIPS": "22127", "County": "Winn", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71422", "FIPS": "22127", "County": "Winn", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71440", "FIPS": "22127", "County": "Winn", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71471", "FIPS": "22127", "County": "Winn", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71473", "FIPS": "22127", "County": "Winn", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "71483", "FIPS": "22127", "County": "Winn", "State": "LA", "DMA": {"Code": 628, "Name": "MONROE-EL DORADO"}},
  {"Postal_Code": "04210", "FIPS": "23001", "County": "Androscoggin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04211", "FIPS": "23001", "County": "Androscoggin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04212", "FIPS": "23001", "County": "Androscoggin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04222", "FIPS": "23001", "County": "Androscoggin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04223", "FIPS": "23001", "County": "Androscoggin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04228", "FIPS": "23001", "County": "Androscoggin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04230", "FIPS": "23001", "County": "Androscoggin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04236", "FIPS": "23001", "County": "Androscoggin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04240", "FIPS": "23001", "County": "Androscoggin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04241", "FIPS": "23001", "County": "Androscoggin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04243", "FIPS": "23001", "County": "Androscoggin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04250", "FIPS": "23001", "County": "Androscoggin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04252", "FIPS": "23001", "County": "Androscoggin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04253", "FIPS": "23001", "County": "Androscoggin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04254", "FIPS": "23001", "County": "Androscoggin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04256", "FIPS": "23001", "County": "Androscoggin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04258", "FIPS": "23001", "County": "Androscoggin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04263", "FIPS": "23001", "County": "Androscoggin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04266", "FIPS": "23001", "County": "Androscoggin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04274", "FIPS": "23001", "County": "Androscoggin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04280", "FIPS": "23001", "County": "Androscoggin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04282", "FIPS": "23001", "County": "Androscoggin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04288", "FIPS": "23001", "County": "Androscoggin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04291", "FIPS": "23001", "County": "Androscoggin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04471", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04497", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04730", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04732", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04733", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04734", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04735", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04736", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04737", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04738", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04739", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04740", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04741", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04742", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04743", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04744", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04745", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04746", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04747", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04750", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04751", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04756", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04757", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04758", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04760", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04761", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04762", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04763", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04764", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04766", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04768", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04769", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04772", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04773", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04774", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04775", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04776", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04779", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04780", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04781", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04783", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04785", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04786", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04787", "FIPS": "23003", "County": "Aroostook", "State": "ME", "DMA": {"Code": 552, "Name": "PRESQUE ISLE"}},
  {"Postal_Code": "04003", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04009", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04011", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04013", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04015", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04017", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04019", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04021", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04024", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04029", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04032", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04033", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04034", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04038", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04039", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04040", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04050", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04055", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04057", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04062", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04066", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04069", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04070", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04071", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04074", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04075", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04077", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04078", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04079", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04082", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04084", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04085", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04091", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04092", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04096", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04097", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04098", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04101", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04102", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04103", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04104", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04105", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04106", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04107", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04108", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04109", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04110", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04112", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04116", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04122", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04123", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04124", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04260", "FIPS": "23005", "County": "Cumberland", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04225", "FIPS": "23007", "County": "Franklin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04227", "FIPS": "23007", "County": "Franklin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04234", "FIPS": "23007", "County": "Franklin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04239", "FIPS": "23007", "County": "Franklin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04262", "FIPS": "23007", "County": "Franklin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04285", "FIPS": "23007", "County": "Franklin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04294", "FIPS": "23007", "County": "Franklin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04936", "FIPS": "23007", "County": "Franklin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04938", "FIPS": "23007", "County": "Franklin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04940", "FIPS": "23007", "County": "Franklin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04947", "FIPS": "23007", "County": "Franklin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04955", "FIPS": "23007", "County": "Franklin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04956", "FIPS": "23007", "County": "Franklin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04964", "FIPS": "23007", "County": "Franklin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04966", "FIPS": "23007", "County": "Franklin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04970", "FIPS": "23007", "County": "Franklin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04982", "FIPS": "23007", "County": "Franklin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04983", "FIPS": "23007", "County": "Franklin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04984", "FIPS": "23007", "County": "Franklin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04992", "FIPS": "23007", "County": "Franklin", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04408", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04416", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04420", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04421", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04431", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04472", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04476", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04605", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04607", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04609", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04612", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04613", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04614", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04616", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04617", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04624", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04625", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04627", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04629", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04634", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04635", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04640", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04642", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04644", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04646", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04650", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04653", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04660", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04662", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04664", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04669", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04672", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04673", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04674", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04675", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04676", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04677", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04679", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04681", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04683", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04684", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04685", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04693", "FIPS": "23009", "County": "Hancock", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04259", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04265", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04284", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04330", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04332", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04333", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04336", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04338", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04343", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04344", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04345", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04346", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04347", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04349", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04350", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04351", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04352", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04355", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04358", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04359", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04360", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04363", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04364", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04901", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04903", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04910", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04917", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04918", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04926", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04927", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04935", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04962", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04963", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04989", "FIPS": "23011", "County": "Kennebec", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04547", "FIPS": "23013", "County": "Knox", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04563", "FIPS": "23013", "County": "Knox", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04574", "FIPS": "23013", "County": "Knox", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04645", "FIPS": "23013", "County": "Knox", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04841", "FIPS": "23013", "County": "Knox", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04843", "FIPS": "23013", "County": "Knox", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04846", "FIPS": "23013", "County": "Knox", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04847", "FIPS": "23013", "County": "Knox", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04851", "FIPS": "23013", "County": "Knox", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04853", "FIPS": "23013", "County": "Knox", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04854", "FIPS": "23013", "County": "Knox", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04855", "FIPS": "23013", "County": "Knox", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04856", "FIPS": "23013", "County": "Knox", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04858", "FIPS": "23013", "County": "Knox", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04859", "FIPS": "23013", "County": "Knox", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04860", "FIPS": "23013", "County": "Knox", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04861", "FIPS": "23013", "County": "Knox", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04862", "FIPS": "23013", "County": "Knox", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04863", "FIPS": "23013", "County": "Knox", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04864", "FIPS": "23013", "County": "Knox", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04865", "FIPS": "23013", "County": "Knox", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04341", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04342", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04348", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04353", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04535", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04537", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04538", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04539", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04541", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04543", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04544", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04549", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04551", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04553", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04554", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04555", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04556", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04558", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04564", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04568", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04570", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04571", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04572", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04573", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04575", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04576", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04578", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04852", "FIPS": "23015", "County": "Lincoln", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04010", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04016", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04022", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04037", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04041", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04051", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04068", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04088", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04216", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04217", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04219", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04220", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04221", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04224", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04226", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04231", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04237", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04238", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04255", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04257", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04261", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04267", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04268", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04270", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04271", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04275", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04276", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04278", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04281", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04286", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04289", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04290", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04292", "FIPS": "23017", "County": "Oxford", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04401", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04402", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04410", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04411", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04412", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04417", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04418", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04419", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04422", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04427", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04428", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04429", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04430", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04434", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04435", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04444", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04448", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04449", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04450", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04451", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04453", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04455", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04456", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04457", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04459", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04460", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04461", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04462", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04467", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04468", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04469", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04473", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04474", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04475", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04487", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04488", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04489", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04493", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04495", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04765", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04777", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04928", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04930", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04932", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04933", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04939", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04953", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04969", "FIPS": "23019", "County": "Penobscot", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04406", "FIPS": "23021", "County": "Piscataquis", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04414", "FIPS": "23021", "County": "Piscataquis", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04415", "FIPS": "23021", "County": "Piscataquis", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04426", "FIPS": "23021", "County": "Piscataquis", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04441", "FIPS": "23021", "County": "Piscataquis", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04442", "FIPS": "23021", "County": "Piscataquis", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04443", "FIPS": "23021", "County": "Piscataquis", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04463", "FIPS": "23021", "County": "Piscataquis", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04464", "FIPS": "23021", "County": "Piscataquis", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04479", "FIPS": "23021", "County": "Piscataquis", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04481", "FIPS": "23021", "County": "Piscataquis", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04485", "FIPS": "23021", "County": "Piscataquis", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04008", "FIPS": "23023", "County": "Sagadahoc", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04086", "FIPS": "23023", "County": "Sagadahoc", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04287", "FIPS": "23023", "County": "Sagadahoc", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04357", "FIPS": "23023", "County": "Sagadahoc", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04530", "FIPS": "23023", "County": "Sagadahoc", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04548", "FIPS": "23023", "County": "Sagadahoc", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04562", "FIPS": "23023", "County": "Sagadahoc", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04565", "FIPS": "23023", "County": "Sagadahoc", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04579", "FIPS": "23023", "County": "Sagadahoc", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04478", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04911", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04912", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04920", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04923", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04924", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04925", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04929", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04937", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04942", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04943", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04944", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04945", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04950", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04954", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04957", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04958", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04961", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04965", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04967", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04971", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04975", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04976", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04978", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04979", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04985", "FIPS": "23025", "County": "Somerset", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04354", "FIPS": "23027", "County": "Waldo", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04438", "FIPS": "23027", "County": "Waldo", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04496", "FIPS": "23027", "County": "Waldo", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04848", "FIPS": "23027", "County": "Waldo", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04849", "FIPS": "23027", "County": "Waldo", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04850", "FIPS": "23027", "County": "Waldo", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04915", "FIPS": "23027", "County": "Waldo", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04921", "FIPS": "23027", "County": "Waldo", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04922", "FIPS": "23027", "County": "Waldo", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04941", "FIPS": "23027", "County": "Waldo", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04949", "FIPS": "23027", "County": "Waldo", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04951", "FIPS": "23027", "County": "Waldo", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04952", "FIPS": "23027", "County": "Waldo", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04972", "FIPS": "23027", "County": "Waldo", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04973", "FIPS": "23027", "County": "Waldo", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04974", "FIPS": "23027", "County": "Waldo", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04981", "FIPS": "23027", "County": "Waldo", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04986", "FIPS": "23027", "County": "Waldo", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04987", "FIPS": "23027", "County": "Waldo", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04988", "FIPS": "23027", "County": "Waldo", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04413", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04424", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04454", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04490", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04491", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04492", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04606", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04611", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04619", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04622", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04623", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04626", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04628", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04630", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04631", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04637", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04643", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04648", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04649", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04652", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04654", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04655", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04657", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04658", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04666", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04667", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04668", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04671", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04680", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04686", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04691", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "04694", "FIPS": "23029", "County": "Washington", "State": "ME", "DMA": {"Code": 537, "Name": "BANGOR"}},
  {"Postal_Code": "03901", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03902", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03903", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03904", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03905", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03906", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03907", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03908", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03909", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03910", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03911", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04001", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04002", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04004", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04005", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04006", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04007", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04014", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04020", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04027", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04028", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04030", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04042", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04043", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04046", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04047", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04048", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04049", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04054", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04056", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04061", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04063", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04064", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04072", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04073", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04076", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04083", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04087", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04090", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04093", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04094", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "04095", "FIPS": "23031", "County": "York", "State": "ME", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "21501", "FIPS": "24001", "County": "Allegany", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21502", "FIPS": "24001", "County": "Allegany", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21503", "FIPS": "24001", "County": "Allegany", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21504", "FIPS": "24001", "County": "Allegany", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21505", "FIPS": "24001", "County": "Allegany", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21521", "FIPS": "24001", "County": "Allegany", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21524", "FIPS": "24001", "County": "Allegany", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21528", "FIPS": "24001", "County": "Allegany", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21529", "FIPS": "24001", "County": "Allegany", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21530", "FIPS": "24001", "County": "Allegany", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21532", "FIPS": "24001", "County": "Allegany", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21539", "FIPS": "24001", "County": "Allegany", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21540", "FIPS": "24001", "County": "Allegany", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21542", "FIPS": "24001", "County": "Allegany", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21543", "FIPS": "24001", "County": "Allegany", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21545", "FIPS": "24001", "County": "Allegany", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21555", "FIPS": "24001", "County": "Allegany", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21556", "FIPS": "24001", "County": "Allegany", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21557", "FIPS": "24001", "County": "Allegany", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21560", "FIPS": "24001", "County": "Allegany", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21562", "FIPS": "24001", "County": "Allegany", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21766", "FIPS": "24001", "County": "Allegany", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20711", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "20724", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "20733", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "20751", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "20755", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "20758", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "20764", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "20765", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "20776", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "20778", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "20779", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21012", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21032", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21035", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21037", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21054", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21056", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21060", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21061", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21062", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21076", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21077", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21090", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21098", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21106", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21108", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21113", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21114", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21122", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21123", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21140", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21144", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21146", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21226", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21240", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21401", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21402", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21403", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21404", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21405", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21409", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21411", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21412", "FIPS": "24003", "County": "Anne Arundel", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21013", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21020", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21022", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21023", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21027", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21030", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21031", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21051", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21052", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21053", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21057", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21065", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21071", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21082", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21087", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21092", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21093", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21094", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21105", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21111", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21117", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21120", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21128", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21131", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21133", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21136", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21139", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21152", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21153", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21155", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21156", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21162", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21204", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21207", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21208", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21219", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21220", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21221", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21222", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21227", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21228", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21234", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21236", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21237", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21244", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21250", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21252", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21282", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21284", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21285", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21286", "FIPS": "24005", "County": "Baltimore", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "20610", "FIPS": "24009", "County": "Calvert", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20615", "FIPS": "24009", "County": "Calvert", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20629", "FIPS": "24009", "County": "Calvert", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20639", "FIPS": "24009", "County": "Calvert", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20657", "FIPS": "24009", "County": "Calvert", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20676", "FIPS": "24009", "County": "Calvert", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20678", "FIPS": "24009", "County": "Calvert", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20685", "FIPS": "24009", "County": "Calvert", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20688", "FIPS": "24009", "County": "Calvert", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20689", "FIPS": "24009", "County": "Calvert", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20714", "FIPS": "24009", "County": "Calvert", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20732", "FIPS": "24009", "County": "Calvert", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20736", "FIPS": "24009", "County": "Calvert", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20754", "FIPS": "24009", "County": "Calvert", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21609", "FIPS": "24011", "County": "Caroline", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21629", "FIPS": "24011", "County": "Caroline", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21632", "FIPS": "24011", "County": "Caroline", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21636", "FIPS": "24011", "County": "Caroline", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21639", "FIPS": "24011", "County": "Caroline", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21640", "FIPS": "24011", "County": "Caroline", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21641", "FIPS": "24011", "County": "Caroline", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21649", "FIPS": "24011", "County": "Caroline", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21655", "FIPS": "24011", "County": "Caroline", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21660", "FIPS": "24011", "County": "Caroline", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21670", "FIPS": "24011", "County": "Caroline", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21681", "FIPS": "24011", "County": "Caroline", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21682", "FIPS": "24011", "County": "Caroline", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21683", "FIPS": "24011", "County": "Caroline", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21684", "FIPS": "24011", "County": "Caroline", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21685", "FIPS": "24011", "County": "Caroline", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21686", "FIPS": "24011", "County": "Caroline", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21687", "FIPS": "24011", "County": "Caroline", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21688", "FIPS": "24011", "County": "Caroline", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21048", "FIPS": "24013", "County": "Carroll", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21074", "FIPS": "24013", "County": "Carroll", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21088", "FIPS": "24013", "County": "Carroll", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21102", "FIPS": "24013", "County": "Carroll", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21104", "FIPS": "24013", "County": "Carroll", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21157", "FIPS": "24013", "County": "Carroll", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21158", "FIPS": "24013", "County": "Carroll", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21757", "FIPS": "24013", "County": "Carroll", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21776", "FIPS": "24013", "County": "Carroll", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21784", "FIPS": "24013", "County": "Carroll", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21787", "FIPS": "24013", "County": "Carroll", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21791", "FIPS": "24013", "County": "Carroll", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21901", "FIPS": "24015", "County": "Cecil", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21902", "FIPS": "24015", "County": "Cecil", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21903", "FIPS": "24015", "County": "Cecil", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21904", "FIPS": "24015", "County": "Cecil", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21911", "FIPS": "24015", "County": "Cecil", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21912", "FIPS": "24015", "County": "Cecil", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21913", "FIPS": "24015", "County": "Cecil", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21914", "FIPS": "24015", "County": "Cecil", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21915", "FIPS": "24015", "County": "Cecil", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21916", "FIPS": "24015", "County": "Cecil", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21917", "FIPS": "24015", "County": "Cecil", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21918", "FIPS": "24015", "County": "Cecil", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21919", "FIPS": "24015", "County": "Cecil", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21920", "FIPS": "24015", "County": "Cecil", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21921", "FIPS": "24015", "County": "Cecil", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21922", "FIPS": "24015", "County": "Cecil", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21930", "FIPS": "24015", "County": "Cecil", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "20601", "FIPS": "24017", "County": "Charles", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20602", "FIPS": "24017", "County": "Charles", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20603", "FIPS": "24017", "County": "Charles", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20604", "FIPS": "24017", "County": "Charles", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20611", "FIPS": "24017", "County": "Charles", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20612", "FIPS": "24017", "County": "Charles", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20616", "FIPS": "24017", "County": "Charles", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20617", "FIPS": "24017", "County": "Charles", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20625", "FIPS": "24017", "County": "Charles", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20632", "FIPS": "24017", "County": "Charles", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20637", "FIPS": "24017", "County": "Charles", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20640", "FIPS": "24017", "County": "Charles", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20643", "FIPS": "24017", "County": "Charles", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20645", "FIPS": "24017", "County": "Charles", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20646", "FIPS": "24017", "County": "Charles", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20658", "FIPS": "24017", "County": "Charles", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20661", "FIPS": "24017", "County": "Charles", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20662", "FIPS": "24017", "County": "Charles", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20664", "FIPS": "24017", "County": "Charles", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20675", "FIPS": "24017", "County": "Charles", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20677", "FIPS": "24017", "County": "Charles", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20682", "FIPS": "24017", "County": "Charles", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20693", "FIPS": "24017", "County": "Charles", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20695", "FIPS": "24017", "County": "Charles", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21613", "FIPS": "24019", "County": "Dorchester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21622", "FIPS": "24019", "County": "Dorchester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21626", "FIPS": "24019", "County": "Dorchester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21627", "FIPS": "24019", "County": "Dorchester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21631", "FIPS": "24019", "County": "Dorchester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21634", "FIPS": "24019", "County": "Dorchester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21643", "FIPS": "24019", "County": "Dorchester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21648", "FIPS": "24019", "County": "Dorchester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21659", "FIPS": "24019", "County": "Dorchester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21664", "FIPS": "24019", "County": "Dorchester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21669", "FIPS": "24019", "County": "Dorchester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21672", "FIPS": "24019", "County": "Dorchester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21675", "FIPS": "24019", "County": "Dorchester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21677", "FIPS": "24019", "County": "Dorchester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21835", "FIPS": "24019", "County": "Dorchester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21869", "FIPS": "24019", "County": "Dorchester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21701", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21702", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21703", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21704", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21705", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21709", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21710", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21714", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21716", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21717", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21718", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21727", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21754", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21755", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21758", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21759", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21762", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21769", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21770", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21771", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21773", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21774", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21775", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21777", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21778", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21780", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21788", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21790", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21792", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21793", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21798", "FIPS": "24021", "County": "Frederick", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21520", "FIPS": "24023", "County": "Garrett", "State": "MD", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "21522", "FIPS": "24023", "County": "Garrett", "State": "MD", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "21523", "FIPS": "24023", "County": "Garrett", "State": "MD", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "21531", "FIPS": "24023", "County": "Garrett", "State": "MD", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "21536", "FIPS": "24023", "County": "Garrett", "State": "MD", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "21538", "FIPS": "24023", "County": "Garrett", "State": "MD", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "21541", "FIPS": "24023", "County": "Garrett", "State": "MD", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "21550", "FIPS": "24023", "County": "Garrett", "State": "MD", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "21561", "FIPS": "24023", "County": "Garrett", "State": "MD", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "21001", "FIPS": "24025", "County": "Harford", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21005", "FIPS": "24025", "County": "Harford", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21009", "FIPS": "24025", "County": "Harford", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21010", "FIPS": "24025", "County": "Harford", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21014", "FIPS": "24025", "County": "Harford", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21015", "FIPS": "24025", "County": "Harford", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21017", "FIPS": "24025", "County": "Harford", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21018", "FIPS": "24025", "County": "Harford", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21028", "FIPS": "24025", "County": "Harford", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21034", "FIPS": "24025", "County": "Harford", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21040", "FIPS": "24025", "County": "Harford", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21047", "FIPS": "24025", "County": "Harford", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21050", "FIPS": "24025", "County": "Harford", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21078", "FIPS": "24025", "County": "Harford", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21084", "FIPS": "24025", "County": "Harford", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21085", "FIPS": "24025", "County": "Harford", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21130", "FIPS": "24025", "County": "Harford", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21132", "FIPS": "24025", "County": "Harford", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21154", "FIPS": "24025", "County": "Harford", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21160", "FIPS": "24025", "County": "Harford", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21161", "FIPS": "24025", "County": "Harford", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "20701", "FIPS": "24027", "County": "Howard", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "20723", "FIPS": "24027", "County": "Howard", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "20759", "FIPS": "24027", "County": "Howard", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "20763", "FIPS": "24027", "County": "Howard", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "20777", "FIPS": "24027", "County": "Howard", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "20794", "FIPS": "24027", "County": "Howard", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21029", "FIPS": "24027", "County": "Howard", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21036", "FIPS": "24027", "County": "Howard", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21041", "FIPS": "24027", "County": "Howard", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21042", "FIPS": "24027", "County": "Howard", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21043", "FIPS": "24027", "County": "Howard", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21044", "FIPS": "24027", "County": "Howard", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21045", "FIPS": "24027", "County": "Howard", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21046", "FIPS": "24027", "County": "Howard", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21075", "FIPS": "24027", "County": "Howard", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21150", "FIPS": "24027", "County": "Howard", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21163", "FIPS": "24027", "County": "Howard", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21723", "FIPS": "24027", "County": "Howard", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21737", "FIPS": "24027", "County": "Howard", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21738", "FIPS": "24027", "County": "Howard", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21765", "FIPS": "24027", "County": "Howard", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21794", "FIPS": "24027", "County": "Howard", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21797", "FIPS": "24027", "County": "Howard", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21610", "FIPS": "24029", "County": "Kent", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21620", "FIPS": "24029", "County": "Kent", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21635", "FIPS": "24029", "County": "Kent", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21645", "FIPS": "24029", "County": "Kent", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21650", "FIPS": "24029", "County": "Kent", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21651", "FIPS": "24029", "County": "Kent", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21661", "FIPS": "24029", "County": "Kent", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21667", "FIPS": "24029", "County": "Kent", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21678", "FIPS": "24029", "County": "Kent", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "20810", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20811", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20812", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20813", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20814", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20815", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20816", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20817", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20818", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20824", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20825", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20827", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20830", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20832", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20833", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20837", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20838", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20839", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20841", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20842", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20847", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20848", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20849", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20850", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20851", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20852", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20853", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20854", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20855", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20857", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20859", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20860", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20861", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20862", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20866", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20868", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20871", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20872", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20874", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20875", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20876", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20877", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20878", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20879", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20880", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20882", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20883", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20884", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20885", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20886", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20889", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20891", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20892", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20894", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20895", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20896", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20897", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20898", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20899", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20901", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20902", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20903", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20904", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20905", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20906", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20907", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20908", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20910", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20911", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20912", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20913", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20914", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20915", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20916", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20918", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20993", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20997", "FIPS": "24031", "County": "Montgomery", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20607", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20608", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20613", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20623", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20697", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20703", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20704", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20705", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20706", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20707", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20708", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20709", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20710", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20712", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20715", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20716", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20717", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20718", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20719", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20720", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20721", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20722", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20725", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20726", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20731", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20735", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20737", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20738", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20740", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20741", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20742", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20743", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20744", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20745", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20746", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20747", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20748", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20749", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20750", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20752", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20753", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20757", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20762", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20768", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20769", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20770", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20771", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20772", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20773", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20774", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20775", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20781", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20782", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20783", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20784", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20785", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20787", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20788", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20790", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20791", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20792", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20797", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20799", "FIPS": "24033", "County": "Prince Georges", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21607", "FIPS": "24035", "County": "Queen Annes", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21617", "FIPS": "24035", "County": "Queen Annes", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21619", "FIPS": "24035", "County": "Queen Annes", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21623", "FIPS": "24035", "County": "Queen Annes", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21628", "FIPS": "24035", "County": "Queen Annes", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21638", "FIPS": "24035", "County": "Queen Annes", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21644", "FIPS": "24035", "County": "Queen Annes", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21656", "FIPS": "24035", "County": "Queen Annes", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21657", "FIPS": "24035", "County": "Queen Annes", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21658", "FIPS": "24035", "County": "Queen Annes", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21666", "FIPS": "24035", "County": "Queen Annes", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21668", "FIPS": "24035", "County": "Queen Annes", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21690", "FIPS": "24035", "County": "Queen Annes", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "20606", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20609", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20618", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20619", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20620", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20621", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20622", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20624", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20626", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20627", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20628", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20630", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20634", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20635", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20636", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20650", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20653", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20656", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20659", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20660", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20667", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20670", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20674", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20680", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20684", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20686", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20687", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20690", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20692", "FIPS": "24037", "County": "Saint Marys", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21817", "FIPS": "24039", "County": "Somerset", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21821", "FIPS": "24039", "County": "Somerset", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21824", "FIPS": "24039", "County": "Somerset", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21836", "FIPS": "24039", "County": "Somerset", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21838", "FIPS": "24039", "County": "Somerset", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21853", "FIPS": "24039", "County": "Somerset", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21857", "FIPS": "24039", "County": "Somerset", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21866", "FIPS": "24039", "County": "Somerset", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21867", "FIPS": "24039", "County": "Somerset", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21871", "FIPS": "24039", "County": "Somerset", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21890", "FIPS": "24039", "County": "Somerset", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21601", "FIPS": "24041", "County": "Talbot", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21606", "FIPS": "24041", "County": "Talbot", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21612", "FIPS": "24041", "County": "Talbot", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21624", "FIPS": "24041", "County": "Talbot", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21625", "FIPS": "24041", "County": "Talbot", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21647", "FIPS": "24041", "County": "Talbot", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21652", "FIPS": "24041", "County": "Talbot", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21653", "FIPS": "24041", "County": "Talbot", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21654", "FIPS": "24041", "County": "Talbot", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21662", "FIPS": "24041", "County": "Talbot", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21663", "FIPS": "24041", "County": "Talbot", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21665", "FIPS": "24041", "County": "Talbot", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21671", "FIPS": "24041", "County": "Talbot", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21673", "FIPS": "24041", "County": "Talbot", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21676", "FIPS": "24041", "County": "Talbot", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21679", "FIPS": "24041", "County": "Talbot", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21711", "FIPS": "24043", "County": "Washington", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21713", "FIPS": "24043", "County": "Washington", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21715", "FIPS": "24043", "County": "Washington", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21719", "FIPS": "24043", "County": "Washington", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21720", "FIPS": "24043", "County": "Washington", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21721", "FIPS": "24043", "County": "Washington", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21722", "FIPS": "24043", "County": "Washington", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21733", "FIPS": "24043", "County": "Washington", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21734", "FIPS": "24043", "County": "Washington", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21740", "FIPS": "24043", "County": "Washington", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21741", "FIPS": "24043", "County": "Washington", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21742", "FIPS": "24043", "County": "Washington", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21746", "FIPS": "24043", "County": "Washington", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21747", "FIPS": "24043", "County": "Washington", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21748", "FIPS": "24043", "County": "Washington", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21749", "FIPS": "24043", "County": "Washington", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21750", "FIPS": "24043", "County": "Washington", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21756", "FIPS": "24043", "County": "Washington", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21767", "FIPS": "24043", "County": "Washington", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21779", "FIPS": "24043", "County": "Washington", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21781", "FIPS": "24043", "County": "Washington", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21782", "FIPS": "24043", "County": "Washington", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21783", "FIPS": "24043", "County": "Washington", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21795", "FIPS": "24043", "County": "Washington", "State": "MD", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "21801", "FIPS": "24045", "County": "Wicomico", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21802", "FIPS": "24045", "County": "Wicomico", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21803", "FIPS": "24045", "County": "Wicomico", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21804", "FIPS": "24045", "County": "Wicomico", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21810", "FIPS": "24045", "County": "Wicomico", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21814", "FIPS": "24045", "County": "Wicomico", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21826", "FIPS": "24045", "County": "Wicomico", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21830", "FIPS": "24045", "County": "Wicomico", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21837", "FIPS": "24045", "County": "Wicomico", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21840", "FIPS": "24045", "County": "Wicomico", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21849", "FIPS": "24045", "County": "Wicomico", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21850", "FIPS": "24045", "County": "Wicomico", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21852", "FIPS": "24045", "County": "Wicomico", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21856", "FIPS": "24045", "County": "Wicomico", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21861", "FIPS": "24045", "County": "Wicomico", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21865", "FIPS": "24045", "County": "Wicomico", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21874", "FIPS": "24045", "County": "Wicomico", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21875", "FIPS": "24045", "County": "Wicomico", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21811", "FIPS": "24047", "County": "Worcester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21813", "FIPS": "24047", "County": "Worcester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21822", "FIPS": "24047", "County": "Worcester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21829", "FIPS": "24047", "County": "Worcester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21841", "FIPS": "24047", "County": "Worcester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21842", "FIPS": "24047", "County": "Worcester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21843", "FIPS": "24047", "County": "Worcester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21851", "FIPS": "24047", "County": "Worcester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21862", "FIPS": "24047", "County": "Worcester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21863", "FIPS": "24047", "County": "Worcester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21864", "FIPS": "24047", "County": "Worcester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21872", "FIPS": "24047", "County": "Worcester", "State": "MD", "DMA": {"Code": 576, "Name": "SALISBURY"}},
  {"Postal_Code": "21201", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21202", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21203", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21205", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21206", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21209", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21210", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21211", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21212", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21213", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21214", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21215", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21216", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21217", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21218", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21223", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21224", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21225", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21229", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21230", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21231", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21233", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21235", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21239", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21241", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21251", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21263", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21264", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21265", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21268", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21270", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21273", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21274", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21275", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21278", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21279", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21280", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21281", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21283", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21287", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21288", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21289", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21290", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21297", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "21298", "FIPS": "24510", "County": "Baltimore City", "State": "MD", "DMA": {"Code": 512, "Name": "BALTIMORE"}},
  {"Postal_Code": "02532", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02534", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02536", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02537", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02540", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02541", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02542", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02543", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02553", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02556", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02559", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02561", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02562", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02563", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02565", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02574", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02601", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02630", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02631", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02632", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02633", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02634", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02635", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02636", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02637", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02638", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02639", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02641", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02642", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02643", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02644", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02645", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02646", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02647", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02648", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02649", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02650", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02651", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02652", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02653", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02655", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02657", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02659", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02660", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02661", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02662", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02663", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02664", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02666", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02667", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02668", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02669", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02670", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02671", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02672", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02673", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02675", "FIPS": "25001", "County": "Barnstable", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01029", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01201", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01202", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01203", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01220", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01222", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01223", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01224", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01225", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01226", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01227", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01229", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01230", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01235", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01236", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01237", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01238", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01240", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01242", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01244", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01245", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01247", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01252", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01253", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01254", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01255", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01256", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01257", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01258", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01259", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01260", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01262", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01263", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01264", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01266", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01267", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01270", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "01343", "FIPS": "25003", "County": "Berkshire", "State": "MA", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "02031", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02048", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02334", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02356", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02357", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02375", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02702", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02703", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02712", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02714", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02715", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02717", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02718", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02719", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02720", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02721", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02722", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02723", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02724", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02725", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02726", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02740", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02741", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02742", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02743", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02744", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02745", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02746", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02747", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02748", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02760", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02761", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02763", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02764", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02766", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02767", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02768", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02769", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02771", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02777", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02779", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02780", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02783", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02790", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02791", "FIPS": "25005", "County": "Bristol", "State": "MA", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02535", "FIPS": "25007", "County": "Dukes", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02539", "FIPS": "25007", "County": "Dukes", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02552", "FIPS": "25007", "County": "Dukes", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02557", "FIPS": "25007", "County": "Dukes", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02568", "FIPS": "25007", "County": "Dukes", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02573", "FIPS": "25007", "County": "Dukes", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02575", "FIPS": "25007", "County": "Dukes", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02713", "FIPS": "25007", "County": "Dukes", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01810", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01812", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01830", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01831", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01832", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01833", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01834", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01835", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01840", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01841", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01842", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01843", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01844", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01845", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01860", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01885", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01899", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01901", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01902", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01903", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01904", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01905", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01906", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01907", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01908", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01910", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01913", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01915", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01921", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01922", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01923", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01929", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01930", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01931", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01936", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01937", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01938", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01940", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01944", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01945", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01949", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01950", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01951", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01952", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01960", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01961", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01965", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01966", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01969", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01970", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01971", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01982", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01983", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01984", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01985", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05501", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05544", "FIPS": "25009", "County": "Essex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01054", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01072", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01093", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01301", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01302", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01330", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01337", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01338", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01339", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01340", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01341", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01342", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01344", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01346", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01347", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01349", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01350", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01351", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01354", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01355", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01360", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01364", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01367", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01370", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01373", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01375", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01376", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01378", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01379", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01380", "FIPS": "25011", "County": "Franklin", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01001", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01008", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01009", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01010", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01011", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01013", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01014", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01020", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01021", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01022", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01028", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01030", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01034", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01036", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01040", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01041", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01056", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01057", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01069", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01071", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01077", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01079", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01080", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01081", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01085", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01086", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01089", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01090", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01095", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01097", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01101", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01102", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01103", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01104", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01105", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01106", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01107", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01108", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01109", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01111", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01115", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01116", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01118", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01119", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01128", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01129", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01133", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01138", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01139", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01144", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01151", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01152", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01195", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01199", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01521", "FIPS": "25013", "County": "Hampden", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01002", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01003", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01004", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01007", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01012", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01026", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01027", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01032", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01033", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01035", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01038", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01039", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01050", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01053", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01059", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01060", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01061", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01062", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01063", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01066", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01070", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01073", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01075", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01082", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01084", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01088", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01096", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01098", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01243", "FIPS": "25015", "County": "Hampshire", "State": "MA", "DMA": {"Code": 543, "Name": "SPRINGFIELD-HOLYOKE"}},
  {"Postal_Code": "01431", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01432", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01434", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01450", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01460", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01463", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01464", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01469", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01470", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01471", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01472", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01474", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01701", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01702", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01703", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01704", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01705", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01718", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01719", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01720", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01721", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01730", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01731", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01741", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01742", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01746", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01748", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01749", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01752", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01754", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01760", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01770", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01773", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01775", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01776", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01778", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01784", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01801", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01803", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01805", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01806", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01807", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01808", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01813", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01815", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01821", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01822", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01824", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01826", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01827", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01850", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01851", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01852", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01853", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01854", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01862", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01863", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01864", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01865", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01866", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01867", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01876", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01879", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01880", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01886", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01887", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01888", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01889", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01890", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02138", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02139", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02140", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02141", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02142", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02143", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02144", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02145", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02148", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02149", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02153", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02155", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02156", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02176", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02180", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02238", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02239", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02420", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02421", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02451", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02452", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02453", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02454", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02455", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02456", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02458", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02459", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02460", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02461", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02462", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02464", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02465", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02466", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02467", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02468", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02471", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02472", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02474", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02475", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02476", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02477", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02478", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02479", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02493", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02495", "FIPS": "25017", "County": "Middlesex", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02554", "FIPS": "25019", "County": "Nantucket", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02564", "FIPS": "25019", "County": "Nantucket", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02584", "FIPS": "25019", "County": "Nantucket", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02019", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02021", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02025", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02026", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02027", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02030", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02032", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02035", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02038", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02052", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02053", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02054", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02056", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02062", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02067", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02070", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02071", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02072", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02081", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02090", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02093", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02169", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02170", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02171", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02184", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02185", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02186", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02187", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02188", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02189", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02190", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02191", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02269", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02322", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02343", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02368", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02445", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02446", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02447", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02457", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02481", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02482", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02492", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02494", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02762", "FIPS": "25021", "County": "Norfolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02018", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02020", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02040", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02041", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02043", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02044", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02045", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02047", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02050", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02051", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02055", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02059", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02060", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02061", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02065", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02066", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02301", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02302", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02303", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02304", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02305", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02324", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02325", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02327", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02330", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02331", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02332", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02333", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02337", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02338", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02339", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02340", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02341", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02344", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02345", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02346", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02347", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02348", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02349", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02350", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02351", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02355", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02358", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02359", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02360", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02361", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02362", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02364", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02366", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02367", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02370", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02379", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02381", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02382", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02538", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02558", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02571", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02576", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02738", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02739", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02770", "FIPS": "25023", "County": "Plymouth", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02108", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02109", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02110", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02111", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02112", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02113", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02114", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02115", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02116", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02117", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02118", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02119", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02120", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02121", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02122", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02123", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02124", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02125", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02126", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02127", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02128", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02129", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02130", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02131", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02132", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02133", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02134", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02135", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02136", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02137", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02150", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02151", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02152", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02163", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02196", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02199", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02201", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02203", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02204", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02205", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02206", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02207", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02210", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02211", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02212", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02215", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02216", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02217", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02222", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02228", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02241", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02266", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02283", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02284", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02293", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02295", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02297", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "02298", "FIPS": "25025", "County": "Suffolk", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01005", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01031", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01037", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01068", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01074", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01083", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01092", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01094", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01331", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01366", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01368", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01420", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01430", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01436", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01438", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01440", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01441", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01451", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01452", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01453", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01462", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01467", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01468", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01473", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01475", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01477", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01501", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01503", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01504", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01505", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01506", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01507", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01508", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01509", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01510", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01515", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01516", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01517", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01518", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01519", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01520", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01522", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01523", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01524", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01525", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01526", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01527", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01529", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01531", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01532", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01534", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01535", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01536", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01537", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01538", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01540", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01541", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01542", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01543", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01545", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01546", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01550", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01560", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01561", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01562", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01564", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01566", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01568", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01569", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01570", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01571", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01580", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01581", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01582", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01583", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01585", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01586", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01588", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01590", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01601", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01602", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01603", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01604", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01605", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01606", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01607", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01608", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01609", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01610", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01611", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01612", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01613", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01614", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01615", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01653", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01654", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01655", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01740", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01745", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01747", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01756", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01757", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "01772", "FIPS": "25027", "County": "Worcester", "State": "MA", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "48705", "FIPS": "26001", "County": "Alcona", "State": "MI", "DMA": {"Code": 583, "Name": "ALPENA"}},
  {"Postal_Code": "48721", "FIPS": "26001", "County": "Alcona", "State": "MI", "DMA": {"Code": 583, "Name": "ALPENA"}},
  {"Postal_Code": "48728", "FIPS": "26001", "County": "Alcona", "State": "MI", "DMA": {"Code": 583, "Name": "ALPENA"}},
  {"Postal_Code": "48737", "FIPS": "26001", "County": "Alcona", "State": "MI", "DMA": {"Code": 583, "Name": "ALPENA"}},
  {"Postal_Code": "48738", "FIPS": "26001", "County": "Alcona", "State": "MI", "DMA": {"Code": 583, "Name": "ALPENA"}},
  {"Postal_Code": "48740", "FIPS": "26001", "County": "Alcona", "State": "MI", "DMA": {"Code": 583, "Name": "ALPENA"}},
  {"Postal_Code": "48742", "FIPS": "26001", "County": "Alcona", "State": "MI", "DMA": {"Code": 583, "Name": "ALPENA"}},
  {"Postal_Code": "48745", "FIPS": "26001", "County": "Alcona", "State": "MI", "DMA": {"Code": 583, "Name": "ALPENA"}},
  {"Postal_Code": "48762", "FIPS": "26001", "County": "Alcona", "State": "MI", "DMA": {"Code": 583, "Name": "ALPENA"}},
  {"Postal_Code": "49806", "FIPS": "26003", "County": "Alger", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49816", "FIPS": "26003", "County": "Alger", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49822", "FIPS": "26003", "County": "Alger", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49825", "FIPS": "26003", "County": "Alger", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49826", "FIPS": "26003", "County": "Alger", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49839", "FIPS": "26003", "County": "Alger", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49862", "FIPS": "26003", "County": "Alger", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49884", "FIPS": "26003", "County": "Alger", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49891", "FIPS": "26003", "County": "Alger", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49895", "FIPS": "26003", "County": "Alger", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49010", "FIPS": "26005", "County": "Allegan", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49070", "FIPS": "26005", "County": "Allegan", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49078", "FIPS": "26005", "County": "Allegan", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49080", "FIPS": "26005", "County": "Allegan", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49311", "FIPS": "26005", "County": "Allegan", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49314", "FIPS": "26005", "County": "Allegan", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49323", "FIPS": "26005", "County": "Allegan", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49328", "FIPS": "26005", "County": "Allegan", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49335", "FIPS": "26005", "County": "Allegan", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49344", "FIPS": "26005", "County": "Allegan", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49348", "FIPS": "26005", "County": "Allegan", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49406", "FIPS": "26005", "County": "Allegan", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49408", "FIPS": "26005", "County": "Allegan", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49416", "FIPS": "26005", "County": "Allegan", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49419", "FIPS": "26005", "County": "Allegan", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49450", "FIPS": "26005", "County": "Allegan", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49453", "FIPS": "26005", "County": "Allegan", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49707", "FIPS": "26007", "County": "Alpena", "State": "MI", "DMA": {"Code": 583, "Name": "ALPENA"}},
  {"Postal_Code": "49744", "FIPS": "26007", "County": "Alpena", "State": "MI", "DMA": {"Code": 583, "Name": "ALPENA"}},
  {"Postal_Code": "49747", "FIPS": "26007", "County": "Alpena", "State": "MI", "DMA": {"Code": 583, "Name": "ALPENA"}},
  {"Postal_Code": "49753", "FIPS": "26007", "County": "Alpena", "State": "MI", "DMA": {"Code": 583, "Name": "ALPENA"}},
  {"Postal_Code": "49766", "FIPS": "26007", "County": "Alpena", "State": "MI", "DMA": {"Code": 583, "Name": "ALPENA"}},
  {"Postal_Code": "49611", "FIPS": "26009", "County": "Antrim", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49612", "FIPS": "26009", "County": "Antrim", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49615", "FIPS": "26009", "County": "Antrim", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49622", "FIPS": "26009", "County": "Antrim", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49627", "FIPS": "26009", "County": "Antrim", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49629", "FIPS": "26009", "County": "Antrim", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49648", "FIPS": "26009", "County": "Antrim", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49659", "FIPS": "26009", "County": "Antrim", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49729", "FIPS": "26009", "County": "Antrim", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "48610", "FIPS": "26011", "County": "Arenac", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48658", "FIPS": "26011", "County": "Arenac", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48659", "FIPS": "26011", "County": "Arenac", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48703", "FIPS": "26011", "County": "Arenac", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48749", "FIPS": "26011", "County": "Arenac", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48765", "FIPS": "26011", "County": "Arenac", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48766", "FIPS": "26011", "County": "Arenac", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "49908", "FIPS": "26013", "County": "Baraga", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49919", "FIPS": "26013", "County": "Baraga", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49946", "FIPS": "26013", "County": "Baraga", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49962", "FIPS": "26013", "County": "Baraga", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49970", "FIPS": "26013", "County": "Baraga", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "48897", "FIPS": "26015", "County": "Barry", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49035", "FIPS": "26015", "County": "Barry", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49046", "FIPS": "26015", "County": "Barry", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49050", "FIPS": "26015", "County": "Barry", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49058", "FIPS": "26015", "County": "Barry", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49060", "FIPS": "26015", "County": "Barry", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49073", "FIPS": "26015", "County": "Barry", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49325", "FIPS": "26015", "County": "Barry", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49333", "FIPS": "26015", "County": "Barry", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48611", "FIPS": "26017", "County": "Bay", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48613", "FIPS": "26017", "County": "Bay", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48631", "FIPS": "26017", "County": "Bay", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48634", "FIPS": "26017", "County": "Bay", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48650", "FIPS": "26017", "County": "Bay", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48706", "FIPS": "26017", "County": "Bay", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48707", "FIPS": "26017", "County": "Bay", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48708", "FIPS": "26017", "County": "Bay", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48710", "FIPS": "26017", "County": "Bay", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48732", "FIPS": "26017", "County": "Bay", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48747", "FIPS": "26017", "County": "Bay", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "49616", "FIPS": "26019", "County": "Benzie", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49617", "FIPS": "26019", "County": "Benzie", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49628", "FIPS": "26019", "County": "Benzie", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49635", "FIPS": "26019", "County": "Benzie", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49640", "FIPS": "26019", "County": "Benzie", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49650", "FIPS": "26019", "County": "Benzie", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49683", "FIPS": "26019", "County": "Benzie", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49022", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49023", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49038", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49039", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49084", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49085", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49098", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49101", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49102", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49103", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49104", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49106", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49107", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49111", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49113", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49115", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49116", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49117", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49119", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49120", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49121", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49125", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49126", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49127", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49128", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49129", "FIPS": "26021", "County": "Berrien", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49028", "FIPS": "26023", "County": "Branch", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49036", "FIPS": "26023", "County": "Branch", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49082", "FIPS": "26023", "County": "Branch", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49089", "FIPS": "26023", "County": "Branch", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49094", "FIPS": "26023", "County": "Branch", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49255", "FIPS": "26023", "County": "Branch", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49011", "FIPS": "26025", "County": "Calhoun", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49014", "FIPS": "26025", "County": "Calhoun", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49015", "FIPS": "26025", "County": "Calhoun", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49016", "FIPS": "26025", "County": "Calhoun", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49017", "FIPS": "26025", "County": "Calhoun", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49018", "FIPS": "26025", "County": "Calhoun", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49020", "FIPS": "26025", "County": "Calhoun", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49029", "FIPS": "26025", "County": "Calhoun", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49033", "FIPS": "26025", "County": "Calhoun", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49051", "FIPS": "26025", "County": "Calhoun", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49068", "FIPS": "26025", "County": "Calhoun", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49069", "FIPS": "26025", "County": "Calhoun", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49092", "FIPS": "26025", "County": "Calhoun", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49224", "FIPS": "26025", "County": "Calhoun", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49245", "FIPS": "26025", "County": "Calhoun", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49031", "FIPS": "26027", "County": "Cass", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49047", "FIPS": "26027", "County": "Cass", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49061", "FIPS": "26027", "County": "Cass", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49067", "FIPS": "26027", "County": "Cass", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49095", "FIPS": "26027", "County": "Cass", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49112", "FIPS": "26027", "County": "Cass", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49130", "FIPS": "26027", "County": "Cass", "State": "MI", "DMA": {"Code": 588, "Name": "SOUTH BEND-ELKHART"}},
  {"Postal_Code": "49711", "FIPS": "26029", "County": "Charlevoix", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49712", "FIPS": "26029", "County": "Charlevoix", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49713", "FIPS": "26029", "County": "Charlevoix", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49720", "FIPS": "26029", "County": "Charlevoix", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49727", "FIPS": "26029", "County": "Charlevoix", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49782", "FIPS": "26029", "County": "Charlevoix", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49796", "FIPS": "26029", "County": "Charlevoix", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49701", "FIPS": "26031", "County": "Cheboygan", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49705", "FIPS": "26031", "County": "Cheboygan", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49717", "FIPS": "26031", "County": "Cheboygan", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49721", "FIPS": "26031", "County": "Cheboygan", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49749", "FIPS": "26031", "County": "Cheboygan", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49761", "FIPS": "26031", "County": "Cheboygan", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49791", "FIPS": "26031", "County": "Cheboygan", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49792", "FIPS": "26031", "County": "Cheboygan", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49799", "FIPS": "26031", "County": "Cheboygan", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49710", "FIPS": "26033", "County": "Chippewa", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49715", "FIPS": "26033", "County": "Chippewa", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49724", "FIPS": "26033", "County": "Chippewa", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49725", "FIPS": "26033", "County": "Chippewa", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49726", "FIPS": "26033", "County": "Chippewa", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49728", "FIPS": "26033", "County": "Chippewa", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49736", "FIPS": "26033", "County": "Chippewa", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49748", "FIPS": "26033", "County": "Chippewa", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49752", "FIPS": "26033", "County": "Chippewa", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49768", "FIPS": "26033", "County": "Chippewa", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49774", "FIPS": "26033", "County": "Chippewa", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49780", "FIPS": "26033", "County": "Chippewa", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49783", "FIPS": "26033", "County": "Chippewa", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49784", "FIPS": "26033", "County": "Chippewa", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49785", "FIPS": "26033", "County": "Chippewa", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49786", "FIPS": "26033", "County": "Chippewa", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49788", "FIPS": "26033", "County": "Chippewa", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49790", "FIPS": "26033", "County": "Chippewa", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49793", "FIPS": "26033", "County": "Chippewa", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "48617", "FIPS": "26035", "County": "Clare", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "48622", "FIPS": "26035", "County": "Clare", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "48625", "FIPS": "26035", "County": "Clare", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "48632", "FIPS": "26035", "County": "Clare", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "48633", "FIPS": "26035", "County": "Clare", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "48808", "FIPS": "26037", "County": "Clinton", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48820", "FIPS": "26037", "County": "Clinton", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48822", "FIPS": "26037", "County": "Clinton", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48831", "FIPS": "26037", "County": "Clinton", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48833", "FIPS": "26037", "County": "Clinton", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48835", "FIPS": "26037", "County": "Clinton", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48853", "FIPS": "26037", "County": "Clinton", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48866", "FIPS": "26037", "County": "Clinton", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48879", "FIPS": "26037", "County": "Clinton", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48894", "FIPS": "26037", "County": "Clinton", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49733", "FIPS": "26039", "County": "Crawford", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49738", "FIPS": "26039", "County": "Crawford", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49739", "FIPS": "26039", "County": "Crawford", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49807", "FIPS": "26041", "County": "Delta", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49818", "FIPS": "26041", "County": "Delta", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49829", "FIPS": "26041", "County": "Delta", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49835", "FIPS": "26041", "County": "Delta", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49837", "FIPS": "26041", "County": "Delta", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49864", "FIPS": "26041", "County": "Delta", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49872", "FIPS": "26041", "County": "Delta", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49878", "FIPS": "26041", "County": "Delta", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49880", "FIPS": "26041", "County": "Delta", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49894", "FIPS": "26041", "County": "Delta", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49801", "FIPS": "26043", "County": "Dickinson", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49802", "FIPS": "26043", "County": "Dickinson", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49815", "FIPS": "26043", "County": "Dickinson", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49831", "FIPS": "26043", "County": "Dickinson", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49834", "FIPS": "26043", "County": "Dickinson", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49852", "FIPS": "26043", "County": "Dickinson", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49870", "FIPS": "26043", "County": "Dickinson", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49876", "FIPS": "26043", "County": "Dickinson", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49877", "FIPS": "26043", "County": "Dickinson", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49881", "FIPS": "26043", "County": "Dickinson", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49892", "FIPS": "26043", "County": "Dickinson", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "48813", "FIPS": "26045", "County": "Eaton", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48821", "FIPS": "26045", "County": "Eaton", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48827", "FIPS": "26045", "County": "Eaton", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48837", "FIPS": "26045", "County": "Eaton", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48861", "FIPS": "26045", "County": "Eaton", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48876", "FIPS": "26045", "County": "Eaton", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48890", "FIPS": "26045", "County": "Eaton", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48908", "FIPS": "26045", "County": "Eaton", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48917", "FIPS": "26045", "County": "Eaton", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49021", "FIPS": "26045", "County": "Eaton", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49076", "FIPS": "26045", "County": "Eaton", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49096", "FIPS": "26045", "County": "Eaton", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49706", "FIPS": "26047", "County": "Emmet", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49716", "FIPS": "26047", "County": "Emmet", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49718", "FIPS": "26047", "County": "Emmet", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49722", "FIPS": "26047", "County": "Emmet", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49723", "FIPS": "26047", "County": "Emmet", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49737", "FIPS": "26047", "County": "Emmet", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49740", "FIPS": "26047", "County": "Emmet", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49755", "FIPS": "26047", "County": "Emmet", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49764", "FIPS": "26047", "County": "Emmet", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49769", "FIPS": "26047", "County": "Emmet", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49770", "FIPS": "26047", "County": "Emmet", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "48411", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48420", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48423", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48430", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48433", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48436", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48437", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48438", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48439", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48449", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48451", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48457", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48458", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48463", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48473", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48480", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48501", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48502", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48503", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48504", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48505", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48506", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48507", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48509", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48519", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48529", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48531", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48532", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48550", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48551", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48552", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48553", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48554", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48555", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48556", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48557", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48559", "FIPS": "26049", "County": "Genesee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48612", "FIPS": "26051", "County": "Gladwin", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48624", "FIPS": "26051", "County": "Gladwin", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48652", "FIPS": "26051", "County": "Gladwin", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "49911", "FIPS": "26053", "County": "Gogebic", "State": "MI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "49938", "FIPS": "26053", "County": "Gogebic", "State": "MI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "49947", "FIPS": "26053", "County": "Gogebic", "State": "MI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "49959", "FIPS": "26053", "County": "Gogebic", "State": "MI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "49968", "FIPS": "26053", "County": "Gogebic", "State": "MI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "49969", "FIPS": "26053", "County": "Gogebic", "State": "MI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "49610", "FIPS": "26055", "County": "Grand Traverse", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49637", "FIPS": "26055", "County": "Grand Traverse", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49643", "FIPS": "26055", "County": "Grand Traverse", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49649", "FIPS": "26055", "County": "Grand Traverse", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49666", "FIPS": "26055", "County": "Grand Traverse", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49673", "FIPS": "26055", "County": "Grand Traverse", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49684", "FIPS": "26055", "County": "Grand Traverse", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49685", "FIPS": "26055", "County": "Grand Traverse", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49686", "FIPS": "26055", "County": "Grand Traverse", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49690", "FIPS": "26055", "County": "Grand Traverse", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49696", "FIPS": "26055", "County": "Grand Traverse", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "48615", "FIPS": "26057", "County": "Gratiot", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48662", "FIPS": "26057", "County": "Gratiot", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48801", "FIPS": "26057", "County": "Gratiot", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48802", "FIPS": "26057", "County": "Gratiot", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48806", "FIPS": "26057", "County": "Gratiot", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48807", "FIPS": "26057", "County": "Gratiot", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48830", "FIPS": "26057", "County": "Gratiot", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48832", "FIPS": "26057", "County": "Gratiot", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48847", "FIPS": "26057", "County": "Gratiot", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48856", "FIPS": "26057", "County": "Gratiot", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48862", "FIPS": "26057", "County": "Gratiot", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48871", "FIPS": "26057", "County": "Gratiot", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48874", "FIPS": "26057", "County": "Gratiot", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48877", "FIPS": "26057", "County": "Gratiot", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48880", "FIPS": "26057", "County": "Gratiot", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48889", "FIPS": "26057", "County": "Gratiot", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "49227", "FIPS": "26059", "County": "Hillsdale", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49232", "FIPS": "26059", "County": "Hillsdale", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49239", "FIPS": "26059", "County": "Hillsdale", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49242", "FIPS": "26059", "County": "Hillsdale", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49249", "FIPS": "26059", "County": "Hillsdale", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49250", "FIPS": "26059", "County": "Hillsdale", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49252", "FIPS": "26059", "County": "Hillsdale", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49257", "FIPS": "26059", "County": "Hillsdale", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49258", "FIPS": "26059", "County": "Hillsdale", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49262", "FIPS": "26059", "County": "Hillsdale", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49266", "FIPS": "26059", "County": "Hillsdale", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49271", "FIPS": "26059", "County": "Hillsdale", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49274", "FIPS": "26059", "County": "Hillsdale", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49281", "FIPS": "26059", "County": "Hillsdale", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49282", "FIPS": "26059", "County": "Hillsdale", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49288", "FIPS": "26059", "County": "Hillsdale", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49905", "FIPS": "26061", "County": "Houghton", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49913", "FIPS": "26061", "County": "Houghton", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49916", "FIPS": "26061", "County": "Houghton", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49917", "FIPS": "26061", "County": "Houghton", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49921", "FIPS": "26061", "County": "Houghton", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49922", "FIPS": "26061", "County": "Houghton", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49930", "FIPS": "26061", "County": "Houghton", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49931", "FIPS": "26061", "County": "Houghton", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49934", "FIPS": "26061", "County": "Houghton", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49942", "FIPS": "26061", "County": "Houghton", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49945", "FIPS": "26061", "County": "Houghton", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49952", "FIPS": "26061", "County": "Houghton", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49955", "FIPS": "26061", "County": "Houghton", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49958", "FIPS": "26061", "County": "Houghton", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49961", "FIPS": "26061", "County": "Houghton", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49963", "FIPS": "26061", "County": "Houghton", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49965", "FIPS": "26061", "County": "Houghton", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "48413", "FIPS": "26063", "County": "Huron", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48432", "FIPS": "26063", "County": "Huron", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48441", "FIPS": "26063", "County": "Huron", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48445", "FIPS": "26063", "County": "Huron", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48467", "FIPS": "26063", "County": "Huron", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48468", "FIPS": "26063", "County": "Huron", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48470", "FIPS": "26063", "County": "Huron", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48475", "FIPS": "26063", "County": "Huron", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48720", "FIPS": "26063", "County": "Huron", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48725", "FIPS": "26063", "County": "Huron", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48731", "FIPS": "26063", "County": "Huron", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48754", "FIPS": "26063", "County": "Huron", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48755", "FIPS": "26063", "County": "Huron", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48759", "FIPS": "26063", "County": "Huron", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48805", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48819", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48823", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48824", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48825", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48826", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48840", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48842", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48854", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48864", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48892", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48895", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48901", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48906", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48909", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48910", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48911", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48912", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48913", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48915", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48916", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48918", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48919", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48921", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48922", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48924", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48929", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48930", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48933", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48937", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48950", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48951", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48956", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48980", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49251", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49264", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49285", "FIPS": "26065", "County": "Ingham", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "48809", "FIPS": "26067", "County": "Ionia", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48815", "FIPS": "26067", "County": "Ionia", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48845", "FIPS": "26067", "County": "Ionia", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48846", "FIPS": "26067", "County": "Ionia", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48849", "FIPS": "26067", "County": "Ionia", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48851", "FIPS": "26067", "County": "Ionia", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48860", "FIPS": "26067", "County": "Ionia", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48865", "FIPS": "26067", "County": "Ionia", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48870", "FIPS": "26067", "County": "Ionia", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48873", "FIPS": "26067", "County": "Ionia", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48875", "FIPS": "26067", "County": "Ionia", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48881", "FIPS": "26067", "County": "Ionia", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48887", "FIPS": "26067", "County": "Ionia", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48730", "FIPS": "26069", "County": "Iosco", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48739", "FIPS": "26069", "County": "Iosco", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48743", "FIPS": "26069", "County": "Iosco", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48748", "FIPS": "26069", "County": "Iosco", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48750", "FIPS": "26069", "County": "Iosco", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48763", "FIPS": "26069", "County": "Iosco", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48764", "FIPS": "26069", "County": "Iosco", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48770", "FIPS": "26069", "County": "Iosco", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "49902", "FIPS": "26071", "County": "Iron", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49903", "FIPS": "26071", "County": "Iron", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49915", "FIPS": "26071", "County": "Iron", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49920", "FIPS": "26071", "County": "Iron", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49927", "FIPS": "26071", "County": "Iron", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49935", "FIPS": "26071", "County": "Iron", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49964", "FIPS": "26071", "County": "Iron", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "48804", "FIPS": "26073", "County": "Isabella", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48858", "FIPS": "26073", "County": "Isabella", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48859", "FIPS": "26073", "County": "Isabella", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48878", "FIPS": "26073", "County": "Isabella", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48883", "FIPS": "26073", "County": "Isabella", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48893", "FIPS": "26073", "County": "Isabella", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48896", "FIPS": "26073", "County": "Isabella", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "49310", "FIPS": "26073", "County": "Isabella", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "49201", "FIPS": "26075", "County": "Jackson", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49202", "FIPS": "26075", "County": "Jackson", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49203", "FIPS": "26075", "County": "Jackson", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49204", "FIPS": "26075", "County": "Jackson", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49230", "FIPS": "26075", "County": "Jackson", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49234", "FIPS": "26075", "County": "Jackson", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49237", "FIPS": "26075", "County": "Jackson", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49240", "FIPS": "26075", "County": "Jackson", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49241", "FIPS": "26075", "County": "Jackson", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49246", "FIPS": "26075", "County": "Jackson", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49254", "FIPS": "26075", "County": "Jackson", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49259", "FIPS": "26075", "County": "Jackson", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49261", "FIPS": "26075", "County": "Jackson", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49263", "FIPS": "26075", "County": "Jackson", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49269", "FIPS": "26075", "County": "Jackson", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49272", "FIPS": "26075", "County": "Jackson", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49277", "FIPS": "26075", "County": "Jackson", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49283", "FIPS": "26075", "County": "Jackson", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49284", "FIPS": "26075", "County": "Jackson", "State": "MI", "DMA": {"Code": 551, "Name": "LANSING"}},
  {"Postal_Code": "49001", "FIPS": "26077", "County": "Kalamazoo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49002", "FIPS": "26077", "County": "Kalamazoo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49003", "FIPS": "26077", "County": "Kalamazoo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49004", "FIPS": "26077", "County": "Kalamazoo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49005", "FIPS": "26077", "County": "Kalamazoo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49006", "FIPS": "26077", "County": "Kalamazoo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49007", "FIPS": "26077", "County": "Kalamazoo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49008", "FIPS": "26077", "County": "Kalamazoo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49009", "FIPS": "26077", "County": "Kalamazoo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49012", "FIPS": "26077", "County": "Kalamazoo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49019", "FIPS": "26077", "County": "Kalamazoo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49024", "FIPS": "26077", "County": "Kalamazoo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49034", "FIPS": "26077", "County": "Kalamazoo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49041", "FIPS": "26077", "County": "Kalamazoo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49048", "FIPS": "26077", "County": "Kalamazoo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49052", "FIPS": "26077", "County": "Kalamazoo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49053", "FIPS": "26077", "County": "Kalamazoo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49074", "FIPS": "26077", "County": "Kalamazoo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49077", "FIPS": "26077", "County": "Kalamazoo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49081", "FIPS": "26077", "County": "Kalamazoo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49083", "FIPS": "26077", "County": "Kalamazoo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49087", "FIPS": "26077", "County": "Kalamazoo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49088", "FIPS": "26077", "County": "Kalamazoo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49097", "FIPS": "26077", "County": "Kalamazoo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49633", "FIPS": "26079", "County": "Kalkaska", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49646", "FIPS": "26079", "County": "Kalkaska", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49676", "FIPS": "26079", "County": "Kalkaska", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49680", "FIPS": "26079", "County": "Kalkaska", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49301", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49302", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49306", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49315", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49316", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49317", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49319", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49321", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49326", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49330", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49331", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49341", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49343", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49345", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49351", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49355", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49356", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49357", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49418", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49468", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49501", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49502", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49503", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49504", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49505", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49506", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49507", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49508", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49509", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49510", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49512", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49514", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49515", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49516", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49518", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49519", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49523", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49525", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49528", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49530", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49534", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49544", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49546", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49548", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49550", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49555", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49560", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49588", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49599", "FIPS": "26081", "County": "Kent", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49805", "FIPS": "26083", "County": "Keweenaw", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49901", "FIPS": "26083", "County": "Keweenaw", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49918", "FIPS": "26083", "County": "Keweenaw", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49950", "FIPS": "26083", "County": "Keweenaw", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49304", "FIPS": "26085", "County": "Lake", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49623", "FIPS": "26085", "County": "Lake", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49642", "FIPS": "26085", "County": "Lake", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49644", "FIPS": "26085", "County": "Lake", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49656", "FIPS": "26085", "County": "Lake", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "48003", "FIPS": "26087", "County": "Lapeer", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48412", "FIPS": "26087", "County": "Lapeer", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48421", "FIPS": "26087", "County": "Lapeer", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48428", "FIPS": "26087", "County": "Lapeer", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48440", "FIPS": "26087", "County": "Lapeer", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48444", "FIPS": "26087", "County": "Lapeer", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48446", "FIPS": "26087", "County": "Lapeer", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48455", "FIPS": "26087", "County": "Lapeer", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48461", "FIPS": "26087", "County": "Lapeer", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48464", "FIPS": "26087", "County": "Lapeer", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48727", "FIPS": "26087", "County": "Lapeer", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "49621", "FIPS": "26089", "County": "Leelanau", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49630", "FIPS": "26089", "County": "Leelanau", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49636", "FIPS": "26089", "County": "Leelanau", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49653", "FIPS": "26089", "County": "Leelanau", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49654", "FIPS": "26089", "County": "Leelanau", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49664", "FIPS": "26089", "County": "Leelanau", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49670", "FIPS": "26089", "County": "Leelanau", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49674", "FIPS": "26089", "County": "Leelanau", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49682", "FIPS": "26089", "County": "Leelanau", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49220", "FIPS": "26091", "County": "Lenawee", "State": "MI", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "49221", "FIPS": "26091", "County": "Lenawee", "State": "MI", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "49228", "FIPS": "26091", "County": "Lenawee", "State": "MI", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "49229", "FIPS": "26091", "County": "Lenawee", "State": "MI", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "49233", "FIPS": "26091", "County": "Lenawee", "State": "MI", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "49235", "FIPS": "26091", "County": "Lenawee", "State": "MI", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "49236", "FIPS": "26091", "County": "Lenawee", "State": "MI", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "49238", "FIPS": "26091", "County": "Lenawee", "State": "MI", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "49247", "FIPS": "26091", "County": "Lenawee", "State": "MI", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "49248", "FIPS": "26091", "County": "Lenawee", "State": "MI", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "49253", "FIPS": "26091", "County": "Lenawee", "State": "MI", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "49256", "FIPS": "26091", "County": "Lenawee", "State": "MI", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "49265", "FIPS": "26091", "County": "Lenawee", "State": "MI", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "49268", "FIPS": "26091", "County": "Lenawee", "State": "MI", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "49276", "FIPS": "26091", "County": "Lenawee", "State": "MI", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "49279", "FIPS": "26091", "County": "Lenawee", "State": "MI", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "49286", "FIPS": "26091", "County": "Lenawee", "State": "MI", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "49287", "FIPS": "26091", "County": "Lenawee", "State": "MI", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "49289", "FIPS": "26091", "County": "Lenawee", "State": "MI", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "48114", "FIPS": "26093", "County": "Livingston", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48116", "FIPS": "26093", "County": "Livingston", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48137", "FIPS": "26093", "County": "Livingston", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48139", "FIPS": "26093", "County": "Livingston", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48143", "FIPS": "26093", "County": "Livingston", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48169", "FIPS": "26093", "County": "Livingston", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48353", "FIPS": "26093", "County": "Livingston", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48816", "FIPS": "26093", "County": "Livingston", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48836", "FIPS": "26093", "County": "Livingston", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48843", "FIPS": "26093", "County": "Livingston", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48844", "FIPS": "26093", "County": "Livingston", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48855", "FIPS": "26093", "County": "Livingston", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48863", "FIPS": "26093", "County": "Livingston", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "49853", "FIPS": "26095", "County": "Luce", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49868", "FIPS": "26095", "County": "Luce", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49719", "FIPS": "26097", "County": "Mackinac", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49745", "FIPS": "26097", "County": "Mackinac", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49757", "FIPS": "26097", "County": "Mackinac", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49760", "FIPS": "26097", "County": "Mackinac", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49762", "FIPS": "26097", "County": "Mackinac", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49775", "FIPS": "26097", "County": "Mackinac", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49781", "FIPS": "26097", "County": "Mackinac", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49820", "FIPS": "26097", "County": "Mackinac", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49827", "FIPS": "26097", "County": "Mackinac", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49838", "FIPS": "26097", "County": "Mackinac", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "48005", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48015", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48021", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48026", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48035", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48036", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48038", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48042", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48043", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48044", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48045", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48046", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48047", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48048", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48050", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48051", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48062", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48065", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48066", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48080", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48081", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48082", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48088", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48089", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48090", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48091", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48092", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48093", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48094", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48095", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48096", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48310", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48311", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48312", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48313", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48314", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48315", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48316", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48317", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48318", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48397", "FIPS": "26099", "County": "Macomb", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "49613", "FIPS": "26101", "County": "Manistee", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49614", "FIPS": "26101", "County": "Manistee", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49619", "FIPS": "26101", "County": "Manistee", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49625", "FIPS": "26101", "County": "Manistee", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49626", "FIPS": "26101", "County": "Manistee", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49634", "FIPS": "26101", "County": "Manistee", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49645", "FIPS": "26101", "County": "Manistee", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49660", "FIPS": "26101", "County": "Manistee", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49675", "FIPS": "26101", "County": "Manistee", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49689", "FIPS": "26101", "County": "Manistee", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49808", "FIPS": "26103", "County": "Marquette", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49814", "FIPS": "26103", "County": "Marquette", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49819", "FIPS": "26103", "County": "Marquette", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49833", "FIPS": "26103", "County": "Marquette", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49841", "FIPS": "26103", "County": "Marquette", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49849", "FIPS": "26103", "County": "Marquette", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49855", "FIPS": "26103", "County": "Marquette", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49861", "FIPS": "26103", "County": "Marquette", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49865", "FIPS": "26103", "County": "Marquette", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49866", "FIPS": "26103", "County": "Marquette", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49871", "FIPS": "26103", "County": "Marquette", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49879", "FIPS": "26103", "County": "Marquette", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49885", "FIPS": "26103", "County": "Marquette", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49402", "FIPS": "26105", "County": "Mason", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49405", "FIPS": "26105", "County": "Mason", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49410", "FIPS": "26105", "County": "Mason", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49411", "FIPS": "26105", "County": "Mason", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49431", "FIPS": "26105", "County": "Mason", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49454", "FIPS": "26105", "County": "Mason", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49458", "FIPS": "26105", "County": "Mason", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49305", "FIPS": "26107", "County": "Mecosta", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49307", "FIPS": "26107", "County": "Mecosta", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49320", "FIPS": "26107", "County": "Mecosta", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49332", "FIPS": "26107", "County": "Mecosta", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49336", "FIPS": "26107", "County": "Mecosta", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49338", "FIPS": "26107", "County": "Mecosta", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49340", "FIPS": "26107", "County": "Mecosta", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49342", "FIPS": "26107", "County": "Mecosta", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49346", "FIPS": "26107", "County": "Mecosta", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49812", "FIPS": "26109", "County": "Menominee", "State": "MI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "49821", "FIPS": "26109", "County": "Menominee", "State": "MI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "49845", "FIPS": "26109", "County": "Menominee", "State": "MI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "49847", "FIPS": "26109", "County": "Menominee", "State": "MI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "49848", "FIPS": "26109", "County": "Menominee", "State": "MI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "49858", "FIPS": "26109", "County": "Menominee", "State": "MI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "49863", "FIPS": "26109", "County": "Menominee", "State": "MI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "49873", "FIPS": "26109", "County": "Menominee", "State": "MI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "49874", "FIPS": "26109", "County": "Menominee", "State": "MI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "49886", "FIPS": "26109", "County": "Menominee", "State": "MI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "49887", "FIPS": "26109", "County": "Menominee", "State": "MI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "49893", "FIPS": "26109", "County": "Menominee", "State": "MI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "49896", "FIPS": "26109", "County": "Menominee", "State": "MI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "48618", "FIPS": "26111", "County": "Midland", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48620", "FIPS": "26111", "County": "Midland", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48628", "FIPS": "26111", "County": "Midland", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48640", "FIPS": "26111", "County": "Midland", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48641", "FIPS": "26111", "County": "Midland", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48642", "FIPS": "26111", "County": "Midland", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48657", "FIPS": "26111", "County": "Midland", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48667", "FIPS": "26111", "County": "Midland", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48670", "FIPS": "26111", "County": "Midland", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48674", "FIPS": "26111", "County": "Midland", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48686", "FIPS": "26111", "County": "Midland", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "49632", "FIPS": "26113", "County": "Missaukee", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49651", "FIPS": "26113", "County": "Missaukee", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49657", "FIPS": "26113", "County": "Missaukee", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49667", "FIPS": "26113", "County": "Missaukee", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "48110", "FIPS": "26115", "County": "Monroe", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48117", "FIPS": "26115", "County": "Monroe", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48131", "FIPS": "26115", "County": "Monroe", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48133", "FIPS": "26115", "County": "Monroe", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48140", "FIPS": "26115", "County": "Monroe", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48144", "FIPS": "26115", "County": "Monroe", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48145", "FIPS": "26115", "County": "Monroe", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48157", "FIPS": "26115", "County": "Monroe", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48159", "FIPS": "26115", "County": "Monroe", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48160", "FIPS": "26115", "County": "Monroe", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48161", "FIPS": "26115", "County": "Monroe", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48162", "FIPS": "26115", "County": "Monroe", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48166", "FIPS": "26115", "County": "Monroe", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48177", "FIPS": "26115", "County": "Monroe", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48179", "FIPS": "26115", "County": "Monroe", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48182", "FIPS": "26115", "County": "Monroe", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "49267", "FIPS": "26115", "County": "Monroe", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "49270", "FIPS": "26115", "County": "Monroe", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48811", "FIPS": "26117", "County": "Montcalm", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48812", "FIPS": "26117", "County": "Montcalm", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48818", "FIPS": "26117", "County": "Montcalm", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48829", "FIPS": "26117", "County": "Montcalm", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48834", "FIPS": "26117", "County": "Montcalm", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48838", "FIPS": "26117", "County": "Montcalm", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48850", "FIPS": "26117", "County": "Montcalm", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48852", "FIPS": "26117", "County": "Montcalm", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48884", "FIPS": "26117", "County": "Montcalm", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48885", "FIPS": "26117", "County": "Montcalm", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48886", "FIPS": "26117", "County": "Montcalm", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48888", "FIPS": "26117", "County": "Montcalm", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48891", "FIPS": "26117", "County": "Montcalm", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49322", "FIPS": "26117", "County": "Montcalm", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49329", "FIPS": "26117", "County": "Montcalm", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49339", "FIPS": "26117", "County": "Montcalm", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49347", "FIPS": "26117", "County": "Montcalm", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49709", "FIPS": "26119", "County": "Montmorency", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49746", "FIPS": "26119", "County": "Montmorency", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49756", "FIPS": "26119", "County": "Montmorency", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49303", "FIPS": "26121", "County": "Muskegon", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49318", "FIPS": "26121", "County": "Muskegon", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49415", "FIPS": "26121", "County": "Muskegon", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49425", "FIPS": "26121", "County": "Muskegon", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49437", "FIPS": "26121", "County": "Muskegon", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49440", "FIPS": "26121", "County": "Muskegon", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49441", "FIPS": "26121", "County": "Muskegon", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49442", "FIPS": "26121", "County": "Muskegon", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49443", "FIPS": "26121", "County": "Muskegon", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49444", "FIPS": "26121", "County": "Muskegon", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49445", "FIPS": "26121", "County": "Muskegon", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49451", "FIPS": "26121", "County": "Muskegon", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49457", "FIPS": "26121", "County": "Muskegon", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49461", "FIPS": "26121", "County": "Muskegon", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49463", "FIPS": "26121", "County": "Muskegon", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49309", "FIPS": "26123", "County": "Newaygo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49312", "FIPS": "26123", "County": "Newaygo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49327", "FIPS": "26123", "County": "Newaygo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49337", "FIPS": "26123", "County": "Newaygo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49349", "FIPS": "26123", "County": "Newaygo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49412", "FIPS": "26123", "County": "Newaygo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49413", "FIPS": "26123", "County": "Newaygo", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48007", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48009", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48012", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48017", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48025", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48030", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48033", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48034", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48037", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48067", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48068", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48069", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48070", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48071", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48072", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48073", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48075", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48076", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48083", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48084", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48085", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48086", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48098", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48099", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48165", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48178", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48220", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48237", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48301", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48302", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48303", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48304", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48306", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48307", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48308", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48309", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48320", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48321", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48322", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48323", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48324", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48325", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48326", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48327", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48328", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48329", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48330", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48331", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48332", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48333", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48334", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48335", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48336", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48340", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48341", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48342", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48343", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48346", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48347", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48348", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48350", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48356", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48357", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48359", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48360", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48361", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48362", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48363", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48366", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48367", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48370", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48371", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48374", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48375", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48376", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48377", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48380", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48381", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48382", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48383", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48386", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48387", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48390", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48391", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48393", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48442", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48462", "FIPS": "26125", "County": "Oakland", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "49420", "FIPS": "26127", "County": "Oceana", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49421", "FIPS": "26127", "County": "Oceana", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49436", "FIPS": "26127", "County": "Oceana", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49446", "FIPS": "26127", "County": "Oceana", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49449", "FIPS": "26127", "County": "Oceana", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49452", "FIPS": "26127", "County": "Oceana", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49455", "FIPS": "26127", "County": "Oceana", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49459", "FIPS": "26127", "County": "Oceana", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48635", "FIPS": "26129", "County": "Ogemaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48654", "FIPS": "26129", "County": "Ogemaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48661", "FIPS": "26129", "County": "Ogemaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48756", "FIPS": "26129", "County": "Ogemaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48761", "FIPS": "26129", "County": "Ogemaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "49910", "FIPS": "26131", "County": "Ontonagon", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49912", "FIPS": "26131", "County": "Ontonagon", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49925", "FIPS": "26131", "County": "Ontonagon", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49929", "FIPS": "26131", "County": "Ontonagon", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49948", "FIPS": "26131", "County": "Ontonagon", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49953", "FIPS": "26131", "County": "Ontonagon", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49960", "FIPS": "26131", "County": "Ontonagon", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49967", "FIPS": "26131", "County": "Ontonagon", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49971", "FIPS": "26131", "County": "Ontonagon", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49631", "FIPS": "26133", "County": "Osceola", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49639", "FIPS": "26133", "County": "Osceola", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49655", "FIPS": "26133", "County": "Osceola", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49665", "FIPS": "26133", "County": "Osceola", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49677", "FIPS": "26133", "County": "Osceola", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49679", "FIPS": "26133", "County": "Osceola", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49688", "FIPS": "26133", "County": "Osceola", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "48619", "FIPS": "26135", "County": "Oscoda", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "48621", "FIPS": "26135", "County": "Oscoda", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "48636", "FIPS": "26135", "County": "Oscoda", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "48647", "FIPS": "26135", "County": "Oscoda", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49730", "FIPS": "26137", "County": "Otsego", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49734", "FIPS": "26137", "County": "Otsego", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49735", "FIPS": "26137", "County": "Otsego", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49751", "FIPS": "26137", "County": "Otsego", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49795", "FIPS": "26137", "County": "Otsego", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49797", "FIPS": "26137", "County": "Otsego", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49401", "FIPS": "26139", "County": "Ottawa", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49403", "FIPS": "26139", "County": "Ottawa", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49404", "FIPS": "26139", "County": "Ottawa", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49409", "FIPS": "26139", "County": "Ottawa", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49417", "FIPS": "26139", "County": "Ottawa", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49422", "FIPS": "26139", "County": "Ottawa", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49423", "FIPS": "26139", "County": "Ottawa", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49424", "FIPS": "26139", "County": "Ottawa", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49426", "FIPS": "26139", "County": "Ottawa", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49427", "FIPS": "26139", "County": "Ottawa", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49428", "FIPS": "26139", "County": "Ottawa", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49429", "FIPS": "26139", "County": "Ottawa", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49430", "FIPS": "26139", "County": "Ottawa", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49434", "FIPS": "26139", "County": "Ottawa", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49435", "FIPS": "26139", "County": "Ottawa", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49448", "FIPS": "26139", "County": "Ottawa", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49456", "FIPS": "26139", "County": "Ottawa", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49460", "FIPS": "26139", "County": "Ottawa", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49464", "FIPS": "26139", "County": "Ottawa", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49743", "FIPS": "26141", "County": "Presque Isle", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49759", "FIPS": "26141", "County": "Presque Isle", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49765", "FIPS": "26141", "County": "Presque Isle", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49776", "FIPS": "26141", "County": "Presque Isle", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49777", "FIPS": "26141", "County": "Presque Isle", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49779", "FIPS": "26141", "County": "Presque Isle", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "48627", "FIPS": "26143", "County": "Roscommon", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "48629", "FIPS": "26143", "County": "Roscommon", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "48630", "FIPS": "26143", "County": "Roscommon", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "48651", "FIPS": "26143", "County": "Roscommon", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "48653", "FIPS": "26143", "County": "Roscommon", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "48656", "FIPS": "26143", "County": "Roscommon", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "48415", "FIPS": "26145", "County": "Saginaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48417", "FIPS": "26145", "County": "Saginaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48601", "FIPS": "26145", "County": "Saginaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48602", "FIPS": "26145", "County": "Saginaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48603", "FIPS": "26145", "County": "Saginaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48604", "FIPS": "26145", "County": "Saginaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48605", "FIPS": "26145", "County": "Saginaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48606", "FIPS": "26145", "County": "Saginaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48607", "FIPS": "26145", "County": "Saginaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48608", "FIPS": "26145", "County": "Saginaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48609", "FIPS": "26145", "County": "Saginaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48614", "FIPS": "26145", "County": "Saginaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48616", "FIPS": "26145", "County": "Saginaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48623", "FIPS": "26145", "County": "Saginaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48626", "FIPS": "26145", "County": "Saginaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48637", "FIPS": "26145", "County": "Saginaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48638", "FIPS": "26145", "County": "Saginaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48649", "FIPS": "26145", "County": "Saginaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48655", "FIPS": "26145", "County": "Saginaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48663", "FIPS": "26145", "County": "Saginaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48722", "FIPS": "26145", "County": "Saginaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48724", "FIPS": "26145", "County": "Saginaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48734", "FIPS": "26145", "County": "Saginaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48787", "FIPS": "26145", "County": "Saginaw", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48001", "FIPS": "26147", "County": "Saint Clair", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48002", "FIPS": "26147", "County": "Saint Clair", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48004", "FIPS": "26147", "County": "Saint Clair", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48006", "FIPS": "26147", "County": "Saint Clair", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48014", "FIPS": "26147", "County": "Saint Clair", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48022", "FIPS": "26147", "County": "Saint Clair", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48023", "FIPS": "26147", "County": "Saint Clair", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48027", "FIPS": "26147", "County": "Saint Clair", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48028", "FIPS": "26147", "County": "Saint Clair", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48032", "FIPS": "26147", "County": "Saint Clair", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48039", "FIPS": "26147", "County": "Saint Clair", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48040", "FIPS": "26147", "County": "Saint Clair", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48041", "FIPS": "26147", "County": "Saint Clair", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48049", "FIPS": "26147", "County": "Saint Clair", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48054", "FIPS": "26147", "County": "Saint Clair", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48059", "FIPS": "26147", "County": "Saint Clair", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48060", "FIPS": "26147", "County": "Saint Clair", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48061", "FIPS": "26147", "County": "Saint Clair", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48063", "FIPS": "26147", "County": "Saint Clair", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48064", "FIPS": "26147", "County": "Saint Clair", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48074", "FIPS": "26147", "County": "Saint Clair", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48079", "FIPS": "26147", "County": "Saint Clair", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48097", "FIPS": "26147", "County": "Saint Clair", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "49030", "FIPS": "26149", "County": "Saint Joseph", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49032", "FIPS": "26149", "County": "Saint Joseph", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49040", "FIPS": "26149", "County": "Saint Joseph", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49042", "FIPS": "26149", "County": "Saint Joseph", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49066", "FIPS": "26149", "County": "Saint Joseph", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49072", "FIPS": "26149", "County": "Saint Joseph", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49075", "FIPS": "26149", "County": "Saint Joseph", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49091", "FIPS": "26149", "County": "Saint Joseph", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49093", "FIPS": "26149", "County": "Saint Joseph", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49099", "FIPS": "26149", "County": "Saint Joseph", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48401", "FIPS": "26151", "County": "Sanilac", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48410", "FIPS": "26151", "County": "Sanilac", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48416", "FIPS": "26151", "County": "Sanilac", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48419", "FIPS": "26151", "County": "Sanilac", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48422", "FIPS": "26151", "County": "Sanilac", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48426", "FIPS": "26151", "County": "Sanilac", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48427", "FIPS": "26151", "County": "Sanilac", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48434", "FIPS": "26151", "County": "Sanilac", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48450", "FIPS": "26151", "County": "Sanilac", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48453", "FIPS": "26151", "County": "Sanilac", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48454", "FIPS": "26151", "County": "Sanilac", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48456", "FIPS": "26151", "County": "Sanilac", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48465", "FIPS": "26151", "County": "Sanilac", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48466", "FIPS": "26151", "County": "Sanilac", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48469", "FIPS": "26151", "County": "Sanilac", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48471", "FIPS": "26151", "County": "Sanilac", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48472", "FIPS": "26151", "County": "Sanilac", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "49817", "FIPS": "26153", "County": "Schoolcraft", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49836", "FIPS": "26153", "County": "Schoolcraft", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49840", "FIPS": "26153", "County": "Schoolcraft", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49854", "FIPS": "26153", "County": "Schoolcraft", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "49883", "FIPS": "26153", "County": "Schoolcraft", "State": "MI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "48414", "FIPS": "26155", "County": "Shiawassee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48418", "FIPS": "26155", "County": "Shiawassee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48429", "FIPS": "26155", "County": "Shiawassee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48460", "FIPS": "26155", "County": "Shiawassee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48476", "FIPS": "26155", "County": "Shiawassee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48817", "FIPS": "26155", "County": "Shiawassee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48841", "FIPS": "26155", "County": "Shiawassee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48848", "FIPS": "26155", "County": "Shiawassee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48857", "FIPS": "26155", "County": "Shiawassee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48867", "FIPS": "26155", "County": "Shiawassee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48872", "FIPS": "26155", "County": "Shiawassee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48882", "FIPS": "26155", "County": "Shiawassee", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48435", "FIPS": "26157", "County": "Tuscola", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48701", "FIPS": "26157", "County": "Tuscola", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48723", "FIPS": "26157", "County": "Tuscola", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48726", "FIPS": "26157", "County": "Tuscola", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48729", "FIPS": "26157", "County": "Tuscola", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48733", "FIPS": "26157", "County": "Tuscola", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48735", "FIPS": "26157", "County": "Tuscola", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48736", "FIPS": "26157", "County": "Tuscola", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48741", "FIPS": "26157", "County": "Tuscola", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48744", "FIPS": "26157", "County": "Tuscola", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48746", "FIPS": "26157", "County": "Tuscola", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48757", "FIPS": "26157", "County": "Tuscola", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48758", "FIPS": "26157", "County": "Tuscola", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48760", "FIPS": "26157", "County": "Tuscola", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48767", "FIPS": "26157", "County": "Tuscola", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48768", "FIPS": "26157", "County": "Tuscola", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "48769", "FIPS": "26157", "County": "Tuscola", "State": "MI", "DMA": {"Code": 513, "Name": "FLINT-SAGINAW-BAY CITY"}},
  {"Postal_Code": "49013", "FIPS": "26159", "County": "Van Buren", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49026", "FIPS": "26159", "County": "Van Buren", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49027", "FIPS": "26159", "County": "Van Buren", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49043", "FIPS": "26159", "County": "Van Buren", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49045", "FIPS": "26159", "County": "Van Buren", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49055", "FIPS": "26159", "County": "Van Buren", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49056", "FIPS": "26159", "County": "Van Buren", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49057", "FIPS": "26159", "County": "Van Buren", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49062", "FIPS": "26159", "County": "Van Buren", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49063", "FIPS": "26159", "County": "Van Buren", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49064", "FIPS": "26159", "County": "Van Buren", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49065", "FIPS": "26159", "County": "Van Buren", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49071", "FIPS": "26159", "County": "Van Buren", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49079", "FIPS": "26159", "County": "Van Buren", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "49090", "FIPS": "26159", "County": "Van Buren", "State": "MI", "DMA": {"Code": 563, "Name": "GRAND RAPIDS-KALAMAZOO-BATTLE CREEK"}},
  {"Postal_Code": "48103", "FIPS": "26161", "County": "Washtenaw", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48104", "FIPS": "26161", "County": "Washtenaw", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48105", "FIPS": "26161", "County": "Washtenaw", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48106", "FIPS": "26161", "County": "Washtenaw", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48107", "FIPS": "26161", "County": "Washtenaw", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48108", "FIPS": "26161", "County": "Washtenaw", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48109", "FIPS": "26161", "County": "Washtenaw", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48113", "FIPS": "26161", "County": "Washtenaw", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48115", "FIPS": "26161", "County": "Washtenaw", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48118", "FIPS": "26161", "County": "Washtenaw", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48130", "FIPS": "26161", "County": "Washtenaw", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48158", "FIPS": "26161", "County": "Washtenaw", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48175", "FIPS": "26161", "County": "Washtenaw", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48176", "FIPS": "26161", "County": "Washtenaw", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48189", "FIPS": "26161", "County": "Washtenaw", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48190", "FIPS": "26161", "County": "Washtenaw", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48191", "FIPS": "26161", "County": "Washtenaw", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48197", "FIPS": "26161", "County": "Washtenaw", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48198", "FIPS": "26161", "County": "Washtenaw", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48101", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48111", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48112", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48120", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48121", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48122", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48123", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48124", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48125", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48126", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48127", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48128", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48134", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48135", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48136", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48138", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48141", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48146", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48150", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48151", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48152", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48153", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48154", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48164", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48167", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48168", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48170", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48173", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48174", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48180", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48183", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48184", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48185", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48186", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48187", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48188", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48192", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48193", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48195", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48201", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48202", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48203", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48204", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48205", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48206", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48207", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48208", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48209", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48210", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48211", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48212", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48213", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48214", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48215", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48216", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48217", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48218", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48219", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48221", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48222", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48223", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48224", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48225", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48226", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48227", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48228", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48229", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48230", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48231", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48232", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48233", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48234", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48235", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48236", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48238", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48239", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48240", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48242", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48243", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48244", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48255", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48260", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48264", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48265", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48266", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48267", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48268", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48269", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48272", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48275", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48277", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48278", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48279", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "48288", "FIPS": "26163", "County": "Wayne", "State": "MI", "DMA": {"Code": 505, "Name": "DETROIT"}},
  {"Postal_Code": "49601", "FIPS": "26165", "County": "Wexford", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49618", "FIPS": "26165", "County": "Wexford", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49620", "FIPS": "26165", "County": "Wexford", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49638", "FIPS": "26165", "County": "Wexford", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49663", "FIPS": "26165", "County": "Wexford", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "49668", "FIPS": "26165", "County": "Wexford", "State": "MI", "DMA": {"Code": 540, "Name": "TRAVERSE CITY-CADILLAC"}},
  {"Postal_Code": "55748", "FIPS": "27001", "County": "Aitkin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55760", "FIPS": "27001", "County": "Aitkin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55787", "FIPS": "27001", "County": "Aitkin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56350", "FIPS": "27001", "County": "Aitkin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56431", "FIPS": "27001", "County": "Aitkin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56469", "FIPS": "27001", "County": "Aitkin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55005", "FIPS": "27003", "County": "Anoka", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55011", "FIPS": "27003", "County": "Anoka", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55014", "FIPS": "27003", "County": "Anoka", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55070", "FIPS": "27003", "County": "Anoka", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55303", "FIPS": "27003", "County": "Anoka", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55304", "FIPS": "27003", "County": "Anoka", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55421", "FIPS": "27003", "County": "Anoka", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55432", "FIPS": "27003", "County": "Anoka", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55433", "FIPS": "27003", "County": "Anoka", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55434", "FIPS": "27003", "County": "Anoka", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55448", "FIPS": "27003", "County": "Anoka", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55449", "FIPS": "27003", "County": "Anoka", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56501", "FIPS": "27005", "County": "Becker", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56502", "FIPS": "27005", "County": "Becker", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56511", "FIPS": "27005", "County": "Becker", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56521", "FIPS": "27005", "County": "Becker", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56544", "FIPS": "27005", "County": "Becker", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56554", "FIPS": "27005", "County": "Becker", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56569", "FIPS": "27005", "County": "Becker", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56570", "FIPS": "27005", "County": "Becker", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56575", "FIPS": "27005", "County": "Becker", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56577", "FIPS": "27005", "County": "Becker", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56578", "FIPS": "27005", "County": "Becker", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56589", "FIPS": "27005", "County": "Becker", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56591", "FIPS": "27005", "County": "Becker", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56593", "FIPS": "27005", "County": "Becker", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56601", "FIPS": "27007", "County": "Beltrami", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56619", "FIPS": "27007", "County": "Beltrami", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56630", "FIPS": "27007", "County": "Beltrami", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56647", "FIPS": "27007", "County": "Beltrami", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56650", "FIPS": "27007", "County": "Beltrami", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56663", "FIPS": "27007", "County": "Beltrami", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56666", "FIPS": "27007", "County": "Beltrami", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56667", "FIPS": "27007", "County": "Beltrami", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56670", "FIPS": "27007", "County": "Beltrami", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56671", "FIPS": "27007", "County": "Beltrami", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56683", "FIPS": "27007", "County": "Beltrami", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56685", "FIPS": "27007", "County": "Beltrami", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56687", "FIPS": "27007", "County": "Beltrami", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56304", "FIPS": "27009", "County": "Benton", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56329", "FIPS": "27009", "County": "Benton", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56333", "FIPS": "27009", "County": "Benton", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56357", "FIPS": "27009", "County": "Benton", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56367", "FIPS": "27009", "County": "Benton", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56379", "FIPS": "27009", "County": "Benton", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56210", "FIPS": "27011", "County": "Big Stone", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56211", "FIPS": "27011", "County": "Big Stone", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56225", "FIPS": "27011", "County": "Big Stone", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56227", "FIPS": "27011", "County": "Big Stone", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56240", "FIPS": "27011", "County": "Big Stone", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56276", "FIPS": "27011", "County": "Big Stone", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56278", "FIPS": "27011", "County": "Big Stone", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56001", "FIPS": "27013", "County": "Blue Earth", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56002", "FIPS": "27013", "County": "Blue Earth", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56006", "FIPS": "27013", "County": "Blue Earth", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56010", "FIPS": "27013", "County": "Blue Earth", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56024", "FIPS": "27013", "County": "Blue Earth", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56034", "FIPS": "27013", "County": "Blue Earth", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56037", "FIPS": "27013", "County": "Blue Earth", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56055", "FIPS": "27013", "County": "Blue Earth", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56063", "FIPS": "27013", "County": "Blue Earth", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56065", "FIPS": "27013", "County": "Blue Earth", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56078", "FIPS": "27013", "County": "Blue Earth", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56080", "FIPS": "27013", "County": "Blue Earth", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56090", "FIPS": "27013", "County": "Blue Earth", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56019", "FIPS": "27015", "County": "Brown", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56030", "FIPS": "27015", "County": "Brown", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56041", "FIPS": "27015", "County": "Brown", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56073", "FIPS": "27015", "County": "Brown", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56084", "FIPS": "27015", "County": "Brown", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56085", "FIPS": "27015", "County": "Brown", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56087", "FIPS": "27015", "County": "Brown", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "55707", "FIPS": "27017", "County": "Carlton", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55718", "FIPS": "27017", "County": "Carlton", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55720", "FIPS": "27017", "County": "Carlton", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55726", "FIPS": "27017", "County": "Carlton", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55733", "FIPS": "27017", "County": "Carlton", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55749", "FIPS": "27017", "County": "Carlton", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55757", "FIPS": "27017", "County": "Carlton", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55767", "FIPS": "27017", "County": "Carlton", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55780", "FIPS": "27017", "County": "Carlton", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55797", "FIPS": "27017", "County": "Carlton", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55798", "FIPS": "27017", "County": "Carlton", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55315", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55317", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55318", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55322", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55339", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55360", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55367", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55368", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55383", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55386", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55387", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55388", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55394", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55397", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55399", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55550", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55551", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55552", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55553", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55554", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55555", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55556", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55557", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55558", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55559", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55560", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55561", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55562", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55563", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55564", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55566", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55567", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55568", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55583", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55594", "FIPS": "27019", "County": "Carver", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55785", "FIPS": "27021", "County": "Cass", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56430", "FIPS": "27021", "County": "Cass", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56435", "FIPS": "27021", "County": "Cass", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56452", "FIPS": "27021", "County": "Cass", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56473", "FIPS": "27021", "County": "Cass", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56474", "FIPS": "27021", "County": "Cass", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56484", "FIPS": "27021", "County": "Cass", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56626", "FIPS": "27021", "County": "Cass", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56633", "FIPS": "27021", "County": "Cass", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56641", "FIPS": "27021", "County": "Cass", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56655", "FIPS": "27021", "County": "Cass", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56662", "FIPS": "27021", "County": "Cass", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56672", "FIPS": "27021", "County": "Cass", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56222", "FIPS": "27023", "County": "Chippewa", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56260", "FIPS": "27023", "County": "Chippewa", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56262", "FIPS": "27023", "County": "Chippewa", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56265", "FIPS": "27023", "County": "Chippewa", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56295", "FIPS": "27023", "County": "Chippewa", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55002", "FIPS": "27025", "County": "Chisago", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55012", "FIPS": "27025", "County": "Chisago", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55013", "FIPS": "27025", "County": "Chisago", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55032", "FIPS": "27025", "County": "Chisago", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55045", "FIPS": "27025", "County": "Chisago", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55056", "FIPS": "27025", "County": "Chisago", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55069", "FIPS": "27025", "County": "Chisago", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55074", "FIPS": "27025", "County": "Chisago", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55079", "FIPS": "27025", "County": "Chisago", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55084", "FIPS": "27025", "County": "Chisago", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55092", "FIPS": "27025", "County": "Chisago", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56514", "FIPS": "27027", "County": "Clay", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56525", "FIPS": "27027", "County": "Clay", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56529", "FIPS": "27027", "County": "Clay", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56536", "FIPS": "27027", "County": "Clay", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56546", "FIPS": "27027", "County": "Clay", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56547", "FIPS": "27027", "County": "Clay", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56549", "FIPS": "27027", "County": "Clay", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56552", "FIPS": "27027", "County": "Clay", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56560", "FIPS": "27027", "County": "Clay", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56561", "FIPS": "27027", "County": "Clay", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56562", "FIPS": "27027", "County": "Clay", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56563", "FIPS": "27027", "County": "Clay", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56580", "FIPS": "27027", "County": "Clay", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56585", "FIPS": "27027", "County": "Clay", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56621", "FIPS": "27029", "County": "Clearwater", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56634", "FIPS": "27029", "County": "Clearwater", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56644", "FIPS": "27029", "County": "Clearwater", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56652", "FIPS": "27029", "County": "Clearwater", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56676", "FIPS": "27029", "County": "Clearwater", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "55604", "FIPS": "27031", "County": "Cook", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55605", "FIPS": "27031", "County": "Cook", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55606", "FIPS": "27031", "County": "Cook", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55612", "FIPS": "27031", "County": "Cook", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55613", "FIPS": "27031", "County": "Cook", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55615", "FIPS": "27031", "County": "Cook", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "56101", "FIPS": "27033", "County": "Cottonwood", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56118", "FIPS": "27033", "County": "Cottonwood", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56145", "FIPS": "27033", "County": "Cottonwood", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56159", "FIPS": "27033", "County": "Cottonwood", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56174", "FIPS": "27033", "County": "Cottonwood", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56183", "FIPS": "27033", "County": "Cottonwood", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56401", "FIPS": "27035", "County": "Crow Wing", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56425", "FIPS": "27035", "County": "Crow Wing", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56441", "FIPS": "27035", "County": "Crow Wing", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56442", "FIPS": "27035", "County": "Crow Wing", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56444", "FIPS": "27035", "County": "Crow Wing", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56447", "FIPS": "27035", "County": "Crow Wing", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56448", "FIPS": "27035", "County": "Crow Wing", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56449", "FIPS": "27035", "County": "Crow Wing", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56450", "FIPS": "27035", "County": "Crow Wing", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56455", "FIPS": "27035", "County": "Crow Wing", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56456", "FIPS": "27035", "County": "Crow Wing", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56459", "FIPS": "27035", "County": "Crow Wing", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56465", "FIPS": "27035", "County": "Crow Wing", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56468", "FIPS": "27035", "County": "Crow Wing", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56472", "FIPS": "27035", "County": "Crow Wing", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55010", "FIPS": "27037", "County": "Dakota", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55024", "FIPS": "27037", "County": "Dakota", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55031", "FIPS": "27037", "County": "Dakota", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55033", "FIPS": "27037", "County": "Dakota", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55044", "FIPS": "27037", "County": "Dakota", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55065", "FIPS": "27037", "County": "Dakota", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55068", "FIPS": "27037", "County": "Dakota", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55075", "FIPS": "27037", "County": "Dakota", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55076", "FIPS": "27037", "County": "Dakota", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55077", "FIPS": "27037", "County": "Dakota", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55085", "FIPS": "27037", "County": "Dakota", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55118", "FIPS": "27037", "County": "Dakota", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55120", "FIPS": "27037", "County": "Dakota", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55121", "FIPS": "27037", "County": "Dakota", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55122", "FIPS": "27037", "County": "Dakota", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55123", "FIPS": "27037", "County": "Dakota", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55124", "FIPS": "27037", "County": "Dakota", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55150", "FIPS": "27037", "County": "Dakota", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55306", "FIPS": "27037", "County": "Dakota", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55337", "FIPS": "27037", "County": "Dakota", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55924", "FIPS": "27039", "County": "Dodge", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55927", "FIPS": "27039", "County": "Dodge", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55940", "FIPS": "27039", "County": "Dodge", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55944", "FIPS": "27039", "County": "Dodge", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55955", "FIPS": "27039", "County": "Dodge", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55985", "FIPS": "27039", "County": "Dodge", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "56308", "FIPS": "27041", "County": "Douglas", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56315", "FIPS": "27041", "County": "Douglas", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56319", "FIPS": "27041", "County": "Douglas", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56326", "FIPS": "27041", "County": "Douglas", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56327", "FIPS": "27041", "County": "Douglas", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56332", "FIPS": "27041", "County": "Douglas", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56341", "FIPS": "27041", "County": "Douglas", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56343", "FIPS": "27041", "County": "Douglas", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56354", "FIPS": "27041", "County": "Douglas", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56355", "FIPS": "27041", "County": "Douglas", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56360", "FIPS": "27041", "County": "Douglas", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56013", "FIPS": "27043", "County": "Faribault", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56014", "FIPS": "27043", "County": "Faribault", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56023", "FIPS": "27043", "County": "Faribault", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56025", "FIPS": "27043", "County": "Faribault", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56027", "FIPS": "27043", "County": "Faribault", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56033", "FIPS": "27043", "County": "Faribault", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56047", "FIPS": "27043", "County": "Faribault", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56051", "FIPS": "27043", "County": "Faribault", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56068", "FIPS": "27043", "County": "Faribault", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56097", "FIPS": "27043", "County": "Faribault", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56098", "FIPS": "27043", "County": "Faribault", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55922", "FIPS": "27045", "County": "Fillmore", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55923", "FIPS": "27045", "County": "Fillmore", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55935", "FIPS": "27045", "County": "Fillmore", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55939", "FIPS": "27045", "County": "Fillmore", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55949", "FIPS": "27045", "County": "Fillmore", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55954", "FIPS": "27045", "County": "Fillmore", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55961", "FIPS": "27045", "County": "Fillmore", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55962", "FIPS": "27045", "County": "Fillmore", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55965", "FIPS": "27045", "County": "Fillmore", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55971", "FIPS": "27045", "County": "Fillmore", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55975", "FIPS": "27045", "County": "Fillmore", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55990", "FIPS": "27045", "County": "Fillmore", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "56007", "FIPS": "27047", "County": "Freeborn", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "56009", "FIPS": "27047", "County": "Freeborn", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "56016", "FIPS": "27047", "County": "Freeborn", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "56020", "FIPS": "27047", "County": "Freeborn", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "56029", "FIPS": "27047", "County": "Freeborn", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "56032", "FIPS": "27047", "County": "Freeborn", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "56035", "FIPS": "27047", "County": "Freeborn", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "56036", "FIPS": "27047", "County": "Freeborn", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "56042", "FIPS": "27047", "County": "Freeborn", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "56043", "FIPS": "27047", "County": "Freeborn", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "56045", "FIPS": "27047", "County": "Freeborn", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "56089", "FIPS": "27047", "County": "Freeborn", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55009", "FIPS": "27049", "County": "Goodhue", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55018", "FIPS": "27049", "County": "Goodhue", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55026", "FIPS": "27049", "County": "Goodhue", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55027", "FIPS": "27049", "County": "Goodhue", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55066", "FIPS": "27049", "County": "Goodhue", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55089", "FIPS": "27049", "County": "Goodhue", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55946", "FIPS": "27049", "County": "Goodhue", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55963", "FIPS": "27049", "County": "Goodhue", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55983", "FIPS": "27049", "County": "Goodhue", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55992", "FIPS": "27049", "County": "Goodhue", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56248", "FIPS": "27051", "County": "Grant", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56274", "FIPS": "27051", "County": "Grant", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56309", "FIPS": "27051", "County": "Grant", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56311", "FIPS": "27051", "County": "Grant", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56339", "FIPS": "27051", "County": "Grant", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56531", "FIPS": "27051", "County": "Grant", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56590", "FIPS": "27051", "County": "Grant", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55111", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55305", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55311", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55316", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55323", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55327", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55331", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55340", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55343", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55344", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55345", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55346", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55347", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55348", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55356", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55357", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55359", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55361", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55364", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55369", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55374", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55375", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55384", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55391", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55392", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55401", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55402", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55403", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55404", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55405", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55406", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55407", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55408", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55409", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55410", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55411", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55412", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55413", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55414", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55415", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55416", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55417", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55418", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55419", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55420", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55422", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55423", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55424", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55425", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55426", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55427", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55428", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55429", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55430", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55431", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55435", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55436", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55437", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55438", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55439", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55440", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55441", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55442", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55443", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55444", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55445", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55446", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55447", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55450", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55454", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55455", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55458", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55459", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55460", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55467", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55468", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55470", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55472", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55473", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55474", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55478", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55479", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55480", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55483", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55484", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55485", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55486", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55487", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55488", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55569", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55570", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55571", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55572", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55573", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55574", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55576", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55577", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55578", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55579", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55593", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55595", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55596", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55597", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55598", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55599", "FIPS": "27053", "County": "Hennepin", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55919", "FIPS": "27055", "County": "Houston", "State": "MN", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "55921", "FIPS": "27055", "County": "Houston", "State": "MN", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "55931", "FIPS": "27055", "County": "Houston", "State": "MN", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "55941", "FIPS": "27055", "County": "Houston", "State": "MN", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "55943", "FIPS": "27055", "County": "Houston", "State": "MN", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "55947", "FIPS": "27055", "County": "Houston", "State": "MN", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "55974", "FIPS": "27055", "County": "Houston", "State": "MN", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "56433", "FIPS": "27057", "County": "Hubbard", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56436", "FIPS": "27057", "County": "Hubbard", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56458", "FIPS": "27057", "County": "Hubbard", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56461", "FIPS": "27057", "County": "Hubbard", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56467", "FIPS": "27057", "County": "Hubbard", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56470", "FIPS": "27057", "County": "Hubbard", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56678", "FIPS": "27057", "County": "Hubbard", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55006", "FIPS": "27059", "County": "Isanti", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55008", "FIPS": "27059", "County": "Isanti", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55017", "FIPS": "27059", "County": "Isanti", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55029", "FIPS": "27059", "County": "Isanti", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55040", "FIPS": "27059", "County": "Isanti", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55080", "FIPS": "27059", "County": "Isanti", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55709", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55716", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55721", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55722", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55730", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55742", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55744", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55745", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55752", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55753", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55764", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55769", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55775", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55784", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55786", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55793", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "56628", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "56631", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "56636", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "56637", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "56639", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "56657", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "56659", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "56680", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "56681", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "56688", "FIPS": "27061", "County": "Itasca", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "56111", "FIPS": "27063", "County": "Jackson", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56137", "FIPS": "27063", "County": "Jackson", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56143", "FIPS": "27063", "County": "Jackson", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56150", "FIPS": "27063", "County": "Jackson", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56161", "FIPS": "27063", "County": "Jackson", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55051", "FIPS": "27065", "County": "Kanabec", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56358", "FIPS": "27065", "County": "Kanabec", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56201", "FIPS": "27067", "County": "Kandiyohi", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56209", "FIPS": "27067", "County": "Kandiyohi", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56216", "FIPS": "27067", "County": "Kandiyohi", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56251", "FIPS": "27067", "County": "Kandiyohi", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56253", "FIPS": "27067", "County": "Kandiyohi", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56273", "FIPS": "27067", "County": "Kandiyohi", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56279", "FIPS": "27067", "County": "Kandiyohi", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56281", "FIPS": "27067", "County": "Kandiyohi", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56282", "FIPS": "27067", "County": "Kandiyohi", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56288", "FIPS": "27067", "County": "Kandiyohi", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56289", "FIPS": "27067", "County": "Kandiyohi", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56720", "FIPS": "27069", "County": "Kittson", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56728", "FIPS": "27069", "County": "Kittson", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56729", "FIPS": "27069", "County": "Kittson", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56731", "FIPS": "27069", "County": "Kittson", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56732", "FIPS": "27069", "County": "Kittson", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56733", "FIPS": "27069", "County": "Kittson", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56734", "FIPS": "27069", "County": "Kittson", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56735", "FIPS": "27069", "County": "Kittson", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56740", "FIPS": "27069", "County": "Kittson", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56755", "FIPS": "27069", "County": "Kittson", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56627", "FIPS": "27071", "County": "Koochiching", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "56629", "FIPS": "27071", "County": "Koochiching", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "56649", "FIPS": "27071", "County": "Koochiching", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "56653", "FIPS": "27071", "County": "Koochiching", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "56654", "FIPS": "27071", "County": "Koochiching", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "56658", "FIPS": "27071", "County": "Koochiching", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "56660", "FIPS": "27071", "County": "Koochiching", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "56661", "FIPS": "27071", "County": "Koochiching", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "56668", "FIPS": "27071", "County": "Koochiching", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "56669", "FIPS": "27071", "County": "Koochiching", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "56679", "FIPS": "27071", "County": "Koochiching", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "56212", "FIPS": "27073", "County": "Lac Qui Parle", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56218", "FIPS": "27073", "County": "Lac Qui Parle", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56232", "FIPS": "27073", "County": "Lac Qui Parle", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56256", "FIPS": "27073", "County": "Lac Qui Parle", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56257", "FIPS": "27073", "County": "Lac Qui Parle", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55601", "FIPS": "27075", "County": "Lake", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55603", "FIPS": "27075", "County": "Lake", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55607", "FIPS": "27075", "County": "Lake", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55609", "FIPS": "27075", "County": "Lake", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55614", "FIPS": "27075", "County": "Lake", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55616", "FIPS": "27075", "County": "Lake", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "56623", "FIPS": "27077", "County": "Lake of the Woods", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56686", "FIPS": "27077", "County": "Lake of the Woods", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56711", "FIPS": "27077", "County": "Lake of the Woods", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56741", "FIPS": "27077", "County": "Lake of the Woods", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56017", "FIPS": "27079", "County": "Le Sueur", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56028", "FIPS": "27079", "County": "Le Sueur", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56050", "FIPS": "27079", "County": "Le Sueur", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56057", "FIPS": "27079", "County": "Le Sueur", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56058", "FIPS": "27079", "County": "Le Sueur", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56069", "FIPS": "27079", "County": "Le Sueur", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56096", "FIPS": "27079", "County": "Le Sueur", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56113", "FIPS": "27081", "County": "Lincoln", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56136", "FIPS": "27081", "County": "Lincoln", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56142", "FIPS": "27081", "County": "Lincoln", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56149", "FIPS": "27081", "County": "Lincoln", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56178", "FIPS": "27081", "County": "Lincoln", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56115", "FIPS": "27083", "County": "Lyon", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56132", "FIPS": "27083", "County": "Lyon", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56157", "FIPS": "27083", "County": "Lyon", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56169", "FIPS": "27083", "County": "Lyon", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56175", "FIPS": "27083", "County": "Lyon", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56229", "FIPS": "27083", "County": "Lyon", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56239", "FIPS": "27083", "County": "Lyon", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56258", "FIPS": "27083", "County": "Lyon", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56264", "FIPS": "27083", "County": "Lyon", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56291", "FIPS": "27083", "County": "Lyon", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55312", "FIPS": "27085", "County": "McLeod", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55336", "FIPS": "27085", "County": "McLeod", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55350", "FIPS": "27085", "County": "McLeod", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55354", "FIPS": "27085", "County": "McLeod", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55370", "FIPS": "27085", "County": "McLeod", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55381", "FIPS": "27085", "County": "McLeod", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55385", "FIPS": "27085", "County": "McLeod", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55395", "FIPS": "27085", "County": "McLeod", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56516", "FIPS": "27087", "County": "Mahnomen", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56557", "FIPS": "27087", "County": "Mahnomen", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56566", "FIPS": "27087", "County": "Mahnomen", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56651", "FIPS": "27087", "County": "Mahnomen", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56710", "FIPS": "27089", "County": "Marshall", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56713", "FIPS": "27089", "County": "Marshall", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56724", "FIPS": "27089", "County": "Marshall", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56727", "FIPS": "27089", "County": "Marshall", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56737", "FIPS": "27089", "County": "Marshall", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56738", "FIPS": "27089", "County": "Marshall", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56744", "FIPS": "27089", "County": "Marshall", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56757", "FIPS": "27089", "County": "Marshall", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56758", "FIPS": "27089", "County": "Marshall", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56760", "FIPS": "27089", "County": "Marshall", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56762", "FIPS": "27089", "County": "Marshall", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56031", "FIPS": "27091", "County": "Martin", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56039", "FIPS": "27091", "County": "Martin", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56075", "FIPS": "27091", "County": "Martin", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56088", "FIPS": "27091", "County": "Martin", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56121", "FIPS": "27091", "County": "Martin", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56127", "FIPS": "27091", "County": "Martin", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56162", "FIPS": "27091", "County": "Martin", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56171", "FIPS": "27091", "County": "Martin", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56176", "FIPS": "27091", "County": "Martin", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56181", "FIPS": "27091", "County": "Martin", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "55324", "FIPS": "27093", "County": "Meeker", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55325", "FIPS": "27093", "County": "Meeker", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55329", "FIPS": "27093", "County": "Meeker", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55355", "FIPS": "27093", "County": "Meeker", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55389", "FIPS": "27093", "County": "Meeker", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56228", "FIPS": "27093", "County": "Meeker", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56243", "FIPS": "27093", "County": "Meeker", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55371", "FIPS": "27095", "County": "Mille Lacs", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56313", "FIPS": "27095", "County": "Mille Lacs", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56330", "FIPS": "27095", "County": "Mille Lacs", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56342", "FIPS": "27095", "County": "Mille Lacs", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56353", "FIPS": "27095", "County": "Mille Lacs", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56359", "FIPS": "27095", "County": "Mille Lacs", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56363", "FIPS": "27095", "County": "Mille Lacs", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56386", "FIPS": "27095", "County": "Mille Lacs", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56314", "FIPS": "27097", "County": "Morrison", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56317", "FIPS": "27097", "County": "Morrison", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56328", "FIPS": "27097", "County": "Morrison", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56338", "FIPS": "27097", "County": "Morrison", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56344", "FIPS": "27097", "County": "Morrison", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56345", "FIPS": "27097", "County": "Morrison", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56364", "FIPS": "27097", "County": "Morrison", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56373", "FIPS": "27097", "County": "Morrison", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56382", "FIPS": "27097", "County": "Morrison", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56384", "FIPS": "27097", "County": "Morrison", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56443", "FIPS": "27097", "County": "Morrison", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56466", "FIPS": "27097", "County": "Morrison", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56475", "FIPS": "27097", "County": "Morrison", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55909", "FIPS": "27099", "County": "Mower", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55912", "FIPS": "27099", "County": "Mower", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55918", "FIPS": "27099", "County": "Mower", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55926", "FIPS": "27099", "County": "Mower", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55933", "FIPS": "27099", "County": "Mower", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55936", "FIPS": "27099", "County": "Mower", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55950", "FIPS": "27099", "County": "Mower", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55951", "FIPS": "27099", "County": "Mower", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55953", "FIPS": "27099", "County": "Mower", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55967", "FIPS": "27099", "County": "Mower", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55970", "FIPS": "27099", "County": "Mower", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55973", "FIPS": "27099", "County": "Mower", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55977", "FIPS": "27099", "County": "Mower", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55982", "FIPS": "27099", "County": "Mower", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "56114", "FIPS": "27101", "County": "Murray", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56122", "FIPS": "27101", "County": "Murray", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56123", "FIPS": "27101", "County": "Murray", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56125", "FIPS": "27101", "County": "Murray", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56131", "FIPS": "27101", "County": "Murray", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56141", "FIPS": "27101", "County": "Murray", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56151", "FIPS": "27101", "County": "Murray", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56172", "FIPS": "27101", "County": "Murray", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56003", "FIPS": "27103", "County": "Nicollet", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56021", "FIPS": "27103", "County": "Nicollet", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56054", "FIPS": "27103", "County": "Nicollet", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56074", "FIPS": "27103", "County": "Nicollet", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56082", "FIPS": "27103", "County": "Nicollet", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56110", "FIPS": "27105", "County": "Nobles", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56117", "FIPS": "27105", "County": "Nobles", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56119", "FIPS": "27105", "County": "Nobles", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56129", "FIPS": "27105", "County": "Nobles", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56146", "FIPS": "27105", "County": "Nobles", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56153", "FIPS": "27105", "County": "Nobles", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56155", "FIPS": "27105", "County": "Nobles", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56165", "FIPS": "27105", "County": "Nobles", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56167", "FIPS": "27105", "County": "Nobles", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56168", "FIPS": "27105", "County": "Nobles", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56185", "FIPS": "27105", "County": "Nobles", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56187", "FIPS": "27105", "County": "Nobles", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56510", "FIPS": "27107", "County": "Norman", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56519", "FIPS": "27107", "County": "Norman", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56541", "FIPS": "27107", "County": "Norman", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56545", "FIPS": "27107", "County": "Norman", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56548", "FIPS": "27107", "County": "Norman", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56550", "FIPS": "27107", "County": "Norman", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56574", "FIPS": "27107", "County": "Norman", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56581", "FIPS": "27107", "County": "Norman", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56584", "FIPS": "27107", "County": "Norman", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "55901", "FIPS": "27109", "County": "Olmsted", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55902", "FIPS": "27109", "County": "Olmsted", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55903", "FIPS": "27109", "County": "Olmsted", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55904", "FIPS": "27109", "County": "Olmsted", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55905", "FIPS": "27109", "County": "Olmsted", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55906", "FIPS": "27109", "County": "Olmsted", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55920", "FIPS": "27109", "County": "Olmsted", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55929", "FIPS": "27109", "County": "Olmsted", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55934", "FIPS": "27109", "County": "Olmsted", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55960", "FIPS": "27109", "County": "Olmsted", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "55976", "FIPS": "27109", "County": "Olmsted", "State": "MN", "DMA": {"Code": 611, "Name": "ROCHESTER-MASON CITY-AUSTIN"}},
  {"Postal_Code": "56324", "FIPS": "27111", "County": "Otter Tail", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56361", "FIPS": "27111", "County": "Otter Tail", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56515", "FIPS": "27111", "County": "Otter Tail", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56518", "FIPS": "27111", "County": "Otter Tail", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56524", "FIPS": "27111", "County": "Otter Tail", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56527", "FIPS": "27111", "County": "Otter Tail", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56528", "FIPS": "27111", "County": "Otter Tail", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56533", "FIPS": "27111", "County": "Otter Tail", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56534", "FIPS": "27111", "County": "Otter Tail", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56537", "FIPS": "27111", "County": "Otter Tail", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56538", "FIPS": "27111", "County": "Otter Tail", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56551", "FIPS": "27111", "County": "Otter Tail", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56567", "FIPS": "27111", "County": "Otter Tail", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56571", "FIPS": "27111", "County": "Otter Tail", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56572", "FIPS": "27111", "County": "Otter Tail", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56573", "FIPS": "27111", "County": "Otter Tail", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56576", "FIPS": "27111", "County": "Otter Tail", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56586", "FIPS": "27111", "County": "Otter Tail", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56587", "FIPS": "27111", "County": "Otter Tail", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56588", "FIPS": "27111", "County": "Otter Tail", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56701", "FIPS": "27113", "County": "Pennington", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56725", "FIPS": "27113", "County": "Pennington", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56754", "FIPS": "27113", "County": "Pennington", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "55007", "FIPS": "27115", "County": "Pine", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55030", "FIPS": "27115", "County": "Pine", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55036", "FIPS": "27115", "County": "Pine", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55037", "FIPS": "27115", "County": "Pine", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55063", "FIPS": "27115", "County": "Pine", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55067", "FIPS": "27115", "County": "Pine", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55072", "FIPS": "27115", "County": "Pine", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55704", "FIPS": "27115", "County": "Pine", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55712", "FIPS": "27115", "County": "Pine", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55735", "FIPS": "27115", "County": "Pine", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55756", "FIPS": "27115", "County": "Pine", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55783", "FIPS": "27115", "County": "Pine", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55795", "FIPS": "27115", "County": "Pine", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56128", "FIPS": "27117", "County": "Pipestone", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56139", "FIPS": "27117", "County": "Pipestone", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56140", "FIPS": "27117", "County": "Pipestone", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56164", "FIPS": "27117", "County": "Pipestone", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56170", "FIPS": "27117", "County": "Pipestone", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56177", "FIPS": "27117", "County": "Pipestone", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56186", "FIPS": "27117", "County": "Pipestone", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56517", "FIPS": "27119", "County": "Polk", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56523", "FIPS": "27119", "County": "Polk", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56535", "FIPS": "27119", "County": "Polk", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56540", "FIPS": "27119", "County": "Polk", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56542", "FIPS": "27119", "County": "Polk", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56556", "FIPS": "27119", "County": "Polk", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56568", "FIPS": "27119", "County": "Polk", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56592", "FIPS": "27119", "County": "Polk", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56646", "FIPS": "27119", "County": "Polk", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56684", "FIPS": "27119", "County": "Polk", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56716", "FIPS": "27119", "County": "Polk", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56721", "FIPS": "27119", "County": "Polk", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56722", "FIPS": "27119", "County": "Polk", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56723", "FIPS": "27119", "County": "Polk", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56736", "FIPS": "27119", "County": "Polk", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56323", "FIPS": "27121", "County": "Pope", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56334", "FIPS": "27121", "County": "Pope", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56349", "FIPS": "27121", "County": "Pope", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56381", "FIPS": "27121", "County": "Pope", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56385", "FIPS": "27121", "County": "Pope", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55101", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55102", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55103", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55104", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55105", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55106", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55107", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55108", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55109", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55110", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55112", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55113", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55114", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55116", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55117", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55119", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55126", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55127", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55130", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55133", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55144", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55145", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55146", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55155", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55161", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55164", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55165", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55166", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55168", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55169", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55170", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55171", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55172", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55175", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55177", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55187", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55188", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55191", "FIPS": "27123", "County": "Ramsey", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56715", "FIPS": "27125", "County": "Red Lake", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56742", "FIPS": "27125", "County": "Red Lake", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56748", "FIPS": "27125", "County": "Red Lake", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56750", "FIPS": "27125", "County": "Red Lake", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56083", "FIPS": "27127", "County": "Redwood", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56152", "FIPS": "27127", "County": "Redwood", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56166", "FIPS": "27127", "County": "Redwood", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56180", "FIPS": "27127", "County": "Redwood", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56214", "FIPS": "27127", "County": "Redwood", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56224", "FIPS": "27127", "County": "Redwood", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56255", "FIPS": "27127", "County": "Redwood", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56263", "FIPS": "27127", "County": "Redwood", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56266", "FIPS": "27127", "County": "Redwood", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56283", "FIPS": "27127", "County": "Redwood", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56287", "FIPS": "27127", "County": "Redwood", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56292", "FIPS": "27127", "County": "Redwood", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56293", "FIPS": "27127", "County": "Redwood", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56294", "FIPS": "27127", "County": "Redwood", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55310", "FIPS": "27129", "County": "Renville", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55314", "FIPS": "27129", "County": "Renville", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55332", "FIPS": "27129", "County": "Renville", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55333", "FIPS": "27129", "County": "Renville", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55342", "FIPS": "27129", "County": "Renville", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56230", "FIPS": "27129", "County": "Renville", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56270", "FIPS": "27129", "County": "Renville", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56277", "FIPS": "27129", "County": "Renville", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56284", "FIPS": "27129", "County": "Renville", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56285", "FIPS": "27129", "County": "Renville", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55019", "FIPS": "27131", "County": "Rice", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55021", "FIPS": "27131", "County": "Rice", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55046", "FIPS": "27131", "County": "Rice", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55052", "FIPS": "27131", "County": "Rice", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55053", "FIPS": "27131", "County": "Rice", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55057", "FIPS": "27131", "County": "Rice", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55087", "FIPS": "27131", "County": "Rice", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55088", "FIPS": "27131", "County": "Rice", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56052", "FIPS": "27131", "County": "Rice", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56116", "FIPS": "27133", "County": "Rock", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56134", "FIPS": "27133", "County": "Rock", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56138", "FIPS": "27133", "County": "Rock", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56144", "FIPS": "27133", "County": "Rock", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56147", "FIPS": "27133", "County": "Rock", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56156", "FIPS": "27133", "County": "Rock", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56158", "FIPS": "27133", "County": "Rock", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56173", "FIPS": "27133", "County": "Rock", "State": "MN", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "56673", "FIPS": "27135", "County": "Roseau", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56682", "FIPS": "27135", "County": "Roseau", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56714", "FIPS": "27135", "County": "Roseau", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56726", "FIPS": "27135", "County": "Roseau", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56751", "FIPS": "27135", "County": "Roseau", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56756", "FIPS": "27135", "County": "Roseau", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56759", "FIPS": "27135", "County": "Roseau", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56761", "FIPS": "27135", "County": "Roseau", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56763", "FIPS": "27135", "County": "Roseau", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "55602", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55701", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55702", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55703", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55705", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55706", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55708", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55710", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55711", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55713", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55717", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55719", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55723", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55724", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55725", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55731", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55732", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55734", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55736", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55738", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55741", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55746", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55747", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55750", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55751", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55758", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55763", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55765", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55766", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55768", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55771", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55772", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55777", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55779", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55781", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55782", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55790", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55791", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55792", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55796", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55801", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55802", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55803", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55804", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55805", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55806", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55807", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55808", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55810", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55811", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55812", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55814", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55815", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55816", "FIPS": "27137", "County": "Saint Louis", "State": "MN", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "55020", "FIPS": "27139", "County": "Scott", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55054", "FIPS": "27139", "County": "Scott", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55352", "FIPS": "27139", "County": "Scott", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55372", "FIPS": "27139", "County": "Scott", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55378", "FIPS": "27139", "County": "Scott", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55379", "FIPS": "27139", "County": "Scott", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56011", "FIPS": "27139", "County": "Scott", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56071", "FIPS": "27139", "County": "Scott", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55308", "FIPS": "27141", "County": "Sherburne", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55309", "FIPS": "27141", "County": "Sherburne", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55319", "FIPS": "27141", "County": "Sherburne", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55330", "FIPS": "27141", "County": "Sherburne", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55377", "FIPS": "27141", "County": "Sherburne", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55398", "FIPS": "27141", "County": "Sherburne", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55307", "FIPS": "27143", "County": "Sibley", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55334", "FIPS": "27143", "County": "Sibley", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55335", "FIPS": "27143", "County": "Sibley", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55338", "FIPS": "27143", "County": "Sibley", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55366", "FIPS": "27143", "County": "Sibley", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55396", "FIPS": "27143", "County": "Sibley", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56044", "FIPS": "27143", "County": "Sibley", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55353", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56301", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56302", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56303", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56307", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56310", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56312", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56316", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56320", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56321", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56325", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56331", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56335", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56340", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56352", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56356", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56362", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56368", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56369", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56371", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56372", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56374", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56375", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56376", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56377", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56378", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56387", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56388", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56393", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56395", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56396", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56397", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56398", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56399", "FIPS": "27145", "County": "Stearns", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55049", "FIPS": "27147", "County": "Steele", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55060", "FIPS": "27147", "County": "Steele", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55917", "FIPS": "27147", "County": "Steele", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56026", "FIPS": "27147", "County": "Steele", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56046", "FIPS": "27147", "County": "Steele", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56207", "FIPS": "27149", "County": "Stevens", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56221", "FIPS": "27149", "County": "Stevens", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56235", "FIPS": "27149", "County": "Stevens", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56244", "FIPS": "27149", "County": "Stevens", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56267", "FIPS": "27149", "County": "Stevens", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56208", "FIPS": "27151", "County": "Swift", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56215", "FIPS": "27151", "County": "Swift", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56226", "FIPS": "27151", "County": "Swift", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56231", "FIPS": "27151", "County": "Swift", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56249", "FIPS": "27151", "County": "Swift", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56252", "FIPS": "27151", "County": "Swift", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56271", "FIPS": "27151", "County": "Swift", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56318", "FIPS": "27153", "County": "Todd", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56336", "FIPS": "27153", "County": "Todd", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56347", "FIPS": "27153", "County": "Todd", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56389", "FIPS": "27153", "County": "Todd", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56437", "FIPS": "27153", "County": "Todd", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56438", "FIPS": "27153", "County": "Todd", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56440", "FIPS": "27153", "County": "Todd", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56446", "FIPS": "27153", "County": "Todd", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56453", "FIPS": "27153", "County": "Todd", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56479", "FIPS": "27153", "County": "Todd", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56219", "FIPS": "27155", "County": "Traverse", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56236", "FIPS": "27155", "County": "Traverse", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56296", "FIPS": "27155", "County": "Traverse", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56583", "FIPS": "27155", "County": "Traverse", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55041", "FIPS": "27157", "County": "Wabasha", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55932", "FIPS": "27157", "County": "Wabasha", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55945", "FIPS": "27157", "County": "Wabasha", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55956", "FIPS": "27157", "County": "Wabasha", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55957", "FIPS": "27157", "County": "Wabasha", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55964", "FIPS": "27157", "County": "Wabasha", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55968", "FIPS": "27157", "County": "Wabasha", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55981", "FIPS": "27157", "County": "Wabasha", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55991", "FIPS": "27157", "County": "Wabasha", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56434", "FIPS": "27159", "County": "Wadena", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56464", "FIPS": "27159", "County": "Wadena", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56477", "FIPS": "27159", "County": "Wadena", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56478", "FIPS": "27159", "County": "Wadena", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56481", "FIPS": "27159", "County": "Wadena", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56482", "FIPS": "27159", "County": "Wadena", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56048", "FIPS": "27161", "County": "Waseca", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56072", "FIPS": "27161", "County": "Waseca", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56091", "FIPS": "27161", "County": "Waseca", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56093", "FIPS": "27161", "County": "Waseca", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55001", "FIPS": "27163", "County": "Washington", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55003", "FIPS": "27163", "County": "Washington", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55016", "FIPS": "27163", "County": "Washington", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55025", "FIPS": "27163", "County": "Washington", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55038", "FIPS": "27163", "County": "Washington", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55042", "FIPS": "27163", "County": "Washington", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55043", "FIPS": "27163", "County": "Washington", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55047", "FIPS": "27163", "County": "Washington", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55055", "FIPS": "27163", "County": "Washington", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55071", "FIPS": "27163", "County": "Washington", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55073", "FIPS": "27163", "County": "Washington", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55078", "FIPS": "27163", "County": "Washington", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55082", "FIPS": "27163", "County": "Washington", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55083", "FIPS": "27163", "County": "Washington", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55090", "FIPS": "27163", "County": "Washington", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55115", "FIPS": "27163", "County": "Washington", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55125", "FIPS": "27163", "County": "Washington", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55128", "FIPS": "27163", "County": "Washington", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55129", "FIPS": "27163", "County": "Washington", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56022", "FIPS": "27165", "County": "Watonwan", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56056", "FIPS": "27165", "County": "Watonwan", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56060", "FIPS": "27165", "County": "Watonwan", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56062", "FIPS": "27165", "County": "Watonwan", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56081", "FIPS": "27165", "County": "Watonwan", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56120", "FIPS": "27165", "County": "Watonwan", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56160", "FIPS": "27165", "County": "Watonwan", "State": "MN", "DMA": {"Code": 737, "Name": "MANKATO"}},
  {"Postal_Code": "56520", "FIPS": "27167", "County": "Wilkin", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56522", "FIPS": "27167", "County": "Wilkin", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56543", "FIPS": "27167", "County": "Wilkin", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56553", "FIPS": "27167", "County": "Wilkin", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56565", "FIPS": "27167", "County": "Wilkin", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56579", "FIPS": "27167", "County": "Wilkin", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "56594", "FIPS": "27167", "County": "Wilkin", "State": "MN", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "55910", "FIPS": "27169", "County": "Winona", "State": "MN", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "55925", "FIPS": "27169", "County": "Winona", "State": "MN", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "55942", "FIPS": "27169", "County": "Winona", "State": "MN", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "55952", "FIPS": "27169", "County": "Winona", "State": "MN", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "55959", "FIPS": "27169", "County": "Winona", "State": "MN", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "55969", "FIPS": "27169", "County": "Winona", "State": "MN", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "55972", "FIPS": "27169", "County": "Winona", "State": "MN", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "55979", "FIPS": "27169", "County": "Winona", "State": "MN", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "55987", "FIPS": "27169", "County": "Winona", "State": "MN", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "55988", "FIPS": "27169", "County": "Winona", "State": "MN", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "55301", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55302", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55313", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55320", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55321", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55328", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55341", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55349", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55358", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55362", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55363", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55365", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55373", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55376", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55380", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55382", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55390", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55393", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55565", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55575", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55580", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55581", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55582", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55584", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55585", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55586", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55587", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55588", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55589", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55590", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55591", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "55592", "FIPS": "27171", "County": "Wright", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56220", "FIPS": "27173", "County": "Yellow Medicine", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56223", "FIPS": "27173", "County": "Yellow Medicine", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56237", "FIPS": "27173", "County": "Yellow Medicine", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56241", "FIPS": "27173", "County": "Yellow Medicine", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56245", "FIPS": "27173", "County": "Yellow Medicine", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56280", "FIPS": "27173", "County": "Yellow Medicine", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "56297", "FIPS": "27173", "County": "Yellow Medicine", "State": "MN", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "39120", "FIPS": "28001", "County": "Adams", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39121", "FIPS": "28001", "County": "Adams", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39122", "FIPS": "28001", "County": "Adams", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39165", "FIPS": "28001", "County": "Adams", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39190", "FIPS": "28001", "County": "Adams", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "38834", "FIPS": "28003", "County": "Alcorn", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38835", "FIPS": "28003", "County": "Alcorn", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38846", "FIPS": "28003", "County": "Alcorn", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38865", "FIPS": "28003", "County": "Alcorn", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "39633", "FIPS": "28005", "County": "Amite", "State": "MS", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "39638", "FIPS": "28005", "County": "Amite", "State": "MS", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "39645", "FIPS": "28005", "County": "Amite", "State": "MS", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "39664", "FIPS": "28005", "County": "Amite", "State": "MS", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "39067", "FIPS": "28007", "County": "Attala", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39090", "FIPS": "28007", "County": "Attala", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39107", "FIPS": "28007", "County": "Attala", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39108", "FIPS": "28007", "County": "Attala", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39160", "FIPS": "28007", "County": "Attala", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "38603", "FIPS": "28009", "County": "Benton", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38633", "FIPS": "28009", "County": "Benton", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38647", "FIPS": "28009", "County": "Benton", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38720", "FIPS": "28011", "County": "Bolivar", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38725", "FIPS": "28011", "County": "Bolivar", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38726", "FIPS": "28011", "County": "Bolivar", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38730", "FIPS": "28011", "County": "Bolivar", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38732", "FIPS": "28011", "County": "Bolivar", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38733", "FIPS": "28011", "County": "Bolivar", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38740", "FIPS": "28011", "County": "Bolivar", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38746", "FIPS": "28011", "County": "Bolivar", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38759", "FIPS": "28011", "County": "Bolivar", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38762", "FIPS": "28011", "County": "Bolivar", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38764", "FIPS": "28011", "County": "Bolivar", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38769", "FIPS": "28011", "County": "Bolivar", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38772", "FIPS": "28011", "County": "Bolivar", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38773", "FIPS": "28011", "County": "Bolivar", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38774", "FIPS": "28011", "County": "Bolivar", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38781", "FIPS": "28011", "County": "Bolivar", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38839", "FIPS": "28013", "County": "Calhoun", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38878", "FIPS": "28013", "County": "Calhoun", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38913", "FIPS": "28013", "County": "Calhoun", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38914", "FIPS": "28013", "County": "Calhoun", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38915", "FIPS": "28013", "County": "Calhoun", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38916", "FIPS": "28013", "County": "Calhoun", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38951", "FIPS": "28013", "County": "Calhoun", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38955", "FIPS": "28013", "County": "Calhoun", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38917", "FIPS": "28015", "County": "Carroll", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38923", "FIPS": "28015", "County": "Carroll", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38943", "FIPS": "28015", "County": "Carroll", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38947", "FIPS": "28015", "County": "Carroll", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38954", "FIPS": "28015", "County": "Carroll", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "39176", "FIPS": "28015", "County": "Carroll", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38850", "FIPS": "28017", "County": "Chickasaw", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38851", "FIPS": "28017", "County": "Chickasaw", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38860", "FIPS": "28017", "County": "Chickasaw", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38875", "FIPS": "28017", "County": "Chickasaw", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38877", "FIPS": "28017", "County": "Chickasaw", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39776", "FIPS": "28017", "County": "Chickasaw", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39735", "FIPS": "28019", "County": "Choctaw", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39745", "FIPS": "28019", "County": "Choctaw", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39772", "FIPS": "28019", "County": "Choctaw", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39086", "FIPS": "28021", "County": "Claiborne", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39144", "FIPS": "28021", "County": "Claiborne", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39150", "FIPS": "28021", "County": "Claiborne", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39330", "FIPS": "28023", "County": "Clarke", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39347", "FIPS": "28023", "County": "Clarke", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39355", "FIPS": "28023", "County": "Clarke", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39360", "FIPS": "28023", "County": "Clarke", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39363", "FIPS": "28023", "County": "Clarke", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39741", "FIPS": "28025", "County": "Clay", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39751", "FIPS": "28025", "County": "Clay", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39754", "FIPS": "28025", "County": "Clay", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39755", "FIPS": "28025", "County": "Clay", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39773", "FIPS": "28025", "County": "Clay", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38614", "FIPS": "28027", "County": "Coahoma", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38617", "FIPS": "28027", "County": "Coahoma", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38630", "FIPS": "28027", "County": "Coahoma", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38631", "FIPS": "28027", "County": "Coahoma", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38639", "FIPS": "28027", "County": "Coahoma", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38644", "FIPS": "28027", "County": "Coahoma", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38645", "FIPS": "28027", "County": "Coahoma", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38669", "FIPS": "28027", "County": "Coahoma", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38739", "FIPS": "28027", "County": "Coahoma", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38767", "FIPS": "28027", "County": "Coahoma", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "39059", "FIPS": "28029", "County": "Copiah", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39077", "FIPS": "28029", "County": "Copiah", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39078", "FIPS": "28029", "County": "Copiah", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39083", "FIPS": "28029", "County": "Copiah", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39191", "FIPS": "28029", "County": "Copiah", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39119", "FIPS": "28031", "County": "Covington", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39428", "FIPS": "28031", "County": "Covington", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39479", "FIPS": "28031", "County": "Covington", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "38632", "FIPS": "28033", "County": "DeSoto", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38637", "FIPS": "28033", "County": "DeSoto", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38641", "FIPS": "28033", "County": "DeSoto", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38651", "FIPS": "28033", "County": "DeSoto", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38654", "FIPS": "28033", "County": "DeSoto", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38671", "FIPS": "28033", "County": "DeSoto", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38672", "FIPS": "28033", "County": "DeSoto", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38680", "FIPS": "28033", "County": "DeSoto", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38686", "FIPS": "28033", "County": "DeSoto", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "39401", "FIPS": "28035", "County": "Forrest", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39402", "FIPS": "28035", "County": "Forrest", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39403", "FIPS": "28035", "County": "Forrest", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39404", "FIPS": "28035", "County": "Forrest", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39406", "FIPS": "28035", "County": "Forrest", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39407", "FIPS": "28035", "County": "Forrest", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39425", "FIPS": "28035", "County": "Forrest", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39465", "FIPS": "28035", "County": "Forrest", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39630", "FIPS": "28037", "County": "Franklin", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39647", "FIPS": "28037", "County": "Franklin", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39653", "FIPS": "28037", "County": "Franklin", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39661", "FIPS": "28037", "County": "Franklin", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39452", "FIPS": "28039", "County": "George", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39362", "FIPS": "28041", "County": "Greene", "State": "MS", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "39451", "FIPS": "28041", "County": "Greene", "State": "MS", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "39456", "FIPS": "28041", "County": "Greene", "State": "MS", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "39461", "FIPS": "28041", "County": "Greene", "State": "MS", "DMA": {"Code": 686, "Name": "MOBILE-PENSACOLA"}},
  {"Postal_Code": "38901", "FIPS": "28043", "County": "Grenada", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38902", "FIPS": "28043", "County": "Grenada", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38926", "FIPS": "28043", "County": "Grenada", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38929", "FIPS": "28043", "County": "Grenada", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38940", "FIPS": "28043", "County": "Grenada", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38960", "FIPS": "28043", "County": "Grenada", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "39520", "FIPS": "28045", "County": "Hancock", "State": "MS", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "39521", "FIPS": "28045", "County": "Hancock", "State": "MS", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "39522", "FIPS": "28045", "County": "Hancock", "State": "MS", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "39525", "FIPS": "28045", "County": "Hancock", "State": "MS", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "39529", "FIPS": "28045", "County": "Hancock", "State": "MS", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "39556", "FIPS": "28045", "County": "Hancock", "State": "MS", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "39558", "FIPS": "28045", "County": "Hancock", "State": "MS", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "39572", "FIPS": "28045", "County": "Hancock", "State": "MS", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "39576", "FIPS": "28045", "County": "Hancock", "State": "MS", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "39501", "FIPS": "28047", "County": "Harrison", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39502", "FIPS": "28047", "County": "Harrison", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39503", "FIPS": "28047", "County": "Harrison", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39505", "FIPS": "28047", "County": "Harrison", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39506", "FIPS": "28047", "County": "Harrison", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39507", "FIPS": "28047", "County": "Harrison", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39530", "FIPS": "28047", "County": "Harrison", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39531", "FIPS": "28047", "County": "Harrison", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39532", "FIPS": "28047", "County": "Harrison", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39533", "FIPS": "28047", "County": "Harrison", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39534", "FIPS": "28047", "County": "Harrison", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39535", "FIPS": "28047", "County": "Harrison", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39540", "FIPS": "28047", "County": "Harrison", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39560", "FIPS": "28047", "County": "Harrison", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39571", "FIPS": "28047", "County": "Harrison", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39574", "FIPS": "28047", "County": "Harrison", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39041", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39056", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39058", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39060", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39066", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39072", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39154", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39170", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39174", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39175", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39201", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39202", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39203", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39204", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39205", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39206", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39207", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39209", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39210", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39211", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39212", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39213", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39215", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39216", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39217", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39225", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39235", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39236", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39250", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39269", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39271", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39272", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39282", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39283", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39284", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39286", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39289", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39296", "FIPS": "28049", "County": "Hinds", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "38924", "FIPS": "28051", "County": "Holmes", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39063", "FIPS": "28051", "County": "Holmes", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39079", "FIPS": "28051", "County": "Holmes", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39095", "FIPS": "28051", "County": "Holmes", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39146", "FIPS": "28051", "County": "Holmes", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39169", "FIPS": "28051", "County": "Holmes", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39192", "FIPS": "28051", "County": "Holmes", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "38754", "FIPS": "28053", "County": "Humphreys", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39038", "FIPS": "28053", "County": "Humphreys", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39097", "FIPS": "28053", "County": "Humphreys", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39115", "FIPS": "28053", "County": "Humphreys", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39166", "FIPS": "28053", "County": "Humphreys", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "38745", "FIPS": "28055", "County": "Issaquena", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39113", "FIPS": "28055", "County": "Issaquena", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39177", "FIPS": "28055", "County": "Issaquena", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "38843", "FIPS": "28057", "County": "Itawamba", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38847", "FIPS": "28057", "County": "Itawamba", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38855", "FIPS": "28057", "County": "Itawamba", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38858", "FIPS": "28057", "County": "Itawamba", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38876", "FIPS": "28057", "County": "Itawamba", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39552", "FIPS": "28059", "County": "Jackson", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39553", "FIPS": "28059", "County": "Jackson", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39555", "FIPS": "28059", "County": "Jackson", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39562", "FIPS": "28059", "County": "Jackson", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39563", "FIPS": "28059", "County": "Jackson", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39564", "FIPS": "28059", "County": "Jackson", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39565", "FIPS": "28059", "County": "Jackson", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39566", "FIPS": "28059", "County": "Jackson", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39567", "FIPS": "28059", "County": "Jackson", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39568", "FIPS": "28059", "County": "Jackson", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39569", "FIPS": "28059", "County": "Jackson", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39581", "FIPS": "28059", "County": "Jackson", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39595", "FIPS": "28059", "County": "Jackson", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39338", "FIPS": "28061", "County": "Jasper", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39348", "FIPS": "28061", "County": "Jasper", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39356", "FIPS": "28061", "County": "Jasper", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39366", "FIPS": "28061", "County": "Jasper", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39422", "FIPS": "28061", "County": "Jasper", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39439", "FIPS": "28061", "County": "Jasper", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39460", "FIPS": "28061", "County": "Jasper", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39481", "FIPS": "28061", "County": "Jasper", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39069", "FIPS": "28063", "County": "Jefferson", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39081", "FIPS": "28063", "County": "Jefferson", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39096", "FIPS": "28063", "County": "Jefferson", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39668", "FIPS": "28063", "County": "Jefferson", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39421", "FIPS": "28065", "County": "Jefferson Davis", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39427", "FIPS": "28065", "County": "Jefferson Davis", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39474", "FIPS": "28065", "County": "Jefferson Davis", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39436", "FIPS": "28067", "County": "Jones", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39437", "FIPS": "28067", "County": "Jones", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39440", "FIPS": "28067", "County": "Jones", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39441", "FIPS": "28067", "County": "Jones", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39442", "FIPS": "28067", "County": "Jones", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39443", "FIPS": "28067", "County": "Jones", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39459", "FIPS": "28067", "County": "Jones", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39464", "FIPS": "28067", "County": "Jones", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39477", "FIPS": "28067", "County": "Jones", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39480", "FIPS": "28067", "County": "Jones", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39328", "FIPS": "28069", "County": "Kemper", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39352", "FIPS": "28069", "County": "Kemper", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39354", "FIPS": "28069", "County": "Kemper", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39358", "FIPS": "28069", "County": "Kemper", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "38601", "FIPS": "28071", "County": "Lafayette", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38655", "FIPS": "28071", "County": "Lafayette", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38673", "FIPS": "28071", "County": "Lafayette", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38675", "FIPS": "28071", "County": "Lafayette", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38677", "FIPS": "28071", "County": "Lafayette", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38874", "FIPS": "28071", "County": "Lafayette", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38949", "FIPS": "28071", "County": "Lafayette", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "39455", "FIPS": "28073", "County": "Lamar", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39475", "FIPS": "28073", "County": "Lamar", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39482", "FIPS": "28073", "County": "Lamar", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39301", "FIPS": "28075", "County": "Lauderdale", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39302", "FIPS": "28075", "County": "Lauderdale", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39303", "FIPS": "28075", "County": "Lauderdale", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39304", "FIPS": "28075", "County": "Lauderdale", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39305", "FIPS": "28075", "County": "Lauderdale", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39307", "FIPS": "28075", "County": "Lauderdale", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39309", "FIPS": "28075", "County": "Lauderdale", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39320", "FIPS": "28075", "County": "Lauderdale", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39325", "FIPS": "28075", "County": "Lauderdale", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39326", "FIPS": "28075", "County": "Lauderdale", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39335", "FIPS": "28075", "County": "Lauderdale", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39342", "FIPS": "28075", "County": "Lauderdale", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39364", "FIPS": "28075", "County": "Lauderdale", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39140", "FIPS": "28077", "County": "Lawrence", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39641", "FIPS": "28077", "County": "Lawrence", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39654", "FIPS": "28077", "County": "Lawrence", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39656", "FIPS": "28077", "County": "Lawrence", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39663", "FIPS": "28077", "County": "Lawrence", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39665", "FIPS": "28077", "County": "Lawrence", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39051", "FIPS": "28079", "County": "Leake", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39094", "FIPS": "28079", "County": "Leake", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39109", "FIPS": "28079", "County": "Leake", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39171", "FIPS": "28079", "County": "Leake", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39189", "FIPS": "28079", "County": "Leake", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "38801", "FIPS": "28081", "County": "Lee", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38802", "FIPS": "28081", "County": "Lee", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38803", "FIPS": "28081", "County": "Lee", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38804", "FIPS": "28081", "County": "Lee", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38824", "FIPS": "28081", "County": "Lee", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38826", "FIPS": "28081", "County": "Lee", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38849", "FIPS": "28081", "County": "Lee", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38857", "FIPS": "28081", "County": "Lee", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38862", "FIPS": "28081", "County": "Lee", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38866", "FIPS": "28081", "County": "Lee", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38868", "FIPS": "28081", "County": "Lee", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38879", "FIPS": "28081", "County": "Lee", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38930", "FIPS": "28083", "County": "Leflore", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38935", "FIPS": "28083", "County": "Leflore", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38941", "FIPS": "28083", "County": "Leflore", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38944", "FIPS": "28083", "County": "Leflore", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38945", "FIPS": "28083", "County": "Leflore", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38946", "FIPS": "28083", "County": "Leflore", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38952", "FIPS": "28083", "County": "Leflore", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38959", "FIPS": "28083", "County": "Leflore", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "39601", "FIPS": "28085", "County": "Lincoln", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39602", "FIPS": "28085", "County": "Lincoln", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39603", "FIPS": "28085", "County": "Lincoln", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39629", "FIPS": "28085", "County": "Lincoln", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39662", "FIPS": "28085", "County": "Lincoln", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39701", "FIPS": "28087", "County": "Lowndes", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39702", "FIPS": "28087", "County": "Lowndes", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39703", "FIPS": "28087", "County": "Lowndes", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39704", "FIPS": "28087", "County": "Lowndes", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39705", "FIPS": "28087", "County": "Lowndes", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39710", "FIPS": "28087", "County": "Lowndes", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39736", "FIPS": "28087", "County": "Lowndes", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39740", "FIPS": "28087", "County": "Lowndes", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39743", "FIPS": "28087", "County": "Lowndes", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39753", "FIPS": "28087", "County": "Lowndes", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39766", "FIPS": "28087", "County": "Lowndes", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39045", "FIPS": "28089", "County": "Madison", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39046", "FIPS": "28089", "County": "Madison", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39071", "FIPS": "28089", "County": "Madison", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39110", "FIPS": "28089", "County": "Madison", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39130", "FIPS": "28089", "County": "Madison", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39157", "FIPS": "28089", "County": "Madison", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39158", "FIPS": "28089", "County": "Madison", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39163", "FIPS": "28089", "County": "Madison", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39429", "FIPS": "28091", "County": "Marion", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39478", "FIPS": "28091", "County": "Marion", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39483", "FIPS": "28091", "County": "Marion", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39643", "FIPS": "28091", "County": "Marion", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "38611", "FIPS": "28093", "County": "Marshall", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38634", "FIPS": "28093", "County": "Marshall", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38635", "FIPS": "28093", "County": "Marshall", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38642", "FIPS": "28093", "County": "Marshall", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38649", "FIPS": "28093", "County": "Marshall", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38659", "FIPS": "28093", "County": "Marshall", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38661", "FIPS": "28093", "County": "Marshall", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38679", "FIPS": "28093", "County": "Marshall", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38685", "FIPS": "28093", "County": "Marshall", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38821", "FIPS": "28095", "County": "Monroe", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38825", "FIPS": "28095", "County": "Monroe", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38844", "FIPS": "28095", "County": "Monroe", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38848", "FIPS": "28095", "County": "Monroe", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38870", "FIPS": "28095", "County": "Monroe", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39730", "FIPS": "28095", "County": "Monroe", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39746", "FIPS": "28095", "County": "Monroe", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39756", "FIPS": "28095", "County": "Monroe", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38925", "FIPS": "28097", "County": "Montgomery", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38967", "FIPS": "28097", "County": "Montgomery", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39747", "FIPS": "28097", "County": "Montgomery", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39767", "FIPS": "28097", "County": "Montgomery", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39350", "FIPS": "28099", "County": "Neshoba", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39365", "FIPS": "28099", "County": "Neshoba", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39057", "FIPS": "28101", "County": "Newton", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39323", "FIPS": "28101", "County": "Newton", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39327", "FIPS": "28101", "County": "Newton", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39332", "FIPS": "28101", "County": "Newton", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39336", "FIPS": "28101", "County": "Newton", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39337", "FIPS": "28101", "County": "Newton", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39345", "FIPS": "28101", "County": "Newton", "State": "MS", "DMA": {"Code": 711, "Name": "MERIDIAN"}},
  {"Postal_Code": "39341", "FIPS": "28103", "County": "Noxubee", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39361", "FIPS": "28103", "County": "Noxubee", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39739", "FIPS": "28103", "County": "Noxubee", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39759", "FIPS": "28105", "County": "Oktibbeha", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39760", "FIPS": "28105", "County": "Oktibbeha", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39762", "FIPS": "28105", "County": "Oktibbeha", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39769", "FIPS": "28105", "County": "Oktibbeha", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38606", "FIPS": "28107", "County": "Panola", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38619", "FIPS": "28107", "County": "Panola", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38620", "FIPS": "28107", "County": "Panola", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38621", "FIPS": "28107", "County": "Panola", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38658", "FIPS": "28107", "County": "Panola", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38665", "FIPS": "28107", "County": "Panola", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38666", "FIPS": "28107", "County": "Panola", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "39426", "FIPS": "28109", "County": "Pearl River", "State": "MS", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "39457", "FIPS": "28109", "County": "Pearl River", "State": "MS", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "39463", "FIPS": "28109", "County": "Pearl River", "State": "MS", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "39466", "FIPS": "28109", "County": "Pearl River", "State": "MS", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "39470", "FIPS": "28109", "County": "Pearl River", "State": "MS", "DMA": {"Code": 622, "Name": "NEW ORLEANS"}},
  {"Postal_Code": "39423", "FIPS": "28111", "County": "Perry", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39462", "FIPS": "28111", "County": "Perry", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39476", "FIPS": "28111", "County": "Perry", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39632", "FIPS": "28113", "County": "Pike", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39635", "FIPS": "28113", "County": "Pike", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39648", "FIPS": "28113", "County": "Pike", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39649", "FIPS": "28113", "County": "Pike", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39652", "FIPS": "28113", "County": "Pike", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39657", "FIPS": "28113", "County": "Pike", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39666", "FIPS": "28113", "County": "Pike", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "38820", "FIPS": "28115", "County": "Pontotoc", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38841", "FIPS": "28115", "County": "Pontotoc", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38863", "FIPS": "28115", "County": "Pontotoc", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38864", "FIPS": "28115", "County": "Pontotoc", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38869", "FIPS": "28115", "County": "Pontotoc", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38871", "FIPS": "28115", "County": "Pontotoc", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38829", "FIPS": "28117", "County": "Prentiss", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38856", "FIPS": "28117", "County": "Prentiss", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38859", "FIPS": "28117", "County": "Prentiss", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38880", "FIPS": "28117", "County": "Prentiss", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38609", "FIPS": "28119", "County": "Quitman", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38622", "FIPS": "28119", "County": "Quitman", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38623", "FIPS": "28119", "County": "Quitman", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38628", "FIPS": "28119", "County": "Quitman", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38643", "FIPS": "28119", "County": "Quitman", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38646", "FIPS": "28119", "County": "Quitman", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38964", "FIPS": "28119", "County": "Quitman", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "39042", "FIPS": "28121", "County": "Rankin", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39043", "FIPS": "28121", "County": "Rankin", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39047", "FIPS": "28121", "County": "Rankin", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39073", "FIPS": "28121", "County": "Rankin", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39145", "FIPS": "28121", "County": "Rankin", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39148", "FIPS": "28121", "County": "Rankin", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39151", "FIPS": "28121", "County": "Rankin", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39161", "FIPS": "28121", "County": "Rankin", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39167", "FIPS": "28121", "County": "Rankin", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39193", "FIPS": "28121", "County": "Rankin", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39208", "FIPS": "28121", "County": "Rankin", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39218", "FIPS": "28121", "County": "Rankin", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39232", "FIPS": "28121", "County": "Rankin", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39288", "FIPS": "28121", "County": "Rankin", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39298", "FIPS": "28121", "County": "Rankin", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39074", "FIPS": "28123", "County": "Scott", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39080", "FIPS": "28123", "County": "Scott", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39087", "FIPS": "28123", "County": "Scott", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39092", "FIPS": "28123", "County": "Scott", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39098", "FIPS": "28123", "County": "Scott", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39117", "FIPS": "28123", "County": "Scott", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39152", "FIPS": "28123", "County": "Scott", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39359", "FIPS": "28123", "County": "Scott", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "38721", "FIPS": "28125", "County": "Sharkey", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "38765", "FIPS": "28125", "County": "Sharkey", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39054", "FIPS": "28125", "County": "Sharkey", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39061", "FIPS": "28125", "County": "Sharkey", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39159", "FIPS": "28125", "County": "Sharkey", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39044", "FIPS": "28127", "County": "Simpson", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39062", "FIPS": "28127", "County": "Simpson", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39082", "FIPS": "28127", "County": "Simpson", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39111", "FIPS": "28127", "County": "Simpson", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39114", "FIPS": "28127", "County": "Simpson", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39149", "FIPS": "28127", "County": "Simpson", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39116", "FIPS": "28129", "County": "Smith", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39153", "FIPS": "28129", "County": "Smith", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39168", "FIPS": "28129", "County": "Smith", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39561", "FIPS": "28131", "County": "Stone", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39573", "FIPS": "28131", "County": "Stone", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "39577", "FIPS": "28131", "County": "Stone", "State": "MS", "DMA": {"Code": 746, "Name": "BILOXI-GULFPORT"}},
  {"Postal_Code": "38736", "FIPS": "28133", "County": "Sunflower", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38737", "FIPS": "28133", "County": "Sunflower", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38738", "FIPS": "28133", "County": "Sunflower", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38749", "FIPS": "28133", "County": "Sunflower", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38751", "FIPS": "28133", "County": "Sunflower", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38753", "FIPS": "28133", "County": "Sunflower", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38761", "FIPS": "28133", "County": "Sunflower", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38768", "FIPS": "28133", "County": "Sunflower", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38771", "FIPS": "28133", "County": "Sunflower", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38778", "FIPS": "28133", "County": "Sunflower", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38920", "FIPS": "28135", "County": "Tallahatchie", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38921", "FIPS": "28135", "County": "Tallahatchie", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38927", "FIPS": "28135", "County": "Tallahatchie", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38928", "FIPS": "28135", "County": "Tallahatchie", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38950", "FIPS": "28135", "County": "Tallahatchie", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38957", "FIPS": "28135", "County": "Tallahatchie", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38958", "FIPS": "28135", "County": "Tallahatchie", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38962", "FIPS": "28135", "County": "Tallahatchie", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38963", "FIPS": "28135", "County": "Tallahatchie", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38966", "FIPS": "28135", "County": "Tallahatchie", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38602", "FIPS": "28137", "County": "Tate", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38618", "FIPS": "28137", "County": "Tate", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38638", "FIPS": "28137", "County": "Tate", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38668", "FIPS": "28137", "County": "Tate", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38610", "FIPS": "28139", "County": "Tippah", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38625", "FIPS": "28139", "County": "Tippah", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38629", "FIPS": "28139", "County": "Tippah", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38663", "FIPS": "28139", "County": "Tippah", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38674", "FIPS": "28139", "County": "Tippah", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38683", "FIPS": "28139", "County": "Tippah", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38827", "FIPS": "28141", "County": "Tishomingo", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38833", "FIPS": "28141", "County": "Tishomingo", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38838", "FIPS": "28141", "County": "Tishomingo", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38852", "FIPS": "28141", "County": "Tishomingo", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38873", "FIPS": "28141", "County": "Tishomingo", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38626", "FIPS": "28143", "County": "Tunica", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38664", "FIPS": "28143", "County": "Tunica", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38670", "FIPS": "28143", "County": "Tunica", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38676", "FIPS": "28143", "County": "Tunica", "State": "MS", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38627", "FIPS": "28145", "County": "Union", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38650", "FIPS": "28145", "County": "Union", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38652", "FIPS": "28145", "County": "Union", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38828", "FIPS": "28145", "County": "Union", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39667", "FIPS": "28147", "County": "Walthall", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39156", "FIPS": "28149", "County": "Warren", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39180", "FIPS": "28149", "County": "Warren", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39181", "FIPS": "28149", "County": "Warren", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39182", "FIPS": "28149", "County": "Warren", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39183", "FIPS": "28149", "County": "Warren", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "38701", "FIPS": "28151", "County": "Washington", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38702", "FIPS": "28151", "County": "Washington", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38703", "FIPS": "28151", "County": "Washington", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38704", "FIPS": "28151", "County": "Washington", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38722", "FIPS": "28151", "County": "Washington", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38723", "FIPS": "28151", "County": "Washington", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38731", "FIPS": "28151", "County": "Washington", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38744", "FIPS": "28151", "County": "Washington", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38748", "FIPS": "28151", "County": "Washington", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38756", "FIPS": "28151", "County": "Washington", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38760", "FIPS": "28151", "County": "Washington", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38776", "FIPS": "28151", "County": "Washington", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38780", "FIPS": "28151", "County": "Washington", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "38782", "FIPS": "28151", "County": "Washington", "State": "MS", "DMA": {"Code": 647, "Name": "GREENWOOD-GREENVILLE"}},
  {"Postal_Code": "39322", "FIPS": "28153", "County": "Wayne", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39324", "FIPS": "28153", "County": "Wayne", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39367", "FIPS": "28153", "County": "Wayne", "State": "MS", "DMA": {"Code": 710, "Name": "HATTIESBURG-LAUREL"}},
  {"Postal_Code": "39737", "FIPS": "28155", "County": "Webster", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39744", "FIPS": "28155", "County": "Webster", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39750", "FIPS": "28155", "County": "Webster", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39752", "FIPS": "28155", "County": "Webster", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39771", "FIPS": "28155", "County": "Webster", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39631", "FIPS": "28157", "County": "Wilkinson", "State": "MS", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "39669", "FIPS": "28157", "County": "Wilkinson", "State": "MS", "DMA": {"Code": 716, "Name": "BATON ROUGE"}},
  {"Postal_Code": "39339", "FIPS": "28159", "County": "Winston", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39346", "FIPS": "28159", "County": "Winston", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38922", "FIPS": "28161", "County": "Yalobusha", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38948", "FIPS": "28161", "County": "Yalobusha", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38953", "FIPS": "28161", "County": "Yalobusha", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38961", "FIPS": "28161", "County": "Yalobusha", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "38965", "FIPS": "28161", "County": "Yalobusha", "State": "MS", "DMA": {"Code": 673, "Name": "COLUMBUS-TUPELO-WEST POINT"}},
  {"Postal_Code": "39039", "FIPS": "28163", "County": "Yazoo", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39040", "FIPS": "28163", "County": "Yazoo", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39088", "FIPS": "28163", "County": "Yazoo", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39162", "FIPS": "28163", "County": "Yazoo", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39173", "FIPS": "28163", "County": "Yazoo", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39179", "FIPS": "28163", "County": "Yazoo", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "39194", "FIPS": "28163", "County": "Yazoo", "State": "MS", "DMA": {"Code": 718, "Name": "JACKSON, MS"}},
  {"Postal_Code": "63501", "FIPS": "29001", "County": "Adair", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63533", "FIPS": "29001", "County": "Adair", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63540", "FIPS": "29001", "County": "Adair", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63546", "FIPS": "29001", "County": "Adair", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63559", "FIPS": "29001", "County": "Adair", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "64421", "FIPS": "29003", "County": "Andrew", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64427", "FIPS": "29003", "County": "Andrew", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64436", "FIPS": "29003", "County": "Andrew", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64449", "FIPS": "29003", "County": "Andrew", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64459", "FIPS": "29003", "County": "Andrew", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64480", "FIPS": "29003", "County": "Andrew", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64483", "FIPS": "29003", "County": "Andrew", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64485", "FIPS": "29003", "County": "Andrew", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64446", "FIPS": "29005", "County": "Atchison", "State": "MO", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "64482", "FIPS": "29005", "County": "Atchison", "State": "MO", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "64491", "FIPS": "29005", "County": "Atchison", "State": "MO", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "64496", "FIPS": "29005", "County": "Atchison", "State": "MO", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "64498", "FIPS": "29005", "County": "Atchison", "State": "MO", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "63345", "FIPS": "29007", "County": "Audrain", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "63352", "FIPS": "29007", "County": "Audrain", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "63382", "FIPS": "29007", "County": "Audrain", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65232", "FIPS": "29007", "County": "Audrain", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65264", "FIPS": "29007", "County": "Audrain", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65265", "FIPS": "29007", "County": "Audrain", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65280", "FIPS": "29007", "County": "Audrain", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65285", "FIPS": "29007", "County": "Audrain", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "64874", "FIPS": "29009", "County": "Barry", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65623", "FIPS": "29009", "County": "Barry", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65625", "FIPS": "29009", "County": "Barry", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65641", "FIPS": "29009", "County": "Barry", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65647", "FIPS": "29009", "County": "Barry", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65658", "FIPS": "29009", "County": "Barry", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65708", "FIPS": "29009", "County": "Barry", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65734", "FIPS": "29009", "County": "Barry", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65745", "FIPS": "29009", "County": "Barry", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65747", "FIPS": "29009", "County": "Barry", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65772", "FIPS": "29009", "County": "Barry", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "64748", "FIPS": "29011", "County": "Barton", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64759", "FIPS": "29011", "County": "Barton", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64762", "FIPS": "29011", "County": "Barton", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64766", "FIPS": "29011", "County": "Barton", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64769", "FIPS": "29011", "County": "Barton", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64832", "FIPS": "29011", "County": "Barton", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64720", "FIPS": "29013", "County": "Bates", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64722", "FIPS": "29013", "County": "Bates", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64723", "FIPS": "29013", "County": "Bates", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64730", "FIPS": "29013", "County": "Bates", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64742", "FIPS": "29013", "County": "Bates", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64745", "FIPS": "29013", "County": "Bates", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64752", "FIPS": "29013", "County": "Bates", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64779", "FIPS": "29013", "County": "Bates", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64780", "FIPS": "29013", "County": "Bates", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65325", "FIPS": "29015", "County": "Benton", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65326", "FIPS": "29015", "County": "Benton", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65335", "FIPS": "29015", "County": "Benton", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65338", "FIPS": "29015", "County": "Benton", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65355", "FIPS": "29015", "County": "Benton", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "63662", "FIPS": "29017", "County": "Bollinger", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63750", "FIPS": "29017", "County": "Bollinger", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63751", "FIPS": "29017", "County": "Bollinger", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63760", "FIPS": "29017", "County": "Bollinger", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63764", "FIPS": "29017", "County": "Bollinger", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63781", "FIPS": "29017", "County": "Bollinger", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63782", "FIPS": "29017", "County": "Bollinger", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63787", "FIPS": "29017", "County": "Bollinger", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "65010", "FIPS": "29019", "County": "Boone", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65039", "FIPS": "29019", "County": "Boone", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65201", "FIPS": "29019", "County": "Boone", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65202", "FIPS": "29019", "County": "Boone", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65203", "FIPS": "29019", "County": "Boone", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65205", "FIPS": "29019", "County": "Boone", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65211", "FIPS": "29019", "County": "Boone", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65212", "FIPS": "29019", "County": "Boone", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65215", "FIPS": "29019", "County": "Boone", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65216", "FIPS": "29019", "County": "Boone", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65217", "FIPS": "29019", "County": "Boone", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65218", "FIPS": "29019", "County": "Boone", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65240", "FIPS": "29019", "County": "Boone", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65255", "FIPS": "29019", "County": "Boone", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65256", "FIPS": "29019", "County": "Boone", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65279", "FIPS": "29019", "County": "Boone", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65284", "FIPS": "29019", "County": "Boone", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65299", "FIPS": "29019", "County": "Boone", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "64401", "FIPS": "29021", "County": "Buchanan", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64440", "FIPS": "29021", "County": "Buchanan", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64443", "FIPS": "29021", "County": "Buchanan", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64448", "FIPS": "29021", "County": "Buchanan", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64484", "FIPS": "29021", "County": "Buchanan", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64501", "FIPS": "29021", "County": "Buchanan", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64502", "FIPS": "29021", "County": "Buchanan", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64503", "FIPS": "29021", "County": "Buchanan", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64504", "FIPS": "29021", "County": "Buchanan", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64505", "FIPS": "29021", "County": "Buchanan", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64506", "FIPS": "29021", "County": "Buchanan", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64507", "FIPS": "29021", "County": "Buchanan", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64508", "FIPS": "29021", "County": "Buchanan", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "63901", "FIPS": "29023", "County": "Butler", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63902", "FIPS": "29023", "County": "Butler", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63932", "FIPS": "29023", "County": "Butler", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63938", "FIPS": "29023", "County": "Butler", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63940", "FIPS": "29023", "County": "Butler", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63945", "FIPS": "29023", "County": "Butler", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63954", "FIPS": "29023", "County": "Butler", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63961", "FIPS": "29023", "County": "Butler", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63962", "FIPS": "29023", "County": "Butler", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "64624", "FIPS": "29025", "County": "Caldwell", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64625", "FIPS": "29025", "County": "Caldwell", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64637", "FIPS": "29025", "County": "Caldwell", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64644", "FIPS": "29025", "County": "Caldwell", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64649", "FIPS": "29025", "County": "Caldwell", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64650", "FIPS": "29025", "County": "Caldwell", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64671", "FIPS": "29025", "County": "Caldwell", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "63388", "FIPS": "29027", "County": "Callaway", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65043", "FIPS": "29027", "County": "Callaway", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65059", "FIPS": "29027", "County": "Callaway", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65063", "FIPS": "29027", "County": "Callaway", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65067", "FIPS": "29027", "County": "Callaway", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65077", "FIPS": "29027", "County": "Callaway", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65080", "FIPS": "29027", "County": "Callaway", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65231", "FIPS": "29027", "County": "Callaway", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65251", "FIPS": "29027", "County": "Callaway", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65262", "FIPS": "29027", "County": "Callaway", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65020", "FIPS": "29029", "County": "Camden", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65049", "FIPS": "29029", "County": "Camden", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65052", "FIPS": "29029", "County": "Camden", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65065", "FIPS": "29029", "County": "Camden", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65079", "FIPS": "29029", "County": "Camden", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65324", "FIPS": "29029", "County": "Camden", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65591", "FIPS": "29029", "County": "Camden", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65786", "FIPS": "29029", "County": "Camden", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65787", "FIPS": "29029", "County": "Camden", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "63701", "FIPS": "29031", "County": "Cape Girardeau", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63702", "FIPS": "29031", "County": "Cape Girardeau", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63703", "FIPS": "29031", "County": "Cape Girardeau", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63732", "FIPS": "29031", "County": "Cape Girardeau", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63739", "FIPS": "29031", "County": "Cape Girardeau", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63743", "FIPS": "29031", "County": "Cape Girardeau", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63744", "FIPS": "29031", "County": "Cape Girardeau", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63745", "FIPS": "29031", "County": "Cape Girardeau", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63747", "FIPS": "29031", "County": "Cape Girardeau", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63752", "FIPS": "29031", "County": "Cape Girardeau", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63755", "FIPS": "29031", "County": "Cape Girardeau", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63766", "FIPS": "29031", "County": "Cape Girardeau", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63769", "FIPS": "29031", "County": "Cape Girardeau", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63770", "FIPS": "29031", "County": "Cape Girardeau", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63779", "FIPS": "29031", "County": "Cape Girardeau", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63785", "FIPS": "29031", "County": "Cape Girardeau", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "64622", "FIPS": "29033", "County": "Carroll", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64623", "FIPS": "29033", "County": "Carroll", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64633", "FIPS": "29033", "County": "Carroll", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64639", "FIPS": "29033", "County": "Carroll", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64643", "FIPS": "29033", "County": "Carroll", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64668", "FIPS": "29033", "County": "Carroll", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64680", "FIPS": "29033", "County": "Carroll", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64682", "FIPS": "29033", "County": "Carroll", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "63937", "FIPS": "29035", "County": "Carter", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63941", "FIPS": "29035", "County": "Carter", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63943", "FIPS": "29035", "County": "Carter", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63965", "FIPS": "29035", "County": "Carter", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "64012", "FIPS": "29037", "County": "Cass", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64078", "FIPS": "29037", "County": "Cass", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64080", "FIPS": "29037", "County": "Cass", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64083", "FIPS": "29037", "County": "Cass", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64090", "FIPS": "29037", "County": "Cass", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64701", "FIPS": "29037", "County": "Cass", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64725", "FIPS": "29037", "County": "Cass", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64734", "FIPS": "29037", "County": "Cass", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64743", "FIPS": "29037", "County": "Cass", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64746", "FIPS": "29037", "County": "Cass", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64747", "FIPS": "29037", "County": "Cass", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64744", "FIPS": "29039", "County": "Cedar", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "64756", "FIPS": "29039", "County": "Cedar", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65607", "FIPS": "29039", "County": "Cedar", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65785", "FIPS": "29039", "County": "Cedar", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "64660", "FIPS": "29041", "County": "Chariton", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "64676", "FIPS": "29041", "County": "Chariton", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "64681", "FIPS": "29041", "County": "Chariton", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65236", "FIPS": "29041", "County": "Chariton", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65246", "FIPS": "29041", "County": "Chariton", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65261", "FIPS": "29041", "County": "Chariton", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65281", "FIPS": "29041", "County": "Chariton", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65286", "FIPS": "29041", "County": "Chariton", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65610", "FIPS": "29043", "County": "Christian", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65620", "FIPS": "29043", "County": "Christian", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65629", "FIPS": "29043", "County": "Christian", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65630", "FIPS": "29043", "County": "Christian", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65631", "FIPS": "29043", "County": "Christian", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65657", "FIPS": "29043", "County": "Christian", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65669", "FIPS": "29043", "County": "Christian", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65714", "FIPS": "29043", "County": "Christian", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65720", "FIPS": "29043", "County": "Christian", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65721", "FIPS": "29043", "County": "Christian", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65753", "FIPS": "29043", "County": "Christian", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65754", "FIPS": "29043", "County": "Christian", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "63430", "FIPS": "29045", "County": "Clark", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63445", "FIPS": "29045", "County": "Clark", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63453", "FIPS": "29045", "County": "Clark", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63465", "FIPS": "29045", "County": "Clark", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63466", "FIPS": "29045", "County": "Clark", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63472", "FIPS": "29045", "County": "Clark", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63474", "FIPS": "29045", "County": "Clark", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "64024", "FIPS": "29047", "County": "Clay", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64048", "FIPS": "29047", "County": "Clay", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64060", "FIPS": "29047", "County": "Clay", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64068", "FIPS": "29047", "County": "Clay", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64069", "FIPS": "29047", "County": "Clay", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64072", "FIPS": "29047", "County": "Clay", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64073", "FIPS": "29047", "County": "Clay", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64089", "FIPS": "29047", "County": "Clay", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64116", "FIPS": "29047", "County": "Clay", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64117", "FIPS": "29047", "County": "Clay", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64118", "FIPS": "29047", "County": "Clay", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64119", "FIPS": "29047", "County": "Clay", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64144", "FIPS": "29047", "County": "Clay", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64155", "FIPS": "29047", "County": "Clay", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64156", "FIPS": "29047", "County": "Clay", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64157", "FIPS": "29047", "County": "Clay", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64158", "FIPS": "29047", "County": "Clay", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64161", "FIPS": "29047", "County": "Clay", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64165", "FIPS": "29047", "County": "Clay", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64166", "FIPS": "29047", "County": "Clay", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64167", "FIPS": "29047", "County": "Clay", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64188", "FIPS": "29047", "County": "Clay", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64429", "FIPS": "29049", "County": "Clinton", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64454", "FIPS": "29049", "County": "Clinton", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64465", "FIPS": "29049", "County": "Clinton", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64477", "FIPS": "29049", "County": "Clinton", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64492", "FIPS": "29049", "County": "Clinton", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64493", "FIPS": "29049", "County": "Clinton", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65023", "FIPS": "29051", "County": "Cole", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65032", "FIPS": "29051", "County": "Cole", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65040", "FIPS": "29051", "County": "Cole", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65053", "FIPS": "29051", "County": "Cole", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65074", "FIPS": "29051", "County": "Cole", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65076", "FIPS": "29051", "County": "Cole", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65101", "FIPS": "29051", "County": "Cole", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65102", "FIPS": "29051", "County": "Cole", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65103", "FIPS": "29051", "County": "Cole", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65104", "FIPS": "29051", "County": "Cole", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65105", "FIPS": "29051", "County": "Cole", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65106", "FIPS": "29051", "County": "Cole", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65107", "FIPS": "29051", "County": "Cole", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65108", "FIPS": "29051", "County": "Cole", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65109", "FIPS": "29051", "County": "Cole", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65110", "FIPS": "29051", "County": "Cole", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65111", "FIPS": "29051", "County": "Cole", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65068", "FIPS": "29053", "County": "Cooper", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65233", "FIPS": "29053", "County": "Cooper", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65237", "FIPS": "29053", "County": "Cooper", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65276", "FIPS": "29053", "County": "Cooper", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65287", "FIPS": "29053", "County": "Cooper", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65322", "FIPS": "29053", "County": "Cooper", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65348", "FIPS": "29053", "County": "Cooper", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65441", "FIPS": "29055", "County": "Crawford", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65446", "FIPS": "29055", "County": "Crawford", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65449", "FIPS": "29055", "County": "Crawford", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65453", "FIPS": "29055", "County": "Crawford", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65456", "FIPS": "29055", "County": "Crawford", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65535", "FIPS": "29055", "County": "Crawford", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65565", "FIPS": "29055", "County": "Crawford", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65586", "FIPS": "29055", "County": "Crawford", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65603", "FIPS": "29057", "County": "Dade", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65635", "FIPS": "29057", "County": "Dade", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65646", "FIPS": "29057", "County": "Dade", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65661", "FIPS": "29057", "County": "Dade", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65682", "FIPS": "29057", "County": "Dade", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65752", "FIPS": "29057", "County": "Dade", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65590", "FIPS": "29059", "County": "Dallas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65622", "FIPS": "29059", "County": "Dallas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65644", "FIPS": "29059", "County": "Dallas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65685", "FIPS": "29059", "County": "Dallas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65764", "FIPS": "29059", "County": "Dallas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65767", "FIPS": "29059", "County": "Dallas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65783", "FIPS": "29059", "County": "Dallas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "64620", "FIPS": "29061", "County": "Daviess", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64636", "FIPS": "29061", "County": "Daviess", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64640", "FIPS": "29061", "County": "Daviess", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64647", "FIPS": "29061", "County": "Daviess", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64648", "FIPS": "29061", "County": "Daviess", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64654", "FIPS": "29061", "County": "Daviess", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64670", "FIPS": "29061", "County": "Daviess", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64689", "FIPS": "29061", "County": "Daviess", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64422", "FIPS": "29063", "County": "Dekalb", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64430", "FIPS": "29063", "County": "Dekalb", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64447", "FIPS": "29063", "County": "Dekalb", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64469", "FIPS": "29063", "County": "Dekalb", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64474", "FIPS": "29063", "County": "Dekalb", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64490", "FIPS": "29063", "County": "Dekalb", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64494", "FIPS": "29063", "County": "Dekalb", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "64497", "FIPS": "29063", "County": "Dekalb", "State": "MO", "DMA": {"Code": 638, "Name": "SAINT JOSEPH"}},
  {"Postal_Code": "65440", "FIPS": "29065", "County": "Dent", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65501", "FIPS": "29065", "County": "Dent", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65532", "FIPS": "29065", "County": "Dent", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65541", "FIPS": "29065", "County": "Dent", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65560", "FIPS": "29065", "County": "Dent", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65608", "FIPS": "29067", "County": "Douglas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65638", "FIPS": "29067", "County": "Douglas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65701", "FIPS": "29067", "County": "Douglas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65755", "FIPS": "29067", "County": "Douglas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65768", "FIPS": "29067", "County": "Douglas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "63821", "FIPS": "29069", "County": "Dunklin", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63829", "FIPS": "29069", "County": "Dunklin", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63837", "FIPS": "29069", "County": "Dunklin", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63847", "FIPS": "29069", "County": "Dunklin", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63852", "FIPS": "29069", "County": "Dunklin", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63855", "FIPS": "29069", "County": "Dunklin", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63857", "FIPS": "29069", "County": "Dunklin", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63863", "FIPS": "29069", "County": "Dunklin", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63875", "FIPS": "29069", "County": "Dunklin", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63876", "FIPS": "29069", "County": "Dunklin", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63880", "FIPS": "29069", "County": "Dunklin", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63933", "FIPS": "29069", "County": "Dunklin", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63013", "FIPS": "29071", "County": "Franklin", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63014", "FIPS": "29071", "County": "Franklin", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63015", "FIPS": "29071", "County": "Franklin", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63037", "FIPS": "29071", "County": "Franklin", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63039", "FIPS": "29071", "County": "Franklin", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63041", "FIPS": "29071", "County": "Franklin", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63055", "FIPS": "29071", "County": "Franklin", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63056", "FIPS": "29071", "County": "Franklin", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63060", "FIPS": "29071", "County": "Franklin", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63061", "FIPS": "29071", "County": "Franklin", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63068", "FIPS": "29071", "County": "Franklin", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63069", "FIPS": "29071", "County": "Franklin", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63072", "FIPS": "29071", "County": "Franklin", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63073", "FIPS": "29071", "County": "Franklin", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63077", "FIPS": "29071", "County": "Franklin", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63079", "FIPS": "29071", "County": "Franklin", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63080", "FIPS": "29071", "County": "Franklin", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63084", "FIPS": "29071", "County": "Franklin", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63089", "FIPS": "29071", "County": "Franklin", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63090", "FIPS": "29071", "County": "Franklin", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63091", "FIPS": "29073", "County": "Gasconade", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65014", "FIPS": "29073", "County": "Gasconade", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65036", "FIPS": "29073", "County": "Gasconade", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65041", "FIPS": "29073", "County": "Gasconade", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65061", "FIPS": "29073", "County": "Gasconade", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65062", "FIPS": "29073", "County": "Gasconade", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65066", "FIPS": "29073", "County": "Gasconade", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "64402", "FIPS": "29075", "County": "Gentry", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64438", "FIPS": "29075", "County": "Gentry", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64453", "FIPS": "29075", "County": "Gentry", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64463", "FIPS": "29075", "County": "Gentry", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64489", "FIPS": "29075", "County": "Gentry", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64657", "FIPS": "29075", "County": "Gentry", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65604", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65612", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65619", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65648", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65725", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65738", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65757", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65765", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65770", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65781", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65801", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65802", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65803", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65804", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65805", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65806", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65807", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65808", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65809", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65810", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65814", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65817", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65890", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65897", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65898", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65899", "FIPS": "29077", "County": "Greene", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "64641", "FIPS": "29079", "County": "Grundy", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64652", "FIPS": "29079", "County": "Grundy", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64679", "FIPS": "29079", "County": "Grundy", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64683", "FIPS": "29079", "County": "Grundy", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64424", "FIPS": "29081", "County": "Harrison", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64426", "FIPS": "29081", "County": "Harrison", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64442", "FIPS": "29081", "County": "Harrison", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64458", "FIPS": "29081", "County": "Harrison", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64467", "FIPS": "29081", "County": "Harrison", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64471", "FIPS": "29081", "County": "Harrison", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64481", "FIPS": "29081", "County": "Harrison", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64632", "FIPS": "29081", "County": "Harrison", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64642", "FIPS": "29081", "County": "Harrison", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64726", "FIPS": "29083", "County": "Henry", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64735", "FIPS": "29083", "County": "Henry", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64739", "FIPS": "29083", "County": "Henry", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64740", "FIPS": "29083", "County": "Henry", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64770", "FIPS": "29083", "County": "Henry", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64788", "FIPS": "29083", "County": "Henry", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65323", "FIPS": "29083", "County": "Henry", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65360", "FIPS": "29083", "County": "Henry", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65634", "FIPS": "29085", "County": "Hickory", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65668", "FIPS": "29085", "County": "Hickory", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65724", "FIPS": "29085", "County": "Hickory", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65732", "FIPS": "29085", "County": "Hickory", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65735", "FIPS": "29085", "County": "Hickory", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65774", "FIPS": "29085", "County": "Hickory", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65779", "FIPS": "29085", "County": "Hickory", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "64437", "FIPS": "29087", "County": "Holt", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64451", "FIPS": "29087", "County": "Holt", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64466", "FIPS": "29087", "County": "Holt", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64470", "FIPS": "29087", "County": "Holt", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64473", "FIPS": "29087", "County": "Holt", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65230", "FIPS": "29089", "County": "Howard", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65248", "FIPS": "29089", "County": "Howard", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65250", "FIPS": "29089", "County": "Howard", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65254", "FIPS": "29089", "County": "Howard", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65274", "FIPS": "29089", "County": "Howard", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65548", "FIPS": "29091", "County": "Howell", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65626", "FIPS": "29091", "County": "Howell", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65688", "FIPS": "29091", "County": "Howell", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65689", "FIPS": "29091", "County": "Howell", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65775", "FIPS": "29091", "County": "Howell", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65776", "FIPS": "29091", "County": "Howell", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65777", "FIPS": "29091", "County": "Howell", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65788", "FIPS": "29091", "County": "Howell", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65789", "FIPS": "29091", "County": "Howell", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65790", "FIPS": "29091", "County": "Howell", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65793", "FIPS": "29091", "County": "Howell", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "63620", "FIPS": "29093", "County": "Iron", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63621", "FIPS": "29093", "County": "Iron", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63623", "FIPS": "29093", "County": "Iron", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63636", "FIPS": "29093", "County": "Iron", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63650", "FIPS": "29093", "County": "Iron", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63656", "FIPS": "29093", "County": "Iron", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63663", "FIPS": "29093", "County": "Iron", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63675", "FIPS": "29093", "County": "Iron", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65439", "FIPS": "29093", "County": "Iron", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65566", "FIPS": "29093", "County": "Iron", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "64013", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64014", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64015", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64016", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64029", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64030", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64034", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64050", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64051", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64052", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64053", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64054", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64055", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64056", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64057", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64058", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64063", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64064", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64065", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64066", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64070", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64075", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64081", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64082", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64086", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64088", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64101", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64102", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64105", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64106", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64108", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64109", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64110", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64111", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64112", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64113", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64114", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64120", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64121", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64123", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64124", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64125", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64126", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64127", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64128", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64129", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64130", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64131", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64132", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64133", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64134", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64136", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64137", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64138", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64139", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64141", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64145", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64146", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64147", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64148", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64149", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64170", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64171", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64172", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64179", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64180", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64183", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64184", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64185", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64187", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64191", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64192", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64193", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64194", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64196", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64197", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64198", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64199", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64944", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64999", "FIPS": "29095", "County": "Jackson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64755", "FIPS": "29097", "County": "Jasper", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64801", "FIPS": "29097", "County": "Jasper", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64802", "FIPS": "29097", "County": "Jasper", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64803", "FIPS": "29097", "County": "Jasper", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64804", "FIPS": "29097", "County": "Jasper", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64830", "FIPS": "29097", "County": "Jasper", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64833", "FIPS": "29097", "County": "Jasper", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64834", "FIPS": "29097", "County": "Jasper", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64835", "FIPS": "29097", "County": "Jasper", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64836", "FIPS": "29097", "County": "Jasper", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64841", "FIPS": "29097", "County": "Jasper", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64849", "FIPS": "29097", "County": "Jasper", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64855", "FIPS": "29097", "County": "Jasper", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64857", "FIPS": "29097", "County": "Jasper", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64859", "FIPS": "29097", "County": "Jasper", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64862", "FIPS": "29097", "County": "Jasper", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64869", "FIPS": "29097", "County": "Jasper", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64870", "FIPS": "29097", "County": "Jasper", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "63010", "FIPS": "29099", "County": "Jefferson", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63012", "FIPS": "29099", "County": "Jefferson", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63016", "FIPS": "29099", "County": "Jefferson", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63019", "FIPS": "29099", "County": "Jefferson", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63020", "FIPS": "29099", "County": "Jefferson", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63023", "FIPS": "29099", "County": "Jefferson", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63028", "FIPS": "29099", "County": "Jefferson", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63030", "FIPS": "29099", "County": "Jefferson", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63047", "FIPS": "29099", "County": "Jefferson", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63048", "FIPS": "29099", "County": "Jefferson", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63049", "FIPS": "29099", "County": "Jefferson", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63050", "FIPS": "29099", "County": "Jefferson", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63051", "FIPS": "29099", "County": "Jefferson", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63052", "FIPS": "29099", "County": "Jefferson", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63053", "FIPS": "29099", "County": "Jefferson", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63057", "FIPS": "29099", "County": "Jefferson", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63065", "FIPS": "29099", "County": "Jefferson", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63066", "FIPS": "29099", "County": "Jefferson", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63070", "FIPS": "29099", "County": "Jefferson", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "64019", "FIPS": "29101", "County": "Johnson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64040", "FIPS": "29101", "County": "Johnson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64061", "FIPS": "29101", "County": "Johnson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64093", "FIPS": "29101", "County": "Johnson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64733", "FIPS": "29101", "County": "Johnson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64761", "FIPS": "29101", "County": "Johnson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65305", "FIPS": "29101", "County": "Johnson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65336", "FIPS": "29101", "County": "Johnson", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "63446", "FIPS": "29103", "County": "Knox", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63458", "FIPS": "29103", "County": "Knox", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63460", "FIPS": "29103", "County": "Knox", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63464", "FIPS": "29103", "County": "Knox", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63531", "FIPS": "29103", "County": "Knox", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63537", "FIPS": "29103", "County": "Knox", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63547", "FIPS": "29103", "County": "Knox", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "65463", "FIPS": "29105", "County": "Laclede", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65470", "FIPS": "29105", "County": "Laclede", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65536", "FIPS": "29105", "County": "Laclede", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65543", "FIPS": "29105", "County": "Laclede", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65567", "FIPS": "29105", "County": "Laclede", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65632", "FIPS": "29105", "County": "Laclede", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65722", "FIPS": "29105", "County": "Laclede", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "64001", "FIPS": "29107", "County": "Lafayette", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64011", "FIPS": "29107", "County": "Lafayette", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64020", "FIPS": "29107", "County": "Lafayette", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64021", "FIPS": "29107", "County": "Lafayette", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64022", "FIPS": "29107", "County": "Lafayette", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64037", "FIPS": "29107", "County": "Lafayette", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64067", "FIPS": "29107", "County": "Lafayette", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64071", "FIPS": "29107", "County": "Lafayette", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64074", "FIPS": "29107", "County": "Lafayette", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64076", "FIPS": "29107", "County": "Lafayette", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64096", "FIPS": "29107", "County": "Lafayette", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64097", "FIPS": "29107", "County": "Lafayette", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65327", "FIPS": "29107", "County": "Lafayette", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64848", "FIPS": "29109", "County": "Lawrence", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "64873", "FIPS": "29109", "County": "Lawrence", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65605", "FIPS": "29109", "County": "Lawrence", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65654", "FIPS": "29109", "County": "Lawrence", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65664", "FIPS": "29109", "County": "Lawrence", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65705", "FIPS": "29109", "County": "Lawrence", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65707", "FIPS": "29109", "County": "Lawrence", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65712", "FIPS": "29109", "County": "Lawrence", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65723", "FIPS": "29109", "County": "Lawrence", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65756", "FIPS": "29109", "County": "Lawrence", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65769", "FIPS": "29109", "County": "Lawrence", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "63435", "FIPS": "29111", "County": "Lewis", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63438", "FIPS": "29111", "County": "Lewis", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63440", "FIPS": "29111", "County": "Lewis", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63447", "FIPS": "29111", "County": "Lewis", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63448", "FIPS": "29111", "County": "Lewis", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63452", "FIPS": "29111", "County": "Lewis", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63457", "FIPS": "29111", "County": "Lewis", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63473", "FIPS": "29111", "County": "Lewis", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63343", "FIPS": "29113", "County": "Lincoln", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63347", "FIPS": "29113", "County": "Lincoln", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63349", "FIPS": "29113", "County": "Lincoln", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63362", "FIPS": "29113", "County": "Lincoln", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63369", "FIPS": "29113", "County": "Lincoln", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63370", "FIPS": "29113", "County": "Lincoln", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63377", "FIPS": "29113", "County": "Lincoln", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63379", "FIPS": "29113", "County": "Lincoln", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63381", "FIPS": "29113", "County": "Lincoln", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63387", "FIPS": "29113", "County": "Lincoln", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63389", "FIPS": "29113", "County": "Lincoln", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63557", "FIPS": "29115", "County": "Linn", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64628", "FIPS": "29115", "County": "Linn", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64630", "FIPS": "29115", "County": "Linn", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64631", "FIPS": "29115", "County": "Linn", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64651", "FIPS": "29115", "County": "Linn", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64653", "FIPS": "29115", "County": "Linn", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64658", "FIPS": "29115", "County": "Linn", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64659", "FIPS": "29115", "County": "Linn", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64674", "FIPS": "29115", "County": "Linn", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64601", "FIPS": "29117", "County": "Livingston", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64635", "FIPS": "29117", "County": "Livingston", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64638", "FIPS": "29117", "County": "Livingston", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64656", "FIPS": "29117", "County": "Livingston", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64664", "FIPS": "29117", "County": "Livingston", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64686", "FIPS": "29117", "County": "Livingston", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64688", "FIPS": "29117", "County": "Livingston", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64831", "FIPS": "29119", "County": "McDonald", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64843", "FIPS": "29119", "County": "McDonald", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64847", "FIPS": "29119", "County": "McDonald", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64854", "FIPS": "29119", "County": "McDonald", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64856", "FIPS": "29119", "County": "McDonald", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64861", "FIPS": "29119", "County": "McDonald", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64863", "FIPS": "29119", "County": "McDonald", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64868", "FIPS": "29119", "County": "McDonald", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "65730", "FIPS": "29119", "County": "McDonald", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "63431", "FIPS": "29121", "County": "Macon", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63530", "FIPS": "29121", "County": "Macon", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63532", "FIPS": "29121", "County": "Macon", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63534", "FIPS": "29121", "County": "Macon", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63538", "FIPS": "29121", "County": "Macon", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63539", "FIPS": "29121", "County": "Macon", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63549", "FIPS": "29121", "County": "Macon", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63552", "FIPS": "29121", "County": "Macon", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63558", "FIPS": "29121", "County": "Macon", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "65247", "FIPS": "29121", "County": "Macon", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63645", "FIPS": "29123", "County": "Madison", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63655", "FIPS": "29123", "County": "Madison", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "65013", "FIPS": "29125", "County": "Maries", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65443", "FIPS": "29125", "County": "Maries", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65580", "FIPS": "29125", "County": "Maries", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65582", "FIPS": "29125", "County": "Maries", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "63401", "FIPS": "29127", "County": "Marion", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63454", "FIPS": "29127", "County": "Marion", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63461", "FIPS": "29127", "County": "Marion", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63463", "FIPS": "29127", "County": "Marion", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63471", "FIPS": "29127", "County": "Marion", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "64661", "FIPS": "29129", "County": "Mercer", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64673", "FIPS": "29129", "County": "Mercer", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65017", "FIPS": "29131", "County": "Miller", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65026", "FIPS": "29131", "County": "Miller", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65047", "FIPS": "29131", "County": "Miller", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65064", "FIPS": "29131", "County": "Miller", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65075", "FIPS": "29131", "County": "Miller", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65082", "FIPS": "29131", "County": "Miller", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65083", "FIPS": "29131", "County": "Miller", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65486", "FIPS": "29131", "County": "Miller", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "63820", "FIPS": "29133", "County": "Mississippi", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63823", "FIPS": "29133", "County": "Mississippi", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63834", "FIPS": "29133", "County": "Mississippi", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63845", "FIPS": "29133", "County": "Mississippi", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63881", "FIPS": "29133", "County": "Mississippi", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63882", "FIPS": "29133", "County": "Mississippi", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "65018", "FIPS": "29135", "County": "Moniteau", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65025", "FIPS": "29135", "County": "Moniteau", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65034", "FIPS": "29135", "County": "Moniteau", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65042", "FIPS": "29135", "County": "Moniteau", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65046", "FIPS": "29135", "County": "Moniteau", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65050", "FIPS": "29135", "County": "Moniteau", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65055", "FIPS": "29135", "County": "Moniteau", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65081", "FIPS": "29135", "County": "Moniteau", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "63456", "FIPS": "29137", "County": "Monroe", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "65258", "FIPS": "29137", "County": "Monroe", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "65263", "FIPS": "29137", "County": "Monroe", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "65275", "FIPS": "29137", "County": "Monroe", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "65282", "FIPS": "29137", "County": "Monroe", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "65283", "FIPS": "29137", "County": "Monroe", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63333", "FIPS": "29139", "County": "Montgomery", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "63350", "FIPS": "29139", "County": "Montgomery", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "63351", "FIPS": "29139", "County": "Montgomery", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "63359", "FIPS": "29139", "County": "Montgomery", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "63361", "FIPS": "29139", "County": "Montgomery", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "63363", "FIPS": "29139", "County": "Montgomery", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "63384", "FIPS": "29139", "County": "Montgomery", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65069", "FIPS": "29139", "County": "Montgomery", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65011", "FIPS": "29141", "County": "Morgan", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65037", "FIPS": "29141", "County": "Morgan", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65038", "FIPS": "29141", "County": "Morgan", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65072", "FIPS": "29141", "County": "Morgan", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65078", "FIPS": "29141", "County": "Morgan", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65084", "FIPS": "29141", "County": "Morgan", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65329", "FIPS": "29141", "County": "Morgan", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65354", "FIPS": "29141", "County": "Morgan", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "63828", "FIPS": "29143", "County": "New Madrid", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63833", "FIPS": "29143", "County": "New Madrid", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63848", "FIPS": "29143", "County": "New Madrid", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63860", "FIPS": "29143", "County": "New Madrid", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63862", "FIPS": "29143", "County": "New Madrid", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63866", "FIPS": "29143", "County": "New Madrid", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63867", "FIPS": "29143", "County": "New Madrid", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63868", "FIPS": "29143", "County": "New Madrid", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63869", "FIPS": "29143", "County": "New Madrid", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63870", "FIPS": "29143", "County": "New Madrid", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63873", "FIPS": "29143", "County": "New Madrid", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63874", "FIPS": "29143", "County": "New Madrid", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63878", "FIPS": "29143", "County": "New Madrid", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "64840", "FIPS": "29145", "County": "Newton", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64842", "FIPS": "29145", "County": "Newton", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64844", "FIPS": "29145", "County": "Newton", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64850", "FIPS": "29145", "County": "Newton", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64853", "FIPS": "29145", "County": "Newton", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64858", "FIPS": "29145", "County": "Newton", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64864", "FIPS": "29145", "County": "Newton", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64865", "FIPS": "29145", "County": "Newton", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64866", "FIPS": "29145", "County": "Newton", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64867", "FIPS": "29145", "County": "Newton", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64423", "FIPS": "29147", "County": "Nodaway", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64428", "FIPS": "29147", "County": "Nodaway", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64431", "FIPS": "29147", "County": "Nodaway", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64432", "FIPS": "29147", "County": "Nodaway", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64433", "FIPS": "29147", "County": "Nodaway", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64434", "FIPS": "29147", "County": "Nodaway", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64445", "FIPS": "29147", "County": "Nodaway", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64455", "FIPS": "29147", "County": "Nodaway", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64457", "FIPS": "29147", "County": "Nodaway", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64461", "FIPS": "29147", "County": "Nodaway", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64468", "FIPS": "29147", "County": "Nodaway", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64475", "FIPS": "29147", "County": "Nodaway", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64476", "FIPS": "29147", "County": "Nodaway", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64479", "FIPS": "29147", "County": "Nodaway", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64487", "FIPS": "29147", "County": "Nodaway", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65606", "FIPS": "29149", "County": "Oregon", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65690", "FIPS": "29149", "County": "Oregon", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65692", "FIPS": "29149", "County": "Oregon", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65778", "FIPS": "29149", "County": "Oregon", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65791", "FIPS": "29149", "County": "Oregon", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65001", "FIPS": "29151", "County": "Osage", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65016", "FIPS": "29151", "County": "Osage", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65024", "FIPS": "29151", "County": "Osage", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65035", "FIPS": "29151", "County": "Osage", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65048", "FIPS": "29151", "County": "Osage", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65051", "FIPS": "29151", "County": "Osage", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65054", "FIPS": "29151", "County": "Osage", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65058", "FIPS": "29151", "County": "Osage", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65085", "FIPS": "29151", "County": "Osage", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65609", "FIPS": "29153", "County": "Ozark", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65618", "FIPS": "29153", "County": "Ozark", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65637", "FIPS": "29153", "County": "Ozark", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65655", "FIPS": "29153", "County": "Ozark", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65666", "FIPS": "29153", "County": "Ozark", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65676", "FIPS": "29153", "County": "Ozark", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65715", "FIPS": "29153", "County": "Ozark", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65729", "FIPS": "29153", "County": "Ozark", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65741", "FIPS": "29153", "County": "Ozark", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65760", "FIPS": "29153", "County": "Ozark", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65761", "FIPS": "29153", "County": "Ozark", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65762", "FIPS": "29153", "County": "Ozark", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65766", "FIPS": "29153", "County": "Ozark", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65773", "FIPS": "29153", "County": "Ozark", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65784", "FIPS": "29153", "County": "Ozark", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "63826", "FIPS": "29155", "County": "Pemiscot", "State": "MO", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "63827", "FIPS": "29155", "County": "Pemiscot", "State": "MO", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "63830", "FIPS": "29155", "County": "Pemiscot", "State": "MO", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "63839", "FIPS": "29155", "County": "Pemiscot", "State": "MO", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "63840", "FIPS": "29155", "County": "Pemiscot", "State": "MO", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "63849", "FIPS": "29155", "County": "Pemiscot", "State": "MO", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "63851", "FIPS": "29155", "County": "Pemiscot", "State": "MO", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "63853", "FIPS": "29155", "County": "Pemiscot", "State": "MO", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "63877", "FIPS": "29155", "County": "Pemiscot", "State": "MO", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "63879", "FIPS": "29155", "County": "Pemiscot", "State": "MO", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "63737", "FIPS": "29157", "County": "Perry", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63746", "FIPS": "29157", "County": "Perry", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63748", "FIPS": "29157", "County": "Perry", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63775", "FIPS": "29157", "County": "Perry", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63776", "FIPS": "29157", "County": "Perry", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63783", "FIPS": "29157", "County": "Perry", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "65301", "FIPS": "29159", "County": "Pettis", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65302", "FIPS": "29159", "County": "Pettis", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65332", "FIPS": "29159", "County": "Pettis", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65333", "FIPS": "29159", "County": "Pettis", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65334", "FIPS": "29159", "County": "Pettis", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65337", "FIPS": "29159", "County": "Pettis", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65345", "FIPS": "29159", "County": "Pettis", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65350", "FIPS": "29159", "County": "Pettis", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65401", "FIPS": "29161", "County": "Phelps", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65402", "FIPS": "29161", "County": "Phelps", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65409", "FIPS": "29161", "County": "Phelps", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65436", "FIPS": "29161", "County": "Phelps", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65461", "FIPS": "29161", "County": "Phelps", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65462", "FIPS": "29161", "County": "Phelps", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65529", "FIPS": "29161", "County": "Phelps", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65550", "FIPS": "29161", "County": "Phelps", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65559", "FIPS": "29161", "County": "Phelps", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63330", "FIPS": "29163", "County": "Pike", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63334", "FIPS": "29163", "County": "Pike", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63336", "FIPS": "29163", "County": "Pike", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63339", "FIPS": "29163", "County": "Pike", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63344", "FIPS": "29163", "County": "Pike", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63353", "FIPS": "29163", "County": "Pike", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63433", "FIPS": "29163", "County": "Pike", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63441", "FIPS": "29163", "County": "Pike", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "64018", "FIPS": "29165", "County": "Platte", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64028", "FIPS": "29165", "County": "Platte", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64079", "FIPS": "29165", "County": "Platte", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64092", "FIPS": "29165", "County": "Platte", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64098", "FIPS": "29165", "County": "Platte", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64150", "FIPS": "29165", "County": "Platte", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64151", "FIPS": "29165", "County": "Platte", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64152", "FIPS": "29165", "County": "Platte", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64153", "FIPS": "29165", "County": "Platte", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64154", "FIPS": "29165", "County": "Platte", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64163", "FIPS": "29165", "County": "Platte", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64164", "FIPS": "29165", "County": "Platte", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64168", "FIPS": "29165", "County": "Platte", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64190", "FIPS": "29165", "County": "Platte", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64195", "FIPS": "29165", "County": "Platte", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64439", "FIPS": "29165", "County": "Platte", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64444", "FIPS": "29165", "County": "Platte", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65601", "FIPS": "29167", "County": "Polk", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65613", "FIPS": "29167", "County": "Polk", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65617", "FIPS": "29167", "County": "Polk", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65640", "FIPS": "29167", "County": "Polk", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65645", "FIPS": "29167", "County": "Polk", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65649", "FIPS": "29167", "County": "Polk", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65650", "FIPS": "29167", "County": "Polk", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65663", "FIPS": "29167", "County": "Polk", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65674", "FIPS": "29167", "County": "Polk", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65710", "FIPS": "29167", "County": "Polk", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65727", "FIPS": "29167", "County": "Polk", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65452", "FIPS": "29169", "County": "Pulaski", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65457", "FIPS": "29169", "County": "Pulaski", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65459", "FIPS": "29169", "County": "Pulaski", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65473", "FIPS": "29169", "County": "Pulaski", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65534", "FIPS": "29169", "County": "Pulaski", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65556", "FIPS": "29169", "County": "Pulaski", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65583", "FIPS": "29169", "County": "Pulaski", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65584", "FIPS": "29169", "County": "Pulaski", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "63551", "FIPS": "29171", "County": "Putnam", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63565", "FIPS": "29171", "County": "Putnam", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63567", "FIPS": "29171", "County": "Putnam", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "64655", "FIPS": "29171", "County": "Putnam", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "64672", "FIPS": "29171", "County": "Putnam", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63436", "FIPS": "29173", "County": "Ralls", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63459", "FIPS": "29173", "County": "Ralls", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63462", "FIPS": "29173", "County": "Ralls", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63467", "FIPS": "29173", "County": "Ralls", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "65239", "FIPS": "29175", "County": "Randolph", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65243", "FIPS": "29175", "County": "Randolph", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65244", "FIPS": "29175", "County": "Randolph", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65257", "FIPS": "29175", "County": "Randolph", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65259", "FIPS": "29175", "County": "Randolph", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65260", "FIPS": "29175", "County": "Randolph", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65270", "FIPS": "29175", "County": "Randolph", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "65278", "FIPS": "29175", "County": "Randolph", "State": "MO", "DMA": {"Code": 604, "Name": "COLUMBIA-JEFFERSON CITY"}},
  {"Postal_Code": "64017", "FIPS": "29177", "County": "Ray", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64035", "FIPS": "29177", "County": "Ray", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64036", "FIPS": "29177", "County": "Ray", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64062", "FIPS": "29177", "County": "Ray", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64077", "FIPS": "29177", "County": "Ray", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64084", "FIPS": "29177", "County": "Ray", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64085", "FIPS": "29177", "County": "Ray", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "63625", "FIPS": "29179", "County": "Reynolds", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63629", "FIPS": "29179", "County": "Reynolds", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63633", "FIPS": "29179", "County": "Reynolds", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63638", "FIPS": "29179", "County": "Reynolds", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63654", "FIPS": "29179", "County": "Reynolds", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63665", "FIPS": "29179", "County": "Reynolds", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63666", "FIPS": "29179", "County": "Reynolds", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63931", "FIPS": "29181", "County": "Ripley", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63935", "FIPS": "29181", "County": "Ripley", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63939", "FIPS": "29181", "County": "Ripley", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63942", "FIPS": "29181", "County": "Ripley", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63953", "FIPS": "29181", "County": "Ripley", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63955", "FIPS": "29181", "County": "Ripley", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63301", "FIPS": "29183", "County": "Saint Charles", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63302", "FIPS": "29183", "County": "Saint Charles", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63303", "FIPS": "29183", "County": "Saint Charles", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63304", "FIPS": "29183", "County": "Saint Charles", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63332", "FIPS": "29183", "County": "Saint Charles", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63338", "FIPS": "29183", "County": "Saint Charles", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63341", "FIPS": "29183", "County": "Saint Charles", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63346", "FIPS": "29183", "County": "Saint Charles", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63348", "FIPS": "29183", "County": "Saint Charles", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63365", "FIPS": "29183", "County": "Saint Charles", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63366", "FIPS": "29183", "County": "Saint Charles", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63367", "FIPS": "29183", "County": "Saint Charles", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63368", "FIPS": "29183", "County": "Saint Charles", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63373", "FIPS": "29183", "County": "Saint Charles", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63376", "FIPS": "29183", "County": "Saint Charles", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63385", "FIPS": "29183", "County": "Saint Charles", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63386", "FIPS": "29183", "County": "Saint Charles", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "64724", "FIPS": "29185", "County": "Saint Clair", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "64738", "FIPS": "29185", "County": "Saint Clair", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "64763", "FIPS": "29185", "County": "Saint Clair", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "64776", "FIPS": "29185", "County": "Saint Clair", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "64781", "FIPS": "29185", "County": "Saint Clair", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "64789", "FIPS": "29185", "County": "Saint Clair", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "63627", "FIPS": "29186", "County": "Sainte Genevieve", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63670", "FIPS": "29186", "County": "Sainte Genevieve", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63673", "FIPS": "29186", "County": "Sainte Genevieve", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63036", "FIPS": "29187", "County": "Saint Francois", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63087", "FIPS": "29187", "County": "Saint Francois", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63601", "FIPS": "29187", "County": "Saint Francois", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63624", "FIPS": "29187", "County": "Saint Francois", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63626", "FIPS": "29187", "County": "Saint Francois", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63628", "FIPS": "29187", "County": "Saint Francois", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63637", "FIPS": "29187", "County": "Saint Francois", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63640", "FIPS": "29187", "County": "Saint Francois", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63651", "FIPS": "29187", "County": "Saint Francois", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63653", "FIPS": "29187", "County": "Saint Francois", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63001", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63005", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63006", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63011", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63017", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63021", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63022", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63024", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63025", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63026", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63031", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63032", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63033", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63034", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63038", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63040", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63042", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63043", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63044", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63045", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63074", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63088", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63099", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63105", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63114", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63117", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63119", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63121", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63122", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63123", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63124", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63125", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63126", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63127", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63128", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63129", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63130", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63131", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63132", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63133", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63134", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63135", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63136", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63137", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63138", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63140", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63141", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63143", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63144", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63145", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63146", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63151", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63167", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63198", "FIPS": "29189", "County": "Saint Louis", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "65320", "FIPS": "29195", "County": "Saline", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65321", "FIPS": "29195", "County": "Saline", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65330", "FIPS": "29195", "County": "Saline", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65339", "FIPS": "29195", "County": "Saline", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65340", "FIPS": "29195", "County": "Saline", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65344", "FIPS": "29195", "County": "Saline", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65347", "FIPS": "29195", "County": "Saline", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65349", "FIPS": "29195", "County": "Saline", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65351", "FIPS": "29195", "County": "Saline", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "63535", "FIPS": "29197", "County": "Schuyler", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63536", "FIPS": "29197", "County": "Schuyler", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63541", "FIPS": "29197", "County": "Schuyler", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63548", "FIPS": "29197", "County": "Schuyler", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63561", "FIPS": "29197", "County": "Schuyler", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63432", "FIPS": "29199", "County": "Scotland", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63442", "FIPS": "29199", "County": "Scotland", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63543", "FIPS": "29199", "County": "Scotland", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63555", "FIPS": "29199", "County": "Scotland", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63563", "FIPS": "29199", "County": "Scotland", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63736", "FIPS": "29201", "County": "Scott", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63740", "FIPS": "29201", "County": "Scott", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63742", "FIPS": "29201", "County": "Scott", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63758", "FIPS": "29201", "County": "Scott", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63767", "FIPS": "29201", "County": "Scott", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63771", "FIPS": "29201", "County": "Scott", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63774", "FIPS": "29201", "County": "Scott", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63780", "FIPS": "29201", "County": "Scott", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63784", "FIPS": "29201", "County": "Scott", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63801", "FIPS": "29201", "County": "Scott", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63824", "FIPS": "29201", "County": "Scott", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "65438", "FIPS": "29203", "County": "Shannon", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65466", "FIPS": "29203", "County": "Shannon", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65546", "FIPS": "29203", "County": "Shannon", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65588", "FIPS": "29203", "County": "Shannon", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "63434", "FIPS": "29205", "County": "Shelby", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63437", "FIPS": "29205", "County": "Shelby", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63439", "FIPS": "29205", "County": "Shelby", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63443", "FIPS": "29205", "County": "Shelby", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63450", "FIPS": "29205", "County": "Shelby", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63451", "FIPS": "29205", "County": "Shelby", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63468", "FIPS": "29205", "County": "Shelby", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63469", "FIPS": "29205", "County": "Shelby", "State": "MO", "DMA": {"Code": 717, "Name": "QUINCY-HANNIBAL-KEOKUK"}},
  {"Postal_Code": "63730", "FIPS": "29207", "County": "Stoddard", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63735", "FIPS": "29207", "County": "Stoddard", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63738", "FIPS": "29207", "County": "Stoddard", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63822", "FIPS": "29207", "County": "Stoddard", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63825", "FIPS": "29207", "County": "Stoddard", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63841", "FIPS": "29207", "County": "Stoddard", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63846", "FIPS": "29207", "County": "Stoddard", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63850", "FIPS": "29207", "County": "Stoddard", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63936", "FIPS": "29207", "County": "Stoddard", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63960", "FIPS": "29207", "County": "Stoddard", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "65611", "FIPS": "29209", "County": "Stone", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65624", "FIPS": "29209", "County": "Stone", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65633", "FIPS": "29209", "County": "Stone", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65656", "FIPS": "29209", "County": "Stone", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65675", "FIPS": "29209", "County": "Stone", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65681", "FIPS": "29209", "County": "Stone", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65686", "FIPS": "29209", "County": "Stone", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65728", "FIPS": "29209", "County": "Stone", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65737", "FIPS": "29209", "County": "Stone", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "63544", "FIPS": "29211", "County": "Sullivan", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63545", "FIPS": "29211", "County": "Sullivan", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63556", "FIPS": "29211", "County": "Sullivan", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63560", "FIPS": "29211", "County": "Sullivan", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "63566", "FIPS": "29211", "County": "Sullivan", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "64645", "FIPS": "29211", "County": "Sullivan", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "64646", "FIPS": "29211", "County": "Sullivan", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "64667", "FIPS": "29211", "County": "Sullivan", "State": "MO", "DMA": {"Code": 631, "Name": "OTTUMWA-KIRKSVILLE"}},
  {"Postal_Code": "65614", "FIPS": "29213", "County": "Taney", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65615", "FIPS": "29213", "County": "Taney", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65616", "FIPS": "29213", "County": "Taney", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65627", "FIPS": "29213", "County": "Taney", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65653", "FIPS": "29213", "County": "Taney", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65672", "FIPS": "29213", "County": "Taney", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65673", "FIPS": "29213", "County": "Taney", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65679", "FIPS": "29213", "County": "Taney", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65680", "FIPS": "29213", "County": "Taney", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65726", "FIPS": "29213", "County": "Taney", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65731", "FIPS": "29213", "County": "Taney", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65733", "FIPS": "29213", "County": "Taney", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65739", "FIPS": "29213", "County": "Taney", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65740", "FIPS": "29213", "County": "Taney", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65744", "FIPS": "29213", "County": "Taney", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65759", "FIPS": "29213", "County": "Taney", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65771", "FIPS": "29213", "County": "Taney", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65444", "FIPS": "29215", "County": "Texas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65464", "FIPS": "29215", "County": "Texas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65468", "FIPS": "29215", "County": "Texas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65479", "FIPS": "29215", "County": "Texas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65483", "FIPS": "29215", "County": "Texas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65484", "FIPS": "29215", "County": "Texas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65542", "FIPS": "29215", "County": "Texas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65552", "FIPS": "29215", "County": "Texas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65555", "FIPS": "29215", "County": "Texas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65557", "FIPS": "29215", "County": "Texas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65564", "FIPS": "29215", "County": "Texas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65570", "FIPS": "29215", "County": "Texas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65571", "FIPS": "29215", "County": "Texas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65589", "FIPS": "29215", "County": "Texas", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "64728", "FIPS": "29217", "County": "Vernon", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64741", "FIPS": "29217", "County": "Vernon", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64750", "FIPS": "29217", "County": "Vernon", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64765", "FIPS": "29217", "County": "Vernon", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64767", "FIPS": "29217", "County": "Vernon", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64771", "FIPS": "29217", "County": "Vernon", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64772", "FIPS": "29217", "County": "Vernon", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64778", "FIPS": "29217", "County": "Vernon", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64783", "FIPS": "29217", "County": "Vernon", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64784", "FIPS": "29217", "County": "Vernon", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "64790", "FIPS": "29217", "County": "Vernon", "State": "MO", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "63342", "FIPS": "29219", "County": "Warren", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63357", "FIPS": "29219", "County": "Warren", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63378", "FIPS": "29219", "County": "Warren", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63383", "FIPS": "29219", "County": "Warren", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63390", "FIPS": "29219", "County": "Warren", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63071", "FIPS": "29221", "County": "Washington", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63622", "FIPS": "29221", "County": "Washington", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63630", "FIPS": "29221", "County": "Washington", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63631", "FIPS": "29221", "County": "Washington", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63648", "FIPS": "29221", "County": "Washington", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63660", "FIPS": "29221", "County": "Washington", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63664", "FIPS": "29221", "County": "Washington", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63674", "FIPS": "29221", "County": "Washington", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63632", "FIPS": "29223", "County": "Wayne", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63763", "FIPS": "29223", "County": "Wayne", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63934", "FIPS": "29223", "County": "Wayne", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63944", "FIPS": "29223", "County": "Wayne", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63950", "FIPS": "29223", "County": "Wayne", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63951", "FIPS": "29223", "County": "Wayne", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63952", "FIPS": "29223", "County": "Wayne", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63956", "FIPS": "29223", "County": "Wayne", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63957", "FIPS": "29223", "County": "Wayne", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63963", "FIPS": "29223", "County": "Wayne", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63964", "FIPS": "29223", "County": "Wayne", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63966", "FIPS": "29223", "County": "Wayne", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "63967", "FIPS": "29223", "County": "Wayne", "State": "MO", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "65636", "FIPS": "29225", "County": "Webster", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65652", "FIPS": "29225", "County": "Webster", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65706", "FIPS": "29225", "County": "Webster", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65713", "FIPS": "29225", "County": "Webster", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65742", "FIPS": "29225", "County": "Webster", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65746", "FIPS": "29225", "County": "Webster", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "64420", "FIPS": "29227", "County": "Worth", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64441", "FIPS": "29227", "County": "Worth", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64456", "FIPS": "29227", "County": "Worth", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64486", "FIPS": "29227", "County": "Worth", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "64499", "FIPS": "29227", "County": "Worth", "State": "MO", "DMA": {"Code": 616, "Name": "KANSAS CITY"}},
  {"Postal_Code": "65660", "FIPS": "29229", "County": "Wright", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65662", "FIPS": "29229", "County": "Wright", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65667", "FIPS": "29229", "County": "Wright", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65702", "FIPS": "29229", "County": "Wright", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65704", "FIPS": "29229", "County": "Wright", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65711", "FIPS": "29229", "County": "Wright", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "65717", "FIPS": "29229", "County": "Wright", "State": "MO", "DMA": {"Code": 619, "Name": "SPRINGFIELD, MO"}},
  {"Postal_Code": "63101", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63102", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63103", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63104", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63106", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63107", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63108", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63109", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63110", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63111", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63112", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63113", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63115", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63116", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63118", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63120", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63139", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63147", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63150", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63155", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63156", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63157", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63158", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63160", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63163", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63164", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63166", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63169", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63171", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63177", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63178", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63179", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63180", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63182", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63188", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63190", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63195", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63196", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63197", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "63199", "FIPS": "29510", "County": "Saint Louis City", "State": "MO", "DMA": {"Code": 609, "Name": "SAINT LOUIS"}},
  {"Postal_Code": "59724", "FIPS": "30001", "County": "Beaverhead", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59725", "FIPS": "30001", "County": "Beaverhead", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59732", "FIPS": "30001", "County": "Beaverhead", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59736", "FIPS": "30001", "County": "Beaverhead", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59739", "FIPS": "30001", "County": "Beaverhead", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59746", "FIPS": "30001", "County": "Beaverhead", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59761", "FIPS": "30001", "County": "Beaverhead", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59762", "FIPS": "30001", "County": "Beaverhead", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59016", "FIPS": "30003", "County": "Big Horn", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59022", "FIPS": "30003", "County": "Big Horn", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59025", "FIPS": "30003", "County": "Big Horn", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59031", "FIPS": "30003", "County": "Big Horn", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59034", "FIPS": "30003", "County": "Big Horn", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59035", "FIPS": "30003", "County": "Big Horn", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59050", "FIPS": "30003", "County": "Big Horn", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59066", "FIPS": "30003", "County": "Big Horn", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59075", "FIPS": "30003", "County": "Big Horn", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59089", "FIPS": "30003", "County": "Big Horn", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59523", "FIPS": "30005", "County": "Blaine", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59526", "FIPS": "30005", "County": "Blaine", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59527", "FIPS": "30005", "County": "Blaine", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59529", "FIPS": "30005", "County": "Blaine", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59535", "FIPS": "30005", "County": "Blaine", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59542", "FIPS": "30005", "County": "Blaine", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59547", "FIPS": "30005", "County": "Blaine", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59641", "FIPS": "30007", "County": "Broadwater", "State": "MT", "DMA": {"Code": 766, "Name": "HELENA"}},
  {"Postal_Code": "59643", "FIPS": "30007", "County": "Broadwater", "State": "MT", "DMA": {"Code": 766, "Name": "HELENA"}},
  {"Postal_Code": "59644", "FIPS": "30007", "County": "Broadwater", "State": "MT", "DMA": {"Code": 766, "Name": "HELENA"}},
  {"Postal_Code": "59647", "FIPS": "30007", "County": "Broadwater", "State": "MT", "DMA": {"Code": 766, "Name": "HELENA"}},
  {"Postal_Code": "59007", "FIPS": "30009", "County": "Carbon", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59008", "FIPS": "30009", "County": "Carbon", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59013", "FIPS": "30009", "County": "Carbon", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59014", "FIPS": "30009", "County": "Carbon", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59026", "FIPS": "30009", "County": "Carbon", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59029", "FIPS": "30009", "County": "Carbon", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59041", "FIPS": "30009", "County": "Carbon", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59068", "FIPS": "30009", "County": "Carbon", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59070", "FIPS": "30009", "County": "Carbon", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59071", "FIPS": "30009", "County": "Carbon", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59311", "FIPS": "30011", "County": "Carter", "State": "MT", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "59316", "FIPS": "30011", "County": "Carter", "State": "MT", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "59319", "FIPS": "30011", "County": "Carter", "State": "MT", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "59324", "FIPS": "30011", "County": "Carter", "State": "MT", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "59332", "FIPS": "30011", "County": "Carter", "State": "MT", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "59401", "FIPS": "30013", "County": "Cascade", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59402", "FIPS": "30013", "County": "Cascade", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59403", "FIPS": "30013", "County": "Cascade", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59404", "FIPS": "30013", "County": "Cascade", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59405", "FIPS": "30013", "County": "Cascade", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59406", "FIPS": "30013", "County": "Cascade", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59412", "FIPS": "30013", "County": "Cascade", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59414", "FIPS": "30013", "County": "Cascade", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59421", "FIPS": "30013", "County": "Cascade", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59443", "FIPS": "30013", "County": "Cascade", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59463", "FIPS": "30013", "County": "Cascade", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59465", "FIPS": "30013", "County": "Cascade", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59472", "FIPS": "30013", "County": "Cascade", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59477", "FIPS": "30013", "County": "Cascade", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59480", "FIPS": "30013", "County": "Cascade", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59483", "FIPS": "30013", "County": "Cascade", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59485", "FIPS": "30013", "County": "Cascade", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59487", "FIPS": "30013", "County": "Cascade", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59420", "FIPS": "30015", "County": "Chouteau", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59440", "FIPS": "30015", "County": "Chouteau", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59442", "FIPS": "30015", "County": "Chouteau", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59446", "FIPS": "30015", "County": "Chouteau", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59450", "FIPS": "30015", "County": "Chouteau", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59460", "FIPS": "30015", "County": "Chouteau", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59520", "FIPS": "30015", "County": "Chouteau", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59301", "FIPS": "30017", "County": "Custer", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59336", "FIPS": "30017", "County": "Custer", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59338", "FIPS": "30017", "County": "Custer", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59351", "FIPS": "30017", "County": "Custer", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59222", "FIPS": "30019", "County": "Daniels", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59253", "FIPS": "30019", "County": "Daniels", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59263", "FIPS": "30019", "County": "Daniels", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59276", "FIPS": "30019", "County": "Daniels", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59259", "FIPS": "30021", "County": "Dawson", "State": "MT", "DMA": {"Code": 798, "Name": "GLENDIVE"}},
  {"Postal_Code": "59315", "FIPS": "30021", "County": "Dawson", "State": "MT", "DMA": {"Code": 798, "Name": "GLENDIVE"}},
  {"Postal_Code": "59330", "FIPS": "30021", "County": "Dawson", "State": "MT", "DMA": {"Code": 798, "Name": "GLENDIVE"}},
  {"Postal_Code": "59339", "FIPS": "30021", "County": "Dawson", "State": "MT", "DMA": {"Code": 798, "Name": "GLENDIVE"}},
  {"Postal_Code": "59711", "FIPS": "30023", "County": "Deer Lodge", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59756", "FIPS": "30023", "County": "Deer Lodge", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59313", "FIPS": "30025", "County": "Fallon", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59344", "FIPS": "30025", "County": "Fallon", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59354", "FIPS": "30025", "County": "Fallon", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59032", "FIPS": "30027", "County": "Fergus", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59418", "FIPS": "30027", "County": "Fergus", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59424", "FIPS": "30027", "County": "Fergus", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59430", "FIPS": "30027", "County": "Fergus", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59441", "FIPS": "30027", "County": "Fergus", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59451", "FIPS": "30027", "County": "Fergus", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59457", "FIPS": "30027", "County": "Fergus", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59464", "FIPS": "30027", "County": "Fergus", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59471", "FIPS": "30027", "County": "Fergus", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59489", "FIPS": "30027", "County": "Fergus", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59901", "FIPS": "30029", "County": "Flathead", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59903", "FIPS": "30029", "County": "Flathead", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59904", "FIPS": "30029", "County": "Flathead", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59911", "FIPS": "30029", "County": "Flathead", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59912", "FIPS": "30029", "County": "Flathead", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59913", "FIPS": "30029", "County": "Flathead", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59916", "FIPS": "30029", "County": "Flathead", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59919", "FIPS": "30029", "County": "Flathead", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59920", "FIPS": "30029", "County": "Flathead", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59921", "FIPS": "30029", "County": "Flathead", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59922", "FIPS": "30029", "County": "Flathead", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59925", "FIPS": "30029", "County": "Flathead", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59926", "FIPS": "30029", "County": "Flathead", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59927", "FIPS": "30029", "County": "Flathead", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59928", "FIPS": "30029", "County": "Flathead", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59932", "FIPS": "30029", "County": "Flathead", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59936", "FIPS": "30029", "County": "Flathead", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59937", "FIPS": "30029", "County": "Flathead", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59714", "FIPS": "30031", "County": "Gallatin", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59715", "FIPS": "30031", "County": "Gallatin", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59716", "FIPS": "30031", "County": "Gallatin", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59717", "FIPS": "30031", "County": "Gallatin", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59718", "FIPS": "30031", "County": "Gallatin", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59719", "FIPS": "30031", "County": "Gallatin", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59730", "FIPS": "30031", "County": "Gallatin", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59741", "FIPS": "30031", "County": "Gallatin", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59752", "FIPS": "30031", "County": "Gallatin", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59758", "FIPS": "30031", "County": "Gallatin", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59760", "FIPS": "30031", "County": "Gallatin", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59771", "FIPS": "30031", "County": "Gallatin", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59772", "FIPS": "30031", "County": "Gallatin", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59773", "FIPS": "30031", "County": "Gallatin", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59058", "FIPS": "30033", "County": "Garfield", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59077", "FIPS": "30033", "County": "Garfield", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59318", "FIPS": "30033", "County": "Garfield", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59322", "FIPS": "30033", "County": "Garfield", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59337", "FIPS": "30033", "County": "Garfield", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59411", "FIPS": "30035", "County": "Glacier", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59417", "FIPS": "30035", "County": "Glacier", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59427", "FIPS": "30035", "County": "Glacier", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59434", "FIPS": "30035", "County": "Glacier", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59046", "FIPS": "30037", "County": "Golden Valley", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59074", "FIPS": "30037", "County": "Golden Valley", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59832", "FIPS": "30039", "County": "Granite", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59837", "FIPS": "30039", "County": "Granite", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59858", "FIPS": "30039", "County": "Granite", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59501", "FIPS": "30041", "County": "Hill", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59521", "FIPS": "30041", "County": "Hill", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59525", "FIPS": "30041", "County": "Hill", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59528", "FIPS": "30041", "County": "Hill", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59530", "FIPS": "30041", "County": "Hill", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59532", "FIPS": "30041", "County": "Hill", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59540", "FIPS": "30041", "County": "Hill", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59631", "FIPS": "30043", "County": "Jefferson", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59632", "FIPS": "30043", "County": "Jefferson", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59634", "FIPS": "30043", "County": "Jefferson", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59638", "FIPS": "30043", "County": "Jefferson", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59721", "FIPS": "30043", "County": "Jefferson", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59759", "FIPS": "30043", "County": "Jefferson", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59447", "FIPS": "30045", "County": "Judith Basin", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59452", "FIPS": "30045", "County": "Judith Basin", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59462", "FIPS": "30045", "County": "Judith Basin", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59469", "FIPS": "30045", "County": "Judith Basin", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59479", "FIPS": "30045", "County": "Judith Basin", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59821", "FIPS": "30047", "County": "Lake", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59824", "FIPS": "30047", "County": "Lake", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59855", "FIPS": "30047", "County": "Lake", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59860", "FIPS": "30047", "County": "Lake", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59863", "FIPS": "30047", "County": "Lake", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59864", "FIPS": "30047", "County": "Lake", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59865", "FIPS": "30047", "County": "Lake", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59910", "FIPS": "30047", "County": "Lake", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59914", "FIPS": "30047", "County": "Lake", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59915", "FIPS": "30047", "County": "Lake", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59929", "FIPS": "30047", "County": "Lake", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59931", "FIPS": "30047", "County": "Lake", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59410", "FIPS": "30049", "County": "Lewis and Clark", "State": "MT", "DMA": {"Code": 766, "Name": "HELENA"}},
  {"Postal_Code": "59601", "FIPS": "30049", "County": "Lewis and Clark", "State": "MT", "DMA": {"Code": 766, "Name": "HELENA"}},
  {"Postal_Code": "59602", "FIPS": "30049", "County": "Lewis and Clark", "State": "MT", "DMA": {"Code": 766, "Name": "HELENA"}},
  {"Postal_Code": "59604", "FIPS": "30049", "County": "Lewis and Clark", "State": "MT", "DMA": {"Code": 766, "Name": "HELENA"}},
  {"Postal_Code": "59620", "FIPS": "30049", "County": "Lewis and Clark", "State": "MT", "DMA": {"Code": 766, "Name": "HELENA"}},
  {"Postal_Code": "59623", "FIPS": "30049", "County": "Lewis and Clark", "State": "MT", "DMA": {"Code": 766, "Name": "HELENA"}},
  {"Postal_Code": "59624", "FIPS": "30049", "County": "Lewis and Clark", "State": "MT", "DMA": {"Code": 766, "Name": "HELENA"}},
  {"Postal_Code": "59625", "FIPS": "30049", "County": "Lewis and Clark", "State": "MT", "DMA": {"Code": 766, "Name": "HELENA"}},
  {"Postal_Code": "59626", "FIPS": "30049", "County": "Lewis and Clark", "State": "MT", "DMA": {"Code": 766, "Name": "HELENA"}},
  {"Postal_Code": "59633", "FIPS": "30049", "County": "Lewis and Clark", "State": "MT", "DMA": {"Code": 766, "Name": "HELENA"}},
  {"Postal_Code": "59635", "FIPS": "30049", "County": "Lewis and Clark", "State": "MT", "DMA": {"Code": 766, "Name": "HELENA"}},
  {"Postal_Code": "59636", "FIPS": "30049", "County": "Lewis and Clark", "State": "MT", "DMA": {"Code": 766, "Name": "HELENA"}},
  {"Postal_Code": "59639", "FIPS": "30049", "County": "Lewis and Clark", "State": "MT", "DMA": {"Code": 766, "Name": "HELENA"}},
  {"Postal_Code": "59640", "FIPS": "30049", "County": "Lewis and Clark", "State": "MT", "DMA": {"Code": 766, "Name": "HELENA"}},
  {"Postal_Code": "59648", "FIPS": "30049", "County": "Lewis and Clark", "State": "MT", "DMA": {"Code": 766, "Name": "HELENA"}},
  {"Postal_Code": "59461", "FIPS": "30051", "County": "Liberty", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59522", "FIPS": "30051", "County": "Liberty", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59531", "FIPS": "30051", "County": "Liberty", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59545", "FIPS": "30051", "County": "Liberty", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59917", "FIPS": "30053", "County": "Lincoln", "State": "MT", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "59918", "FIPS": "30053", "County": "Lincoln", "State": "MT", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "59923", "FIPS": "30053", "County": "Lincoln", "State": "MT", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "59930", "FIPS": "30053", "County": "Lincoln", "State": "MT", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "59933", "FIPS": "30053", "County": "Lincoln", "State": "MT", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "59934", "FIPS": "30053", "County": "Lincoln", "State": "MT", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "59935", "FIPS": "30053", "County": "Lincoln", "State": "MT", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "59214", "FIPS": "30055", "County": "McCone", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59215", "FIPS": "30055", "County": "McCone", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59274", "FIPS": "30055", "County": "McCone", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59710", "FIPS": "30057", "County": "Madison", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59720", "FIPS": "30057", "County": "Madison", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59729", "FIPS": "30057", "County": "Madison", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59735", "FIPS": "30057", "County": "Madison", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59740", "FIPS": "30057", "County": "Madison", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59745", "FIPS": "30057", "County": "Madison", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59747", "FIPS": "30057", "County": "Madison", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59749", "FIPS": "30057", "County": "Madison", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59751", "FIPS": "30057", "County": "Madison", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59754", "FIPS": "30057", "County": "Madison", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59755", "FIPS": "30057", "County": "Madison", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59053", "FIPS": "30059", "County": "Meagher", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59642", "FIPS": "30059", "County": "Meagher", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59645", "FIPS": "30059", "County": "Meagher", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59820", "FIPS": "30061", "County": "Mineral", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59830", "FIPS": "30061", "County": "Mineral", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59842", "FIPS": "30061", "County": "Mineral", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59866", "FIPS": "30061", "County": "Mineral", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59867", "FIPS": "30061", "County": "Mineral", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59872", "FIPS": "30061", "County": "Mineral", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59801", "FIPS": "30063", "County": "Missoula", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59802", "FIPS": "30063", "County": "Missoula", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59803", "FIPS": "30063", "County": "Missoula", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59804", "FIPS": "30063", "County": "Missoula", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59806", "FIPS": "30063", "County": "Missoula", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59807", "FIPS": "30063", "County": "Missoula", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59808", "FIPS": "30063", "County": "Missoula", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59812", "FIPS": "30063", "County": "Missoula", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59823", "FIPS": "30063", "County": "Missoula", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59825", "FIPS": "30063", "County": "Missoula", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59826", "FIPS": "30063", "County": "Missoula", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59834", "FIPS": "30063", "County": "Missoula", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59846", "FIPS": "30063", "County": "Missoula", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59847", "FIPS": "30063", "County": "Missoula", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59851", "FIPS": "30063", "County": "Missoula", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59868", "FIPS": "30063", "County": "Missoula", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59054", "FIPS": "30065", "County": "Musselshell", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59059", "FIPS": "30065", "County": "Musselshell", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59072", "FIPS": "30065", "County": "Musselshell", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59073", "FIPS": "30065", "County": "Musselshell", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59018", "FIPS": "30067", "County": "Park", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59020", "FIPS": "30067", "County": "Park", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59027", "FIPS": "30067", "County": "Park", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59030", "FIPS": "30067", "County": "Park", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59047", "FIPS": "30067", "County": "Park", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59065", "FIPS": "30067", "County": "Park", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59081", "FIPS": "30067", "County": "Park", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59082", "FIPS": "30067", "County": "Park", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59086", "FIPS": "30067", "County": "Park", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59084", "FIPS": "30069", "County": "Petroleum", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59087", "FIPS": "30069", "County": "Petroleum", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59261", "FIPS": "30071", "County": "Phillips", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59524", "FIPS": "30071", "County": "Phillips", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59537", "FIPS": "30071", "County": "Phillips", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59538", "FIPS": "30071", "County": "Phillips", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59544", "FIPS": "30071", "County": "Phillips", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59546", "FIPS": "30071", "County": "Phillips", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59416", "FIPS": "30073", "County": "Pondera", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59425", "FIPS": "30073", "County": "Pondera", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59432", "FIPS": "30073", "County": "Pondera", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59448", "FIPS": "30073", "County": "Pondera", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59456", "FIPS": "30073", "County": "Pondera", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59486", "FIPS": "30073", "County": "Pondera", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59062", "FIPS": "30075", "County": "Powder River", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59314", "FIPS": "30075", "County": "Powder River", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59317", "FIPS": "30075", "County": "Powder River", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59343", "FIPS": "30075", "County": "Powder River", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59345", "FIPS": "30075", "County": "Powder River", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59713", "FIPS": "30077", "County": "Powell", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59722", "FIPS": "30077", "County": "Powell", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59728", "FIPS": "30077", "County": "Powell", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59731", "FIPS": "30077", "County": "Powell", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59733", "FIPS": "30077", "County": "Powell", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59843", "FIPS": "30077", "County": "Powell", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59854", "FIPS": "30077", "County": "Powell", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59326", "FIPS": "30079", "County": "Prairie", "State": "MT", "DMA": {"Code": 798, "Name": "GLENDIVE"}},
  {"Postal_Code": "59341", "FIPS": "30079", "County": "Prairie", "State": "MT", "DMA": {"Code": 798, "Name": "GLENDIVE"}},
  {"Postal_Code": "59349", "FIPS": "30079", "County": "Prairie", "State": "MT", "DMA": {"Code": 798, "Name": "GLENDIVE"}},
  {"Postal_Code": "59827", "FIPS": "30081", "County": "Ravalli", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59828", "FIPS": "30081", "County": "Ravalli", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59829", "FIPS": "30081", "County": "Ravalli", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59833", "FIPS": "30081", "County": "Ravalli", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59835", "FIPS": "30081", "County": "Ravalli", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59840", "FIPS": "30081", "County": "Ravalli", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59841", "FIPS": "30081", "County": "Ravalli", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59870", "FIPS": "30081", "County": "Ravalli", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59871", "FIPS": "30081", "County": "Ravalli", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59875", "FIPS": "30081", "County": "Ravalli", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59217", "FIPS": "30083", "County": "Richland", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59221", "FIPS": "30083", "County": "Richland", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59243", "FIPS": "30083", "County": "Richland", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59262", "FIPS": "30083", "County": "Richland", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59270", "FIPS": "30083", "County": "Richland", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59201", "FIPS": "30085", "County": "Roosevelt", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59212", "FIPS": "30085", "County": "Roosevelt", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59213", "FIPS": "30085", "County": "Roosevelt", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59218", "FIPS": "30085", "County": "Roosevelt", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59226", "FIPS": "30085", "County": "Roosevelt", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59255", "FIPS": "30085", "County": "Roosevelt", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59003", "FIPS": "30087", "County": "Rosebud", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59004", "FIPS": "30087", "County": "Rosebud", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59012", "FIPS": "30087", "County": "Rosebud", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59039", "FIPS": "30087", "County": "Rosebud", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59043", "FIPS": "30087", "County": "Rosebud", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59083", "FIPS": "30087", "County": "Rosebud", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59312", "FIPS": "30087", "County": "Rosebud", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59323", "FIPS": "30087", "County": "Rosebud", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59327", "FIPS": "30087", "County": "Rosebud", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59333", "FIPS": "30087", "County": "Rosebud", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59347", "FIPS": "30087", "County": "Rosebud", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59831", "FIPS": "30089", "County": "Sanders", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59844", "FIPS": "30089", "County": "Sanders", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59845", "FIPS": "30089", "County": "Sanders", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59848", "FIPS": "30089", "County": "Sanders", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59853", "FIPS": "30089", "County": "Sanders", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59856", "FIPS": "30089", "County": "Sanders", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59859", "FIPS": "30089", "County": "Sanders", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59873", "FIPS": "30089", "County": "Sanders", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59874", "FIPS": "30089", "County": "Sanders", "State": "MT", "DMA": {"Code": 762, "Name": "MISSOULA"}},
  {"Postal_Code": "59211", "FIPS": "30091", "County": "Sheridan", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59219", "FIPS": "30091", "County": "Sheridan", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59242", "FIPS": "30091", "County": "Sheridan", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59247", "FIPS": "30091", "County": "Sheridan", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59252", "FIPS": "30091", "County": "Sheridan", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59254", "FIPS": "30091", "County": "Sheridan", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59256", "FIPS": "30091", "County": "Sheridan", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59257", "FIPS": "30091", "County": "Sheridan", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59258", "FIPS": "30091", "County": "Sheridan", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59275", "FIPS": "30091", "County": "Sheridan", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59701", "FIPS": "30093", "County": "Silver Bow", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59702", "FIPS": "30093", "County": "Silver Bow", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59703", "FIPS": "30093", "County": "Silver Bow", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59707", "FIPS": "30093", "County": "Silver Bow", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59727", "FIPS": "30093", "County": "Silver Bow", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59743", "FIPS": "30093", "County": "Silver Bow", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59748", "FIPS": "30093", "County": "Silver Bow", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59750", "FIPS": "30093", "County": "Silver Bow", "State": "MT", "DMA": {"Code": 754, "Name": "BUTTE-BOZEMAN"}},
  {"Postal_Code": "59001", "FIPS": "30095", "County": "Stillwater", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59019", "FIPS": "30095", "County": "Stillwater", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59028", "FIPS": "30095", "County": "Stillwater", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59061", "FIPS": "30095", "County": "Stillwater", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59063", "FIPS": "30095", "County": "Stillwater", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59067", "FIPS": "30095", "County": "Stillwater", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59069", "FIPS": "30095", "County": "Stillwater", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59011", "FIPS": "30097", "County": "Sweet Grass", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59033", "FIPS": "30097", "County": "Sweet Grass", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59052", "FIPS": "30097", "County": "Sweet Grass", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59055", "FIPS": "30097", "County": "Sweet Grass", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59419", "FIPS": "30099", "County": "Teton", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59422", "FIPS": "30099", "County": "Teton", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59433", "FIPS": "30099", "County": "Teton", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59436", "FIPS": "30099", "County": "Teton", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59467", "FIPS": "30099", "County": "Teton", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59468", "FIPS": "30099", "County": "Teton", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59435", "FIPS": "30101", "County": "Toole", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59444", "FIPS": "30101", "County": "Toole", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59454", "FIPS": "30101", "County": "Toole", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59466", "FIPS": "30101", "County": "Toole", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59474", "FIPS": "30101", "County": "Toole", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59482", "FIPS": "30101", "County": "Toole", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59484", "FIPS": "30101", "County": "Toole", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59010", "FIPS": "30103", "County": "Treasure", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59038", "FIPS": "30103", "County": "Treasure", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59076", "FIPS": "30103", "County": "Treasure", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59223", "FIPS": "30105", "County": "Valley", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59225", "FIPS": "30105", "County": "Valley", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59230", "FIPS": "30105", "County": "Valley", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59231", "FIPS": "30105", "County": "Valley", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59240", "FIPS": "30105", "County": "Valley", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59241", "FIPS": "30105", "County": "Valley", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59244", "FIPS": "30105", "County": "Valley", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59248", "FIPS": "30105", "County": "Valley", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59250", "FIPS": "30105", "County": "Valley", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59260", "FIPS": "30105", "County": "Valley", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59273", "FIPS": "30105", "County": "Valley", "State": "MT", "DMA": {"Code": 755, "Name": "GREAT FALLS"}},
  {"Postal_Code": "59036", "FIPS": "30107", "County": "Wheatland", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59078", "FIPS": "30107", "County": "Wheatland", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59085", "FIPS": "30107", "County": "Wheatland", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59453", "FIPS": "30107", "County": "Wheatland", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59353", "FIPS": "30109", "County": "Wibaux", "State": "MT", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "59002", "FIPS": "30111", "County": "Yellowstone", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59006", "FIPS": "30111", "County": "Yellowstone", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59015", "FIPS": "30111", "County": "Yellowstone", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59024", "FIPS": "30111", "County": "Yellowstone", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59037", "FIPS": "30111", "County": "Yellowstone", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59044", "FIPS": "30111", "County": "Yellowstone", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59057", "FIPS": "30111", "County": "Yellowstone", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59064", "FIPS": "30111", "County": "Yellowstone", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59079", "FIPS": "30111", "County": "Yellowstone", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59088", "FIPS": "30111", "County": "Yellowstone", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59101", "FIPS": "30111", "County": "Yellowstone", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59102", "FIPS": "30111", "County": "Yellowstone", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59103", "FIPS": "30111", "County": "Yellowstone", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59104", "FIPS": "30111", "County": "Yellowstone", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59105", "FIPS": "30111", "County": "Yellowstone", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59106", "FIPS": "30111", "County": "Yellowstone", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59107", "FIPS": "30111", "County": "Yellowstone", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59108", "FIPS": "30111", "County": "Yellowstone", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59111", "FIPS": "30111", "County": "Yellowstone", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59112", "FIPS": "30111", "County": "Yellowstone", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59114", "FIPS": "30111", "County": "Yellowstone", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59115", "FIPS": "30111", "County": "Yellowstone", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59116", "FIPS": "30111", "County": "Yellowstone", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "59117", "FIPS": "30111", "County": "Yellowstone", "State": "MT", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "68901", "FIPS": "31001", "County": "Adams", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68902", "FIPS": "31001", "County": "Adams", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68925", "FIPS": "31001", "County": "Adams", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68950", "FIPS": "31001", "County": "Adams", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68955", "FIPS": "31001", "County": "Adams", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68956", "FIPS": "31001", "County": "Adams", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68973", "FIPS": "31001", "County": "Adams", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68636", "FIPS": "31003", "County": "Antelope", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68720", "FIPS": "31003", "County": "Antelope", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68726", "FIPS": "31003", "County": "Antelope", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68756", "FIPS": "31003", "County": "Antelope", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68761", "FIPS": "31003", "County": "Antelope", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68764", "FIPS": "31003", "County": "Antelope", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68773", "FIPS": "31003", "County": "Antelope", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69121", "FIPS": "31005", "County": "Arthur", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69345", "FIPS": "31007", "County": "Banner", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "68821", "FIPS": "31009", "County": "Blaine", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68833", "FIPS": "31009", "County": "Blaine", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69157", "FIPS": "31009", "County": "Blaine", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68620", "FIPS": "31011", "County": "Boone", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68627", "FIPS": "31011", "County": "Boone", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68652", "FIPS": "31011", "County": "Boone", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68655", "FIPS": "31011", "County": "Boone", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68660", "FIPS": "31011", "County": "Boone", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69301", "FIPS": "31013", "County": "Box Butte", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69348", "FIPS": "31013", "County": "Box Butte", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "68719", "FIPS": "31015", "County": "Boyd", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68722", "FIPS": "31015", "County": "Boyd", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68746", "FIPS": "31015", "County": "Boyd", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68755", "FIPS": "31015", "County": "Boyd", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68777", "FIPS": "31015", "County": "Boyd", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69135", "FIPS": "31017", "County": "Brown", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69210", "FIPS": "31017", "County": "Brown", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69214", "FIPS": "31017", "County": "Brown", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69217", "FIPS": "31017", "County": "Brown", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68812", "FIPS": "31019", "County": "Buffalo", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68836", "FIPS": "31019", "County": "Buffalo", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68840", "FIPS": "31019", "County": "Buffalo", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68845", "FIPS": "31019", "County": "Buffalo", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68847", "FIPS": "31019", "County": "Buffalo", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68848", "FIPS": "31019", "County": "Buffalo", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68849", "FIPS": "31019", "County": "Buffalo", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68858", "FIPS": "31019", "County": "Buffalo", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68861", "FIPS": "31019", "County": "Buffalo", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68866", "FIPS": "31019", "County": "Buffalo", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68869", "FIPS": "31019", "County": "Buffalo", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68870", "FIPS": "31019", "County": "Buffalo", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68876", "FIPS": "31019", "County": "Buffalo", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68019", "FIPS": "31021", "County": "Burt", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68020", "FIPS": "31021", "County": "Burt", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68038", "FIPS": "31021", "County": "Burt", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68045", "FIPS": "31021", "County": "Burt", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68061", "FIPS": "31021", "County": "Burt", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68001", "FIPS": "31023", "County": "Butler", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68014", "FIPS": "31023", "County": "Butler", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68036", "FIPS": "31023", "County": "Butler", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68624", "FIPS": "31023", "County": "Butler", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68626", "FIPS": "31023", "County": "Butler", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68632", "FIPS": "31023", "County": "Butler", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68635", "FIPS": "31023", "County": "Butler", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68658", "FIPS": "31023", "County": "Butler", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68667", "FIPS": "31023", "County": "Butler", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68669", "FIPS": "31023", "County": "Butler", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68016", "FIPS": "31025", "County": "Cass", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68037", "FIPS": "31025", "County": "Cass", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68048", "FIPS": "31025", "County": "Cass", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68058", "FIPS": "31025", "County": "Cass", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68304", "FIPS": "31025", "County": "Cass", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68307", "FIPS": "31025", "County": "Cass", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68347", "FIPS": "31025", "County": "Cass", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68349", "FIPS": "31025", "County": "Cass", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68366", "FIPS": "31025", "County": "Cass", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68403", "FIPS": "31025", "County": "Cass", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68407", "FIPS": "31025", "County": "Cass", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68409", "FIPS": "31025", "County": "Cass", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68413", "FIPS": "31025", "County": "Cass", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68455", "FIPS": "31025", "County": "Cass", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68463", "FIPS": "31025", "County": "Cass", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68717", "FIPS": "31027", "County": "Cedar", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68727", "FIPS": "31027", "County": "Cedar", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68736", "FIPS": "31027", "County": "Cedar", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68739", "FIPS": "31027", "County": "Cedar", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68745", "FIPS": "31027", "County": "Cedar", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68749", "FIPS": "31027", "County": "Cedar", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68771", "FIPS": "31027", "County": "Cedar", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68774", "FIPS": "31027", "County": "Cedar", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68792", "FIPS": "31027", "County": "Cedar", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "69023", "FIPS": "31029", "County": "Chase", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69027", "FIPS": "31029", "County": "Chase", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69033", "FIPS": "31029", "County": "Chase", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69045", "FIPS": "31029", "County": "Chase", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69201", "FIPS": "31031", "County": "Cherry", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69211", "FIPS": "31031", "County": "Cherry", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69212", "FIPS": "31031", "County": "Cherry", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69216", "FIPS": "31031", "County": "Cherry", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69218", "FIPS": "31031", "County": "Cherry", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69219", "FIPS": "31031", "County": "Cherry", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69220", "FIPS": "31031", "County": "Cherry", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69221", "FIPS": "31031", "County": "Cherry", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69131", "FIPS": "31033", "County": "Cheyenne", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69141", "FIPS": "31033", "County": "Cheyenne", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69149", "FIPS": "31033", "County": "Cheyenne", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69156", "FIPS": "31033", "County": "Cheyenne", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69160", "FIPS": "31033", "County": "Cheyenne", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69162", "FIPS": "31033", "County": "Cheyenne", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "68452", "FIPS": "31035", "County": "Clay", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68933", "FIPS": "31035", "County": "Clay", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68934", "FIPS": "31035", "County": "Clay", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68935", "FIPS": "31035", "County": "Clay", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68938", "FIPS": "31035", "County": "Clay", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68941", "FIPS": "31035", "County": "Clay", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68944", "FIPS": "31035", "County": "Clay", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68954", "FIPS": "31035", "County": "Clay", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68975", "FIPS": "31035", "County": "Clay", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68979", "FIPS": "31035", "County": "Clay", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68980", "FIPS": "31035", "County": "Clay", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68629", "FIPS": "31037", "County": "Colfax", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68641", "FIPS": "31037", "County": "Colfax", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68643", "FIPS": "31037", "County": "Colfax", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68659", "FIPS": "31037", "County": "Colfax", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68661", "FIPS": "31037", "County": "Colfax", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68004", "FIPS": "31039", "County": "Cuming", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68716", "FIPS": "31039", "County": "Cuming", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68788", "FIPS": "31039", "County": "Cuming", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68791", "FIPS": "31039", "County": "Cuming", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68813", "FIPS": "31041", "County": "Custer", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68814", "FIPS": "31041", "County": "Custer", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68822", "FIPS": "31041", "County": "Custer", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68825", "FIPS": "31041", "County": "Custer", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68828", "FIPS": "31041", "County": "Custer", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68855", "FIPS": "31041", "County": "Custer", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68856", "FIPS": "31041", "County": "Custer", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68860", "FIPS": "31041", "County": "Custer", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68874", "FIPS": "31041", "County": "Custer", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68881", "FIPS": "31041", "County": "Custer", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69120", "FIPS": "31041", "County": "Custer", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68030", "FIPS": "31043", "County": "Dakota", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68731", "FIPS": "31043", "County": "Dakota", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68733", "FIPS": "31043", "County": "Dakota", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68741", "FIPS": "31043", "County": "Dakota", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68743", "FIPS": "31043", "County": "Dakota", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68776", "FIPS": "31043", "County": "Dakota", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "69337", "FIPS": "31045", "County": "Dawes", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69339", "FIPS": "31045", "County": "Dawes", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69354", "FIPS": "31045", "County": "Dawes", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69367", "FIPS": "31045", "County": "Dawes", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "68834", "FIPS": "31047", "County": "Dawson", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68850", "FIPS": "31047", "County": "Dawson", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68863", "FIPS": "31047", "County": "Dawson", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68878", "FIPS": "31047", "County": "Dawson", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69029", "FIPS": "31047", "County": "Dawson", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69130", "FIPS": "31047", "County": "Dawson", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69138", "FIPS": "31047", "County": "Dawson", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69171", "FIPS": "31047", "County": "Dawson", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69122", "FIPS": "31049", "County": "Deuel", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69129", "FIPS": "31049", "County": "Deuel", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "68710", "FIPS": "31051", "County": "Dixon", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68728", "FIPS": "31051", "County": "Dixon", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68732", "FIPS": "31051", "County": "Dixon", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68751", "FIPS": "31051", "County": "Dixon", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68757", "FIPS": "31051", "County": "Dixon", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68770", "FIPS": "31051", "County": "Dixon", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68784", "FIPS": "31051", "County": "Dixon", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68785", "FIPS": "31051", "County": "Dixon", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68025", "FIPS": "31053", "County": "Dodge", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68026", "FIPS": "31053", "County": "Dodge", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68031", "FIPS": "31053", "County": "Dodge", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68044", "FIPS": "31053", "County": "Dodge", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68057", "FIPS": "31053", "County": "Dodge", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68063", "FIPS": "31053", "County": "Dodge", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68072", "FIPS": "31053", "County": "Dodge", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68621", "FIPS": "31053", "County": "Dodge", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68633", "FIPS": "31053", "County": "Dodge", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68649", "FIPS": "31053", "County": "Dodge", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68664", "FIPS": "31053", "County": "Dodge", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68007", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68010", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68022", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68064", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68069", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68101", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68102", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68103", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68104", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68105", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68106", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68107", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68108", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68109", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68110", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68111", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68112", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68114", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68116", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68117", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68118", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68119", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68120", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68122", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68124", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68127", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68130", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68131", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68132", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68134", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68135", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68137", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68139", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68142", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68144", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68145", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68152", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68154", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68155", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68164", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68172", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68175", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68176", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68178", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68179", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68180", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68181", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68182", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68183", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68197", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68198", "FIPS": "31055", "County": "Douglas", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "69021", "FIPS": "31057", "County": "Dundy", "State": "NE", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "69030", "FIPS": "31057", "County": "Dundy", "State": "NE", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "69037", "FIPS": "31057", "County": "Dundy", "State": "NE", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "69041", "FIPS": "31057", "County": "Dundy", "State": "NE", "DMA": {"Code": 678, "Name": "WICHITA-HUTCHINSON PLUS"}},
  {"Postal_Code": "68351", "FIPS": "31059", "County": "Fillmore", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68354", "FIPS": "31059", "County": "Fillmore", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68361", "FIPS": "31059", "County": "Fillmore", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68365", "FIPS": "31059", "County": "Fillmore", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68406", "FIPS": "31059", "County": "Fillmore", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68416", "FIPS": "31059", "County": "Fillmore", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68436", "FIPS": "31059", "County": "Fillmore", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68444", "FIPS": "31059", "County": "Fillmore", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68929", "FIPS": "31061", "County": "Franklin", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68932", "FIPS": "31061", "County": "Franklin", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68939", "FIPS": "31061", "County": "Franklin", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68947", "FIPS": "31061", "County": "Franklin", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68960", "FIPS": "31061", "County": "Franklin", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68972", "FIPS": "31061", "County": "Franklin", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68981", "FIPS": "31061", "County": "Franklin", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69025", "FIPS": "31063", "County": "Frontier", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69028", "FIPS": "31063", "County": "Frontier", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69038", "FIPS": "31063", "County": "Frontier", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69039", "FIPS": "31063", "County": "Frontier", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69042", "FIPS": "31063", "County": "Frontier", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68922", "FIPS": "31065", "County": "Furnas", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68926", "FIPS": "31065", "County": "Furnas", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68936", "FIPS": "31065", "County": "Furnas", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68946", "FIPS": "31065", "County": "Furnas", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68948", "FIPS": "31065", "County": "Furnas", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68967", "FIPS": "31065", "County": "Furnas", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69022", "FIPS": "31065", "County": "Furnas", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69046", "FIPS": "31065", "County": "Furnas", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68301", "FIPS": "31067", "County": "Gage", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68309", "FIPS": "31067", "County": "Gage", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68310", "FIPS": "31067", "County": "Gage", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68318", "FIPS": "31067", "County": "Gage", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68328", "FIPS": "31067", "County": "Gage", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68331", "FIPS": "31067", "County": "Gage", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68357", "FIPS": "31067", "County": "Gage", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68381", "FIPS": "31067", "County": "Gage", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68415", "FIPS": "31067", "County": "Gage", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68422", "FIPS": "31067", "County": "Gage", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68458", "FIPS": "31067", "County": "Gage", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68466", "FIPS": "31067", "County": "Gage", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69147", "FIPS": "31069", "County": "Garden", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69148", "FIPS": "31069", "County": "Garden", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69154", "FIPS": "31069", "County": "Garden", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69190", "FIPS": "31069", "County": "Garden", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "68823", "FIPS": "31071", "County": "Garfield", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68937", "FIPS": "31073", "County": "Gosper", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68976", "FIPS": "31073", "County": "Gosper", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69333", "FIPS": "31075", "County": "Grant", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69350", "FIPS": "31075", "County": "Grant", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69366", "FIPS": "31075", "County": "Grant", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "68665", "FIPS": "31077", "County": "Greeley", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68842", "FIPS": "31077", "County": "Greeley", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68875", "FIPS": "31077", "County": "Greeley", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68882", "FIPS": "31077", "County": "Greeley", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68801", "FIPS": "31079", "County": "Hall", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68802", "FIPS": "31079", "County": "Hall", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68803", "FIPS": "31079", "County": "Hall", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68810", "FIPS": "31079", "County": "Hall", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68824", "FIPS": "31079", "County": "Hall", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68832", "FIPS": "31079", "County": "Hall", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68883", "FIPS": "31079", "County": "Hall", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68818", "FIPS": "31081", "County": "Hamilton", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68841", "FIPS": "31081", "County": "Hamilton", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68843", "FIPS": "31081", "County": "Hamilton", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68846", "FIPS": "31081", "County": "Hamilton", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68854", "FIPS": "31081", "County": "Hamilton", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68865", "FIPS": "31081", "County": "Hamilton", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68920", "FIPS": "31083", "County": "Harlan", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68966", "FIPS": "31083", "County": "Harlan", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68969", "FIPS": "31083", "County": "Harlan", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68971", "FIPS": "31083", "County": "Harlan", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68977", "FIPS": "31083", "County": "Harlan", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69032", "FIPS": "31085", "County": "Hayes", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69024", "FIPS": "31087", "County": "Hitchcock", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69040", "FIPS": "31087", "County": "Hitchcock", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69043", "FIPS": "31087", "County": "Hitchcock", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69044", "FIPS": "31087", "County": "Hitchcock", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68711", "FIPS": "31089", "County": "Holt", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68713", "FIPS": "31089", "County": "Holt", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68725", "FIPS": "31089", "County": "Holt", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68734", "FIPS": "31089", "County": "Holt", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68735", "FIPS": "31089", "County": "Holt", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68742", "FIPS": "31089", "County": "Holt", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68763", "FIPS": "31089", "County": "Holt", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68766", "FIPS": "31089", "County": "Holt", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68780", "FIPS": "31089", "County": "Holt", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69152", "FIPS": "31091", "County": "Hooker", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "68820", "FIPS": "31093", "County": "Howard", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68831", "FIPS": "31093", "County": "Howard", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68835", "FIPS": "31093", "County": "Howard", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68838", "FIPS": "31093", "County": "Howard", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68872", "FIPS": "31093", "County": "Howard", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68873", "FIPS": "31093", "County": "Howard", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68338", "FIPS": "31095", "County": "Jefferson", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68342", "FIPS": "31095", "County": "Jefferson", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68350", "FIPS": "31095", "County": "Jefferson", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68352", "FIPS": "31095", "County": "Jefferson", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68377", "FIPS": "31095", "County": "Jefferson", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68424", "FIPS": "31095", "County": "Jefferson", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68440", "FIPS": "31095", "County": "Jefferson", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68329", "FIPS": "31097", "County": "Johnson", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68332", "FIPS": "31097", "County": "Johnson", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68348", "FIPS": "31097", "County": "Johnson", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68443", "FIPS": "31097", "County": "Johnson", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68450", "FIPS": "31097", "County": "Johnson", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68924", "FIPS": "31099", "County": "Kearney", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68945", "FIPS": "31099", "County": "Kearney", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68959", "FIPS": "31099", "County": "Kearney", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68982", "FIPS": "31099", "County": "Kearney", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69127", "FIPS": "31101", "County": "Keith", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69144", "FIPS": "31101", "County": "Keith", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69146", "FIPS": "31101", "County": "Keith", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69153", "FIPS": "31101", "County": "Keith", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69155", "FIPS": "31101", "County": "Keith", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "68753", "FIPS": "31103", "County": "Keya Paha", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68759", "FIPS": "31103", "County": "Keya Paha", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68778", "FIPS": "31103", "County": "Keya Paha", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69128", "FIPS": "31105", "County": "Kimball", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69133", "FIPS": "31105", "County": "Kimball", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69145", "FIPS": "31105", "County": "Kimball", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "68718", "FIPS": "31107", "County": "Knox", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68724", "FIPS": "31107", "County": "Knox", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68729", "FIPS": "31107", "County": "Knox", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68730", "FIPS": "31107", "County": "Knox", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68760", "FIPS": "31107", "County": "Knox", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68783", "FIPS": "31107", "County": "Knox", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68786", "FIPS": "31107", "County": "Knox", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68789", "FIPS": "31107", "County": "Knox", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68317", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68336", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68339", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68358", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68368", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68372", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68402", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68404", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68419", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68428", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68430", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68438", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68461", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68462", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68501", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68502", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68503", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68504", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68505", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68506", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68507", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68508", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68509", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68510", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68512", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68514", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68516", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68517", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68520", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68521", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68522", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68523", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68524", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68526", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68527", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68528", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68529", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68531", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68532", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68542", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68583", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68588", "FIPS": "31109", "County": "Lancaster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69101", "FIPS": "31111", "County": "Lincoln", "State": "NE", "DMA": {"Code": 740, "Name": "NORTH PLATTE"}},
  {"Postal_Code": "69103", "FIPS": "31111", "County": "Lincoln", "State": "NE", "DMA": {"Code": 740, "Name": "NORTH PLATTE"}},
  {"Postal_Code": "69123", "FIPS": "31111", "County": "Lincoln", "State": "NE", "DMA": {"Code": 740, "Name": "NORTH PLATTE"}},
  {"Postal_Code": "69132", "FIPS": "31111", "County": "Lincoln", "State": "NE", "DMA": {"Code": 740, "Name": "NORTH PLATTE"}},
  {"Postal_Code": "69143", "FIPS": "31111", "County": "Lincoln", "State": "NE", "DMA": {"Code": 740, "Name": "NORTH PLATTE"}},
  {"Postal_Code": "69151", "FIPS": "31111", "County": "Lincoln", "State": "NE", "DMA": {"Code": 740, "Name": "NORTH PLATTE"}},
  {"Postal_Code": "69165", "FIPS": "31111", "County": "Lincoln", "State": "NE", "DMA": {"Code": 740, "Name": "NORTH PLATTE"}},
  {"Postal_Code": "69169", "FIPS": "31111", "County": "Lincoln", "State": "NE", "DMA": {"Code": 740, "Name": "NORTH PLATTE"}},
  {"Postal_Code": "69170", "FIPS": "31111", "County": "Lincoln", "State": "NE", "DMA": {"Code": 740, "Name": "NORTH PLATTE"}},
  {"Postal_Code": "69163", "FIPS": "31113", "County": "Logan", "State": "NE", "DMA": {"Code": 740, "Name": "NORTH PLATTE"}},
  {"Postal_Code": "68879", "FIPS": "31115", "County": "Loup", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69167", "FIPS": "31117", "County": "McPherson", "State": "NE", "DMA": {"Code": 740, "Name": "NORTH PLATTE"}},
  {"Postal_Code": "68701", "FIPS": "31119", "County": "Madison", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68702", "FIPS": "31119", "County": "Madison", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68715", "FIPS": "31119", "County": "Madison", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68748", "FIPS": "31119", "County": "Madison", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68752", "FIPS": "31119", "County": "Madison", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68758", "FIPS": "31119", "County": "Madison", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68781", "FIPS": "31119", "County": "Madison", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68628", "FIPS": "31121", "County": "Merrick", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68663", "FIPS": "31121", "County": "Merrick", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68816", "FIPS": "31121", "County": "Merrick", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68826", "FIPS": "31121", "County": "Merrick", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68827", "FIPS": "31121", "County": "Merrick", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68864", "FIPS": "31121", "County": "Merrick", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69125", "FIPS": "31123", "County": "Morrill", "State": "NE", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "69331", "FIPS": "31123", "County": "Morrill", "State": "NE", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "69334", "FIPS": "31123", "County": "Morrill", "State": "NE", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "69336", "FIPS": "31123", "County": "Morrill", "State": "NE", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "68623", "FIPS": "31125", "County": "Nance", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68638", "FIPS": "31125", "County": "Nance", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68640", "FIPS": "31125", "County": "Nance", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68305", "FIPS": "31127", "County": "Nemaha", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68320", "FIPS": "31127", "County": "Nemaha", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68321", "FIPS": "31127", "County": "Nemaha", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68378", "FIPS": "31127", "County": "Nemaha", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68414", "FIPS": "31127", "County": "Nemaha", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68421", "FIPS": "31127", "County": "Nemaha", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68943", "FIPS": "31129", "County": "Nuckolls", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68957", "FIPS": "31129", "County": "Nuckolls", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68961", "FIPS": "31129", "County": "Nuckolls", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68964", "FIPS": "31129", "County": "Nuckolls", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68974", "FIPS": "31129", "County": "Nuckolls", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68978", "FIPS": "31129", "County": "Nuckolls", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68324", "FIPS": "31131", "County": "Otoe", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68344", "FIPS": "31131", "County": "Otoe", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68346", "FIPS": "31131", "County": "Otoe", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68382", "FIPS": "31131", "County": "Otoe", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68410", "FIPS": "31131", "County": "Otoe", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68417", "FIPS": "31131", "County": "Otoe", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68418", "FIPS": "31131", "County": "Otoe", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68446", "FIPS": "31131", "County": "Otoe", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68448", "FIPS": "31131", "County": "Otoe", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68454", "FIPS": "31131", "County": "Otoe", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68323", "FIPS": "31133", "County": "Pawnee", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68345", "FIPS": "31133", "County": "Pawnee", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68380", "FIPS": "31133", "County": "Pawnee", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68420", "FIPS": "31133", "County": "Pawnee", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68441", "FIPS": "31133", "County": "Pawnee", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68447", "FIPS": "31133", "County": "Pawnee", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69134", "FIPS": "31135", "County": "Perkins", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69140", "FIPS": "31135", "County": "Perkins", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69150", "FIPS": "31135", "County": "Perkins", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69168", "FIPS": "31135", "County": "Perkins", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68923", "FIPS": "31137", "County": "Phelps", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68927", "FIPS": "31137", "County": "Phelps", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68940", "FIPS": "31137", "County": "Phelps", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68949", "FIPS": "31137", "County": "Phelps", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68958", "FIPS": "31137", "County": "Phelps", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68738", "FIPS": "31139", "County": "Pierce", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68747", "FIPS": "31139", "County": "Pierce", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68765", "FIPS": "31139", "County": "Pierce", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68767", "FIPS": "31139", "County": "Pierce", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68769", "FIPS": "31139", "County": "Pierce", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68601", "FIPS": "31141", "County": "Platte", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68602", "FIPS": "31141", "County": "Platte", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68631", "FIPS": "31141", "County": "Platte", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68634", "FIPS": "31141", "County": "Platte", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68642", "FIPS": "31141", "County": "Platte", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68644", "FIPS": "31141", "County": "Platte", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68647", "FIPS": "31141", "County": "Platte", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68653", "FIPS": "31141", "County": "Platte", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68651", "FIPS": "31143", "County": "Polk", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68654", "FIPS": "31143", "County": "Polk", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68662", "FIPS": "31143", "County": "Polk", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68666", "FIPS": "31143", "County": "Polk", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69001", "FIPS": "31145", "County": "Red Willow", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69020", "FIPS": "31145", "County": "Red Willow", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69026", "FIPS": "31145", "County": "Red Willow", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69034", "FIPS": "31145", "County": "Red Willow", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69036", "FIPS": "31145", "County": "Red Willow", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68337", "FIPS": "31147", "County": "Richardson", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68355", "FIPS": "31147", "County": "Richardson", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68376", "FIPS": "31147", "County": "Richardson", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68431", "FIPS": "31147", "County": "Richardson", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68433", "FIPS": "31147", "County": "Richardson", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68437", "FIPS": "31147", "County": "Richardson", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68442", "FIPS": "31147", "County": "Richardson", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68457", "FIPS": "31147", "County": "Richardson", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68714", "FIPS": "31149", "County": "Rock", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68333", "FIPS": "31151", "County": "Saline", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68341", "FIPS": "31151", "County": "Saline", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68343", "FIPS": "31151", "County": "Saline", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68359", "FIPS": "31151", "County": "Saline", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68445", "FIPS": "31151", "County": "Saline", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68453", "FIPS": "31151", "County": "Saline", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68464", "FIPS": "31151", "County": "Saline", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68465", "FIPS": "31151", "County": "Saline", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68005", "FIPS": "31153", "County": "Sarpy", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68028", "FIPS": "31153", "County": "Sarpy", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68046", "FIPS": "31153", "County": "Sarpy", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68056", "FIPS": "31153", "County": "Sarpy", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68059", "FIPS": "31153", "County": "Sarpy", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68113", "FIPS": "31153", "County": "Sarpy", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68123", "FIPS": "31153", "County": "Sarpy", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68128", "FIPS": "31153", "County": "Sarpy", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68133", "FIPS": "31153", "County": "Sarpy", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68136", "FIPS": "31153", "County": "Sarpy", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68138", "FIPS": "31153", "County": "Sarpy", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68147", "FIPS": "31153", "County": "Sarpy", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68157", "FIPS": "31153", "County": "Sarpy", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68003", "FIPS": "31155", "County": "Saunders", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68015", "FIPS": "31155", "County": "Saunders", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68017", "FIPS": "31155", "County": "Saunders", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68018", "FIPS": "31155", "County": "Saunders", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68033", "FIPS": "31155", "County": "Saunders", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68040", "FIPS": "31155", "County": "Saunders", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68041", "FIPS": "31155", "County": "Saunders", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68042", "FIPS": "31155", "County": "Saunders", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68050", "FIPS": "31155", "County": "Saunders", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68065", "FIPS": "31155", "County": "Saunders", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68066", "FIPS": "31155", "County": "Saunders", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68070", "FIPS": "31155", "County": "Saunders", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68073", "FIPS": "31155", "County": "Saunders", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68648", "FIPS": "31155", "County": "Saunders", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "69341", "FIPS": "31157", "County": "Scotts Bluff", "State": "NE", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "69352", "FIPS": "31157", "County": "Scotts Bluff", "State": "NE", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "69353", "FIPS": "31157", "County": "Scotts Bluff", "State": "NE", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "69355", "FIPS": "31157", "County": "Scotts Bluff", "State": "NE", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "69356", "FIPS": "31157", "County": "Scotts Bluff", "State": "NE", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "69357", "FIPS": "31157", "County": "Scotts Bluff", "State": "NE", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "69358", "FIPS": "31157", "County": "Scotts Bluff", "State": "NE", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "69361", "FIPS": "31157", "County": "Scotts Bluff", "State": "NE", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "69363", "FIPS": "31157", "County": "Scotts Bluff", "State": "NE", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "68313", "FIPS": "31159", "County": "Seward", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68314", "FIPS": "31159", "County": "Seward", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68330", "FIPS": "31159", "County": "Seward", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68360", "FIPS": "31159", "County": "Seward", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68364", "FIPS": "31159", "County": "Seward", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68405", "FIPS": "31159", "County": "Seward", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68423", "FIPS": "31159", "County": "Seward", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68434", "FIPS": "31159", "County": "Seward", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68439", "FIPS": "31159", "County": "Seward", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68456", "FIPS": "31159", "County": "Seward", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69335", "FIPS": "31161", "County": "Sheridan", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69340", "FIPS": "31161", "County": "Sheridan", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69343", "FIPS": "31161", "County": "Sheridan", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69347", "FIPS": "31161", "County": "Sheridan", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69351", "FIPS": "31161", "County": "Sheridan", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69360", "FIPS": "31161", "County": "Sheridan", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "69365", "FIPS": "31161", "County": "Sheridan", "State": "NE", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "68817", "FIPS": "31163", "County": "Sherman", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68844", "FIPS": "31163", "County": "Sherman", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68852", "FIPS": "31163", "County": "Sherman", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68853", "FIPS": "31163", "County": "Sherman", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68871", "FIPS": "31163", "County": "Sherman", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69346", "FIPS": "31165", "County": "Sioux", "State": "NE", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "68768", "FIPS": "31167", "County": "Stanton", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68779", "FIPS": "31167", "County": "Stanton", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68303", "FIPS": "31169", "County": "Thayer", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68315", "FIPS": "31169", "County": "Thayer", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68322", "FIPS": "31169", "County": "Thayer", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68325", "FIPS": "31169", "County": "Thayer", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68326", "FIPS": "31169", "County": "Thayer", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68327", "FIPS": "31169", "County": "Thayer", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68335", "FIPS": "31169", "County": "Thayer", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68340", "FIPS": "31169", "County": "Thayer", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68362", "FIPS": "31169", "County": "Thayer", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68370", "FIPS": "31169", "County": "Thayer", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68375", "FIPS": "31169", "County": "Thayer", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68429", "FIPS": "31169", "County": "Thayer", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "69142", "FIPS": "31171", "County": "Thomas", "State": "NE", "DMA": {"Code": 740, "Name": "NORTH PLATTE"}},
  {"Postal_Code": "69161", "FIPS": "31171", "County": "Thomas", "State": "NE", "DMA": {"Code": 740, "Name": "NORTH PLATTE"}},
  {"Postal_Code": "69166", "FIPS": "31171", "County": "Thomas", "State": "NE", "DMA": {"Code": 740, "Name": "NORTH PLATTE"}},
  {"Postal_Code": "68039", "FIPS": "31173", "County": "Thurston", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68047", "FIPS": "31173", "County": "Thurston", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68055", "FIPS": "31173", "County": "Thurston", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68062", "FIPS": "31173", "County": "Thurston", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68067", "FIPS": "31173", "County": "Thurston", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68071", "FIPS": "31173", "County": "Thurston", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68815", "FIPS": "31175", "County": "Valley", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68837", "FIPS": "31175", "County": "Valley", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68859", "FIPS": "31175", "County": "Valley", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68862", "FIPS": "31175", "County": "Valley", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68002", "FIPS": "31177", "County": "Washington", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68008", "FIPS": "31177", "County": "Washington", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68009", "FIPS": "31177", "County": "Washington", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68023", "FIPS": "31177", "County": "Washington", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68029", "FIPS": "31177", "County": "Washington", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68034", "FIPS": "31177", "County": "Washington", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68068", "FIPS": "31177", "County": "Washington", "State": "NE", "DMA": {"Code": 652, "Name": "OMAHA"}},
  {"Postal_Code": "68723", "FIPS": "31179", "County": "Wayne", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68740", "FIPS": "31179", "County": "Wayne", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68787", "FIPS": "31179", "County": "Wayne", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68790", "FIPS": "31179", "County": "Wayne", "State": "NE", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "68928", "FIPS": "31181", "County": "Webster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68930", "FIPS": "31181", "County": "Webster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68942", "FIPS": "31181", "County": "Webster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68952", "FIPS": "31181", "County": "Webster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68970", "FIPS": "31181", "County": "Webster", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68622", "FIPS": "31183", "County": "Wheeler", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68637", "FIPS": "31183", "County": "Wheeler", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68316", "FIPS": "31185", "County": "York", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68319", "FIPS": "31185", "County": "York", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68367", "FIPS": "31185", "County": "York", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68371", "FIPS": "31185", "County": "York", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68401", "FIPS": "31185", "County": "York", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68460", "FIPS": "31185", "County": "York", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "68467", "FIPS": "31185", "County": "York", "State": "NE", "DMA": {"Code": 722, "Name": "LINCOLN-HASTINGS-KEARNEY PLUS"}},
  {"Postal_Code": "89406", "FIPS": "32001", "County": "Churchill", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89407", "FIPS": "32001", "County": "Churchill", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89496", "FIPS": "32001", "County": "Churchill", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "88901", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "88905", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89002", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89004", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89005", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89006", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89007", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89009", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89011", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89012", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89014", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89015", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89016", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89018", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89019", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89021", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89024", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89025", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89026", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89027", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89028", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89029", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89030", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89031", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89032", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89033", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89036", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89037", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89039", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89040", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89044", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89046", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89052", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89053", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89067", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89070", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89074", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89077", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89081", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89084", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89085", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89086", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89087", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89101", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89102", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89103", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89104", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89105", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89106", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89107", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89108", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89109", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89110", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89111", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89112", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89113", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89114", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89115", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89116", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89117", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89118", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89119", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89120", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89121", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89122", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89123", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89124", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89125", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89126", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89127", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89128", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89129", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89130", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89131", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89132", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89133", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89134", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89135", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89136", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89137", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89138", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89139", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89140", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89141", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89142", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89143", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89144", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89145", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89146", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89147", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89148", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89149", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89150", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89151", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89152", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89153", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89154", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89155", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89156", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89157", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89159", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89160", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89161", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89162", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89163", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89164", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89165", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89166", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89169", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89170", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89173", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89177", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89178", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89179", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89180", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89183", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89185", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89191", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89193", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89195", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89199", "FIPS": "32003", "County": "Clark", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89410", "FIPS": "32005", "County": "Douglas", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89411", "FIPS": "32005", "County": "Douglas", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89413", "FIPS": "32005", "County": "Douglas", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89423", "FIPS": "32005", "County": "Douglas", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89448", "FIPS": "32005", "County": "Douglas", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89449", "FIPS": "32005", "County": "Douglas", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89460", "FIPS": "32005", "County": "Douglas", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89705", "FIPS": "32005", "County": "Douglas", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89801", "FIPS": "32007", "County": "Elko", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89802", "FIPS": "32007", "County": "Elko", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89803", "FIPS": "32007", "County": "Elko", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89815", "FIPS": "32007", "County": "Elko", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89822", "FIPS": "32007", "County": "Elko", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89823", "FIPS": "32007", "County": "Elko", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89824", "FIPS": "32007", "County": "Elko", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89825", "FIPS": "32007", "County": "Elko", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89826", "FIPS": "32007", "County": "Elko", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89828", "FIPS": "32007", "County": "Elko", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89830", "FIPS": "32007", "County": "Elko", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89831", "FIPS": "32007", "County": "Elko", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89832", "FIPS": "32007", "County": "Elko", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89833", "FIPS": "32007", "County": "Elko", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89834", "FIPS": "32007", "County": "Elko", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89835", "FIPS": "32007", "County": "Elko", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89883", "FIPS": "32007", "County": "Elko", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89010", "FIPS": "32009", "County": "Esmeralda", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89013", "FIPS": "32009", "County": "Esmeralda", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89047", "FIPS": "32009", "County": "Esmeralda", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89316", "FIPS": "32011", "County": "Eureka", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89821", "FIPS": "32011", "County": "Eureka", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89404", "FIPS": "32013", "County": "Humboldt", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89414", "FIPS": "32013", "County": "Humboldt", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89421", "FIPS": "32013", "County": "Humboldt", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89425", "FIPS": "32013", "County": "Humboldt", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89426", "FIPS": "32013", "County": "Humboldt", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89438", "FIPS": "32013", "County": "Humboldt", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89445", "FIPS": "32013", "County": "Humboldt", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89446", "FIPS": "32013", "County": "Humboldt", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89310", "FIPS": "32015", "County": "Lander", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89820", "FIPS": "32015", "County": "Lander", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89001", "FIPS": "32017", "County": "Lincoln", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89008", "FIPS": "32017", "County": "Lincoln", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89017", "FIPS": "32017", "County": "Lincoln", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89042", "FIPS": "32017", "County": "Lincoln", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89043", "FIPS": "32017", "County": "Lincoln", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89403", "FIPS": "32019", "County": "Lyon", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89408", "FIPS": "32019", "County": "Lyon", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89428", "FIPS": "32019", "County": "Lyon", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89429", "FIPS": "32019", "County": "Lyon", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89430", "FIPS": "32019", "County": "Lyon", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89444", "FIPS": "32019", "County": "Lyon", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89447", "FIPS": "32019", "County": "Lyon", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89415", "FIPS": "32021", "County": "Mineral", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89420", "FIPS": "32021", "County": "Mineral", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89422", "FIPS": "32021", "County": "Mineral", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89427", "FIPS": "32021", "County": "Mineral", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89003", "FIPS": "32023", "County": "Nye", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89020", "FIPS": "32023", "County": "Nye", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89022", "FIPS": "32023", "County": "Nye", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89023", "FIPS": "32023", "County": "Nye", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89041", "FIPS": "32023", "County": "Nye", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89045", "FIPS": "32023", "County": "Nye", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89048", "FIPS": "32023", "County": "Nye", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89049", "FIPS": "32023", "County": "Nye", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89060", "FIPS": "32023", "County": "Nye", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89061", "FIPS": "32023", "County": "Nye", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89409", "FIPS": "32023", "County": "Nye", "State": "NV", "DMA": {"Code": 839, "Name": "LAS VEGAS"}},
  {"Postal_Code": "89418", "FIPS": "32027", "County": "Pershing", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89419", "FIPS": "32027", "County": "Pershing", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89440", "FIPS": "32029", "County": "Storey", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89402", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89405", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89412", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89424", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89431", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89432", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89433", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89434", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89435", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89436", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89439", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89441", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89442", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89450", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89451", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89452", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89501", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89502", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89503", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89504", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89505", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89506", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89507", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89509", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89510", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89511", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89512", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89513", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89515", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89519", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89520", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89521", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89523", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89533", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89555", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89557", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89570", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89595", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89599", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89704", "FIPS": "32031", "County": "Washoe", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89301", "FIPS": "32033", "County": "White Pine", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89311", "FIPS": "32033", "County": "White Pine", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89314", "FIPS": "32033", "County": "White Pine", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89315", "FIPS": "32033", "County": "White Pine", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89317", "FIPS": "32033", "County": "White Pine", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89318", "FIPS": "32033", "County": "White Pine", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89319", "FIPS": "32033", "County": "White Pine", "State": "NV", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "89701", "FIPS": "32510", "County": "Carson City", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89702", "FIPS": "32510", "County": "Carson City", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89703", "FIPS": "32510", "County": "Carson City", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89706", "FIPS": "32510", "County": "Carson City", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89711", "FIPS": "32510", "County": "Carson City", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89712", "FIPS": "32510", "County": "Carson City", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89713", "FIPS": "32510", "County": "Carson City", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89714", "FIPS": "32510", "County": "Carson City", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "89721", "FIPS": "32510", "County": "Carson City", "State": "NV", "DMA": {"Code": 811, "Name": "RENO"}},
  {"Postal_Code": "03218", "FIPS": "33001", "County": "Belknap", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03220", "FIPS": "33001", "County": "Belknap", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03225", "FIPS": "33001", "County": "Belknap", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03226", "FIPS": "33001", "County": "Belknap", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03237", "FIPS": "33001", "County": "Belknap", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03246", "FIPS": "33001", "County": "Belknap", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03247", "FIPS": "33001", "County": "Belknap", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03249", "FIPS": "33001", "County": "Belknap", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03252", "FIPS": "33001", "County": "Belknap", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03253", "FIPS": "33001", "County": "Belknap", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03256", "FIPS": "33001", "County": "Belknap", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03269", "FIPS": "33001", "County": "Belknap", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03276", "FIPS": "33001", "County": "Belknap", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03289", "FIPS": "33001", "County": "Belknap", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03298", "FIPS": "33001", "County": "Belknap", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03299", "FIPS": "33001", "County": "Belknap", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03809", "FIPS": "33001", "County": "Belknap", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03810", "FIPS": "33001", "County": "Belknap", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03837", "FIPS": "33001", "County": "Belknap", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03227", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03254", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03259", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03812", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03813", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03814", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03816", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03817", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03818", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03830", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03832", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03836", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03838", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03845", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03846", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03847", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03849", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03850", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03853", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03860", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03864", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03872", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03875", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03882", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03883", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03886", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03890", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03894", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03896", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03897", "FIPS": "33003", "County": "Carroll", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03431", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03435", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03441", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03443", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03444", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03445", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03446", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03447", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03448", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03450", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03451", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03452", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03455", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03456", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03457", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03461", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03462", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03464", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03465", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03466", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03467", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03469", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03470", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03602", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03604", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03608", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03609", "FIPS": "33005", "County": "Cheshire", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03570", "FIPS": "33007", "County": "Coos", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03575", "FIPS": "33007", "County": "Coos", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03576", "FIPS": "33007", "County": "Coos", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03579", "FIPS": "33007", "County": "Coos", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03581", "FIPS": "33007", "County": "Coos", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03582", "FIPS": "33007", "County": "Coos", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03583", "FIPS": "33007", "County": "Coos", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03584", "FIPS": "33007", "County": "Coos", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03588", "FIPS": "33007", "County": "Coos", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03589", "FIPS": "33007", "County": "Coos", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03590", "FIPS": "33007", "County": "Coos", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03592", "FIPS": "33007", "County": "Coos", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03593", "FIPS": "33007", "County": "Coos", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03595", "FIPS": "33007", "County": "Coos", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03597", "FIPS": "33007", "County": "Coos", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03598", "FIPS": "33007", "County": "Coos", "State": "NH", "DMA": {"Code": 500, "Name": "PORTLAND-AUBURN"}},
  {"Postal_Code": "03215", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03217", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03222", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03223", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03238", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03240", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03241", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03245", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03251", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03262", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03264", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03266", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03274", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03279", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03282", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03293", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03561", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03574", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03580", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03585", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03586", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03740", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03741", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03748", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03749", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03750", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03755", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03756", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03765", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03766", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03768", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03769", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03771", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03774", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03777", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03779", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03780", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03784", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03785", "FIPS": "33009", "County": "Grafton", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03031", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03033", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03043", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03045", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03047", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03048", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03049", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03051", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03052", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03054", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03055", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03057", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03060", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03061", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03062", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03063", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03064", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03070", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03071", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03076", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03082", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03084", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03086", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03101", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03102", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03103", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03104", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03105", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03107", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03108", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03109", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03110", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03111", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03244", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03281", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03440", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03442", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03449", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03458", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03468", "FIPS": "33011", "County": "Hillsborough", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03046", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03106", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03216", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03221", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03224", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03229", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03230", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03231", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03233", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03234", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03235", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03242", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03243", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03255", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03257", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03258", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03260", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03263", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03268", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03272", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03273", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03275", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03278", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03287", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03301", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03302", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03303", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03304", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03305", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03307", "FIPS": "33013", "County": "Merrimack", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03032", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03034", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03036", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03037", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03038", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03040", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03041", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03042", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03044", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03053", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03073", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03077", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03079", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03087", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03261", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03290", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03291", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03801", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03802", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03803", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03804", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03811", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03819", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03826", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03827", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03833", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03840", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03841", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03842", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03843", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03844", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03848", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03854", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03856", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03857", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03858", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03859", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03862", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03865", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03870", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03871", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03873", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03874", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03885", "FIPS": "33015", "County": "Rockingham", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03805", "FIPS": "33017", "County": "Strafford", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03815", "FIPS": "33017", "County": "Strafford", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03820", "FIPS": "33017", "County": "Strafford", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03821", "FIPS": "33017", "County": "Strafford", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03822", "FIPS": "33017", "County": "Strafford", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03823", "FIPS": "33017", "County": "Strafford", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03824", "FIPS": "33017", "County": "Strafford", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03825", "FIPS": "33017", "County": "Strafford", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03835", "FIPS": "33017", "County": "Strafford", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03839", "FIPS": "33017", "County": "Strafford", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03851", "FIPS": "33017", "County": "Strafford", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03852", "FIPS": "33017", "County": "Strafford", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03855", "FIPS": "33017", "County": "Strafford", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03866", "FIPS": "33017", "County": "Strafford", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03867", "FIPS": "33017", "County": "Strafford", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03868", "FIPS": "33017", "County": "Strafford", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03869", "FIPS": "33017", "County": "Strafford", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03878", "FIPS": "33017", "County": "Strafford", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03884", "FIPS": "33017", "County": "Strafford", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03887", "FIPS": "33017", "County": "Strafford", "State": "NH", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "03280", "FIPS": "33019", "County": "Sullivan", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03284", "FIPS": "33019", "County": "Sullivan", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03601", "FIPS": "33019", "County": "Sullivan", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03603", "FIPS": "33019", "County": "Sullivan", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03605", "FIPS": "33019", "County": "Sullivan", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03607", "FIPS": "33019", "County": "Sullivan", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03743", "FIPS": "33019", "County": "Sullivan", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03745", "FIPS": "33019", "County": "Sullivan", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03746", "FIPS": "33019", "County": "Sullivan", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03751", "FIPS": "33019", "County": "Sullivan", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03752", "FIPS": "33019", "County": "Sullivan", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03753", "FIPS": "33019", "County": "Sullivan", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03754", "FIPS": "33019", "County": "Sullivan", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03770", "FIPS": "33019", "County": "Sullivan", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03773", "FIPS": "33019", "County": "Sullivan", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03781", "FIPS": "33019", "County": "Sullivan", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "03782", "FIPS": "33019", "County": "Sullivan", "State": "NH", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "08037", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08201", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08203", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08205", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08213", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08215", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08217", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08220", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08221", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08225", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08231", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08232", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08234", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08240", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08241", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08244", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08310", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08317", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08319", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08326", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08330", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08340", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08341", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08342", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08346", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08350", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08401", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08402", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08403", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08404", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08405", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08406", "FIPS": "34001", "County": "Atlantic", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "07010", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07020", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07022", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07024", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07026", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07031", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07057", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07070", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07071", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07072", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07073", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07074", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07075", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07401", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07407", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07410", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07417", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07423", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07430", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07432", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07436", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07446", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07450", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07451", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07452", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07458", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07463", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07481", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07495", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07601", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07602", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07603", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07604", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07605", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07606", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07607", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07608", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07620", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07621", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07624", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07626", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07627", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07628", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07630", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07631", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07632", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07640", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07641", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07642", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07643", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07644", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07645", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07646", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07647", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07648", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07649", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07650", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07652", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07653", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07656", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07657", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07660", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07661", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07662", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07663", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07666", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07670", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07675", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07676", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07677", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07699", "FIPS": "34003", "County": "Bergen", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08010", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08011", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08015", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08016", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08019", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08022", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08036", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08041", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08042", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08046", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08048", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08052", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08053", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08054", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08055", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08057", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08060", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08064", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08065", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08068", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08073", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08075", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08076", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08077", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08088", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08224", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08505", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08511", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08515", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08518", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08554", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08562", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08640", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08641", "FIPS": "34005", "County": "Burlington", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08002", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08003", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08004", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08007", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08009", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08012", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08018", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08021", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08026", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08029", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08030", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08031", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08033", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08034", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08035", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08043", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08045", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08049", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08059", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08078", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08081", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08083", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08084", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08089", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08091", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08095", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08099", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08101", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08102", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08103", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08104", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08105", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08106", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08107", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08108", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08109", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08110", "FIPS": "34007", "County": "Camden", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08202", "FIPS": "34009", "County": "Cape May", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08204", "FIPS": "34009", "County": "Cape May", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08210", "FIPS": "34009", "County": "Cape May", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08212", "FIPS": "34009", "County": "Cape May", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08214", "FIPS": "34009", "County": "Cape May", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08218", "FIPS": "34009", "County": "Cape May", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08219", "FIPS": "34009", "County": "Cape May", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08223", "FIPS": "34009", "County": "Cape May", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08226", "FIPS": "34009", "County": "Cape May", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08230", "FIPS": "34009", "County": "Cape May", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08242", "FIPS": "34009", "County": "Cape May", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08243", "FIPS": "34009", "County": "Cape May", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08245", "FIPS": "34009", "County": "Cape May", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08246", "FIPS": "34009", "County": "Cape May", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08247", "FIPS": "34009", "County": "Cape May", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08248", "FIPS": "34009", "County": "Cape May", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08250", "FIPS": "34009", "County": "Cape May", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08251", "FIPS": "34009", "County": "Cape May", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08252", "FIPS": "34009", "County": "Cape May", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08260", "FIPS": "34009", "County": "Cape May", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08270", "FIPS": "34009", "County": "Cape May", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08302", "FIPS": "34011", "County": "Cumberland", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08311", "FIPS": "34011", "County": "Cumberland", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08313", "FIPS": "34011", "County": "Cumberland", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08314", "FIPS": "34011", "County": "Cumberland", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08315", "FIPS": "34011", "County": "Cumberland", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08316", "FIPS": "34011", "County": "Cumberland", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08320", "FIPS": "34011", "County": "Cumberland", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08321", "FIPS": "34011", "County": "Cumberland", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08323", "FIPS": "34011", "County": "Cumberland", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08324", "FIPS": "34011", "County": "Cumberland", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08327", "FIPS": "34011", "County": "Cumberland", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08329", "FIPS": "34011", "County": "Cumberland", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08332", "FIPS": "34011", "County": "Cumberland", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08345", "FIPS": "34011", "County": "Cumberland", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08348", "FIPS": "34011", "County": "Cumberland", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08349", "FIPS": "34011", "County": "Cumberland", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08352", "FIPS": "34011", "County": "Cumberland", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08353", "FIPS": "34011", "County": "Cumberland", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08360", "FIPS": "34011", "County": "Cumberland", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08361", "FIPS": "34011", "County": "Cumberland", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08362", "FIPS": "34011", "County": "Cumberland", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "07003", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07004", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07006", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07007", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07009", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07017", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07018", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07019", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07021", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07028", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07039", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07040", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07041", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07042", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07043", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07044", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07050", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07051", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07052", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07068", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07078", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07079", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07101", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07102", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07103", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07104", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07105", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07106", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07107", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07108", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07109", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07110", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07111", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07112", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07114", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07175", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07182", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07184", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07188", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07189", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07191", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07192", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07193", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07194", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07195", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07197", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07198", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07199", "FIPS": "34013", "County": "Essex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08014", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08020", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08025", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08027", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08028", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08032", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08039", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08051", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08056", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08061", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08062", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08063", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08066", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08071", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08074", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08080", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08085", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08086", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08090", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08093", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08094", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08096", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08097", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08312", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08322", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08328", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08343", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08344", "FIPS": "34015", "County": "Gloucester", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "07002", "FIPS": "34017", "County": "Hudson", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07029", "FIPS": "34017", "County": "Hudson", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07030", "FIPS": "34017", "County": "Hudson", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07032", "FIPS": "34017", "County": "Hudson", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07047", "FIPS": "34017", "County": "Hudson", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07086", "FIPS": "34017", "County": "Hudson", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07087", "FIPS": "34017", "County": "Hudson", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07093", "FIPS": "34017", "County": "Hudson", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07094", "FIPS": "34017", "County": "Hudson", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07096", "FIPS": "34017", "County": "Hudson", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07097", "FIPS": "34017", "County": "Hudson", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07099", "FIPS": "34017", "County": "Hudson", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07302", "FIPS": "34017", "County": "Hudson", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07303", "FIPS": "34017", "County": "Hudson", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07304", "FIPS": "34017", "County": "Hudson", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07305", "FIPS": "34017", "County": "Hudson", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07306", "FIPS": "34017", "County": "Hudson", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07307", "FIPS": "34017", "County": "Hudson", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07308", "FIPS": "34017", "County": "Hudson", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07309", "FIPS": "34017", "County": "Hudson", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07310", "FIPS": "34017", "County": "Hudson", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07311", "FIPS": "34017", "County": "Hudson", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07395", "FIPS": "34017", "County": "Hudson", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07399", "FIPS": "34017", "County": "Hudson", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07830", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07979", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08530", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08551", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08556", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08557", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08559", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08801", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08802", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08803", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08804", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08809", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08822", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08825", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08826", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08827", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08829", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08833", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08834", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08848", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08858", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08867", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08868", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08870", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08885", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08887", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08888", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08889", "FIPS": "34019", "County": "Hunterdon", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08520", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08525", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08534", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08540", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08541", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08542", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08543", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08544", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08550", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08560", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08561", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08601", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08602", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08603", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08604", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08605", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08606", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08607", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08608", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08609", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08610", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08611", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08618", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08619", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08620", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08625", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08628", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08629", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08638", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08645", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08646", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08647", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08648", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08650", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08666", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08677", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08690", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08691", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08695", "FIPS": "34021", "County": "Mercer", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "07001", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07008", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07064", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07067", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07077", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07080", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07095", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08512", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08536", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08810", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08812", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08816", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08817", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08818", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08820", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08824", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08828", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08830", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08831", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08832", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08837", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08840", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08846", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08850", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08852", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08854", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08855", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08857", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08859", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08861", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08862", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08863", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08871", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08872", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08879", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08882", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08884", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08899", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08901", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08902", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08903", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08904", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08905", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08906", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08922", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08933", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08988", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08989", "FIPS": "34023", "County": "Middlesex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07701", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07702", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07703", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07704", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07709", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07710", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07711", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07712", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07715", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07716", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07717", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07718", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07719", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07720", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07721", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07722", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07723", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07724", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07726", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07727", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07728", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07730", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07731", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07732", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07733", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07734", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07735", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07737", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07738", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07739", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07740", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07746", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07747", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07748", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07750", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07751", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07752", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07753", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07754", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07755", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07756", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07757", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07758", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07760", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07762", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07763", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07764", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07765", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07777", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07799", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08501", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08510", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08514", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08526", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08535", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08555", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08720", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08730", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08736", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08750", "FIPS": "34025", "County": "Monmouth", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07005", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07034", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07035", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07045", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07046", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07054", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07058", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07082", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07405", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07440", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07444", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07457", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07801", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07802", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07803", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07806", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07828", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07834", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07836", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07842", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07845", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07847", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07849", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07850", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07852", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07853", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07856", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07857", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07866", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07869", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07870", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07876", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07878", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07885", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07926", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07927", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07928", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07930", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07932", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07933", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07935", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07936", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07940", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07945", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07946", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07950", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07960", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07961", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07962", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07963", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07970", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07976", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07980", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07981", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07983", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07999", "FIPS": "34027", "County": "Morris", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08005", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08006", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08008", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08050", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08087", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08092", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08527", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08533", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08701", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08721", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08722", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08723", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08724", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08731", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08732", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08733", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08734", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08735", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08738", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08739", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08740", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08741", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08742", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08751", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08752", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08753", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08754", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08755", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08756", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08757", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08758", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08759", "FIPS": "34029", "County": "Ocean", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07011", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07012", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07013", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07014", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07015", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07055", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07403", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07420", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07421", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07424", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07435", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07438", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07442", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07456", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07465", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07470", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07474", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07477", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07480", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07501", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07502", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07503", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07504", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07505", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07506", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07507", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07508", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07509", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07510", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07511", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07512", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07513", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07514", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07522", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07524", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07533", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07538", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07543", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07544", "FIPS": "34031", "County": "Passaic", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08001", "FIPS": "34033", "County": "Salem", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08023", "FIPS": "34033", "County": "Salem", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08038", "FIPS": "34033", "County": "Salem", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08067", "FIPS": "34033", "County": "Salem", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08069", "FIPS": "34033", "County": "Salem", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08070", "FIPS": "34033", "County": "Salem", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08072", "FIPS": "34033", "County": "Salem", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08079", "FIPS": "34033", "County": "Salem", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08098", "FIPS": "34033", "County": "Salem", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08318", "FIPS": "34033", "County": "Salem", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "08347", "FIPS": "34033", "County": "Salem", "State": "NJ", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "07059", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07069", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07920", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07921", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07924", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07931", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07934", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07938", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07939", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07977", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07978", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08502", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08504", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08528", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08553", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08558", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08805", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08807", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08821", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08823", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08835", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08836", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08844", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08853", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08869", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08873", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08875", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08876", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08880", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08890", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08896", "FIPS": "34035", "County": "Somerset", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07416", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07418", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07419", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07422", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07428", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07439", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07460", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07461", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07462", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07821", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07822", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07826", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07827", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07837", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07839", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07843", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07848", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07851", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07855", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07860", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07871", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07874", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07875", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07877", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07879", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07881", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07890", "FIPS": "34037", "County": "Sussex", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07016", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07023", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07027", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07033", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07036", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07060", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07061", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07062", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07063", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07065", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07066", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07076", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07081", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07083", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07088", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07090", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07091", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07092", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07201", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07202", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07203", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07204", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07205", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07206", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07207", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07208", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07901", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07902", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07922", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07974", "FIPS": "34039", "County": "Union", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07820", "FIPS": "34041", "County": "Warren", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07823", "FIPS": "34041", "County": "Warren", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07825", "FIPS": "34041", "County": "Warren", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07829", "FIPS": "34041", "County": "Warren", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07831", "FIPS": "34041", "County": "Warren", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07832", "FIPS": "34041", "County": "Warren", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07833", "FIPS": "34041", "County": "Warren", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07838", "FIPS": "34041", "County": "Warren", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07840", "FIPS": "34041", "County": "Warren", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07844", "FIPS": "34041", "County": "Warren", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07846", "FIPS": "34041", "County": "Warren", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07863", "FIPS": "34041", "County": "Warren", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07865", "FIPS": "34041", "County": "Warren", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07880", "FIPS": "34041", "County": "Warren", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "07882", "FIPS": "34041", "County": "Warren", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08808", "FIPS": "34041", "County": "Warren", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08865", "FIPS": "34041", "County": "Warren", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "08886", "FIPS": "34041", "County": "Warren", "State": "NJ", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "87008", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87022", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87047", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87059", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87101", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87102", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87103", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87104", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87105", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87106", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87107", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87108", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87109", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87110", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87111", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87112", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87113", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87114", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87115", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87116", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87117", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87119", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87120", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87121", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87122", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87123", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87125", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87131", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87151", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87153", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87154", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87158", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87176", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87181", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87184", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87185", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87187", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87190", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87191", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87192", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87193", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87194", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87195", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87196", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87197", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87198", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87199", "FIPS": "35001", "County": "Bernalillo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87820", "FIPS": "35003", "County": "Catron", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87821", "FIPS": "35003", "County": "Catron", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87824", "FIPS": "35003", "County": "Catron", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87827", "FIPS": "35003", "County": "Catron", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87829", "FIPS": "35003", "County": "Catron", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87830", "FIPS": "35003", "County": "Catron", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88039", "FIPS": "35003", "County": "Catron", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88201", "FIPS": "35005", "County": "Chaves", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88202", "FIPS": "35005", "County": "Chaves", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88203", "FIPS": "35005", "County": "Chaves", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88230", "FIPS": "35005", "County": "Chaves", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88232", "FIPS": "35005", "County": "Chaves", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88253", "FIPS": "35005", "County": "Chaves", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87005", "FIPS": "35006", "County": "Cibola", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87007", "FIPS": "35006", "County": "Cibola", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87014", "FIPS": "35006", "County": "Cibola", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87020", "FIPS": "35006", "County": "Cibola", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87021", "FIPS": "35006", "County": "Cibola", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87026", "FIPS": "35006", "County": "Cibola", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87038", "FIPS": "35006", "County": "Cibola", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87040", "FIPS": "35006", "County": "Cibola", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87049", "FIPS": "35006", "County": "Cibola", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87051", "FIPS": "35006", "County": "Cibola", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87315", "FIPS": "35006", "County": "Cibola", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87357", "FIPS": "35006", "County": "Cibola", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87710", "FIPS": "35007", "County": "Colfax", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87714", "FIPS": "35007", "County": "Colfax", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87718", "FIPS": "35007", "County": "Colfax", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87728", "FIPS": "35007", "County": "Colfax", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87729", "FIPS": "35007", "County": "Colfax", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87740", "FIPS": "35007", "County": "Colfax", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87747", "FIPS": "35007", "County": "Colfax", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87749", "FIPS": "35007", "County": "Colfax", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88101", "FIPS": "35009", "County": "Curry", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88102", "FIPS": "35009", "County": "Curry", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88103", "FIPS": "35009", "County": "Curry", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88112", "FIPS": "35009", "County": "Curry", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88120", "FIPS": "35009", "County": "Curry", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88124", "FIPS": "35009", "County": "Curry", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88133", "FIPS": "35009", "County": "Curry", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88135", "FIPS": "35009", "County": "Curry", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88119", "FIPS": "35011", "County": "De Baca", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88134", "FIPS": "35011", "County": "De Baca", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88136", "FIPS": "35011", "County": "De Baca", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87936", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "87937", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "87940", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "87941", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88001", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88002", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88003", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88004", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88005", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88006", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88007", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88008", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88011", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88012", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88013", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88021", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88024", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88027", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88032", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88033", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88044", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88046", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88047", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88048", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88052", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88054", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88058", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88063", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88072", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88081", "FIPS": "35013", "County": "Dona Ana", "State": "NM", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88210", "FIPS": "35015", "County": "Eddy", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88211", "FIPS": "35015", "County": "Eddy", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88220", "FIPS": "35015", "County": "Eddy", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88221", "FIPS": "35015", "County": "Eddy", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88250", "FIPS": "35015", "County": "Eddy", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88254", "FIPS": "35015", "County": "Eddy", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88255", "FIPS": "35015", "County": "Eddy", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88256", "FIPS": "35015", "County": "Eddy", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88263", "FIPS": "35015", "County": "Eddy", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88268", "FIPS": "35015", "County": "Eddy", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88022", "FIPS": "35017", "County": "Grant", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88023", "FIPS": "35017", "County": "Grant", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88025", "FIPS": "35017", "County": "Grant", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88026", "FIPS": "35017", "County": "Grant", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88028", "FIPS": "35017", "County": "Grant", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88034", "FIPS": "35017", "County": "Grant", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88036", "FIPS": "35017", "County": "Grant", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88038", "FIPS": "35017", "County": "Grant", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88040", "FIPS": "35017", "County": "Grant", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88041", "FIPS": "35017", "County": "Grant", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88043", "FIPS": "35017", "County": "Grant", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88049", "FIPS": "35017", "County": "Grant", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88051", "FIPS": "35017", "County": "Grant", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88053", "FIPS": "35017", "County": "Grant", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88055", "FIPS": "35017", "County": "Grant", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88061", "FIPS": "35017", "County": "Grant", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88062", "FIPS": "35017", "County": "Grant", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88065", "FIPS": "35017", "County": "Grant", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87711", "FIPS": "35019", "County": "Guadalupe", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87724", "FIPS": "35019", "County": "Guadalupe", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88353", "FIPS": "35019", "County": "Guadalupe", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88417", "FIPS": "35019", "County": "Guadalupe", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88431", "FIPS": "35019", "County": "Guadalupe", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88435", "FIPS": "35019", "County": "Guadalupe", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87730", "FIPS": "35021", "County": "Harding", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87733", "FIPS": "35021", "County": "Harding", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87743", "FIPS": "35021", "County": "Harding", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87746", "FIPS": "35021", "County": "Harding", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88009", "FIPS": "35023", "County": "Hidalgo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88020", "FIPS": "35023", "County": "Hidalgo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88045", "FIPS": "35023", "County": "Hidalgo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88056", "FIPS": "35023", "County": "Hidalgo", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88114", "FIPS": "35025", "County": "Lea", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88213", "FIPS": "35025", "County": "Lea", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88231", "FIPS": "35025", "County": "Lea", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88240", "FIPS": "35025", "County": "Lea", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88241", "FIPS": "35025", "County": "Lea", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88242", "FIPS": "35025", "County": "Lea", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88244", "FIPS": "35025", "County": "Lea", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88252", "FIPS": "35025", "County": "Lea", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88260", "FIPS": "35025", "County": "Lea", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88262", "FIPS": "35025", "County": "Lea", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88264", "FIPS": "35025", "County": "Lea", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88265", "FIPS": "35025", "County": "Lea", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88267", "FIPS": "35025", "County": "Lea", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88301", "FIPS": "35027", "County": "Lincoln", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88312", "FIPS": "35027", "County": "Lincoln", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88316", "FIPS": "35027", "County": "Lincoln", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88318", "FIPS": "35027", "County": "Lincoln", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88323", "FIPS": "35027", "County": "Lincoln", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88324", "FIPS": "35027", "County": "Lincoln", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88336", "FIPS": "35027", "County": "Lincoln", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88338", "FIPS": "35027", "County": "Lincoln", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88341", "FIPS": "35027", "County": "Lincoln", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88343", "FIPS": "35027", "County": "Lincoln", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88345", "FIPS": "35027", "County": "Lincoln", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88346", "FIPS": "35027", "County": "Lincoln", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88348", "FIPS": "35027", "County": "Lincoln", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88351", "FIPS": "35027", "County": "Lincoln", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88355", "FIPS": "35027", "County": "Lincoln", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87544", "FIPS": "35028", "County": "Los Alamos", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87545", "FIPS": "35028", "County": "Los Alamos", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88029", "FIPS": "35029", "County": "Luna", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88030", "FIPS": "35029", "County": "Luna", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88031", "FIPS": "35029", "County": "Luna", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87045", "FIPS": "35031", "County": "McKinley", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87301", "FIPS": "35031", "County": "McKinley", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87302", "FIPS": "35031", "County": "McKinley", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87305", "FIPS": "35031", "County": "McKinley", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87310", "FIPS": "35031", "County": "McKinley", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87311", "FIPS": "35031", "County": "McKinley", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87312", "FIPS": "35031", "County": "McKinley", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87313", "FIPS": "35031", "County": "McKinley", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87316", "FIPS": "35031", "County": "McKinley", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87317", "FIPS": "35031", "County": "McKinley", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87319", "FIPS": "35031", "County": "McKinley", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87320", "FIPS": "35031", "County": "McKinley", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87321", "FIPS": "35031", "County": "McKinley", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87322", "FIPS": "35031", "County": "McKinley", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87323", "FIPS": "35031", "County": "McKinley", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87325", "FIPS": "35031", "County": "McKinley", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87326", "FIPS": "35031", "County": "McKinley", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87327", "FIPS": "35031", "County": "McKinley", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87328", "FIPS": "35031", "County": "McKinley", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87347", "FIPS": "35031", "County": "McKinley", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87365", "FIPS": "35031", "County": "McKinley", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87375", "FIPS": "35031", "County": "McKinley", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87712", "FIPS": "35033", "County": "Mora", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87713", "FIPS": "35033", "County": "Mora", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87715", "FIPS": "35033", "County": "Mora", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87722", "FIPS": "35033", "County": "Mora", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87723", "FIPS": "35033", "County": "Mora", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87732", "FIPS": "35033", "County": "Mora", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87734", "FIPS": "35033", "County": "Mora", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87735", "FIPS": "35033", "County": "Mora", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87736", "FIPS": "35033", "County": "Mora", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87750", "FIPS": "35033", "County": "Mora", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87752", "FIPS": "35033", "County": "Mora", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87753", "FIPS": "35033", "County": "Mora", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88310", "FIPS": "35035", "County": "Otero", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88311", "FIPS": "35035", "County": "Otero", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88314", "FIPS": "35035", "County": "Otero", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88317", "FIPS": "35035", "County": "Otero", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88325", "FIPS": "35035", "County": "Otero", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88330", "FIPS": "35035", "County": "Otero", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88337", "FIPS": "35035", "County": "Otero", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88339", "FIPS": "35035", "County": "Otero", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88340", "FIPS": "35035", "County": "Otero", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88342", "FIPS": "35035", "County": "Otero", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88344", "FIPS": "35035", "County": "Otero", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88347", "FIPS": "35035", "County": "Otero", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88349", "FIPS": "35035", "County": "Otero", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88350", "FIPS": "35035", "County": "Otero", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88352", "FIPS": "35035", "County": "Otero", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88354", "FIPS": "35035", "County": "Otero", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88121", "FIPS": "35037", "County": "Quay", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88401", "FIPS": "35037", "County": "Quay", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88411", "FIPS": "35037", "County": "Quay", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88426", "FIPS": "35037", "County": "Quay", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88427", "FIPS": "35037", "County": "Quay", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88430", "FIPS": "35037", "County": "Quay", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88433", "FIPS": "35037", "County": "Quay", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88434", "FIPS": "35037", "County": "Quay", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "87012", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87017", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87029", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87064", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87510", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87511", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87515", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87516", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87518", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87520", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87522", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87523", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87527", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87528", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87530", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87531", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87532", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87533", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87537", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87539", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87548", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87551", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87554", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87566", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87575", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87578", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87581", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87582", "FIPS": "35039", "County": "Rio Arriba", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88113", "FIPS": "35041", "County": "Roosevelt", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88115", "FIPS": "35041", "County": "Roosevelt", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88116", "FIPS": "35041", "County": "Roosevelt", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88118", "FIPS": "35041", "County": "Roosevelt", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88122", "FIPS": "35041", "County": "Roosevelt", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88123", "FIPS": "35041", "County": "Roosevelt", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88125", "FIPS": "35041", "County": "Roosevelt", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88126", "FIPS": "35041", "County": "Roosevelt", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88130", "FIPS": "35041", "County": "Roosevelt", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88132", "FIPS": "35041", "County": "Roosevelt", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "87001", "FIPS": "35043", "County": "Sandoval", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87004", "FIPS": "35043", "County": "Sandoval", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87013", "FIPS": "35043", "County": "Sandoval", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87018", "FIPS": "35043", "County": "Sandoval", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87024", "FIPS": "35043", "County": "Sandoval", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87025", "FIPS": "35043", "County": "Sandoval", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87027", "FIPS": "35043", "County": "Sandoval", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87041", "FIPS": "35043", "County": "Sandoval", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87043", "FIPS": "35043", "County": "Sandoval", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87044", "FIPS": "35043", "County": "Sandoval", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87046", "FIPS": "35043", "County": "Sandoval", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87048", "FIPS": "35043", "County": "Sandoval", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87052", "FIPS": "35043", "County": "Sandoval", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87053", "FIPS": "35043", "County": "Sandoval", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87072", "FIPS": "35043", "County": "Sandoval", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87083", "FIPS": "35043", "County": "Sandoval", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87124", "FIPS": "35043", "County": "Sandoval", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87144", "FIPS": "35043", "County": "Sandoval", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87165", "FIPS": "35043", "County": "Sandoval", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87174", "FIPS": "35043", "County": "Sandoval", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87037", "FIPS": "35045", "County": "San Juan", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87364", "FIPS": "35045", "County": "San Juan", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87401", "FIPS": "35045", "County": "San Juan", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87402", "FIPS": "35045", "County": "San Juan", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87410", "FIPS": "35045", "County": "San Juan", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87412", "FIPS": "35045", "County": "San Juan", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87413", "FIPS": "35045", "County": "San Juan", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87415", "FIPS": "35045", "County": "San Juan", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87416", "FIPS": "35045", "County": "San Juan", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87417", "FIPS": "35045", "County": "San Juan", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87418", "FIPS": "35045", "County": "San Juan", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87419", "FIPS": "35045", "County": "San Juan", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87420", "FIPS": "35045", "County": "San Juan", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87421", "FIPS": "35045", "County": "San Juan", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87455", "FIPS": "35045", "County": "San Juan", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87461", "FIPS": "35045", "County": "San Juan", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87499", "FIPS": "35045", "County": "San Juan", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87538", "FIPS": "35047", "County": "San Miguel", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87552", "FIPS": "35047", "County": "San Miguel", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87560", "FIPS": "35047", "County": "San Miguel", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87562", "FIPS": "35047", "County": "San Miguel", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87565", "FIPS": "35047", "County": "San Miguel", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87569", "FIPS": "35047", "County": "San Miguel", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87573", "FIPS": "35047", "County": "San Miguel", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87583", "FIPS": "35047", "County": "San Miguel", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87701", "FIPS": "35047", "County": "San Miguel", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87731", "FIPS": "35047", "County": "San Miguel", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87742", "FIPS": "35047", "County": "San Miguel", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87745", "FIPS": "35047", "County": "San Miguel", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88416", "FIPS": "35047", "County": "San Miguel", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88421", "FIPS": "35047", "County": "San Miguel", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88439", "FIPS": "35047", "County": "San Miguel", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87010", "FIPS": "35049", "County": "Santa Fe", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87015", "FIPS": "35049", "County": "Santa Fe", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87056", "FIPS": "35049", "County": "Santa Fe", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87501", "FIPS": "35049", "County": "Santa Fe", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87502", "FIPS": "35049", "County": "Santa Fe", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87503", "FIPS": "35049", "County": "Santa Fe", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87504", "FIPS": "35049", "County": "Santa Fe", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87505", "FIPS": "35049", "County": "Santa Fe", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87506", "FIPS": "35049", "County": "Santa Fe", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87507", "FIPS": "35049", "County": "Santa Fe", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87508", "FIPS": "35049", "County": "Santa Fe", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87509", "FIPS": "35049", "County": "Santa Fe", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87535", "FIPS": "35049", "County": "Santa Fe", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87540", "FIPS": "35049", "County": "Santa Fe", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87567", "FIPS": "35049", "County": "Santa Fe", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87574", "FIPS": "35049", "County": "Santa Fe", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87592", "FIPS": "35049", "County": "Santa Fe", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87594", "FIPS": "35049", "County": "Santa Fe", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87901", "FIPS": "35051", "County": "Sierra", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87930", "FIPS": "35051", "County": "Sierra", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87931", "FIPS": "35051", "County": "Sierra", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87933", "FIPS": "35051", "County": "Sierra", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87935", "FIPS": "35051", "County": "Sierra", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87939", "FIPS": "35051", "County": "Sierra", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87942", "FIPS": "35051", "County": "Sierra", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87943", "FIPS": "35051", "County": "Sierra", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88042", "FIPS": "35051", "County": "Sierra", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87011", "FIPS": "35053", "County": "Socorro", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87028", "FIPS": "35053", "County": "Socorro", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87062", "FIPS": "35053", "County": "Socorro", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87801", "FIPS": "35053", "County": "Socorro", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87823", "FIPS": "35053", "County": "Socorro", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87825", "FIPS": "35053", "County": "Socorro", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87828", "FIPS": "35053", "County": "Socorro", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87831", "FIPS": "35053", "County": "Socorro", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87832", "FIPS": "35053", "County": "Socorro", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87512", "FIPS": "35055", "County": "Taos", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87513", "FIPS": "35055", "County": "Taos", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87514", "FIPS": "35055", "County": "Taos", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87517", "FIPS": "35055", "County": "Taos", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87519", "FIPS": "35055", "County": "Taos", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87521", "FIPS": "35055", "County": "Taos", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87524", "FIPS": "35055", "County": "Taos", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87525", "FIPS": "35055", "County": "Taos", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87529", "FIPS": "35055", "County": "Taos", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87543", "FIPS": "35055", "County": "Taos", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87549", "FIPS": "35055", "County": "Taos", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87553", "FIPS": "35055", "County": "Taos", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87556", "FIPS": "35055", "County": "Taos", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87557", "FIPS": "35055", "County": "Taos", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87558", "FIPS": "35055", "County": "Taos", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87564", "FIPS": "35055", "County": "Taos", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87571", "FIPS": "35055", "County": "Taos", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87576", "FIPS": "35055", "County": "Taos", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87577", "FIPS": "35055", "County": "Taos", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87579", "FIPS": "35055", "County": "Taos", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87580", "FIPS": "35055", "County": "Taos", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87009", "FIPS": "35057", "County": "Torrance", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87016", "FIPS": "35057", "County": "Torrance", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87032", "FIPS": "35057", "County": "Torrance", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87035", "FIPS": "35057", "County": "Torrance", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87036", "FIPS": "35057", "County": "Torrance", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87061", "FIPS": "35057", "County": "Torrance", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87063", "FIPS": "35057", "County": "Torrance", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87070", "FIPS": "35057", "County": "Torrance", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88321", "FIPS": "35057", "County": "Torrance", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "88410", "FIPS": "35059", "County": "Union", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88414", "FIPS": "35059", "County": "Union", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88415", "FIPS": "35059", "County": "Union", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88418", "FIPS": "35059", "County": "Union", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88419", "FIPS": "35059", "County": "Union", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88422", "FIPS": "35059", "County": "Union", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88424", "FIPS": "35059", "County": "Union", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "88436", "FIPS": "35059", "County": "Union", "State": "NM", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "87002", "FIPS": "35061", "County": "Valencia", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87006", "FIPS": "35061", "County": "Valencia", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87023", "FIPS": "35061", "County": "Valencia", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87031", "FIPS": "35061", "County": "Valencia", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87034", "FIPS": "35061", "County": "Valencia", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87042", "FIPS": "35061", "County": "Valencia", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87060", "FIPS": "35061", "County": "Valencia", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "87068", "FIPS": "35061", "County": "Valencia", "State": "NM", "DMA": {"Code": 790, "Name": "ALBUQUERQUE-SANTA FE"}},
  {"Postal_Code": "12007", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12009", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12023", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12041", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12045", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12046", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12047", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12054", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12055", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12059", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12067", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12077", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12084", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12085", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12107", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12110", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12120", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12128", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12143", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12147", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12158", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12159", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12161", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12183", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12186", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12189", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12193", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12201", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12202", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12203", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12204", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12205", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12206", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12207", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12208", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12209", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12210", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12211", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12212", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12214", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12220", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12222", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12223", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12224", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12225", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12226", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12227", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12228", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12229", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12230", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12231", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12232", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12233", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12234", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12235", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12236", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12237", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12238", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12239", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12240", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12241", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12242", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12243", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12244", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12245", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12246", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12247", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12248", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12249", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12250", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12252", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12255", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12256", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12257", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12260", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12261", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12288", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12469", "FIPS": "36001", "County": "Albany", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "14029", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14707", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14708", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14709", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14711", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14714", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14715", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14717", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14721", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14727", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14735", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14739", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14744", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14745", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14754", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14774", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14777", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14786", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14802", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14803", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14804", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14806", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14813", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14822", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14880", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14884", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14895", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14897", "FIPS": "36003", "County": "Allegany", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "10451", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10452", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10453", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10454", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10455", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10456", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10457", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10458", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10459", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10460", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10461", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10462", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10463", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10464", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10465", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10466", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10467", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10468", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10469", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10470", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10471", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10472", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10473", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10474", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10475", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10499", "FIPS": "36005", "County": "Bronx", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "13737", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13744", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13745", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13746", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13748", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13749", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13754", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13760", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13761", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13762", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13763", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13777", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13787", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13790", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13794", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13795", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13797", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13802", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13813", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13826", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13833", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13848", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13850", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13851", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13862", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13865", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13901", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13902", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13903", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13904", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13905", "FIPS": "36007", "County": "Broome", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "14041", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14042", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14060", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14065", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14070", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14101", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14129", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14133", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14138", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14168", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14171", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14173", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14706", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14719", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14726", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14729", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14730", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14731", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14737", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14741", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14743", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14748", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14751", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14753", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14755", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14760", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14766", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14770", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14772", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14778", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14779", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14783", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14788", "FIPS": "36009", "County": "Cattaraugus", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "13021", "FIPS": "36011", "County": "Cayuga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13022", "FIPS": "36011", "County": "Cayuga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13024", "FIPS": "36011", "County": "Cayuga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13026", "FIPS": "36011", "County": "Cayuga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13033", "FIPS": "36011", "County": "Cayuga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13034", "FIPS": "36011", "County": "Cayuga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13064", "FIPS": "36011", "County": "Cayuga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13071", "FIPS": "36011", "County": "Cayuga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13081", "FIPS": "36011", "County": "Cayuga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13111", "FIPS": "36011", "County": "Cayuga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13113", "FIPS": "36011", "County": "Cayuga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13117", "FIPS": "36011", "County": "Cayuga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13118", "FIPS": "36011", "County": "Cayuga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13139", "FIPS": "36011", "County": "Cayuga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13140", "FIPS": "36011", "County": "Cayuga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13147", "FIPS": "36011", "County": "Cayuga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13156", "FIPS": "36011", "County": "Cayuga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13160", "FIPS": "36011", "County": "Cayuga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13166", "FIPS": "36011", "County": "Cayuga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "14048", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14062", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14063", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14081", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14135", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14136", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14166", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14701", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14702", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14710", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14712", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14716", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14718", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14720", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14722", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14723", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14724", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14728", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14732", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14733", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14736", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14738", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14740", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14742", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14747", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14750", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14752", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14756", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14757", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14758", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14767", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14769", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14775", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14781", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14782", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14784", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14785", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14787", "FIPS": "36013", "County": "Chautauqua", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14814", "FIPS": "36015", "County": "Chemung", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14816", "FIPS": "36015", "County": "Chemung", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14825", "FIPS": "36015", "County": "Chemung", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14838", "FIPS": "36015", "County": "Chemung", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14845", "FIPS": "36015", "County": "Chemung", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14861", "FIPS": "36015", "County": "Chemung", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14864", "FIPS": "36015", "County": "Chemung", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14871", "FIPS": "36015", "County": "Chemung", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14872", "FIPS": "36015", "County": "Chemung", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14889", "FIPS": "36015", "County": "Chemung", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14894", "FIPS": "36015", "County": "Chemung", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14901", "FIPS": "36015", "County": "Chemung", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14902", "FIPS": "36015", "County": "Chemung", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14903", "FIPS": "36015", "County": "Chemung", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14904", "FIPS": "36015", "County": "Chemung", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14905", "FIPS": "36015", "County": "Chemung", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14925", "FIPS": "36015", "County": "Chemung", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "13124", "FIPS": "36017", "County": "Chenango", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13136", "FIPS": "36017", "County": "Chenango", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13155", "FIPS": "36017", "County": "Chenango", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13332", "FIPS": "36017", "County": "Chenango", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13411", "FIPS": "36017", "County": "Chenango", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13460", "FIPS": "36017", "County": "Chenango", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13464", "FIPS": "36017", "County": "Chenango", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13730", "FIPS": "36017", "County": "Chenango", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13733", "FIPS": "36017", "County": "Chenango", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13758", "FIPS": "36017", "County": "Chenango", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13778", "FIPS": "36017", "County": "Chenango", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13780", "FIPS": "36017", "County": "Chenango", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13801", "FIPS": "36017", "County": "Chenango", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13809", "FIPS": "36017", "County": "Chenango", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13814", "FIPS": "36017", "County": "Chenango", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13815", "FIPS": "36017", "County": "Chenango", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13830", "FIPS": "36017", "County": "Chenango", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13832", "FIPS": "36017", "County": "Chenango", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13841", "FIPS": "36017", "County": "Chenango", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13843", "FIPS": "36017", "County": "Chenango", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13844", "FIPS": "36017", "County": "Chenango", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "12901", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12903", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12910", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12911", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12912", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12918", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12919", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12921", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12923", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12924", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12929", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12933", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12934", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12935", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12944", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12952", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12955", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12958", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12959", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12962", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12972", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12978", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12979", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12981", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12985", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12992", "FIPS": "36019", "County": "Clinton", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12017", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12029", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12037", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12050", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12060", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12062", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12075", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12106", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12115", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12125", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12130", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12132", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12136", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12165", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12172", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12173", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12174", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12184", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12195", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12502", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12503", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12513", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12516", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12517", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12521", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12523", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12526", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12529", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12530", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12534", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12541", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12544", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12565", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12593", "FIPS": "36021", "County": "Columbia", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "13040", "FIPS": "36023", "County": "Cortland", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13045", "FIPS": "36023", "County": "Cortland", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13056", "FIPS": "36023", "County": "Cortland", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13077", "FIPS": "36023", "County": "Cortland", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13087", "FIPS": "36023", "County": "Cortland", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13101", "FIPS": "36023", "County": "Cortland", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13141", "FIPS": "36023", "County": "Cortland", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13158", "FIPS": "36023", "County": "Cortland", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13738", "FIPS": "36023", "County": "Cortland", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13784", "FIPS": "36023", "County": "Cortland", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13803", "FIPS": "36023", "County": "Cortland", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13863", "FIPS": "36023", "County": "Cortland", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "12167", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "12406", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "12421", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "12430", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "12434", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "12438", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "12455", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "12459", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "12474", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13731", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13739", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13740", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13750", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13751", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13752", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13753", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13755", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13756", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13757", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13774", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13775", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13782", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13783", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13786", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13788", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13804", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13806", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13837", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13838", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13839", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13842", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13846", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13847", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13856", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13860", "FIPS": "36025", "County": "Delaware", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "12501", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12504", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12506", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12507", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12508", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12510", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12511", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12512", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12514", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12522", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12524", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12527", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12531", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12533", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12537", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12538", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12540", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12545", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12546", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12564", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12567", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12569", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12570", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12571", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12572", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12574", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12578", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12580", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12581", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12582", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12583", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12585", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12590", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12592", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12594", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12601", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12602", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12603", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12604", "FIPS": "36027", "County": "Dutchess", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "14001", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14004", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14006", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14010", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14025", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14026", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14027", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14030", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14031", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14032", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14033", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14034", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14035", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14038", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14043", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14047", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14051", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14052", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14055", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14057", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14059", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14061", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14068", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14069", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14072", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14075", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14080", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14085", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14086", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14091", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14102", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14110", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14111", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14112", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14127", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14134", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14139", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14140", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14141", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14150", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14151", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14169", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14170", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14201", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14202", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14203", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14204", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14205", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14206", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14207", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14208", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14209", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14210", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14211", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14212", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14213", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14214", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14215", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14216", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14217", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14218", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14219", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14220", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14221", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14222", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14223", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14224", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14225", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14226", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14227", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14228", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14231", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14233", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14240", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14241", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14260", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14261", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14263", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14264", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14265", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14267", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14269", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14270", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14272", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14273", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14276", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14280", "FIPS": "36029", "County": "Erie", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "12851", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12852", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12855", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12857", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12858", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12870", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12872", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12879", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12883", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12913", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12928", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12932", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12936", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12941", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12942", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12943", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12946", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12950", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12956", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12960", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12961", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12964", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12974", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12975", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12977", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12987", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12993", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12996", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12997", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12998", "FIPS": "36031", "County": "Essex", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12914", "FIPS": "36033", "County": "Franklin", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12915", "FIPS": "36033", "County": "Franklin", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12916", "FIPS": "36033", "County": "Franklin", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12917", "FIPS": "36033", "County": "Franklin", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12920", "FIPS": "36033", "County": "Franklin", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12926", "FIPS": "36033", "County": "Franklin", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12930", "FIPS": "36033", "County": "Franklin", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12937", "FIPS": "36033", "County": "Franklin", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12939", "FIPS": "36033", "County": "Franklin", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12945", "FIPS": "36033", "County": "Franklin", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12953", "FIPS": "36033", "County": "Franklin", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12957", "FIPS": "36033", "County": "Franklin", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12966", "FIPS": "36033", "County": "Franklin", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12969", "FIPS": "36033", "County": "Franklin", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12970", "FIPS": "36033", "County": "Franklin", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12976", "FIPS": "36033", "County": "Franklin", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12980", "FIPS": "36033", "County": "Franklin", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12983", "FIPS": "36033", "County": "Franklin", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12986", "FIPS": "36033", "County": "Franklin", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12989", "FIPS": "36033", "County": "Franklin", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12995", "FIPS": "36033", "County": "Franklin", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "13655", "FIPS": "36033", "County": "Franklin", "State": "NY", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "12025", "FIPS": "36035", "County": "Fulton", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12032", "FIPS": "36035", "County": "Fulton", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12078", "FIPS": "36035", "County": "Fulton", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12095", "FIPS": "36035", "County": "Fulton", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12117", "FIPS": "36035", "County": "Fulton", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12134", "FIPS": "36035", "County": "Fulton", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "13470", "FIPS": "36035", "County": "Fulton", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "14005", "FIPS": "36037", "County": "Genesee", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14013", "FIPS": "36037", "County": "Genesee", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14020", "FIPS": "36037", "County": "Genesee", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14021", "FIPS": "36037", "County": "Genesee", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14036", "FIPS": "36037", "County": "Genesee", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14040", "FIPS": "36037", "County": "Genesee", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14054", "FIPS": "36037", "County": "Genesee", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14056", "FIPS": "36037", "County": "Genesee", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14058", "FIPS": "36037", "County": "Genesee", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14125", "FIPS": "36037", "County": "Genesee", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14143", "FIPS": "36037", "County": "Genesee", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14416", "FIPS": "36037", "County": "Genesee", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14422", "FIPS": "36037", "County": "Genesee", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14482", "FIPS": "36037", "County": "Genesee", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14486", "FIPS": "36037", "County": "Genesee", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14525", "FIPS": "36037", "County": "Genesee", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14557", "FIPS": "36037", "County": "Genesee", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "12015", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12042", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12051", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12058", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12083", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12087", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12124", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12176", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12192", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12405", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12407", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12413", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12414", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12418", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12422", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12423", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12424", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12427", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12431", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12436", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12439", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12442", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12444", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12450", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12451", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12452", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12454", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12460", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12463", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12468", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12470", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12473", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12482", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12485", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12492", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12496", "FIPS": "36039", "County": "Greene", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12108", "FIPS": "36041", "County": "Hamilton", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12139", "FIPS": "36041", "County": "Hamilton", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12164", "FIPS": "36041", "County": "Hamilton", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12190", "FIPS": "36041", "County": "Hamilton", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12812", "FIPS": "36041", "County": "Hamilton", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12842", "FIPS": "36041", "County": "Hamilton", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12847", "FIPS": "36041", "County": "Hamilton", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12864", "FIPS": "36041", "County": "Hamilton", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "13353", "FIPS": "36041", "County": "Hamilton", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "13360", "FIPS": "36041", "County": "Hamilton", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "13436", "FIPS": "36041", "County": "Hamilton", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "13324", "FIPS": "36043", "County": "Herkimer", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13329", "FIPS": "36043", "County": "Herkimer", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13331", "FIPS": "36043", "County": "Herkimer", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13340", "FIPS": "36043", "County": "Herkimer", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13350", "FIPS": "36043", "County": "Herkimer", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13357", "FIPS": "36043", "County": "Herkimer", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13361", "FIPS": "36043", "County": "Herkimer", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13365", "FIPS": "36043", "County": "Herkimer", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13406", "FIPS": "36043", "County": "Herkimer", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13407", "FIPS": "36043", "County": "Herkimer", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13416", "FIPS": "36043", "County": "Herkimer", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13420", "FIPS": "36043", "County": "Herkimer", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13431", "FIPS": "36043", "County": "Herkimer", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13454", "FIPS": "36043", "County": "Herkimer", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13472", "FIPS": "36043", "County": "Herkimer", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13475", "FIPS": "36043", "County": "Herkimer", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13491", "FIPS": "36043", "County": "Herkimer", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13601", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13602", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13603", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13605", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13606", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13607", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13608", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13611", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13612", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13615", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13616", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13618", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13619", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13622", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13624", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13628", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13632", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13634", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13636", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13637", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13638", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13640", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13641", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13643", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13650", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13651", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13656", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13657", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13659", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13661", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13665", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13671", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13673", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13674", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13675", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13679", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13682", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13685", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13691", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13692", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13693", "FIPS": "36045", "County": "Jefferson", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "11201", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11202", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11203", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11204", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11205", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11206", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11207", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11208", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11209", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11210", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11211", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11212", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11213", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11214", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11215", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11216", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11217", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11218", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11219", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11220", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11221", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11222", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11223", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11224", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11225", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11226", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11228", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11229", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11230", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11231", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11232", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11233", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11234", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11235", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11236", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11237", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11238", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11239", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11240", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11241", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11242", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11243", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11244", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11245", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11247", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11248", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11249", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11251", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11252", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11254", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11255", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11256", "FIPS": "36047", "County": "Kings", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "13305", "FIPS": "36049", "County": "Lewis", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13312", "FIPS": "36049", "County": "Lewis", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13325", "FIPS": "36049", "County": "Lewis", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13327", "FIPS": "36049", "County": "Lewis", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13343", "FIPS": "36049", "County": "Lewis", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13345", "FIPS": "36049", "County": "Lewis", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13367", "FIPS": "36049", "County": "Lewis", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13368", "FIPS": "36049", "County": "Lewis", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13404", "FIPS": "36049", "County": "Lewis", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13433", "FIPS": "36049", "County": "Lewis", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13473", "FIPS": "36049", "County": "Lewis", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13489", "FIPS": "36049", "County": "Lewis", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13620", "FIPS": "36049", "County": "Lewis", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13626", "FIPS": "36049", "County": "Lewis", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13627", "FIPS": "36049", "County": "Lewis", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13631", "FIPS": "36049", "County": "Lewis", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13648", "FIPS": "36049", "County": "Lewis", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "14414", "FIPS": "36051", "County": "Livingston", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14423", "FIPS": "36051", "County": "Livingston", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14435", "FIPS": "36051", "County": "Livingston", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14437", "FIPS": "36051", "County": "Livingston", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14454", "FIPS": "36051", "County": "Livingston", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14462", "FIPS": "36051", "County": "Livingston", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14466", "FIPS": "36051", "County": "Livingston", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14480", "FIPS": "36051", "County": "Livingston", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14481", "FIPS": "36051", "County": "Livingston", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14485", "FIPS": "36051", "County": "Livingston", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14487", "FIPS": "36051", "County": "Livingston", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14488", "FIPS": "36051", "County": "Livingston", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14510", "FIPS": "36051", "County": "Livingston", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14517", "FIPS": "36051", "County": "Livingston", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14533", "FIPS": "36051", "County": "Livingston", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14539", "FIPS": "36051", "County": "Livingston", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14545", "FIPS": "36051", "County": "Livingston", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14556", "FIPS": "36051", "County": "Livingston", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14558", "FIPS": "36051", "County": "Livingston", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14560", "FIPS": "36051", "County": "Livingston", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14592", "FIPS": "36051", "County": "Livingston", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14836", "FIPS": "36051", "County": "Livingston", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14846", "FIPS": "36051", "County": "Livingston", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "13032", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13035", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13037", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13043", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13052", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13061", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13072", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13082", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13122", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13134", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13163", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13310", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13314", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13334", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13346", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13355", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13364", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13402", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13408", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13409", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13418", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13421", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13465", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13484", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13485", "FIPS": "36053", "County": "Madison", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "14410", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14420", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14428", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14430", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14445", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14450", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14464", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14467", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14468", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14472", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14506", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14511", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14514", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14515", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14526", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14534", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14543", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14546", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14559", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14580", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14586", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14602", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14603", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14604", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14605", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14606", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14607", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14608", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14609", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14610", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14611", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14612", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14613", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14614", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14615", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14616", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14617", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14618", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14619", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14620", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14621", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14622", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14623", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14624", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14625", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14626", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14627", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14638", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14639", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14642", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14643", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14644", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14645", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14646", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14647", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14649", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14650", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14651", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14652", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14653", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14664", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14673", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14683", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14692", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14694", "FIPS": "36055", "County": "Monroe", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "12010", "FIPS": "36057", "County": "Montgomery", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12016", "FIPS": "36057", "County": "Montgomery", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12066", "FIPS": "36057", "County": "Montgomery", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12068", "FIPS": "36057", "County": "Montgomery", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12069", "FIPS": "36057", "County": "Montgomery", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12070", "FIPS": "36057", "County": "Montgomery", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12072", "FIPS": "36057", "County": "Montgomery", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12086", "FIPS": "36057", "County": "Montgomery", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12166", "FIPS": "36057", "County": "Montgomery", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12177", "FIPS": "36057", "County": "Montgomery", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "13317", "FIPS": "36057", "County": "Montgomery", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "13339", "FIPS": "36057", "County": "Montgomery", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "13410", "FIPS": "36057", "County": "Montgomery", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "13428", "FIPS": "36057", "County": "Montgomery", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "13452", "FIPS": "36057", "County": "Montgomery", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "11001", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11002", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11003", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11010", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11020", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11021", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11022", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11023", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11024", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11025", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11026", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11027", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11030", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11040", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11041", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11042", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11043", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11044", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11050", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11051", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11052", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11053", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11054", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11055", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11096", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11099", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11501", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11507", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11509", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11510", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11514", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11516", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11518", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11520", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11530", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11531", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11535", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11536", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11542", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11545", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11547", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11548", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11549", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11550", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11551", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11552", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11553", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11554", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11555", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11556", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11557", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11558", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11559", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11560", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11561", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11563", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11565", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11566", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11568", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11569", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11570", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11571", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11572", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11575", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11576", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11577", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11579", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11580", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11581", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11582", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11590", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11592", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11594", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11595", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11596", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11597", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11598", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11599", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11709", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11710", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11714", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11732", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11735", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11736", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11737", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11753", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11756", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11758", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11762", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11765", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11771", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11773", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11774", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11783", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11791", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11793", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11797", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11801", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11802", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11803", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11804", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11815", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11819", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11853", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11854", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11855", "FIPS": "36059", "County": "Nassau", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10001", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10002", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10003", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10004", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10005", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10006", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10007", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10008", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10009", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10010", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10011", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10012", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10013", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10014", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10015", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10016", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10017", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10018", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10019", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10020", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10021", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10022", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10023", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10024", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10025", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10026", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10027", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10028", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10029", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10030", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10031", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10032", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10033", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10034", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10035", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10036", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10037", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10038", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10039", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10040", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10041", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10043", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10044", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10045", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10046", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10047", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10048", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10055", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10060", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10069", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10072", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10079", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10080", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10081", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10082", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10087", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10090", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10094", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10095", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10096", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10098", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10099", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10101", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10102", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10103", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10104", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10105", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10106", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10107", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10108", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10109", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10110", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10111", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10112", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10113", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10114", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10115", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10116", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10117", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10118", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10119", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10120", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10121", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10122", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10123", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10124", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10125", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10126", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10128", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10129", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10130", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10131", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10132", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10133", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10138", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10149", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10150", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10151", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10152", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10153", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10154", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10155", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10156", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10157", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10158", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10159", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10160", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10161", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10162", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10163", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10164", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10165", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10166", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10167", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10168", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10169", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10170", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10171", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10172", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10173", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10174", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10175", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10176", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10177", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10178", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10179", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10184", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10185", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10196", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10197", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10199", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10203", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10211", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10212", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10213", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10242", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10249", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10256", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10257", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10258", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10259", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10260", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10261", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10265", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10268", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10269", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10270", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10271", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10272", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10273", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10274", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10275", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10276", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10277", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10278", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10279", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10280", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10281", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10282", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10285", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10286", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10292", "FIPS": "36061", "County": "New York", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "14008", "FIPS": "36063", "County": "Niagara", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14012", "FIPS": "36063", "County": "Niagara", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14028", "FIPS": "36063", "County": "Niagara", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14067", "FIPS": "36063", "County": "Niagara", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14092", "FIPS": "36063", "County": "Niagara", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14094", "FIPS": "36063", "County": "Niagara", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14095", "FIPS": "36063", "County": "Niagara", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14105", "FIPS": "36063", "County": "Niagara", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14107", "FIPS": "36063", "County": "Niagara", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14108", "FIPS": "36063", "County": "Niagara", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14109", "FIPS": "36063", "County": "Niagara", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14120", "FIPS": "36063", "County": "Niagara", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14126", "FIPS": "36063", "County": "Niagara", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14131", "FIPS": "36063", "County": "Niagara", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14132", "FIPS": "36063", "County": "Niagara", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14144", "FIPS": "36063", "County": "Niagara", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14172", "FIPS": "36063", "County": "Niagara", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14174", "FIPS": "36063", "County": "Niagara", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14301", "FIPS": "36063", "County": "Niagara", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14302", "FIPS": "36063", "County": "Niagara", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14303", "FIPS": "36063", "County": "Niagara", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14304", "FIPS": "36063", "County": "Niagara", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14305", "FIPS": "36063", "County": "Niagara", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "13054", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13123", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13157", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13162", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13301", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13303", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13304", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13308", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13309", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13313", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13316", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13318", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13319", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13321", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13322", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13323", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13328", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13338", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13341", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13352", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13354", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13362", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13363", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13401", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13403", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13413", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13417", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13424", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13425", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13435", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13438", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13440", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13441", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13442", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13449", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13455", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13456", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13461", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13469", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13471", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13476", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13477", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13478", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13479", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13480", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13483", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13486", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13490", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13492", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13494", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13495", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13501", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13502", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13503", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13504", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13505", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13599", "FIPS": "36065", "County": "Oneida", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13020", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13027", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13029", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13030", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13031", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13039", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13041", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13051", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13057", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13060", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13063", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13066", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13078", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13080", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13084", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13088", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13089", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13090", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13104", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13108", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13110", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13112", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13116", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13119", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13120", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13137", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13138", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13152", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13153", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13159", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13164", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13201", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13202", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13203", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13204", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13205", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13206", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13207", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13208", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13209", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13210", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13211", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13212", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13214", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13215", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13217", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13218", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13219", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13220", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13221", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13224", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13225", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13235", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13244", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13250", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13251", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13252", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13261", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13290", "FIPS": "36067", "County": "Onondaga", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "14424", "FIPS": "36069", "County": "Ontario", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14425", "FIPS": "36069", "County": "Ontario", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14432", "FIPS": "36069", "County": "Ontario", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14443", "FIPS": "36069", "County": "Ontario", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14453", "FIPS": "36069", "County": "Ontario", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14456", "FIPS": "36069", "County": "Ontario", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14461", "FIPS": "36069", "County": "Ontario", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14463", "FIPS": "36069", "County": "Ontario", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14469", "FIPS": "36069", "County": "Ontario", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14471", "FIPS": "36069", "County": "Ontario", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14475", "FIPS": "36069", "County": "Ontario", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14504", "FIPS": "36069", "County": "Ontario", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14512", "FIPS": "36069", "County": "Ontario", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14518", "FIPS": "36069", "County": "Ontario", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14532", "FIPS": "36069", "County": "Ontario", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14537", "FIPS": "36069", "County": "Ontario", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14547", "FIPS": "36069", "County": "Ontario", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14548", "FIPS": "36069", "County": "Ontario", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14561", "FIPS": "36069", "County": "Ontario", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14564", "FIPS": "36069", "County": "Ontario", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14585", "FIPS": "36069", "County": "Ontario", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "10910", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10912", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10914", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10915", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10916", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10917", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10918", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10919", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10921", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10922", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10924", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10925", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10926", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10928", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10930", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10932", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10933", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10940", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10941", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10943", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10949", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10950", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10953", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10958", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10959", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10963", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10969", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10973", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10975", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10979", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10981", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10985", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10987", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10988", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10990", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10992", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10996", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10997", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10998", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12518", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12520", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12543", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12549", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12550", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12551", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12552", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12553", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12555", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12566", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12575", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12577", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12584", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12586", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12729", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12746", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12771", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12780", "FIPS": "36071", "County": "Orange", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "14098", "FIPS": "36073", "County": "Orleans", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14103", "FIPS": "36073", "County": "Orleans", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14411", "FIPS": "36073", "County": "Orleans", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14429", "FIPS": "36073", "County": "Orleans", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14452", "FIPS": "36073", "County": "Orleans", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14470", "FIPS": "36073", "County": "Orleans", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14476", "FIPS": "36073", "County": "Orleans", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14477", "FIPS": "36073", "County": "Orleans", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14479", "FIPS": "36073", "County": "Orleans", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14508", "FIPS": "36073", "County": "Orleans", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14571", "FIPS": "36073", "County": "Orleans", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "13028", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13036", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13042", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13044", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13069", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13074", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13076", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13083", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13093", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13103", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13107", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13114", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13115", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13121", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13126", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13131", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13132", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13135", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13142", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13144", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13145", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13167", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13302", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13426", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13437", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13493", "FIPS": "36075", "County": "Oswego", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "12064", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "12116", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "12155", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "12197", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13315", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13320", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13326", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13333", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13335", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13337", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13342", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13348", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13415", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13439", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13450", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13457", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13468", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13482", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13488", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13747", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13776", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13796", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13807", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13808", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13810", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13820", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13825", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13834", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13849", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13859", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "13861", "FIPS": "36077", "County": "Otsego", "State": "NY", "DMA": {"Code": 526, "Name": "UTICA"}},
  {"Postal_Code": "10509", "FIPS": "36079", "County": "Putnam", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10512", "FIPS": "36079", "County": "Putnam", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10516", "FIPS": "36079", "County": "Putnam", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10524", "FIPS": "36079", "County": "Putnam", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10537", "FIPS": "36079", "County": "Putnam", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10541", "FIPS": "36079", "County": "Putnam", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10542", "FIPS": "36079", "County": "Putnam", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10579", "FIPS": "36079", "County": "Putnam", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12563", "FIPS": "36079", "County": "Putnam", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11004", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11005", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11101", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11102", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11103", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11104", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11105", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11106", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11109", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11120", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11351", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11352", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11354", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11355", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11356", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11357", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11358", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11359", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11360", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11361", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11362", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11363", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11364", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11365", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11366", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11367", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11368", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11369", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11370", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11371", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11372", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11373", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11374", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11375", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11377", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11378", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11379", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11380", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11381", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11385", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11386", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11390", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11405", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11411", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11412", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11413", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11414", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11415", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11416", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11417", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11418", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11419", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11420", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11421", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11422", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11423", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11424", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11425", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11426", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11427", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11428", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11429", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11430", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11431", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11432", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11433", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11434", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11435", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11436", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11439", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11451", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11499", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11690", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11691", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11692", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11693", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11694", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11695", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11697", "FIPS": "36081", "County": "Queens", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12018", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12022", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12024", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12028", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12033", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12040", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12052", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12057", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12061", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12063", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12082", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12089", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12090", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12094", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12121", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12123", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12133", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12138", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12140", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12144", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12153", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12154", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12156", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12168", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12169", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12180", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12181", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12182", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12185", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12196", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12198", "FIPS": "36083", "County": "Rensselaer", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "10301", "FIPS": "36085", "County": "Richmond", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10302", "FIPS": "36085", "County": "Richmond", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10303", "FIPS": "36085", "County": "Richmond", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10304", "FIPS": "36085", "County": "Richmond", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10305", "FIPS": "36085", "County": "Richmond", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10306", "FIPS": "36085", "County": "Richmond", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10307", "FIPS": "36085", "County": "Richmond", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10308", "FIPS": "36085", "County": "Richmond", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10309", "FIPS": "36085", "County": "Richmond", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10310", "FIPS": "36085", "County": "Richmond", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10311", "FIPS": "36085", "County": "Richmond", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10312", "FIPS": "36085", "County": "Richmond", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10313", "FIPS": "36085", "County": "Richmond", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10314", "FIPS": "36085", "County": "Richmond", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10901", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10911", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10913", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10920", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10923", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10927", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10931", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10952", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10954", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10956", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10960", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10962", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10964", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10965", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10968", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10970", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10974", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10976", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10977", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10980", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10982", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10983", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10984", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10986", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10989", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10993", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10994", "FIPS": "36087", "County": "Rockland", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12922", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "12927", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "12949", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "12965", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "12967", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "12973", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13613", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13614", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13617", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13621", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13623", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13625", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13630", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13633", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13635", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13639", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13642", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13645", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13646", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13647", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13649", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13652", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13654", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13658", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13660", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13662", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13664", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13666", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13667", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13668", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13669", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13670", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13672", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13676", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13677", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13678", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13680", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13681", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13683", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13684", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13687", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13690", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13694", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13695", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13696", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13697", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "13699", "FIPS": "36089", "County": "Saint Lawrence", "State": "NY", "DMA": {"Code": 549, "Name": "WATERTOWN"}},
  {"Postal_Code": "12019", "FIPS": "36091", "County": "Saratoga", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12020", "FIPS": "36091", "County": "Saratoga", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12027", "FIPS": "36091", "County": "Saratoga", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12065", "FIPS": "36091", "County": "Saratoga", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12074", "FIPS": "36091", "County": "Saratoga", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12118", "FIPS": "36091", "County": "Saratoga", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12148", "FIPS": "36091", "County": "Saratoga", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12151", "FIPS": "36091", "County": "Saratoga", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12170", "FIPS": "36091", "County": "Saratoga", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12188", "FIPS": "36091", "County": "Saratoga", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12803", "FIPS": "36091", "County": "Saratoga", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12822", "FIPS": "36091", "County": "Saratoga", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12831", "FIPS": "36091", "County": "Saratoga", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12833", "FIPS": "36091", "County": "Saratoga", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12835", "FIPS": "36091", "County": "Saratoga", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12850", "FIPS": "36091", "County": "Saratoga", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12859", "FIPS": "36091", "County": "Saratoga", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12863", "FIPS": "36091", "County": "Saratoga", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12866", "FIPS": "36091", "County": "Saratoga", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12871", "FIPS": "36091", "County": "Saratoga", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12884", "FIPS": "36091", "County": "Saratoga", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12008", "FIPS": "36093", "County": "Schenectady", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12053", "FIPS": "36093", "County": "Schenectady", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12056", "FIPS": "36093", "County": "Schenectady", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12137", "FIPS": "36093", "County": "Schenectady", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12141", "FIPS": "36093", "County": "Schenectady", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12150", "FIPS": "36093", "County": "Schenectady", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12301", "FIPS": "36093", "County": "Schenectady", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12302", "FIPS": "36093", "County": "Schenectady", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12303", "FIPS": "36093", "County": "Schenectady", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12304", "FIPS": "36093", "County": "Schenectady", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12305", "FIPS": "36093", "County": "Schenectady", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12306", "FIPS": "36093", "County": "Schenectady", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12307", "FIPS": "36093", "County": "Schenectady", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12308", "FIPS": "36093", "County": "Schenectady", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12309", "FIPS": "36093", "County": "Schenectady", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12325", "FIPS": "36093", "County": "Schenectady", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12345", "FIPS": "36093", "County": "Schenectady", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12031", "FIPS": "36095", "County": "Schoharie", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12035", "FIPS": "36095", "County": "Schoharie", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12036", "FIPS": "36095", "County": "Schoharie", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12043", "FIPS": "36095", "County": "Schoharie", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12071", "FIPS": "36095", "County": "Schoharie", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12073", "FIPS": "36095", "County": "Schoharie", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12076", "FIPS": "36095", "County": "Schoharie", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12092", "FIPS": "36095", "County": "Schoharie", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12093", "FIPS": "36095", "County": "Schoharie", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12122", "FIPS": "36095", "County": "Schoharie", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12131", "FIPS": "36095", "County": "Schoharie", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12149", "FIPS": "36095", "County": "Schoharie", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12157", "FIPS": "36095", "County": "Schoharie", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12160", "FIPS": "36095", "County": "Schoharie", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12175", "FIPS": "36095", "County": "Schoharie", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12187", "FIPS": "36095", "County": "Schoharie", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12194", "FIPS": "36095", "County": "Schoharie", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "13459", "FIPS": "36095", "County": "Schoharie", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "14805", "FIPS": "36097", "County": "Schuyler", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14812", "FIPS": "36097", "County": "Schuyler", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14815", "FIPS": "36097", "County": "Schuyler", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14818", "FIPS": "36097", "County": "Schuyler", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14824", "FIPS": "36097", "County": "Schuyler", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14841", "FIPS": "36097", "County": "Schuyler", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14863", "FIPS": "36097", "County": "Schuyler", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14865", "FIPS": "36097", "County": "Schuyler", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14869", "FIPS": "36097", "County": "Schuyler", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14876", "FIPS": "36097", "County": "Schuyler", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14878", "FIPS": "36097", "County": "Schuyler", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14887", "FIPS": "36097", "County": "Schuyler", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14891", "FIPS": "36097", "County": "Schuyler", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14893", "FIPS": "36097", "County": "Schuyler", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "13065", "FIPS": "36099", "County": "Seneca", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13148", "FIPS": "36099", "County": "Seneca", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13165", "FIPS": "36099", "County": "Seneca", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "14521", "FIPS": "36099", "County": "Seneca", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "14541", "FIPS": "36099", "County": "Seneca", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "14588", "FIPS": "36099", "County": "Seneca", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "14847", "FIPS": "36099", "County": "Seneca", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "14860", "FIPS": "36099", "County": "Seneca", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "14529", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14572", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14801", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14807", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14808", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14809", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14810", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14819", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14820", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14821", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14823", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14826", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14827", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14830", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14831", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14839", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14840", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14843", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14855", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14856", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14858", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14870", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14873", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14874", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14877", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14879", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14885", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "14898", "FIPS": "36101", "County": "Steuben", "State": "NY", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "00501", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "00544", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "06390", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11701", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11702", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11703", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11704", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11705", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11706", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11707", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11708", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11713", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11715", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11716", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11717", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11718", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11719", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11720", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11721", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11722", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11724", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11725", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11726", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11727", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11729", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11730", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11731", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11733", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11738", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11739", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11740", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11741", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11742", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11743", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11746", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11747", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11749", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11750", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11751", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11752", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11754", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11755", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11757", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11760", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11763", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11764", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11766", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11767", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11768", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11769", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11770", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11772", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11775", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11776", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11777", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11778", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11779", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11780", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11782", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11784", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11786", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11787", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11788", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11789", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11790", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11792", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11794", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11795", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11796", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11798", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11901", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11930", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11931", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11932", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11933", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11934", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11935", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11937", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11939", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11940", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11941", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11942", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11944", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11946", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11947", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11948", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11949", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11950", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11951", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11952", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11953", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11954", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11955", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11956", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11957", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11958", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11959", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11960", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11961", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11962", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11963", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11964", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11965", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11967", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11968", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11969", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11970", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11971", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11972", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11973", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11975", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11976", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11977", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11978", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "11980", "FIPS": "36103", "County": "Suffolk", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12701", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12719", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12720", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12721", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12722", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12723", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12724", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12725", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12726", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12727", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12732", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12733", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12734", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12736", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12737", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12738", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12740", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12741", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12742", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12743", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12745", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12747", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12748", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12749", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12750", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12751", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12752", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12754", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12758", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12759", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12760", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12762", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12763", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12764", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12765", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12766", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12767", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12768", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12769", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12770", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12775", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12776", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12777", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12778", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12779", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12781", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12783", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12784", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12785", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12786", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12787", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12788", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12789", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12790", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12791", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12792", "FIPS": "36105", "County": "Sullivan", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "13732", "FIPS": "36107", "County": "Tioga", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13734", "FIPS": "36107", "County": "Tioga", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13736", "FIPS": "36107", "County": "Tioga", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13743", "FIPS": "36107", "County": "Tioga", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13811", "FIPS": "36107", "County": "Tioga", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13812", "FIPS": "36107", "County": "Tioga", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13827", "FIPS": "36107", "County": "Tioga", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13835", "FIPS": "36107", "County": "Tioga", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13840", "FIPS": "36107", "County": "Tioga", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13845", "FIPS": "36107", "County": "Tioga", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13864", "FIPS": "36107", "County": "Tioga", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "14859", "FIPS": "36107", "County": "Tioga", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "14883", "FIPS": "36107", "County": "Tioga", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "14892", "FIPS": "36107", "County": "Tioga", "State": "NY", "DMA": {"Code": 502, "Name": "BINGHAMTON"}},
  {"Postal_Code": "13053", "FIPS": "36109", "County": "Tompkins", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13062", "FIPS": "36109", "County": "Tompkins", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13068", "FIPS": "36109", "County": "Tompkins", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13073", "FIPS": "36109", "County": "Tompkins", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13092", "FIPS": "36109", "County": "Tompkins", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "13102", "FIPS": "36109", "County": "Tompkins", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "14817", "FIPS": "36109", "County": "Tompkins", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "14850", "FIPS": "36109", "County": "Tompkins", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "14851", "FIPS": "36109", "County": "Tompkins", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "14852", "FIPS": "36109", "County": "Tompkins", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "14853", "FIPS": "36109", "County": "Tompkins", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "14854", "FIPS": "36109", "County": "Tompkins", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "14867", "FIPS": "36109", "County": "Tompkins", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "14881", "FIPS": "36109", "County": "Tompkins", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "14882", "FIPS": "36109", "County": "Tompkins", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "14886", "FIPS": "36109", "County": "Tompkins", "State": "NY", "DMA": {"Code": 555, "Name": "SYRACUSE"}},
  {"Postal_Code": "12401", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12402", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12404", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12409", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12410", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12411", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12412", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12416", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12417", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12419", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12420", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12428", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12429", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12432", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12433", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12435", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12440", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12441", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12443", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12446", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12448", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12449", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12453", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12456", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12457", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12458", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12461", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12464", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12465", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12466", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12471", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12472", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12475", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12477", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12480", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12481", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12483", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12484", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12486", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12487", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12489", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12490", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12491", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12493", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12494", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12495", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12498", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12515", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12525", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12528", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12542", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12547", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12548", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12561", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12568", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12588", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12589", "FIPS": "36111", "County": "Ulster", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "12801", "FIPS": "36113", "County": "Warren", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12804", "FIPS": "36113", "County": "Warren", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12808", "FIPS": "36113", "County": "Warren", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12810", "FIPS": "36113", "County": "Warren", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12811", "FIPS": "36113", "County": "Warren", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12814", "FIPS": "36113", "County": "Warren", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12815", "FIPS": "36113", "County": "Warren", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12817", "FIPS": "36113", "County": "Warren", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12820", "FIPS": "36113", "County": "Warren", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12824", "FIPS": "36113", "County": "Warren", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12836", "FIPS": "36113", "County": "Warren", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12843", "FIPS": "36113", "County": "Warren", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12844", "FIPS": "36113", "County": "Warren", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12845", "FIPS": "36113", "County": "Warren", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12846", "FIPS": "36113", "County": "Warren", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12853", "FIPS": "36113", "County": "Warren", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12856", "FIPS": "36113", "County": "Warren", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12860", "FIPS": "36113", "County": "Warren", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12862", "FIPS": "36113", "County": "Warren", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12874", "FIPS": "36113", "County": "Warren", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12878", "FIPS": "36113", "County": "Warren", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12885", "FIPS": "36113", "County": "Warren", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12886", "FIPS": "36113", "County": "Warren", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12809", "FIPS": "36115", "County": "Washington", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12816", "FIPS": "36115", "County": "Washington", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12819", "FIPS": "36115", "County": "Washington", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12821", "FIPS": "36115", "County": "Washington", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12823", "FIPS": "36115", "County": "Washington", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12827", "FIPS": "36115", "County": "Washington", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12828", "FIPS": "36115", "County": "Washington", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12832", "FIPS": "36115", "County": "Washington", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12834", "FIPS": "36115", "County": "Washington", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12837", "FIPS": "36115", "County": "Washington", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12838", "FIPS": "36115", "County": "Washington", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12839", "FIPS": "36115", "County": "Washington", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12841", "FIPS": "36115", "County": "Washington", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12848", "FIPS": "36115", "County": "Washington", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12849", "FIPS": "36115", "County": "Washington", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12854", "FIPS": "36115", "County": "Washington", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12861", "FIPS": "36115", "County": "Washington", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12865", "FIPS": "36115", "County": "Washington", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12873", "FIPS": "36115", "County": "Washington", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "12887", "FIPS": "36115", "County": "Washington", "State": "NY", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "13143", "FIPS": "36117", "County": "Wayne", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "13146", "FIPS": "36117", "County": "Wayne", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "13154", "FIPS": "36117", "County": "Wayne", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14413", "FIPS": "36117", "County": "Wayne", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14433", "FIPS": "36117", "County": "Wayne", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14449", "FIPS": "36117", "County": "Wayne", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14489", "FIPS": "36117", "County": "Wayne", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14502", "FIPS": "36117", "County": "Wayne", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14505", "FIPS": "36117", "County": "Wayne", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14513", "FIPS": "36117", "County": "Wayne", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14516", "FIPS": "36117", "County": "Wayne", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14519", "FIPS": "36117", "County": "Wayne", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14520", "FIPS": "36117", "County": "Wayne", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14522", "FIPS": "36117", "County": "Wayne", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14538", "FIPS": "36117", "County": "Wayne", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14542", "FIPS": "36117", "County": "Wayne", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14551", "FIPS": "36117", "County": "Wayne", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14555", "FIPS": "36117", "County": "Wayne", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14563", "FIPS": "36117", "County": "Wayne", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14568", "FIPS": "36117", "County": "Wayne", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14589", "FIPS": "36117", "County": "Wayne", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14590", "FIPS": "36117", "County": "Wayne", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "10501", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10502", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10503", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10504", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10505", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10506", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10507", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10510", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10511", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10514", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10517", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10518", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10519", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10520", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10521", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10522", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10523", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10526", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10527", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10528", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10530", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10532", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10533", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10535", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10536", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10538", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10540", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10543", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10545", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10546", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10547", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10548", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10549", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10550", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10551", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10552", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10553", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10557", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10558", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10560", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10562", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10566", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10567", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10570", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10571", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10572", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10573", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10576", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10577", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10578", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10580", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10583", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10587", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10588", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10589", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10590", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10591", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10594", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10595", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10596", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10597", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10598", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10601", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10602", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10603", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10604", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10605", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10606", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10607", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10610", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10701", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10702", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10703", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10704", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10705", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10706", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10707", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10708", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10709", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10710", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10801", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10802", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10803", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10804", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "10805", "FIPS": "36119", "County": "Westchester", "State": "NY", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "14009", "FIPS": "36121", "County": "Wyoming", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14011", "FIPS": "36121", "County": "Wyoming", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14024", "FIPS": "36121", "County": "Wyoming", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14037", "FIPS": "36121", "County": "Wyoming", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14039", "FIPS": "36121", "County": "Wyoming", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14066", "FIPS": "36121", "County": "Wyoming", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14082", "FIPS": "36121", "County": "Wyoming", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14083", "FIPS": "36121", "County": "Wyoming", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14113", "FIPS": "36121", "County": "Wyoming", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14130", "FIPS": "36121", "County": "Wyoming", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14145", "FIPS": "36121", "County": "Wyoming", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14167", "FIPS": "36121", "County": "Wyoming", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14427", "FIPS": "36121", "County": "Wyoming", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14530", "FIPS": "36121", "County": "Wyoming", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14536", "FIPS": "36121", "County": "Wyoming", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14549", "FIPS": "36121", "County": "Wyoming", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14550", "FIPS": "36121", "County": "Wyoming", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14569", "FIPS": "36121", "County": "Wyoming", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14591", "FIPS": "36121", "County": "Wyoming", "State": "NY", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "14415", "FIPS": "36123", "County": "Yates", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14418", "FIPS": "36123", "County": "Yates", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14441", "FIPS": "36123", "County": "Yates", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14478", "FIPS": "36123", "County": "Yates", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14507", "FIPS": "36123", "County": "Yates", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14527", "FIPS": "36123", "County": "Yates", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14544", "FIPS": "36123", "County": "Yates", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14837", "FIPS": "36123", "County": "Yates", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14842", "FIPS": "36123", "County": "Yates", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "14857", "FIPS": "36123", "County": "Yates", "State": "NY", "DMA": {"Code": 538, "Name": "ROCHESTER, NY"}},
  {"Postal_Code": "27201", "FIPS": "37001", "County": "Alamance", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27202", "FIPS": "37001", "County": "Alamance", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27215", "FIPS": "37001", "County": "Alamance", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27216", "FIPS": "37001", "County": "Alamance", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27217", "FIPS": "37001", "County": "Alamance", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27220", "FIPS": "37001", "County": "Alamance", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27244", "FIPS": "37001", "County": "Alamance", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27253", "FIPS": "37001", "County": "Alamance", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27258", "FIPS": "37001", "County": "Alamance", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27302", "FIPS": "37001", "County": "Alamance", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27340", "FIPS": "37001", "County": "Alamance", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27349", "FIPS": "37001", "County": "Alamance", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27359", "FIPS": "37001", "County": "Alamance", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28636", "FIPS": "37003", "County": "Alexander", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28678", "FIPS": "37003", "County": "Alexander", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28681", "FIPS": "37003", "County": "Alexander", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28623", "FIPS": "37005", "County": "Alleghany", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28627", "FIPS": "37005", "County": "Alleghany", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28644", "FIPS": "37005", "County": "Alleghany", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28663", "FIPS": "37005", "County": "Alleghany", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28668", "FIPS": "37005", "County": "Alleghany", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28675", "FIPS": "37005", "County": "Alleghany", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28007", "FIPS": "37007", "County": "Anson", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28091", "FIPS": "37007", "County": "Anson", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28102", "FIPS": "37007", "County": "Anson", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28119", "FIPS": "37007", "County": "Anson", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28133", "FIPS": "37007", "County": "Anson", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28135", "FIPS": "37007", "County": "Anson", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28170", "FIPS": "37007", "County": "Anson", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28615", "FIPS": "37009", "County": "Ashe", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28617", "FIPS": "37009", "County": "Ashe", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28626", "FIPS": "37009", "County": "Ashe", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28629", "FIPS": "37009", "County": "Ashe", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28631", "FIPS": "37009", "County": "Ashe", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28640", "FIPS": "37009", "County": "Ashe", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28643", "FIPS": "37009", "County": "Ashe", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28672", "FIPS": "37009", "County": "Ashe", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28684", "FIPS": "37009", "County": "Ashe", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28693", "FIPS": "37009", "County": "Ashe", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28694", "FIPS": "37009", "County": "Ashe", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28604", "FIPS": "37011", "County": "Avery", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28616", "FIPS": "37011", "County": "Avery", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28622", "FIPS": "37011", "County": "Avery", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28646", "FIPS": "37011", "County": "Avery", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28652", "FIPS": "37011", "County": "Avery", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28653", "FIPS": "37011", "County": "Avery", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28657", "FIPS": "37011", "County": "Avery", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28662", "FIPS": "37011", "County": "Avery", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28664", "FIPS": "37011", "County": "Avery", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "27806", "FIPS": "37013", "County": "Beaufort", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27808", "FIPS": "37013", "County": "Beaufort", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27810", "FIPS": "37013", "County": "Beaufort", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27814", "FIPS": "37013", "County": "Beaufort", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27817", "FIPS": "37013", "County": "Beaufort", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27821", "FIPS": "37013", "County": "Beaufort", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27860", "FIPS": "37013", "County": "Beaufort", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27865", "FIPS": "37013", "County": "Beaufort", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27889", "FIPS": "37013", "County": "Beaufort", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27805", "FIPS": "37015", "County": "Bertie", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27847", "FIPS": "37015", "County": "Bertie", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27849", "FIPS": "37015", "County": "Bertie", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27872", "FIPS": "37015", "County": "Bertie", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27924", "FIPS": "37015", "County": "Bertie", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27957", "FIPS": "37015", "County": "Bertie", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27967", "FIPS": "37015", "County": "Bertie", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27983", "FIPS": "37015", "County": "Bertie", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28320", "FIPS": "37017", "County": "Bladen", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28332", "FIPS": "37017", "County": "Bladen", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28337", "FIPS": "37017", "County": "Bladen", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28392", "FIPS": "37017", "County": "Bladen", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28399", "FIPS": "37017", "County": "Bladen", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28433", "FIPS": "37017", "County": "Bladen", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28434", "FIPS": "37017", "County": "Bladen", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28448", "FIPS": "37017", "County": "Bladen", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28420", "FIPS": "37019", "County": "Brunswick", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28422", "FIPS": "37019", "County": "Brunswick", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28451", "FIPS": "37019", "County": "Brunswick", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28452", "FIPS": "37019", "County": "Brunswick", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28459", "FIPS": "37019", "County": "Brunswick", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28461", "FIPS": "37019", "County": "Brunswick", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28462", "FIPS": "37019", "County": "Brunswick", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28465", "FIPS": "37019", "County": "Brunswick", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28467", "FIPS": "37019", "County": "Brunswick", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28468", "FIPS": "37019", "County": "Brunswick", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28469", "FIPS": "37019", "County": "Brunswick", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28470", "FIPS": "37019", "County": "Brunswick", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28479", "FIPS": "37019", "County": "Brunswick", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28701", "FIPS": "37021", "County": "Buncombe", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28704", "FIPS": "37021", "County": "Buncombe", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28709", "FIPS": "37021", "County": "Buncombe", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28711", "FIPS": "37021", "County": "Buncombe", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28715", "FIPS": "37021", "County": "Buncombe", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28728", "FIPS": "37021", "County": "Buncombe", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28730", "FIPS": "37021", "County": "Buncombe", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28748", "FIPS": "37021", "County": "Buncombe", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28757", "FIPS": "37021", "County": "Buncombe", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28770", "FIPS": "37021", "County": "Buncombe", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28776", "FIPS": "37021", "County": "Buncombe", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28778", "FIPS": "37021", "County": "Buncombe", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28787", "FIPS": "37021", "County": "Buncombe", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28801", "FIPS": "37021", "County": "Buncombe", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28802", "FIPS": "37021", "County": "Buncombe", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28803", "FIPS": "37021", "County": "Buncombe", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28804", "FIPS": "37021", "County": "Buncombe", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28805", "FIPS": "37021", "County": "Buncombe", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28806", "FIPS": "37021", "County": "Buncombe", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28810", "FIPS": "37021", "County": "Buncombe", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28813", "FIPS": "37021", "County": "Buncombe", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28814", "FIPS": "37021", "County": "Buncombe", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28815", "FIPS": "37021", "County": "Buncombe", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28816", "FIPS": "37021", "County": "Buncombe", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28612", "FIPS": "37023", "County": "Burke", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28619", "FIPS": "37023", "County": "Burke", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28628", "FIPS": "37023", "County": "Burke", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28637", "FIPS": "37023", "County": "Burke", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28641", "FIPS": "37023", "County": "Burke", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28647", "FIPS": "37023", "County": "Burke", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28655", "FIPS": "37023", "County": "Burke", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28666", "FIPS": "37023", "County": "Burke", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28671", "FIPS": "37023", "County": "Burke", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28680", "FIPS": "37023", "County": "Burke", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28690", "FIPS": "37023", "County": "Burke", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28025", "FIPS": "37025", "County": "Cabarrus", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28026", "FIPS": "37025", "County": "Cabarrus", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28027", "FIPS": "37025", "County": "Cabarrus", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28075", "FIPS": "37025", "County": "Cabarrus", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28081", "FIPS": "37025", "County": "Cabarrus", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28082", "FIPS": "37025", "County": "Cabarrus", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28083", "FIPS": "37025", "County": "Cabarrus", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28107", "FIPS": "37025", "County": "Cabarrus", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28124", "FIPS": "37025", "County": "Cabarrus", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28611", "FIPS": "37027", "County": "Caldwell", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28630", "FIPS": "37027", "County": "Caldwell", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28633", "FIPS": "37027", "County": "Caldwell", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28638", "FIPS": "37027", "County": "Caldwell", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28645", "FIPS": "37027", "County": "Caldwell", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28661", "FIPS": "37027", "County": "Caldwell", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28667", "FIPS": "37027", "County": "Caldwell", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "27921", "FIPS": "37029", "County": "Camden", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27974", "FIPS": "37029", "County": "Camden", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27976", "FIPS": "37029", "County": "Camden", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "28511", "FIPS": "37031", "County": "Carteret", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28512", "FIPS": "37031", "County": "Carteret", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28516", "FIPS": "37031", "County": "Carteret", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28520", "FIPS": "37031", "County": "Carteret", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28524", "FIPS": "37031", "County": "Carteret", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28528", "FIPS": "37031", "County": "Carteret", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28531", "FIPS": "37031", "County": "Carteret", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28553", "FIPS": "37031", "County": "Carteret", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28557", "FIPS": "37031", "County": "Carteret", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28570", "FIPS": "37031", "County": "Carteret", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28575", "FIPS": "37031", "County": "Carteret", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28577", "FIPS": "37031", "County": "Carteret", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28579", "FIPS": "37031", "County": "Carteret", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28581", "FIPS": "37031", "County": "Carteret", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28582", "FIPS": "37031", "County": "Carteret", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28589", "FIPS": "37031", "County": "Carteret", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28594", "FIPS": "37031", "County": "Carteret", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27212", "FIPS": "37033", "County": "Caswell", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27291", "FIPS": "37033", "County": "Caswell", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27305", "FIPS": "37033", "County": "Caswell", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27311", "FIPS": "37033", "County": "Caswell", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27314", "FIPS": "37033", "County": "Caswell", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27315", "FIPS": "37033", "County": "Caswell", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27379", "FIPS": "37033", "County": "Caswell", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28601", "FIPS": "37035", "County": "Catawba", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28602", "FIPS": "37035", "County": "Catawba", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28603", "FIPS": "37035", "County": "Catawba", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28609", "FIPS": "37035", "County": "Catawba", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28610", "FIPS": "37035", "County": "Catawba", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28613", "FIPS": "37035", "County": "Catawba", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28650", "FIPS": "37035", "County": "Catawba", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28658", "FIPS": "37035", "County": "Catawba", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28673", "FIPS": "37035", "County": "Catawba", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28682", "FIPS": "37035", "County": "Catawba", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "27207", "FIPS": "37037", "County": "Chatham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27208", "FIPS": "37037", "County": "Chatham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27213", "FIPS": "37037", "County": "Chatham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27228", "FIPS": "37037", "County": "Chatham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27252", "FIPS": "37037", "County": "Chatham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27256", "FIPS": "37037", "County": "Chatham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27312", "FIPS": "37037", "County": "Chatham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27344", "FIPS": "37037", "County": "Chatham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27559", "FIPS": "37037", "County": "Chatham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28781", "FIPS": "37039", "County": "Cherokee", "State": "NC", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "28901", "FIPS": "37039", "County": "Cherokee", "State": "NC", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "28903", "FIPS": "37039", "County": "Cherokee", "State": "NC", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "28905", "FIPS": "37039", "County": "Cherokee", "State": "NC", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "28906", "FIPS": "37039", "County": "Cherokee", "State": "NC", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "27932", "FIPS": "37041", "County": "Chowan", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27980", "FIPS": "37041", "County": "Chowan", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "28902", "FIPS": "37043", "County": "Clay", "State": "NC", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "28904", "FIPS": "37043", "County": "Clay", "State": "NC", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "28909", "FIPS": "37043", "County": "Clay", "State": "NC", "DMA": {"Code": 524, "Name": "ATLANTA"}},
  {"Postal_Code": "28017", "FIPS": "37045", "County": "Cleveland", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28020", "FIPS": "37045", "County": "Cleveland", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28038", "FIPS": "37045", "County": "Cleveland", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28042", "FIPS": "37045", "County": "Cleveland", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28073", "FIPS": "37045", "County": "Cleveland", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28086", "FIPS": "37045", "County": "Cleveland", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28089", "FIPS": "37045", "County": "Cleveland", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28090", "FIPS": "37045", "County": "Cleveland", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28114", "FIPS": "37045", "County": "Cleveland", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28136", "FIPS": "37045", "County": "Cleveland", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28150", "FIPS": "37045", "County": "Cleveland", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28151", "FIPS": "37045", "County": "Cleveland", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28152", "FIPS": "37045", "County": "Cleveland", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28169", "FIPS": "37045", "County": "Cleveland", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28423", "FIPS": "37047", "County": "Columbus", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28424", "FIPS": "37047", "County": "Columbus", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28430", "FIPS": "37047", "County": "Columbus", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28431", "FIPS": "37047", "County": "Columbus", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28432", "FIPS": "37047", "County": "Columbus", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28436", "FIPS": "37047", "County": "Columbus", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28438", "FIPS": "37047", "County": "Columbus", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28439", "FIPS": "37047", "County": "Columbus", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28442", "FIPS": "37047", "County": "Columbus", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28450", "FIPS": "37047", "County": "Columbus", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28455", "FIPS": "37047", "County": "Columbus", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28456", "FIPS": "37047", "County": "Columbus", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28463", "FIPS": "37047", "County": "Columbus", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28472", "FIPS": "37047", "County": "Columbus", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28519", "FIPS": "37049", "County": "Craven", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28523", "FIPS": "37049", "County": "Craven", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28526", "FIPS": "37049", "County": "Craven", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28527", "FIPS": "37049", "County": "Craven", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28532", "FIPS": "37049", "County": "Craven", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28533", "FIPS": "37049", "County": "Craven", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28560", "FIPS": "37049", "County": "Craven", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28561", "FIPS": "37049", "County": "Craven", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28562", "FIPS": "37049", "County": "Craven", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28563", "FIPS": "37049", "County": "Craven", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28564", "FIPS": "37049", "County": "Craven", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28586", "FIPS": "37049", "County": "Craven", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28301", "FIPS": "37051", "County": "Cumberland", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28302", "FIPS": "37051", "County": "Cumberland", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28303", "FIPS": "37051", "County": "Cumberland", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28304", "FIPS": "37051", "County": "Cumberland", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28305", "FIPS": "37051", "County": "Cumberland", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28306", "FIPS": "37051", "County": "Cumberland", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28307", "FIPS": "37051", "County": "Cumberland", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28308", "FIPS": "37051", "County": "Cumberland", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28309", "FIPS": "37051", "County": "Cumberland", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28310", "FIPS": "37051", "County": "Cumberland", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28311", "FIPS": "37051", "County": "Cumberland", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28312", "FIPS": "37051", "County": "Cumberland", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28314", "FIPS": "37051", "County": "Cumberland", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28331", "FIPS": "37051", "County": "Cumberland", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28342", "FIPS": "37051", "County": "Cumberland", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28344", "FIPS": "37051", "County": "Cumberland", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28348", "FIPS": "37051", "County": "Cumberland", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28356", "FIPS": "37051", "County": "Cumberland", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28390", "FIPS": "37051", "County": "Cumberland", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28391", "FIPS": "37051", "County": "Cumberland", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28395", "FIPS": "37051", "County": "Cumberland", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27916", "FIPS": "37053", "County": "Currituck", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27917", "FIPS": "37053", "County": "Currituck", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27923", "FIPS": "37053", "County": "Currituck", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27927", "FIPS": "37053", "County": "Currituck", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27929", "FIPS": "37053", "County": "Currituck", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27939", "FIPS": "37053", "County": "Currituck", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27941", "FIPS": "37053", "County": "Currituck", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27947", "FIPS": "37053", "County": "Currituck", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27950", "FIPS": "37053", "County": "Currituck", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27956", "FIPS": "37053", "County": "Currituck", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27958", "FIPS": "37053", "County": "Currituck", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27964", "FIPS": "37053", "County": "Currituck", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27965", "FIPS": "37053", "County": "Currituck", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27966", "FIPS": "37053", "County": "Currituck", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27973", "FIPS": "37053", "County": "Currituck", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27915", "FIPS": "37055", "County": "Dare", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27920", "FIPS": "37055", "County": "Dare", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27936", "FIPS": "37055", "County": "Dare", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27943", "FIPS": "37055", "County": "Dare", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27948", "FIPS": "37055", "County": "Dare", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27949", "FIPS": "37055", "County": "Dare", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27953", "FIPS": "37055", "County": "Dare", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27954", "FIPS": "37055", "County": "Dare", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27959", "FIPS": "37055", "County": "Dare", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27968", "FIPS": "37055", "County": "Dare", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27972", "FIPS": "37055", "County": "Dare", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27978", "FIPS": "37055", "County": "Dare", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27981", "FIPS": "37055", "County": "Dare", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27982", "FIPS": "37055", "County": "Dare", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27239", "FIPS": "37057", "County": "Davidson", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27292", "FIPS": "37057", "County": "Davidson", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27293", "FIPS": "37057", "County": "Davidson", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27294", "FIPS": "37057", "County": "Davidson", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27295", "FIPS": "37057", "County": "Davidson", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27299", "FIPS": "37057", "County": "Davidson", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27351", "FIPS": "37057", "County": "Davidson", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27360", "FIPS": "37057", "County": "Davidson", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27361", "FIPS": "37057", "County": "Davidson", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27373", "FIPS": "37057", "County": "Davidson", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27374", "FIPS": "37057", "County": "Davidson", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27006", "FIPS": "37059", "County": "Davie", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27014", "FIPS": "37059", "County": "Davie", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27028", "FIPS": "37059", "County": "Davie", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28325", "FIPS": "37061", "County": "Duplin", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28341", "FIPS": "37061", "County": "Duplin", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28349", "FIPS": "37061", "County": "Duplin", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28398", "FIPS": "37061", "County": "Duplin", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28453", "FIPS": "37061", "County": "Duplin", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28458", "FIPS": "37061", "County": "Duplin", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28464", "FIPS": "37061", "County": "Duplin", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28466", "FIPS": "37061", "County": "Duplin", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28508", "FIPS": "37061", "County": "Duplin", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28518", "FIPS": "37061", "County": "Duplin", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28521", "FIPS": "37061", "County": "Duplin", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28572", "FIPS": "37061", "County": "Duplin", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27503", "FIPS": "37063", "County": "Durham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27572", "FIPS": "37063", "County": "Durham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27701", "FIPS": "37063", "County": "Durham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27702", "FIPS": "37063", "County": "Durham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27703", "FIPS": "37063", "County": "Durham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27704", "FIPS": "37063", "County": "Durham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27705", "FIPS": "37063", "County": "Durham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27706", "FIPS": "37063", "County": "Durham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27707", "FIPS": "37063", "County": "Durham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27708", "FIPS": "37063", "County": "Durham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27709", "FIPS": "37063", "County": "Durham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27710", "FIPS": "37063", "County": "Durham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27711", "FIPS": "37063", "County": "Durham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27712", "FIPS": "37063", "County": "Durham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27713", "FIPS": "37063", "County": "Durham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27715", "FIPS": "37063", "County": "Durham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27717", "FIPS": "37063", "County": "Durham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27722", "FIPS": "37063", "County": "Durham", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27801", "FIPS": "37065", "County": "Edgecombe", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27802", "FIPS": "37065", "County": "Edgecombe", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27809", "FIPS": "37065", "County": "Edgecombe", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27819", "FIPS": "37065", "County": "Edgecombe", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27852", "FIPS": "37065", "County": "Edgecombe", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27864", "FIPS": "37065", "County": "Edgecombe", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27881", "FIPS": "37065", "County": "Edgecombe", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27886", "FIPS": "37065", "County": "Edgecombe", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27009", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27010", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27012", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27023", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27040", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27045", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27050", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27051", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27094", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27098", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27099", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27101", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27102", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27103", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27104", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27105", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27106", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27107", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27108", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27109", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27110", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27111", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27113", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27114", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27115", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27116", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27117", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27120", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27127", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27130", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27150", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27151", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27152", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27155", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27156", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27157", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27198", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27199", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27284", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27285", "FIPS": "37067", "County": "Forsyth", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27508", "FIPS": "37069", "County": "Franklin", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27525", "FIPS": "37069", "County": "Franklin", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27549", "FIPS": "37069", "County": "Franklin", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27596", "FIPS": "37069", "County": "Franklin", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28006", "FIPS": "37071", "County": "Gaston", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28012", "FIPS": "37071", "County": "Gaston", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28016", "FIPS": "37071", "County": "Gaston", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28021", "FIPS": "37071", "County": "Gaston", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28032", "FIPS": "37071", "County": "Gaston", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28034", "FIPS": "37071", "County": "Gaston", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28052", "FIPS": "37071", "County": "Gaston", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28053", "FIPS": "37071", "County": "Gaston", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28054", "FIPS": "37071", "County": "Gaston", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28055", "FIPS": "37071", "County": "Gaston", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28056", "FIPS": "37071", "County": "Gaston", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28077", "FIPS": "37071", "County": "Gaston", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28098", "FIPS": "37071", "County": "Gaston", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28101", "FIPS": "37071", "County": "Gaston", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28120", "FIPS": "37071", "County": "Gaston", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28164", "FIPS": "37071", "County": "Gaston", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "27926", "FIPS": "37073", "County": "Gates", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27935", "FIPS": "37073", "County": "Gates", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27937", "FIPS": "37073", "County": "Gates", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27938", "FIPS": "37073", "County": "Gates", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27946", "FIPS": "37073", "County": "Gates", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27969", "FIPS": "37073", "County": "Gates", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27979", "FIPS": "37073", "County": "Gates", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "28733", "FIPS": "37075", "County": "Graham", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28771", "FIPS": "37075", "County": "Graham", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "27507", "FIPS": "37077", "County": "Granville", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27509", "FIPS": "37077", "County": "Granville", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27522", "FIPS": "37077", "County": "Granville", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27564", "FIPS": "37077", "County": "Granville", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27565", "FIPS": "37077", "County": "Granville", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27581", "FIPS": "37077", "County": "Granville", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27582", "FIPS": "37077", "County": "Granville", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27888", "FIPS": "37079", "County": "Greene", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28538", "FIPS": "37079", "County": "Greene", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28554", "FIPS": "37079", "County": "Greene", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28580", "FIPS": "37079", "County": "Greene", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27214", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27233", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27235", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27249", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27260", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27261", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27262", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27263", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27264", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27265", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27282", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27283", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27301", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27310", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27313", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27342", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27357", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27358", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27377", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27395", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27401", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27402", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27403", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27404", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27405", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27406", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27407", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27408", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27409", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27410", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27411", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27412", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27413", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27415", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27416", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27417", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27419", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27420", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27425", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27427", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27429", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27435", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27438", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27455", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27480", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27495", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27497", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27498", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27499", "FIPS": "37081", "County": "Guilford", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27823", "FIPS": "37083", "County": "Halifax", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27839", "FIPS": "37083", "County": "Halifax", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27843", "FIPS": "37083", "County": "Halifax", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27844", "FIPS": "37083", "County": "Halifax", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27850", "FIPS": "37083", "County": "Halifax", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27870", "FIPS": "37083", "County": "Halifax", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27874", "FIPS": "37083", "County": "Halifax", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27887", "FIPS": "37083", "County": "Halifax", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27890", "FIPS": "37083", "County": "Halifax", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27501", "FIPS": "37085", "County": "Harnett", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27506", "FIPS": "37085", "County": "Harnett", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27521", "FIPS": "37085", "County": "Harnett", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27543", "FIPS": "37085", "County": "Harnett", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27546", "FIPS": "37085", "County": "Harnett", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27552", "FIPS": "37085", "County": "Harnett", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28323", "FIPS": "37085", "County": "Harnett", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28326", "FIPS": "37085", "County": "Harnett", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28334", "FIPS": "37085", "County": "Harnett", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28335", "FIPS": "37085", "County": "Harnett", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28339", "FIPS": "37085", "County": "Harnett", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28368", "FIPS": "37085", "County": "Harnett", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28716", "FIPS": "37087", "County": "Haywood", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28721", "FIPS": "37087", "County": "Haywood", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28738", "FIPS": "37087", "County": "Haywood", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28745", "FIPS": "37087", "County": "Haywood", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28751", "FIPS": "37087", "County": "Haywood", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28785", "FIPS": "37087", "County": "Haywood", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28786", "FIPS": "37087", "County": "Haywood", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28710", "FIPS": "37089", "County": "Henderson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28724", "FIPS": "37089", "County": "Henderson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28726", "FIPS": "37089", "County": "Henderson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28727", "FIPS": "37089", "County": "Henderson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28729", "FIPS": "37089", "County": "Henderson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28731", "FIPS": "37089", "County": "Henderson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28732", "FIPS": "37089", "County": "Henderson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28735", "FIPS": "37089", "County": "Henderson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28739", "FIPS": "37089", "County": "Henderson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28742", "FIPS": "37089", "County": "Henderson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28758", "FIPS": "37089", "County": "Henderson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28760", "FIPS": "37089", "County": "Henderson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28784", "FIPS": "37089", "County": "Henderson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28790", "FIPS": "37089", "County": "Henderson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28791", "FIPS": "37089", "County": "Henderson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28792", "FIPS": "37089", "County": "Henderson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28793", "FIPS": "37089", "County": "Henderson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "27818", "FIPS": "37091", "County": "Hertford", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27855", "FIPS": "37091", "County": "Hertford", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27910", "FIPS": "37091", "County": "Hertford", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27922", "FIPS": "37091", "County": "Hertford", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27942", "FIPS": "37091", "County": "Hertford", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27986", "FIPS": "37091", "County": "Hertford", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "28376", "FIPS": "37093", "County": "Hoke", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27824", "FIPS": "37095", "County": "Hyde", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27826", "FIPS": "37095", "County": "Hyde", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27875", "FIPS": "37095", "County": "Hyde", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27885", "FIPS": "37095", "County": "Hyde", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27960", "FIPS": "37095", "County": "Hyde", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28010", "FIPS": "37097", "County": "Iredell", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28115", "FIPS": "37097", "County": "Iredell", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28117", "FIPS": "37097", "County": "Iredell", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28123", "FIPS": "37097", "County": "Iredell", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28166", "FIPS": "37097", "County": "Iredell", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28625", "FIPS": "37097", "County": "Iredell", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28634", "FIPS": "37097", "County": "Iredell", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28660", "FIPS": "37097", "County": "Iredell", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28677", "FIPS": "37097", "County": "Iredell", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28687", "FIPS": "37097", "County": "Iredell", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28688", "FIPS": "37097", "County": "Iredell", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28689", "FIPS": "37097", "County": "Iredell", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28699", "FIPS": "37097", "County": "Iredell", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28707", "FIPS": "37099", "County": "Jackson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28717", "FIPS": "37099", "County": "Jackson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28723", "FIPS": "37099", "County": "Jackson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28725", "FIPS": "37099", "County": "Jackson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28736", "FIPS": "37099", "County": "Jackson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28779", "FIPS": "37099", "County": "Jackson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28783", "FIPS": "37099", "County": "Jackson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28788", "FIPS": "37099", "County": "Jackson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28789", "FIPS": "37099", "County": "Jackson", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "27504", "FIPS": "37101", "County": "Johnston", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27520", "FIPS": "37101", "County": "Johnston", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27524", "FIPS": "37101", "County": "Johnston", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27527", "FIPS": "37101", "County": "Johnston", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27528", "FIPS": "37101", "County": "Johnston", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27542", "FIPS": "37101", "County": "Johnston", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27555", "FIPS": "37101", "County": "Johnston", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27568", "FIPS": "37101", "County": "Johnston", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27569", "FIPS": "37101", "County": "Johnston", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27576", "FIPS": "37101", "County": "Johnston", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27577", "FIPS": "37101", "County": "Johnston", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27593", "FIPS": "37101", "County": "Johnston", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28522", "FIPS": "37103", "County": "Jones", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28555", "FIPS": "37103", "County": "Jones", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28573", "FIPS": "37103", "County": "Jones", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28585", "FIPS": "37103", "County": "Jones", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27237", "FIPS": "37105", "County": "Lee", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27330", "FIPS": "37105", "County": "Lee", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27331", "FIPS": "37105", "County": "Lee", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27332", "FIPS": "37105", "County": "Lee", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27505", "FIPS": "37105", "County": "Lee", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28355", "FIPS": "37105", "County": "Lee", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28501", "FIPS": "37107", "County": "Lenoir", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28502", "FIPS": "37107", "County": "Lenoir", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28503", "FIPS": "37107", "County": "Lenoir", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28504", "FIPS": "37107", "County": "Lenoir", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28525", "FIPS": "37107", "County": "Lenoir", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28551", "FIPS": "37107", "County": "Lenoir", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28033", "FIPS": "37109", "County": "Lincoln", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28037", "FIPS": "37109", "County": "Lincoln", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28080", "FIPS": "37109", "County": "Lincoln", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28092", "FIPS": "37109", "County": "Lincoln", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28093", "FIPS": "37109", "County": "Lincoln", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28168", "FIPS": "37109", "County": "Lincoln", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28737", "FIPS": "37111", "County": "McDowell", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28749", "FIPS": "37111", "County": "McDowell", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28752", "FIPS": "37111", "County": "McDowell", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28761", "FIPS": "37111", "County": "McDowell", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28762", "FIPS": "37111", "County": "McDowell", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28734", "FIPS": "37113", "County": "Macon", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28741", "FIPS": "37113", "County": "Macon", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28744", "FIPS": "37113", "County": "Macon", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28763", "FIPS": "37113", "County": "Macon", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28775", "FIPS": "37113", "County": "Macon", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28743", "FIPS": "37115", "County": "Madison", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28753", "FIPS": "37115", "County": "Madison", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28754", "FIPS": "37115", "County": "Madison", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "27825", "FIPS": "37117", "County": "Martin", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27840", "FIPS": "37117", "County": "Martin", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27841", "FIPS": "37117", "County": "Martin", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27846", "FIPS": "37117", "County": "Martin", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27857", "FIPS": "37117", "County": "Martin", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27861", "FIPS": "37117", "County": "Martin", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27871", "FIPS": "37117", "County": "Martin", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27892", "FIPS": "37117", "County": "Martin", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28031", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28035", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28036", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28070", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28078", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28105", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28106", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28126", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28130", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28134", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28201", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28202", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28203", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28204", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28205", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28206", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28207", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28208", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28209", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28210", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28211", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28212", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28213", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28214", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28215", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28216", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28217", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28218", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28219", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28220", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28221", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28222", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28223", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28224", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28226", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28227", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28228", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28229", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28230", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28231", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28232", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28233", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28234", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28235", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28236", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28237", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28241", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28242", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28243", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28244", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28246", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28247", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28250", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28253", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28254", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28255", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28256", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28258", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28260", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28262", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28265", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28266", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28269", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28270", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28271", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28272", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28273", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28274", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28275", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28277", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28278", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28280", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28281", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28282", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28284", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28285", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28287", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28288", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28289", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28290", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28296", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28297", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28299", "FIPS": "37119", "County": "Mecklenburg", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28705", "FIPS": "37121", "County": "Mitchell", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28765", "FIPS": "37121", "County": "Mitchell", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28777", "FIPS": "37121", "County": "Mitchell", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "27209", "FIPS": "37123", "County": "Montgomery", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27229", "FIPS": "37123", "County": "Montgomery", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27247", "FIPS": "37123", "County": "Montgomery", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27306", "FIPS": "37123", "County": "Montgomery", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27356", "FIPS": "37123", "County": "Montgomery", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27371", "FIPS": "37123", "County": "Montgomery", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27242", "FIPS": "37125", "County": "Moore", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27259", "FIPS": "37125", "County": "Moore", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27281", "FIPS": "37125", "County": "Moore", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27325", "FIPS": "37125", "County": "Moore", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27376", "FIPS": "37125", "County": "Moore", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28315", "FIPS": "37125", "County": "Moore", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28327", "FIPS": "37125", "County": "Moore", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28350", "FIPS": "37125", "County": "Moore", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28370", "FIPS": "37125", "County": "Moore", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28373", "FIPS": "37125", "County": "Moore", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28374", "FIPS": "37125", "County": "Moore", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28387", "FIPS": "37125", "County": "Moore", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28388", "FIPS": "37125", "County": "Moore", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28394", "FIPS": "37125", "County": "Moore", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27557", "FIPS": "37127", "County": "Nash", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27803", "FIPS": "37127", "County": "Nash", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27804", "FIPS": "37127", "County": "Nash", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27807", "FIPS": "37127", "County": "Nash", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27816", "FIPS": "37127", "County": "Nash", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27856", "FIPS": "37127", "County": "Nash", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27868", "FIPS": "37127", "County": "Nash", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27878", "FIPS": "37127", "County": "Nash", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27882", "FIPS": "37127", "County": "Nash", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27891", "FIPS": "37127", "County": "Nash", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28401", "FIPS": "37129", "County": "New Hanover", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28402", "FIPS": "37129", "County": "New Hanover", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28403", "FIPS": "37129", "County": "New Hanover", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28404", "FIPS": "37129", "County": "New Hanover", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28405", "FIPS": "37129", "County": "New Hanover", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28406", "FIPS": "37129", "County": "New Hanover", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28407", "FIPS": "37129", "County": "New Hanover", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28408", "FIPS": "37129", "County": "New Hanover", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28409", "FIPS": "37129", "County": "New Hanover", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28410", "FIPS": "37129", "County": "New Hanover", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28411", "FIPS": "37129", "County": "New Hanover", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28412", "FIPS": "37129", "County": "New Hanover", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28428", "FIPS": "37129", "County": "New Hanover", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28429", "FIPS": "37129", "County": "New Hanover", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28449", "FIPS": "37129", "County": "New Hanover", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28480", "FIPS": "37129", "County": "New Hanover", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "27820", "FIPS": "37131", "County": "Northampton", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27831", "FIPS": "37131", "County": "Northampton", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27832", "FIPS": "37131", "County": "Northampton", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27842", "FIPS": "37131", "County": "Northampton", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27845", "FIPS": "37131", "County": "Northampton", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27853", "FIPS": "37131", "County": "Northampton", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27854", "FIPS": "37131", "County": "Northampton", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27862", "FIPS": "37131", "County": "Northampton", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27866", "FIPS": "37131", "County": "Northampton", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27867", "FIPS": "37131", "County": "Northampton", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27869", "FIPS": "37131", "County": "Northampton", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27876", "FIPS": "37131", "County": "Northampton", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27877", "FIPS": "37131", "County": "Northampton", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27897", "FIPS": "37131", "County": "Northampton", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "28445", "FIPS": "37133", "County": "Onslow", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28460", "FIPS": "37133", "County": "Onslow", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28539", "FIPS": "37133", "County": "Onslow", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28540", "FIPS": "37133", "County": "Onslow", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28541", "FIPS": "37133", "County": "Onslow", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28542", "FIPS": "37133", "County": "Onslow", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28543", "FIPS": "37133", "County": "Onslow", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28544", "FIPS": "37133", "County": "Onslow", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28545", "FIPS": "37133", "County": "Onslow", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28546", "FIPS": "37133", "County": "Onslow", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28547", "FIPS": "37133", "County": "Onslow", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28574", "FIPS": "37133", "County": "Onslow", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28584", "FIPS": "37133", "County": "Onslow", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27231", "FIPS": "37135", "County": "Orange", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27243", "FIPS": "37135", "County": "Orange", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27278", "FIPS": "37135", "County": "Orange", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27510", "FIPS": "37135", "County": "Orange", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27514", "FIPS": "37135", "County": "Orange", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27515", "FIPS": "37135", "County": "Orange", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27516", "FIPS": "37135", "County": "Orange", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27517", "FIPS": "37135", "County": "Orange", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27599", "FIPS": "37135", "County": "Orange", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28509", "FIPS": "37137", "County": "Pamlico", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28510", "FIPS": "37137", "County": "Pamlico", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28515", "FIPS": "37137", "County": "Pamlico", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28529", "FIPS": "37137", "County": "Pamlico", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28537", "FIPS": "37137", "County": "Pamlico", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28552", "FIPS": "37137", "County": "Pamlico", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28556", "FIPS": "37137", "County": "Pamlico", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28571", "FIPS": "37137", "County": "Pamlico", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28583", "FIPS": "37137", "County": "Pamlico", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28587", "FIPS": "37137", "County": "Pamlico", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27906", "FIPS": "37139", "County": "Pasquotank", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27907", "FIPS": "37139", "County": "Pasquotank", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27909", "FIPS": "37139", "County": "Pasquotank", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "28421", "FIPS": "37141", "County": "Pender", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28425", "FIPS": "37141", "County": "Pender", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28435", "FIPS": "37141", "County": "Pender", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28443", "FIPS": "37141", "County": "Pender", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28454", "FIPS": "37141", "County": "Pender", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28457", "FIPS": "37141", "County": "Pender", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "28478", "FIPS": "37141", "County": "Pender", "State": "NC", "DMA": {"Code": 550, "Name": "WILMINGTON"}},
  {"Postal_Code": "27919", "FIPS": "37143", "County": "Perquimans", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27930", "FIPS": "37143", "County": "Perquimans", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27944", "FIPS": "37143", "County": "Perquimans", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27985", "FIPS": "37143", "County": "Perquimans", "State": "NC", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "27343", "FIPS": "37145", "County": "Person", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27541", "FIPS": "37145", "County": "Person", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27573", "FIPS": "37145", "County": "Person", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27574", "FIPS": "37145", "County": "Person", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27583", "FIPS": "37145", "County": "Person", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27811", "FIPS": "37147", "County": "Pitt", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27812", "FIPS": "37147", "County": "Pitt", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27827", "FIPS": "37147", "County": "Pitt", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27828", "FIPS": "37147", "County": "Pitt", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27829", "FIPS": "37147", "County": "Pitt", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27833", "FIPS": "37147", "County": "Pitt", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27834", "FIPS": "37147", "County": "Pitt", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27835", "FIPS": "37147", "County": "Pitt", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27836", "FIPS": "37147", "County": "Pitt", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27837", "FIPS": "37147", "County": "Pitt", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27858", "FIPS": "37147", "County": "Pitt", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27879", "FIPS": "37147", "County": "Pitt", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27884", "FIPS": "37147", "County": "Pitt", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28513", "FIPS": "37147", "County": "Pitt", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28530", "FIPS": "37147", "County": "Pitt", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28590", "FIPS": "37147", "County": "Pitt", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28722", "FIPS": "37149", "County": "Polk", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28750", "FIPS": "37149", "County": "Polk", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28756", "FIPS": "37149", "County": "Polk", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28773", "FIPS": "37149", "County": "Polk", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28782", "FIPS": "37149", "County": "Polk", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "27203", "FIPS": "37151", "County": "Randolph", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27204", "FIPS": "37151", "County": "Randolph", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27205", "FIPS": "37151", "County": "Randolph", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27230", "FIPS": "37151", "County": "Randolph", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27248", "FIPS": "37151", "County": "Randolph", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27298", "FIPS": "37151", "County": "Randolph", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27316", "FIPS": "37151", "County": "Randolph", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27317", "FIPS": "37151", "County": "Randolph", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27341", "FIPS": "37151", "County": "Randolph", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27350", "FIPS": "37151", "County": "Randolph", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27355", "FIPS": "37151", "County": "Randolph", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27370", "FIPS": "37151", "County": "Randolph", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28330", "FIPS": "37153", "County": "Richmond", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28338", "FIPS": "37153", "County": "Richmond", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28345", "FIPS": "37153", "County": "Richmond", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28347", "FIPS": "37153", "County": "Richmond", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28363", "FIPS": "37153", "County": "Richmond", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28367", "FIPS": "37153", "County": "Richmond", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28379", "FIPS": "37153", "County": "Richmond", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28380", "FIPS": "37153", "County": "Richmond", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28319", "FIPS": "37155", "County": "Robeson", "State": "NC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "28340", "FIPS": "37155", "County": "Robeson", "State": "NC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "28357", "FIPS": "37155", "County": "Robeson", "State": "NC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "28358", "FIPS": "37155", "County": "Robeson", "State": "NC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "28359", "FIPS": "37155", "County": "Robeson", "State": "NC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "28360", "FIPS": "37155", "County": "Robeson", "State": "NC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "28362", "FIPS": "37155", "County": "Robeson", "State": "NC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "28364", "FIPS": "37155", "County": "Robeson", "State": "NC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "28369", "FIPS": "37155", "County": "Robeson", "State": "NC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "28371", "FIPS": "37155", "County": "Robeson", "State": "NC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "28372", "FIPS": "37155", "County": "Robeson", "State": "NC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "28375", "FIPS": "37155", "County": "Robeson", "State": "NC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "28377", "FIPS": "37155", "County": "Robeson", "State": "NC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "28378", "FIPS": "37155", "County": "Robeson", "State": "NC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "28383", "FIPS": "37155", "County": "Robeson", "State": "NC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "28384", "FIPS": "37155", "County": "Robeson", "State": "NC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "28386", "FIPS": "37155", "County": "Robeson", "State": "NC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "27025", "FIPS": "37157", "County": "Rockingham", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27027", "FIPS": "37157", "County": "Rockingham", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27048", "FIPS": "37157", "County": "Rockingham", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27288", "FIPS": "37157", "County": "Rockingham", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27289", "FIPS": "37157", "County": "Rockingham", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27320", "FIPS": "37157", "County": "Rockingham", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27321", "FIPS": "37157", "County": "Rockingham", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27322", "FIPS": "37157", "County": "Rockingham", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27323", "FIPS": "37157", "County": "Rockingham", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27326", "FIPS": "37157", "County": "Rockingham", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27375", "FIPS": "37157", "County": "Rockingham", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27013", "FIPS": "37159", "County": "Rowan", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "27054", "FIPS": "37159", "County": "Rowan", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28023", "FIPS": "37159", "County": "Rowan", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28039", "FIPS": "37159", "County": "Rowan", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28041", "FIPS": "37159", "County": "Rowan", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28071", "FIPS": "37159", "County": "Rowan", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28072", "FIPS": "37159", "County": "Rowan", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28088", "FIPS": "37159", "County": "Rowan", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28125", "FIPS": "37159", "County": "Rowan", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28138", "FIPS": "37159", "County": "Rowan", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28144", "FIPS": "37159", "County": "Rowan", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28145", "FIPS": "37159", "County": "Rowan", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28146", "FIPS": "37159", "County": "Rowan", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28147", "FIPS": "37159", "County": "Rowan", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28159", "FIPS": "37159", "County": "Rowan", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28018", "FIPS": "37161", "County": "Rutherford", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28019", "FIPS": "37161", "County": "Rutherford", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28024", "FIPS": "37161", "County": "Rutherford", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28040", "FIPS": "37161", "County": "Rutherford", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28043", "FIPS": "37161", "County": "Rutherford", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28074", "FIPS": "37161", "County": "Rutherford", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28076", "FIPS": "37161", "County": "Rutherford", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28139", "FIPS": "37161", "County": "Rutherford", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28160", "FIPS": "37161", "County": "Rutherford", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28167", "FIPS": "37161", "County": "Rutherford", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28720", "FIPS": "37161", "County": "Rutherford", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28746", "FIPS": "37161", "County": "Rutherford", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28318", "FIPS": "37163", "County": "Sampson", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28328", "FIPS": "37163", "County": "Sampson", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28329", "FIPS": "37163", "County": "Sampson", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28366", "FIPS": "37163", "County": "Sampson", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28382", "FIPS": "37163", "County": "Sampson", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28385", "FIPS": "37163", "County": "Sampson", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28393", "FIPS": "37163", "County": "Sampson", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28441", "FIPS": "37163", "County": "Sampson", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28444", "FIPS": "37163", "County": "Sampson", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28447", "FIPS": "37163", "County": "Sampson", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28343", "FIPS": "37165", "County": "Scotland", "State": "NC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "28351", "FIPS": "37165", "County": "Scotland", "State": "NC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "28352", "FIPS": "37165", "County": "Scotland", "State": "NC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "28353", "FIPS": "37165", "County": "Scotland", "State": "NC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "28396", "FIPS": "37165", "County": "Scotland", "State": "NC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "28001", "FIPS": "37167", "County": "Stanly", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28002", "FIPS": "37167", "County": "Stanly", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28009", "FIPS": "37167", "County": "Stanly", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28097", "FIPS": "37167", "County": "Stanly", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28109", "FIPS": "37167", "County": "Stanly", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28127", "FIPS": "37167", "County": "Stanly", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28128", "FIPS": "37167", "County": "Stanly", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28129", "FIPS": "37167", "County": "Stanly", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28137", "FIPS": "37167", "County": "Stanly", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28163", "FIPS": "37167", "County": "Stanly", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "27016", "FIPS": "37169", "County": "Stokes", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27019", "FIPS": "37169", "County": "Stokes", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27021", "FIPS": "37169", "County": "Stokes", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27022", "FIPS": "37169", "County": "Stokes", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27042", "FIPS": "37169", "County": "Stokes", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27043", "FIPS": "37169", "County": "Stokes", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27046", "FIPS": "37169", "County": "Stokes", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27052", "FIPS": "37169", "County": "Stokes", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27007", "FIPS": "37171", "County": "Surry", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27017", "FIPS": "37171", "County": "Surry", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27024", "FIPS": "37171", "County": "Surry", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27030", "FIPS": "37171", "County": "Surry", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27031", "FIPS": "37171", "County": "Surry", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27041", "FIPS": "37171", "County": "Surry", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27047", "FIPS": "37171", "County": "Surry", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27049", "FIPS": "37171", "County": "Surry", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27053", "FIPS": "37171", "County": "Surry", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28621", "FIPS": "37171", "County": "Surry", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28676", "FIPS": "37171", "County": "Surry", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28702", "FIPS": "37173", "County": "Swain", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28713", "FIPS": "37173", "County": "Swain", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28719", "FIPS": "37173", "County": "Swain", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28708", "FIPS": "37175", "County": "Transylvania", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28712", "FIPS": "37175", "County": "Transylvania", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28718", "FIPS": "37175", "County": "Transylvania", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28747", "FIPS": "37175", "County": "Transylvania", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28766", "FIPS": "37175", "County": "Transylvania", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28768", "FIPS": "37175", "County": "Transylvania", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28772", "FIPS": "37175", "County": "Transylvania", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28774", "FIPS": "37175", "County": "Transylvania", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "27925", "FIPS": "37177", "County": "Tyrrell", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28079", "FIPS": "37179", "County": "Union", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28103", "FIPS": "37179", "County": "Union", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28104", "FIPS": "37179", "County": "Union", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28108", "FIPS": "37179", "County": "Union", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28110", "FIPS": "37179", "County": "Union", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28111", "FIPS": "37179", "County": "Union", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28112", "FIPS": "37179", "County": "Union", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28173", "FIPS": "37179", "County": "Union", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28174", "FIPS": "37179", "County": "Union", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "27536", "FIPS": "37181", "County": "Vance", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27537", "FIPS": "37181", "County": "Vance", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27544", "FIPS": "37181", "County": "Vance", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27553", "FIPS": "37181", "County": "Vance", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27556", "FIPS": "37181", "County": "Vance", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27584", "FIPS": "37181", "County": "Vance", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27502", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27511", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27512", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27513", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27518", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27519", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27523", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27526", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27529", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27539", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27540", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27545", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27560", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27562", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27571", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27587", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27588", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27591", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27592", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27597", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27601", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27602", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27603", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27604", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27605", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27606", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27607", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27608", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27609", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27610", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27611", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27612", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27613", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27614", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27615", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27616", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27617", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27619", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27620", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27621", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27622", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27623", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27624", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27625", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27626", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27627", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27628", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27629", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27634", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27635", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27636", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27640", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27650", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27656", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27658", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27661", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27668", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27675", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27676", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27690", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27695", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27697", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27698", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27699", "FIPS": "37183", "County": "Wake", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27551", "FIPS": "37185", "County": "Warren", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27563", "FIPS": "37185", "County": "Warren", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27570", "FIPS": "37185", "County": "Warren", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27586", "FIPS": "37185", "County": "Warren", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27589", "FIPS": "37185", "County": "Warren", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27594", "FIPS": "37185", "County": "Warren", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27928", "FIPS": "37187", "County": "Washington", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27962", "FIPS": "37187", "County": "Washington", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "27970", "FIPS": "37187", "County": "Washington", "State": "NC", "DMA": {"Code": 545, "Name": "GREENVILLE-NEW BERN-WASHINGTON"}},
  {"Postal_Code": "28605", "FIPS": "37189", "County": "Watauga", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28607", "FIPS": "37189", "County": "Watauga", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28608", "FIPS": "37189", "County": "Watauga", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28618", "FIPS": "37189", "County": "Watauga", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28679", "FIPS": "37189", "County": "Watauga", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28691", "FIPS": "37189", "County": "Watauga", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28692", "FIPS": "37189", "County": "Watauga", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "28698", "FIPS": "37189", "County": "Watauga", "State": "NC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "27530", "FIPS": "37191", "County": "Wayne", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27531", "FIPS": "37191", "County": "Wayne", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27532", "FIPS": "37191", "County": "Wayne", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27533", "FIPS": "37191", "County": "Wayne", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27534", "FIPS": "37191", "County": "Wayne", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27830", "FIPS": "37191", "County": "Wayne", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27863", "FIPS": "37191", "County": "Wayne", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28333", "FIPS": "37191", "County": "Wayne", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28365", "FIPS": "37191", "County": "Wayne", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28578", "FIPS": "37191", "County": "Wayne", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "28606", "FIPS": "37193", "County": "Wilkes", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28624", "FIPS": "37193", "County": "Wilkes", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28635", "FIPS": "37193", "County": "Wilkes", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28649", "FIPS": "37193", "County": "Wilkes", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28651", "FIPS": "37193", "County": "Wilkes", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28654", "FIPS": "37193", "County": "Wilkes", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28656", "FIPS": "37193", "County": "Wilkes", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28659", "FIPS": "37193", "County": "Wilkes", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28665", "FIPS": "37193", "County": "Wilkes", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28669", "FIPS": "37193", "County": "Wilkes", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28670", "FIPS": "37193", "County": "Wilkes", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28674", "FIPS": "37193", "County": "Wilkes", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28683", "FIPS": "37193", "County": "Wilkes", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28685", "FIPS": "37193", "County": "Wilkes", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28697", "FIPS": "37193", "County": "Wilkes", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27813", "FIPS": "37195", "County": "Wilson", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27822", "FIPS": "37195", "County": "Wilson", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27851", "FIPS": "37195", "County": "Wilson", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27873", "FIPS": "37195", "County": "Wilson", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27880", "FIPS": "37195", "County": "Wilson", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27883", "FIPS": "37195", "County": "Wilson", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27893", "FIPS": "37195", "County": "Wilson", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27894", "FIPS": "37195", "County": "Wilson", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27895", "FIPS": "37195", "County": "Wilson", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27896", "FIPS": "37195", "County": "Wilson", "State": "NC", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "27011", "FIPS": "37197", "County": "Yadkin", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27018", "FIPS": "37197", "County": "Yadkin", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27020", "FIPS": "37197", "County": "Yadkin", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "27055", "FIPS": "37197", "County": "Yadkin", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28642", "FIPS": "37197", "County": "Yadkin", "State": "NC", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "28714", "FIPS": "37199", "County": "Yancey", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28740", "FIPS": "37199", "County": "Yancey", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "28755", "FIPS": "37199", "County": "Yancey", "State": "NC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "58639", "FIPS": "38001", "County": "Adams", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58649", "FIPS": "38001", "County": "Adams", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58031", "FIPS": "38003", "County": "Barnes", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58049", "FIPS": "38003", "County": "Barnes", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58062", "FIPS": "38003", "County": "Barnes", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58063", "FIPS": "38003", "County": "Barnes", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58065", "FIPS": "38003", "County": "Barnes", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58072", "FIPS": "38003", "County": "Barnes", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58429", "FIPS": "38003", "County": "Barnes", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58461", "FIPS": "38003", "County": "Barnes", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58479", "FIPS": "38003", "County": "Barnes", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58480", "FIPS": "38003", "County": "Barnes", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58481", "FIPS": "38003", "County": "Barnes", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58492", "FIPS": "38003", "County": "Barnes", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58332", "FIPS": "38005", "County": "Benson", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58335", "FIPS": "38005", "County": "Benson", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58343", "FIPS": "38005", "County": "Benson", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58346", "FIPS": "38005", "County": "Benson", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58348", "FIPS": "38005", "County": "Benson", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58351", "FIPS": "38005", "County": "Benson", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58357", "FIPS": "38005", "County": "Benson", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58370", "FIPS": "38005", "County": "Benson", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58379", "FIPS": "38005", "County": "Benson", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58381", "FIPS": "38005", "County": "Benson", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58386", "FIPS": "38005", "County": "Benson", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58627", "FIPS": "38007", "County": "Billings", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58645", "FIPS": "38007", "County": "Billings", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58318", "FIPS": "38009", "County": "Bottineau", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58384", "FIPS": "38009", "County": "Bottineau", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58711", "FIPS": "38009", "County": "Bottineau", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58748", "FIPS": "38009", "County": "Bottineau", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58750", "FIPS": "38009", "County": "Bottineau", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58760", "FIPS": "38009", "County": "Bottineau", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58762", "FIPS": "38009", "County": "Bottineau", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58783", "FIPS": "38009", "County": "Bottineau", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58793", "FIPS": "38009", "County": "Bottineau", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58623", "FIPS": "38011", "County": "Bowman", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58651", "FIPS": "38011", "County": "Bowman", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58653", "FIPS": "38011", "County": "Bowman", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58721", "FIPS": "38013", "County": "Burke", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58727", "FIPS": "38013", "County": "Burke", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58737", "FIPS": "38013", "County": "Burke", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58752", "FIPS": "38013", "County": "Burke", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58772", "FIPS": "38013", "County": "Burke", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58773", "FIPS": "38013", "County": "Burke", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58477", "FIPS": "38015", "County": "Burleigh", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58494", "FIPS": "38015", "County": "Burleigh", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58501", "FIPS": "38015", "County": "Burleigh", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58502", "FIPS": "38015", "County": "Burleigh", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58503", "FIPS": "38015", "County": "Burleigh", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58504", "FIPS": "38015", "County": "Burleigh", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58505", "FIPS": "38015", "County": "Burleigh", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58506", "FIPS": "38015", "County": "Burleigh", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58507", "FIPS": "38015", "County": "Burleigh", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58521", "FIPS": "38015", "County": "Burleigh", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58532", "FIPS": "38015", "County": "Burleigh", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58558", "FIPS": "38015", "County": "Burleigh", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58560", "FIPS": "38015", "County": "Burleigh", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58572", "FIPS": "38015", "County": "Burleigh", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58002", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58004", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58005", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58006", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58007", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58011", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58012", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58021", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58029", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58036", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58038", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58042", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58047", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58048", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58051", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58052", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58059", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58064", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58071", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58078", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58079", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58102", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58103", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58104", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58105", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58106", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58107", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58108", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58109", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58121", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58122", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58124", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58125", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58126", "FIPS": "38017", "County": "Cass", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58239", "FIPS": "38019", "County": "Cavalier", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58249", "FIPS": "38019", "County": "Cavalier", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58255", "FIPS": "38019", "County": "Cavalier", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58260", "FIPS": "38019", "County": "Cavalier", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58269", "FIPS": "38019", "County": "Cavalier", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58281", "FIPS": "38019", "County": "Cavalier", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58311", "FIPS": "38019", "County": "Cavalier", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58323", "FIPS": "38019", "County": "Cavalier", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58352", "FIPS": "38019", "County": "Cavalier", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58355", "FIPS": "38019", "County": "Cavalier", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58372", "FIPS": "38019", "County": "Cavalier", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58436", "FIPS": "38021", "County": "Dickey", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58439", "FIPS": "38021", "County": "Dickey", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58441", "FIPS": "38021", "County": "Dickey", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58474", "FIPS": "38021", "County": "Dickey", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58730", "FIPS": "38023", "County": "Divide", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58765", "FIPS": "38023", "County": "Divide", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58833", "FIPS": "38023", "County": "Divide", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58844", "FIPS": "38023", "County": "Divide", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58625", "FIPS": "38025", "County": "Dunn", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58626", "FIPS": "38025", "County": "Dunn", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58636", "FIPS": "38025", "County": "Dunn", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58640", "FIPS": "38025", "County": "Dunn", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58642", "FIPS": "38025", "County": "Dunn", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58644", "FIPS": "38025", "County": "Dunn", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58356", "FIPS": "38027", "County": "Eddy", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58374", "FIPS": "38027", "County": "Eddy", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58524", "FIPS": "38029", "County": "Emmons", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58542", "FIPS": "38029", "County": "Emmons", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58544", "FIPS": "38029", "County": "Emmons", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58549", "FIPS": "38029", "County": "Emmons", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58552", "FIPS": "38029", "County": "Emmons", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58573", "FIPS": "38029", "County": "Emmons", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58421", "FIPS": "38031", "County": "Foster", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58443", "FIPS": "38031", "County": "Foster", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58445", "FIPS": "38031", "County": "Foster", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58464", "FIPS": "38031", "County": "Foster", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58621", "FIPS": "38033", "County": "Golden Valley", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58632", "FIPS": "38033", "County": "Golden Valley", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58654", "FIPS": "38033", "County": "Golden Valley", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58201", "FIPS": "38035", "County": "Grand Forks", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58202", "FIPS": "38035", "County": "Grand Forks", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58203", "FIPS": "38035", "County": "Grand Forks", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58204", "FIPS": "38035", "County": "Grand Forks", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58205", "FIPS": "38035", "County": "Grand Forks", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58206", "FIPS": "38035", "County": "Grand Forks", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58207", "FIPS": "38035", "County": "Grand Forks", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58208", "FIPS": "38035", "County": "Grand Forks", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58214", "FIPS": "38035", "County": "Grand Forks", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58228", "FIPS": "38035", "County": "Grand Forks", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58235", "FIPS": "38035", "County": "Grand Forks", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58244", "FIPS": "38035", "County": "Grand Forks", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58251", "FIPS": "38035", "County": "Grand Forks", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58256", "FIPS": "38035", "County": "Grand Forks", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58258", "FIPS": "38035", "County": "Grand Forks", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58266", "FIPS": "38035", "County": "Grand Forks", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58267", "FIPS": "38035", "County": "Grand Forks", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58275", "FIPS": "38035", "County": "Grand Forks", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58278", "FIPS": "38035", "County": "Grand Forks", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58529", "FIPS": "38037", "County": "Grant", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58533", "FIPS": "38037", "County": "Grant", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58562", "FIPS": "38037", "County": "Grant", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58564", "FIPS": "38037", "County": "Grant", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58569", "FIPS": "38037", "County": "Grant", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58416", "FIPS": "38039", "County": "Griggs", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58425", "FIPS": "38039", "County": "Griggs", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58448", "FIPS": "38039", "County": "Griggs", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58452", "FIPS": "38039", "County": "Griggs", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58484", "FIPS": "38039", "County": "Griggs", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58646", "FIPS": "38041", "County": "Hettinger", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58647", "FIPS": "38041", "County": "Hettinger", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58650", "FIPS": "38041", "County": "Hettinger", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58428", "FIPS": "38043", "County": "Kidder", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58475", "FIPS": "38043", "County": "Kidder", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58478", "FIPS": "38043", "County": "Kidder", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58482", "FIPS": "38043", "County": "Kidder", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58487", "FIPS": "38043", "County": "Kidder", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58488", "FIPS": "38043", "County": "Kidder", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58415", "FIPS": "38045", "County": "Lamoure", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58431", "FIPS": "38045", "County": "Lamoure", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58433", "FIPS": "38045", "County": "Lamoure", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58454", "FIPS": "38045", "County": "Lamoure", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58456", "FIPS": "38045", "County": "Lamoure", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58458", "FIPS": "38045", "County": "Lamoure", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58466", "FIPS": "38045", "County": "Lamoure", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58490", "FIPS": "38045", "County": "Lamoure", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58440", "FIPS": "38047", "County": "Logan", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58442", "FIPS": "38047", "County": "Logan", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58561", "FIPS": "38047", "County": "Logan", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58710", "FIPS": "38049", "County": "McHenry", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58712", "FIPS": "38049", "County": "McHenry", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58713", "FIPS": "38049", "County": "McHenry", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58731", "FIPS": "38049", "County": "McHenry", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58736", "FIPS": "38049", "County": "McHenry", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58741", "FIPS": "38049", "County": "McHenry", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58744", "FIPS": "38049", "County": "McHenry", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58768", "FIPS": "38049", "County": "McHenry", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58788", "FIPS": "38049", "County": "McHenry", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58789", "FIPS": "38049", "County": "McHenry", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58790", "FIPS": "38049", "County": "McHenry", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58792", "FIPS": "38049", "County": "McHenry", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58413", "FIPS": "38051", "County": "McIntosh", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58460", "FIPS": "38051", "County": "McIntosh", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58495", "FIPS": "38051", "County": "McIntosh", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58581", "FIPS": "38051", "County": "McIntosh", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58634", "FIPS": "38053", "County": "McKenzie", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58757", "FIPS": "38053", "County": "McKenzie", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58831", "FIPS": "38053", "County": "McKenzie", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58835", "FIPS": "38053", "County": "McKenzie", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58838", "FIPS": "38053", "County": "McKenzie", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58847", "FIPS": "38053", "County": "McKenzie", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58854", "FIPS": "38053", "County": "McKenzie", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58531", "FIPS": "38055", "County": "McLean", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58540", "FIPS": "38055", "County": "McLean", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58559", "FIPS": "38055", "County": "McLean", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58565", "FIPS": "38055", "County": "McLean", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58575", "FIPS": "38055", "County": "McLean", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58576", "FIPS": "38055", "County": "McLean", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58577", "FIPS": "38055", "County": "McLean", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58579", "FIPS": "38055", "County": "McLean", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58716", "FIPS": "38055", "County": "McLean", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58723", "FIPS": "38055", "County": "McLean", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58759", "FIPS": "38055", "County": "McLean", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58775", "FIPS": "38055", "County": "McLean", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58778", "FIPS": "38055", "County": "McLean", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58523", "FIPS": "38057", "County": "Mercer", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58541", "FIPS": "38057", "County": "Mercer", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58545", "FIPS": "38057", "County": "Mercer", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58571", "FIPS": "38057", "County": "Mercer", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58580", "FIPS": "38057", "County": "Mercer", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58520", "FIPS": "38059", "County": "Morton", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58535", "FIPS": "38059", "County": "Morton", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58554", "FIPS": "38059", "County": "Morton", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58563", "FIPS": "38059", "County": "Morton", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58566", "FIPS": "38059", "County": "Morton", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58631", "FIPS": "38059", "County": "Morton", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58638", "FIPS": "38059", "County": "Morton", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58763", "FIPS": "38061", "County": "Mountrail", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58769", "FIPS": "38061", "County": "Mountrail", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58770", "FIPS": "38061", "County": "Mountrail", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58771", "FIPS": "38061", "County": "Mountrail", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58776", "FIPS": "38061", "County": "Mountrail", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58784", "FIPS": "38061", "County": "Mountrail", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58794", "FIPS": "38061", "County": "Mountrail", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58212", "FIPS": "38063", "County": "Nelson", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58224", "FIPS": "38063", "County": "Nelson", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58254", "FIPS": "38063", "County": "Nelson", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58259", "FIPS": "38063", "County": "Nelson", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58272", "FIPS": "38063", "County": "Nelson", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58344", "FIPS": "38063", "County": "Nelson", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58361", "FIPS": "38063", "County": "Nelson", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58380", "FIPS": "38063", "County": "Nelson", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58530", "FIPS": "38065", "County": "Oliver", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58216", "FIPS": "38067", "County": "Pembina", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58220", "FIPS": "38067", "County": "Pembina", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58222", "FIPS": "38067", "County": "Pembina", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58225", "FIPS": "38067", "County": "Pembina", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58236", "FIPS": "38067", "County": "Pembina", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58238", "FIPS": "38067", "County": "Pembina", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58241", "FIPS": "38067", "County": "Pembina", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58262", "FIPS": "38067", "County": "Pembina", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58265", "FIPS": "38067", "County": "Pembina", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58271", "FIPS": "38067", "County": "Pembina", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58276", "FIPS": "38067", "County": "Pembina", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58282", "FIPS": "38067", "County": "Pembina", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58313", "FIPS": "38069", "County": "Pierce", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58368", "FIPS": "38069", "County": "Pierce", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58385", "FIPS": "38069", "County": "Pierce", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58301", "FIPS": "38071", "County": "Ramsey", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58321", "FIPS": "38071", "County": "Ramsey", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58325", "FIPS": "38071", "County": "Ramsey", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58327", "FIPS": "38071", "County": "Ramsey", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58330", "FIPS": "38071", "County": "Ramsey", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58338", "FIPS": "38071", "County": "Ramsey", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58345", "FIPS": "38071", "County": "Ramsey", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58362", "FIPS": "38071", "County": "Ramsey", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58377", "FIPS": "38071", "County": "Ramsey", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58382", "FIPS": "38071", "County": "Ramsey", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58027", "FIPS": "38073", "County": "Ransom", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58033", "FIPS": "38073", "County": "Ransom", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58054", "FIPS": "38073", "County": "Ransom", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58068", "FIPS": "38073", "County": "Ransom", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58740", "FIPS": "38075", "County": "Renville", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58761", "FIPS": "38075", "County": "Renville", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58782", "FIPS": "38075", "County": "Renville", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58787", "FIPS": "38075", "County": "Renville", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58001", "FIPS": "38077", "County": "Richland", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58008", "FIPS": "38077", "County": "Richland", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58015", "FIPS": "38077", "County": "Richland", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58018", "FIPS": "38077", "County": "Richland", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58030", "FIPS": "38077", "County": "Richland", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58041", "FIPS": "38077", "County": "Richland", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58053", "FIPS": "38077", "County": "Richland", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58057", "FIPS": "38077", "County": "Richland", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58058", "FIPS": "38077", "County": "Richland", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58061", "FIPS": "38077", "County": "Richland", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58074", "FIPS": "38077", "County": "Richland", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58075", "FIPS": "38077", "County": "Richland", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58076", "FIPS": "38077", "County": "Richland", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58077", "FIPS": "38077", "County": "Richland", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58081", "FIPS": "38077", "County": "Richland", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58310", "FIPS": "38079", "County": "Rolette", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58316", "FIPS": "38079", "County": "Rolette", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58329", "FIPS": "38079", "County": "Rolette", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58353", "FIPS": "38079", "County": "Rolette", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58366", "FIPS": "38079", "County": "Rolette", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58367", "FIPS": "38079", "County": "Rolette", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58369", "FIPS": "38079", "County": "Rolette", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58013", "FIPS": "38081", "County": "Sargent", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58017", "FIPS": "38081", "County": "Sargent", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58032", "FIPS": "38081", "County": "Sargent", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58040", "FIPS": "38081", "County": "Sargent", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58043", "FIPS": "38081", "County": "Sargent", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58060", "FIPS": "38081", "County": "Sargent", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58067", "FIPS": "38081", "County": "Sargent", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58069", "FIPS": "38081", "County": "Sargent", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58430", "FIPS": "38083", "County": "Sheridan", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58444", "FIPS": "38083", "County": "Sheridan", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58463", "FIPS": "38083", "County": "Sheridan", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58758", "FIPS": "38083", "County": "Sheridan", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58528", "FIPS": "38085", "County": "Sioux", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58538", "FIPS": "38085", "County": "Sioux", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58568", "FIPS": "38085", "County": "Sioux", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58570", "FIPS": "38085", "County": "Sioux", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58620", "FIPS": "38087", "County": "Slope", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58643", "FIPS": "38087", "County": "Slope", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58601", "FIPS": "38089", "County": "Stark", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58602", "FIPS": "38089", "County": "Stark", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58622", "FIPS": "38089", "County": "Stark", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58630", "FIPS": "38089", "County": "Stark", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58641", "FIPS": "38089", "County": "Stark", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58652", "FIPS": "38089", "County": "Stark", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58655", "FIPS": "38089", "County": "Stark", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58656", "FIPS": "38089", "County": "Stark", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58046", "FIPS": "38091", "County": "Steele", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58056", "FIPS": "38091", "County": "Steele", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58230", "FIPS": "38091", "County": "Steele", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58277", "FIPS": "38091", "County": "Steele", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58401", "FIPS": "38093", "County": "Stutsman", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58402", "FIPS": "38093", "County": "Stutsman", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58405", "FIPS": "38093", "County": "Stutsman", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58420", "FIPS": "38093", "County": "Stutsman", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58424", "FIPS": "38093", "County": "Stutsman", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58426", "FIPS": "38093", "County": "Stutsman", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58455", "FIPS": "38093", "County": "Stutsman", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58467", "FIPS": "38093", "County": "Stutsman", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58472", "FIPS": "38093", "County": "Stutsman", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58476", "FIPS": "38093", "County": "Stutsman", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58483", "FIPS": "38093", "County": "Stutsman", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58496", "FIPS": "38093", "County": "Stutsman", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58497", "FIPS": "38093", "County": "Stutsman", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58317", "FIPS": "38095", "County": "Towner", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58324", "FIPS": "38095", "County": "Towner", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58331", "FIPS": "38095", "County": "Towner", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58339", "FIPS": "38095", "County": "Towner", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58363", "FIPS": "38095", "County": "Towner", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58365", "FIPS": "38095", "County": "Towner", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58009", "FIPS": "38097", "County": "Traill", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58016", "FIPS": "38097", "County": "Traill", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58035", "FIPS": "38097", "County": "Traill", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58045", "FIPS": "38097", "County": "Traill", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58218", "FIPS": "38097", "County": "Traill", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58219", "FIPS": "38097", "County": "Traill", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58223", "FIPS": "38097", "County": "Traill", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58240", "FIPS": "38097", "County": "Traill", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58257", "FIPS": "38097", "County": "Traill", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58274", "FIPS": "38097", "County": "Traill", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58210", "FIPS": "38099", "County": "Walsh", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58227", "FIPS": "38099", "County": "Walsh", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58229", "FIPS": "38099", "County": "Walsh", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58231", "FIPS": "38099", "County": "Walsh", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58233", "FIPS": "38099", "County": "Walsh", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58237", "FIPS": "38099", "County": "Walsh", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58243", "FIPS": "38099", "County": "Walsh", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58250", "FIPS": "38099", "County": "Walsh", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58261", "FIPS": "38099", "County": "Walsh", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58270", "FIPS": "38099", "County": "Walsh", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58273", "FIPS": "38099", "County": "Walsh", "State": "ND", "DMA": {"Code": 724, "Name": "FARGO-VALLEY CITY"}},
  {"Postal_Code": "58701", "FIPS": "38101", "County": "Ward", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58702", "FIPS": "38101", "County": "Ward", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58703", "FIPS": "38101", "County": "Ward", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58704", "FIPS": "38101", "County": "Ward", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58705", "FIPS": "38101", "County": "Ward", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58707", "FIPS": "38101", "County": "Ward", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58718", "FIPS": "38101", "County": "Ward", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58722", "FIPS": "38101", "County": "Ward", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58725", "FIPS": "38101", "County": "Ward", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58733", "FIPS": "38101", "County": "Ward", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58734", "FIPS": "38101", "County": "Ward", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58735", "FIPS": "38101", "County": "Ward", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58746", "FIPS": "38101", "County": "Ward", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58756", "FIPS": "38101", "County": "Ward", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58779", "FIPS": "38101", "County": "Ward", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58781", "FIPS": "38101", "County": "Ward", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58785", "FIPS": "38101", "County": "Ward", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58319", "FIPS": "38103", "County": "Wells", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58341", "FIPS": "38103", "County": "Wells", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58418", "FIPS": "38103", "County": "Wells", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58422", "FIPS": "38103", "County": "Wells", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58423", "FIPS": "38103", "County": "Wells", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58438", "FIPS": "38103", "County": "Wells", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58451", "FIPS": "38103", "County": "Wells", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58486", "FIPS": "38103", "County": "Wells", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58755", "FIPS": "38105", "County": "Williams", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58795", "FIPS": "38105", "County": "Williams", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58801", "FIPS": "38105", "County": "Williams", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58802", "FIPS": "38105", "County": "Williams", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58830", "FIPS": "38105", "County": "Williams", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58843", "FIPS": "38105", "County": "Williams", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58845", "FIPS": "38105", "County": "Williams", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58849", "FIPS": "38105", "County": "Williams", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58852", "FIPS": "38105", "County": "Williams", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58853", "FIPS": "38105", "County": "Williams", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "58856", "FIPS": "38105", "County": "Williams", "State": "ND", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "45105", "FIPS": "39001", "County": "Adams", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45144", "FIPS": "39001", "County": "Adams", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45616", "FIPS": "39001", "County": "Adams", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45618", "FIPS": "39001", "County": "Adams", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45650", "FIPS": "39001", "County": "Adams", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45660", "FIPS": "39001", "County": "Adams", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45679", "FIPS": "39001", "County": "Adams", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45693", "FIPS": "39001", "County": "Adams", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45697", "FIPS": "39001", "County": "Adams", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45801", "FIPS": "39003", "County": "Allen", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45802", "FIPS": "39003", "County": "Allen", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45804", "FIPS": "39003", "County": "Allen", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45805", "FIPS": "39003", "County": "Allen", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45807", "FIPS": "39003", "County": "Allen", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45808", "FIPS": "39003", "County": "Allen", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45809", "FIPS": "39003", "County": "Allen", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45817", "FIPS": "39003", "County": "Allen", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45820", "FIPS": "39003", "County": "Allen", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45833", "FIPS": "39003", "County": "Allen", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45850", "FIPS": "39003", "County": "Allen", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45854", "FIPS": "39003", "County": "Allen", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45887", "FIPS": "39003", "County": "Allen", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "44805", "FIPS": "39005", "County": "Ashland", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44838", "FIPS": "39005", "County": "Ashland", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44840", "FIPS": "39005", "County": "Ashland", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44842", "FIPS": "39005", "County": "Ashland", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44848", "FIPS": "39005", "County": "Ashland", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44859", "FIPS": "39005", "County": "Ashland", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44864", "FIPS": "39005", "County": "Ashland", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44866", "FIPS": "39005", "County": "Ashland", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44874", "FIPS": "39005", "County": "Ashland", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44880", "FIPS": "39005", "County": "Ashland", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44003", "FIPS": "39007", "County": "Ashtabula", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44004", "FIPS": "39007", "County": "Ashtabula", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44005", "FIPS": "39007", "County": "Ashtabula", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44010", "FIPS": "39007", "County": "Ashtabula", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44030", "FIPS": "39007", "County": "Ashtabula", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44032", "FIPS": "39007", "County": "Ashtabula", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44041", "FIPS": "39007", "County": "Ashtabula", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44047", "FIPS": "39007", "County": "Ashtabula", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44048", "FIPS": "39007", "County": "Ashtabula", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44068", "FIPS": "39007", "County": "Ashtabula", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44076", "FIPS": "39007", "County": "Ashtabula", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44082", "FIPS": "39007", "County": "Ashtabula", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44084", "FIPS": "39007", "County": "Ashtabula", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44085", "FIPS": "39007", "County": "Ashtabula", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44088", "FIPS": "39007", "County": "Ashtabula", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44093", "FIPS": "39007", "County": "Ashtabula", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44099", "FIPS": "39007", "County": "Ashtabula", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "45701", "FIPS": "39009", "County": "Athens", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45710", "FIPS": "39009", "County": "Athens", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45711", "FIPS": "39009", "County": "Athens", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45716", "FIPS": "39009", "County": "Athens", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45717", "FIPS": "39009", "County": "Athens", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45719", "FIPS": "39009", "County": "Athens", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45723", "FIPS": "39009", "County": "Athens", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45732", "FIPS": "39009", "County": "Athens", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45735", "FIPS": "39009", "County": "Athens", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45739", "FIPS": "39009", "County": "Athens", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45740", "FIPS": "39009", "County": "Athens", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45761", "FIPS": "39009", "County": "Athens", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45764", "FIPS": "39009", "County": "Athens", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45766", "FIPS": "39009", "County": "Athens", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45777", "FIPS": "39009", "County": "Athens", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45778", "FIPS": "39009", "County": "Athens", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45780", "FIPS": "39009", "County": "Athens", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45782", "FIPS": "39009", "County": "Athens", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45806", "FIPS": "39011", "County": "Auglaize", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45819", "FIPS": "39011", "County": "Auglaize", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45865", "FIPS": "39011", "County": "Auglaize", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45869", "FIPS": "39011", "County": "Auglaize", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45870", "FIPS": "39011", "County": "Auglaize", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45871", "FIPS": "39011", "County": "Auglaize", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45884", "FIPS": "39011", "County": "Auglaize", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45885", "FIPS": "39011", "County": "Auglaize", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45888", "FIPS": "39011", "County": "Auglaize", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45895", "FIPS": "39011", "County": "Auglaize", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45896", "FIPS": "39011", "County": "Auglaize", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "43713", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43718", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43719", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43759", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43902", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43905", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43906", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43909", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43912", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43916", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43927", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43928", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43933", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43934", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43935", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43937", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43940", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43942", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43947", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43950", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43951", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43967", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43972", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43977", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43983", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43985", "FIPS": "39013", "County": "Belmont", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "45101", "FIPS": "39015", "County": "Brown", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45115", "FIPS": "39015", "County": "Brown", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45118", "FIPS": "39015", "County": "Brown", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45119", "FIPS": "39015", "County": "Brown", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45121", "FIPS": "39015", "County": "Brown", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45130", "FIPS": "39015", "County": "Brown", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45131", "FIPS": "39015", "County": "Brown", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45154", "FIPS": "39015", "County": "Brown", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45167", "FIPS": "39015", "County": "Brown", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45168", "FIPS": "39015", "County": "Brown", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45171", "FIPS": "39015", "County": "Brown", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45003", "FIPS": "39017", "County": "Butler", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45004", "FIPS": "39017", "County": "Butler", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45011", "FIPS": "39017", "County": "Butler", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45012", "FIPS": "39017", "County": "Butler", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45013", "FIPS": "39017", "County": "Butler", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45014", "FIPS": "39017", "County": "Butler", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45015", "FIPS": "39017", "County": "Butler", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45018", "FIPS": "39017", "County": "Butler", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45025", "FIPS": "39017", "County": "Butler", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45026", "FIPS": "39017", "County": "Butler", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45042", "FIPS": "39017", "County": "Butler", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45043", "FIPS": "39017", "County": "Butler", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45044", "FIPS": "39017", "County": "Butler", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45050", "FIPS": "39017", "County": "Butler", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45053", "FIPS": "39017", "County": "Butler", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45055", "FIPS": "39017", "County": "Butler", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45056", "FIPS": "39017", "County": "Butler", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45061", "FIPS": "39017", "County": "Butler", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45062", "FIPS": "39017", "County": "Butler", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45063", "FIPS": "39017", "County": "Butler", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45064", "FIPS": "39017", "County": "Butler", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45067", "FIPS": "39017", "County": "Butler", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45069", "FIPS": "39017", "County": "Butler", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45071", "FIPS": "39017", "County": "Butler", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "44607", "FIPS": "39019", "County": "Carroll", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44615", "FIPS": "39019", "County": "Carroll", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44620", "FIPS": "39019", "County": "Carroll", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44631", "FIPS": "39019", "County": "Carroll", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44639", "FIPS": "39019", "County": "Carroll", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44644", "FIPS": "39019", "County": "Carroll", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44651", "FIPS": "39019", "County": "Carroll", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44675", "FIPS": "39019", "County": "Carroll", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "43009", "FIPS": "39021", "County": "Champaign", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43044", "FIPS": "39021", "County": "Champaign", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43047", "FIPS": "39021", "County": "Champaign", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43060", "FIPS": "39021", "County": "Champaign", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43070", "FIPS": "39021", "County": "Champaign", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43072", "FIPS": "39021", "County": "Champaign", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43078", "FIPS": "39021", "County": "Champaign", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43083", "FIPS": "39021", "County": "Champaign", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43084", "FIPS": "39021", "County": "Champaign", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45389", "FIPS": "39021", "County": "Champaign", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43010", "FIPS": "39023", "County": "Clark", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45319", "FIPS": "39023", "County": "Clark", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45323", "FIPS": "39023", "County": "Clark", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45341", "FIPS": "39023", "County": "Clark", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45344", "FIPS": "39023", "County": "Clark", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45349", "FIPS": "39023", "County": "Clark", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45368", "FIPS": "39023", "County": "Clark", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45369", "FIPS": "39023", "County": "Clark", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45372", "FIPS": "39023", "County": "Clark", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45501", "FIPS": "39023", "County": "Clark", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45502", "FIPS": "39023", "County": "Clark", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45503", "FIPS": "39023", "County": "Clark", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45504", "FIPS": "39023", "County": "Clark", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45505", "FIPS": "39023", "County": "Clark", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45506", "FIPS": "39023", "County": "Clark", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45102", "FIPS": "39025", "County": "Clermont", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45103", "FIPS": "39025", "County": "Clermont", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45106", "FIPS": "39025", "County": "Clermont", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45112", "FIPS": "39025", "County": "Clermont", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45120", "FIPS": "39025", "County": "Clermont", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45122", "FIPS": "39025", "County": "Clermont", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45140", "FIPS": "39025", "County": "Clermont", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45145", "FIPS": "39025", "County": "Clermont", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45147", "FIPS": "39025", "County": "Clermont", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45150", "FIPS": "39025", "County": "Clermont", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45153", "FIPS": "39025", "County": "Clermont", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45156", "FIPS": "39025", "County": "Clermont", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45157", "FIPS": "39025", "County": "Clermont", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45158", "FIPS": "39025", "County": "Clermont", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45160", "FIPS": "39025", "County": "Clermont", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45176", "FIPS": "39025", "County": "Clermont", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45245", "FIPS": "39025", "County": "Clermont", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45107", "FIPS": "39027", "County": "Clinton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45113", "FIPS": "39027", "County": "Clinton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45114", "FIPS": "39027", "County": "Clinton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45138", "FIPS": "39027", "County": "Clinton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45146", "FIPS": "39027", "County": "Clinton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45148", "FIPS": "39027", "County": "Clinton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45159", "FIPS": "39027", "County": "Clinton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45164", "FIPS": "39027", "County": "Clinton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45166", "FIPS": "39027", "County": "Clinton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45169", "FIPS": "39027", "County": "Clinton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45177", "FIPS": "39027", "County": "Clinton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "43920", "FIPS": "39029", "County": "Columbiana", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "43945", "FIPS": "39029", "County": "Columbiana", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "43962", "FIPS": "39029", "County": "Columbiana", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "43968", "FIPS": "39029", "County": "Columbiana", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44408", "FIPS": "39029", "County": "Columbiana", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44413", "FIPS": "39029", "County": "Columbiana", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44415", "FIPS": "39029", "County": "Columbiana", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44423", "FIPS": "39029", "County": "Columbiana", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44427", "FIPS": "39029", "County": "Columbiana", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44431", "FIPS": "39029", "County": "Columbiana", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44432", "FIPS": "39029", "County": "Columbiana", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44441", "FIPS": "39029", "County": "Columbiana", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44445", "FIPS": "39029", "County": "Columbiana", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44455", "FIPS": "39029", "County": "Columbiana", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44460", "FIPS": "39029", "County": "Columbiana", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44490", "FIPS": "39029", "County": "Columbiana", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44492", "FIPS": "39029", "County": "Columbiana", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44493", "FIPS": "39029", "County": "Columbiana", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44625", "FIPS": "39029", "County": "Columbiana", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44634", "FIPS": "39029", "County": "Columbiana", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44665", "FIPS": "39029", "County": "Columbiana", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "43803", "FIPS": "39031", "County": "Coshocton", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43805", "FIPS": "39031", "County": "Coshocton", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43811", "FIPS": "39031", "County": "Coshocton", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43812", "FIPS": "39031", "County": "Coshocton", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43824", "FIPS": "39031", "County": "Coshocton", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43828", "FIPS": "39031", "County": "Coshocton", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43836", "FIPS": "39031", "County": "Coshocton", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43843", "FIPS": "39031", "County": "Coshocton", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43844", "FIPS": "39031", "County": "Coshocton", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43845", "FIPS": "39031", "County": "Coshocton", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "44820", "FIPS": "39033", "County": "Crawford", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "44825", "FIPS": "39033", "County": "Crawford", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "44827", "FIPS": "39033", "County": "Crawford", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "44833", "FIPS": "39033", "County": "Crawford", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "44854", "FIPS": "39033", "County": "Crawford", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "44856", "FIPS": "39033", "County": "Crawford", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "44860", "FIPS": "39033", "County": "Crawford", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "44881", "FIPS": "39033", "County": "Crawford", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "44887", "FIPS": "39033", "County": "Crawford", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "44017", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44022", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44040", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44070", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44101", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44102", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44103", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44104", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44105", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44106", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44107", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44108", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44109", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44110", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44111", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44112", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44113", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44114", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44115", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44116", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44117", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44118", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44119", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44120", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44121", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44122", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44123", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44124", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44125", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44126", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44127", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44128", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44129", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44130", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44131", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44132", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44133", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44134", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44135", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44136", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44137", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44138", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44139", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44140", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44141", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44142", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44143", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44144", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44145", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44146", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44147", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44149", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44178", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44181", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44185", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44188", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44189", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44190", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44191", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44192", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44193", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44194", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44195", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44197", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44198", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44199", "FIPS": "39035", "County": "Cuyahoga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "45303", "FIPS": "39037", "County": "Darke", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45304", "FIPS": "39037", "County": "Darke", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45328", "FIPS": "39037", "County": "Darke", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45331", "FIPS": "39037", "County": "Darke", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45332", "FIPS": "39037", "County": "Darke", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45346", "FIPS": "39037", "County": "Darke", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45348", "FIPS": "39037", "County": "Darke", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45350", "FIPS": "39037", "County": "Darke", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45351", "FIPS": "39037", "County": "Darke", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45352", "FIPS": "39037", "County": "Darke", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45358", "FIPS": "39037", "County": "Darke", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45362", "FIPS": "39037", "County": "Darke", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45380", "FIPS": "39037", "County": "Darke", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45388", "FIPS": "39037", "County": "Darke", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45390", "FIPS": "39037", "County": "Darke", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43512", "FIPS": "39039", "County": "Defiance", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43519", "FIPS": "39039", "County": "Defiance", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43520", "FIPS": "39039", "County": "Defiance", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43526", "FIPS": "39039", "County": "Defiance", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43530", "FIPS": "39039", "County": "Defiance", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43536", "FIPS": "39039", "County": "Defiance", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43549", "FIPS": "39039", "County": "Defiance", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43556", "FIPS": "39039", "County": "Defiance", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43003", "FIPS": "39041", "County": "Delaware", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43015", "FIPS": "39041", "County": "Delaware", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43021", "FIPS": "39041", "County": "Delaware", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43032", "FIPS": "39041", "County": "Delaware", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43035", "FIPS": "39041", "County": "Delaware", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43061", "FIPS": "39041", "County": "Delaware", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43065", "FIPS": "39041", "County": "Delaware", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43066", "FIPS": "39041", "County": "Delaware", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43074", "FIPS": "39041", "County": "Delaware", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43082", "FIPS": "39041", "County": "Delaware", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43240", "FIPS": "39041", "County": "Delaware", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43438", "FIPS": "39043", "County": "Erie", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44089", "FIPS": "39043", "County": "Erie", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44814", "FIPS": "39043", "County": "Erie", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44816", "FIPS": "39043", "County": "Erie", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44824", "FIPS": "39043", "County": "Erie", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44839", "FIPS": "39043", "County": "Erie", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44846", "FIPS": "39043", "County": "Erie", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44870", "FIPS": "39043", "County": "Erie", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44871", "FIPS": "39043", "County": "Erie", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "43046", "FIPS": "39045", "County": "Fairfield", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43102", "FIPS": "39045", "County": "Fairfield", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43105", "FIPS": "39045", "County": "Fairfield", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43107", "FIPS": "39045", "County": "Fairfield", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43112", "FIPS": "39045", "County": "Fairfield", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43130", "FIPS": "39045", "County": "Fairfield", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43136", "FIPS": "39045", "County": "Fairfield", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43147", "FIPS": "39045", "County": "Fairfield", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43148", "FIPS": "39045", "County": "Fairfield", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43150", "FIPS": "39045", "County": "Fairfield", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43154", "FIPS": "39045", "County": "Fairfield", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43155", "FIPS": "39045", "County": "Fairfield", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43157", "FIPS": "39045", "County": "Fairfield", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43163", "FIPS": "39045", "County": "Fairfield", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43106", "FIPS": "39047", "County": "Fayette", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43128", "FIPS": "39047", "County": "Fayette", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43142", "FIPS": "39047", "County": "Fayette", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43160", "FIPS": "39047", "County": "Fayette", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43002", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43004", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43016", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43017", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43026", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43054", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43068", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43081", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43085", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43086", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43109", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43110", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43119", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43123", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43125", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43126", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43137", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43195", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43196", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43198", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43199", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43201", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43202", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43203", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43204", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43205", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43206", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43207", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43209", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43210", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43211", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43212", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43213", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43214", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43215", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43216", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43217", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43218", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43219", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43220", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43221", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43222", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43223", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43224", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43226", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43227", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43228", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43229", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43230", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43231", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43232", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43234", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43235", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43236", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43251", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43260", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43265", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43266", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43268", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43270", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43271", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43272", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43279", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43287", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43291", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43299", "FIPS": "39049", "County": "Franklin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43502", "FIPS": "39051", "County": "Fulton", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43515", "FIPS": "39051", "County": "Fulton", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43521", "FIPS": "39051", "County": "Fulton", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43533", "FIPS": "39051", "County": "Fulton", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43540", "FIPS": "39051", "County": "Fulton", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43553", "FIPS": "39051", "County": "Fulton", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43558", "FIPS": "39051", "County": "Fulton", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43567", "FIPS": "39051", "County": "Fulton", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "45614", "FIPS": "39053", "County": "Gallia", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45620", "FIPS": "39053", "County": "Gallia", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45623", "FIPS": "39053", "County": "Gallia", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45631", "FIPS": "39053", "County": "Gallia", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45643", "FIPS": "39053", "County": "Gallia", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45658", "FIPS": "39053", "County": "Gallia", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45674", "FIPS": "39053", "County": "Gallia", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45685", "FIPS": "39053", "County": "Gallia", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45686", "FIPS": "39053", "County": "Gallia", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "44021", "FIPS": "39055", "County": "Geauga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44023", "FIPS": "39055", "County": "Geauga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44024", "FIPS": "39055", "County": "Geauga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44026", "FIPS": "39055", "County": "Geauga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44033", "FIPS": "39055", "County": "Geauga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44046", "FIPS": "39055", "County": "Geauga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44062", "FIPS": "39055", "County": "Geauga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44064", "FIPS": "39055", "County": "Geauga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44065", "FIPS": "39055", "County": "Geauga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44072", "FIPS": "39055", "County": "Geauga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44073", "FIPS": "39055", "County": "Geauga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44080", "FIPS": "39055", "County": "Geauga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44086", "FIPS": "39055", "County": "Geauga", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "45301", "FIPS": "39057", "County": "Greene", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45305", "FIPS": "39057", "County": "Greene", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45307", "FIPS": "39057", "County": "Greene", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45314", "FIPS": "39057", "County": "Greene", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45316", "FIPS": "39057", "County": "Greene", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45324", "FIPS": "39057", "County": "Greene", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45335", "FIPS": "39057", "County": "Greene", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45370", "FIPS": "39057", "County": "Greene", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45384", "FIPS": "39057", "County": "Greene", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45385", "FIPS": "39057", "County": "Greene", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45387", "FIPS": "39057", "County": "Greene", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45431", "FIPS": "39057", "County": "Greene", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45432", "FIPS": "39057", "County": "Greene", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45433", "FIPS": "39057", "County": "Greene", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45434", "FIPS": "39057", "County": "Greene", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45435", "FIPS": "39057", "County": "Greene", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43722", "FIPS": "39059", "County": "Guernsey", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43723", "FIPS": "39059", "County": "Guernsey", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43725", "FIPS": "39059", "County": "Guernsey", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43732", "FIPS": "39059", "County": "Guernsey", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43733", "FIPS": "39059", "County": "Guernsey", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43736", "FIPS": "39059", "County": "Guernsey", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43749", "FIPS": "39059", "County": "Guernsey", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43750", "FIPS": "39059", "County": "Guernsey", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43755", "FIPS": "39059", "County": "Guernsey", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43768", "FIPS": "39059", "County": "Guernsey", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43772", "FIPS": "39059", "County": "Guernsey", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43773", "FIPS": "39059", "County": "Guernsey", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43778", "FIPS": "39059", "County": "Guernsey", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43780", "FIPS": "39059", "County": "Guernsey", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45001", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45002", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45030", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45033", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45041", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45051", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45052", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45111", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45174", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45201", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45202", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45203", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45204", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45205", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45206", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45207", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45208", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45209", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45211", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45212", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45213", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45214", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45215", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45216", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45217", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45218", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45219", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45220", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45221", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45222", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45223", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45224", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45225", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45226", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45227", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45228", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45229", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45230", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45231", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45232", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45233", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45234", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45235", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45236", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45237", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45238", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45239", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45240", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45241", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45242", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45243", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45244", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45246", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45247", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45248", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45249", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45250", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45251", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45252", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45253", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45254", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45255", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45258", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45262", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45263", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45264", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45267", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45268", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45269", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45270", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45271", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45273", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45274", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45277", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45280", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45296", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45298", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45299", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45999", "FIPS": "39061", "County": "Hamilton", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "44804", "FIPS": "39063", "County": "Hancock", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "45814", "FIPS": "39063", "County": "Hancock", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "45816", "FIPS": "39063", "County": "Hancock", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "45839", "FIPS": "39063", "County": "Hancock", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "45840", "FIPS": "39063", "County": "Hancock", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "45841", "FIPS": "39063", "County": "Hancock", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "45858", "FIPS": "39063", "County": "Hancock", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "45867", "FIPS": "39063", "County": "Hancock", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "45868", "FIPS": "39063", "County": "Hancock", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "45881", "FIPS": "39063", "County": "Hancock", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "45889", "FIPS": "39063", "County": "Hancock", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "45890", "FIPS": "39063", "County": "Hancock", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "45897", "FIPS": "39063", "County": "Hancock", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43326", "FIPS": "39065", "County": "Hardin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43340", "FIPS": "39065", "County": "Hardin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43345", "FIPS": "39065", "County": "Hardin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43346", "FIPS": "39065", "County": "Hardin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45810", "FIPS": "39065", "County": "Hardin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45812", "FIPS": "39065", "County": "Hardin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45835", "FIPS": "39065", "County": "Hardin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45836", "FIPS": "39065", "County": "Hardin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45843", "FIPS": "39065", "County": "Hardin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45859", "FIPS": "39065", "County": "Hardin", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43907", "FIPS": "39067", "County": "Harrison", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43973", "FIPS": "39067", "County": "Harrison", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43974", "FIPS": "39067", "County": "Harrison", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43976", "FIPS": "39067", "County": "Harrison", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43981", "FIPS": "39067", "County": "Harrison", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43984", "FIPS": "39067", "County": "Harrison", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43986", "FIPS": "39067", "County": "Harrison", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43988", "FIPS": "39067", "County": "Harrison", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "44693", "FIPS": "39067", "County": "Harrison", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "44695", "FIPS": "39067", "County": "Harrison", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "44699", "FIPS": "39067", "County": "Harrison", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43510", "FIPS": "39069", "County": "Henry", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43516", "FIPS": "39069", "County": "Henry", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43523", "FIPS": "39069", "County": "Henry", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43524", "FIPS": "39069", "County": "Henry", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43527", "FIPS": "39069", "County": "Henry", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43532", "FIPS": "39069", "County": "Henry", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43534", "FIPS": "39069", "County": "Henry", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43535", "FIPS": "39069", "County": "Henry", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43545", "FIPS": "39069", "County": "Henry", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43548", "FIPS": "39069", "County": "Henry", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43550", "FIPS": "39069", "County": "Henry", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43555", "FIPS": "39069", "County": "Henry", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "45110", "FIPS": "39071", "County": "Highland", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45123", "FIPS": "39071", "County": "Highland", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45132", "FIPS": "39071", "County": "Highland", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45133", "FIPS": "39071", "County": "Highland", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45135", "FIPS": "39071", "County": "Highland", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45142", "FIPS": "39071", "County": "Highland", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45155", "FIPS": "39071", "County": "Highland", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45172", "FIPS": "39071", "County": "Highland", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "43111", "FIPS": "39073", "County": "Hocking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43127", "FIPS": "39073", "County": "Hocking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43135", "FIPS": "39073", "County": "Hocking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43138", "FIPS": "39073", "County": "Hocking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43144", "FIPS": "39073", "County": "Hocking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43149", "FIPS": "39073", "County": "Hocking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43152", "FIPS": "39073", "County": "Hocking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43158", "FIPS": "39073", "County": "Hocking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "44610", "FIPS": "39075", "County": "Holmes", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44611", "FIPS": "39075", "County": "Holmes", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44617", "FIPS": "39075", "County": "Holmes", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44628", "FIPS": "39075", "County": "Holmes", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44633", "FIPS": "39075", "County": "Holmes", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44637", "FIPS": "39075", "County": "Holmes", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44638", "FIPS": "39075", "County": "Holmes", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44654", "FIPS": "39075", "County": "Holmes", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44660", "FIPS": "39075", "County": "Holmes", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44661", "FIPS": "39075", "County": "Holmes", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44687", "FIPS": "39075", "County": "Holmes", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44689", "FIPS": "39075", "County": "Holmes", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44690", "FIPS": "39075", "County": "Holmes", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44811", "FIPS": "39077", "County": "Huron", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44826", "FIPS": "39077", "County": "Huron", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44837", "FIPS": "39077", "County": "Huron", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44847", "FIPS": "39077", "County": "Huron", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44850", "FIPS": "39077", "County": "Huron", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44851", "FIPS": "39077", "County": "Huron", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44855", "FIPS": "39077", "County": "Huron", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44857", "FIPS": "39077", "County": "Huron", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44865", "FIPS": "39077", "County": "Huron", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44888", "FIPS": "39077", "County": "Huron", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44889", "FIPS": "39077", "County": "Huron", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44890", "FIPS": "39077", "County": "Huron", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "45621", "FIPS": "39079", "County": "Jackson", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45640", "FIPS": "39079", "County": "Jackson", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45656", "FIPS": "39079", "County": "Jackson", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45692", "FIPS": "39079", "County": "Jackson", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "43901", "FIPS": "39081", "County": "Jefferson", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43903", "FIPS": "39081", "County": "Jefferson", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43908", "FIPS": "39081", "County": "Jefferson", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43910", "FIPS": "39081", "County": "Jefferson", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43913", "FIPS": "39081", "County": "Jefferson", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43917", "FIPS": "39081", "County": "Jefferson", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43925", "FIPS": "39081", "County": "Jefferson", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43926", "FIPS": "39081", "County": "Jefferson", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43930", "FIPS": "39081", "County": "Jefferson", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43932", "FIPS": "39081", "County": "Jefferson", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43938", "FIPS": "39081", "County": "Jefferson", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43939", "FIPS": "39081", "County": "Jefferson", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43941", "FIPS": "39081", "County": "Jefferson", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43943", "FIPS": "39081", "County": "Jefferson", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43944", "FIPS": "39081", "County": "Jefferson", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43948", "FIPS": "39081", "County": "Jefferson", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43952", "FIPS": "39081", "County": "Jefferson", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43953", "FIPS": "39081", "County": "Jefferson", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43961", "FIPS": "39081", "County": "Jefferson", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43963", "FIPS": "39081", "County": "Jefferson", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43964", "FIPS": "39081", "County": "Jefferson", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43970", "FIPS": "39081", "County": "Jefferson", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43971", "FIPS": "39081", "County": "Jefferson", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43005", "FIPS": "39083", "County": "Knox", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43006", "FIPS": "39083", "County": "Knox", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43011", "FIPS": "39083", "County": "Knox", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43014", "FIPS": "39083", "County": "Knox", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43019", "FIPS": "39083", "County": "Knox", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43022", "FIPS": "39083", "County": "Knox", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43028", "FIPS": "39083", "County": "Knox", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43037", "FIPS": "39083", "County": "Knox", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43048", "FIPS": "39083", "County": "Knox", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43050", "FIPS": "39083", "County": "Knox", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "44045", "FIPS": "39085", "County": "Lake", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44057", "FIPS": "39085", "County": "Lake", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44060", "FIPS": "39085", "County": "Lake", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44061", "FIPS": "39085", "County": "Lake", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44077", "FIPS": "39085", "County": "Lake", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44081", "FIPS": "39085", "County": "Lake", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44092", "FIPS": "39085", "County": "Lake", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44094", "FIPS": "39085", "County": "Lake", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44095", "FIPS": "39085", "County": "Lake", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44096", "FIPS": "39085", "County": "Lake", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44097", "FIPS": "39085", "County": "Lake", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "45619", "FIPS": "39087", "County": "Lawrence", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45638", "FIPS": "39087", "County": "Lawrence", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45645", "FIPS": "39087", "County": "Lawrence", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45659", "FIPS": "39087", "County": "Lawrence", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45669", "FIPS": "39087", "County": "Lawrence", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45675", "FIPS": "39087", "County": "Lawrence", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45678", "FIPS": "39087", "County": "Lawrence", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45680", "FIPS": "39087", "County": "Lawrence", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45688", "FIPS": "39087", "County": "Lawrence", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45696", "FIPS": "39087", "County": "Lawrence", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "43001", "FIPS": "39089", "County": "Licking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43008", "FIPS": "39089", "County": "Licking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43013", "FIPS": "39089", "County": "Licking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43018", "FIPS": "39089", "County": "Licking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43023", "FIPS": "39089", "County": "Licking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43025", "FIPS": "39089", "County": "Licking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43027", "FIPS": "39089", "County": "Licking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43030", "FIPS": "39089", "County": "Licking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43031", "FIPS": "39089", "County": "Licking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43033", "FIPS": "39089", "County": "Licking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43055", "FIPS": "39089", "County": "Licking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43056", "FIPS": "39089", "County": "Licking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43058", "FIPS": "39089", "County": "Licking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43062", "FIPS": "39089", "County": "Licking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43071", "FIPS": "39089", "County": "Licking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43073", "FIPS": "39089", "County": "Licking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43080", "FIPS": "39089", "County": "Licking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43093", "FIPS": "39089", "County": "Licking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43098", "FIPS": "39089", "County": "Licking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43721", "FIPS": "39089", "County": "Licking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43740", "FIPS": "39089", "County": "Licking", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43310", "FIPS": "39091", "County": "Logan", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43311", "FIPS": "39091", "County": "Logan", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43318", "FIPS": "39091", "County": "Logan", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43319", "FIPS": "39091", "County": "Logan", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43324", "FIPS": "39091", "County": "Logan", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43331", "FIPS": "39091", "County": "Logan", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43333", "FIPS": "39091", "County": "Logan", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43336", "FIPS": "39091", "County": "Logan", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43343", "FIPS": "39091", "County": "Logan", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43347", "FIPS": "39091", "County": "Logan", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43348", "FIPS": "39091", "County": "Logan", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43357", "FIPS": "39091", "County": "Logan", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43358", "FIPS": "39091", "County": "Logan", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43360", "FIPS": "39091", "County": "Logan", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "44001", "FIPS": "39093", "County": "Lorain", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44011", "FIPS": "39093", "County": "Lorain", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44012", "FIPS": "39093", "County": "Lorain", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44028", "FIPS": "39093", "County": "Lorain", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44035", "FIPS": "39093", "County": "Lorain", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44036", "FIPS": "39093", "County": "Lorain", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44039", "FIPS": "39093", "County": "Lorain", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44044", "FIPS": "39093", "County": "Lorain", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44049", "FIPS": "39093", "County": "Lorain", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44050", "FIPS": "39093", "County": "Lorain", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44052", "FIPS": "39093", "County": "Lorain", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44053", "FIPS": "39093", "County": "Lorain", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44054", "FIPS": "39093", "County": "Lorain", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44055", "FIPS": "39093", "County": "Lorain", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44074", "FIPS": "39093", "County": "Lorain", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44090", "FIPS": "39093", "County": "Lorain", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "43434", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43504", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43528", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43537", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43542", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43547", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43560", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43566", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43571", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43601", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43602", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43603", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43604", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43605", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43606", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43607", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43608", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43609", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43610", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43611", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43612", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43613", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43614", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43615", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43616", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43617", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43618", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43620", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43623", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43624", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43635", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43652", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43656", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43657", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43659", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43660", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43661", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43666", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43667", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43681", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43682", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43697", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43699", "FIPS": "39095", "County": "Lucas", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43064", "FIPS": "39097", "County": "Madison", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43140", "FIPS": "39097", "County": "Madison", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43143", "FIPS": "39097", "County": "Madison", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43151", "FIPS": "39097", "County": "Madison", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43153", "FIPS": "39097", "County": "Madison", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43162", "FIPS": "39097", "County": "Madison", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "44401", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44405", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44406", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44416", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44422", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44429", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44436", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44442", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44443", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44451", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44452", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44454", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44471", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44501", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44502", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44503", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44504", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44505", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44506", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44507", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44509", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44510", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44511", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44512", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44513", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44514", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44515", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44555", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44609", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44619", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44672", "FIPS": "39099", "County": "Mahoning", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "43301", "FIPS": "39101", "County": "Marion", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43302", "FIPS": "39101", "County": "Marion", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43306", "FIPS": "39101", "County": "Marion", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43307", "FIPS": "39101", "County": "Marion", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43314", "FIPS": "39101", "County": "Marion", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43322", "FIPS": "39101", "County": "Marion", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43332", "FIPS": "39101", "County": "Marion", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43335", "FIPS": "39101", "County": "Marion", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43337", "FIPS": "39101", "County": "Marion", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43341", "FIPS": "39101", "County": "Marion", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43342", "FIPS": "39101", "County": "Marion", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43356", "FIPS": "39101", "County": "Marion", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "44212", "FIPS": "39103", "County": "Medina", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44215", "FIPS": "39103", "County": "Medina", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44233", "FIPS": "39103", "County": "Medina", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44235", "FIPS": "39103", "County": "Medina", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44251", "FIPS": "39103", "County": "Medina", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44253", "FIPS": "39103", "County": "Medina", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44254", "FIPS": "39103", "County": "Medina", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44256", "FIPS": "39103", "County": "Medina", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44258", "FIPS": "39103", "County": "Medina", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44273", "FIPS": "39103", "County": "Medina", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44274", "FIPS": "39103", "County": "Medina", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44275", "FIPS": "39103", "County": "Medina", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44280", "FIPS": "39103", "County": "Medina", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44281", "FIPS": "39103", "County": "Medina", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44282", "FIPS": "39103", "County": "Medina", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "45720", "FIPS": "39105", "County": "Meigs", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45741", "FIPS": "39105", "County": "Meigs", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45743", "FIPS": "39105", "County": "Meigs", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45760", "FIPS": "39105", "County": "Meigs", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45769", "FIPS": "39105", "County": "Meigs", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45770", "FIPS": "39105", "County": "Meigs", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45771", "FIPS": "39105", "County": "Meigs", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45772", "FIPS": "39105", "County": "Meigs", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45775", "FIPS": "39105", "County": "Meigs", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45776", "FIPS": "39105", "County": "Meigs", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45779", "FIPS": "39105", "County": "Meigs", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45783", "FIPS": "39105", "County": "Meigs", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45310", "FIPS": "39107", "County": "Mercer", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45822", "FIPS": "39107", "County": "Mercer", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45826", "FIPS": "39107", "County": "Mercer", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45828", "FIPS": "39107", "County": "Mercer", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45846", "FIPS": "39107", "County": "Mercer", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45860", "FIPS": "39107", "County": "Mercer", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45862", "FIPS": "39107", "County": "Mercer", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45866", "FIPS": "39107", "County": "Mercer", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45882", "FIPS": "39107", "County": "Mercer", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45883", "FIPS": "39107", "County": "Mercer", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45308", "FIPS": "39109", "County": "Miami", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45312", "FIPS": "39109", "County": "Miami", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45317", "FIPS": "39109", "County": "Miami", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45318", "FIPS": "39109", "County": "Miami", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45326", "FIPS": "39109", "County": "Miami", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45337", "FIPS": "39109", "County": "Miami", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45339", "FIPS": "39109", "County": "Miami", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45356", "FIPS": "39109", "County": "Miami", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45359", "FIPS": "39109", "County": "Miami", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45361", "FIPS": "39109", "County": "Miami", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45371", "FIPS": "39109", "County": "Miami", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45373", "FIPS": "39109", "County": "Miami", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45374", "FIPS": "39109", "County": "Miami", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45383", "FIPS": "39109", "County": "Miami", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43716", "FIPS": "39111", "County": "Monroe", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43747", "FIPS": "39111", "County": "Monroe", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43752", "FIPS": "39111", "County": "Monroe", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43754", "FIPS": "39111", "County": "Monroe", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43757", "FIPS": "39111", "County": "Monroe", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43786", "FIPS": "39111", "County": "Monroe", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43789", "FIPS": "39111", "County": "Monroe", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43793", "FIPS": "39111", "County": "Monroe", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43914", "FIPS": "39111", "County": "Monroe", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43915", "FIPS": "39111", "County": "Monroe", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43931", "FIPS": "39111", "County": "Monroe", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "43946", "FIPS": "39111", "County": "Monroe", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "45734", "FIPS": "39111", "County": "Monroe", "State": "OH", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "45309", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45315", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45322", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45325", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45327", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45342", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45343", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45345", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45354", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45377", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45401", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45402", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45403", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45404", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45405", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45406", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45408", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45409", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45410", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45412", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45413", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45414", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45415", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45416", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45417", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45418", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45419", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45420", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45422", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45423", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45424", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45426", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45427", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45428", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45429", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45430", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45437", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45439", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45440", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45441", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45448", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45449", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45454", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45458", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45459", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45463", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45469", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45470", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45475", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45479", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45481", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45482", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45490", "FIPS": "39113", "County": "Montgomery", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "43728", "FIPS": "39115", "County": "Morgan", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43756", "FIPS": "39115", "County": "Morgan", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43758", "FIPS": "39115", "County": "Morgan", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43787", "FIPS": "39115", "County": "Morgan", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43315", "FIPS": "39117", "County": "Morrow", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43317", "FIPS": "39117", "County": "Morrow", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43320", "FIPS": "39117", "County": "Morrow", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43321", "FIPS": "39117", "County": "Morrow", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43325", "FIPS": "39117", "County": "Morrow", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43334", "FIPS": "39117", "County": "Morrow", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43338", "FIPS": "39117", "County": "Morrow", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43349", "FIPS": "39117", "County": "Morrow", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43350", "FIPS": "39117", "County": "Morrow", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43701", "FIPS": "39119", "County": "Muskingum", "State": "OH", "DMA": {"Code": 596, "Name": "ZANESVILLE"}},
  {"Postal_Code": "43702", "FIPS": "39119", "County": "Muskingum", "State": "OH", "DMA": {"Code": 596, "Name": "ZANESVILLE"}},
  {"Postal_Code": "43720", "FIPS": "39119", "County": "Muskingum", "State": "OH", "DMA": {"Code": 596, "Name": "ZANESVILLE"}},
  {"Postal_Code": "43727", "FIPS": "39119", "County": "Muskingum", "State": "OH", "DMA": {"Code": 596, "Name": "ZANESVILLE"}},
  {"Postal_Code": "43734", "FIPS": "39119", "County": "Muskingum", "State": "OH", "DMA": {"Code": 596, "Name": "ZANESVILLE"}},
  {"Postal_Code": "43735", "FIPS": "39119", "County": "Muskingum", "State": "OH", "DMA": {"Code": 596, "Name": "ZANESVILLE"}},
  {"Postal_Code": "43738", "FIPS": "39119", "County": "Muskingum", "State": "OH", "DMA": {"Code": 596, "Name": "ZANESVILLE"}},
  {"Postal_Code": "43746", "FIPS": "39119", "County": "Muskingum", "State": "OH", "DMA": {"Code": 596, "Name": "ZANESVILLE"}},
  {"Postal_Code": "43762", "FIPS": "39119", "County": "Muskingum", "State": "OH", "DMA": {"Code": 596, "Name": "ZANESVILLE"}},
  {"Postal_Code": "43767", "FIPS": "39119", "County": "Muskingum", "State": "OH", "DMA": {"Code": 596, "Name": "ZANESVILLE"}},
  {"Postal_Code": "43771", "FIPS": "39119", "County": "Muskingum", "State": "OH", "DMA": {"Code": 596, "Name": "ZANESVILLE"}},
  {"Postal_Code": "43777", "FIPS": "39119", "County": "Muskingum", "State": "OH", "DMA": {"Code": 596, "Name": "ZANESVILLE"}},
  {"Postal_Code": "43791", "FIPS": "39119", "County": "Muskingum", "State": "OH", "DMA": {"Code": 596, "Name": "ZANESVILLE"}},
  {"Postal_Code": "43802", "FIPS": "39119", "County": "Muskingum", "State": "OH", "DMA": {"Code": 596, "Name": "ZANESVILLE"}},
  {"Postal_Code": "43821", "FIPS": "39119", "County": "Muskingum", "State": "OH", "DMA": {"Code": 596, "Name": "ZANESVILLE"}},
  {"Postal_Code": "43822", "FIPS": "39119", "County": "Muskingum", "State": "OH", "DMA": {"Code": 596, "Name": "ZANESVILLE"}},
  {"Postal_Code": "43830", "FIPS": "39119", "County": "Muskingum", "State": "OH", "DMA": {"Code": 596, "Name": "ZANESVILLE"}},
  {"Postal_Code": "43842", "FIPS": "39119", "County": "Muskingum", "State": "OH", "DMA": {"Code": 596, "Name": "ZANESVILLE"}},
  {"Postal_Code": "43711", "FIPS": "39121", "County": "Noble", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43717", "FIPS": "39121", "County": "Noble", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43724", "FIPS": "39121", "County": "Noble", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43779", "FIPS": "39121", "County": "Noble", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43788", "FIPS": "39121", "County": "Noble", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45727", "FIPS": "39121", "County": "Noble", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43408", "FIPS": "39123", "County": "Ottawa", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43412", "FIPS": "39123", "County": "Ottawa", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43416", "FIPS": "39123", "County": "Ottawa", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43430", "FIPS": "39123", "County": "Ottawa", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43432", "FIPS": "39123", "County": "Ottawa", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43433", "FIPS": "39123", "County": "Ottawa", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43436", "FIPS": "39123", "County": "Ottawa", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43439", "FIPS": "39123", "County": "Ottawa", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43440", "FIPS": "39123", "County": "Ottawa", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43445", "FIPS": "39123", "County": "Ottawa", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43446", "FIPS": "39123", "County": "Ottawa", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43449", "FIPS": "39123", "County": "Ottawa", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43452", "FIPS": "39123", "County": "Ottawa", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43456", "FIPS": "39123", "County": "Ottawa", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43458", "FIPS": "39123", "County": "Ottawa", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43468", "FIPS": "39123", "County": "Ottawa", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "45813", "FIPS": "39125", "County": "Paulding", "State": "OH", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "45821", "FIPS": "39125", "County": "Paulding", "State": "OH", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "45849", "FIPS": "39125", "County": "Paulding", "State": "OH", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "45851", "FIPS": "39125", "County": "Paulding", "State": "OH", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "45855", "FIPS": "39125", "County": "Paulding", "State": "OH", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "45861", "FIPS": "39125", "County": "Paulding", "State": "OH", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "45873", "FIPS": "39125", "County": "Paulding", "State": "OH", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "45879", "FIPS": "39125", "County": "Paulding", "State": "OH", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "45880", "FIPS": "39125", "County": "Paulding", "State": "OH", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "43076", "FIPS": "39127", "County": "Perry", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43730", "FIPS": "39127", "County": "Perry", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43731", "FIPS": "39127", "County": "Perry", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43739", "FIPS": "39127", "County": "Perry", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43748", "FIPS": "39127", "County": "Perry", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43760", "FIPS": "39127", "County": "Perry", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43761", "FIPS": "39127", "County": "Perry", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43764", "FIPS": "39127", "County": "Perry", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43766", "FIPS": "39127", "County": "Perry", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43782", "FIPS": "39127", "County": "Perry", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43783", "FIPS": "39127", "County": "Perry", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43103", "FIPS": "39129", "County": "Pickaway", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43113", "FIPS": "39129", "County": "Pickaway", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43116", "FIPS": "39129", "County": "Pickaway", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43117", "FIPS": "39129", "County": "Pickaway", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43145", "FIPS": "39129", "County": "Pickaway", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43146", "FIPS": "39129", "County": "Pickaway", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43156", "FIPS": "39129", "County": "Pickaway", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43164", "FIPS": "39129", "County": "Pickaway", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45613", "FIPS": "39131", "County": "Pike", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45624", "FIPS": "39131", "County": "Pike", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45642", "FIPS": "39131", "County": "Pike", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45646", "FIPS": "39131", "County": "Pike", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45661", "FIPS": "39131", "County": "Pike", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45683", "FIPS": "39131", "County": "Pike", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45687", "FIPS": "39131", "County": "Pike", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45690", "FIPS": "39131", "County": "Pike", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "44201", "FIPS": "39133", "County": "Portage", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44202", "FIPS": "39133", "County": "Portage", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44211", "FIPS": "39133", "County": "Portage", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44231", "FIPS": "39133", "County": "Portage", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44234", "FIPS": "39133", "County": "Portage", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44240", "FIPS": "39133", "County": "Portage", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44241", "FIPS": "39133", "County": "Portage", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44242", "FIPS": "39133", "County": "Portage", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44243", "FIPS": "39133", "County": "Portage", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44255", "FIPS": "39133", "County": "Portage", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44260", "FIPS": "39133", "County": "Portage", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44265", "FIPS": "39133", "County": "Portage", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44266", "FIPS": "39133", "County": "Portage", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44272", "FIPS": "39133", "County": "Portage", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44285", "FIPS": "39133", "County": "Portage", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44288", "FIPS": "39133", "County": "Portage", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44411", "FIPS": "39133", "County": "Portage", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44412", "FIPS": "39133", "County": "Portage", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44449", "FIPS": "39133", "County": "Portage", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "45070", "FIPS": "39135", "County": "Preble", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45311", "FIPS": "39135", "County": "Preble", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45320", "FIPS": "39135", "County": "Preble", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45321", "FIPS": "39135", "County": "Preble", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45330", "FIPS": "39135", "County": "Preble", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45338", "FIPS": "39135", "County": "Preble", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45347", "FIPS": "39135", "County": "Preble", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45378", "FIPS": "39135", "County": "Preble", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45381", "FIPS": "39135", "County": "Preble", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45382", "FIPS": "39135", "County": "Preble", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45815", "FIPS": "39137", "County": "Putnam", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45827", "FIPS": "39137", "County": "Putnam", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45830", "FIPS": "39137", "County": "Putnam", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45831", "FIPS": "39137", "County": "Putnam", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45837", "FIPS": "39137", "County": "Putnam", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45844", "FIPS": "39137", "County": "Putnam", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45848", "FIPS": "39137", "County": "Putnam", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45853", "FIPS": "39137", "County": "Putnam", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45856", "FIPS": "39137", "County": "Putnam", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45864", "FIPS": "39137", "County": "Putnam", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45875", "FIPS": "39137", "County": "Putnam", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45876", "FIPS": "39137", "County": "Putnam", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45877", "FIPS": "39137", "County": "Putnam", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "45893", "FIPS": "39137", "County": "Putnam", "State": "OH", "DMA": {"Code": 558, "Name": "LIMA"}},
  {"Postal_Code": "44813", "FIPS": "39139", "County": "Richland", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44822", "FIPS": "39139", "County": "Richland", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44843", "FIPS": "39139", "County": "Richland", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44862", "FIPS": "39139", "County": "Richland", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44875", "FIPS": "39139", "County": "Richland", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44878", "FIPS": "39139", "County": "Richland", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44901", "FIPS": "39139", "County": "Richland", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44902", "FIPS": "39139", "County": "Richland", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44903", "FIPS": "39139", "County": "Richland", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44904", "FIPS": "39139", "County": "Richland", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44905", "FIPS": "39139", "County": "Richland", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44906", "FIPS": "39139", "County": "Richland", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44907", "FIPS": "39139", "County": "Richland", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44999", "FIPS": "39139", "County": "Richland", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "43101", "FIPS": "39141", "County": "Ross", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43115", "FIPS": "39141", "County": "Ross", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45601", "FIPS": "39141", "County": "Ross", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45612", "FIPS": "39141", "County": "Ross", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45617", "FIPS": "39141", "County": "Ross", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45628", "FIPS": "39141", "County": "Ross", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45633", "FIPS": "39141", "County": "Ross", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45644", "FIPS": "39141", "County": "Ross", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45647", "FIPS": "39141", "County": "Ross", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45673", "FIPS": "39141", "County": "Ross", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45681", "FIPS": "39141", "County": "Ross", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43407", "FIPS": "39143", "County": "Sandusky", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43410", "FIPS": "39143", "County": "Sandusky", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43420", "FIPS": "39143", "County": "Sandusky", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43431", "FIPS": "39143", "County": "Sandusky", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43435", "FIPS": "39143", "County": "Sandusky", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43442", "FIPS": "39143", "County": "Sandusky", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43464", "FIPS": "39143", "County": "Sandusky", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43469", "FIPS": "39143", "County": "Sandusky", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "45629", "FIPS": "39145", "County": "Scioto", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45630", "FIPS": "39145", "County": "Scioto", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45636", "FIPS": "39145", "County": "Scioto", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45648", "FIPS": "39145", "County": "Scioto", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45652", "FIPS": "39145", "County": "Scioto", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45653", "FIPS": "39145", "County": "Scioto", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45657", "FIPS": "39145", "County": "Scioto", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45662", "FIPS": "39145", "County": "Scioto", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45663", "FIPS": "39145", "County": "Scioto", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45671", "FIPS": "39145", "County": "Scioto", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45677", "FIPS": "39145", "County": "Scioto", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45682", "FIPS": "39145", "County": "Scioto", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45684", "FIPS": "39145", "County": "Scioto", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45694", "FIPS": "39145", "County": "Scioto", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45699", "FIPS": "39145", "County": "Scioto", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "44802", "FIPS": "39147", "County": "Seneca", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "44807", "FIPS": "39147", "County": "Seneca", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "44809", "FIPS": "39147", "County": "Seneca", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "44815", "FIPS": "39147", "County": "Seneca", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "44818", "FIPS": "39147", "County": "Seneca", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "44828", "FIPS": "39147", "County": "Seneca", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "44830", "FIPS": "39147", "County": "Seneca", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "44836", "FIPS": "39147", "County": "Seneca", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "44841", "FIPS": "39147", "County": "Seneca", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "44845", "FIPS": "39147", "County": "Seneca", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "44853", "FIPS": "39147", "County": "Seneca", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "44861", "FIPS": "39147", "County": "Seneca", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "44867", "FIPS": "39147", "County": "Seneca", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "44883", "FIPS": "39147", "County": "Seneca", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "45302", "FIPS": "39149", "County": "Shelby", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45306", "FIPS": "39149", "County": "Shelby", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45333", "FIPS": "39149", "County": "Shelby", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45334", "FIPS": "39149", "County": "Shelby", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45336", "FIPS": "39149", "County": "Shelby", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45340", "FIPS": "39149", "County": "Shelby", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45353", "FIPS": "39149", "County": "Shelby", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45360", "FIPS": "39149", "County": "Shelby", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45363", "FIPS": "39149", "County": "Shelby", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45365", "FIPS": "39149", "County": "Shelby", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45367", "FIPS": "39149", "County": "Shelby", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "45845", "FIPS": "39149", "County": "Shelby", "State": "OH", "DMA": {"Code": 542, "Name": "DAYTON"}},
  {"Postal_Code": "44601", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44608", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44613", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44614", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44626", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44630", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44632", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44640", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44641", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44643", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44646", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44647", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44648", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44650", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44652", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44657", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44662", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44666", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44669", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44670", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44685", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44688", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44701", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44702", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44703", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44704", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44705", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44706", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44707", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44708", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44709", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44710", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44711", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44712", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44714", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44718", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44720", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44721", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44730", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44735", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44750", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44767", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44799", "FIPS": "39151", "County": "Stark", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44056", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44067", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44087", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44203", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44210", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44216", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44221", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44222", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44223", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44224", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44232", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44236", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44237", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44250", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44262", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44264", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44278", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44286", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44301", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44302", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44303", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44304", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44305", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44306", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44307", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44308", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44309", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44310", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44311", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44312", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44313", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44314", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44315", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44316", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44317", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44319", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44320", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44321", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44322", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44325", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44326", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44328", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44333", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44334", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44372", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44393", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44396", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44398", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44399", "FIPS": "39153", "County": "Summit", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44402", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44403", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44404", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44410", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44417", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44418", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44420", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44424", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44425", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44428", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44430", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44437", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44438", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44439", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44440", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44444", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44446", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44450", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44453", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44470", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44473", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44481", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44482", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44483", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44484", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44485", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44486", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44488", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "44491", "FIPS": "39155", "County": "Trumbull", "State": "OH", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "43804", "FIPS": "39157", "County": "Tuscarawas", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "43832", "FIPS": "39157", "County": "Tuscarawas", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "43837", "FIPS": "39157", "County": "Tuscarawas", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "43840", "FIPS": "39157", "County": "Tuscarawas", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44612", "FIPS": "39157", "County": "Tuscarawas", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44621", "FIPS": "39157", "County": "Tuscarawas", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44622", "FIPS": "39157", "County": "Tuscarawas", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44624", "FIPS": "39157", "County": "Tuscarawas", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44629", "FIPS": "39157", "County": "Tuscarawas", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44653", "FIPS": "39157", "County": "Tuscarawas", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44656", "FIPS": "39157", "County": "Tuscarawas", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44663", "FIPS": "39157", "County": "Tuscarawas", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44671", "FIPS": "39157", "County": "Tuscarawas", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44678", "FIPS": "39157", "County": "Tuscarawas", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44679", "FIPS": "39157", "County": "Tuscarawas", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44680", "FIPS": "39157", "County": "Tuscarawas", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44681", "FIPS": "39157", "County": "Tuscarawas", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44682", "FIPS": "39157", "County": "Tuscarawas", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44683", "FIPS": "39157", "County": "Tuscarawas", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44697", "FIPS": "39157", "County": "Tuscarawas", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "43007", "FIPS": "39159", "County": "Union", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43029", "FIPS": "39159", "County": "Union", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43036", "FIPS": "39159", "County": "Union", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43040", "FIPS": "39159", "County": "Union", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43041", "FIPS": "39159", "County": "Union", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43045", "FIPS": "39159", "County": "Union", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43067", "FIPS": "39159", "County": "Union", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43077", "FIPS": "39159", "County": "Union", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "43344", "FIPS": "39159", "County": "Union", "State": "OH", "DMA": {"Code": 535, "Name": "COLUMBUS, OH"}},
  {"Postal_Code": "45832", "FIPS": "39161", "County": "Van Wert", "State": "OH", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "45838", "FIPS": "39161", "County": "Van Wert", "State": "OH", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "45863", "FIPS": "39161", "County": "Van Wert", "State": "OH", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "45874", "FIPS": "39161", "County": "Van Wert", "State": "OH", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "45886", "FIPS": "39161", "County": "Van Wert", "State": "OH", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "45891", "FIPS": "39161", "County": "Van Wert", "State": "OH", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "45894", "FIPS": "39161", "County": "Van Wert", "State": "OH", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "45898", "FIPS": "39161", "County": "Van Wert", "State": "OH", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "45899", "FIPS": "39161", "County": "Van Wert", "State": "OH", "DMA": {"Code": 509, "Name": "FORT WAYNE"}},
  {"Postal_Code": "45622", "FIPS": "39163", "County": "Vinton", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45634", "FIPS": "39163", "County": "Vinton", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45651", "FIPS": "39163", "County": "Vinton", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45654", "FIPS": "39163", "County": "Vinton", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45672", "FIPS": "39163", "County": "Vinton", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45695", "FIPS": "39163", "County": "Vinton", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45698", "FIPS": "39163", "County": "Vinton", "State": "OH", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "45005", "FIPS": "39165", "County": "Warren", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45032", "FIPS": "39165", "County": "Warren", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45034", "FIPS": "39165", "County": "Warren", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45036", "FIPS": "39165", "County": "Warren", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45039", "FIPS": "39165", "County": "Warren", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45040", "FIPS": "39165", "County": "Warren", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45054", "FIPS": "39165", "County": "Warren", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45065", "FIPS": "39165", "County": "Warren", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45066", "FIPS": "39165", "County": "Warren", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45068", "FIPS": "39165", "County": "Warren", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45152", "FIPS": "39165", "County": "Warren", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45162", "FIPS": "39165", "County": "Warren", "State": "OH", "DMA": {"Code": 515, "Name": "CINCINNATI"}},
  {"Postal_Code": "45712", "FIPS": "39167", "County": "Washington", "State": "OH", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "45713", "FIPS": "39167", "County": "Washington", "State": "OH", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "45714", "FIPS": "39167", "County": "Washington", "State": "OH", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "45715", "FIPS": "39167", "County": "Washington", "State": "OH", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "45721", "FIPS": "39167", "County": "Washington", "State": "OH", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "45724", "FIPS": "39167", "County": "Washington", "State": "OH", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "45729", "FIPS": "39167", "County": "Washington", "State": "OH", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "45742", "FIPS": "39167", "County": "Washington", "State": "OH", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "45744", "FIPS": "39167", "County": "Washington", "State": "OH", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "45745", "FIPS": "39167", "County": "Washington", "State": "OH", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "45746", "FIPS": "39167", "County": "Washington", "State": "OH", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "45750", "FIPS": "39167", "County": "Washington", "State": "OH", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "45767", "FIPS": "39167", "County": "Washington", "State": "OH", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "45768", "FIPS": "39167", "County": "Washington", "State": "OH", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "45773", "FIPS": "39167", "County": "Washington", "State": "OH", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "45784", "FIPS": "39167", "County": "Washington", "State": "OH", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "45786", "FIPS": "39167", "County": "Washington", "State": "OH", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "45787", "FIPS": "39167", "County": "Washington", "State": "OH", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "45788", "FIPS": "39167", "County": "Washington", "State": "OH", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "45789", "FIPS": "39167", "County": "Washington", "State": "OH", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "44214", "FIPS": "39169", "County": "Wayne", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44217", "FIPS": "39169", "County": "Wayne", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44230", "FIPS": "39169", "County": "Wayne", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44270", "FIPS": "39169", "County": "Wayne", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44276", "FIPS": "39169", "County": "Wayne", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44287", "FIPS": "39169", "County": "Wayne", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44606", "FIPS": "39169", "County": "Wayne", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44618", "FIPS": "39169", "County": "Wayne", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44627", "FIPS": "39169", "County": "Wayne", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44636", "FIPS": "39169", "County": "Wayne", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44645", "FIPS": "39169", "County": "Wayne", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44659", "FIPS": "39169", "County": "Wayne", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44667", "FIPS": "39169", "County": "Wayne", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44676", "FIPS": "39169", "County": "Wayne", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44677", "FIPS": "39169", "County": "Wayne", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "44691", "FIPS": "39169", "County": "Wayne", "State": "OH", "DMA": {"Code": 510, "Name": "CLEVELAND-AKRON"}},
  {"Postal_Code": "43501", "FIPS": "39171", "County": "Williams", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43505", "FIPS": "39171", "County": "Williams", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43506", "FIPS": "39171", "County": "Williams", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43517", "FIPS": "39171", "County": "Williams", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43518", "FIPS": "39171", "County": "Williams", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43531", "FIPS": "39171", "County": "Williams", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43543", "FIPS": "39171", "County": "Williams", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43554", "FIPS": "39171", "County": "Williams", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43557", "FIPS": "39171", "County": "Williams", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43570", "FIPS": "39171", "County": "Williams", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43402", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43403", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43406", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43413", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43414", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43437", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43441", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43443", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43447", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43450", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43451", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43457", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43460", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43462", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43463", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43465", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43466", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43467", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43511", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43522", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43525", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43529", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43541", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43551", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43552", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43565", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43569", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43619", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43654", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "44817", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "45872", "FIPS": "39173", "County": "Wood", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43316", "FIPS": "39175", "County": "Wyandot", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43323", "FIPS": "39175", "County": "Wyandot", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43330", "FIPS": "39175", "County": "Wyandot", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43351", "FIPS": "39175", "County": "Wyandot", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "43359", "FIPS": "39175", "County": "Wyandot", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "44844", "FIPS": "39175", "County": "Wyandot", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "44849", "FIPS": "39175", "County": "Wyandot", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "44882", "FIPS": "39175", "County": "Wyandot", "State": "OH", "DMA": {"Code": 547, "Name": "TOLEDO"}},
  {"Postal_Code": "74457", "FIPS": "40001", "County": "Adair", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74931", "FIPS": "40001", "County": "Adair", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74960", "FIPS": "40001", "County": "Adair", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74964", "FIPS": "40001", "County": "Adair", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74965", "FIPS": "40001", "County": "Adair", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "73716", "FIPS": "40003", "County": "Alfalfa", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73719", "FIPS": "40003", "County": "Alfalfa", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73722", "FIPS": "40003", "County": "Alfalfa", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73726", "FIPS": "40003", "County": "Alfalfa", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73728", "FIPS": "40003", "County": "Alfalfa", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73739", "FIPS": "40003", "County": "Alfalfa", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73741", "FIPS": "40003", "County": "Alfalfa", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73749", "FIPS": "40003", "County": "Alfalfa", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74525", "FIPS": "40005", "County": "Atoka", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74533", "FIPS": "40005", "County": "Atoka", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74540", "FIPS": "40005", "County": "Atoka", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74542", "FIPS": "40005", "County": "Atoka", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74555", "FIPS": "40005", "County": "Atoka", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74569", "FIPS": "40005", "County": "Atoka", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73844", "FIPS": "40007", "County": "Beaver", "State": "OK", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "73931", "FIPS": "40007", "County": "Beaver", "State": "OK", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "73932", "FIPS": "40007", "County": "Beaver", "State": "OK", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "73938", "FIPS": "40007", "County": "Beaver", "State": "OK", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "73950", "FIPS": "40007", "County": "Beaver", "State": "OK", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "73627", "FIPS": "40009", "County": "Beckham", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73644", "FIPS": "40009", "County": "Beckham", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73645", "FIPS": "40009", "County": "Beckham", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73648", "FIPS": "40009", "County": "Beckham", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73662", "FIPS": "40009", "County": "Beckham", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73668", "FIPS": "40009", "County": "Beckham", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73040", "FIPS": "40011", "County": "Blaine", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73043", "FIPS": "40011", "County": "Blaine", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73724", "FIPS": "40011", "County": "Blaine", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73744", "FIPS": "40011", "County": "Blaine", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73755", "FIPS": "40011", "County": "Blaine", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73763", "FIPS": "40011", "County": "Blaine", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73770", "FIPS": "40011", "County": "Blaine", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73772", "FIPS": "40011", "County": "Blaine", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73449", "FIPS": "40013", "County": "Bryan", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74701", "FIPS": "40013", "County": "Bryan", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74702", "FIPS": "40013", "County": "Bryan", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74720", "FIPS": "40013", "County": "Bryan", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74721", "FIPS": "40013", "County": "Bryan", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74723", "FIPS": "40013", "County": "Bryan", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74726", "FIPS": "40013", "County": "Bryan", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74729", "FIPS": "40013", "County": "Bryan", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74730", "FIPS": "40013", "County": "Bryan", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74731", "FIPS": "40013", "County": "Bryan", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74733", "FIPS": "40013", "County": "Bryan", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74741", "FIPS": "40013", "County": "Bryan", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74747", "FIPS": "40013", "County": "Bryan", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74753", "FIPS": "40013", "County": "Bryan", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73001", "FIPS": "40015", "County": "Caddo", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73005", "FIPS": "40015", "County": "Caddo", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73006", "FIPS": "40015", "County": "Caddo", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73009", "FIPS": "40015", "County": "Caddo", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73015", "FIPS": "40015", "County": "Caddo", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73017", "FIPS": "40015", "County": "Caddo", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73029", "FIPS": "40015", "County": "Caddo", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73033", "FIPS": "40015", "County": "Caddo", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73038", "FIPS": "40015", "County": "Caddo", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73042", "FIPS": "40015", "County": "Caddo", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73047", "FIPS": "40015", "County": "Caddo", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73048", "FIPS": "40015", "County": "Caddo", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73053", "FIPS": "40015", "County": "Caddo", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73094", "FIPS": "40015", "County": "Caddo", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73014", "FIPS": "40017", "County": "Canadian", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73022", "FIPS": "40017", "County": "Canadian", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73036", "FIPS": "40017", "County": "Canadian", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73064", "FIPS": "40017", "County": "Canadian", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73078", "FIPS": "40017", "County": "Canadian", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73085", "FIPS": "40017", "County": "Canadian", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73090", "FIPS": "40017", "County": "Canadian", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73099", "FIPS": "40017", "County": "Canadian", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73401", "FIPS": "40019", "County": "Carter", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73402", "FIPS": "40019", "County": "Carter", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73403", "FIPS": "40019", "County": "Carter", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73435", "FIPS": "40019", "County": "Carter", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73436", "FIPS": "40019", "County": "Carter", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73437", "FIPS": "40019", "County": "Carter", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73438", "FIPS": "40019", "County": "Carter", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73443", "FIPS": "40019", "County": "Carter", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73444", "FIPS": "40019", "County": "Carter", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73458", "FIPS": "40019", "County": "Carter", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73463", "FIPS": "40019", "County": "Carter", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73481", "FIPS": "40019", "County": "Carter", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73487", "FIPS": "40019", "County": "Carter", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73488", "FIPS": "40019", "County": "Carter", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74427", "FIPS": "40021", "County": "Cherokee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74441", "FIPS": "40021", "County": "Cherokee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74444", "FIPS": "40021", "County": "Cherokee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74451", "FIPS": "40021", "County": "Cherokee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74452", "FIPS": "40021", "County": "Cherokee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74464", "FIPS": "40021", "County": "Cherokee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74465", "FIPS": "40021", "County": "Cherokee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74471", "FIPS": "40021", "County": "Cherokee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74727", "FIPS": "40023", "County": "Choctaw", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74735", "FIPS": "40023", "County": "Choctaw", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74738", "FIPS": "40023", "County": "Choctaw", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74743", "FIPS": "40023", "County": "Choctaw", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74756", "FIPS": "40023", "County": "Choctaw", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74759", "FIPS": "40023", "County": "Choctaw", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74760", "FIPS": "40023", "County": "Choctaw", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74761", "FIPS": "40023", "County": "Choctaw", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73933", "FIPS": "40025", "County": "Cimarron", "State": "OK", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "73937", "FIPS": "40025", "County": "Cimarron", "State": "OK", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "73946", "FIPS": "40025", "County": "Cimarron", "State": "OK", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "73947", "FIPS": "40025", "County": "Cimarron", "State": "OK", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "73019", "FIPS": "40027", "County": "Cleveland", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73026", "FIPS": "40027", "County": "Cleveland", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73051", "FIPS": "40027", "County": "Cleveland", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73068", "FIPS": "40027", "County": "Cleveland", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73069", "FIPS": "40027", "County": "Cleveland", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73070", "FIPS": "40027", "County": "Cleveland", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73071", "FIPS": "40027", "County": "Cleveland", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73072", "FIPS": "40027", "County": "Cleveland", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73153", "FIPS": "40027", "County": "Cleveland", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73160", "FIPS": "40027", "County": "Cleveland", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73165", "FIPS": "40027", "County": "Cleveland", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73170", "FIPS": "40027", "County": "Cleveland", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73189", "FIPS": "40027", "County": "Cleveland", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74857", "FIPS": "40027", "County": "Cleveland", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74534", "FIPS": "40029", "County": "Coal", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74535", "FIPS": "40029", "County": "Coal", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74538", "FIPS": "40029", "County": "Coal", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74556", "FIPS": "40029", "County": "Coal", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74572", "FIPS": "40029", "County": "Coal", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73501", "FIPS": "40031", "County": "Comanche", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73502", "FIPS": "40031", "County": "Comanche", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73503", "FIPS": "40031", "County": "Comanche", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73505", "FIPS": "40031", "County": "Comanche", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73506", "FIPS": "40031", "County": "Comanche", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73507", "FIPS": "40031", "County": "Comanche", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73527", "FIPS": "40031", "County": "Comanche", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73528", "FIPS": "40031", "County": "Comanche", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73538", "FIPS": "40031", "County": "Comanche", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73540", "FIPS": "40031", "County": "Comanche", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73541", "FIPS": "40031", "County": "Comanche", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73543", "FIPS": "40031", "County": "Comanche", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73552", "FIPS": "40031", "County": "Comanche", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73557", "FIPS": "40031", "County": "Comanche", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73558", "FIPS": "40031", "County": "Comanche", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73567", "FIPS": "40031", "County": "Comanche", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73531", "FIPS": "40033", "County": "Cotton", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73562", "FIPS": "40033", "County": "Cotton", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73568", "FIPS": "40033", "County": "Cotton", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73572", "FIPS": "40033", "County": "Cotton", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "74301", "FIPS": "40035", "County": "Craig", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74332", "FIPS": "40035", "County": "Craig", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74369", "FIPS": "40035", "County": "Craig", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74010", "FIPS": "40037", "County": "Creek", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74028", "FIPS": "40037", "County": "Creek", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74030", "FIPS": "40037", "County": "Creek", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74039", "FIPS": "40037", "County": "Creek", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74041", "FIPS": "40037", "County": "Creek", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74044", "FIPS": "40037", "County": "Creek", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74046", "FIPS": "40037", "County": "Creek", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74047", "FIPS": "40037", "County": "Creek", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74052", "FIPS": "40037", "County": "Creek", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74066", "FIPS": "40037", "County": "Creek", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74067", "FIPS": "40037", "County": "Creek", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74068", "FIPS": "40037", "County": "Creek", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74071", "FIPS": "40037", "County": "Creek", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74131", "FIPS": "40037", "County": "Creek", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "73096", "FIPS": "40039", "County": "Custer", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73601", "FIPS": "40039", "County": "Custer", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73620", "FIPS": "40039", "County": "Custer", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73625", "FIPS": "40039", "County": "Custer", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73639", "FIPS": "40039", "County": "Custer", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73669", "FIPS": "40039", "County": "Custer", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74338", "FIPS": "40041", "County": "Delaware", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74342", "FIPS": "40041", "County": "Delaware", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74344", "FIPS": "40041", "County": "Delaware", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74345", "FIPS": "40041", "County": "Delaware", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74346", "FIPS": "40041", "County": "Delaware", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74347", "FIPS": "40041", "County": "Delaware", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74359", "FIPS": "40041", "County": "Delaware", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74364", "FIPS": "40041", "County": "Delaware", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74368", "FIPS": "40041", "County": "Delaware", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "73646", "FIPS": "40043", "County": "Dewey", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73654", "FIPS": "40043", "County": "Dewey", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73658", "FIPS": "40043", "County": "Dewey", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73659", "FIPS": "40043", "County": "Dewey", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73663", "FIPS": "40043", "County": "Dewey", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73667", "FIPS": "40043", "County": "Dewey", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73835", "FIPS": "40043", "County": "Dewey", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73859", "FIPS": "40043", "County": "Dewey", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73832", "FIPS": "40045", "County": "Ellis", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73840", "FIPS": "40045", "County": "Ellis", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73843", "FIPS": "40045", "County": "Ellis", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73858", "FIPS": "40045", "County": "Ellis", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73701", "FIPS": "40047", "County": "Garfield", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73702", "FIPS": "40047", "County": "Garfield", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73703", "FIPS": "40047", "County": "Garfield", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73705", "FIPS": "40047", "County": "Garfield", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73706", "FIPS": "40047", "County": "Garfield", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73720", "FIPS": "40047", "County": "Garfield", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73727", "FIPS": "40047", "County": "Garfield", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73730", "FIPS": "40047", "County": "Garfield", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73733", "FIPS": "40047", "County": "Garfield", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73735", "FIPS": "40047", "County": "Garfield", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73736", "FIPS": "40047", "County": "Garfield", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73738", "FIPS": "40047", "County": "Garfield", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73743", "FIPS": "40047", "County": "Garfield", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73753", "FIPS": "40047", "County": "Garfield", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73754", "FIPS": "40047", "County": "Garfield", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73773", "FIPS": "40047", "County": "Garfield", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74640", "FIPS": "40047", "County": "Garfield", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73052", "FIPS": "40049", "County": "Garvin", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73057", "FIPS": "40049", "County": "Garvin", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73074", "FIPS": "40049", "County": "Garvin", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73075", "FIPS": "40049", "County": "Garvin", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73098", "FIPS": "40049", "County": "Garvin", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73433", "FIPS": "40049", "County": "Garvin", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74872", "FIPS": "40049", "County": "Garvin", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73002", "FIPS": "40051", "County": "Grady", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73004", "FIPS": "40051", "County": "Grady", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73011", "FIPS": "40051", "County": "Grady", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73018", "FIPS": "40051", "County": "Grady", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73023", "FIPS": "40051", "County": "Grady", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73059", "FIPS": "40051", "County": "Grady", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73067", "FIPS": "40051", "County": "Grady", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73079", "FIPS": "40051", "County": "Grady", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73082", "FIPS": "40051", "County": "Grady", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73089", "FIPS": "40051", "County": "Grady", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73092", "FIPS": "40051", "County": "Grady", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73758", "FIPS": "40053", "County": "Grant", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73759", "FIPS": "40053", "County": "Grant", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73761", "FIPS": "40053", "County": "Grant", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73766", "FIPS": "40053", "County": "Grant", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73771", "FIPS": "40053", "County": "Grant", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74636", "FIPS": "40053", "County": "Grant", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74643", "FIPS": "40053", "County": "Grant", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73547", "FIPS": "40055", "County": "Greer", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73554", "FIPS": "40055", "County": "Greer", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73673", "FIPS": "40055", "County": "Greer", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73544", "FIPS": "40057", "County": "Harmon", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73550", "FIPS": "40057", "County": "Harmon", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73571", "FIPS": "40057", "County": "Harmon", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73834", "FIPS": "40059", "County": "Harper", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73848", "FIPS": "40059", "County": "Harper", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73851", "FIPS": "40059", "County": "Harper", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73855", "FIPS": "40059", "County": "Harper", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74440", "FIPS": "40061", "County": "Haskell", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74462", "FIPS": "40061", "County": "Haskell", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74472", "FIPS": "40061", "County": "Haskell", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74552", "FIPS": "40061", "County": "Haskell", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74941", "FIPS": "40061", "County": "Haskell", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74943", "FIPS": "40061", "County": "Haskell", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74944", "FIPS": "40061", "County": "Haskell", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74531", "FIPS": "40063", "County": "Hughes", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74570", "FIPS": "40063", "County": "Hughes", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74827", "FIPS": "40063", "County": "Hughes", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74839", "FIPS": "40063", "County": "Hughes", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74848", "FIPS": "40063", "County": "Hughes", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74850", "FIPS": "40063", "County": "Hughes", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74883", "FIPS": "40063", "County": "Hughes", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73521", "FIPS": "40065", "County": "Jackson", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73522", "FIPS": "40065", "County": "Jackson", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73523", "FIPS": "40065", "County": "Jackson", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73526", "FIPS": "40065", "County": "Jackson", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73532", "FIPS": "40065", "County": "Jackson", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73537", "FIPS": "40065", "County": "Jackson", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73539", "FIPS": "40065", "County": "Jackson", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73549", "FIPS": "40065", "County": "Jackson", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73556", "FIPS": "40065", "County": "Jackson", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73560", "FIPS": "40065", "County": "Jackson", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73456", "FIPS": "40067", "County": "Jefferson", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73520", "FIPS": "40067", "County": "Jefferson", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73548", "FIPS": "40067", "County": "Jefferson", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73561", "FIPS": "40067", "County": "Jefferson", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73565", "FIPS": "40067", "County": "Jefferson", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73569", "FIPS": "40067", "County": "Jefferson", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73573", "FIPS": "40067", "County": "Jefferson", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73432", "FIPS": "40069", "County": "Johnston", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73447", "FIPS": "40069", "County": "Johnston", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73450", "FIPS": "40069", "County": "Johnston", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73455", "FIPS": "40069", "County": "Johnston", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73460", "FIPS": "40069", "County": "Johnston", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73461", "FIPS": "40069", "County": "Johnston", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74530", "FIPS": "40069", "County": "Johnston", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74748", "FIPS": "40069", "County": "Johnston", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74836", "FIPS": "40069", "County": "Johnston", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74856", "FIPS": "40069", "County": "Johnston", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74601", "FIPS": "40071", "County": "Kay", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74602", "FIPS": "40071", "County": "Kay", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74604", "FIPS": "40071", "County": "Kay", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74631", "FIPS": "40071", "County": "Kay", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74632", "FIPS": "40071", "County": "Kay", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74641", "FIPS": "40071", "County": "Kay", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74646", "FIPS": "40071", "County": "Kay", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74647", "FIPS": "40071", "County": "Kay", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74653", "FIPS": "40071", "County": "Kay", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73016", "FIPS": "40073", "County": "Kingfisher", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73734", "FIPS": "40073", "County": "Kingfisher", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73742", "FIPS": "40073", "County": "Kingfisher", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73750", "FIPS": "40073", "County": "Kingfisher", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73756", "FIPS": "40073", "County": "Kingfisher", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73762", "FIPS": "40073", "County": "Kingfisher", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73764", "FIPS": "40073", "County": "Kingfisher", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73041", "FIPS": "40075", "County": "Kiowa", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73062", "FIPS": "40075", "County": "Kiowa", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73559", "FIPS": "40075", "County": "Kiowa", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73564", "FIPS": "40075", "County": "Kiowa", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73566", "FIPS": "40075", "County": "Kiowa", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73651", "FIPS": "40075", "County": "Kiowa", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73655", "FIPS": "40075", "County": "Kiowa", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74545", "FIPS": "40077", "County": "Latimer", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74559", "FIPS": "40077", "County": "Latimer", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74563", "FIPS": "40077", "County": "Latimer", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74571", "FIPS": "40077", "County": "Latimer", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74578", "FIPS": "40077", "County": "Latimer", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74549", "FIPS": "40079", "County": "LeFlore", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74577", "FIPS": "40079", "County": "LeFlore", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74901", "FIPS": "40079", "County": "LeFlore", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74902", "FIPS": "40079", "County": "LeFlore", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74930", "FIPS": "40079", "County": "LeFlore", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74932", "FIPS": "40079", "County": "LeFlore", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74935", "FIPS": "40079", "County": "LeFlore", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74937", "FIPS": "40079", "County": "LeFlore", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74939", "FIPS": "40079", "County": "LeFlore", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74940", "FIPS": "40079", "County": "LeFlore", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74942", "FIPS": "40079", "County": "LeFlore", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74947", "FIPS": "40079", "County": "LeFlore", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74949", "FIPS": "40079", "County": "LeFlore", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74951", "FIPS": "40079", "County": "LeFlore", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74953", "FIPS": "40079", "County": "LeFlore", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74956", "FIPS": "40079", "County": "LeFlore", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74959", "FIPS": "40079", "County": "LeFlore", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74966", "FIPS": "40079", "County": "LeFlore", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74026", "FIPS": "40081", "County": "Lincoln", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74079", "FIPS": "40081", "County": "Lincoln", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74824", "FIPS": "40081", "County": "Lincoln", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74832", "FIPS": "40081", "County": "Lincoln", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74834", "FIPS": "40081", "County": "Lincoln", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74855", "FIPS": "40081", "County": "Lincoln", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74864", "FIPS": "40081", "County": "Lincoln", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74869", "FIPS": "40081", "County": "Lincoln", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74875", "FIPS": "40081", "County": "Lincoln", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74881", "FIPS": "40081", "County": "Lincoln", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73027", "FIPS": "40083", "County": "Logan", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73028", "FIPS": "40083", "County": "Logan", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73044", "FIPS": "40083", "County": "Logan", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73050", "FIPS": "40083", "County": "Logan", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73056", "FIPS": "40083", "County": "Logan", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73058", "FIPS": "40083", "County": "Logan", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73063", "FIPS": "40083", "County": "Logan", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73073", "FIPS": "40083", "County": "Logan", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73430", "FIPS": "40085", "County": "Love", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73441", "FIPS": "40085", "County": "Love", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73448", "FIPS": "40085", "County": "Love", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73453", "FIPS": "40085", "County": "Love", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73459", "FIPS": "40085", "County": "Love", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73010", "FIPS": "40087", "County": "McClain", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73031", "FIPS": "40087", "County": "McClain", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73065", "FIPS": "40087", "County": "McClain", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73080", "FIPS": "40087", "County": "McClain", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73093", "FIPS": "40087", "County": "McClain", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73095", "FIPS": "40087", "County": "McClain", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74831", "FIPS": "40087", "County": "McClain", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74722", "FIPS": "40089", "County": "McCurtain", "State": "OK", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "74724", "FIPS": "40089", "County": "McCurtain", "State": "OK", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "74728", "FIPS": "40089", "County": "McCurtain", "State": "OK", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "74734", "FIPS": "40089", "County": "McCurtain", "State": "OK", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "74736", "FIPS": "40089", "County": "McCurtain", "State": "OK", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "74737", "FIPS": "40089", "County": "McCurtain", "State": "OK", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "74740", "FIPS": "40089", "County": "McCurtain", "State": "OK", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "74745", "FIPS": "40089", "County": "McCurtain", "State": "OK", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "74750", "FIPS": "40089", "County": "McCurtain", "State": "OK", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "74752", "FIPS": "40089", "County": "McCurtain", "State": "OK", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "74754", "FIPS": "40089", "County": "McCurtain", "State": "OK", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "74755", "FIPS": "40089", "County": "McCurtain", "State": "OK", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "74764", "FIPS": "40089", "County": "McCurtain", "State": "OK", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "74766", "FIPS": "40089", "County": "McCurtain", "State": "OK", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "74957", "FIPS": "40089", "County": "McCurtain", "State": "OK", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "74963", "FIPS": "40089", "County": "McCurtain", "State": "OK", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "74426", "FIPS": "40091", "County": "McIntosh", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74432", "FIPS": "40091", "County": "McIntosh", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74438", "FIPS": "40091", "County": "McIntosh", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74459", "FIPS": "40091", "County": "McIntosh", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74461", "FIPS": "40091", "County": "McIntosh", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74845", "FIPS": "40091", "County": "McIntosh", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "73718", "FIPS": "40093", "County": "Major", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73729", "FIPS": "40093", "County": "Major", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73737", "FIPS": "40093", "County": "Major", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73747", "FIPS": "40093", "County": "Major", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73760", "FIPS": "40093", "County": "Major", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73768", "FIPS": "40093", "County": "Major", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73838", "FIPS": "40093", "County": "Major", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73439", "FIPS": "40095", "County": "Marshall", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73440", "FIPS": "40095", "County": "Marshall", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73446", "FIPS": "40095", "County": "Marshall", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74330", "FIPS": "40097", "County": "Mayes", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74337", "FIPS": "40097", "County": "Mayes", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74340", "FIPS": "40097", "County": "Mayes", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74349", "FIPS": "40097", "County": "Mayes", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74350", "FIPS": "40097", "County": "Mayes", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74352", "FIPS": "40097", "County": "Mayes", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74361", "FIPS": "40097", "County": "Mayes", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74362", "FIPS": "40097", "County": "Mayes", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74365", "FIPS": "40097", "County": "Mayes", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74366", "FIPS": "40097", "County": "Mayes", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74367", "FIPS": "40097", "County": "Mayes", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "73030", "FIPS": "40099", "County": "Murray", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73032", "FIPS": "40099", "County": "Murray", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73039", "FIPS": "40099", "County": "Murray", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73086", "FIPS": "40099", "County": "Murray", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74401", "FIPS": "40101", "County": "Muskogee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74402", "FIPS": "40101", "County": "Muskogee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74403", "FIPS": "40101", "County": "Muskogee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74422", "FIPS": "40101", "County": "Muskogee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74423", "FIPS": "40101", "County": "Muskogee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74428", "FIPS": "40101", "County": "Muskogee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74434", "FIPS": "40101", "County": "Muskogee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74436", "FIPS": "40101", "County": "Muskogee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74439", "FIPS": "40101", "County": "Muskogee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74450", "FIPS": "40101", "County": "Muskogee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74455", "FIPS": "40101", "County": "Muskogee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74463", "FIPS": "40101", "County": "Muskogee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74468", "FIPS": "40101", "County": "Muskogee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74469", "FIPS": "40101", "County": "Muskogee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74470", "FIPS": "40101", "County": "Muskogee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "73061", "FIPS": "40103", "County": "Noble", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73077", "FIPS": "40103", "County": "Noble", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73757", "FIPS": "40103", "County": "Noble", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74630", "FIPS": "40103", "County": "Noble", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74644", "FIPS": "40103", "County": "Noble", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74651", "FIPS": "40103", "County": "Noble", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74027", "FIPS": "40105", "County": "Nowata", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74042", "FIPS": "40105", "County": "Nowata", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74048", "FIPS": "40105", "County": "Nowata", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74072", "FIPS": "40105", "County": "Nowata", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74083", "FIPS": "40105", "County": "Nowata", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74829", "FIPS": "40107", "County": "Okfuskee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74833", "FIPS": "40107", "County": "Okfuskee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74859", "FIPS": "40107", "County": "Okfuskee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74860", "FIPS": "40107", "County": "Okfuskee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74880", "FIPS": "40107", "County": "Okfuskee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "73003", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73007", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73008", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73013", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73020", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73034", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73045", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73049", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73054", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73066", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73083", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73084", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73097", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73101", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73102", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73103", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73104", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73105", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73106", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73107", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73108", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73109", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73110", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73111", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73112", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73113", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73114", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73115", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73116", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73117", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73118", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73119", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73120", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73121", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73122", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73123", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73124", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73125", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73126", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73127", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73128", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73129", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73130", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73131", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73132", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73134", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73135", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73136", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73137", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73139", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73140", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73141", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73142", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73143", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73144", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73145", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73146", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73147", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73148", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73149", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73150", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73151", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73152", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73154", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73155", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73156", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73157", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73159", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73162", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73163", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73164", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73167", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73169", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73172", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73173", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73178", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73179", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73184", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73185", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73190", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73193", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73194", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73195", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73196", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73197", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73198", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73199", "FIPS": "40109", "County": "Oklahoma", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74421", "FIPS": "40111", "County": "Okmulgee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74431", "FIPS": "40111", "County": "Okmulgee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74437", "FIPS": "40111", "County": "Okmulgee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74445", "FIPS": "40111", "County": "Okmulgee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74447", "FIPS": "40111", "County": "Okmulgee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74456", "FIPS": "40111", "County": "Okmulgee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74460", "FIPS": "40111", "County": "Okmulgee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74001", "FIPS": "40113", "County": "Osage", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74002", "FIPS": "40113", "County": "Osage", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74035", "FIPS": "40113", "County": "Osage", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74054", "FIPS": "40113", "County": "Osage", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74056", "FIPS": "40113", "County": "Osage", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74060", "FIPS": "40113", "County": "Osage", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74070", "FIPS": "40113", "County": "Osage", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74084", "FIPS": "40113", "County": "Osage", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74633", "FIPS": "40113", "County": "Osage", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74637", "FIPS": "40113", "County": "Osage", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74652", "FIPS": "40113", "County": "Osage", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74331", "FIPS": "40115", "County": "Ottawa", "State": "OK", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "74333", "FIPS": "40115", "County": "Ottawa", "State": "OK", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "74335", "FIPS": "40115", "County": "Ottawa", "State": "OK", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "74339", "FIPS": "40115", "County": "Ottawa", "State": "OK", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "74343", "FIPS": "40115", "County": "Ottawa", "State": "OK", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "74354", "FIPS": "40115", "County": "Ottawa", "State": "OK", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "74355", "FIPS": "40115", "County": "Ottawa", "State": "OK", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "74358", "FIPS": "40115", "County": "Ottawa", "State": "OK", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "74360", "FIPS": "40115", "County": "Ottawa", "State": "OK", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "74363", "FIPS": "40115", "County": "Ottawa", "State": "OK", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "74370", "FIPS": "40115", "County": "Ottawa", "State": "OK", "DMA": {"Code": 603, "Name": "JOPLIN-PITTSBURG"}},
  {"Postal_Code": "74020", "FIPS": "40117", "County": "Pawnee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74034", "FIPS": "40117", "County": "Pawnee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74038", "FIPS": "40117", "County": "Pawnee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74045", "FIPS": "40117", "County": "Pawnee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74058", "FIPS": "40117", "County": "Pawnee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74081", "FIPS": "40117", "County": "Pawnee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74650", "FIPS": "40117", "County": "Pawnee", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74023", "FIPS": "40119", "County": "Payne", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74032", "FIPS": "40119", "County": "Payne", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74059", "FIPS": "40119", "County": "Payne", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74062", "FIPS": "40119", "County": "Payne", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74074", "FIPS": "40119", "County": "Payne", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74075", "FIPS": "40119", "County": "Payne", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74076", "FIPS": "40119", "County": "Payne", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74077", "FIPS": "40119", "County": "Payne", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74078", "FIPS": "40119", "County": "Payne", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74085", "FIPS": "40119", "County": "Payne", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74425", "FIPS": "40121", "County": "Pittsburg", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74430", "FIPS": "40121", "County": "Pittsburg", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74442", "FIPS": "40121", "County": "Pittsburg", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74501", "FIPS": "40121", "County": "Pittsburg", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74502", "FIPS": "40121", "County": "Pittsburg", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74522", "FIPS": "40121", "County": "Pittsburg", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74528", "FIPS": "40121", "County": "Pittsburg", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74529", "FIPS": "40121", "County": "Pittsburg", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74546", "FIPS": "40121", "County": "Pittsburg", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74547", "FIPS": "40121", "County": "Pittsburg", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74553", "FIPS": "40121", "County": "Pittsburg", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74554", "FIPS": "40121", "County": "Pittsburg", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74560", "FIPS": "40121", "County": "Pittsburg", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74561", "FIPS": "40121", "County": "Pittsburg", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74565", "FIPS": "40121", "County": "Pittsburg", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74576", "FIPS": "40121", "County": "Pittsburg", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74820", "FIPS": "40123", "County": "Pontotoc", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74821", "FIPS": "40123", "County": "Pontotoc", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74825", "FIPS": "40123", "County": "Pontotoc", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74842", "FIPS": "40123", "County": "Pontotoc", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74843", "FIPS": "40123", "County": "Pontotoc", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74844", "FIPS": "40123", "County": "Pontotoc", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74865", "FIPS": "40123", "County": "Pontotoc", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74871", "FIPS": "40123", "County": "Pontotoc", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74801", "FIPS": "40125", "County": "Pottawatomie", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74802", "FIPS": "40125", "County": "Pottawatomie", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74804", "FIPS": "40125", "County": "Pottawatomie", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74826", "FIPS": "40125", "County": "Pottawatomie", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74840", "FIPS": "40125", "County": "Pottawatomie", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74851", "FIPS": "40125", "County": "Pottawatomie", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74852", "FIPS": "40125", "County": "Pottawatomie", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74854", "FIPS": "40125", "County": "Pottawatomie", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74866", "FIPS": "40125", "County": "Pottawatomie", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74873", "FIPS": "40125", "County": "Pottawatomie", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74878", "FIPS": "40125", "County": "Pottawatomie", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74521", "FIPS": "40127", "County": "Pushmataha", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74523", "FIPS": "40127", "County": "Pushmataha", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74536", "FIPS": "40127", "County": "Pushmataha", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74543", "FIPS": "40127", "County": "Pushmataha", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74557", "FIPS": "40127", "County": "Pushmataha", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74558", "FIPS": "40127", "County": "Pushmataha", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74562", "FIPS": "40127", "County": "Pushmataha", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74567", "FIPS": "40127", "County": "Pushmataha", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "74574", "FIPS": "40127", "County": "Pushmataha", "State": "OK", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "73628", "FIPS": "40129", "County": "Roger Mills", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73638", "FIPS": "40129", "County": "Roger Mills", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73642", "FIPS": "40129", "County": "Roger Mills", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73650", "FIPS": "40129", "County": "Roger Mills", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73660", "FIPS": "40129", "County": "Roger Mills", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73666", "FIPS": "40129", "County": "Roger Mills", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74015", "FIPS": "40131", "County": "Rogers", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74016", "FIPS": "40131", "County": "Rogers", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74017", "FIPS": "40131", "County": "Rogers", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74018", "FIPS": "40131", "County": "Rogers", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74019", "FIPS": "40131", "County": "Rogers", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74031", "FIPS": "40131", "County": "Rogers", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74036", "FIPS": "40131", "County": "Rogers", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74053", "FIPS": "40131", "County": "Rogers", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74080", "FIPS": "40131", "County": "Rogers", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74818", "FIPS": "40133", "County": "Seminole", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74830", "FIPS": "40133", "County": "Seminole", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74837", "FIPS": "40133", "County": "Seminole", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74849", "FIPS": "40133", "County": "Seminole", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74867", "FIPS": "40133", "County": "Seminole", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74868", "FIPS": "40133", "County": "Seminole", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74884", "FIPS": "40133", "County": "Seminole", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "74435", "FIPS": "40135", "County": "Sequoyah", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74936", "FIPS": "40135", "County": "Sequoyah", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74945", "FIPS": "40135", "County": "Sequoyah", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74946", "FIPS": "40135", "County": "Sequoyah", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74948", "FIPS": "40135", "County": "Sequoyah", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74954", "FIPS": "40135", "County": "Sequoyah", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74955", "FIPS": "40135", "County": "Sequoyah", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "74962", "FIPS": "40135", "County": "Sequoyah", "State": "OK", "DMA": {"Code": 670, "Name": "FORT SMITH-FAY-SPRNGDL"}},
  {"Postal_Code": "73055", "FIPS": "40137", "County": "Stephens", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73425", "FIPS": "40137", "County": "Stephens", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73434", "FIPS": "40137", "County": "Stephens", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73442", "FIPS": "40137", "County": "Stephens", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73491", "FIPS": "40137", "County": "Stephens", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73529", "FIPS": "40137", "County": "Stephens", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73533", "FIPS": "40137", "County": "Stephens", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73534", "FIPS": "40137", "County": "Stephens", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73536", "FIPS": "40137", "County": "Stephens", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73901", "FIPS": "40139", "County": "Texas", "State": "OK", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "73939", "FIPS": "40139", "County": "Texas", "State": "OK", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "73942", "FIPS": "40139", "County": "Texas", "State": "OK", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "73944", "FIPS": "40139", "County": "Texas", "State": "OK", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "73945", "FIPS": "40139", "County": "Texas", "State": "OK", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "73949", "FIPS": "40139", "County": "Texas", "State": "OK", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "73951", "FIPS": "40139", "County": "Texas", "State": "OK", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "73530", "FIPS": "40141", "County": "Tillman", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73542", "FIPS": "40141", "County": "Tillman", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73546", "FIPS": "40141", "County": "Tillman", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73551", "FIPS": "40141", "County": "Tillman", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73553", "FIPS": "40141", "County": "Tillman", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73555", "FIPS": "40141", "County": "Tillman", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "73570", "FIPS": "40141", "County": "Tillman", "State": "OK", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "74008", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74011", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74012", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74013", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74021", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74033", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74037", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74043", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74050", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74055", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74063", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74073", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74101", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74102", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74103", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74104", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74105", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74106", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74107", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74108", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74110", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74112", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74114", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74115", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74116", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74117", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74119", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74120", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74121", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74126", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74127", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74128", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74129", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74130", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74132", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74133", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74134", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74135", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74136", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74137", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74141", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74145", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74146", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74147", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74148", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74149", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74150", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74152", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74153", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74155", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74156", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74157", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74158", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74159", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74169", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74170", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74171", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74172", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74182", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74183", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74184", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74186", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74187", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74189", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74192", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74193", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74194", "FIPS": "40143", "County": "Tulsa", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74014", "FIPS": "40145", "County": "Wagoner", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74429", "FIPS": "40145", "County": "Wagoner", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74446", "FIPS": "40145", "County": "Wagoner", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74454", "FIPS": "40145", "County": "Wagoner", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74458", "FIPS": "40145", "County": "Wagoner", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74467", "FIPS": "40145", "County": "Wagoner", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74477", "FIPS": "40145", "County": "Wagoner", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74003", "FIPS": "40147", "County": "Washington", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74004", "FIPS": "40147", "County": "Washington", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74005", "FIPS": "40147", "County": "Washington", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74006", "FIPS": "40147", "County": "Washington", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74022", "FIPS": "40147", "County": "Washington", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74029", "FIPS": "40147", "County": "Washington", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74051", "FIPS": "40147", "County": "Washington", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74061", "FIPS": "40147", "County": "Washington", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "74082", "FIPS": "40147", "County": "Washington", "State": "OK", "DMA": {"Code": 671, "Name": "TULSA"}},
  {"Postal_Code": "73021", "FIPS": "40149", "County": "Washita", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73024", "FIPS": "40149", "County": "Washita", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73622", "FIPS": "40149", "County": "Washita", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73624", "FIPS": "40149", "County": "Washita", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73626", "FIPS": "40149", "County": "Washita", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73632", "FIPS": "40149", "County": "Washita", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73641", "FIPS": "40149", "County": "Washita", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73647", "FIPS": "40149", "County": "Washita", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73661", "FIPS": "40149", "County": "Washita", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73664", "FIPS": "40149", "County": "Washita", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73717", "FIPS": "40151", "County": "Woods", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73731", "FIPS": "40151", "County": "Woods", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73746", "FIPS": "40151", "County": "Woods", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73842", "FIPS": "40151", "County": "Woods", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73860", "FIPS": "40151", "County": "Woods", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73801", "FIPS": "40153", "County": "Woodward", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73802", "FIPS": "40153", "County": "Woodward", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73841", "FIPS": "40153", "County": "Woodward", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73852", "FIPS": "40153", "County": "Woodward", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73853", "FIPS": "40153", "County": "Woodward", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "73857", "FIPS": "40153", "County": "Woodward", "State": "OK", "DMA": {"Code": 650, "Name": "OKLAHOMA CITY"}},
  {"Postal_Code": "97814", "FIPS": "41001", "County": "Baker", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97819", "FIPS": "41001", "County": "Baker", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97833", "FIPS": "41001", "County": "Baker", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97834", "FIPS": "41001", "County": "Baker", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97837", "FIPS": "41001", "County": "Baker", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97840", "FIPS": "41001", "County": "Baker", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97870", "FIPS": "41001", "County": "Baker", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97877", "FIPS": "41001", "County": "Baker", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97884", "FIPS": "41001", "County": "Baker", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97905", "FIPS": "41001", "County": "Baker", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97907", "FIPS": "41001", "County": "Baker", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97324", "FIPS": "41003", "County": "Benton", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97326", "FIPS": "41003", "County": "Benton", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97330", "FIPS": "41003", "County": "Benton", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97331", "FIPS": "41003", "County": "Benton", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97333", "FIPS": "41003", "County": "Benton", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97339", "FIPS": "41003", "County": "Benton", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97370", "FIPS": "41003", "County": "Benton", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97456", "FIPS": "41003", "County": "Benton", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97004", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97009", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97011", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97013", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97015", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97017", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97022", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97023", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97027", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97028", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97034", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97035", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97036", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97038", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97042", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97045", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97049", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97055", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97067", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97068", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97070", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97086", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97089", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97222", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97267", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97268", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97269", "FIPS": "41005", "County": "Clackamas", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97102", "FIPS": "41007", "County": "Clatsop", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97103", "FIPS": "41007", "County": "Clatsop", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97110", "FIPS": "41007", "County": "Clatsop", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97121", "FIPS": "41007", "County": "Clatsop", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97138", "FIPS": "41007", "County": "Clatsop", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97145", "FIPS": "41007", "County": "Clatsop", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97146", "FIPS": "41007", "County": "Clatsop", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97016", "FIPS": "41009", "County": "Columbia", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97018", "FIPS": "41009", "County": "Columbia", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97048", "FIPS": "41009", "County": "Columbia", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97051", "FIPS": "41009", "County": "Columbia", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97053", "FIPS": "41009", "County": "Columbia", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97054", "FIPS": "41009", "County": "Columbia", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97056", "FIPS": "41009", "County": "Columbia", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97064", "FIPS": "41009", "County": "Columbia", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97407", "FIPS": "41011", "County": "Coos", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97411", "FIPS": "41011", "County": "Coos", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97414", "FIPS": "41011", "County": "Coos", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97420", "FIPS": "41011", "County": "Coos", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97423", "FIPS": "41011", "County": "Coos", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97449", "FIPS": "41011", "County": "Coos", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97458", "FIPS": "41011", "County": "Coos", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97459", "FIPS": "41011", "County": "Coos", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97466", "FIPS": "41011", "County": "Coos", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97751", "FIPS": "41013", "County": "Crook", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97752", "FIPS": "41013", "County": "Crook", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97753", "FIPS": "41013", "County": "Crook", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97754", "FIPS": "41013", "County": "Crook", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97406", "FIPS": "41015", "County": "Curry", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97415", "FIPS": "41015", "County": "Curry", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97444", "FIPS": "41015", "County": "Curry", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97450", "FIPS": "41015", "County": "Curry", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97464", "FIPS": "41015", "County": "Curry", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97465", "FIPS": "41015", "County": "Curry", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97476", "FIPS": "41015", "County": "Curry", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97491", "FIPS": "41015", "County": "Curry", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97701", "FIPS": "41017", "County": "Deschutes", "State": "OR", "DMA": {"Code": 821, "Name": "BEND, OR"}},
  {"Postal_Code": "97702", "FIPS": "41017", "County": "Deschutes", "State": "OR", "DMA": {"Code": 821, "Name": "BEND, OR"}},
  {"Postal_Code": "97707", "FIPS": "41017", "County": "Deschutes", "State": "OR", "DMA": {"Code": 821, "Name": "BEND, OR"}},
  {"Postal_Code": "97708", "FIPS": "41017", "County": "Deschutes", "State": "OR", "DMA": {"Code": 821, "Name": "BEND, OR"}},
  {"Postal_Code": "97709", "FIPS": "41017", "County": "Deschutes", "State": "OR", "DMA": {"Code": 821, "Name": "BEND, OR"}},
  {"Postal_Code": "97712", "FIPS": "41017", "County": "Deschutes", "State": "OR", "DMA": {"Code": 821, "Name": "BEND, OR"}},
  {"Postal_Code": "97739", "FIPS": "41017", "County": "Deschutes", "State": "OR", "DMA": {"Code": 821, "Name": "BEND, OR"}},
  {"Postal_Code": "97756", "FIPS": "41017", "County": "Deschutes", "State": "OR", "DMA": {"Code": 821, "Name": "BEND, OR"}},
  {"Postal_Code": "97759", "FIPS": "41017", "County": "Deschutes", "State": "OR", "DMA": {"Code": 821, "Name": "BEND, OR"}},
  {"Postal_Code": "97410", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97416", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97417", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97428", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97429", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97432", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97435", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97436", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97441", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97442", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97443", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97447", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97457", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97462", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97467", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97469", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97470", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97473", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97479", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97481", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97484", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97486", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97494", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97495", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97496", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97499", "FIPS": "41019", "County": "Douglas", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97812", "FIPS": "41021", "County": "Gilliam", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97823", "FIPS": "41021", "County": "Gilliam", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97861", "FIPS": "41021", "County": "Gilliam", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97817", "FIPS": "41023", "County": "Grant", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97820", "FIPS": "41023", "County": "Grant", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97825", "FIPS": "41023", "County": "Grant", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97845", "FIPS": "41023", "County": "Grant", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97848", "FIPS": "41023", "County": "Grant", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97856", "FIPS": "41023", "County": "Grant", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97864", "FIPS": "41023", "County": "Grant", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97865", "FIPS": "41023", "County": "Grant", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97869", "FIPS": "41023", "County": "Grant", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97873", "FIPS": "41023", "County": "Grant", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97710", "FIPS": "41025", "County": "Harney", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97720", "FIPS": "41025", "County": "Harney", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97721", "FIPS": "41025", "County": "Harney", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97722", "FIPS": "41025", "County": "Harney", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97732", "FIPS": "41025", "County": "Harney", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97736", "FIPS": "41025", "County": "Harney", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97738", "FIPS": "41025", "County": "Harney", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97758", "FIPS": "41025", "County": "Harney", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97904", "FIPS": "41025", "County": "Harney", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97014", "FIPS": "41027", "County": "Hood River", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97031", "FIPS": "41027", "County": "Hood River", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97041", "FIPS": "41027", "County": "Hood River", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97044", "FIPS": "41027", "County": "Hood River", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97501", "FIPS": "41029", "County": "Jackson", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97502", "FIPS": "41029", "County": "Jackson", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97503", "FIPS": "41029", "County": "Jackson", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97504", "FIPS": "41029", "County": "Jackson", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97520", "FIPS": "41029", "County": "Jackson", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97522", "FIPS": "41029", "County": "Jackson", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97524", "FIPS": "41029", "County": "Jackson", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97525", "FIPS": "41029", "County": "Jackson", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97530", "FIPS": "41029", "County": "Jackson", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97535", "FIPS": "41029", "County": "Jackson", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97536", "FIPS": "41029", "County": "Jackson", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97537", "FIPS": "41029", "County": "Jackson", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97539", "FIPS": "41029", "County": "Jackson", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97540", "FIPS": "41029", "County": "Jackson", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97541", "FIPS": "41029", "County": "Jackson", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97711", "FIPS": "41031", "County": "Jefferson", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97730", "FIPS": "41031", "County": "Jefferson", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97734", "FIPS": "41031", "County": "Jefferson", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97741", "FIPS": "41031", "County": "Jefferson", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97760", "FIPS": "41031", "County": "Jefferson", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97761", "FIPS": "41031", "County": "Jefferson", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97497", "FIPS": "41033", "County": "Josephine", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97523", "FIPS": "41033", "County": "Josephine", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97526", "FIPS": "41033", "County": "Josephine", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97527", "FIPS": "41033", "County": "Josephine", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97528", "FIPS": "41033", "County": "Josephine", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97531", "FIPS": "41033", "County": "Josephine", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97532", "FIPS": "41033", "County": "Josephine", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97533", "FIPS": "41033", "County": "Josephine", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97534", "FIPS": "41033", "County": "Josephine", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97538", "FIPS": "41033", "County": "Josephine", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97543", "FIPS": "41033", "County": "Josephine", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97544", "FIPS": "41033", "County": "Josephine", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97425", "FIPS": "41035", "County": "Klamath", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97601", "FIPS": "41035", "County": "Klamath", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97602", "FIPS": "41035", "County": "Klamath", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97603", "FIPS": "41035", "County": "Klamath", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97604", "FIPS": "41035", "County": "Klamath", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97621", "FIPS": "41035", "County": "Klamath", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97622", "FIPS": "41035", "County": "Klamath", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97623", "FIPS": "41035", "County": "Klamath", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97624", "FIPS": "41035", "County": "Klamath", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97625", "FIPS": "41035", "County": "Klamath", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97626", "FIPS": "41035", "County": "Klamath", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97627", "FIPS": "41035", "County": "Klamath", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97632", "FIPS": "41035", "County": "Klamath", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97633", "FIPS": "41035", "County": "Klamath", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97634", "FIPS": "41035", "County": "Klamath", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97639", "FIPS": "41035", "County": "Klamath", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97731", "FIPS": "41035", "County": "Klamath", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97733", "FIPS": "41035", "County": "Klamath", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97737", "FIPS": "41035", "County": "Klamath", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97620", "FIPS": "41037", "County": "Lake", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97630", "FIPS": "41037", "County": "Lake", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97635", "FIPS": "41037", "County": "Lake", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97636", "FIPS": "41037", "County": "Lake", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97637", "FIPS": "41037", "County": "Lake", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97638", "FIPS": "41037", "County": "Lake", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97640", "FIPS": "41037", "County": "Lake", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97641", "FIPS": "41037", "County": "Lake", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97735", "FIPS": "41037", "County": "Lake", "State": "OR", "DMA": {"Code": 813, "Name": "MEDFORD-KLAMATH FALLS"}},
  {"Postal_Code": "97401", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97402", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97403", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97404", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97405", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97408", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97409", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97412", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97413", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97419", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97424", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97426", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97427", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97430", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97431", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97434", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97437", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97438", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97439", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97440", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97448", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97451", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97452", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97453", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97454", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97455", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97461", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97463", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97472", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97477", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97478", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97480", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97482", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97487", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97488", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97489", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97490", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97492", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97493", "FIPS": "41039", "County": "Lane", "State": "OR", "DMA": {"Code": 801, "Name": "EUGENE"}},
  {"Postal_Code": "97341", "FIPS": "41041", "County": "Lincoln", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97343", "FIPS": "41041", "County": "Lincoln", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97357", "FIPS": "41041", "County": "Lincoln", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97364", "FIPS": "41041", "County": "Lincoln", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97365", "FIPS": "41041", "County": "Lincoln", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97366", "FIPS": "41041", "County": "Lincoln", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97367", "FIPS": "41041", "County": "Lincoln", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97368", "FIPS": "41041", "County": "Lincoln", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97369", "FIPS": "41041", "County": "Lincoln", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97372", "FIPS": "41041", "County": "Lincoln", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97376", "FIPS": "41041", "County": "Lincoln", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97380", "FIPS": "41041", "County": "Lincoln", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97388", "FIPS": "41041", "County": "Lincoln", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97390", "FIPS": "41041", "County": "Lincoln", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97391", "FIPS": "41041", "County": "Lincoln", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97394", "FIPS": "41041", "County": "Lincoln", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97498", "FIPS": "41041", "County": "Lincoln", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97321", "FIPS": "41043", "County": "Linn", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97322", "FIPS": "41043", "County": "Linn", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97327", "FIPS": "41043", "County": "Linn", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97329", "FIPS": "41043", "County": "Linn", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97335", "FIPS": "41043", "County": "Linn", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97336", "FIPS": "41043", "County": "Linn", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97345", "FIPS": "41043", "County": "Linn", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97348", "FIPS": "41043", "County": "Linn", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97355", "FIPS": "41043", "County": "Linn", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97358", "FIPS": "41043", "County": "Linn", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97360", "FIPS": "41043", "County": "Linn", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97374", "FIPS": "41043", "County": "Linn", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97377", "FIPS": "41043", "County": "Linn", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97386", "FIPS": "41043", "County": "Linn", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97389", "FIPS": "41043", "County": "Linn", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97446", "FIPS": "41043", "County": "Linn", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97901", "FIPS": "41045", "County": "Malheur", "State": "OR", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "97902", "FIPS": "41045", "County": "Malheur", "State": "OR", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "97903", "FIPS": "41045", "County": "Malheur", "State": "OR", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "97906", "FIPS": "41045", "County": "Malheur", "State": "OR", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "97908", "FIPS": "41045", "County": "Malheur", "State": "OR", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "97909", "FIPS": "41045", "County": "Malheur", "State": "OR", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "97910", "FIPS": "41045", "County": "Malheur", "State": "OR", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "97911", "FIPS": "41045", "County": "Malheur", "State": "OR", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "97913", "FIPS": "41045", "County": "Malheur", "State": "OR", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "97914", "FIPS": "41045", "County": "Malheur", "State": "OR", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "97917", "FIPS": "41045", "County": "Malheur", "State": "OR", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "97918", "FIPS": "41045", "County": "Malheur", "State": "OR", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "97920", "FIPS": "41045", "County": "Malheur", "State": "OR", "DMA": {"Code": 757, "Name": "BOISE"}},
  {"Postal_Code": "97002", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97020", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97026", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97032", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97071", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97137", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97301", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97302", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97303", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97305", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97306", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97307", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97308", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97309", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97310", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97311", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97312", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97313", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97314", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97317", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97325", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97342", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97346", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97350", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97352", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97362", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97373", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97375", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97381", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97383", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97384", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97385", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97392", "FIPS": "41047", "County": "Marion", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97818", "FIPS": "41049", "County": "Morrow", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97836", "FIPS": "41049", "County": "Morrow", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97839", "FIPS": "41049", "County": "Morrow", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97843", "FIPS": "41049", "County": "Morrow", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97844", "FIPS": "41049", "County": "Morrow", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97010", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97019", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97024", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97030", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97060", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97080", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97201", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97202", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97203", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97204", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97205", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97206", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97207", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97208", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97209", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97210", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97211", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97212", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97213", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97214", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97215", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97216", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97217", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97218", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97219", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97220", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97221", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97227", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97228", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97230", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97231", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97232", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97233", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97236", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97238", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97239", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97240", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97242", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97251", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97253", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97254", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97255", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97256", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97258", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97259", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97266", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97271", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97272", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97280", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97282", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97283", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97286", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97290", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97292", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97293", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97294", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97296", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97299", "FIPS": "41051", "County": "Multnomah", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97304", "FIPS": "41053", "County": "Polk", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97338", "FIPS": "41053", "County": "Polk", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97344", "FIPS": "41053", "County": "Polk", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97347", "FIPS": "41053", "County": "Polk", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97351", "FIPS": "41053", "County": "Polk", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97361", "FIPS": "41053", "County": "Polk", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97371", "FIPS": "41053", "County": "Polk", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97029", "FIPS": "41055", "County": "Sherman", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97033", "FIPS": "41055", "County": "Sherman", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97039", "FIPS": "41055", "County": "Sherman", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97050", "FIPS": "41055", "County": "Sherman", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97065", "FIPS": "41055", "County": "Sherman", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97107", "FIPS": "41057", "County": "Tillamook", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97108", "FIPS": "41057", "County": "Tillamook", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97112", "FIPS": "41057", "County": "Tillamook", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97118", "FIPS": "41057", "County": "Tillamook", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97122", "FIPS": "41057", "County": "Tillamook", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97130", "FIPS": "41057", "County": "Tillamook", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97131", "FIPS": "41057", "County": "Tillamook", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97134", "FIPS": "41057", "County": "Tillamook", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97135", "FIPS": "41057", "County": "Tillamook", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97136", "FIPS": "41057", "County": "Tillamook", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97141", "FIPS": "41057", "County": "Tillamook", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97143", "FIPS": "41057", "County": "Tillamook", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97147", "FIPS": "41057", "County": "Tillamook", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97149", "FIPS": "41057", "County": "Tillamook", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97801", "FIPS": "41059", "County": "Umatilla", "State": "OR", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "97810", "FIPS": "41059", "County": "Umatilla", "State": "OR", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "97813", "FIPS": "41059", "County": "Umatilla", "State": "OR", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "97826", "FIPS": "41059", "County": "Umatilla", "State": "OR", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "97835", "FIPS": "41059", "County": "Umatilla", "State": "OR", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "97838", "FIPS": "41059", "County": "Umatilla", "State": "OR", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "97859", "FIPS": "41059", "County": "Umatilla", "State": "OR", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "97862", "FIPS": "41059", "County": "Umatilla", "State": "OR", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "97868", "FIPS": "41059", "County": "Umatilla", "State": "OR", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "97875", "FIPS": "41059", "County": "Umatilla", "State": "OR", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "97880", "FIPS": "41059", "County": "Umatilla", "State": "OR", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "97882", "FIPS": "41059", "County": "Umatilla", "State": "OR", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "97886", "FIPS": "41059", "County": "Umatilla", "State": "OR", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "97824", "FIPS": "41061", "County": "Union", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97827", "FIPS": "41061", "County": "Union", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97841", "FIPS": "41061", "County": "Union", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97850", "FIPS": "41061", "County": "Union", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97867", "FIPS": "41061", "County": "Union", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97876", "FIPS": "41061", "County": "Union", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97883", "FIPS": "41061", "County": "Union", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97828", "FIPS": "41063", "County": "Wallowa", "State": "OR", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "97842", "FIPS": "41063", "County": "Wallowa", "State": "OR", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "97846", "FIPS": "41063", "County": "Wallowa", "State": "OR", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "97857", "FIPS": "41063", "County": "Wallowa", "State": "OR", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "97885", "FIPS": "41063", "County": "Wallowa", "State": "OR", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "97001", "FIPS": "41065", "County": "Wasco", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97021", "FIPS": "41065", "County": "Wasco", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97037", "FIPS": "41065", "County": "Wasco", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97040", "FIPS": "41065", "County": "Wasco", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97057", "FIPS": "41065", "County": "Wasco", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97058", "FIPS": "41065", "County": "Wasco", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97063", "FIPS": "41065", "County": "Wasco", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97005", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97006", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97007", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97008", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97062", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97075", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97076", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97077", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97078", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97106", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97109", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97113", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97116", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97117", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97119", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97123", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97124", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97125", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97133", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97140", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97144", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97223", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97224", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97225", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97229", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97281", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97291", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97298", "FIPS": "41067", "County": "Washington", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97750", "FIPS": "41069", "County": "Wheeler", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97830", "FIPS": "41069", "County": "Wheeler", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97874", "FIPS": "41069", "County": "Wheeler", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97101", "FIPS": "41071", "County": "Yamhill", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97111", "FIPS": "41071", "County": "Yamhill", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97114", "FIPS": "41071", "County": "Yamhill", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97115", "FIPS": "41071", "County": "Yamhill", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97127", "FIPS": "41071", "County": "Yamhill", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97128", "FIPS": "41071", "County": "Yamhill", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97132", "FIPS": "41071", "County": "Yamhill", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97148", "FIPS": "41071", "County": "Yamhill", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97378", "FIPS": "41071", "County": "Yamhill", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "97396", "FIPS": "41071", "County": "Yamhill", "State": "OR", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "17301", "FIPS": "42001", "County": "Adams", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17303", "FIPS": "42001", "County": "Adams", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17304", "FIPS": "42001", "County": "Adams", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17306", "FIPS": "42001", "County": "Adams", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17307", "FIPS": "42001", "County": "Adams", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17310", "FIPS": "42001", "County": "Adams", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17316", "FIPS": "42001", "County": "Adams", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17320", "FIPS": "42001", "County": "Adams", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17324", "FIPS": "42001", "County": "Adams", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17325", "FIPS": "42001", "County": "Adams", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17326", "FIPS": "42001", "County": "Adams", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17337", "FIPS": "42001", "County": "Adams", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17340", "FIPS": "42001", "County": "Adams", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17343", "FIPS": "42001", "County": "Adams", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17344", "FIPS": "42001", "County": "Adams", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17350", "FIPS": "42001", "County": "Adams", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17353", "FIPS": "42001", "County": "Adams", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17372", "FIPS": "42001", "County": "Adams", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17375", "FIPS": "42001", "County": "Adams", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "15006", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15007", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15014", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15015", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15017", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15018", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15020", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15024", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15025", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15028", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15030", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15031", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15032", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15034", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15035", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15037", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15044", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15045", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15046", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15047", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15049", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15051", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15056", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15064", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15065", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15071", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15075", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15076", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15082", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15084", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15086", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15088", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15090", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15091", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15095", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15096", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15101", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15102", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15104", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15106", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15108", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15110", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15112", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15116", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15120", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15122", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15123", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15126", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15127", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15129", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15131", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15132", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15133", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15134", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15135", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15136", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15137", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15139", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15140", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15142", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15143", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15144", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15145", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15146", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15147", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15148", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15201", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15202", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15203", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15204", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15205", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15206", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15207", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15208", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15209", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15210", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15211", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15212", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15213", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15214", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15215", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15216", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15217", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15218", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15219", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15220", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15221", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15222", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15223", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15224", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15225", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15226", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15227", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15228", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15229", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15230", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15231", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15232", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15233", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15234", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15235", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15236", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15237", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15238", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15239", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15240", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15241", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15242", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15243", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15244", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15250", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15251", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15252", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15253", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15254", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15255", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15257", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15258", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15259", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15260", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15261", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15262", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15263", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15264", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15265", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15267", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15268", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15270", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15272", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15274", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15275", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15276", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15277", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15278", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15279", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15281", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15282", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15283", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15285", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15286", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15289", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15290", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15295", "FIPS": "42003", "County": "Allegheny", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15656", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15673", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15682", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15686", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15736", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16049", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16201", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16210", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16212", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16215", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16218", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16222", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16223", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16226", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16228", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16229", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16236", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16238", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16244", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16245", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16249", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16250", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16253", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16259", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16261", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16262", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16263", "FIPS": "42005", "County": "Armstrong", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15001", "FIPS": "42007", "County": "Beaver", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15003", "FIPS": "42007", "County": "Beaver", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15005", "FIPS": "42007", "County": "Beaver", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15009", "FIPS": "42007", "County": "Beaver", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15010", "FIPS": "42007", "County": "Beaver", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15026", "FIPS": "42007", "County": "Beaver", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15027", "FIPS": "42007", "County": "Beaver", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15042", "FIPS": "42007", "County": "Beaver", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15043", "FIPS": "42007", "County": "Beaver", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15050", "FIPS": "42007", "County": "Beaver", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15052", "FIPS": "42007", "County": "Beaver", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15059", "FIPS": "42007", "County": "Beaver", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15061", "FIPS": "42007", "County": "Beaver", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15066", "FIPS": "42007", "County": "Beaver", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15074", "FIPS": "42007", "County": "Beaver", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15077", "FIPS": "42007", "County": "Beaver", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15081", "FIPS": "42007", "County": "Beaver", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16115", "FIPS": "42007", "County": "Beaver", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16123", "FIPS": "42007", "County": "Beaver", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16136", "FIPS": "42007", "County": "Beaver", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16141", "FIPS": "42007", "County": "Beaver", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15521", "FIPS": "42009", "County": "Bedford", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15522", "FIPS": "42009", "County": "Bedford", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15533", "FIPS": "42009", "County": "Bedford", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15534", "FIPS": "42009", "County": "Bedford", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15535", "FIPS": "42009", "County": "Bedford", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15537", "FIPS": "42009", "County": "Bedford", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15539", "FIPS": "42009", "County": "Bedford", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15545", "FIPS": "42009", "County": "Bedford", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15550", "FIPS": "42009", "County": "Bedford", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15554", "FIPS": "42009", "County": "Bedford", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15559", "FIPS": "42009", "County": "Bedford", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16633", "FIPS": "42009", "County": "Bedford", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16650", "FIPS": "42009", "County": "Bedford", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16655", "FIPS": "42009", "County": "Bedford", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16659", "FIPS": "42009", "County": "Bedford", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16664", "FIPS": "42009", "County": "Bedford", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16667", "FIPS": "42009", "County": "Bedford", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16670", "FIPS": "42009", "County": "Bedford", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16672", "FIPS": "42009", "County": "Bedford", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16678", "FIPS": "42009", "County": "Bedford", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16679", "FIPS": "42009", "County": "Bedford", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16694", "FIPS": "42009", "County": "Bedford", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16695", "FIPS": "42009", "County": "Bedford", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "17211", "FIPS": "42009", "County": "Bedford", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "18056", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19503", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19504", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19505", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19506", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19507", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19508", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19510", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19511", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19512", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19516", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19518", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19519", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19522", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19523", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19526", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19529", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19530", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19533", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19534", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19535", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19536", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19538", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19539", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19540", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19541", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19542", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19543", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19544", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19545", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19547", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19548", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19550", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19551", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19554", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19555", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19559", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19560", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19562", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19564", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19565", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19567", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19601", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19602", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19603", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19604", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19605", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19606", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19607", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19608", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19609", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19610", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19611", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19612", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19640", "FIPS": "42011", "County": "Berks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "16601", "FIPS": "42013", "County": "Blair", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16602", "FIPS": "42013", "County": "Blair", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16603", "FIPS": "42013", "County": "Blair", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16617", "FIPS": "42013", "County": "Blair", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16625", "FIPS": "42013", "County": "Blair", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16631", "FIPS": "42013", "County": "Blair", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16635", "FIPS": "42013", "County": "Blair", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16637", "FIPS": "42013", "County": "Blair", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16648", "FIPS": "42013", "County": "Blair", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16662", "FIPS": "42013", "County": "Blair", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16665", "FIPS": "42013", "County": "Blair", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16673", "FIPS": "42013", "County": "Blair", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16682", "FIPS": "42013", "County": "Blair", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16684", "FIPS": "42013", "County": "Blair", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16686", "FIPS": "42013", "County": "Blair", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16693", "FIPS": "42013", "County": "Blair", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16910", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "16914", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "16925", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "16926", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "16945", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "16947", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17724", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17735", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18810", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18814", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18815", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18817", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18829", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18831", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18832", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18833", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18837", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18840", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18845", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18846", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18848", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18850", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18851", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18853", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18854", "FIPS": "42015", "County": "Bradford", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18039", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18077", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18081", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18901", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18902", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18910", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18911", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18912", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18913", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18914", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18916", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18917", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18920", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18921", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18922", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18923", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18925", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18926", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18927", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18928", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18929", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18930", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18931", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18932", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18933", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18934", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18935", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18938", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18940", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18942", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18943", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18944", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18946", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18947", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18949", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18950", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18951", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18953", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18954", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18955", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18956", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18960", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18962", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18963", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18966", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18968", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18970", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18972", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18974", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18976", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18977", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18980", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18981", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18991", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19007", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19020", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19021", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19030", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19047", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19048", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19049", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19053", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19054", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19055", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19056", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19057", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19058", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19067", "FIPS": "42017", "County": "Bucks", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "16001", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16002", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16003", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16016", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16017", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16018", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16020", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16021", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16022", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16023", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16024", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16025", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16027", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16029", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16030", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16033", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16034", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16035", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16037", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16038", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16039", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16040", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16041", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16045", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16046", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16048", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16050", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16051", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16052", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16053", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16055", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16056", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16057", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16059", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16061", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16063", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16066", "FIPS": "42019", "County": "Butler", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15714", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15722", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15737", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15738", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15760", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15762", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15773", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15775", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15901", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15902", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15904", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15905", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15906", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15907", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15909", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15915", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15921", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15922", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15925", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15927", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15930", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15931", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15934", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15938", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15940", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15942", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15943", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15945", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15946", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15948", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15951", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15952", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15955", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15956", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15958", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15960", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15961", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15962", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16613", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16619", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16624", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16629", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16630", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16636", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16639", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16640", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16641", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16644", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16646", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16668", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16675", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16699", "FIPS": "42021", "County": "Cambria", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15832", "FIPS": "42023", "County": "Cameron", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15834", "FIPS": "42023", "County": "Cameron", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15861", "FIPS": "42023", "County": "Cameron", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "18012", "FIPS": "42025", "County": "Carbon", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18030", "FIPS": "42025", "County": "Carbon", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18071", "FIPS": "42025", "County": "Carbon", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18210", "FIPS": "42025", "County": "Carbon", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18212", "FIPS": "42025", "County": "Carbon", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18216", "FIPS": "42025", "County": "Carbon", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18229", "FIPS": "42025", "County": "Carbon", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18230", "FIPS": "42025", "County": "Carbon", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18232", "FIPS": "42025", "County": "Carbon", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18235", "FIPS": "42025", "County": "Carbon", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18240", "FIPS": "42025", "County": "Carbon", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18244", "FIPS": "42025", "County": "Carbon", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18250", "FIPS": "42025", "County": "Carbon", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18254", "FIPS": "42025", "County": "Carbon", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18255", "FIPS": "42025", "County": "Carbon", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18624", "FIPS": "42025", "County": "Carbon", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "16677", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16801", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16802", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16803", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16804", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16805", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16820", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16823", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16826", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16827", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16828", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16829", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16832", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16835", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16841", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16844", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16851", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16852", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16853", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16854", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16856", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16859", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16864", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16865", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16866", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16868", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16870", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16872", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16874", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16875", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16882", "FIPS": "42027", "County": "Centre", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "19301", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19310", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19311", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19312", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19316", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19318", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19320", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19330", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19333", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19335", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19341", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19343", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19344", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19345", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19346", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19347", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19348", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19350", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19351", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19352", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19353", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19354", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19355", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19357", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19358", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19360", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19362", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19363", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19365", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19366", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19367", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19369", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19371", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19372", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19374", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19375", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19376", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19380", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19381", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19382", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19383", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19390", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19395", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19397", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19398", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19399", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19421", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19425", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19432", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19442", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19457", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19460", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19465", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19470", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19475", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19480", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19481", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19482", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19487", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19488", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19489", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19493", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19494", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19495", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19496", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19520", "FIPS": "42029", "County": "Chester", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "16028", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16036", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16054", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16058", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16213", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16214", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16220", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16221", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16224", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16225", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16230", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16232", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16233", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16234", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16235", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16240", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16242", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16248", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16254", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16255", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16257", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16258", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16260", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16326", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16331", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16332", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16334", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16361", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16375", "FIPS": "42031", "County": "Clarion", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15721", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15753", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15757", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15801", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15848", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15849", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15856", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15866", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16616", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16620", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16627", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16645", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16651", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16656", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16661", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16663", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16666", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16671", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16680", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16681", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16692", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16698", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16821", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16825", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16830", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16833", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16834", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16836", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16837", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16838", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16839", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16840", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16843", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16845", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16847", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16849", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16850", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16855", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16858", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16860", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16861", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16863", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16871", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16873", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16876", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16878", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16879", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16881", "FIPS": "42033", "County": "Clearfield", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16822", "FIPS": "42035", "County": "Clinton", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "16848", "FIPS": "42035", "County": "Clinton", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17721", "FIPS": "42035", "County": "Clinton", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17726", "FIPS": "42035", "County": "Clinton", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17738", "FIPS": "42035", "County": "Clinton", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17745", "FIPS": "42035", "County": "Clinton", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17747", "FIPS": "42035", "County": "Clinton", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17748", "FIPS": "42035", "County": "Clinton", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17750", "FIPS": "42035", "County": "Clinton", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17751", "FIPS": "42035", "County": "Clinton", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17760", "FIPS": "42035", "County": "Clinton", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17764", "FIPS": "42035", "County": "Clinton", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17767", "FIPS": "42035", "County": "Clinton", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17773", "FIPS": "42035", "County": "Clinton", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17778", "FIPS": "42035", "County": "Clinton", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17779", "FIPS": "42035", "County": "Clinton", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17814", "FIPS": "42037", "County": "Columbia", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17815", "FIPS": "42037", "County": "Columbia", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17820", "FIPS": "42037", "County": "Columbia", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17839", "FIPS": "42037", "County": "Columbia", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17846", "FIPS": "42037", "County": "Columbia", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17858", "FIPS": "42037", "County": "Columbia", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17859", "FIPS": "42037", "County": "Columbia", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17878", "FIPS": "42037", "County": "Columbia", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17888", "FIPS": "42037", "County": "Columbia", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17920", "FIPS": "42037", "County": "Columbia", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18603", "FIPS": "42037", "County": "Columbia", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18631", "FIPS": "42037", "County": "Columbia", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "16110", "FIPS": "42039", "County": "Crawford", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16111", "FIPS": "42039", "County": "Crawford", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16131", "FIPS": "42039", "County": "Crawford", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16314", "FIPS": "42039", "County": "Crawford", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16316", "FIPS": "42039", "County": "Crawford", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16327", "FIPS": "42039", "County": "Crawford", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16328", "FIPS": "42039", "County": "Crawford", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16335", "FIPS": "42039", "County": "Crawford", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16354", "FIPS": "42039", "County": "Crawford", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16360", "FIPS": "42039", "County": "Crawford", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16388", "FIPS": "42039", "County": "Crawford", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16403", "FIPS": "42039", "County": "Crawford", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16404", "FIPS": "42039", "County": "Crawford", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16406", "FIPS": "42039", "County": "Crawford", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16422", "FIPS": "42039", "County": "Crawford", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16424", "FIPS": "42039", "County": "Crawford", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16432", "FIPS": "42039", "County": "Crawford", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16433", "FIPS": "42039", "County": "Crawford", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16434", "FIPS": "42039", "County": "Crawford", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16435", "FIPS": "42039", "County": "Crawford", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16440", "FIPS": "42039", "County": "Crawford", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "17001", "FIPS": "42041", "County": "Cumberland", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17007", "FIPS": "42041", "County": "Cumberland", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17008", "FIPS": "42041", "County": "Cumberland", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17011", "FIPS": "42041", "County": "Cumberland", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17012", "FIPS": "42041", "County": "Cumberland", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17013", "FIPS": "42041", "County": "Cumberland", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17015", "FIPS": "42041", "County": "Cumberland", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17025", "FIPS": "42041", "County": "Cumberland", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17027", "FIPS": "42041", "County": "Cumberland", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17043", "FIPS": "42041", "County": "Cumberland", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17050", "FIPS": "42041", "County": "Cumberland", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17055", "FIPS": "42041", "County": "Cumberland", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17065", "FIPS": "42041", "County": "Cumberland", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17070", "FIPS": "42041", "County": "Cumberland", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17072", "FIPS": "42041", "County": "Cumberland", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17081", "FIPS": "42041", "County": "Cumberland", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17089", "FIPS": "42041", "County": "Cumberland", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17093", "FIPS": "42041", "County": "Cumberland", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17240", "FIPS": "42041", "County": "Cumberland", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17241", "FIPS": "42041", "County": "Cumberland", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17257", "FIPS": "42041", "County": "Cumberland", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17266", "FIPS": "42041", "County": "Cumberland", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17005", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17018", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17023", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17028", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17030", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17032", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17033", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17034", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17036", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17048", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17057", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17061", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17080", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17097", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17098", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17101", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17102", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17103", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17104", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17105", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17106", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17107", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17108", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17109", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17110", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17111", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17112", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17113", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17120", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17121", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17122", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17123", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17124", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17125", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17126", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17127", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17128", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17129", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17130", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17140", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17177", "FIPS": "42043", "County": "Dauphin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "19008", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19010", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19013", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19014", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19015", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19016", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19017", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19018", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19022", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19023", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19026", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19028", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19029", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19032", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19033", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19036", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19037", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19039", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19043", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19050", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19052", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19061", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19063", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19064", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19065", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19070", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19073", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19074", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19076", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19078", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19079", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19080", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19081", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19082", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19083", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19085", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19086", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19087", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19088", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19089", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19091", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19094", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19098", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19113", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19317", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19319", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19331", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19339", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19340", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19342", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19373", "FIPS": "42045", "County": "Delaware", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "15821", "FIPS": "42047", "County": "Elk", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15822", "FIPS": "42047", "County": "Elk", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15823", "FIPS": "42047", "County": "Elk", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15827", "FIPS": "42047", "County": "Elk", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15831", "FIPS": "42047", "County": "Elk", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15841", "FIPS": "42047", "County": "Elk", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15845", "FIPS": "42047", "County": "Elk", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15846", "FIPS": "42047", "County": "Elk", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15853", "FIPS": "42047", "County": "Elk", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15857", "FIPS": "42047", "County": "Elk", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15868", "FIPS": "42047", "County": "Elk", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15870", "FIPS": "42047", "County": "Elk", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16728", "FIPS": "42047", "County": "Elk", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16734", "FIPS": "42047", "County": "Elk", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16401", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16407", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16410", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16411", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16412", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16413", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16415", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16417", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16421", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16423", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16426", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16427", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16428", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16430", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16438", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16441", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16442", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16443", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16444", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16475", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16501", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16502", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16503", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16504", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16505", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16506", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16507", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16508", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16509", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16510", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16511", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16512", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16514", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16515", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16522", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16530", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16531", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16532", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16533", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16534", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16538", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16541", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16544", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16546", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16550", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16553", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16554", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16563", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16565", "FIPS": "42049", "County": "Erie", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "15012", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15401", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15410", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15413", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15415", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15416", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15417", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15420", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15421", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15422", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15425", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15428", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15430", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15431", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15433", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15435", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15436", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15437", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15438", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15439", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15440", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15442", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15443", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15444", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15445", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15446", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15447", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15449", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15450", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15451", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15454", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15455", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15456", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15458", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15459", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15460", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15461", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15462", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15463", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15464", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15465", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15466", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15467", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15468", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15469", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15470", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15472", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15473", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15474", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15475", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15476", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15478", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15480", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15482", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15484", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15486", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15488", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15489", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15490", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15492", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15631", "FIPS": "42051", "County": "Fayette", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15828", "FIPS": "42053", "County": "Forest", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16217", "FIPS": "42053", "County": "Forest", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16239", "FIPS": "42053", "County": "Forest", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16321", "FIPS": "42053", "County": "Forest", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16322", "FIPS": "42053", "County": "Forest", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16353", "FIPS": "42053", "County": "Forest", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16370", "FIPS": "42053", "County": "Forest", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "17201", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17210", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17214", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17217", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17219", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17220", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17221", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17222", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17224", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17225", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17231", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17232", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17235", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17236", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17237", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17244", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17246", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17247", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17250", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17251", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17252", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17254", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17256", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17261", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17262", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17263", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17265", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17268", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17270", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17271", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17272", "FIPS": "42055", "County": "Franklin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "15536", "FIPS": "42057", "County": "Fulton", "State": "PA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "16689", "FIPS": "42057", "County": "Fulton", "State": "PA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "16691", "FIPS": "42057", "County": "Fulton", "State": "PA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "17212", "FIPS": "42057", "County": "Fulton", "State": "PA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "17215", "FIPS": "42057", "County": "Fulton", "State": "PA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "17223", "FIPS": "42057", "County": "Fulton", "State": "PA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "17228", "FIPS": "42057", "County": "Fulton", "State": "PA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "17229", "FIPS": "42057", "County": "Fulton", "State": "PA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "17233", "FIPS": "42057", "County": "Fulton", "State": "PA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "17238", "FIPS": "42057", "County": "Fulton", "State": "PA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "17267", "FIPS": "42057", "County": "Fulton", "State": "PA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "15310", "FIPS": "42059", "County": "Greene", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15315", "FIPS": "42059", "County": "Greene", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15316", "FIPS": "42059", "County": "Greene", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15320", "FIPS": "42059", "County": "Greene", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15322", "FIPS": "42059", "County": "Greene", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15325", "FIPS": "42059", "County": "Greene", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15327", "FIPS": "42059", "County": "Greene", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15334", "FIPS": "42059", "County": "Greene", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15337", "FIPS": "42059", "County": "Greene", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15338", "FIPS": "42059", "County": "Greene", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15341", "FIPS": "42059", "County": "Greene", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15344", "FIPS": "42059", "County": "Greene", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15346", "FIPS": "42059", "County": "Greene", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15349", "FIPS": "42059", "County": "Greene", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15351", "FIPS": "42059", "County": "Greene", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15352", "FIPS": "42059", "County": "Greene", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15353", "FIPS": "42059", "County": "Greene", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15357", "FIPS": "42059", "County": "Greene", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15359", "FIPS": "42059", "County": "Greene", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15362", "FIPS": "42059", "County": "Greene", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15364", "FIPS": "42059", "County": "Greene", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15370", "FIPS": "42059", "County": "Greene", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15380", "FIPS": "42059", "County": "Greene", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16611", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16621", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16622", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16623", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16634", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16638", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16647", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16652", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16654", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16657", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16660", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16669", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16674", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16683", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16685", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "16877", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "17052", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "17060", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "17066", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "17213", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "17239", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "17243", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "17249", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "17253", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "17255", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "17260", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "17264", "FIPS": "42061", "County": "Huntingdon", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15681", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15701", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15705", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15710", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15712", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15713", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15716", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15717", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15720", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15723", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15724", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15725", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15727", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15728", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15729", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15731", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15732", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15734", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15739", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15741", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15742", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15745", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15746", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15747", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15748", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15750", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15752", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15754", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15756", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15758", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15759", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15761", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15763", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15765", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15771", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15772", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15774", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15777", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15783", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15920", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15929", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15949", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15957", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16211", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16246", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16256", "FIPS": "42063", "County": "Indiana", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15711", "FIPS": "42065", "County": "Jefferson", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15715", "FIPS": "42065", "County": "Jefferson", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15730", "FIPS": "42065", "County": "Jefferson", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15733", "FIPS": "42065", "County": "Jefferson", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15740", "FIPS": "42065", "County": "Jefferson", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15744", "FIPS": "42065", "County": "Jefferson", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15764", "FIPS": "42065", "County": "Jefferson", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15767", "FIPS": "42065", "County": "Jefferson", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15770", "FIPS": "42065", "County": "Jefferson", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15776", "FIPS": "42065", "County": "Jefferson", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15778", "FIPS": "42065", "County": "Jefferson", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15780", "FIPS": "42065", "County": "Jefferson", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15781", "FIPS": "42065", "County": "Jefferson", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15784", "FIPS": "42065", "County": "Jefferson", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15824", "FIPS": "42065", "County": "Jefferson", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15825", "FIPS": "42065", "County": "Jefferson", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15829", "FIPS": "42065", "County": "Jefferson", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15840", "FIPS": "42065", "County": "Jefferson", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15847", "FIPS": "42065", "County": "Jefferson", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15851", "FIPS": "42065", "County": "Jefferson", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15860", "FIPS": "42065", "County": "Jefferson", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15863", "FIPS": "42065", "County": "Jefferson", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15864", "FIPS": "42065", "County": "Jefferson", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15865", "FIPS": "42065", "County": "Jefferson", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "17014", "FIPS": "42067", "County": "Juniata", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17021", "FIPS": "42067", "County": "Juniata", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17035", "FIPS": "42067", "County": "Juniata", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17049", "FIPS": "42067", "County": "Juniata", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17056", "FIPS": "42067", "County": "Juniata", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17058", "FIPS": "42067", "County": "Juniata", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17059", "FIPS": "42067", "County": "Juniata", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17076", "FIPS": "42067", "County": "Juniata", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17082", "FIPS": "42067", "County": "Juniata", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17086", "FIPS": "42067", "County": "Juniata", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17094", "FIPS": "42067", "County": "Juniata", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "18403", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18407", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18410", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18411", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18414", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18416", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18420", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18433", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18434", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18440", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18444", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18447", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18448", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18452", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18471", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18501", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18502", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18503", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18504", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18505", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18507", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18508", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18509", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18510", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18512", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18514", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18515", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18517", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18518", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18519", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18522", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18540", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18577", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18653", "FIPS": "42069", "County": "Lackawanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17022", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17501", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17502", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17503", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17504", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17505", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17506", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17507", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17508", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17509", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17512", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17516", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17517", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17518", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17519", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17520", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17521", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17522", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17527", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17528", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17529", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17532", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17533", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17534", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17535", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17536", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17537", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17538", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17540", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17543", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17545", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17547", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17549", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17550", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17551", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17552", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17554", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17555", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17557", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17560", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17562", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17563", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17564", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17565", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17566", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17567", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17568", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17569", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17570", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17572", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17573", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17575", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17576", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17578", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17579", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17580", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17581", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17582", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17583", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17584", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17585", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17601", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17602", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17603", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17604", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17605", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17606", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17607", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17608", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17611", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17699", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "19501", "FIPS": "42071", "County": "Lancaster", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "16101", "FIPS": "42073", "County": "Lawrence", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16102", "FIPS": "42073", "County": "Lawrence", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16103", "FIPS": "42073", "County": "Lawrence", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16105", "FIPS": "42073", "County": "Lawrence", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16107", "FIPS": "42073", "County": "Lawrence", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16108", "FIPS": "42073", "County": "Lawrence", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16112", "FIPS": "42073", "County": "Lawrence", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16116", "FIPS": "42073", "County": "Lawrence", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16117", "FIPS": "42073", "County": "Lawrence", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16120", "FIPS": "42073", "County": "Lawrence", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16132", "FIPS": "42073", "County": "Lawrence", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16140", "FIPS": "42073", "County": "Lawrence", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16142", "FIPS": "42073", "County": "Lawrence", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16143", "FIPS": "42073", "County": "Lawrence", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16155", "FIPS": "42073", "County": "Lawrence", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16156", "FIPS": "42073", "County": "Lawrence", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16157", "FIPS": "42073", "County": "Lawrence", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16160", "FIPS": "42073", "County": "Lawrence", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16172", "FIPS": "42073", "County": "Lawrence", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "17003", "FIPS": "42075", "County": "Lebanon", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17010", "FIPS": "42075", "County": "Lebanon", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17016", "FIPS": "42075", "County": "Lebanon", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17026", "FIPS": "42075", "County": "Lebanon", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17038", "FIPS": "42075", "County": "Lebanon", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17039", "FIPS": "42075", "County": "Lebanon", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17041", "FIPS": "42075", "County": "Lebanon", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17042", "FIPS": "42075", "County": "Lebanon", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17046", "FIPS": "42075", "County": "Lebanon", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17064", "FIPS": "42075", "County": "Lebanon", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17067", "FIPS": "42075", "County": "Lebanon", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17073", "FIPS": "42075", "County": "Lebanon", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17077", "FIPS": "42075", "County": "Lebanon", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17078", "FIPS": "42075", "County": "Lebanon", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17083", "FIPS": "42075", "County": "Lebanon", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17085", "FIPS": "42075", "County": "Lebanon", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17087", "FIPS": "42075", "County": "Lebanon", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17088", "FIPS": "42075", "County": "Lebanon", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17091", "FIPS": "42075", "County": "Lebanon", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "18011", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18018", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18025", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18031", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18032", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18034", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18036", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18037", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18046", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18049", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18051", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18052", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18053", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18059", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18060", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18062", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18065", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18066", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18068", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18069", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18078", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18079", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18080", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18087", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18092", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18098", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18099", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18101", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18102", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18103", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18104", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18105", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18106", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18109", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18175", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18195", "FIPS": "42077", "County": "Lehigh", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18201", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18202", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18219", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18221", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18222", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18223", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18224", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18225", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18234", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18239", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18246", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18247", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18249", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18251", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18256", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18601", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18602", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18611", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18612", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18617", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18618", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18621", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18622", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18627", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18634", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18635", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18640", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18641", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18642", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18643", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18644", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18651", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18654", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18655", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18656", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18660", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18661", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18690", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18701", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18702", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18703", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18704", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18705", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18706", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18707", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18708", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18709", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18710", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18711", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18762", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18764", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18765", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18766", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18767", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18769", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18773", "FIPS": "42079", "County": "Luzerne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17701", "FIPS": "42081", "County": "Lycoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17702", "FIPS": "42081", "County": "Lycoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17703", "FIPS": "42081", "County": "Lycoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17705", "FIPS": "42081", "County": "Lycoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17720", "FIPS": "42081", "County": "Lycoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17723", "FIPS": "42081", "County": "Lycoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17727", "FIPS": "42081", "County": "Lycoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17728", "FIPS": "42081", "County": "Lycoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17737", "FIPS": "42081", "County": "Lycoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17739", "FIPS": "42081", "County": "Lycoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17740", "FIPS": "42081", "County": "Lycoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17742", "FIPS": "42081", "County": "Lycoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17744", "FIPS": "42081", "County": "Lycoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17752", "FIPS": "42081", "County": "Lycoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17754", "FIPS": "42081", "County": "Lycoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17756", "FIPS": "42081", "County": "Lycoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17762", "FIPS": "42081", "County": "Lycoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17763", "FIPS": "42081", "County": "Lycoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17769", "FIPS": "42081", "County": "Lycoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17771", "FIPS": "42081", "County": "Lycoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17774", "FIPS": "42081", "County": "Lycoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17776", "FIPS": "42081", "County": "Lycoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "16333", "FIPS": "42083", "County": "McKean", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16701", "FIPS": "42083", "County": "McKean", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16724", "FIPS": "42083", "County": "McKean", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16725", "FIPS": "42083", "County": "McKean", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16726", "FIPS": "42083", "County": "McKean", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16727", "FIPS": "42083", "County": "McKean", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16729", "FIPS": "42083", "County": "McKean", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16730", "FIPS": "42083", "County": "McKean", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16731", "FIPS": "42083", "County": "McKean", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16732", "FIPS": "42083", "County": "McKean", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16733", "FIPS": "42083", "County": "McKean", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16735", "FIPS": "42083", "County": "McKean", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16738", "FIPS": "42083", "County": "McKean", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16740", "FIPS": "42083", "County": "McKean", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16743", "FIPS": "42083", "County": "McKean", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16744", "FIPS": "42083", "County": "McKean", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16745", "FIPS": "42083", "County": "McKean", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16749", "FIPS": "42083", "County": "McKean", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16750", "FIPS": "42083", "County": "McKean", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16113", "FIPS": "42085", "County": "Mercer", "State": "PA", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "16114", "FIPS": "42085", "County": "Mercer", "State": "PA", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "16121", "FIPS": "42085", "County": "Mercer", "State": "PA", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "16124", "FIPS": "42085", "County": "Mercer", "State": "PA", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "16125", "FIPS": "42085", "County": "Mercer", "State": "PA", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "16127", "FIPS": "42085", "County": "Mercer", "State": "PA", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "16130", "FIPS": "42085", "County": "Mercer", "State": "PA", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "16133", "FIPS": "42085", "County": "Mercer", "State": "PA", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "16134", "FIPS": "42085", "County": "Mercer", "State": "PA", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "16137", "FIPS": "42085", "County": "Mercer", "State": "PA", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "16145", "FIPS": "42085", "County": "Mercer", "State": "PA", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "16146", "FIPS": "42085", "County": "Mercer", "State": "PA", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "16148", "FIPS": "42085", "County": "Mercer", "State": "PA", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "16150", "FIPS": "42085", "County": "Mercer", "State": "PA", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "16151", "FIPS": "42085", "County": "Mercer", "State": "PA", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "16153", "FIPS": "42085", "County": "Mercer", "State": "PA", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "16154", "FIPS": "42085", "County": "Mercer", "State": "PA", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "16159", "FIPS": "42085", "County": "Mercer", "State": "PA", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "16161", "FIPS": "42085", "County": "Mercer", "State": "PA", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "16311", "FIPS": "42085", "County": "Mercer", "State": "PA", "DMA": {"Code": 536, "Name": "YOUNGSTOWN"}},
  {"Postal_Code": "17002", "FIPS": "42087", "County": "Mifflin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17004", "FIPS": "42087", "County": "Mifflin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17009", "FIPS": "42087", "County": "Mifflin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17029", "FIPS": "42087", "County": "Mifflin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17044", "FIPS": "42087", "County": "Mifflin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17051", "FIPS": "42087", "County": "Mifflin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17054", "FIPS": "42087", "County": "Mifflin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17063", "FIPS": "42087", "County": "Mifflin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17075", "FIPS": "42087", "County": "Mifflin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17084", "FIPS": "42087", "County": "Mifflin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17099", "FIPS": "42087", "County": "Mifflin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17841", "FIPS": "42087", "County": "Mifflin", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "18058", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18301", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18302", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18320", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18321", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18322", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18323", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18325", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18326", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18327", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18330", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18331", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18332", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18333", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18334", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18335", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18341", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18342", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18344", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18346", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18347", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18348", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18349", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18350", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18352", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18353", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18354", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18355", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18356", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18357", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18360", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18370", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18372", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18466", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18610", "FIPS": "42089", "County": "Monroe", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18041", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18054", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18070", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18073", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18074", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18076", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18084", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18915", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18918", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18924", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18936", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18957", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18958", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18964", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18969", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18971", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18979", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19001", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19002", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19003", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19004", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19006", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19009", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19012", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19025", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19027", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19031", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19034", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19035", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19038", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19040", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19041", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19044", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19046", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19066", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19072", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19075", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19090", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19095", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19096", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19401", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19403", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19404", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19405", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19406", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19407", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19408", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19409", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19415", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19420", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19422", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19423", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19424", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19426", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19428", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19429", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19430", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19435", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19436", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19437", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19438", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19440", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19441", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19443", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19444", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19446", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19450", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19451", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19453", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19454", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19455", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19456", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19462", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19464", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19468", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19472", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19473", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19474", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19477", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19478", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19483", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19484", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19485", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19486", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19490", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19492", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19525", "FIPS": "42091", "County": "Montgomery", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "17821", "FIPS": "42093", "County": "Montour", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17822", "FIPS": "42093", "County": "Montour", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17884", "FIPS": "42093", "County": "Montour", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18001", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18002", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18003", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18010", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18013", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18014", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18015", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18016", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18017", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18020", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18035", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18038", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18040", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18042", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18043", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18044", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18045", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18050", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18055", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18063", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18064", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18067", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18072", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18083", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18085", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18086", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18088", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18091", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18343", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18351", "FIPS": "42095", "County": "Northampton", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "17017", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17730", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17749", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17772", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17777", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17801", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17823", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17824", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17830", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17832", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17834", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17836", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17840", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17847", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17850", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17851", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17857", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17860", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17865", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17866", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17867", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17868", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17872", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17877", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17881", "FIPS": "42097", "County": "Northumberland", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17006", "FIPS": "42099", "County": "Perry", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17020", "FIPS": "42099", "County": "Perry", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17024", "FIPS": "42099", "County": "Perry", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17037", "FIPS": "42099", "County": "Perry", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17040", "FIPS": "42099", "County": "Perry", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17045", "FIPS": "42099", "County": "Perry", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17047", "FIPS": "42099", "County": "Perry", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17053", "FIPS": "42099", "County": "Perry", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17062", "FIPS": "42099", "County": "Perry", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17068", "FIPS": "42099", "County": "Perry", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17069", "FIPS": "42099", "County": "Perry", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17071", "FIPS": "42099", "County": "Perry", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17074", "FIPS": "42099", "County": "Perry", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17090", "FIPS": "42099", "County": "Perry", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "19019", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19092", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19093", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19099", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19101", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19102", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19103", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19104", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19105", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19106", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19107", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19108", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19109", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19110", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19111", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19112", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19114", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19115", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19116", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19118", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19119", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19120", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19121", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19122", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19123", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19124", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19125", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19126", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19127", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19128", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19129", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19130", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19131", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19132", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19133", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19134", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19135", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19136", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19137", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19138", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19139", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19140", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19141", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19142", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19143", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19144", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19145", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19146", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19147", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19148", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19149", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19150", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19151", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19152", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19153", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19154", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19155", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19160", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19161", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19162", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19170", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19171", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19172", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19173", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19175", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19176", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19177", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19178", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19179", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19181", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19182", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19183", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19184", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19185", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19187", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19188", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19191", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19192", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19193", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19194", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19195", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19196", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19197", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19244", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "19255", "FIPS": "42101", "County": "Philadelphia", "State": "PA", "DMA": {"Code": 504, "Name": "PHILADELPHIA"}},
  {"Postal_Code": "18324", "FIPS": "42103", "County": "Pike", "State": "PA", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "18328", "FIPS": "42103", "County": "Pike", "State": "PA", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "18336", "FIPS": "42103", "County": "Pike", "State": "PA", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "18337", "FIPS": "42103", "County": "Pike", "State": "PA", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "18340", "FIPS": "42103", "County": "Pike", "State": "PA", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "18371", "FIPS": "42103", "County": "Pike", "State": "PA", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "18373", "FIPS": "42103", "County": "Pike", "State": "PA", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "18425", "FIPS": "42103", "County": "Pike", "State": "PA", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "18426", "FIPS": "42103", "County": "Pike", "State": "PA", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "18435", "FIPS": "42103", "County": "Pike", "State": "PA", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "18451", "FIPS": "42103", "County": "Pike", "State": "PA", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "18457", "FIPS": "42103", "County": "Pike", "State": "PA", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "18458", "FIPS": "42103", "County": "Pike", "State": "PA", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "18464", "FIPS": "42103", "County": "Pike", "State": "PA", "DMA": {"Code": 501, "Name": "NEW YORK"}},
  {"Postal_Code": "16720", "FIPS": "42105", "County": "Potter", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16746", "FIPS": "42105", "County": "Potter", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16748", "FIPS": "42105", "County": "Potter", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16915", "FIPS": "42105", "County": "Potter", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16922", "FIPS": "42105", "County": "Potter", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16923", "FIPS": "42105", "County": "Potter", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16927", "FIPS": "42105", "County": "Potter", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16937", "FIPS": "42105", "County": "Potter", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16941", "FIPS": "42105", "County": "Potter", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "16948", "FIPS": "42105", "County": "Potter", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "17729", "FIPS": "42105", "County": "Potter", "State": "PA", "DMA": {"Code": 514, "Name": "BUFFALO"}},
  {"Postal_Code": "17901", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17921", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17922", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17923", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17925", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17929", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17930", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17931", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17932", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17933", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17934", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17935", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17936", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17938", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17941", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17942", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17943", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17944", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17945", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17946", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17948", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17949", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17951", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17952", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17953", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17954", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17957", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17959", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17960", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17961", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17963", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17964", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17965", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17966", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17967", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17968", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17970", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17972", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17974", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17976", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17978", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17979", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17980", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17981", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17982", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17983", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17985", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18211", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18214", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18218", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18220", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18231", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18237", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18241", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18242", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18245", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18248", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18252", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "19549", "FIPS": "42107", "County": "Schuylkill", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17812", "FIPS": "42109", "County": "Snyder", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17813", "FIPS": "42109", "County": "Snyder", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17827", "FIPS": "42109", "County": "Snyder", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17831", "FIPS": "42109", "County": "Snyder", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17833", "FIPS": "42109", "County": "Snyder", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17842", "FIPS": "42109", "County": "Snyder", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17843", "FIPS": "42109", "County": "Snyder", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17853", "FIPS": "42109", "County": "Snyder", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17861", "FIPS": "42109", "County": "Snyder", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17862", "FIPS": "42109", "County": "Snyder", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17864", "FIPS": "42109", "County": "Snyder", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17870", "FIPS": "42109", "County": "Snyder", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17876", "FIPS": "42109", "County": "Snyder", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17882", "FIPS": "42109", "County": "Snyder", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "15411", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15424", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15485", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15501", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15502", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15510", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15520", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15530", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15531", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15532", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15538", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15540", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15541", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15542", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15544", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15546", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15547", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15548", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15549", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15551", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15552", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15553", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15555", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15557", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15558", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15560", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15561", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15562", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15563", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15564", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15565", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15924", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15926", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15928", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15935", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15936", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15937", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15953", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15959", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "15963", "FIPS": "42111", "County": "Somerset", "State": "PA", "DMA": {"Code": 574, "Name": "JOHNSTOWN-ALTOONA"}},
  {"Postal_Code": "17731", "FIPS": "42113", "County": "Sullivan", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17758", "FIPS": "42113", "County": "Sullivan", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17768", "FIPS": "42113", "County": "Sullivan", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18614", "FIPS": "42113", "County": "Sullivan", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18616", "FIPS": "42113", "County": "Sullivan", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18619", "FIPS": "42113", "County": "Sullivan", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18626", "FIPS": "42113", "County": "Sullivan", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18628", "FIPS": "42113", "County": "Sullivan", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18632", "FIPS": "42113", "County": "Sullivan", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18413", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18421", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18430", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18441", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18465", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18470", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18801", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18812", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18813", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18816", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18818", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18820", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18821", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18822", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18823", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18824", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18825", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18826", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18827", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18828", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18830", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18834", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18842", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18843", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18844", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18847", "FIPS": "42115", "County": "Susquehanna", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "16901", "FIPS": "42117", "County": "Tioga", "State": "PA", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "16911", "FIPS": "42117", "County": "Tioga", "State": "PA", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "16912", "FIPS": "42117", "County": "Tioga", "State": "PA", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "16917", "FIPS": "42117", "County": "Tioga", "State": "PA", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "16918", "FIPS": "42117", "County": "Tioga", "State": "PA", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "16920", "FIPS": "42117", "County": "Tioga", "State": "PA", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "16921", "FIPS": "42117", "County": "Tioga", "State": "PA", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "16928", "FIPS": "42117", "County": "Tioga", "State": "PA", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "16929", "FIPS": "42117", "County": "Tioga", "State": "PA", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "16930", "FIPS": "42117", "County": "Tioga", "State": "PA", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "16932", "FIPS": "42117", "County": "Tioga", "State": "PA", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "16933", "FIPS": "42117", "County": "Tioga", "State": "PA", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "16935", "FIPS": "42117", "County": "Tioga", "State": "PA", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "16936", "FIPS": "42117", "County": "Tioga", "State": "PA", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "16938", "FIPS": "42117", "County": "Tioga", "State": "PA", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "16939", "FIPS": "42117", "County": "Tioga", "State": "PA", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "16940", "FIPS": "42117", "County": "Tioga", "State": "PA", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "16942", "FIPS": "42117", "County": "Tioga", "State": "PA", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "16943", "FIPS": "42117", "County": "Tioga", "State": "PA", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "16946", "FIPS": "42117", "County": "Tioga", "State": "PA", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "16950", "FIPS": "42117", "County": "Tioga", "State": "PA", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "17765", "FIPS": "42117", "County": "Tioga", "State": "PA", "DMA": {"Code": 565, "Name": "ELMIRA"}},
  {"Postal_Code": "17810", "FIPS": "42119", "County": "Union", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17829", "FIPS": "42119", "County": "Union", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17835", "FIPS": "42119", "County": "Union", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17837", "FIPS": "42119", "County": "Union", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17844", "FIPS": "42119", "County": "Union", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17845", "FIPS": "42119", "County": "Union", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17855", "FIPS": "42119", "County": "Union", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17856", "FIPS": "42119", "County": "Union", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17880", "FIPS": "42119", "County": "Union", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17883", "FIPS": "42119", "County": "Union", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17885", "FIPS": "42119", "County": "Union", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17886", "FIPS": "42119", "County": "Union", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17887", "FIPS": "42119", "County": "Union", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17889", "FIPS": "42119", "County": "Union", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "16301", "FIPS": "42121", "County": "Venango", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16317", "FIPS": "42121", "County": "Venango", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16319", "FIPS": "42121", "County": "Venango", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16323", "FIPS": "42121", "County": "Venango", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16341", "FIPS": "42121", "County": "Venango", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16342", "FIPS": "42121", "County": "Venango", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16343", "FIPS": "42121", "County": "Venango", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16344", "FIPS": "42121", "County": "Venango", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16346", "FIPS": "42121", "County": "Venango", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16362", "FIPS": "42121", "County": "Venango", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16364", "FIPS": "42121", "County": "Venango", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16372", "FIPS": "42121", "County": "Venango", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16373", "FIPS": "42121", "County": "Venango", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16374", "FIPS": "42121", "County": "Venango", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "16312", "FIPS": "42123", "County": "Warren", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16313", "FIPS": "42123", "County": "Warren", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16329", "FIPS": "42123", "County": "Warren", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16340", "FIPS": "42123", "County": "Warren", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16345", "FIPS": "42123", "County": "Warren", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16347", "FIPS": "42123", "County": "Warren", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16350", "FIPS": "42123", "County": "Warren", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16351", "FIPS": "42123", "County": "Warren", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16352", "FIPS": "42123", "County": "Warren", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16365", "FIPS": "42123", "County": "Warren", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16366", "FIPS": "42123", "County": "Warren", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16367", "FIPS": "42123", "County": "Warren", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16368", "FIPS": "42123", "County": "Warren", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16369", "FIPS": "42123", "County": "Warren", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16371", "FIPS": "42123", "County": "Warren", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16402", "FIPS": "42123", "County": "Warren", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16405", "FIPS": "42123", "County": "Warren", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16416", "FIPS": "42123", "County": "Warren", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16420", "FIPS": "42123", "County": "Warren", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "16436", "FIPS": "42123", "County": "Warren", "State": "PA", "DMA": {"Code": 516, "Name": "ERIE"}},
  {"Postal_Code": "15004", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15019", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15021", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15022", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15033", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15038", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15053", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15054", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15055", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15057", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15060", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15063", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15067", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15078", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15301", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15311", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15312", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15313", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15314", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15317", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15321", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15323", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15324", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15329", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15330", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15331", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15332", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15333", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15336", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15339", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15340", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15342", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15345", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15347", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15348", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15350", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15358", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15360", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15361", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15363", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15365", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15366", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15367", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15368", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15376", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15377", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15378", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15379", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15412", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15419", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15423", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15427", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15429", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15432", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15434", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15477", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15483", "FIPS": "42125", "County": "Washington", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "18405", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18415", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18417", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18424", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18427", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18428", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18431", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18436", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18437", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18438", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18439", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18443", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18445", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18449", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18453", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18454", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18455", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18456", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18459", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18460", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18461", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18462", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18463", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18469", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18472", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18473", "FIPS": "42127", "County": "Wayne", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "15062", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15068", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15069", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15072", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15083", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15085", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15087", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15089", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15448", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15479", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15601", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15605", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15606", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15610", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15611", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15612", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15613", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15615", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15616", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15617", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15618", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15619", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15620", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15621", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15622", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15623", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15624", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15625", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15626", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15627", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15628", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15629", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15632", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15633", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15634", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15635", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15636", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15637", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15638", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15639", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15640", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15641", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15642", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15644", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15646", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15647", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15650", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15655", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15658", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15660", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15661", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15662", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15663", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15664", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15665", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15666", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15668", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15670", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15671", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15672", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15674", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15675", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15676", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15677", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15678", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15679", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15680", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15683", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15684", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15685", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15687", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15688", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15689", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15690", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15691", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15692", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15693", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15695", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15696", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15697", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15698", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15779", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15923", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15944", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "15954", "FIPS": "42129", "County": "Westmoreland", "State": "PA", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "18419", "FIPS": "42131", "County": "Wyoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18446", "FIPS": "42131", "County": "Wyoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18615", "FIPS": "42131", "County": "Wyoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18623", "FIPS": "42131", "County": "Wyoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18625", "FIPS": "42131", "County": "Wyoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18629", "FIPS": "42131", "County": "Wyoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18630", "FIPS": "42131", "County": "Wyoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18636", "FIPS": "42131", "County": "Wyoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "18657", "FIPS": "42131", "County": "Wyoming", "State": "PA", "DMA": {"Code": 577, "Name": "WILKES BARRE-SCRANTON"}},
  {"Postal_Code": "17019", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17302", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17309", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17311", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17312", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17313", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17314", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17315", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17317", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17318", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17319", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17321", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17322", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17323", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17327", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17329", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17331", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17332", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17333", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17334", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17339", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17342", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17345", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17347", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17349", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17352", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17354", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17355", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17356", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17358", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17360", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17361", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17362", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17363", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17364", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17365", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17366", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17368", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17370", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17371", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17401", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17402", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17403", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17404", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17405", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17406", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17407", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17408", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "17415", "FIPS": "42133", "County": "York", "State": "PA", "DMA": {"Code": 566, "Name": "HARRISBURG-LANCASTER-LEBANON-YORK"}},
  {"Postal_Code": "02806", "FIPS": "44001", "County": "Bristol", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02809", "FIPS": "44001", "County": "Bristol", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02872", "FIPS": "44001", "County": "Bristol", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02885", "FIPS": "44001", "County": "Bristol", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02816", "FIPS": "44003", "County": "Kent", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02817", "FIPS": "44003", "County": "Kent", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02818", "FIPS": "44003", "County": "Kent", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02827", "FIPS": "44003", "County": "Kent", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02886", "FIPS": "44003", "County": "Kent", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02887", "FIPS": "44003", "County": "Kent", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02888", "FIPS": "44003", "County": "Kent", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02889", "FIPS": "44003", "County": "Kent", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02893", "FIPS": "44003", "County": "Kent", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02801", "FIPS": "44005", "County": "Newport", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02835", "FIPS": "44005", "County": "Newport", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02837", "FIPS": "44005", "County": "Newport", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02840", "FIPS": "44005", "County": "Newport", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02841", "FIPS": "44005", "County": "Newport", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02842", "FIPS": "44005", "County": "Newport", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02871", "FIPS": "44005", "County": "Newport", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02878", "FIPS": "44005", "County": "Newport", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02802", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02814", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02815", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02823", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02824", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02825", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02826", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02828", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02829", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02830", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02831", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02838", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02839", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02857", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02858", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02859", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02860", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02861", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02862", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02863", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02864", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02865", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02876", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02895", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02896", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02901", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02902", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02903", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02904", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02905", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02906", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02907", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02908", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02909", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02910", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02911", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02912", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02914", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02915", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02916", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02917", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02918", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02919", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02920", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02921", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02940", "FIPS": "44007", "County": "Providence", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02804", "FIPS": "44009", "County": "Washington", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02807", "FIPS": "44009", "County": "Washington", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02808", "FIPS": "44009", "County": "Washington", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02812", "FIPS": "44009", "County": "Washington", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02813", "FIPS": "44009", "County": "Washington", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02822", "FIPS": "44009", "County": "Washington", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02832", "FIPS": "44009", "County": "Washington", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02833", "FIPS": "44009", "County": "Washington", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02836", "FIPS": "44009", "County": "Washington", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02852", "FIPS": "44009", "County": "Washington", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02854", "FIPS": "44009", "County": "Washington", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02873", "FIPS": "44009", "County": "Washington", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02874", "FIPS": "44009", "County": "Washington", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02875", "FIPS": "44009", "County": "Washington", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02877", "FIPS": "44009", "County": "Washington", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02879", "FIPS": "44009", "County": "Washington", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02880", "FIPS": "44009", "County": "Washington", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02881", "FIPS": "44009", "County": "Washington", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02882", "FIPS": "44009", "County": "Washington", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02883", "FIPS": "44009", "County": "Washington", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02891", "FIPS": "44009", "County": "Washington", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02892", "FIPS": "44009", "County": "Washington", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02894", "FIPS": "44009", "County": "Washington", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "02898", "FIPS": "44009", "County": "Washington", "State": "RI", "DMA": {"Code": 521, "Name": "PROVIDENCE-NEW BEDFORD"}},
  {"Postal_Code": "29620", "FIPS": "45001", "County": "Abbeville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29628", "FIPS": "45001", "County": "Abbeville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29638", "FIPS": "45001", "County": "Abbeville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29639", "FIPS": "45001", "County": "Abbeville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29659", "FIPS": "45001", "County": "Abbeville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29105", "FIPS": "45003", "County": "Aiken", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29137", "FIPS": "45003", "County": "Aiken", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29164", "FIPS": "45003", "County": "Aiken", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29801", "FIPS": "45003", "County": "Aiken", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29802", "FIPS": "45003", "County": "Aiken", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29803", "FIPS": "45003", "County": "Aiken", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29804", "FIPS": "45003", "County": "Aiken", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29805", "FIPS": "45003", "County": "Aiken", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29808", "FIPS": "45003", "County": "Aiken", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29809", "FIPS": "45003", "County": "Aiken", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29816", "FIPS": "45003", "County": "Aiken", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29822", "FIPS": "45003", "County": "Aiken", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29828", "FIPS": "45003", "County": "Aiken", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29829", "FIPS": "45003", "County": "Aiken", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29831", "FIPS": "45003", "County": "Aiken", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29834", "FIPS": "45003", "County": "Aiken", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29839", "FIPS": "45003", "County": "Aiken", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29841", "FIPS": "45003", "County": "Aiken", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29842", "FIPS": "45003", "County": "Aiken", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29850", "FIPS": "45003", "County": "Aiken", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29851", "FIPS": "45003", "County": "Aiken", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29856", "FIPS": "45003", "County": "Aiken", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29860", "FIPS": "45003", "County": "Aiken", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29861", "FIPS": "45003", "County": "Aiken", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29810", "FIPS": "45005", "County": "Allendale", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29827", "FIPS": "45005", "County": "Allendale", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29836", "FIPS": "45005", "County": "Allendale", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29846", "FIPS": "45005", "County": "Allendale", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29849", "FIPS": "45005", "County": "Allendale", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29621", "FIPS": "45007", "County": "Anderson", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29622", "FIPS": "45007", "County": "Anderson", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29623", "FIPS": "45007", "County": "Anderson", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29624", "FIPS": "45007", "County": "Anderson", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29625", "FIPS": "45007", "County": "Anderson", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29626", "FIPS": "45007", "County": "Anderson", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29627", "FIPS": "45007", "County": "Anderson", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29654", "FIPS": "45007", "County": "Anderson", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29655", "FIPS": "45007", "County": "Anderson", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29656", "FIPS": "45007", "County": "Anderson", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29669", "FIPS": "45007", "County": "Anderson", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29670", "FIPS": "45007", "County": "Anderson", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29677", "FIPS": "45007", "County": "Anderson", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29684", "FIPS": "45007", "County": "Anderson", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29689", "FIPS": "45007", "County": "Anderson", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29697", "FIPS": "45007", "County": "Anderson", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29003", "FIPS": "45009", "County": "Bamberg", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29042", "FIPS": "45009", "County": "Bamberg", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29081", "FIPS": "45009", "County": "Bamberg", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29843", "FIPS": "45009", "County": "Bamberg", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29812", "FIPS": "45011", "County": "Barnwell", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29813", "FIPS": "45011", "County": "Barnwell", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29817", "FIPS": "45011", "County": "Barnwell", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29826", "FIPS": "45011", "County": "Barnwell", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29853", "FIPS": "45011", "County": "Barnwell", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29901", "FIPS": "45013", "County": "Beaufort", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29902", "FIPS": "45013", "County": "Beaufort", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29903", "FIPS": "45013", "County": "Beaufort", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29904", "FIPS": "45013", "County": "Beaufort", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29905", "FIPS": "45013", "County": "Beaufort", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29906", "FIPS": "45013", "County": "Beaufort", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29907", "FIPS": "45013", "County": "Beaufort", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29909", "FIPS": "45013", "County": "Beaufort", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29910", "FIPS": "45013", "County": "Beaufort", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29914", "FIPS": "45013", "County": "Beaufort", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29915", "FIPS": "45013", "County": "Beaufort", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29920", "FIPS": "45013", "County": "Beaufort", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29925", "FIPS": "45013", "County": "Beaufort", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29926", "FIPS": "45013", "County": "Beaufort", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29928", "FIPS": "45013", "County": "Beaufort", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29931", "FIPS": "45013", "County": "Beaufort", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29935", "FIPS": "45013", "County": "Beaufort", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29938", "FIPS": "45013", "County": "Beaufort", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29940", "FIPS": "45013", "County": "Beaufort", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29941", "FIPS": "45013", "County": "Beaufort", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29410", "FIPS": "45015", "County": "Berkeley", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29430", "FIPS": "45015", "County": "Berkeley", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29431", "FIPS": "45015", "County": "Berkeley", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29434", "FIPS": "45015", "County": "Berkeley", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29436", "FIPS": "45015", "County": "Berkeley", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29445", "FIPS": "45015", "County": "Berkeley", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29450", "FIPS": "45015", "County": "Berkeley", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29453", "FIPS": "45015", "County": "Berkeley", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29456", "FIPS": "45015", "County": "Berkeley", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29461", "FIPS": "45015", "County": "Berkeley", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29468", "FIPS": "45015", "County": "Berkeley", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29469", "FIPS": "45015", "County": "Berkeley", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29476", "FIPS": "45015", "County": "Berkeley", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29479", "FIPS": "45015", "County": "Berkeley", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29492", "FIPS": "45015", "County": "Berkeley", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29030", "FIPS": "45017", "County": "Calhoun", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29135", "FIPS": "45017", "County": "Calhoun", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29401", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29402", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29403", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29404", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29405", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29406", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29407", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29409", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29412", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29413", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29414", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29415", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29416", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29417", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29418", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29419", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29422", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29423", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29424", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29425", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29426", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29429", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29439", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29449", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29451", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29455", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29457", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29458", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29464", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29465", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29466", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29470", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29482", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29487", "FIPS": "45019", "County": "Charleston", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29340", "FIPS": "45021", "County": "Cherokee", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29341", "FIPS": "45021", "County": "Cherokee", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29342", "FIPS": "45021", "County": "Cherokee", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29702", "FIPS": "45021", "County": "Cherokee", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29014", "FIPS": "45023", "County": "Chester", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29055", "FIPS": "45023", "County": "Chester", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29706", "FIPS": "45023", "County": "Chester", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29712", "FIPS": "45023", "County": "Chester", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29714", "FIPS": "45023", "County": "Chester", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29724", "FIPS": "45023", "County": "Chester", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29729", "FIPS": "45023", "County": "Chester", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29101", "FIPS": "45025", "County": "Chesterfield", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29520", "FIPS": "45025", "County": "Chesterfield", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29584", "FIPS": "45025", "County": "Chesterfield", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29709", "FIPS": "45025", "County": "Chesterfield", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29718", "FIPS": "45025", "County": "Chesterfield", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29727", "FIPS": "45025", "County": "Chesterfield", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29728", "FIPS": "45025", "County": "Chesterfield", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29741", "FIPS": "45025", "County": "Chesterfield", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29001", "FIPS": "45027", "County": "Clarendon", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29041", "FIPS": "45027", "County": "Clarendon", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29051", "FIPS": "45027", "County": "Clarendon", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29102", "FIPS": "45027", "County": "Clarendon", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29111", "FIPS": "45027", "County": "Clarendon", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29143", "FIPS": "45027", "County": "Clarendon", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29148", "FIPS": "45027", "County": "Clarendon", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29162", "FIPS": "45027", "County": "Clarendon", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29082", "FIPS": "45029", "County": "Colleton", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29433", "FIPS": "45029", "County": "Colleton", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29435", "FIPS": "45029", "County": "Colleton", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29438", "FIPS": "45029", "County": "Colleton", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29446", "FIPS": "45029", "County": "Colleton", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29452", "FIPS": "45029", "County": "Colleton", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29474", "FIPS": "45029", "County": "Colleton", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29475", "FIPS": "45029", "County": "Colleton", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29481", "FIPS": "45029", "County": "Colleton", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29488", "FIPS": "45029", "County": "Colleton", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29493", "FIPS": "45029", "County": "Colleton", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29929", "FIPS": "45029", "County": "Colleton", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29069", "FIPS": "45031", "County": "Darlington", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29079", "FIPS": "45031", "County": "Darlington", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29532", "FIPS": "45031", "County": "Darlington", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29540", "FIPS": "45031", "County": "Darlington", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29550", "FIPS": "45031", "County": "Darlington", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29551", "FIPS": "45031", "County": "Darlington", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29593", "FIPS": "45031", "County": "Darlington", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29536", "FIPS": "45033", "County": "Dillon", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29542", "FIPS": "45033", "County": "Dillon", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29543", "FIPS": "45033", "County": "Dillon", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29547", "FIPS": "45033", "County": "Dillon", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29563", "FIPS": "45033", "County": "Dillon", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29565", "FIPS": "45033", "County": "Dillon", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29567", "FIPS": "45033", "County": "Dillon", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29573", "FIPS": "45033", "County": "Dillon", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29420", "FIPS": "45035", "County": "Dorchester", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29437", "FIPS": "45035", "County": "Dorchester", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29447", "FIPS": "45035", "County": "Dorchester", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29448", "FIPS": "45035", "County": "Dorchester", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29471", "FIPS": "45035", "County": "Dorchester", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29472", "FIPS": "45035", "County": "Dorchester", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29477", "FIPS": "45035", "County": "Dorchester", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29483", "FIPS": "45035", "County": "Dorchester", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29484", "FIPS": "45035", "County": "Dorchester", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29485", "FIPS": "45035", "County": "Dorchester", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29824", "FIPS": "45037", "County": "Edgefield", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29832", "FIPS": "45037", "County": "Edgefield", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29847", "FIPS": "45037", "County": "Edgefield", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29015", "FIPS": "45039", "County": "Fairfield", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29065", "FIPS": "45039", "County": "Fairfield", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29130", "FIPS": "45039", "County": "Fairfield", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29132", "FIPS": "45039", "County": "Fairfield", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29180", "FIPS": "45039", "County": "Fairfield", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29114", "FIPS": "45041", "County": "Florence", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29161", "FIPS": "45041", "County": "Florence", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29501", "FIPS": "45041", "County": "Florence", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29502", "FIPS": "45041", "County": "Florence", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29503", "FIPS": "45041", "County": "Florence", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29504", "FIPS": "45041", "County": "Florence", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29505", "FIPS": "45041", "County": "Florence", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29506", "FIPS": "45041", "County": "Florence", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29530", "FIPS": "45041", "County": "Florence", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29541", "FIPS": "45041", "County": "Florence", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29555", "FIPS": "45041", "County": "Florence", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29560", "FIPS": "45041", "County": "Florence", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29583", "FIPS": "45041", "County": "Florence", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29591", "FIPS": "45041", "County": "Florence", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29440", "FIPS": "45043", "County": "Georgetown", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29442", "FIPS": "45043", "County": "Georgetown", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29510", "FIPS": "45043", "County": "Georgetown", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29585", "FIPS": "45043", "County": "Georgetown", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29601", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29602", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29603", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29604", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29605", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29606", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29607", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29608", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29609", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29610", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29611", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29612", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29613", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29614", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29615", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29616", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29617", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29635", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29636", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29644", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29650", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29651", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29652", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29661", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29662", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29673", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29680", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29681", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29683", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29687", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29688", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29690", "FIPS": "45045", "County": "Greenville", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29646", "FIPS": "45047", "County": "Greenwood", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29647", "FIPS": "45047", "County": "Greenwood", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29648", "FIPS": "45047", "County": "Greenwood", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29649", "FIPS": "45047", "County": "Greenwood", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29653", "FIPS": "45047", "County": "Greenwood", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29666", "FIPS": "45047", "County": "Greenwood", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29692", "FIPS": "45047", "County": "Greenwood", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29695", "FIPS": "45047", "County": "Greenwood", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29819", "FIPS": "45047", "County": "Greenwood", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29848", "FIPS": "45047", "County": "Greenwood", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29911", "FIPS": "45049", "County": "Hampton", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29913", "FIPS": "45049", "County": "Hampton", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29916", "FIPS": "45049", "County": "Hampton", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29918", "FIPS": "45049", "County": "Hampton", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29921", "FIPS": "45049", "County": "Hampton", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29922", "FIPS": "45049", "County": "Hampton", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29923", "FIPS": "45049", "County": "Hampton", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29924", "FIPS": "45049", "County": "Hampton", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29932", "FIPS": "45049", "County": "Hampton", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29933", "FIPS": "45049", "County": "Hampton", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29939", "FIPS": "45049", "County": "Hampton", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29944", "FIPS": "45049", "County": "Hampton", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29945", "FIPS": "45049", "County": "Hampton", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29511", "FIPS": "45051", "County": "Horry", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29526", "FIPS": "45051", "County": "Horry", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29527", "FIPS": "45051", "County": "Horry", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29528", "FIPS": "45051", "County": "Horry", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29544", "FIPS": "45051", "County": "Horry", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29545", "FIPS": "45051", "County": "Horry", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29566", "FIPS": "45051", "County": "Horry", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29568", "FIPS": "45051", "County": "Horry", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29569", "FIPS": "45051", "County": "Horry", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29572", "FIPS": "45051", "County": "Horry", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29575", "FIPS": "45051", "County": "Horry", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29576", "FIPS": "45051", "County": "Horry", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29577", "FIPS": "45051", "County": "Horry", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29578", "FIPS": "45051", "County": "Horry", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29579", "FIPS": "45051", "County": "Horry", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29581", "FIPS": "45051", "County": "Horry", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29582", "FIPS": "45051", "County": "Horry", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29587", "FIPS": "45051", "County": "Horry", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29588", "FIPS": "45051", "County": "Horry", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29597", "FIPS": "45051", "County": "Horry", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29598", "FIPS": "45051", "County": "Horry", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29912", "FIPS": "45053", "County": "Jasper", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29927", "FIPS": "45053", "County": "Jasper", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29934", "FIPS": "45053", "County": "Jasper", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29936", "FIPS": "45053", "County": "Jasper", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29943", "FIPS": "45053", "County": "Jasper", "State": "SC", "DMA": {"Code": 507, "Name": "SAVANNAH"}},
  {"Postal_Code": "29009", "FIPS": "45055", "County": "Kershaw", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29020", "FIPS": "45055", "County": "Kershaw", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29021", "FIPS": "45055", "County": "Kershaw", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29032", "FIPS": "45055", "County": "Kershaw", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29045", "FIPS": "45055", "County": "Kershaw", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29074", "FIPS": "45055", "County": "Kershaw", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29078", "FIPS": "45055", "County": "Kershaw", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29175", "FIPS": "45055", "County": "Kershaw", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29058", "FIPS": "45057", "County": "Lancaster", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29067", "FIPS": "45057", "County": "Lancaster", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29720", "FIPS": "45057", "County": "Lancaster", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29721", "FIPS": "45057", "County": "Lancaster", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29722", "FIPS": "45057", "County": "Lancaster", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29744", "FIPS": "45057", "County": "Lancaster", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29325", "FIPS": "45059", "County": "Laurens", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29332", "FIPS": "45059", "County": "Laurens", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29351", "FIPS": "45059", "County": "Laurens", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29360", "FIPS": "45059", "County": "Laurens", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29370", "FIPS": "45059", "County": "Laurens", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29384", "FIPS": "45059", "County": "Laurens", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29645", "FIPS": "45059", "County": "Laurens", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29010", "FIPS": "45061", "County": "Lee", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29046", "FIPS": "45061", "County": "Lee", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29080", "FIPS": "45061", "County": "Lee", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29006", "FIPS": "45063", "County": "Lexington", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29033", "FIPS": "45063", "County": "Lexington", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29036", "FIPS": "45063", "County": "Lexington", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29053", "FIPS": "45063", "County": "Lexington", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29054", "FIPS": "45063", "County": "Lexington", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29070", "FIPS": "45063", "County": "Lexington", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29071", "FIPS": "45063", "County": "Lexington", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29072", "FIPS": "45063", "County": "Lexington", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29073", "FIPS": "45063", "County": "Lexington", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29123", "FIPS": "45063", "County": "Lexington", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29160", "FIPS": "45063", "County": "Lexington", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29169", "FIPS": "45063", "County": "Lexington", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29170", "FIPS": "45063", "County": "Lexington", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29171", "FIPS": "45063", "County": "Lexington", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29172", "FIPS": "45063", "County": "Lexington", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29212", "FIPS": "45063", "County": "Lexington", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29228", "FIPS": "45063", "County": "Lexington", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29821", "FIPS": "45065", "County": "McCormick", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29835", "FIPS": "45065", "County": "McCormick", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29838", "FIPS": "45065", "County": "McCormick", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29840", "FIPS": "45065", "County": "McCormick", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29844", "FIPS": "45065", "County": "McCormick", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29845", "FIPS": "45065", "County": "McCormick", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29899", "FIPS": "45065", "County": "McCormick", "State": "SC", "DMA": {"Code": 520, "Name": "AUGUSTA"}},
  {"Postal_Code": "29519", "FIPS": "45067", "County": "Marion", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29546", "FIPS": "45067", "County": "Marion", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29571", "FIPS": "45067", "County": "Marion", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29574", "FIPS": "45067", "County": "Marion", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29589", "FIPS": "45067", "County": "Marion", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29592", "FIPS": "45067", "County": "Marion", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29512", "FIPS": "45069", "County": "Marlboro", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29516", "FIPS": "45069", "County": "Marlboro", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29525", "FIPS": "45069", "County": "Marlboro", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29570", "FIPS": "45069", "County": "Marlboro", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29594", "FIPS": "45069", "County": "Marlboro", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29596", "FIPS": "45069", "County": "Marlboro", "State": "SC", "DMA": {"Code": 570, "Name": "MYRTLE BEACH-FLORENCE"}},
  {"Postal_Code": "29037", "FIPS": "45071", "County": "Newberry", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29075", "FIPS": "45071", "County": "Newberry", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29108", "FIPS": "45071", "County": "Newberry", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29122", "FIPS": "45071", "County": "Newberry", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29126", "FIPS": "45071", "County": "Newberry", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29127", "FIPS": "45071", "County": "Newberry", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29145", "FIPS": "45071", "County": "Newberry", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29178", "FIPS": "45071", "County": "Newberry", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29355", "FIPS": "45071", "County": "Newberry", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29643", "FIPS": "45073", "County": "Oconee", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29658", "FIPS": "45073", "County": "Oconee", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29664", "FIPS": "45073", "County": "Oconee", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29665", "FIPS": "45073", "County": "Oconee", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29672", "FIPS": "45073", "County": "Oconee", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29675", "FIPS": "45073", "County": "Oconee", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29676", "FIPS": "45073", "County": "Oconee", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29678", "FIPS": "45073", "County": "Oconee", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29679", "FIPS": "45073", "County": "Oconee", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29686", "FIPS": "45073", "County": "Oconee", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29691", "FIPS": "45073", "County": "Oconee", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29693", "FIPS": "45073", "County": "Oconee", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29696", "FIPS": "45073", "County": "Oconee", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29018", "FIPS": "45075", "County": "Orangeburg", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29038", "FIPS": "45075", "County": "Orangeburg", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29039", "FIPS": "45075", "County": "Orangeburg", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29047", "FIPS": "45075", "County": "Orangeburg", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29048", "FIPS": "45075", "County": "Orangeburg", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29059", "FIPS": "45075", "County": "Orangeburg", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29107", "FIPS": "45075", "County": "Orangeburg", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29112", "FIPS": "45075", "County": "Orangeburg", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29113", "FIPS": "45075", "County": "Orangeburg", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29115", "FIPS": "45075", "County": "Orangeburg", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29116", "FIPS": "45075", "County": "Orangeburg", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29117", "FIPS": "45075", "County": "Orangeburg", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29118", "FIPS": "45075", "County": "Orangeburg", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29133", "FIPS": "45075", "County": "Orangeburg", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29142", "FIPS": "45075", "County": "Orangeburg", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29146", "FIPS": "45075", "County": "Orangeburg", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29163", "FIPS": "45075", "County": "Orangeburg", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29432", "FIPS": "45075", "County": "Orangeburg", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29630", "FIPS": "45077", "County": "Pickens", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29631", "FIPS": "45077", "County": "Pickens", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29632", "FIPS": "45077", "County": "Pickens", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29633", "FIPS": "45077", "County": "Pickens", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29634", "FIPS": "45077", "County": "Pickens", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29640", "FIPS": "45077", "County": "Pickens", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29641", "FIPS": "45077", "County": "Pickens", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29642", "FIPS": "45077", "County": "Pickens", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29657", "FIPS": "45077", "County": "Pickens", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29667", "FIPS": "45077", "County": "Pickens", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29671", "FIPS": "45077", "County": "Pickens", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29682", "FIPS": "45077", "County": "Pickens", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29685", "FIPS": "45077", "County": "Pickens", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29002", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29016", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29044", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29052", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29061", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29063", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29147", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29177", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29201", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29202", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29203", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29204", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29205", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29206", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29207", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29208", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29209", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29210", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29211", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29214", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29215", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29216", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29217", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29218", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29219", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29220", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29221", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29222", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29223", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29224", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29225", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29226", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29227", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29229", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29230", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29240", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29250", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29260", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29290", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29292", "FIPS": "45079", "County": "Richland", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29129", "FIPS": "45081", "County": "Saluda", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29138", "FIPS": "45081", "County": "Saluda", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29166", "FIPS": "45081", "County": "Saluda", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29301", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29302", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29303", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29304", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29305", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29306", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29307", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29316", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29318", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29319", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29320", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29322", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29323", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29324", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29329", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29330", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29331", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29333", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29334", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29335", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29336", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29338", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29346", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29348", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29349", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29356", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29365", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29368", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29369", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29372", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29373", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29374", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29375", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29376", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29377", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29378", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29385", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29386", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29388", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29390", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29391", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29698", "FIPS": "45083", "County": "Spartanburg", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29040", "FIPS": "45085", "County": "Sumter", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29062", "FIPS": "45085", "County": "Sumter", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29104", "FIPS": "45085", "County": "Sumter", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29125", "FIPS": "45085", "County": "Sumter", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29128", "FIPS": "45085", "County": "Sumter", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29150", "FIPS": "45085", "County": "Sumter", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29151", "FIPS": "45085", "County": "Sumter", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29152", "FIPS": "45085", "County": "Sumter", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29153", "FIPS": "45085", "County": "Sumter", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29154", "FIPS": "45085", "County": "Sumter", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29168", "FIPS": "45085", "County": "Sumter", "State": "SC", "DMA": {"Code": 546, "Name": "COLUMBIA, SC"}},
  {"Postal_Code": "29031", "FIPS": "45087", "County": "Union", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29321", "FIPS": "45087", "County": "Union", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29353", "FIPS": "45087", "County": "Union", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29364", "FIPS": "45087", "County": "Union", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29379", "FIPS": "45087", "County": "Union", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29395", "FIPS": "45087", "County": "Union", "State": "SC", "DMA": {"Code": 567, "Name": "GREENVILLE-SPARTANBURG-ASHEVILLE"}},
  {"Postal_Code": "29056", "FIPS": "45089", "County": "Williamsburg", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29518", "FIPS": "45089", "County": "Williamsburg", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29554", "FIPS": "45089", "County": "Williamsburg", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29556", "FIPS": "45089", "County": "Williamsburg", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29564", "FIPS": "45089", "County": "Williamsburg", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29580", "FIPS": "45089", "County": "Williamsburg", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29590", "FIPS": "45089", "County": "Williamsburg", "State": "SC", "DMA": {"Code": 519, "Name": "CHARLESTON, SC"}},
  {"Postal_Code": "29703", "FIPS": "45091", "County": "York", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29704", "FIPS": "45091", "County": "York", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29708", "FIPS": "45091", "County": "York", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29710", "FIPS": "45091", "County": "York", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29715", "FIPS": "45091", "County": "York", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29716", "FIPS": "45091", "County": "York", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29717", "FIPS": "45091", "County": "York", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29726", "FIPS": "45091", "County": "York", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29730", "FIPS": "45091", "County": "York", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29731", "FIPS": "45091", "County": "York", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29732", "FIPS": "45091", "County": "York", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29733", "FIPS": "45091", "County": "York", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29734", "FIPS": "45091", "County": "York", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29742", "FIPS": "45091", "County": "York", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29743", "FIPS": "45091", "County": "York", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "29745", "FIPS": "45091", "County": "York", "State": "SC", "DMA": {"Code": 517, "Name": "CHARLOTTE"}},
  {"Postal_Code": "57368", "FIPS": "46003", "County": "Aurora", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57375", "FIPS": "46003", "County": "Aurora", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57383", "FIPS": "46003", "County": "Aurora", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57324", "FIPS": "46005", "County": "Beadle", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57348", "FIPS": "46005", "County": "Beadle", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57350", "FIPS": "46005", "County": "Beadle", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57379", "FIPS": "46005", "County": "Beadle", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57381", "FIPS": "46005", "County": "Beadle", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57384", "FIPS": "46005", "County": "Beadle", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57386", "FIPS": "46005", "County": "Beadle", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57399", "FIPS": "46005", "County": "Beadle", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57551", "FIPS": "46007", "County": "Bennett", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57574", "FIPS": "46007", "County": "Bennett", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57714", "FIPS": "46007", "County": "Bennett", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57059", "FIPS": "46009", "County": "Bon Homme", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57062", "FIPS": "46009", "County": "Bon Homme", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57063", "FIPS": "46009", "County": "Bon Homme", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57066", "FIPS": "46009", "County": "Bon Homme", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57315", "FIPS": "46009", "County": "Bon Homme", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57002", "FIPS": "46011", "County": "Brookings", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57006", "FIPS": "46011", "County": "Brookings", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57007", "FIPS": "46011", "County": "Brookings", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57026", "FIPS": "46011", "County": "Brookings", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57061", "FIPS": "46011", "County": "Brookings", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57071", "FIPS": "46011", "County": "Brookings", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57220", "FIPS": "46011", "County": "Brookings", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57276", "FIPS": "46011", "County": "Brookings", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57401", "FIPS": "46013", "County": "Brown", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57402", "FIPS": "46013", "County": "Brown", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57426", "FIPS": "46013", "County": "Brown", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57427", "FIPS": "46013", "County": "Brown", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57432", "FIPS": "46013", "County": "Brown", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57433", "FIPS": "46013", "County": "Brown", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57439", "FIPS": "46013", "County": "Brown", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57441", "FIPS": "46013", "County": "Brown", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57445", "FIPS": "46013", "County": "Brown", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57446", "FIPS": "46013", "County": "Brown", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57449", "FIPS": "46013", "County": "Brown", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57460", "FIPS": "46013", "County": "Brown", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57474", "FIPS": "46013", "County": "Brown", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57479", "FIPS": "46013", "County": "Brown", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57481", "FIPS": "46013", "County": "Brown", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57325", "FIPS": "46015", "County": "Brule", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57326", "FIPS": "46015", "County": "Brule", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57355", "FIPS": "46015", "County": "Brule", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57370", "FIPS": "46015", "County": "Brule", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57339", "FIPS": "46017", "County": "Buffalo", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57341", "FIPS": "46017", "County": "Buffalo", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57717", "FIPS": "46019", "County": "Butte", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57760", "FIPS": "46019", "County": "Butte", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57762", "FIPS": "46019", "County": "Butte", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57788", "FIPS": "46019", "County": "Butte", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57632", "FIPS": "46021", "County": "Campbell", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57646", "FIPS": "46021", "County": "Campbell", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57648", "FIPS": "46021", "County": "Campbell", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57329", "FIPS": "46023", "County": "Charles Mix", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57342", "FIPS": "46023", "County": "Charles Mix", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57356", "FIPS": "46023", "County": "Charles Mix", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57361", "FIPS": "46023", "County": "Charles Mix", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57367", "FIPS": "46023", "County": "Charles Mix", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57369", "FIPS": "46023", "County": "Charles Mix", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57380", "FIPS": "46023", "County": "Charles Mix", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57217", "FIPS": "46025", "County": "Clark", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57225", "FIPS": "46025", "County": "Clark", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57236", "FIPS": "46025", "County": "Clark", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57258", "FIPS": "46025", "County": "Clark", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57271", "FIPS": "46025", "County": "Clark", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57278", "FIPS": "46025", "County": "Clark", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57322", "FIPS": "46025", "County": "Clark", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57010", "FIPS": "46027", "County": "Clay", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57037", "FIPS": "46027", "County": "Clay", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57069", "FIPS": "46027", "County": "Clay", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57073", "FIPS": "46027", "County": "Clay", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57201", "FIPS": "46029", "County": "Codington", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57235", "FIPS": "46029", "County": "Codington", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57243", "FIPS": "46029", "County": "Codington", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57245", "FIPS": "46029", "County": "Codington", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57263", "FIPS": "46029", "County": "Codington", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57272", "FIPS": "46029", "County": "Codington", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57621", "FIPS": "46031", "County": "Corson", "State": "SD", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "57634", "FIPS": "46031", "County": "Corson", "State": "SD", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "57639", "FIPS": "46031", "County": "Corson", "State": "SD", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "57641", "FIPS": "46031", "County": "Corson", "State": "SD", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "57642", "FIPS": "46031", "County": "Corson", "State": "SD", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "57645", "FIPS": "46031", "County": "Corson", "State": "SD", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "57657", "FIPS": "46031", "County": "Corson", "State": "SD", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "57658", "FIPS": "46031", "County": "Corson", "State": "SD", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "57659", "FIPS": "46031", "County": "Corson", "State": "SD", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "57660", "FIPS": "46031", "County": "Corson", "State": "SD", "DMA": {"Code": 687, "Name": "MINOT-BISMARCK-DICKINSON"}},
  {"Postal_Code": "57722", "FIPS": "46033", "County": "Custer", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57730", "FIPS": "46033", "County": "Custer", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57738", "FIPS": "46033", "County": "Custer", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57744", "FIPS": "46033", "County": "Custer", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57773", "FIPS": "46033", "County": "Custer", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57301", "FIPS": "46035", "County": "Davison", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57334", "FIPS": "46035", "County": "Davison", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57363", "FIPS": "46035", "County": "Davison", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57219", "FIPS": "46037", "County": "Day", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57239", "FIPS": "46037", "County": "Day", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57261", "FIPS": "46037", "County": "Day", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57273", "FIPS": "46037", "County": "Day", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57274", "FIPS": "46037", "County": "Day", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57422", "FIPS": "46037", "County": "Day", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57468", "FIPS": "46037", "County": "Day", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57213", "FIPS": "46039", "County": "Deuel", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57218", "FIPS": "46039", "County": "Deuel", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57226", "FIPS": "46039", "County": "Deuel", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57237", "FIPS": "46039", "County": "Deuel", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57238", "FIPS": "46039", "County": "Deuel", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57268", "FIPS": "46039", "County": "Deuel", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57625", "FIPS": "46041", "County": "Dewey", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57630", "FIPS": "46041", "County": "Dewey", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57633", "FIPS": "46041", "County": "Dewey", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57636", "FIPS": "46041", "County": "Dewey", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57652", "FIPS": "46041", "County": "Dewey", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57656", "FIPS": "46041", "County": "Dewey", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57661", "FIPS": "46041", "County": "Dewey", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57313", "FIPS": "46043", "County": "Douglas", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57328", "FIPS": "46043", "County": "Douglas", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57330", "FIPS": "46043", "County": "Douglas", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57344", "FIPS": "46043", "County": "Douglas", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57364", "FIPS": "46043", "County": "Douglas", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57428", "FIPS": "46045", "County": "Edmunds", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57448", "FIPS": "46045", "County": "Edmunds", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57451", "FIPS": "46045", "County": "Edmunds", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57471", "FIPS": "46045", "County": "Edmunds", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57735", "FIPS": "46047", "County": "Fall River", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57747", "FIPS": "46047", "County": "Fall River", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57763", "FIPS": "46047", "County": "Fall River", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57766", "FIPS": "46047", "County": "Fall River", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57782", "FIPS": "46047", "County": "Fall River", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57435", "FIPS": "46049", "County": "Faulk", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57438", "FIPS": "46049", "County": "Faulk", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57466", "FIPS": "46049", "County": "Faulk", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57470", "FIPS": "46049", "County": "Faulk", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57473", "FIPS": "46049", "County": "Faulk", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57216", "FIPS": "46051", "County": "Grant", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57246", "FIPS": "46051", "County": "Grant", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57251", "FIPS": "46051", "County": "Grant", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57252", "FIPS": "46051", "County": "Grant", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57253", "FIPS": "46051", "County": "Grant", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57259", "FIPS": "46051", "County": "Grant", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57264", "FIPS": "46051", "County": "Grant", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57265", "FIPS": "46051", "County": "Grant", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57269", "FIPS": "46051", "County": "Grant", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57317", "FIPS": "46053", "County": "Gregory", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57335", "FIPS": "46053", "County": "Gregory", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57523", "FIPS": "46053", "County": "Gregory", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57529", "FIPS": "46053", "County": "Gregory", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57533", "FIPS": "46053", "County": "Gregory", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57538", "FIPS": "46053", "County": "Gregory", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57571", "FIPS": "46053", "County": "Gregory", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57552", "FIPS": "46055", "County": "Haakon", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57553", "FIPS": "46055", "County": "Haakon", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57567", "FIPS": "46055", "County": "Haakon", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57221", "FIPS": "46057", "County": "Hamlin", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57223", "FIPS": "46057", "County": "Hamlin", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57234", "FIPS": "46057", "County": "Hamlin", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57241", "FIPS": "46057", "County": "Hamlin", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57242", "FIPS": "46057", "County": "Hamlin", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57248", "FIPS": "46057", "County": "Hamlin", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57362", "FIPS": "46059", "County": "Hand", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57371", "FIPS": "46059", "County": "Hand", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57373", "FIPS": "46059", "County": "Hand", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57467", "FIPS": "46059", "County": "Hand", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57311", "FIPS": "46061", "County": "Hanson", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57332", "FIPS": "46061", "County": "Hanson", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57340", "FIPS": "46061", "County": "Hanson", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57650", "FIPS": "46063", "County": "Harding", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57651", "FIPS": "46063", "County": "Harding", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57720", "FIPS": "46063", "County": "Harding", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57724", "FIPS": "46063", "County": "Harding", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57755", "FIPS": "46063", "County": "Harding", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57776", "FIPS": "46063", "County": "Harding", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57501", "FIPS": "46065", "County": "Hughes", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57522", "FIPS": "46065", "County": "Hughes", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57536", "FIPS": "46065", "County": "Hughes", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57029", "FIPS": "46067", "County": "Hutchinson", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57045", "FIPS": "46067", "County": "Hutchinson", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57052", "FIPS": "46067", "County": "Hutchinson", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57331", "FIPS": "46067", "County": "Hutchinson", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57354", "FIPS": "46067", "County": "Hutchinson", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57366", "FIPS": "46067", "County": "Hutchinson", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57376", "FIPS": "46067", "County": "Hutchinson", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57345", "FIPS": "46069", "County": "Hyde", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57346", "FIPS": "46069", "County": "Hyde", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57540", "FIPS": "46069", "County": "Hyde", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57521", "FIPS": "46071", "County": "Jackson", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57543", "FIPS": "46071", "County": "Jackson", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57547", "FIPS": "46071", "County": "Jackson", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57577", "FIPS": "46071", "County": "Jackson", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57750", "FIPS": "46071", "County": "Jackson", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57312", "FIPS": "46073", "County": "Jerauld", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57358", "FIPS": "46073", "County": "Jerauld", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57382", "FIPS": "46073", "County": "Jerauld", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57531", "FIPS": "46075", "County": "Jones", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57559", "FIPS": "46075", "County": "Jones", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57562", "FIPS": "46075", "County": "Jones", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57051", "FIPS": "46077", "County": "Kingsbury", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57212", "FIPS": "46077", "County": "Kingsbury", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57214", "FIPS": "46077", "County": "Kingsbury", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57231", "FIPS": "46077", "County": "Kingsbury", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57233", "FIPS": "46077", "County": "Kingsbury", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57249", "FIPS": "46077", "County": "Kingsbury", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57353", "FIPS": "46077", "County": "Kingsbury", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57016", "FIPS": "46079", "County": "Lake", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57042", "FIPS": "46079", "County": "Lake", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57050", "FIPS": "46079", "County": "Lake", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57054", "FIPS": "46079", "County": "Lake", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57057", "FIPS": "46079", "County": "Lake", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57075", "FIPS": "46079", "County": "Lake", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57076", "FIPS": "46079", "County": "Lake", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57732", "FIPS": "46081", "County": "Lawrence", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57754", "FIPS": "46081", "County": "Lawrence", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57759", "FIPS": "46081", "County": "Lawrence", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57779", "FIPS": "46081", "County": "Lawrence", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57783", "FIPS": "46081", "County": "Lawrence", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57793", "FIPS": "46081", "County": "Lawrence", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57799", "FIPS": "46081", "County": "Lawrence", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57013", "FIPS": "46083", "County": "Lincoln", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57027", "FIPS": "46083", "County": "Lincoln", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57032", "FIPS": "46083", "County": "Lincoln", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57034", "FIPS": "46083", "County": "Lincoln", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57039", "FIPS": "46083", "County": "Lincoln", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57064", "FIPS": "46083", "County": "Lincoln", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57077", "FIPS": "46083", "County": "Lincoln", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57108", "FIPS": "46083", "County": "Lincoln", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57365", "FIPS": "46085", "County": "Lyman", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57544", "FIPS": "46085", "County": "Lyman", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57548", "FIPS": "46085", "County": "Lyman", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57568", "FIPS": "46085", "County": "Lyman", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57569", "FIPS": "46085", "County": "Lyman", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57576", "FIPS": "46085", "County": "Lyman", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57012", "FIPS": "46087", "County": "McCook", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57048", "FIPS": "46087", "County": "McCook", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57058", "FIPS": "46087", "County": "McCook", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57319", "FIPS": "46087", "County": "McCook", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57374", "FIPS": "46087", "County": "McCook", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57437", "FIPS": "46089", "County": "McPherson", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57456", "FIPS": "46089", "County": "McPherson", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57457", "FIPS": "46089", "County": "McPherson", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57232", "FIPS": "46091", "County": "Marshall", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57247", "FIPS": "46091", "County": "Marshall", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57270", "FIPS": "46091", "County": "Marshall", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57421", "FIPS": "46091", "County": "Marshall", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57430", "FIPS": "46091", "County": "Marshall", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57454", "FIPS": "46091", "County": "Marshall", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57626", "FIPS": "46093", "County": "Meade", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57706", "FIPS": "46093", "County": "Meade", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57718", "FIPS": "46093", "County": "Meade", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57737", "FIPS": "46093", "County": "Meade", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57741", "FIPS": "46093", "County": "Meade", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57748", "FIPS": "46093", "County": "Meade", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57758", "FIPS": "46093", "County": "Meade", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57769", "FIPS": "46093", "County": "Meade", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57785", "FIPS": "46093", "County": "Meade", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57787", "FIPS": "46093", "County": "Meade", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57792", "FIPS": "46093", "County": "Meade", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57560", "FIPS": "46095", "County": "Mellette", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57579", "FIPS": "46095", "County": "Mellette", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57585", "FIPS": "46095", "County": "Mellette", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57321", "FIPS": "46097", "County": "Miner", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57323", "FIPS": "46097", "County": "Miner", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57337", "FIPS": "46097", "County": "Miner", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57349", "FIPS": "46097", "County": "Miner", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57003", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57005", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57018", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57020", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57022", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57030", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57033", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57035", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57041", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57055", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57056", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57068", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57101", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57103", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57104", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57105", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57106", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57107", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57109", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57110", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57117", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57118", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57186", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57188", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57189", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57192", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57193", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57194", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57195", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57196", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57197", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57198", "FIPS": "46099", "County": "Minnehaha", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57017", "FIPS": "46101", "County": "Moody", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57024", "FIPS": "46101", "County": "Moody", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57028", "FIPS": "46101", "County": "Moody", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57065", "FIPS": "46101", "County": "Moody", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57701", "FIPS": "46103", "County": "Pennington", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57702", "FIPS": "46103", "County": "Pennington", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57703", "FIPS": "46103", "County": "Pennington", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57709", "FIPS": "46103", "County": "Pennington", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57719", "FIPS": "46103", "County": "Pennington", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57725", "FIPS": "46103", "County": "Pennington", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57745", "FIPS": "46103", "County": "Pennington", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57751", "FIPS": "46103", "County": "Pennington", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57761", "FIPS": "46103", "County": "Pennington", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57767", "FIPS": "46103", "County": "Pennington", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57775", "FIPS": "46103", "County": "Pennington", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57780", "FIPS": "46103", "County": "Pennington", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57790", "FIPS": "46103", "County": "Pennington", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57791", "FIPS": "46103", "County": "Pennington", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57620", "FIPS": "46105", "County": "Perkins", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57638", "FIPS": "46105", "County": "Perkins", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57640", "FIPS": "46105", "County": "Perkins", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57644", "FIPS": "46105", "County": "Perkins", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57649", "FIPS": "46105", "County": "Perkins", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57442", "FIPS": "46107", "County": "Potter", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57450", "FIPS": "46107", "County": "Potter", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57455", "FIPS": "46107", "County": "Potter", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57475", "FIPS": "46107", "County": "Potter", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57224", "FIPS": "46109", "County": "Roberts", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57227", "FIPS": "46109", "County": "Roberts", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57255", "FIPS": "46109", "County": "Roberts", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57256", "FIPS": "46109", "County": "Roberts", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57257", "FIPS": "46109", "County": "Roberts", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57260", "FIPS": "46109", "County": "Roberts", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57262", "FIPS": "46109", "County": "Roberts", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57266", "FIPS": "46109", "County": "Roberts", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57279", "FIPS": "46109", "County": "Roberts", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57314", "FIPS": "46111", "County": "Sanborn", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57359", "FIPS": "46111", "County": "Sanborn", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57385", "FIPS": "46111", "County": "Sanborn", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57716", "FIPS": "46113", "County": "Shannon", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57752", "FIPS": "46113", "County": "Shannon", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57756", "FIPS": "46113", "County": "Shannon", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57764", "FIPS": "46113", "County": "Shannon", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57770", "FIPS": "46113", "County": "Shannon", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57772", "FIPS": "46113", "County": "Shannon", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57794", "FIPS": "46113", "County": "Shannon", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57424", "FIPS": "46115", "County": "Spink", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57429", "FIPS": "46115", "County": "Spink", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57434", "FIPS": "46115", "County": "Spink", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57436", "FIPS": "46115", "County": "Spink", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57440", "FIPS": "46115", "County": "Spink", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57461", "FIPS": "46115", "County": "Spink", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57465", "FIPS": "46115", "County": "Spink", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57469", "FIPS": "46115", "County": "Spink", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57476", "FIPS": "46115", "County": "Spink", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57477", "FIPS": "46115", "County": "Spink", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57532", "FIPS": "46117", "County": "Stanley", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57537", "FIPS": "46117", "County": "Stanley", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57520", "FIPS": "46119", "County": "Sully", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57564", "FIPS": "46119", "County": "Sully", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57555", "FIPS": "46121", "County": "Todd", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57563", "FIPS": "46121", "County": "Todd", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57566", "FIPS": "46121", "County": "Todd", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57570", "FIPS": "46121", "County": "Todd", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57572", "FIPS": "46121", "County": "Todd", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57528", "FIPS": "46123", "County": "Tripp", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57534", "FIPS": "46123", "County": "Tripp", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57541", "FIPS": "46123", "County": "Tripp", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57580", "FIPS": "46123", "County": "Tripp", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57584", "FIPS": "46123", "County": "Tripp", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57014", "FIPS": "46125", "County": "Turner", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57015", "FIPS": "46125", "County": "Turner", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57021", "FIPS": "46125", "County": "Turner", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57036", "FIPS": "46125", "County": "Turner", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57043", "FIPS": "46125", "County": "Turner", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57047", "FIPS": "46125", "County": "Turner", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57053", "FIPS": "46125", "County": "Turner", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57070", "FIPS": "46125", "County": "Turner", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57001", "FIPS": "46127", "County": "Union", "State": "SD", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "57004", "FIPS": "46127", "County": "Union", "State": "SD", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "57025", "FIPS": "46127", "County": "Union", "State": "SD", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "57038", "FIPS": "46127", "County": "Union", "State": "SD", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "57049", "FIPS": "46127", "County": "Union", "State": "SD", "DMA": {"Code": 624, "Name": "SIOUX CITY"}},
  {"Postal_Code": "57420", "FIPS": "46129", "County": "Walworth", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57452", "FIPS": "46129", "County": "Walworth", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57472", "FIPS": "46129", "County": "Walworth", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57601", "FIPS": "46129", "County": "Walworth", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57631", "FIPS": "46129", "County": "Walworth", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57031", "FIPS": "46135", "County": "Yankton", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57040", "FIPS": "46135", "County": "Yankton", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57046", "FIPS": "46135", "County": "Yankton", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57067", "FIPS": "46135", "County": "Yankton", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57072", "FIPS": "46135", "County": "Yankton", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57078", "FIPS": "46135", "County": "Yankton", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57079", "FIPS": "46135", "County": "Yankton", "State": "SD", "DMA": {"Code": 725, "Name": "SIOUX FALLS (MITCHELL  )"}},
  {"Postal_Code": "57622", "FIPS": "46137", "County": "Ziebach", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "57623", "FIPS": "46137", "County": "Ziebach", "State": "SD", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "37705", "FIPS": "47001", "County": "Anderson", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37710", "FIPS": "47001", "County": "Anderson", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37716", "FIPS": "47001", "County": "Anderson", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37717", "FIPS": "47001", "County": "Anderson", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37769", "FIPS": "47001", "County": "Anderson", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37828", "FIPS": "47001", "County": "Anderson", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37830", "FIPS": "47001", "County": "Anderson", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37831", "FIPS": "47001", "County": "Anderson", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37020", "FIPS": "47003", "County": "Bedford", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37160", "FIPS": "47003", "County": "Bedford", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37161", "FIPS": "47003", "County": "Bedford", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37162", "FIPS": "47003", "County": "Bedford", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37180", "FIPS": "47003", "County": "Bedford", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37183", "FIPS": "47003", "County": "Bedford", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37360", "FIPS": "47003", "County": "Bedford", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38221", "FIPS": "47005", "County": "Benton", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38320", "FIPS": "47005", "County": "Benton", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38333", "FIPS": "47005", "County": "Benton", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38341", "FIPS": "47005", "County": "Benton", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37367", "FIPS": "47007", "County": "Bledsoe", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37701", "FIPS": "47009", "County": "Blount", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37737", "FIPS": "47009", "County": "Blount", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37777", "FIPS": "47009", "County": "Blount", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37801", "FIPS": "47009", "County": "Blount", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37802", "FIPS": "47009", "County": "Blount", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37803", "FIPS": "47009", "County": "Blount", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37804", "FIPS": "47009", "County": "Blount", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37853", "FIPS": "47009", "County": "Blount", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37878", "FIPS": "47009", "County": "Blount", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37882", "FIPS": "47009", "County": "Blount", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37886", "FIPS": "47009", "County": "Blount", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37310", "FIPS": "47011", "County": "Bradley", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37311", "FIPS": "47011", "County": "Bradley", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37312", "FIPS": "47011", "County": "Bradley", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37320", "FIPS": "47011", "County": "Bradley", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37323", "FIPS": "47011", "County": "Bradley", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37353", "FIPS": "47011", "County": "Bradley", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37364", "FIPS": "47011", "County": "Bradley", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37714", "FIPS": "47013", "County": "Campbell", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37729", "FIPS": "47013", "County": "Campbell", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37757", "FIPS": "47013", "County": "Campbell", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37762", "FIPS": "47013", "County": "Campbell", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37766", "FIPS": "47013", "County": "Campbell", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37819", "FIPS": "47013", "County": "Campbell", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37847", "FIPS": "47013", "County": "Campbell", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37016", "FIPS": "47015", "County": "Cannon", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37026", "FIPS": "47015", "County": "Cannon", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37149", "FIPS": "47015", "County": "Cannon", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37190", "FIPS": "47015", "County": "Cannon", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38201", "FIPS": "47017", "County": "Carroll", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38220", "FIPS": "47017", "County": "Carroll", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38235", "FIPS": "47017", "County": "Carroll", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38258", "FIPS": "47017", "County": "Carroll", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38317", "FIPS": "47017", "County": "Carroll", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38318", "FIPS": "47017", "County": "Carroll", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38321", "FIPS": "47017", "County": "Carroll", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38324", "FIPS": "47017", "County": "Carroll", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38342", "FIPS": "47017", "County": "Carroll", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38344", "FIPS": "47017", "County": "Carroll", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38348", "FIPS": "47017", "County": "Carroll", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38387", "FIPS": "47017", "County": "Carroll", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38390", "FIPS": "47017", "County": "Carroll", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "37643", "FIPS": "47019", "County": "Carter", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37644", "FIPS": "47019", "County": "Carter", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37658", "FIPS": "47019", "County": "Carter", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37682", "FIPS": "47019", "County": "Carter", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37687", "FIPS": "47019", "County": "Carter", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37694", "FIPS": "47019", "County": "Carter", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37015", "FIPS": "47021", "County": "Cheatham", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37035", "FIPS": "47021", "County": "Cheatham", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37082", "FIPS": "47021", "County": "Cheatham", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37143", "FIPS": "47021", "County": "Cheatham", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37146", "FIPS": "47021", "County": "Cheatham", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38332", "FIPS": "47023", "County": "Chester", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38340", "FIPS": "47023", "County": "Chester", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38347", "FIPS": "47023", "County": "Chester", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38352", "FIPS": "47023", "County": "Chester", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "37707", "FIPS": "47025", "County": "Claiborne", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37715", "FIPS": "47025", "County": "Claiborne", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37724", "FIPS": "47025", "County": "Claiborne", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37730", "FIPS": "47025", "County": "Claiborne", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37752", "FIPS": "47025", "County": "Claiborne", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37773", "FIPS": "47025", "County": "Claiborne", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37824", "FIPS": "47025", "County": "Claiborne", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37825", "FIPS": "47025", "County": "Claiborne", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37851", "FIPS": "47025", "County": "Claiborne", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37867", "FIPS": "47025", "County": "Claiborne", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37870", "FIPS": "47025", "County": "Claiborne", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37879", "FIPS": "47025", "County": "Claiborne", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "38551", "FIPS": "47027", "County": "Clay", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38575", "FIPS": "47027", "County": "Clay", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37713", "FIPS": "47029", "County": "Cocke", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37722", "FIPS": "47029", "County": "Cocke", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37727", "FIPS": "47029", "County": "Cocke", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37753", "FIPS": "47029", "County": "Cocke", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37821", "FIPS": "47029", "County": "Cocke", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37822", "FIPS": "47029", "County": "Cocke", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37843", "FIPS": "47029", "County": "Cocke", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37018", "FIPS": "47031", "County": "Coffee", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37342", "FIPS": "47031", "County": "Coffee", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37349", "FIPS": "47031", "County": "Coffee", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37355", "FIPS": "47031", "County": "Coffee", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37382", "FIPS": "47031", "County": "Coffee", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37388", "FIPS": "47031", "County": "Coffee", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37389", "FIPS": "47031", "County": "Coffee", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38001", "FIPS": "47033", "County": "Crockett", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38006", "FIPS": "47033", "County": "Crockett", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38021", "FIPS": "47033", "County": "Crockett", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38034", "FIPS": "47033", "County": "Crockett", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38050", "FIPS": "47033", "County": "Crockett", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38336", "FIPS": "47033", "County": "Crockett", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38337", "FIPS": "47033", "County": "Crockett", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "37723", "FIPS": "47035", "County": "Cumberland", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "38555", "FIPS": "47035", "County": "Cumberland", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "38557", "FIPS": "47035", "County": "Cumberland", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "38558", "FIPS": "47035", "County": "Cumberland", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "38571", "FIPS": "47035", "County": "Cumberland", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "38572", "FIPS": "47035", "County": "Cumberland", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "38578", "FIPS": "47035", "County": "Cumberland", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37011", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37013", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37070", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37072", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37076", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37080", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37115", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37116", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37138", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37189", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37201", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37202", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37203", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37204", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37205", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37206", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37207", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37208", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37209", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37210", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37211", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37212", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37213", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37214", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37215", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37216", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37217", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37218", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37219", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37220", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37221", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37222", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37224", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37227", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37228", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37229", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37230", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37232", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37234", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37235", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37236", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37237", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37238", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37240", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37241", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37242", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37243", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37244", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37245", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37246", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37247", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37248", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37249", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37250", "FIPS": "47037", "County": "Davidson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38311", "FIPS": "47039", "County": "Decatur", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38329", "FIPS": "47039", "County": "Decatur", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38363", "FIPS": "47039", "County": "Decatur", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38374", "FIPS": "47039", "County": "Decatur", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38380", "FIPS": "47039", "County": "Decatur", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37012", "FIPS": "47041", "County": "Dekalb", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37059", "FIPS": "47041", "County": "Dekalb", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37095", "FIPS": "47041", "County": "Dekalb", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37166", "FIPS": "47041", "County": "Dekalb", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37029", "FIPS": "47043", "County": "Dickson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37036", "FIPS": "47043", "County": "Dickson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37051", "FIPS": "47043", "County": "Dickson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37055", "FIPS": "47043", "County": "Dickson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37056", "FIPS": "47043", "County": "Dickson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37165", "FIPS": "47043", "County": "Dickson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37181", "FIPS": "47043", "County": "Dickson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37187", "FIPS": "47043", "County": "Dickson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38007", "FIPS": "47045", "County": "Dyer", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38024", "FIPS": "47045", "County": "Dyer", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38025", "FIPS": "47045", "County": "Dyer", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38030", "FIPS": "47045", "County": "Dyer", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38047", "FIPS": "47045", "County": "Dyer", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38059", "FIPS": "47045", "County": "Dyer", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38070", "FIPS": "47045", "County": "Dyer", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38259", "FIPS": "47045", "County": "Dyer", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38010", "FIPS": "47047", "County": "Fayette", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38036", "FIPS": "47047", "County": "Fayette", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38045", "FIPS": "47047", "County": "Fayette", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38046", "FIPS": "47047", "County": "Fayette", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38048", "FIPS": "47047", "County": "Fayette", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38057", "FIPS": "47047", "County": "Fayette", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38060", "FIPS": "47047", "County": "Fayette", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38066", "FIPS": "47047", "County": "Fayette", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38068", "FIPS": "47047", "County": "Fayette", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38076", "FIPS": "47047", "County": "Fayette", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38504", "FIPS": "47049", "County": "Fentress", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "38553", "FIPS": "47049", "County": "Fentress", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "38556", "FIPS": "47049", "County": "Fentress", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "38565", "FIPS": "47049", "County": "Fentress", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "38577", "FIPS": "47049", "County": "Fentress", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37306", "FIPS": "47051", "County": "Franklin", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37318", "FIPS": "47051", "County": "Franklin", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37324", "FIPS": "47051", "County": "Franklin", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37330", "FIPS": "47051", "County": "Franklin", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37345", "FIPS": "47051", "County": "Franklin", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37375", "FIPS": "47051", "County": "Franklin", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37376", "FIPS": "47051", "County": "Franklin", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37383", "FIPS": "47051", "County": "Franklin", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37398", "FIPS": "47051", "County": "Franklin", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38233", "FIPS": "47053", "County": "Gibson", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38316", "FIPS": "47053", "County": "Gibson", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38330", "FIPS": "47053", "County": "Gibson", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38331", "FIPS": "47053", "County": "Gibson", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38338", "FIPS": "47053", "County": "Gibson", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38343", "FIPS": "47053", "County": "Gibson", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38346", "FIPS": "47053", "County": "Gibson", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38355", "FIPS": "47053", "County": "Gibson", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38358", "FIPS": "47053", "County": "Gibson", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38369", "FIPS": "47053", "County": "Gibson", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38382", "FIPS": "47053", "County": "Gibson", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38389", "FIPS": "47053", "County": "Gibson", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38449", "FIPS": "47055", "County": "Giles", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38455", "FIPS": "47055", "County": "Giles", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38460", "FIPS": "47055", "County": "Giles", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38472", "FIPS": "47055", "County": "Giles", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38473", "FIPS": "47055", "County": "Giles", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38477", "FIPS": "47055", "County": "Giles", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38478", "FIPS": "47055", "County": "Giles", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37708", "FIPS": "47057", "County": "Grainger", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37709", "FIPS": "47057", "County": "Grainger", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37848", "FIPS": "47057", "County": "Grainger", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37861", "FIPS": "47057", "County": "Grainger", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37881", "FIPS": "47057", "County": "Grainger", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37888", "FIPS": "47057", "County": "Grainger", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37616", "FIPS": "47059", "County": "Greene", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37641", "FIPS": "47059", "County": "Greene", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37743", "FIPS": "47059", "County": "Greene", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37744", "FIPS": "47059", "County": "Greene", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37745", "FIPS": "47059", "County": "Greene", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37809", "FIPS": "47059", "County": "Greene", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37810", "FIPS": "47059", "County": "Greene", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37818", "FIPS": "47059", "County": "Greene", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37301", "FIPS": "47061", "County": "Grundy", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37305", "FIPS": "47061", "County": "Grundy", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37313", "FIPS": "47061", "County": "Grundy", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37339", "FIPS": "47061", "County": "Grundy", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37356", "FIPS": "47061", "County": "Grundy", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37365", "FIPS": "47061", "County": "Grundy", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37366", "FIPS": "47061", "County": "Grundy", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37387", "FIPS": "47061", "County": "Grundy", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37778", "FIPS": "47063", "County": "Hamblen", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37813", "FIPS": "47063", "County": "Hamblen", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37814", "FIPS": "47063", "County": "Hamblen", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37815", "FIPS": "47063", "County": "Hamblen", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37816", "FIPS": "47063", "County": "Hamblen", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37860", "FIPS": "47063", "County": "Hamblen", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37877", "FIPS": "47063", "County": "Hamblen", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37891", "FIPS": "47063", "County": "Hamblen", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37302", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37304", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37308", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37315", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37341", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37343", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37350", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37351", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37363", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37373", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37377", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37379", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37384", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37401", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37402", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37403", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37404", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37405", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37406", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37407", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37408", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37409", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37410", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37411", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37412", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37414", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37415", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37416", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37419", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37421", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37422", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37424", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37450", "FIPS": "47065", "County": "Hamilton", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37765", "FIPS": "47067", "County": "Hancock", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37869", "FIPS": "47067", "County": "Hancock", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "38008", "FIPS": "47069", "County": "Hardeman", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38039", "FIPS": "47069", "County": "Hardeman", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38042", "FIPS": "47069", "County": "Hardeman", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38044", "FIPS": "47069", "County": "Hardeman", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38052", "FIPS": "47069", "County": "Hardeman", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38061", "FIPS": "47069", "County": "Hardeman", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38067", "FIPS": "47069", "County": "Hardeman", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38075", "FIPS": "47069", "County": "Hardeman", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38377", "FIPS": "47069", "County": "Hardeman", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38381", "FIPS": "47069", "County": "Hardeman", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38326", "FIPS": "47071", "County": "Hardin", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38327", "FIPS": "47071", "County": "Hardin", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38361", "FIPS": "47071", "County": "Hardin", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38365", "FIPS": "47071", "County": "Hardin", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38370", "FIPS": "47071", "County": "Hardin", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38372", "FIPS": "47071", "County": "Hardin", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38376", "FIPS": "47071", "County": "Hardin", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38475", "FIPS": "47071", "County": "Hardin", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "37642", "FIPS": "47073", "County": "Hawkins", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37645", "FIPS": "47073", "County": "Hawkins", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37711", "FIPS": "47073", "County": "Hawkins", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37731", "FIPS": "47073", "County": "Hawkins", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37811", "FIPS": "47073", "County": "Hawkins", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37857", "FIPS": "47073", "County": "Hawkins", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37873", "FIPS": "47073", "County": "Hawkins", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "38012", "FIPS": "47075", "County": "Haywood", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38069", "FIPS": "47075", "County": "Haywood", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38328", "FIPS": "47077", "County": "Henderson", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38345", "FIPS": "47077", "County": "Henderson", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38351", "FIPS": "47077", "County": "Henderson", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38368", "FIPS": "47077", "County": "Henderson", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38371", "FIPS": "47077", "County": "Henderson", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38388", "FIPS": "47077", "County": "Henderson", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38222", "FIPS": "47079", "County": "Henry", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38223", "FIPS": "47079", "County": "Henry", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38224", "FIPS": "47079", "County": "Henry", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38231", "FIPS": "47079", "County": "Henry", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38236", "FIPS": "47079", "County": "Henry", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38242", "FIPS": "47079", "County": "Henry", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38251", "FIPS": "47079", "County": "Henry", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38256", "FIPS": "47079", "County": "Henry", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37025", "FIPS": "47081", "County": "Hickman", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37033", "FIPS": "47081", "County": "Hickman", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37098", "FIPS": "47081", "County": "Hickman", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37137", "FIPS": "47081", "County": "Hickman", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37140", "FIPS": "47081", "County": "Hickman", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38454", "FIPS": "47081", "County": "Hickman", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37061", "FIPS": "47083", "County": "Houston", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37175", "FIPS": "47083", "County": "Houston", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37178", "FIPS": "47083", "County": "Houston", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37078", "FIPS": "47085", "County": "Humphreys", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37101", "FIPS": "47085", "County": "Humphreys", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37134", "FIPS": "47085", "County": "Humphreys", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37185", "FIPS": "47085", "County": "Humphreys", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38562", "FIPS": "47087", "County": "Jackson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38564", "FIPS": "47087", "County": "Jackson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38588", "FIPS": "47087", "County": "Jackson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37725", "FIPS": "47089", "County": "Jefferson", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37760", "FIPS": "47089", "County": "Jefferson", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37820", "FIPS": "47089", "County": "Jefferson", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37871", "FIPS": "47089", "County": "Jefferson", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37890", "FIPS": "47089", "County": "Jefferson", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37640", "FIPS": "47091", "County": "Johnson", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37680", "FIPS": "47091", "County": "Johnson", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37683", "FIPS": "47091", "County": "Johnson", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37688", "FIPS": "47091", "County": "Johnson", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37691", "FIPS": "47091", "County": "Johnson", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37721", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37754", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37806", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37849", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37901", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37902", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37909", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37912", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37914", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37915", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37916", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37917", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37918", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37919", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37920", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37921", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37922", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37923", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37924", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37927", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37928", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37929", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37930", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37931", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37932", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37933", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37934", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37938", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37939", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37940", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37950", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37990", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37995", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37996", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37997", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37998", "FIPS": "47093", "County": "Knox", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "38077", "FIPS": "47095", "County": "Lake", "State": "TN", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "38079", "FIPS": "47095", "County": "Lake", "State": "TN", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "38080", "FIPS": "47095", "County": "Lake", "State": "TN", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "38037", "FIPS": "47097", "County": "Lauderdale", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38040", "FIPS": "47097", "County": "Lauderdale", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38041", "FIPS": "47097", "County": "Lauderdale", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38063", "FIPS": "47097", "County": "Lauderdale", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38456", "FIPS": "47099", "County": "Lawrence", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38457", "FIPS": "47099", "County": "Lawrence", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38463", "FIPS": "47099", "County": "Lawrence", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38464", "FIPS": "47099", "County": "Lawrence", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38468", "FIPS": "47099", "County": "Lawrence", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38469", "FIPS": "47099", "County": "Lawrence", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38481", "FIPS": "47099", "County": "Lawrence", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38483", "FIPS": "47099", "County": "Lawrence", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38486", "FIPS": "47099", "County": "Lawrence", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38462", "FIPS": "47101", "County": "Lewis", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37144", "FIPS": "47103", "County": "Lincoln", "State": "TN", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "37328", "FIPS": "47103", "County": "Lincoln", "State": "TN", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "37334", "FIPS": "47103", "County": "Lincoln", "State": "TN", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "37335", "FIPS": "47103", "County": "Lincoln", "State": "TN", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "37348", "FIPS": "47103", "County": "Lincoln", "State": "TN", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "37359", "FIPS": "47103", "County": "Lincoln", "State": "TN", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "38453", "FIPS": "47103", "County": "Lincoln", "State": "TN", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "38459", "FIPS": "47103", "County": "Lincoln", "State": "TN", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "38488", "FIPS": "47103", "County": "Lincoln", "State": "TN", "DMA": {"Code": 691, "Name": "HUNTSVILLE-DECATUR-FLORENCE"}},
  {"Postal_Code": "37742", "FIPS": "47105", "County": "Loudon", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37771", "FIPS": "47105", "County": "Loudon", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37772", "FIPS": "47105", "County": "Loudon", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37774", "FIPS": "47105", "County": "Loudon", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37846", "FIPS": "47105", "County": "Loudon", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37303", "FIPS": "47107", "County": "McMinn", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37309", "FIPS": "47107", "County": "McMinn", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37329", "FIPS": "47107", "County": "McMinn", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37331", "FIPS": "47107", "County": "McMinn", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37370", "FIPS": "47107", "County": "McMinn", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37371", "FIPS": "47107", "County": "McMinn", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37826", "FIPS": "47107", "County": "McMinn", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "38310", "FIPS": "47109", "County": "McNairy", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38315", "FIPS": "47109", "County": "McNairy", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38334", "FIPS": "47109", "County": "McNairy", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38339", "FIPS": "47109", "County": "McNairy", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38357", "FIPS": "47109", "County": "McNairy", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38359", "FIPS": "47109", "County": "McNairy", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38367", "FIPS": "47109", "County": "McNairy", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38375", "FIPS": "47109", "County": "McNairy", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38379", "FIPS": "47109", "County": "McNairy", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38393", "FIPS": "47109", "County": "McNairy", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "37083", "FIPS": "47111", "County": "Macon", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37150", "FIPS": "47111", "County": "Macon", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38301", "FIPS": "47113", "County": "Madison", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38302", "FIPS": "47113", "County": "Madison", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38303", "FIPS": "47113", "County": "Madison", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38305", "FIPS": "47113", "County": "Madison", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38308", "FIPS": "47113", "County": "Madison", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38313", "FIPS": "47113", "County": "Madison", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38314", "FIPS": "47113", "County": "Madison", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38356", "FIPS": "47113", "County": "Madison", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38362", "FIPS": "47113", "County": "Madison", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38366", "FIPS": "47113", "County": "Madison", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38378", "FIPS": "47113", "County": "Madison", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38391", "FIPS": "47113", "County": "Madison", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "38392", "FIPS": "47113", "County": "Madison", "State": "TN", "DMA": {"Code": 639, "Name": "JACKSON, TN"}},
  {"Postal_Code": "37340", "FIPS": "47115", "County": "Marion", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37347", "FIPS": "47115", "County": "Marion", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37374", "FIPS": "47115", "County": "Marion", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37380", "FIPS": "47115", "County": "Marion", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37396", "FIPS": "47115", "County": "Marion", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37397", "FIPS": "47115", "County": "Marion", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37019", "FIPS": "47117", "County": "Marshall", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37034", "FIPS": "47117", "County": "Marshall", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37047", "FIPS": "47117", "County": "Marshall", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37091", "FIPS": "47117", "County": "Marshall", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37174", "FIPS": "47119", "County": "Maury", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38401", "FIPS": "47119", "County": "Maury", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38402", "FIPS": "47119", "County": "Maury", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38451", "FIPS": "47119", "County": "Maury", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38461", "FIPS": "47119", "County": "Maury", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38474", "FIPS": "47119", "County": "Maury", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38482", "FIPS": "47119", "County": "Maury", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38487", "FIPS": "47119", "County": "Maury", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37322", "FIPS": "47121", "County": "Meigs", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37336", "FIPS": "47121", "County": "Meigs", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37880", "FIPS": "47121", "County": "Meigs", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37314", "FIPS": "47123", "County": "Monroe", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37354", "FIPS": "47123", "County": "Monroe", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37385", "FIPS": "47123", "County": "Monroe", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37874", "FIPS": "47123", "County": "Monroe", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37885", "FIPS": "47123", "County": "Monroe", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37040", "FIPS": "47125", "County": "Montgomery", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37041", "FIPS": "47125", "County": "Montgomery", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37042", "FIPS": "47125", "County": "Montgomery", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37043", "FIPS": "47125", "County": "Montgomery", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37044", "FIPS": "47125", "County": "Montgomery", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37052", "FIPS": "47125", "County": "Montgomery", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37142", "FIPS": "47125", "County": "Montgomery", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37171", "FIPS": "47125", "County": "Montgomery", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37191", "FIPS": "47125", "County": "Montgomery", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37352", "FIPS": "47127", "County": "Moore", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37719", "FIPS": "47129", "County": "Morgan", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37726", "FIPS": "47129", "County": "Morgan", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37733", "FIPS": "47129", "County": "Morgan", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37770", "FIPS": "47129", "County": "Morgan", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37829", "FIPS": "47129", "County": "Morgan", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37845", "FIPS": "47129", "County": "Morgan", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37872", "FIPS": "47129", "County": "Morgan", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37887", "FIPS": "47129", "County": "Morgan", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "38232", "FIPS": "47131", "County": "Obion", "State": "TN", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "38240", "FIPS": "47131", "County": "Obion", "State": "TN", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "38253", "FIPS": "47131", "County": "Obion", "State": "TN", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "38254", "FIPS": "47131", "County": "Obion", "State": "TN", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "38257", "FIPS": "47131", "County": "Obion", "State": "TN", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "38260", "FIPS": "47131", "County": "Obion", "State": "TN", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "38261", "FIPS": "47131", "County": "Obion", "State": "TN", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "38271", "FIPS": "47131", "County": "Obion", "State": "TN", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "38281", "FIPS": "47131", "County": "Obion", "State": "TN", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "38541", "FIPS": "47133", "County": "Overton", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38542", "FIPS": "47133", "County": "Overton", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38543", "FIPS": "47133", "County": "Overton", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38554", "FIPS": "47133", "County": "Overton", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38568", "FIPS": "47133", "County": "Overton", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38570", "FIPS": "47133", "County": "Overton", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38573", "FIPS": "47133", "County": "Overton", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38580", "FIPS": "47133", "County": "Overton", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38589", "FIPS": "47133", "County": "Overton", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37096", "FIPS": "47135", "County": "Perry", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37097", "FIPS": "47135", "County": "Perry", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38549", "FIPS": "47137", "County": "Pickett", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37307", "FIPS": "47139", "County": "Polk", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37316", "FIPS": "47139", "County": "Polk", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37317", "FIPS": "47139", "County": "Polk", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37325", "FIPS": "47139", "County": "Polk", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37326", "FIPS": "47139", "County": "Polk", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37333", "FIPS": "47139", "County": "Polk", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37361", "FIPS": "47139", "County": "Polk", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37362", "FIPS": "47139", "County": "Polk", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37369", "FIPS": "47139", "County": "Polk", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37391", "FIPS": "47139", "County": "Polk", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "38501", "FIPS": "47141", "County": "Putnam", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38502", "FIPS": "47141", "County": "Putnam", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38503", "FIPS": "47141", "County": "Putnam", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38505", "FIPS": "47141", "County": "Putnam", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38506", "FIPS": "47141", "County": "Putnam", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38544", "FIPS": "47141", "County": "Putnam", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38545", "FIPS": "47141", "County": "Putnam", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38548", "FIPS": "47141", "County": "Putnam", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38574", "FIPS": "47141", "County": "Putnam", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38582", "FIPS": "47141", "County": "Putnam", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37321", "FIPS": "47143", "County": "Rhea", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37332", "FIPS": "47143", "County": "Rhea", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37337", "FIPS": "47143", "County": "Rhea", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37338", "FIPS": "47143", "County": "Rhea", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37381", "FIPS": "47143", "County": "Rhea", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37748", "FIPS": "47145", "County": "Roane", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37763", "FIPS": "47145", "County": "Roane", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37840", "FIPS": "47145", "County": "Roane", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37854", "FIPS": "47145", "County": "Roane", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37010", "FIPS": "47147", "County": "Robertson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37032", "FIPS": "47147", "County": "Robertson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37049", "FIPS": "47147", "County": "Robertson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37073", "FIPS": "47147", "County": "Robertson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37141", "FIPS": "47147", "County": "Robertson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37152", "FIPS": "47147", "County": "Robertson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37172", "FIPS": "47147", "County": "Robertson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37188", "FIPS": "47147", "County": "Robertson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37037", "FIPS": "47149", "County": "Rutherford", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37060", "FIPS": "47149", "County": "Rutherford", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37063", "FIPS": "47149", "County": "Rutherford", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37085", "FIPS": "47149", "County": "Rutherford", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37086", "FIPS": "47149", "County": "Rutherford", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37089", "FIPS": "47149", "County": "Rutherford", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37118", "FIPS": "47149", "County": "Rutherford", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37127", "FIPS": "47149", "County": "Rutherford", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37128", "FIPS": "47149", "County": "Rutherford", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37129", "FIPS": "47149", "County": "Rutherford", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37130", "FIPS": "47149", "County": "Rutherford", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37131", "FIPS": "47149", "County": "Rutherford", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37132", "FIPS": "47149", "County": "Rutherford", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37133", "FIPS": "47149", "County": "Rutherford", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37153", "FIPS": "47149", "County": "Rutherford", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37167", "FIPS": "47149", "County": "Rutherford", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37732", "FIPS": "47151", "County": "Scott", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37755", "FIPS": "47151", "County": "Scott", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37756", "FIPS": "47151", "County": "Scott", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37841", "FIPS": "47151", "County": "Scott", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37852", "FIPS": "47151", "County": "Scott", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37892", "FIPS": "47151", "County": "Scott", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37327", "FIPS": "47153", "County": "Sequatchie", "State": "TN", "DMA": {"Code": 575, "Name": "CHATTANOOGA"}},
  {"Postal_Code": "37738", "FIPS": "47155", "County": "Sevier", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37764", "FIPS": "47155", "County": "Sevier", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37862", "FIPS": "47155", "County": "Sevier", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37863", "FIPS": "47155", "County": "Sevier", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37864", "FIPS": "47155", "County": "Sevier", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37865", "FIPS": "47155", "County": "Sevier", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37868", "FIPS": "47155", "County": "Sevier", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37876", "FIPS": "47155", "County": "Sevier", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37501", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "37544", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38002", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38014", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38016", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38017", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38018", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38027", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38028", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38029", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38053", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38054", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38055", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38083", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38088", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38101", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38103", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38104", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38105", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38106", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38107", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38108", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38109", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38110", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38111", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38112", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38113", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38114", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38115", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38116", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38117", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38118", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38119", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38120", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38122", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38124", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38125", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38126", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38127", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38128", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38130", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38131", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38132", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38133", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38134", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38135", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38136", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38137", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38138", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38139", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38141", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38142", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38145", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38147", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38148", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38150", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38151", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38152", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38157", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38159", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38161", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38163", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38165", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38166", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38167", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38168", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38173", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38174", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38175", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38177", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38181", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38182", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38183", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38184", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38186", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38187", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38188", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38190", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38193", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38194", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38197", "FIPS": "47157", "County": "Shelby", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "37030", "FIPS": "47159", "County": "Smith", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37057", "FIPS": "47159", "County": "Smith", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37145", "FIPS": "47159", "County": "Smith", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37151", "FIPS": "47159", "County": "Smith", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38547", "FIPS": "47159", "County": "Smith", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38552", "FIPS": "47159", "County": "Smith", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38560", "FIPS": "47159", "County": "Smith", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38563", "FIPS": "47159", "County": "Smith", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38567", "FIPS": "47159", "County": "Smith", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38569", "FIPS": "47159", "County": "Smith", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37023", "FIPS": "47161", "County": "Stewart", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37028", "FIPS": "47161", "County": "Stewart", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37050", "FIPS": "47161", "County": "Stewart", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37058", "FIPS": "47161", "County": "Stewart", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37079", "FIPS": "47161", "County": "Stewart", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37617", "FIPS": "47163", "County": "Sullivan", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37618", "FIPS": "47163", "County": "Sullivan", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37620", "FIPS": "47163", "County": "Sullivan", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37621", "FIPS": "47163", "County": "Sullivan", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37625", "FIPS": "47163", "County": "Sullivan", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37660", "FIPS": "47163", "County": "Sullivan", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37662", "FIPS": "47163", "County": "Sullivan", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37663", "FIPS": "47163", "County": "Sullivan", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37664", "FIPS": "47163", "County": "Sullivan", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37665", "FIPS": "47163", "County": "Sullivan", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37669", "FIPS": "47163", "County": "Sullivan", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37686", "FIPS": "47163", "County": "Sullivan", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37699", "FIPS": "47163", "County": "Sullivan", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37022", "FIPS": "47165", "County": "Sumner", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37031", "FIPS": "47165", "County": "Sumner", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37048", "FIPS": "47165", "County": "Sumner", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37066", "FIPS": "47165", "County": "Sumner", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37075", "FIPS": "47165", "County": "Sumner", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37077", "FIPS": "47165", "County": "Sumner", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37119", "FIPS": "47165", "County": "Sumner", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37148", "FIPS": "47165", "County": "Sumner", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37186", "FIPS": "47165", "County": "Sumner", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38004", "FIPS": "47167", "County": "Tipton", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38011", "FIPS": "47167", "County": "Tipton", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38015", "FIPS": "47167", "County": "Tipton", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38019", "FIPS": "47167", "County": "Tipton", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38023", "FIPS": "47167", "County": "Tipton", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38049", "FIPS": "47167", "County": "Tipton", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38058", "FIPS": "47167", "County": "Tipton", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "38071", "FIPS": "47167", "County": "Tipton", "State": "TN", "DMA": {"Code": 640, "Name": "MEMPHIS"}},
  {"Postal_Code": "37074", "FIPS": "47169", "County": "Trousdale", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37650", "FIPS": "47171", "County": "Unicoi", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37657", "FIPS": "47171", "County": "Unicoi", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37692", "FIPS": "47171", "County": "Unicoi", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37779", "FIPS": "47173", "County": "Union", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37807", "FIPS": "47173", "County": "Union", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "37866", "FIPS": "47173", "County": "Union", "State": "TN", "DMA": {"Code": 557, "Name": "KNOXVILLE"}},
  {"Postal_Code": "38585", "FIPS": "47175", "County": "Van Buren", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37110", "FIPS": "47177", "County": "Warren", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37111", "FIPS": "47177", "County": "Warren", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37357", "FIPS": "47177", "County": "Warren", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37378", "FIPS": "47177", "County": "Warren", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37394", "FIPS": "47177", "County": "Warren", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38550", "FIPS": "47177", "County": "Warren", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38581", "FIPS": "47177", "County": "Warren", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37601", "FIPS": "47179", "County": "Washington", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37602", "FIPS": "47179", "County": "Washington", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37604", "FIPS": "47179", "County": "Washington", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37605", "FIPS": "47179", "County": "Washington", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37614", "FIPS": "47179", "County": "Washington", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37615", "FIPS": "47179", "County": "Washington", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37656", "FIPS": "47179", "County": "Washington", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37659", "FIPS": "47179", "County": "Washington", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37681", "FIPS": "47179", "County": "Washington", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37684", "FIPS": "47179", "County": "Washington", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "37690", "FIPS": "47179", "County": "Washington", "State": "TN", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "38425", "FIPS": "47181", "County": "Wayne", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38450", "FIPS": "47181", "County": "Wayne", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38452", "FIPS": "47181", "County": "Wayne", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38471", "FIPS": "47181", "County": "Wayne", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38485", "FIPS": "47181", "County": "Wayne", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38225", "FIPS": "47183", "County": "Weakley", "State": "TN", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "38226", "FIPS": "47183", "County": "Weakley", "State": "TN", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "38229", "FIPS": "47183", "County": "Weakley", "State": "TN", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "38230", "FIPS": "47183", "County": "Weakley", "State": "TN", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "38237", "FIPS": "47183", "County": "Weakley", "State": "TN", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "38238", "FIPS": "47183", "County": "Weakley", "State": "TN", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "38241", "FIPS": "47183", "County": "Weakley", "State": "TN", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "38255", "FIPS": "47183", "County": "Weakley", "State": "TN", "DMA": {"Code": 632, "Name": "PADUCAH-CAPE GIRARDEAU-HARRISBURG"}},
  {"Postal_Code": "38559", "FIPS": "47185", "County": "White", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38579", "FIPS": "47185", "County": "White", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38583", "FIPS": "47185", "County": "White", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38587", "FIPS": "47185", "County": "White", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37014", "FIPS": "47187", "County": "Williamson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37024", "FIPS": "47187", "County": "Williamson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37027", "FIPS": "47187", "County": "Williamson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37046", "FIPS": "47187", "County": "Williamson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37062", "FIPS": "47187", "County": "Williamson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37064", "FIPS": "47187", "County": "Williamson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37065", "FIPS": "47187", "County": "Williamson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37067", "FIPS": "47187", "County": "Williamson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37068", "FIPS": "47187", "County": "Williamson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37069", "FIPS": "47187", "County": "Williamson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37135", "FIPS": "47187", "County": "Williamson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37179", "FIPS": "47187", "County": "Williamson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "38476", "FIPS": "47187", "County": "Williamson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37071", "FIPS": "47189", "County": "Wilson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37087", "FIPS": "47189", "County": "Wilson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37088", "FIPS": "47189", "County": "Wilson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37090", "FIPS": "47189", "County": "Wilson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37121", "FIPS": "47189", "County": "Wilson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37122", "FIPS": "47189", "County": "Wilson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37136", "FIPS": "47189", "County": "Wilson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "37184", "FIPS": "47189", "County": "Wilson", "State": "TN", "DMA": {"Code": 659, "Name": "NASHVILLE"}},
  {"Postal_Code": "75763", "FIPS": "48001", "County": "Anderson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75779", "FIPS": "48001", "County": "Anderson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75801", "FIPS": "48001", "County": "Anderson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75802", "FIPS": "48001", "County": "Anderson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75803", "FIPS": "48001", "County": "Anderson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75832", "FIPS": "48001", "County": "Anderson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75839", "FIPS": "48001", "County": "Anderson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75853", "FIPS": "48001", "County": "Anderson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75861", "FIPS": "48001", "County": "Anderson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75880", "FIPS": "48001", "County": "Anderson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75882", "FIPS": "48001", "County": "Anderson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75884", "FIPS": "48001", "County": "Anderson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75886", "FIPS": "48001", "County": "Anderson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "79714", "FIPS": "48003", "County": "Andrews", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "75901", "FIPS": "48005", "County": "Angelina", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75902", "FIPS": "48005", "County": "Angelina", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75903", "FIPS": "48005", "County": "Angelina", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75904", "FIPS": "48005", "County": "Angelina", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75915", "FIPS": "48005", "County": "Angelina", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75941", "FIPS": "48005", "County": "Angelina", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75949", "FIPS": "48005", "County": "Angelina", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75969", "FIPS": "48005", "County": "Angelina", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75980", "FIPS": "48005", "County": "Angelina", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "78358", "FIPS": "48007", "County": "Aransas", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78381", "FIPS": "48007", "County": "Aransas", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78382", "FIPS": "48007", "County": "Aransas", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "76351", "FIPS": "48009", "County": "Archer", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76366", "FIPS": "48009", "County": "Archer", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76370", "FIPS": "48009", "County": "Archer", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76379", "FIPS": "48009", "County": "Archer", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76389", "FIPS": "48009", "County": "Archer", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "79019", "FIPS": "48011", "County": "Armstrong", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79094", "FIPS": "48011", "County": "Armstrong", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "78008", "FIPS": "48013", "County": "Atascosa", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78011", "FIPS": "48013", "County": "Atascosa", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78012", "FIPS": "48013", "County": "Atascosa", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78026", "FIPS": "48013", "County": "Atascosa", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78050", "FIPS": "48013", "County": "Atascosa", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78052", "FIPS": "48013", "County": "Atascosa", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78062", "FIPS": "48013", "County": "Atascosa", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78064", "FIPS": "48013", "County": "Atascosa", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78065", "FIPS": "48013", "County": "Atascosa", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78069", "FIPS": "48013", "County": "Atascosa", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "77418", "FIPS": "48015", "County": "Austin", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77452", "FIPS": "48015", "County": "Austin", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77473", "FIPS": "48015", "County": "Austin", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77474", "FIPS": "48015", "County": "Austin", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77485", "FIPS": "48015", "County": "Austin", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "78931", "FIPS": "48015", "County": "Austin", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "78933", "FIPS": "48015", "County": "Austin", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "78944", "FIPS": "48015", "County": "Austin", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "78950", "FIPS": "48015", "County": "Austin", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "79320", "FIPS": "48017", "County": "Bailey", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79324", "FIPS": "48017", "County": "Bailey", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79344", "FIPS": "48017", "County": "Bailey", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79347", "FIPS": "48017", "County": "Bailey", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "78003", "FIPS": "48019", "County": "Bandera", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78055", "FIPS": "48019", "County": "Bandera", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78063", "FIPS": "48019", "County": "Bandera", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78883", "FIPS": "48019", "County": "Bandera", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78885", "FIPS": "48019", "County": "Bandera", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78602", "FIPS": "48021", "County": "Bastrop", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78612", "FIPS": "48021", "County": "Bastrop", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78621", "FIPS": "48021", "County": "Bastrop", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78650", "FIPS": "48021", "County": "Bastrop", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78659", "FIPS": "48021", "County": "Bastrop", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78662", "FIPS": "48021", "County": "Bastrop", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78953", "FIPS": "48021", "County": "Bastrop", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78957", "FIPS": "48021", "County": "Bastrop", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "76380", "FIPS": "48023", "County": "Baylor", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "78102", "FIPS": "48025", "County": "Bee", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78104", "FIPS": "48025", "County": "Bee", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78125", "FIPS": "48025", "County": "Bee", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78142", "FIPS": "48025", "County": "Bee", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78145", "FIPS": "48025", "County": "Bee", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78146", "FIPS": "48025", "County": "Bee", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78162", "FIPS": "48025", "County": "Bee", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78389", "FIPS": "48025", "County": "Bee", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78391", "FIPS": "48025", "County": "Bee", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "76501", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76502", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76503", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76504", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76505", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76508", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76511", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76513", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76533", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76534", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76540", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76541", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76542", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76543", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76544", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76545", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76546", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76547", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76548", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76549", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76554", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76559", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76564", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76569", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76571", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76579", "FIPS": "48027", "County": "Bell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "78002", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78015", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78023", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78054", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78073", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78101", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78109", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78112", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78148", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78150", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78152", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78201", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78202", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78203", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78204", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78205", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78206", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78207", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78208", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78209", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78210", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78211", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78212", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78213", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78214", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78215", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78216", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78217", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78218", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78219", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78220", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78221", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78222", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78223", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78224", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78225", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78226", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78227", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78228", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78229", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78230", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78231", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78232", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78233", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78234", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78235", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78236", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78237", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78238", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78239", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78240", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78241", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78242", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78243", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78244", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78245", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78246", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78247", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78248", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78249", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78250", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78251", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78252", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78253", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78254", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78255", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78256", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78257", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78258", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78259", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78260", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78261", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78262", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78263", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78264", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78265", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78268", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78269", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78270", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78275", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78278", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78279", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78280", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78283", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78284", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78285", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78286", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78287", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78288", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78289", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78291", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78292", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78293", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78294", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78295", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78296", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78297", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78298", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78299", "FIPS": "48029", "County": "Bexar", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78606", "FIPS": "48031", "County": "Blanco", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78635", "FIPS": "48031", "County": "Blanco", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78636", "FIPS": "48031", "County": "Blanco", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78663", "FIPS": "48031", "County": "Blanco", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "79738", "FIPS": "48033", "County": "Borden", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "76634", "FIPS": "48035", "County": "Bosque", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76637", "FIPS": "48035", "County": "Bosque", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76644", "FIPS": "48035", "County": "Bosque", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76649", "FIPS": "48035", "County": "Bosque", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76652", "FIPS": "48035", "County": "Bosque", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76665", "FIPS": "48035", "County": "Bosque", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76671", "FIPS": "48035", "County": "Bosque", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76689", "FIPS": "48035", "County": "Bosque", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76690", "FIPS": "48035", "County": "Bosque", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75501", "FIPS": "48037", "County": "Bowie", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75503", "FIPS": "48037", "County": "Bowie", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75504", "FIPS": "48037", "County": "Bowie", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75505", "FIPS": "48037", "County": "Bowie", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75507", "FIPS": "48037", "County": "Bowie", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75559", "FIPS": "48037", "County": "Bowie", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75561", "FIPS": "48037", "County": "Bowie", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75567", "FIPS": "48037", "County": "Bowie", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75569", "FIPS": "48037", "County": "Bowie", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75570", "FIPS": "48037", "County": "Bowie", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75573", "FIPS": "48037", "County": "Bowie", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75574", "FIPS": "48037", "County": "Bowie", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75599", "FIPS": "48037", "County": "Bowie", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "77422", "FIPS": "48039", "County": "Brazoria", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77430", "FIPS": "48039", "County": "Brazoria", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77431", "FIPS": "48039", "County": "Brazoria", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77463", "FIPS": "48039", "County": "Brazoria", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77480", "FIPS": "48039", "County": "Brazoria", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77486", "FIPS": "48039", "County": "Brazoria", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77511", "FIPS": "48039", "County": "Brazoria", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77512", "FIPS": "48039", "County": "Brazoria", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77515", "FIPS": "48039", "County": "Brazoria", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77516", "FIPS": "48039", "County": "Brazoria", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77531", "FIPS": "48039", "County": "Brazoria", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77534", "FIPS": "48039", "County": "Brazoria", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77541", "FIPS": "48039", "County": "Brazoria", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77542", "FIPS": "48039", "County": "Brazoria", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77566", "FIPS": "48039", "County": "Brazoria", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77577", "FIPS": "48039", "County": "Brazoria", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77578", "FIPS": "48039", "County": "Brazoria", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77581", "FIPS": "48039", "County": "Brazoria", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77583", "FIPS": "48039", "County": "Brazoria", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77584", "FIPS": "48039", "County": "Brazoria", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77588", "FIPS": "48039", "County": "Brazoria", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77801", "FIPS": "48041", "County": "Brazos", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77802", "FIPS": "48041", "County": "Brazos", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77803", "FIPS": "48041", "County": "Brazos", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77805", "FIPS": "48041", "County": "Brazos", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77806", "FIPS": "48041", "County": "Brazos", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77807", "FIPS": "48041", "County": "Brazos", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77808", "FIPS": "48041", "County": "Brazos", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77840", "FIPS": "48041", "County": "Brazos", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77841", "FIPS": "48041", "County": "Brazos", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77842", "FIPS": "48041", "County": "Brazos", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77843", "FIPS": "48041", "County": "Brazos", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77844", "FIPS": "48041", "County": "Brazos", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77845", "FIPS": "48041", "County": "Brazos", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77862", "FIPS": "48041", "County": "Brazos", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77866", "FIPS": "48041", "County": "Brazos", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77869", "FIPS": "48041", "County": "Brazos", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77881", "FIPS": "48041", "County": "Brazos", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "79830", "FIPS": "48043", "County": "Brewster", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79831", "FIPS": "48043", "County": "Brewster", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79832", "FIPS": "48043", "County": "Brewster", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79834", "FIPS": "48043", "County": "Brewster", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79842", "FIPS": "48043", "County": "Brewster", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79852", "FIPS": "48043", "County": "Brewster", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79255", "FIPS": "48045", "County": "Briscoe", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79257", "FIPS": "48045", "County": "Briscoe", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "78353", "FIPS": "48047", "County": "Brooks", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78355", "FIPS": "48047", "County": "Brooks", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "76432", "FIPS": "48049", "County": "Brown", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76801", "FIPS": "48049", "County": "Brown", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76802", "FIPS": "48049", "County": "Brown", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76803", "FIPS": "48049", "County": "Brown", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76804", "FIPS": "48049", "County": "Brown", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76823", "FIPS": "48049", "County": "Brown", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76827", "FIPS": "48049", "County": "Brown", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76857", "FIPS": "48049", "County": "Brown", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76890", "FIPS": "48049", "County": "Brown", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "77836", "FIPS": "48051", "County": "Burleson", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77838", "FIPS": "48051", "County": "Burleson", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77852", "FIPS": "48051", "County": "Burleson", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77863", "FIPS": "48051", "County": "Burleson", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77878", "FIPS": "48051", "County": "Burleson", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77879", "FIPS": "48051", "County": "Burleson", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "78605", "FIPS": "48053", "County": "Burnet", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78608", "FIPS": "48053", "County": "Burnet", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78611", "FIPS": "48053", "County": "Burnet", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78654", "FIPS": "48053", "County": "Burnet", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78616", "FIPS": "48055", "County": "Caldwell", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78622", "FIPS": "48055", "County": "Caldwell", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78644", "FIPS": "48055", "County": "Caldwell", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78648", "FIPS": "48055", "County": "Caldwell", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78655", "FIPS": "48055", "County": "Caldwell", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78656", "FIPS": "48055", "County": "Caldwell", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78661", "FIPS": "48055", "County": "Caldwell", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "77978", "FIPS": "48057", "County": "Calhoun", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77979", "FIPS": "48057", "County": "Calhoun", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77982", "FIPS": "48057", "County": "Calhoun", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77983", "FIPS": "48057", "County": "Calhoun", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "76443", "FIPS": "48059", "County": "Callahan", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76469", "FIPS": "48059", "County": "Callahan", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79504", "FIPS": "48059", "County": "Callahan", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79510", "FIPS": "48059", "County": "Callahan", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "78520", "FIPS": "48061", "County": "Cameron", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78521", "FIPS": "48061", "County": "Cameron", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78522", "FIPS": "48061", "County": "Cameron", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78523", "FIPS": "48061", "County": "Cameron", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78526", "FIPS": "48061", "County": "Cameron", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78535", "FIPS": "48061", "County": "Cameron", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78550", "FIPS": "48061", "County": "Cameron", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78551", "FIPS": "48061", "County": "Cameron", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78552", "FIPS": "48061", "County": "Cameron", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78553", "FIPS": "48061", "County": "Cameron", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78559", "FIPS": "48061", "County": "Cameron", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78566", "FIPS": "48061", "County": "Cameron", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78567", "FIPS": "48061", "County": "Cameron", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78568", "FIPS": "48061", "County": "Cameron", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78575", "FIPS": "48061", "County": "Cameron", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78578", "FIPS": "48061", "County": "Cameron", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78583", "FIPS": "48061", "County": "Cameron", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78586", "FIPS": "48061", "County": "Cameron", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78592", "FIPS": "48061", "County": "Cameron", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78593", "FIPS": "48061", "County": "Cameron", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78597", "FIPS": "48061", "County": "Cameron", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "75451", "FIPS": "48063", "County": "Camp", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75686", "FIPS": "48063", "County": "Camp", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "79039", "FIPS": "48065", "County": "Carson", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79068", "FIPS": "48065", "County": "Carson", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79080", "FIPS": "48065", "County": "Carson", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79097", "FIPS": "48065", "County": "Carson", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "75551", "FIPS": "48067", "County": "Cass", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75555", "FIPS": "48067", "County": "Cass", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75556", "FIPS": "48067", "County": "Cass", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75560", "FIPS": "48067", "County": "Cass", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75562", "FIPS": "48067", "County": "Cass", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75563", "FIPS": "48067", "County": "Cass", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75565", "FIPS": "48067", "County": "Cass", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75566", "FIPS": "48067", "County": "Cass", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75572", "FIPS": "48067", "County": "Cass", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75630", "FIPS": "48067", "County": "Cass", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75656", "FIPS": "48067", "County": "Cass", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "79027", "FIPS": "48069", "County": "Castro", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79043", "FIPS": "48069", "County": "Castro", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79063", "FIPS": "48069", "County": "Castro", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79085", "FIPS": "48069", "County": "Castro", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "77514", "FIPS": "48071", "County": "Chambers", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77560", "FIPS": "48071", "County": "Chambers", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77580", "FIPS": "48071", "County": "Chambers", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77597", "FIPS": "48071", "County": "Chambers", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77661", "FIPS": "48071", "County": "Chambers", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77665", "FIPS": "48071", "County": "Chambers", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "75759", "FIPS": "48073", "County": "Cherokee", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75764", "FIPS": "48073", "County": "Cherokee", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75766", "FIPS": "48073", "County": "Cherokee", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75772", "FIPS": "48073", "County": "Cherokee", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75780", "FIPS": "48073", "County": "Cherokee", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75784", "FIPS": "48073", "County": "Cherokee", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75785", "FIPS": "48073", "County": "Cherokee", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75925", "FIPS": "48073", "County": "Cherokee", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75976", "FIPS": "48073", "County": "Cherokee", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "79201", "FIPS": "48075", "County": "Childress", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79259", "FIPS": "48075", "County": "Childress", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "76228", "FIPS": "48077", "County": "Clay", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76352", "FIPS": "48077", "County": "Clay", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76357", "FIPS": "48077", "County": "Clay", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76365", "FIPS": "48077", "County": "Clay", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76377", "FIPS": "48077", "County": "Clay", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "79314", "FIPS": "48079", "County": "Cochran", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79346", "FIPS": "48079", "County": "Cochran", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79379", "FIPS": "48079", "County": "Cochran", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "76933", "FIPS": "48081", "County": "Coke", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76945", "FIPS": "48081", "County": "Coke", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76949", "FIPS": "48081", "County": "Coke", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76953", "FIPS": "48081", "County": "Coke", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76828", "FIPS": "48083", "County": "Coleman", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76834", "FIPS": "48083", "County": "Coleman", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76845", "FIPS": "48083", "County": "Coleman", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76873", "FIPS": "48083", "County": "Coleman", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76878", "FIPS": "48083", "County": "Coleman", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76882", "FIPS": "48083", "County": "Coleman", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76884", "FIPS": "48083", "County": "Coleman", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76888", "FIPS": "48083", "County": "Coleman", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79519", "FIPS": "48083", "County": "Coleman", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79538", "FIPS": "48083", "County": "Coleman", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "75002", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75009", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75013", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75023", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75024", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75025", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75026", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75034", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75035", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75069", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75070", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75071", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75074", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75075", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75078", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75086", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75093", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75094", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75097", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75098", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75121", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75164", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75166", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75173", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75252", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75287", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75407", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75409", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75424", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75442", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75454", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75485", "FIPS": "48085", "County": "Collin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "79077", "FIPS": "48087", "County": "Collingsworth", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79095", "FIPS": "48087", "County": "Collingsworth", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79230", "FIPS": "48087", "County": "Collingsworth", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79251", "FIPS": "48087", "County": "Collingsworth", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "77412", "FIPS": "48089", "County": "Colorado", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77434", "FIPS": "48089", "County": "Colorado", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77442", "FIPS": "48089", "County": "Colorado", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77460", "FIPS": "48089", "County": "Colorado", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77470", "FIPS": "48089", "County": "Colorado", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77475", "FIPS": "48089", "County": "Colorado", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "78934", "FIPS": "48089", "County": "Colorado", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "78935", "FIPS": "48089", "County": "Colorado", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "78943", "FIPS": "48089", "County": "Colorado", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "78951", "FIPS": "48089", "County": "Colorado", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "78962", "FIPS": "48089", "County": "Colorado", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "78070", "FIPS": "48091", "County": "Comal", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78130", "FIPS": "48091", "County": "Comal", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78131", "FIPS": "48091", "County": "Comal", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78132", "FIPS": "48091", "County": "Comal", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78133", "FIPS": "48091", "County": "Comal", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78135", "FIPS": "48091", "County": "Comal", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78163", "FIPS": "48091", "County": "Comal", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78266", "FIPS": "48091", "County": "Comal", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78623", "FIPS": "48091", "County": "Comal", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "76442", "FIPS": "48093", "County": "Comanche", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76444", "FIPS": "48093", "County": "Comanche", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76452", "FIPS": "48093", "County": "Comanche", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76455", "FIPS": "48093", "County": "Comanche", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76468", "FIPS": "48093", "County": "Comanche", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76474", "FIPS": "48093", "County": "Comanche", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76837", "FIPS": "48095", "County": "Concho", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76855", "FIPS": "48095", "County": "Concho", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76862", "FIPS": "48095", "County": "Concho", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76866", "FIPS": "48095", "County": "Concho", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76937", "FIPS": "48095", "County": "Concho", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76238", "FIPS": "48097", "County": "Cooke", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76240", "FIPS": "48097", "County": "Cooke", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76241", "FIPS": "48097", "County": "Cooke", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76250", "FIPS": "48097", "County": "Cooke", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76252", "FIPS": "48097", "County": "Cooke", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76253", "FIPS": "48097", "County": "Cooke", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76263", "FIPS": "48097", "County": "Cooke", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76272", "FIPS": "48097", "County": "Cooke", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76522", "FIPS": "48099", "County": "Coryell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76525", "FIPS": "48099", "County": "Coryell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76526", "FIPS": "48099", "County": "Coryell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76528", "FIPS": "48099", "County": "Coryell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76538", "FIPS": "48099", "County": "Coryell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76558", "FIPS": "48099", "County": "Coryell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76561", "FIPS": "48099", "County": "Coryell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76566", "FIPS": "48099", "County": "Coryell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76596", "FIPS": "48099", "County": "Coryell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76597", "FIPS": "48099", "County": "Coryell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76598", "FIPS": "48099", "County": "Coryell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76599", "FIPS": "48099", "County": "Coryell", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "79223", "FIPS": "48101", "County": "Cottle", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79248", "FIPS": "48101", "County": "Cottle", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79731", "FIPS": "48103", "County": "Crane", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "76943", "FIPS": "48105", "County": "Crockett", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "79322", "FIPS": "48107", "County": "Crosby", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79343", "FIPS": "48107", "County": "Crosby", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79357", "FIPS": "48107", "County": "Crosby", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79855", "FIPS": "48109", "County": "Culberson", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79022", "FIPS": "48111", "County": "Dallam", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79051", "FIPS": "48111", "County": "Dallam", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79087", "FIPS": "48111", "County": "Dallam", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "75001", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75006", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75011", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75014", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75015", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75016", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75017", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75019", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75030", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75037", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75038", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75039", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75040", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75041", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75042", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75043", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75044", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75045", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75046", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75047", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75048", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75049", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75050", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75051", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75052", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75053", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75054", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75060", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75061", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75062", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75063", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75080", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75081", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75082", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75083", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75085", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75088", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75089", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75099", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75104", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75106", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75115", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75116", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75123", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75134", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75137", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75138", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75141", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75146", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75149", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75150", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75159", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75172", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75180", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75181", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75182", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75185", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75187", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75201", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75202", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75203", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75204", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75205", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75206", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75207", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75208", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75209", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75210", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75211", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75212", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75214", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75215", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75216", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75217", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75218", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75219", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75220", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75221", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75222", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75223", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75224", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75225", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75226", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75227", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75228", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75229", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75230", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75231", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75232", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75233", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75234", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75235", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75236", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75237", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75238", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75240", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75241", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75242", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75243", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75244", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75245", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75246", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75247", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75248", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75249", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75250", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75251", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75253", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75254", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75258", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75260", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75261", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75262", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75263", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75264", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75265", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75266", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75267", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75270", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75275", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75277", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75283", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75284", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75285", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75286", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75301", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75303", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75310", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75312", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75313", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75315", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75320", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75323", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75326", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75334", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75336", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75339", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75340", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75342", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75343", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75344", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75353", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75354", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75355", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75356", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75357", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75358", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75359", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75360", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75363", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75364", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75367", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75368", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75370", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75371", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75372", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75373", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75374", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75376", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75378", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75379", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75380", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75381", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75382", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75386", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75387", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75388", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75389", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75390", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75391", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75392", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75393", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75394", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75395", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75396", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75397", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75398", "FIPS": "48113", "County": "Dallas", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "79331", "FIPS": "48115", "County": "Dawson", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79377", "FIPS": "48115", "County": "Dawson", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79025", "FIPS": "48117", "County": "Deaf Smith", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79045", "FIPS": "48117", "County": "Deaf Smith", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "75415", "FIPS": "48119", "County": "Delta", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75432", "FIPS": "48119", "County": "Delta", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75441", "FIPS": "48119", "County": "Delta", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75448", "FIPS": "48119", "County": "Delta", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75450", "FIPS": "48119", "County": "Delta", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75469", "FIPS": "48119", "County": "Delta", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75007", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75010", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75022", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75027", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75028", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75029", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75056", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75057", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75065", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75067", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75068", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75077", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76201", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76202", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76203", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76204", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76205", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76206", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76207", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76208", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76209", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76210", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76226", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76227", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76247", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76249", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76258", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76259", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76262", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76266", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76299", "FIPS": "48121", "County": "Denton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "77954", "FIPS": "48123", "County": "De Witt", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "77967", "FIPS": "48123", "County": "De Witt", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "77974", "FIPS": "48123", "County": "De Witt", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "77989", "FIPS": "48123", "County": "De Witt", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "77994", "FIPS": "48123", "County": "De Witt", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78141", "FIPS": "48123", "County": "De Witt", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78164", "FIPS": "48123", "County": "De Witt", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "79220", "FIPS": "48125", "County": "Dickens", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79229", "FIPS": "48125", "County": "Dickens", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79243", "FIPS": "48125", "County": "Dickens", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79370", "FIPS": "48125", "County": "Dickens", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "78827", "FIPS": "48127", "County": "Dimmit", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78830", "FIPS": "48127", "County": "Dimmit", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78834", "FIPS": "48127", "County": "Dimmit", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78836", "FIPS": "48127", "County": "Dimmit", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "79226", "FIPS": "48129", "County": "Donley", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79237", "FIPS": "48129", "County": "Donley", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79240", "FIPS": "48129", "County": "Donley", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "78341", "FIPS": "48131", "County": "Duval", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78349", "FIPS": "48131", "County": "Duval", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78357", "FIPS": "48131", "County": "Duval", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78376", "FIPS": "48131", "County": "Duval", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78384", "FIPS": "48131", "County": "Duval", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "76435", "FIPS": "48133", "County": "Eastland", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76437", "FIPS": "48133", "County": "Eastland", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76445", "FIPS": "48133", "County": "Eastland", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76448", "FIPS": "48133", "County": "Eastland", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76454", "FIPS": "48133", "County": "Eastland", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76466", "FIPS": "48133", "County": "Eastland", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76470", "FIPS": "48133", "County": "Eastland", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76471", "FIPS": "48133", "County": "Eastland", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79741", "FIPS": "48135", "County": "Ector", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79758", "FIPS": "48135", "County": "Ector", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79759", "FIPS": "48135", "County": "Ector", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79760", "FIPS": "48135", "County": "Ector", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79761", "FIPS": "48135", "County": "Ector", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79762", "FIPS": "48135", "County": "Ector", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79763", "FIPS": "48135", "County": "Ector", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79764", "FIPS": "48135", "County": "Ector", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79765", "FIPS": "48135", "County": "Ector", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79766", "FIPS": "48135", "County": "Ector", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79768", "FIPS": "48135", "County": "Ector", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79769", "FIPS": "48135", "County": "Ector", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79776", "FIPS": "48135", "County": "Ector", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "76883", "FIPS": "48137", "County": "Edwards", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78828", "FIPS": "48137", "County": "Edwards", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78880", "FIPS": "48137", "County": "Edwards", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "75101", "FIPS": "48139", "County": "Ellis", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75119", "FIPS": "48139", "County": "Ellis", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75120", "FIPS": "48139", "County": "Ellis", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75125", "FIPS": "48139", "County": "Ellis", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75152", "FIPS": "48139", "County": "Ellis", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75154", "FIPS": "48139", "County": "Ellis", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75165", "FIPS": "48139", "County": "Ellis", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75167", "FIPS": "48139", "County": "Ellis", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75168", "FIPS": "48139", "County": "Ellis", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76041", "FIPS": "48139", "County": "Ellis", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76064", "FIPS": "48139", "County": "Ellis", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76065", "FIPS": "48139", "County": "Ellis", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76623", "FIPS": "48139", "County": "Ellis", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76651", "FIPS": "48139", "County": "Ellis", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76670", "FIPS": "48139", "County": "Ellis", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "79821", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79835", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79836", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79838", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79849", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79853", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79901", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79902", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79903", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79904", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79905", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79906", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79907", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79908", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79910", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79911", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79912", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79913", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79914", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79915", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79916", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79917", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79918", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79920", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79922", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79923", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79924", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79925", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79926", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79927", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79928", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79929", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79930", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79931", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79932", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79934", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79935", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79936", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79937", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79938", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79940", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79941", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79942", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79943", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79944", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79945", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79946", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79947", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79948", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79949", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79950", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79951", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79952", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79953", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79954", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79955", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79958", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79960", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79961", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79968", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79976", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79978", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79980", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79990", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79995", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79996", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79997", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79998", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79999", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88510", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88511", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88512", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88513", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88514", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88515", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88516", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88517", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88518", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88519", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88520", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88521", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88523", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88524", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88525", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88526", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88527", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88528", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88529", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88530", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88531", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88532", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88533", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88534", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88535", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88536", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88538", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88539", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88540", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88541", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88542", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88543", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88544", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88545", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88546", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88547", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88548", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88549", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88550", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88553", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88554", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88555", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88556", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88557", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88558", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88559", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88560", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88561", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88562", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88563", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88565", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88566", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88567", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88568", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88569", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88570", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88571", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88572", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88573", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88574", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88575", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88576", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88577", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88578", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88579", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88580", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88581", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88582", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88583", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88584", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88585", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88586", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88587", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88588", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88589", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88590", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "88595", "FIPS": "48141", "County": "El Paso", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "76401", "FIPS": "48143", "County": "Erath", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76402", "FIPS": "48143", "County": "Erath", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76433", "FIPS": "48143", "County": "Erath", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76446", "FIPS": "48143", "County": "Erath", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76461", "FIPS": "48143", "County": "Erath", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76465", "FIPS": "48143", "County": "Erath", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76570", "FIPS": "48145", "County": "Falls", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76632", "FIPS": "48145", "County": "Falls", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76656", "FIPS": "48145", "County": "Falls", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76661", "FIPS": "48145", "County": "Falls", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76680", "FIPS": "48145", "County": "Falls", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76685", "FIPS": "48145", "County": "Falls", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "75413", "FIPS": "48147", "County": "Fannin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75418", "FIPS": "48147", "County": "Fannin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75438", "FIPS": "48147", "County": "Fannin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75439", "FIPS": "48147", "County": "Fannin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75443", "FIPS": "48147", "County": "Fannin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75446", "FIPS": "48147", "County": "Fannin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75447", "FIPS": "48147", "County": "Fannin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75449", "FIPS": "48147", "County": "Fannin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75452", "FIPS": "48147", "County": "Fannin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75475", "FIPS": "48147", "County": "Fannin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75476", "FIPS": "48147", "County": "Fannin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75479", "FIPS": "48147", "County": "Fannin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75488", "FIPS": "48147", "County": "Fannin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75490", "FIPS": "48147", "County": "Fannin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75492", "FIPS": "48147", "County": "Fannin", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "78932", "FIPS": "48149", "County": "Fayette", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78938", "FIPS": "48149", "County": "Fayette", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78940", "FIPS": "48149", "County": "Fayette", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78941", "FIPS": "48149", "County": "Fayette", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78945", "FIPS": "48149", "County": "Fayette", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78946", "FIPS": "48149", "County": "Fayette", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78949", "FIPS": "48149", "County": "Fayette", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78952", "FIPS": "48149", "County": "Fayette", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78954", "FIPS": "48149", "County": "Fayette", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78956", "FIPS": "48149", "County": "Fayette", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78960", "FIPS": "48149", "County": "Fayette", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78961", "FIPS": "48149", "County": "Fayette", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78963", "FIPS": "48149", "County": "Fayette", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "79534", "FIPS": "48151", "County": "Fisher", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79543", "FIPS": "48151", "County": "Fisher", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79546", "FIPS": "48151", "County": "Fisher", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79560", "FIPS": "48151", "County": "Fisher", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79221", "FIPS": "48153", "County": "Floyd", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79231", "FIPS": "48153", "County": "Floyd", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79235", "FIPS": "48153", "County": "Floyd", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79241", "FIPS": "48153", "County": "Floyd", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79258", "FIPS": "48153", "County": "Floyd", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79227", "FIPS": "48155", "County": "Foard", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "77053", "FIPS": "48157", "County": "Fort Bend", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77406", "FIPS": "48157", "County": "Fort Bend", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77417", "FIPS": "48157", "County": "Fort Bend", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77441", "FIPS": "48157", "County": "Fort Bend", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77444", "FIPS": "48157", "County": "Fort Bend", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77451", "FIPS": "48157", "County": "Fort Bend", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77459", "FIPS": "48157", "County": "Fort Bend", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77461", "FIPS": "48157", "County": "Fort Bend", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77464", "FIPS": "48157", "County": "Fort Bend", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77469", "FIPS": "48157", "County": "Fort Bend", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77471", "FIPS": "48157", "County": "Fort Bend", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77476", "FIPS": "48157", "County": "Fort Bend", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77477", "FIPS": "48157", "County": "Fort Bend", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77478", "FIPS": "48157", "County": "Fort Bend", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77479", "FIPS": "48157", "County": "Fort Bend", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77481", "FIPS": "48157", "County": "Fort Bend", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77487", "FIPS": "48157", "County": "Fort Bend", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77489", "FIPS": "48157", "County": "Fort Bend", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77494", "FIPS": "48157", "County": "Fort Bend", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77496", "FIPS": "48157", "County": "Fort Bend", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77497", "FIPS": "48157", "County": "Fort Bend", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77545", "FIPS": "48157", "County": "Fort Bend", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "75457", "FIPS": "48159", "County": "Franklin", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75480", "FIPS": "48159", "County": "Franklin", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75487", "FIPS": "48159", "County": "Franklin", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75838", "FIPS": "48161", "County": "Freestone", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75840", "FIPS": "48161", "County": "Freestone", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75848", "FIPS": "48161", "County": "Freestone", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75859", "FIPS": "48161", "County": "Freestone", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75860", "FIPS": "48161", "County": "Freestone", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76693", "FIPS": "48161", "County": "Freestone", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "78005", "FIPS": "48163", "County": "Frio", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78017", "FIPS": "48163", "County": "Frio", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78057", "FIPS": "48163", "County": "Frio", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78061", "FIPS": "48163", "County": "Frio", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "79342", "FIPS": "48165", "County": "Gaines", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79359", "FIPS": "48165", "County": "Gaines", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79360", "FIPS": "48165", "County": "Gaines", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "77510", "FIPS": "48167", "County": "Galveston", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77517", "FIPS": "48167", "County": "Galveston", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77518", "FIPS": "48167", "County": "Galveston", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77539", "FIPS": "48167", "County": "Galveston", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77546", "FIPS": "48167", "County": "Galveston", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77549", "FIPS": "48167", "County": "Galveston", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77550", "FIPS": "48167", "County": "Galveston", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77551", "FIPS": "48167", "County": "Galveston", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77552", "FIPS": "48167", "County": "Galveston", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77553", "FIPS": "48167", "County": "Galveston", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77554", "FIPS": "48167", "County": "Galveston", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77555", "FIPS": "48167", "County": "Galveston", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77563", "FIPS": "48167", "County": "Galveston", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77565", "FIPS": "48167", "County": "Galveston", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77568", "FIPS": "48167", "County": "Galveston", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77573", "FIPS": "48167", "County": "Galveston", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77574", "FIPS": "48167", "County": "Galveston", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77590", "FIPS": "48167", "County": "Galveston", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77591", "FIPS": "48167", "County": "Galveston", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77592", "FIPS": "48167", "County": "Galveston", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77617", "FIPS": "48167", "County": "Galveston", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77623", "FIPS": "48167", "County": "Galveston", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77650", "FIPS": "48167", "County": "Galveston", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "79330", "FIPS": "48169", "County": "Garza", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79356", "FIPS": "48169", "County": "Garza", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "78618", "FIPS": "48171", "County": "Gillespie", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78624", "FIPS": "48171", "County": "Gillespie", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78631", "FIPS": "48171", "County": "Gillespie", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78671", "FIPS": "48171", "County": "Gillespie", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78675", "FIPS": "48171", "County": "Gillespie", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "79739", "FIPS": "48173", "County": "Glasscock", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "77960", "FIPS": "48175", "County": "Goliad", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "77963", "FIPS": "48175", "County": "Goliad", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "77993", "FIPS": "48175", "County": "Goliad", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78107", "FIPS": "48175", "County": "Goliad", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78122", "FIPS": "48177", "County": "Gonzales", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78140", "FIPS": "48177", "County": "Gonzales", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78159", "FIPS": "48177", "County": "Gonzales", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78604", "FIPS": "48177", "County": "Gonzales", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78614", "FIPS": "48177", "County": "Gonzales", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78629", "FIPS": "48177", "County": "Gonzales", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78632", "FIPS": "48177", "County": "Gonzales", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78658", "FIPS": "48177", "County": "Gonzales", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78677", "FIPS": "48177", "County": "Gonzales", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78959", "FIPS": "48177", "County": "Gonzales", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "79002", "FIPS": "48179", "County": "Gray", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79054", "FIPS": "48179", "County": "Gray", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79057", "FIPS": "48179", "County": "Gray", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79065", "FIPS": "48179", "County": "Gray", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79066", "FIPS": "48179", "County": "Gray", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "75020", "FIPS": "48181", "County": "Grayson", "State": "TX", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "75021", "FIPS": "48181", "County": "Grayson", "State": "TX", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "75058", "FIPS": "48181", "County": "Grayson", "State": "TX", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "75076", "FIPS": "48181", "County": "Grayson", "State": "TX", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "75090", "FIPS": "48181", "County": "Grayson", "State": "TX", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "75091", "FIPS": "48181", "County": "Grayson", "State": "TX", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "75092", "FIPS": "48181", "County": "Grayson", "State": "TX", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "75414", "FIPS": "48181", "County": "Grayson", "State": "TX", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "75459", "FIPS": "48181", "County": "Grayson", "State": "TX", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "75489", "FIPS": "48181", "County": "Grayson", "State": "TX", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "75491", "FIPS": "48181", "County": "Grayson", "State": "TX", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "75495", "FIPS": "48181", "County": "Grayson", "State": "TX", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "76233", "FIPS": "48181", "County": "Grayson", "State": "TX", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "76245", "FIPS": "48181", "County": "Grayson", "State": "TX", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "76264", "FIPS": "48181", "County": "Grayson", "State": "TX", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "76268", "FIPS": "48181", "County": "Grayson", "State": "TX", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "76271", "FIPS": "48181", "County": "Grayson", "State": "TX", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "76273", "FIPS": "48181", "County": "Grayson", "State": "TX", "DMA": {"Code": 657, "Name": "SHERMAN-ADA"}},
  {"Postal_Code": "75601", "FIPS": "48183", "County": "Gregg", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75602", "FIPS": "48183", "County": "Gregg", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75603", "FIPS": "48183", "County": "Gregg", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75604", "FIPS": "48183", "County": "Gregg", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75605", "FIPS": "48183", "County": "Gregg", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75606", "FIPS": "48183", "County": "Gregg", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75607", "FIPS": "48183", "County": "Gregg", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75608", "FIPS": "48183", "County": "Gregg", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75615", "FIPS": "48183", "County": "Gregg", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75641", "FIPS": "48183", "County": "Gregg", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75647", "FIPS": "48183", "County": "Gregg", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75660", "FIPS": "48183", "County": "Gregg", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75662", "FIPS": "48183", "County": "Gregg", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75663", "FIPS": "48183", "County": "Gregg", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75693", "FIPS": "48183", "County": "Gregg", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "77363", "FIPS": "48185", "County": "Grimes", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77830", "FIPS": "48185", "County": "Grimes", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77831", "FIPS": "48185", "County": "Grimes", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77861", "FIPS": "48185", "County": "Grimes", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77868", "FIPS": "48185", "County": "Grimes", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77873", "FIPS": "48185", "County": "Grimes", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77875", "FIPS": "48185", "County": "Grimes", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77876", "FIPS": "48185", "County": "Grimes", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "78108", "FIPS": "48187", "County": "Guadalupe", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78115", "FIPS": "48187", "County": "Guadalupe", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78123", "FIPS": "48187", "County": "Guadalupe", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78124", "FIPS": "48187", "County": "Guadalupe", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78154", "FIPS": "48187", "County": "Guadalupe", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78155", "FIPS": "48187", "County": "Guadalupe", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78156", "FIPS": "48187", "County": "Guadalupe", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78638", "FIPS": "48187", "County": "Guadalupe", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78670", "FIPS": "48187", "County": "Guadalupe", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "79021", "FIPS": "48189", "County": "Hale", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79032", "FIPS": "48189", "County": "Hale", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79041", "FIPS": "48189", "County": "Hale", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79072", "FIPS": "48189", "County": "Hale", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79073", "FIPS": "48189", "County": "Hale", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79250", "FIPS": "48189", "County": "Hale", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79311", "FIPS": "48189", "County": "Hale", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79233", "FIPS": "48191", "County": "Hall", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79239", "FIPS": "48191", "County": "Hall", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79245", "FIPS": "48191", "County": "Hall", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79261", "FIPS": "48191", "County": "Hall", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "76436", "FIPS": "48193", "County": "Hamilton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76457", "FIPS": "48193", "County": "Hamilton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76531", "FIPS": "48193", "County": "Hamilton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76565", "FIPS": "48193", "County": "Hamilton", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "79040", "FIPS": "48195", "County": "Hansford", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79062", "FIPS": "48195", "County": "Hansford", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79081", "FIPS": "48195", "County": "Hansford", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79225", "FIPS": "48197", "County": "Hardeman", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "79252", "FIPS": "48197", "County": "Hardeman", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "77374", "FIPS": "48199", "County": "Hardin", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77376", "FIPS": "48199", "County": "Hardin", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77519", "FIPS": "48199", "County": "Hardin", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77585", "FIPS": "48199", "County": "Hardin", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77625", "FIPS": "48199", "County": "Hardin", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77656", "FIPS": "48199", "County": "Hardin", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77657", "FIPS": "48199", "County": "Hardin", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77659", "FIPS": "48199", "County": "Hardin", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77663", "FIPS": "48199", "County": "Hardin", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77001", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77002", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77003", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77004", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77005", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77006", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77007", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77008", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77009", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77010", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77011", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77012", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77013", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77014", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77015", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77016", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77017", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77018", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77019", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77020", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77021", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77022", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77023", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77024", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77025", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77026", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77027", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77028", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77029", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77030", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77031", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77032", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77033", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77034", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77035", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77036", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77037", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77038", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77039", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77040", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77041", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77042", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77043", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77044", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77045", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77046", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77047", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77048", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77049", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77050", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77051", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77052", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77054", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77055", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77056", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77057", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77058", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77059", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77060", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77061", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77062", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77063", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77064", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77065", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77066", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77067", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77068", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77069", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77070", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77071", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77072", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77073", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77074", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77075", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77076", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77077", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77078", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77079", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77080", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77081", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77082", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77083", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77084", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77085", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77086", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77087", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77088", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77089", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77090", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77091", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77092", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77093", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77094", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77095", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77096", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77097", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77098", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77099", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77201", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77202", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77203", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77204", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77205", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77206", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77207", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77208", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77209", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77210", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77212", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77213", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77215", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77216", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77217", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77218", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77219", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77220", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77221", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77222", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77223", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77224", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77225", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77226", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77227", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77228", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77229", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77230", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77231", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77233", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77234", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77235", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77236", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77237", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77238", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77240", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77241", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77242", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77243", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77244", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77245", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77246", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77247", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77248", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77249", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77250", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77251", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77252", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77253", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77254", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77255", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77256", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77257", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77258", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77259", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77260", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77261", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77262", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77263", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77265", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77266", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77267", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77268", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77269", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77270", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77271", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77272", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77273", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77274", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77275", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77276", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77277", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77278", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77279", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77280", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77282", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77284", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77285", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77286", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77287", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77288", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77289", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77290", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77291", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77292", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77293", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77294", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77296", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77297", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77298", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77299", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77315", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77325", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77336", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77337", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77338", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77339", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77345", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77346", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77347", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77373", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77375", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77377", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77379", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77383", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77388", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77389", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77391", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77396", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77401", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77402", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77410", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77411", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77413", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77429", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77433", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77447", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77449", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77450", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77484", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77491", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77492", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77493", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77501", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77502", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77503", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77504", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77505", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77506", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77507", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77508", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77520", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77521", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77522", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77530", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77532", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77536", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77547", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77562", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77571", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77572", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77586", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77587", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77598", "FIPS": "48201", "County": "Harris", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "75642", "FIPS": "48203", "County": "Harrison", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75650", "FIPS": "48203", "County": "Harrison", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75651", "FIPS": "48203", "County": "Harrison", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75659", "FIPS": "48203", "County": "Harrison", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75661", "FIPS": "48203", "County": "Harrison", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75670", "FIPS": "48203", "County": "Harrison", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75671", "FIPS": "48203", "County": "Harrison", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75672", "FIPS": "48203", "County": "Harrison", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75688", "FIPS": "48203", "County": "Harrison", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75692", "FIPS": "48203", "County": "Harrison", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75694", "FIPS": "48203", "County": "Harrison", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "79018", "FIPS": "48205", "County": "Hartley", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79044", "FIPS": "48205", "County": "Hartley", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "76388", "FIPS": "48207", "County": "Haskell", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79521", "FIPS": "48207", "County": "Haskell", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79539", "FIPS": "48207", "County": "Haskell", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79544", "FIPS": "48207", "County": "Haskell", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79547", "FIPS": "48207", "County": "Haskell", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79548", "FIPS": "48207", "County": "Haskell", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "78610", "FIPS": "48209", "County": "Hays", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78619", "FIPS": "48209", "County": "Hays", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78620", "FIPS": "48209", "County": "Hays", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78640", "FIPS": "48209", "County": "Hays", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78666", "FIPS": "48209", "County": "Hays", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78667", "FIPS": "48209", "County": "Hays", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78676", "FIPS": "48209", "County": "Hays", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78737", "FIPS": "48209", "County": "Hays", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "79014", "FIPS": "48211", "County": "Hemphill", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "75124", "FIPS": "48213", "County": "Henderson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75148", "FIPS": "48213", "County": "Henderson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75156", "FIPS": "48213", "County": "Henderson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75163", "FIPS": "48213", "County": "Henderson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75751", "FIPS": "48213", "County": "Henderson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75752", "FIPS": "48213", "County": "Henderson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75756", "FIPS": "48213", "County": "Henderson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75758", "FIPS": "48213", "County": "Henderson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75770", "FIPS": "48213", "County": "Henderson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75778", "FIPS": "48213", "County": "Henderson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75782", "FIPS": "48213", "County": "Henderson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "78501", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78502", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78503", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78504", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78505", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78516", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78537", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78538", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78539", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78540", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78541", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78543", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78549", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78557", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78558", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78560", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78562", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78563", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78565", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78570", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78572", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78573", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78574", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78576", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78577", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78579", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78589", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78595", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78596", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78599", "FIPS": "48215", "County": "Hidalgo", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "76055", "FIPS": "48217", "County": "Hill", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76621", "FIPS": "48217", "County": "Hill", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76622", "FIPS": "48217", "County": "Hill", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76627", "FIPS": "48217", "County": "Hill", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76628", "FIPS": "48217", "County": "Hill", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76631", "FIPS": "48217", "County": "Hill", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76636", "FIPS": "48217", "County": "Hill", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76645", "FIPS": "48217", "County": "Hill", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76648", "FIPS": "48217", "County": "Hill", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76650", "FIPS": "48217", "County": "Hill", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76660", "FIPS": "48217", "County": "Hill", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76666", "FIPS": "48217", "County": "Hill", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76673", "FIPS": "48217", "County": "Hill", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76676", "FIPS": "48217", "County": "Hill", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76692", "FIPS": "48217", "County": "Hill", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "79313", "FIPS": "48219", "County": "Hockley", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79336", "FIPS": "48219", "County": "Hockley", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79338", "FIPS": "48219", "County": "Hockley", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79353", "FIPS": "48219", "County": "Hockley", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79358", "FIPS": "48219", "County": "Hockley", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79367", "FIPS": "48219", "County": "Hockley", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79372", "FIPS": "48219", "County": "Hockley", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79380", "FIPS": "48219", "County": "Hockley", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "76035", "FIPS": "48221", "County": "Hood", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76048", "FIPS": "48221", "County": "Hood", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76049", "FIPS": "48221", "County": "Hood", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76462", "FIPS": "48221", "County": "Hood", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76467", "FIPS": "48221", "County": "Hood", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76476", "FIPS": "48221", "County": "Hood", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75420", "FIPS": "48223", "County": "Hopkins", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75431", "FIPS": "48223", "County": "Hopkins", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75433", "FIPS": "48223", "County": "Hopkins", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75437", "FIPS": "48223", "County": "Hopkins", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75471", "FIPS": "48223", "County": "Hopkins", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75478", "FIPS": "48223", "County": "Hopkins", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75481", "FIPS": "48223", "County": "Hopkins", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75482", "FIPS": "48223", "County": "Hopkins", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75483", "FIPS": "48223", "County": "Hopkins", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75835", "FIPS": "48225", "County": "Houston", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75844", "FIPS": "48225", "County": "Houston", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75847", "FIPS": "48225", "County": "Houston", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75849", "FIPS": "48225", "County": "Houston", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75851", "FIPS": "48225", "County": "Houston", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75858", "FIPS": "48225", "County": "Houston", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "79511", "FIPS": "48227", "County": "Howard", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79720", "FIPS": "48227", "County": "Howard", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79721", "FIPS": "48227", "County": "Howard", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79733", "FIPS": "48227", "County": "Howard", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79748", "FIPS": "48227", "County": "Howard", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79837", "FIPS": "48229", "County": "Hudspeth", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79839", "FIPS": "48229", "County": "Hudspeth", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79847", "FIPS": "48229", "County": "Hudspeth", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "79851", "FIPS": "48229", "County": "Hudspeth", "State": "TX", "DMA": {"Code": 765, "Name": "EL PASO"}},
  {"Postal_Code": "75135", "FIPS": "48231", "County": "Hunt", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75401", "FIPS": "48231", "County": "Hunt", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75402", "FIPS": "48231", "County": "Hunt", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75403", "FIPS": "48231", "County": "Hunt", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75404", "FIPS": "48231", "County": "Hunt", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75422", "FIPS": "48231", "County": "Hunt", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75423", "FIPS": "48231", "County": "Hunt", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75428", "FIPS": "48231", "County": "Hunt", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75429", "FIPS": "48231", "County": "Hunt", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75453", "FIPS": "48231", "County": "Hunt", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75458", "FIPS": "48231", "County": "Hunt", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75474", "FIPS": "48231", "County": "Hunt", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75496", "FIPS": "48231", "County": "Hunt", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "79007", "FIPS": "48233", "County": "Hutchinson", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79008", "FIPS": "48233", "County": "Hutchinson", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79036", "FIPS": "48233", "County": "Hutchinson", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79078", "FIPS": "48233", "County": "Hutchinson", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79083", "FIPS": "48233", "County": "Hutchinson", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "76930", "FIPS": "48235", "County": "Irion", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76941", "FIPS": "48235", "County": "Irion", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76427", "FIPS": "48237", "County": "Jack", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76458", "FIPS": "48237", "County": "Jack", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76459", "FIPS": "48237", "County": "Jack", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76486", "FIPS": "48237", "County": "Jack", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "77957", "FIPS": "48239", "County": "Jackson", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77961", "FIPS": "48239", "County": "Jackson", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77962", "FIPS": "48239", "County": "Jackson", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77969", "FIPS": "48239", "County": "Jackson", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77970", "FIPS": "48239", "County": "Jackson", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77971", "FIPS": "48239", "County": "Jackson", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77991", "FIPS": "48239", "County": "Jackson", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "75931", "FIPS": "48241", "County": "Jasper", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "75951", "FIPS": "48241", "County": "Jasper", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "75956", "FIPS": "48241", "County": "Jasper", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77612", "FIPS": "48241", "County": "Jasper", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77615", "FIPS": "48241", "County": "Jasper", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "79734", "FIPS": "48243", "County": "Jeff Davis", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "77613", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77619", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77622", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77627", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77629", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77640", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77641", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77642", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77643", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77651", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77655", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77701", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77702", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77703", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77704", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77705", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77706", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77707", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77708", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77709", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77710", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77713", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77720", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77725", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77726", "FIPS": "48245", "County": "Jefferson", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "78360", "FIPS": "48247", "County": "Jim Hogg", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78361", "FIPS": "48247", "County": "Jim Hogg", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78332", "FIPS": "48249", "County": "Jim Wells", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78333", "FIPS": "48249", "County": "Jim Wells", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78342", "FIPS": "48249", "County": "Jim Wells", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78372", "FIPS": "48249", "County": "Jim Wells", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78375", "FIPS": "48249", "County": "Jim Wells", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78383", "FIPS": "48249", "County": "Jim Wells", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "76009", "FIPS": "48251", "County": "Johnson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76028", "FIPS": "48251", "County": "Johnson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76031", "FIPS": "48251", "County": "Johnson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76033", "FIPS": "48251", "County": "Johnson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76044", "FIPS": "48251", "County": "Johnson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76050", "FIPS": "48251", "County": "Johnson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76058", "FIPS": "48251", "County": "Johnson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76059", "FIPS": "48251", "County": "Johnson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76061", "FIPS": "48251", "County": "Johnson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76084", "FIPS": "48251", "County": "Johnson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76093", "FIPS": "48251", "County": "Johnson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76097", "FIPS": "48251", "County": "Johnson", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "79501", "FIPS": "48253", "County": "Jones", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79503", "FIPS": "48253", "County": "Jones", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79520", "FIPS": "48253", "County": "Jones", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79525", "FIPS": "48253", "County": "Jones", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79533", "FIPS": "48253", "County": "Jones", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79553", "FIPS": "48253", "County": "Jones", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "78111", "FIPS": "48255", "County": "Karnes", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78113", "FIPS": "48255", "County": "Karnes", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78116", "FIPS": "48255", "County": "Karnes", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78117", "FIPS": "48255", "County": "Karnes", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78118", "FIPS": "48255", "County": "Karnes", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78119", "FIPS": "48255", "County": "Karnes", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78144", "FIPS": "48255", "County": "Karnes", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78151", "FIPS": "48255", "County": "Karnes", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "75114", "FIPS": "48257", "County": "Kaufman", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75118", "FIPS": "48257", "County": "Kaufman", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75126", "FIPS": "48257", "County": "Kaufman", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75142", "FIPS": "48257", "County": "Kaufman", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75143", "FIPS": "48257", "County": "Kaufman", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75147", "FIPS": "48257", "County": "Kaufman", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75157", "FIPS": "48257", "County": "Kaufman", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75158", "FIPS": "48257", "County": "Kaufman", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75160", "FIPS": "48257", "County": "Kaufman", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75161", "FIPS": "48257", "County": "Kaufman", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "78004", "FIPS": "48259", "County": "Kendall", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78006", "FIPS": "48259", "County": "Kendall", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78013", "FIPS": "48259", "County": "Kendall", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78027", "FIPS": "48259", "County": "Kendall", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78074", "FIPS": "48259", "County": "Kendall", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78338", "FIPS": "48261", "County": "Kenedy", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78385", "FIPS": "48261", "County": "Kenedy", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "79518", "FIPS": "48263", "County": "Kent", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79528", "FIPS": "48263", "County": "Kent", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "78010", "FIPS": "48265", "County": "Kerr", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78024", "FIPS": "48265", "County": "Kerr", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78025", "FIPS": "48265", "County": "Kerr", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78028", "FIPS": "48265", "County": "Kerr", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78029", "FIPS": "48265", "County": "Kerr", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78058", "FIPS": "48265", "County": "Kerr", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "76849", "FIPS": "48267", "County": "Kimble", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76854", "FIPS": "48267", "County": "Kimble", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76874", "FIPS": "48267", "County": "Kimble", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "79236", "FIPS": "48269", "County": "King", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "78832", "FIPS": "48271", "County": "Kinney", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78363", "FIPS": "48273", "County": "Kleberg", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78364", "FIPS": "48273", "County": "Kleberg", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78379", "FIPS": "48273", "County": "Kleberg", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "76363", "FIPS": "48275", "County": "Knox", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76371", "FIPS": "48275", "County": "Knox", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79505", "FIPS": "48275", "County": "Knox", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79529", "FIPS": "48275", "County": "Knox", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "75411", "FIPS": "48277", "County": "Lamar", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75416", "FIPS": "48277", "County": "Lamar", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75421", "FIPS": "48277", "County": "Lamar", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75425", "FIPS": "48277", "County": "Lamar", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75434", "FIPS": "48277", "County": "Lamar", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75435", "FIPS": "48277", "County": "Lamar", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75460", "FIPS": "48277", "County": "Lamar", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75461", "FIPS": "48277", "County": "Lamar", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75462", "FIPS": "48277", "County": "Lamar", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75468", "FIPS": "48277", "County": "Lamar", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75470", "FIPS": "48277", "County": "Lamar", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75473", "FIPS": "48277", "County": "Lamar", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75477", "FIPS": "48277", "County": "Lamar", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75486", "FIPS": "48277", "County": "Lamar", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "79031", "FIPS": "48279", "County": "Lamb", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79064", "FIPS": "48279", "County": "Lamb", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79082", "FIPS": "48279", "County": "Lamb", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79312", "FIPS": "48279", "County": "Lamb", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79326", "FIPS": "48279", "County": "Lamb", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79339", "FIPS": "48279", "County": "Lamb", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79369", "FIPS": "48279", "County": "Lamb", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79371", "FIPS": "48279", "County": "Lamb", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "76539", "FIPS": "48281", "County": "Lampasas", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76550", "FIPS": "48281", "County": "Lampasas", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76853", "FIPS": "48281", "County": "Lampasas", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "78001", "FIPS": "48283", "County": "La Salle", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78014", "FIPS": "48283", "County": "La Salle", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78019", "FIPS": "48283", "County": "La Salle", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78021", "FIPS": "48283", "County": "La Salle", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "77964", "FIPS": "48285", "County": "Lavaca", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "77975", "FIPS": "48285", "County": "Lavaca", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "77984", "FIPS": "48285", "County": "Lavaca", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "77986", "FIPS": "48285", "County": "Lavaca", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "77987", "FIPS": "48285", "County": "Lavaca", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "77995", "FIPS": "48285", "County": "Lavaca", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "77853", "FIPS": "48287", "County": "Lee", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78942", "FIPS": "48287", "County": "Lee", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78947", "FIPS": "48287", "County": "Lee", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78948", "FIPS": "48287", "County": "Lee", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "75831", "FIPS": "48289", "County": "Leon", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "75833", "FIPS": "48289", "County": "Leon", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "75846", "FIPS": "48289", "County": "Leon", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "75850", "FIPS": "48289", "County": "Leon", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "75855", "FIPS": "48289", "County": "Leon", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77850", "FIPS": "48289", "County": "Leon", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77855", "FIPS": "48289", "County": "Leon", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77865", "FIPS": "48289", "County": "Leon", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77871", "FIPS": "48289", "County": "Leon", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77327", "FIPS": "48291", "County": "Liberty", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77328", "FIPS": "48291", "County": "Liberty", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77368", "FIPS": "48291", "County": "Liberty", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77369", "FIPS": "48291", "County": "Liberty", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77533", "FIPS": "48291", "County": "Liberty", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77535", "FIPS": "48291", "County": "Liberty", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77538", "FIPS": "48291", "County": "Liberty", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77561", "FIPS": "48291", "County": "Liberty", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77564", "FIPS": "48291", "County": "Liberty", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77575", "FIPS": "48291", "County": "Liberty", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77582", "FIPS": "48291", "County": "Liberty", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "76635", "FIPS": "48293", "County": "Limestone", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76642", "FIPS": "48293", "County": "Limestone", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76653", "FIPS": "48293", "County": "Limestone", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76667", "FIPS": "48293", "County": "Limestone", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76678", "FIPS": "48293", "County": "Limestone", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76686", "FIPS": "48293", "County": "Limestone", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76687", "FIPS": "48293", "County": "Limestone", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "79005", "FIPS": "48295", "County": "Lipscomb", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79024", "FIPS": "48295", "County": "Lipscomb", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79034", "FIPS": "48295", "County": "Lipscomb", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79046", "FIPS": "48295", "County": "Lipscomb", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79056", "FIPS": "48295", "County": "Lipscomb", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "78022", "FIPS": "48297", "County": "Live Oak", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78060", "FIPS": "48297", "County": "Live Oak", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78071", "FIPS": "48297", "County": "Live Oak", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78075", "FIPS": "48297", "County": "Live Oak", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78350", "FIPS": "48297", "County": "Live Oak", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "76831", "FIPS": "48299", "County": "Llano", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "76885", "FIPS": "48299", "County": "Llano", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78607", "FIPS": "48299", "County": "Llano", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78609", "FIPS": "48299", "County": "Llano", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78639", "FIPS": "48299", "County": "Llano", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78643", "FIPS": "48299", "County": "Llano", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78657", "FIPS": "48299", "County": "Llano", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78672", "FIPS": "48299", "County": "Llano", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "79754", "FIPS": "48301", "County": "Loving", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79329", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79350", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79363", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79364", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79366", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79382", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79401", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79402", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79403", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79404", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79405", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79406", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79407", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79408", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79409", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79410", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79411", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79412", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79413", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79414", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79415", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79416", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79423", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79424", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79430", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79452", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79453", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79457", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79464", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79490", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79491", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79493", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79499", "FIPS": "48303", "County": "Lubbock", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79351", "FIPS": "48305", "County": "Lynn", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79373", "FIPS": "48305", "County": "Lynn", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79381", "FIPS": "48305", "County": "Lynn", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79383", "FIPS": "48305", "County": "Lynn", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "76825", "FIPS": "48307", "County": "McCulloch", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76836", "FIPS": "48307", "County": "McCulloch", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76852", "FIPS": "48307", "County": "McCulloch", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76858", "FIPS": "48307", "County": "McCulloch", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76872", "FIPS": "48307", "County": "McCulloch", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76887", "FIPS": "48307", "County": "McCulloch", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76524", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76557", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76624", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76630", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76633", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76638", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76640", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76643", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76654", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76655", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76657", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76664", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76682", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76684", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76691", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76701", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76702", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76703", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76704", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76705", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76706", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76707", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76708", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76710", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76711", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76712", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76714", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76715", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76716", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76795", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76797", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76798", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76799", "FIPS": "48309", "County": "McLennan", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "78007", "FIPS": "48311", "County": "McMullen", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78072", "FIPS": "48311", "County": "McMullen", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "75852", "FIPS": "48313", "County": "Madison", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77864", "FIPS": "48313", "County": "Madison", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77872", "FIPS": "48313", "County": "Madison", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "75564", "FIPS": "48315", "County": "Marion", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75657", "FIPS": "48315", "County": "Marion", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "79713", "FIPS": "48317", "County": "Martin", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79749", "FIPS": "48317", "County": "Martin", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79782", "FIPS": "48317", "County": "Martin", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79783", "FIPS": "48317", "County": "Martin", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "76820", "FIPS": "48319", "County": "Mason", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "76842", "FIPS": "48319", "County": "Mason", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "76856", "FIPS": "48319", "County": "Mason", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "76869", "FIPS": "48319", "County": "Mason", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "77404", "FIPS": "48321", "County": "Matagorda", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77414", "FIPS": "48321", "County": "Matagorda", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77415", "FIPS": "48321", "County": "Matagorda", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77419", "FIPS": "48321", "County": "Matagorda", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77428", "FIPS": "48321", "County": "Matagorda", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77440", "FIPS": "48321", "County": "Matagorda", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77456", "FIPS": "48321", "County": "Matagorda", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77457", "FIPS": "48321", "County": "Matagorda", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77458", "FIPS": "48321", "County": "Matagorda", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77465", "FIPS": "48321", "County": "Matagorda", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77468", "FIPS": "48321", "County": "Matagorda", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77482", "FIPS": "48321", "County": "Matagorda", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77483", "FIPS": "48321", "County": "Matagorda", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "78852", "FIPS": "48323", "County": "Maverick", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78853", "FIPS": "48323", "County": "Maverick", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78860", "FIPS": "48323", "County": "Maverick", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78877", "FIPS": "48323", "County": "Maverick", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78009", "FIPS": "48325", "County": "Medina", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78016", "FIPS": "48325", "County": "Medina", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78039", "FIPS": "48325", "County": "Medina", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78056", "FIPS": "48325", "County": "Medina", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78059", "FIPS": "48325", "County": "Medina", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78066", "FIPS": "48325", "County": "Medina", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78850", "FIPS": "48325", "County": "Medina", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78861", "FIPS": "48325", "County": "Medina", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78886", "FIPS": "48325", "County": "Medina", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "76841", "FIPS": "48327", "County": "Menard", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76848", "FIPS": "48327", "County": "Menard", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76859", "FIPS": "48327", "County": "Menard", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "79701", "FIPS": "48329", "County": "Midland", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79702", "FIPS": "48329", "County": "Midland", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79703", "FIPS": "48329", "County": "Midland", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79704", "FIPS": "48329", "County": "Midland", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79705", "FIPS": "48329", "County": "Midland", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79706", "FIPS": "48329", "County": "Midland", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79707", "FIPS": "48329", "County": "Midland", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79708", "FIPS": "48329", "County": "Midland", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79710", "FIPS": "48329", "County": "Midland", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79711", "FIPS": "48329", "County": "Midland", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79712", "FIPS": "48329", "County": "Midland", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "76518", "FIPS": "48331", "County": "Milam", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76519", "FIPS": "48331", "County": "Milam", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76520", "FIPS": "48331", "County": "Milam", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76523", "FIPS": "48331", "County": "Milam", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76556", "FIPS": "48331", "County": "Milam", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76567", "FIPS": "48331", "County": "Milam", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76577", "FIPS": "48331", "County": "Milam", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77857", "FIPS": "48331", "County": "Milam", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76844", "FIPS": "48333", "County": "Mills", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76864", "FIPS": "48333", "County": "Mills", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76870", "FIPS": "48333", "County": "Mills", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76880", "FIPS": "48333", "County": "Mills", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "79512", "FIPS": "48335", "County": "Mitchell", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79532", "FIPS": "48335", "County": "Mitchell", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79565", "FIPS": "48335", "County": "Mitchell", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76230", "FIPS": "48337", "County": "Montague", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76239", "FIPS": "48337", "County": "Montague", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76251", "FIPS": "48337", "County": "Montague", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76255", "FIPS": "48337", "County": "Montague", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76261", "FIPS": "48337", "County": "Montague", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76265", "FIPS": "48337", "County": "Montague", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76270", "FIPS": "48337", "County": "Montague", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "77301", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77302", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77303", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77304", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77305", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77306", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77316", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77318", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77333", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77353", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77354", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77355", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77356", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77357", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77362", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77365", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77372", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77378", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77380", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77381", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77382", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77384", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77385", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77386", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77387", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77393", "FIPS": "48339", "County": "Montgomery", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "79013", "FIPS": "48341", "County": "Moore", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79029", "FIPS": "48341", "County": "Moore", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79058", "FIPS": "48341", "County": "Moore", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79086", "FIPS": "48341", "County": "Moore", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "75568", "FIPS": "48343", "County": "Morris", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75571", "FIPS": "48343", "County": "Morris", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75636", "FIPS": "48343", "County": "Morris", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75638", "FIPS": "48343", "County": "Morris", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75668", "FIPS": "48343", "County": "Morris", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "79234", "FIPS": "48345", "County": "Motley", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79244", "FIPS": "48345", "County": "Motley", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79256", "FIPS": "48345", "County": "Motley", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "75760", "FIPS": "48347", "County": "Nacogdoches", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75788", "FIPS": "48347", "County": "Nacogdoches", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75937", "FIPS": "48347", "County": "Nacogdoches", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75943", "FIPS": "48347", "County": "Nacogdoches", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75944", "FIPS": "48347", "County": "Nacogdoches", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75946", "FIPS": "48347", "County": "Nacogdoches", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75958", "FIPS": "48347", "County": "Nacogdoches", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75961", "FIPS": "48347", "County": "Nacogdoches", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75962", "FIPS": "48347", "County": "Nacogdoches", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75963", "FIPS": "48347", "County": "Nacogdoches", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75964", "FIPS": "48347", "County": "Nacogdoches", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75965", "FIPS": "48347", "County": "Nacogdoches", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75978", "FIPS": "48347", "County": "Nacogdoches", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75102", "FIPS": "48349", "County": "Navarro", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75105", "FIPS": "48349", "County": "Navarro", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75109", "FIPS": "48349", "County": "Navarro", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75110", "FIPS": "48349", "County": "Navarro", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75144", "FIPS": "48349", "County": "Navarro", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75151", "FIPS": "48349", "County": "Navarro", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75153", "FIPS": "48349", "County": "Navarro", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75155", "FIPS": "48349", "County": "Navarro", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76626", "FIPS": "48349", "County": "Navarro", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76639", "FIPS": "48349", "County": "Navarro", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76641", "FIPS": "48349", "County": "Navarro", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76679", "FIPS": "48349", "County": "Navarro", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76681", "FIPS": "48349", "County": "Navarro", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75928", "FIPS": "48351", "County": "Newton", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "75932", "FIPS": "48351", "County": "Newton", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "75933", "FIPS": "48351", "County": "Newton", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "75966", "FIPS": "48351", "County": "Newton", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "75977", "FIPS": "48351", "County": "Newton", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77614", "FIPS": "48351", "County": "Newton", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "79506", "FIPS": "48353", "County": "Nolan", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79535", "FIPS": "48353", "County": "Nolan", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79537", "FIPS": "48353", "County": "Nolan", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79545", "FIPS": "48353", "County": "Nolan", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79556", "FIPS": "48353", "County": "Nolan", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "78330", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78339", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78343", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78347", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78351", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78373", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78380", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78401", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78402", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78403", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78404", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78405", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78406", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78407", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78408", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78409", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78410", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78411", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78412", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78413", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78414", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78415", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78416", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78417", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78418", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78419", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78426", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78427", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78460", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78461", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78463", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78465", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78466", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78467", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78468", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78469", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78470", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78471", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78472", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78473", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78474", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78475", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78476", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78477", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78478", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78480", "FIPS": "48355", "County": "Nueces", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "79033", "FIPS": "48357", "County": "Ochiltree", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79070", "FIPS": "48357", "County": "Ochiltree", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79093", "FIPS": "48357", "County": "Ochiltree", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79001", "FIPS": "48359", "County": "Oldham", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79010", "FIPS": "48359", "County": "Oldham", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79092", "FIPS": "48359", "County": "Oldham", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79098", "FIPS": "48359", "County": "Oldham", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "77611", "FIPS": "48361", "County": "Orange", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77626", "FIPS": "48361", "County": "Orange", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77630", "FIPS": "48361", "County": "Orange", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77631", "FIPS": "48361", "County": "Orange", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77632", "FIPS": "48361", "County": "Orange", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77639", "FIPS": "48361", "County": "Orange", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77662", "FIPS": "48361", "County": "Orange", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77670", "FIPS": "48361", "County": "Orange", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "76067", "FIPS": "48363", "County": "Palo Pinto", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76068", "FIPS": "48363", "County": "Palo Pinto", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76449", "FIPS": "48363", "County": "Palo Pinto", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76453", "FIPS": "48363", "County": "Palo Pinto", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76463", "FIPS": "48363", "County": "Palo Pinto", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76472", "FIPS": "48363", "County": "Palo Pinto", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76475", "FIPS": "48363", "County": "Palo Pinto", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76484", "FIPS": "48363", "County": "Palo Pinto", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75631", "FIPS": "48365", "County": "Panola", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75633", "FIPS": "48365", "County": "Panola", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75637", "FIPS": "48365", "County": "Panola", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75639", "FIPS": "48365", "County": "Panola", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75643", "FIPS": "48365", "County": "Panola", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75669", "FIPS": "48365", "County": "Panola", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75685", "FIPS": "48365", "County": "Panola", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "76008", "FIPS": "48367", "County": "Parker", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76066", "FIPS": "48367", "County": "Parker", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76082", "FIPS": "48367", "County": "Parker", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76085", "FIPS": "48367", "County": "Parker", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76086", "FIPS": "48367", "County": "Parker", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76087", "FIPS": "48367", "County": "Parker", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76088", "FIPS": "48367", "County": "Parker", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76098", "FIPS": "48367", "County": "Parker", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76439", "FIPS": "48367", "County": "Parker", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76485", "FIPS": "48367", "County": "Parker", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76487", "FIPS": "48367", "County": "Parker", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76490", "FIPS": "48367", "County": "Parker", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "79009", "FIPS": "48369", "County": "Parmer", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79035", "FIPS": "48369", "County": "Parmer", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79053", "FIPS": "48369", "County": "Parmer", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79325", "FIPS": "48369", "County": "Parmer", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79730", "FIPS": "48371", "County": "Pecos", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79735", "FIPS": "48371", "County": "Pecos", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79740", "FIPS": "48371", "County": "Pecos", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79743", "FIPS": "48371", "County": "Pecos", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79744", "FIPS": "48371", "County": "Pecos", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79781", "FIPS": "48371", "County": "Pecos", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "75934", "FIPS": "48373", "County": "Polk", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "75939", "FIPS": "48373", "County": "Polk", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "75960", "FIPS": "48373", "County": "Polk", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77326", "FIPS": "48373", "County": "Polk", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77332", "FIPS": "48373", "County": "Polk", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77335", "FIPS": "48373", "County": "Polk", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77350", "FIPS": "48373", "County": "Polk", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77351", "FIPS": "48373", "County": "Polk", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77360", "FIPS": "48373", "County": "Polk", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77399", "FIPS": "48373", "County": "Polk", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "79012", "FIPS": "48375", "County": "Potter", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79101", "FIPS": "48375", "County": "Potter", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79102", "FIPS": "48375", "County": "Potter", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79103", "FIPS": "48375", "County": "Potter", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79104", "FIPS": "48375", "County": "Potter", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79105", "FIPS": "48375", "County": "Potter", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79106", "FIPS": "48375", "County": "Potter", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79107", "FIPS": "48375", "County": "Potter", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79108", "FIPS": "48375", "County": "Potter", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79111", "FIPS": "48375", "County": "Potter", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79116", "FIPS": "48375", "County": "Potter", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79117", "FIPS": "48375", "County": "Potter", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79120", "FIPS": "48375", "County": "Potter", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79124", "FIPS": "48375", "County": "Potter", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79159", "FIPS": "48375", "County": "Potter", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79166", "FIPS": "48375", "County": "Potter", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79168", "FIPS": "48375", "County": "Potter", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79172", "FIPS": "48375", "County": "Potter", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79174", "FIPS": "48375", "County": "Potter", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79178", "FIPS": "48375", "County": "Potter", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79185", "FIPS": "48375", "County": "Potter", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79187", "FIPS": "48375", "County": "Potter", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79189", "FIPS": "48375", "County": "Potter", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79843", "FIPS": "48377", "County": "Presidio", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79845", "FIPS": "48377", "County": "Presidio", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79846", "FIPS": "48377", "County": "Presidio", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79854", "FIPS": "48377", "County": "Presidio", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "75440", "FIPS": "48379", "County": "Rains", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75472", "FIPS": "48379", "County": "Rains", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "79015", "FIPS": "48381", "County": "Randall", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79016", "FIPS": "48381", "County": "Randall", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79091", "FIPS": "48381", "County": "Randall", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79109", "FIPS": "48381", "County": "Randall", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79110", "FIPS": "48381", "County": "Randall", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79114", "FIPS": "48381", "County": "Randall", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79118", "FIPS": "48381", "County": "Randall", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79119", "FIPS": "48381", "County": "Randall", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79121", "FIPS": "48381", "County": "Randall", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "76932", "FIPS": "48383", "County": "Reagan", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "78833", "FIPS": "48385", "County": "Real", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78873", "FIPS": "48385", "County": "Real", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78879", "FIPS": "48385", "County": "Real", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "75412", "FIPS": "48387", "County": "Red River", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75417", "FIPS": "48387", "County": "Red River", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75426", "FIPS": "48387", "County": "Red River", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75436", "FIPS": "48387", "County": "Red River", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75550", "FIPS": "48387", "County": "Red River", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75554", "FIPS": "48387", "County": "Red River", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "79718", "FIPS": "48389", "County": "Reeves", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79770", "FIPS": "48389", "County": "Reeves", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79772", "FIPS": "48389", "County": "Reeves", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79780", "FIPS": "48389", "County": "Reeves", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79785", "FIPS": "48389", "County": "Reeves", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79786", "FIPS": "48389", "County": "Reeves", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "77950", "FIPS": "48391", "County": "Refugio", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "77990", "FIPS": "48391", "County": "Refugio", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78340", "FIPS": "48391", "County": "Refugio", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78377", "FIPS": "48391", "County": "Refugio", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78393", "FIPS": "48391", "County": "Refugio", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "79059", "FIPS": "48393", "County": "Roberts", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "76629", "FIPS": "48395", "County": "Robertson", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77837", "FIPS": "48395", "County": "Robertson", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77856", "FIPS": "48395", "County": "Robertson", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77859", "FIPS": "48395", "County": "Robertson", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77867", "FIPS": "48395", "County": "Robertson", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77870", "FIPS": "48395", "County": "Robertson", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "77882", "FIPS": "48395", "County": "Robertson", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "75032", "FIPS": "48397", "County": "Rockwall", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75087", "FIPS": "48397", "County": "Rockwall", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75132", "FIPS": "48397", "County": "Rockwall", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75189", "FIPS": "48397", "County": "Rockwall", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76821", "FIPS": "48399", "County": "Runnels", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76861", "FIPS": "48399", "County": "Runnels", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76865", "FIPS": "48399", "County": "Runnels", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76875", "FIPS": "48399", "County": "Runnels", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79566", "FIPS": "48399", "County": "Runnels", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79567", "FIPS": "48399", "County": "Runnels", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "75652", "FIPS": "48401", "County": "Rusk", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75653", "FIPS": "48401", "County": "Rusk", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75654", "FIPS": "48401", "County": "Rusk", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75658", "FIPS": "48401", "County": "Rusk", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75666", "FIPS": "48401", "County": "Rusk", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75667", "FIPS": "48401", "County": "Rusk", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75680", "FIPS": "48401", "County": "Rusk", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75681", "FIPS": "48401", "County": "Rusk", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75682", "FIPS": "48401", "County": "Rusk", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75684", "FIPS": "48401", "County": "Rusk", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75687", "FIPS": "48401", "County": "Rusk", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75689", "FIPS": "48401", "County": "Rusk", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75691", "FIPS": "48401", "County": "Rusk", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75930", "FIPS": "48403", "County": "Sabine", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75948", "FIPS": "48403", "County": "Sabine", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75959", "FIPS": "48403", "County": "Sabine", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75968", "FIPS": "48403", "County": "Sabine", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75929", "FIPS": "48405", "County": "San Augustine", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75972", "FIPS": "48405", "County": "San Augustine", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "77331", "FIPS": "48407", "County": "San Jacinto", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77359", "FIPS": "48407", "County": "San Jacinto", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77364", "FIPS": "48407", "County": "San Jacinto", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77371", "FIPS": "48407", "County": "San Jacinto", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "78335", "FIPS": "48409", "County": "San Patricio", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78336", "FIPS": "48409", "County": "San Patricio", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78352", "FIPS": "48409", "County": "San Patricio", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78359", "FIPS": "48409", "County": "San Patricio", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78362", "FIPS": "48409", "County": "San Patricio", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78368", "FIPS": "48409", "County": "San Patricio", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78370", "FIPS": "48409", "County": "San Patricio", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78374", "FIPS": "48409", "County": "San Patricio", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78387", "FIPS": "48409", "County": "San Patricio", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "78390", "FIPS": "48409", "County": "San Patricio", "State": "TX", "DMA": {"Code": 600, "Name": "CORPUS CHRISTI"}},
  {"Postal_Code": "76824", "FIPS": "48411", "County": "San Saba", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76832", "FIPS": "48411", "County": "San Saba", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76871", "FIPS": "48411", "County": "San Saba", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76877", "FIPS": "48411", "County": "San Saba", "State": "TX", "DMA": {"Code": 625, "Name": "WACO-TEMPLE-BRYAN"}},
  {"Postal_Code": "76936", "FIPS": "48413", "County": "Schleicher", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "79516", "FIPS": "48415", "County": "Scurry", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79517", "FIPS": "48415", "County": "Scurry", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79526", "FIPS": "48415", "County": "Scurry", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79527", "FIPS": "48415", "County": "Scurry", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79549", "FIPS": "48415", "County": "Scurry", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79550", "FIPS": "48415", "County": "Scurry", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76430", "FIPS": "48417", "County": "Shackelford", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76464", "FIPS": "48417", "County": "Shackelford", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "75935", "FIPS": "48419", "County": "Shelby", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75954", "FIPS": "48419", "County": "Shelby", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75973", "FIPS": "48419", "County": "Shelby", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75974", "FIPS": "48419", "County": "Shelby", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75975", "FIPS": "48419", "County": "Shelby", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "79084", "FIPS": "48421", "County": "Sherman", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "75701", "FIPS": "48423", "County": "Smith", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75702", "FIPS": "48423", "County": "Smith", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75703", "FIPS": "48423", "County": "Smith", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75704", "FIPS": "48423", "County": "Smith", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75705", "FIPS": "48423", "County": "Smith", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75706", "FIPS": "48423", "County": "Smith", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75707", "FIPS": "48423", "County": "Smith", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75708", "FIPS": "48423", "County": "Smith", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75709", "FIPS": "48423", "County": "Smith", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75710", "FIPS": "48423", "County": "Smith", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75711", "FIPS": "48423", "County": "Smith", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75712", "FIPS": "48423", "County": "Smith", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75713", "FIPS": "48423", "County": "Smith", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75750", "FIPS": "48423", "County": "Smith", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75757", "FIPS": "48423", "County": "Smith", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75762", "FIPS": "48423", "County": "Smith", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75771", "FIPS": "48423", "County": "Smith", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75789", "FIPS": "48423", "County": "Smith", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75791", "FIPS": "48423", "County": "Smith", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75792", "FIPS": "48423", "County": "Smith", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75798", "FIPS": "48423", "County": "Smith", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75799", "FIPS": "48423", "County": "Smith", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "76043", "FIPS": "48425", "County": "Somervell", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76070", "FIPS": "48425", "County": "Somervell", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76077", "FIPS": "48425", "County": "Somervell", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "78536", "FIPS": "48427", "County": "Starr", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78545", "FIPS": "48427", "County": "Starr", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78547", "FIPS": "48427", "County": "Starr", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78548", "FIPS": "48427", "County": "Starr", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78582", "FIPS": "48427", "County": "Starr", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78584", "FIPS": "48427", "County": "Starr", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78585", "FIPS": "48427", "County": "Starr", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78588", "FIPS": "48427", "County": "Starr", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78591", "FIPS": "48427", "County": "Starr", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "76424", "FIPS": "48429", "County": "Stephens", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76429", "FIPS": "48429", "County": "Stephens", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76951", "FIPS": "48431", "County": "Sterling", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "79502", "FIPS": "48433", "County": "Stonewall", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79540", "FIPS": "48433", "County": "Stonewall", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76950", "FIPS": "48435", "County": "Sutton", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "79042", "FIPS": "48437", "County": "Swisher", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79052", "FIPS": "48437", "County": "Swisher", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79088", "FIPS": "48437", "County": "Swisher", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "76001", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76002", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76003", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76004", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76005", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76006", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76007", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76010", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76011", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76012", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76013", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76014", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76015", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76016", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76017", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76018", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76019", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76020", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76021", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76022", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76034", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76036", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76039", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76040", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76051", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76052", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76053", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76054", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76060", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76063", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76092", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76094", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76095", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76096", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76099", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76101", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76102", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76103", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76104", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76105", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76106", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76107", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76108", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76109", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76110", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76111", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76112", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76113", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76114", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76115", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76116", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76117", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76118", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76119", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76120", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76121", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76122", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76123", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76124", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76126", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76127", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76129", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76130", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76131", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76132", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76133", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76134", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76135", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76136", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76137", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76140", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76147", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76148", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76150", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76155", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76161", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76162", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76163", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76164", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76177", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76179", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76180", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76181", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76182", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76185", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76191", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76192", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76193", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76195", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76196", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76197", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76198", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76199", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76244", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76248", "FIPS": "48439", "County": "Tarrant", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "79508", "FIPS": "48441", "County": "Taylor", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79530", "FIPS": "48441", "County": "Taylor", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79536", "FIPS": "48441", "County": "Taylor", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79541", "FIPS": "48441", "County": "Taylor", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79561", "FIPS": "48441", "County": "Taylor", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79562", "FIPS": "48441", "County": "Taylor", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79563", "FIPS": "48441", "County": "Taylor", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79601", "FIPS": "48441", "County": "Taylor", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79602", "FIPS": "48441", "County": "Taylor", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79603", "FIPS": "48441", "County": "Taylor", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79604", "FIPS": "48441", "County": "Taylor", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79605", "FIPS": "48441", "County": "Taylor", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79606", "FIPS": "48441", "County": "Taylor", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79607", "FIPS": "48441", "County": "Taylor", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79608", "FIPS": "48441", "County": "Taylor", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79697", "FIPS": "48441", "County": "Taylor", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79698", "FIPS": "48441", "County": "Taylor", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "79699", "FIPS": "48441", "County": "Taylor", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "78851", "FIPS": "48443", "County": "Terrell", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79848", "FIPS": "48443", "County": "Terrell", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79316", "FIPS": "48445", "County": "Terry", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79345", "FIPS": "48445", "County": "Terry", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79378", "FIPS": "48445", "County": "Terry", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "76483", "FIPS": "48447", "County": "Throckmorton", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "76491", "FIPS": "48447", "County": "Throckmorton", "State": "TX", "DMA": {"Code": 662, "Name": "ABILENE-SWEETWATER"}},
  {"Postal_Code": "75455", "FIPS": "48449", "County": "Titus", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75456", "FIPS": "48449", "County": "Titus", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75493", "FIPS": "48449", "County": "Titus", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "75558", "FIPS": "48449", "County": "Titus", "State": "TX", "DMA": {"Code": 612, "Name": "SHREVEPORT"}},
  {"Postal_Code": "76886", "FIPS": "48451", "County": "Tom Green", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76901", "FIPS": "48451", "County": "Tom Green", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76902", "FIPS": "48451", "County": "Tom Green", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76903", "FIPS": "48451", "County": "Tom Green", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76904", "FIPS": "48451", "County": "Tom Green", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76905", "FIPS": "48451", "County": "Tom Green", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76906", "FIPS": "48451", "County": "Tom Green", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76908", "FIPS": "48451", "County": "Tom Green", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76909", "FIPS": "48451", "County": "Tom Green", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76934", "FIPS": "48451", "County": "Tom Green", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76935", "FIPS": "48451", "County": "Tom Green", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76939", "FIPS": "48451", "County": "Tom Green", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76940", "FIPS": "48451", "County": "Tom Green", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76955", "FIPS": "48451", "County": "Tom Green", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76957", "FIPS": "48451", "County": "Tom Green", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "76958", "FIPS": "48451", "County": "Tom Green", "State": "TX", "DMA": {"Code": 661, "Name": "SAN ANGELO"}},
  {"Postal_Code": "73301", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "73344", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78617", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78645", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78651", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78652", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78653", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78660", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78669", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78691", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78701", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78702", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78703", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78704", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78705", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78708", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78709", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78710", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78711", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78712", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78713", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78714", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78715", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78716", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78718", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78719", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78720", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78721", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78722", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78723", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78724", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78725", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78726", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78727", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78728", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78730", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78731", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78732", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78733", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78734", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78735", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78736", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78738", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78739", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78741", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78742", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78744", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78745", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78746", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78747", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78748", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78749", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78750", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78751", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78752", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78753", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78754", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78755", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78756", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78757", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78758", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78759", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78760", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78761", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78762", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78763", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78764", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78765", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78766", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78767", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78768", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78769", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78772", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78773", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78774", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78778", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78779", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78780", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78781", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78783", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78785", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78786", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78788", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78789", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78798", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78799", "FIPS": "48453", "County": "Travis", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "75834", "FIPS": "48455", "County": "Trinity", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "75845", "FIPS": "48455", "County": "Trinity", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "75856", "FIPS": "48455", "County": "Trinity", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "75862", "FIPS": "48455", "County": "Trinity", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "75865", "FIPS": "48455", "County": "Trinity", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "75926", "FIPS": "48455", "County": "Trinity", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "75936", "FIPS": "48457", "County": "Tyler", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "75938", "FIPS": "48457", "County": "Tyler", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "75942", "FIPS": "48457", "County": "Tyler", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "75979", "FIPS": "48457", "County": "Tyler", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "75990", "FIPS": "48457", "County": "Tyler", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77616", "FIPS": "48457", "County": "Tyler", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77624", "FIPS": "48457", "County": "Tyler", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77660", "FIPS": "48457", "County": "Tyler", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "77664", "FIPS": "48457", "County": "Tyler", "State": "TX", "DMA": {"Code": 692, "Name": "BEAUMONT-PORT ARTHUR"}},
  {"Postal_Code": "75640", "FIPS": "48459", "County": "Upshur", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75644", "FIPS": "48459", "County": "Upshur", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75645", "FIPS": "48459", "County": "Upshur", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75683", "FIPS": "48459", "County": "Upshur", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75755", "FIPS": "48459", "County": "Upshur", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75797", "FIPS": "48459", "County": "Upshur", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "79752", "FIPS": "48461", "County": "Upton", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79755", "FIPS": "48461", "County": "Upton", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79778", "FIPS": "48461", "County": "Upton", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "78801", "FIPS": "48463", "County": "Uvalde", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78802", "FIPS": "48463", "County": "Uvalde", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78838", "FIPS": "48463", "County": "Uvalde", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78870", "FIPS": "48463", "County": "Uvalde", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78881", "FIPS": "48463", "County": "Uvalde", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78884", "FIPS": "48463", "County": "Uvalde", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78837", "FIPS": "48465", "County": "Val Verde", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78840", "FIPS": "48465", "County": "Val Verde", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78841", "FIPS": "48465", "County": "Val Verde", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78842", "FIPS": "48465", "County": "Val Verde", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78843", "FIPS": "48465", "County": "Val Verde", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78847", "FIPS": "48465", "County": "Val Verde", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78871", "FIPS": "48465", "County": "Val Verde", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "75103", "FIPS": "48467", "County": "Van Zandt", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75117", "FIPS": "48467", "County": "Van Zandt", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75127", "FIPS": "48467", "County": "Van Zandt", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75140", "FIPS": "48467", "County": "Van Zandt", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75169", "FIPS": "48467", "County": "Van Zandt", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75754", "FIPS": "48467", "County": "Van Zandt", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75790", "FIPS": "48467", "County": "Van Zandt", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "77901", "FIPS": "48469", "County": "Victoria", "State": "TX", "DMA": {"Code": 626, "Name": "VICTORIA"}},
  {"Postal_Code": "77902", "FIPS": "48469", "County": "Victoria", "State": "TX", "DMA": {"Code": 626, "Name": "VICTORIA"}},
  {"Postal_Code": "77903", "FIPS": "48469", "County": "Victoria", "State": "TX", "DMA": {"Code": 626, "Name": "VICTORIA"}},
  {"Postal_Code": "77904", "FIPS": "48469", "County": "Victoria", "State": "TX", "DMA": {"Code": 626, "Name": "VICTORIA"}},
  {"Postal_Code": "77905", "FIPS": "48469", "County": "Victoria", "State": "TX", "DMA": {"Code": 626, "Name": "VICTORIA"}},
  {"Postal_Code": "77951", "FIPS": "48469", "County": "Victoria", "State": "TX", "DMA": {"Code": 626, "Name": "VICTORIA"}},
  {"Postal_Code": "77968", "FIPS": "48469", "County": "Victoria", "State": "TX", "DMA": {"Code": 626, "Name": "VICTORIA"}},
  {"Postal_Code": "77973", "FIPS": "48469", "County": "Victoria", "State": "TX", "DMA": {"Code": 626, "Name": "VICTORIA"}},
  {"Postal_Code": "77976", "FIPS": "48469", "County": "Victoria", "State": "TX", "DMA": {"Code": 626, "Name": "VICTORIA"}},
  {"Postal_Code": "77977", "FIPS": "48469", "County": "Victoria", "State": "TX", "DMA": {"Code": 626, "Name": "VICTORIA"}},
  {"Postal_Code": "77988", "FIPS": "48469", "County": "Victoria", "State": "TX", "DMA": {"Code": 626, "Name": "VICTORIA"}},
  {"Postal_Code": "77320", "FIPS": "48471", "County": "Walker", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77334", "FIPS": "48471", "County": "Walker", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77340", "FIPS": "48471", "County": "Walker", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77341", "FIPS": "48471", "County": "Walker", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77342", "FIPS": "48471", "County": "Walker", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77343", "FIPS": "48471", "County": "Walker", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77344", "FIPS": "48471", "County": "Walker", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77348", "FIPS": "48471", "County": "Walker", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77349", "FIPS": "48471", "County": "Walker", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77358", "FIPS": "48471", "County": "Walker", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77367", "FIPS": "48471", "County": "Walker", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77423", "FIPS": "48473", "County": "Waller", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77445", "FIPS": "48473", "County": "Waller", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77446", "FIPS": "48473", "County": "Waller", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77466", "FIPS": "48473", "County": "Waller", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "79719", "FIPS": "48475", "County": "Ward", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79742", "FIPS": "48475", "County": "Ward", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79756", "FIPS": "48475", "County": "Ward", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79777", "FIPS": "48475", "County": "Ward", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79788", "FIPS": "48475", "County": "Ward", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "77426", "FIPS": "48477", "County": "Washington", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77833", "FIPS": "48477", "County": "Washington", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77834", "FIPS": "48477", "County": "Washington", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77835", "FIPS": "48477", "County": "Washington", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77880", "FIPS": "48477", "County": "Washington", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "78040", "FIPS": "48479", "County": "Webb", "State": "TX", "DMA": {"Code": 749, "Name": "LAREDO"}},
  {"Postal_Code": "78041", "FIPS": "48479", "County": "Webb", "State": "TX", "DMA": {"Code": 749, "Name": "LAREDO"}},
  {"Postal_Code": "78042", "FIPS": "48479", "County": "Webb", "State": "TX", "DMA": {"Code": 749, "Name": "LAREDO"}},
  {"Postal_Code": "78043", "FIPS": "48479", "County": "Webb", "State": "TX", "DMA": {"Code": 749, "Name": "LAREDO"}},
  {"Postal_Code": "78044", "FIPS": "48479", "County": "Webb", "State": "TX", "DMA": {"Code": 749, "Name": "LAREDO"}},
  {"Postal_Code": "78045", "FIPS": "48479", "County": "Webb", "State": "TX", "DMA": {"Code": 749, "Name": "LAREDO"}},
  {"Postal_Code": "78046", "FIPS": "48479", "County": "Webb", "State": "TX", "DMA": {"Code": 749, "Name": "LAREDO"}},
  {"Postal_Code": "78049", "FIPS": "48479", "County": "Webb", "State": "TX", "DMA": {"Code": 749, "Name": "LAREDO"}},
  {"Postal_Code": "78344", "FIPS": "48479", "County": "Webb", "State": "TX", "DMA": {"Code": 749, "Name": "LAREDO"}},
  {"Postal_Code": "78369", "FIPS": "48479", "County": "Webb", "State": "TX", "DMA": {"Code": 749, "Name": "LAREDO"}},
  {"Postal_Code": "78371", "FIPS": "48479", "County": "Webb", "State": "TX", "DMA": {"Code": 749, "Name": "LAREDO"}},
  {"Postal_Code": "77420", "FIPS": "48481", "County": "Wharton", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77432", "FIPS": "48481", "County": "Wharton", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77435", "FIPS": "48481", "County": "Wharton", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77436", "FIPS": "48481", "County": "Wharton", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77437", "FIPS": "48481", "County": "Wharton", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77443", "FIPS": "48481", "County": "Wharton", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77448", "FIPS": "48481", "County": "Wharton", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77453", "FIPS": "48481", "County": "Wharton", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77454", "FIPS": "48481", "County": "Wharton", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77455", "FIPS": "48481", "County": "Wharton", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77467", "FIPS": "48481", "County": "Wharton", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "77488", "FIPS": "48481", "County": "Wharton", "State": "TX", "DMA": {"Code": 618, "Name": "HOUSTON"}},
  {"Postal_Code": "79003", "FIPS": "48483", "County": "Wheeler", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79011", "FIPS": "48483", "County": "Wheeler", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79061", "FIPS": "48483", "County": "Wheeler", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79079", "FIPS": "48483", "County": "Wheeler", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "79096", "FIPS": "48483", "County": "Wheeler", "State": "TX", "DMA": {"Code": 634, "Name": "AMARILLO"}},
  {"Postal_Code": "76301", "FIPS": "48485", "County": "Wichita", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76302", "FIPS": "48485", "County": "Wichita", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76305", "FIPS": "48485", "County": "Wichita", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76306", "FIPS": "48485", "County": "Wichita", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76307", "FIPS": "48485", "County": "Wichita", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76308", "FIPS": "48485", "County": "Wichita", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76309", "FIPS": "48485", "County": "Wichita", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76310", "FIPS": "48485", "County": "Wichita", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76311", "FIPS": "48485", "County": "Wichita", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76354", "FIPS": "48485", "County": "Wichita", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76360", "FIPS": "48485", "County": "Wichita", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76367", "FIPS": "48485", "County": "Wichita", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76369", "FIPS": "48485", "County": "Wichita", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76364", "FIPS": "48487", "County": "Wilbarger", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76373", "FIPS": "48487", "County": "Wilbarger", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76384", "FIPS": "48487", "County": "Wilbarger", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76385", "FIPS": "48487", "County": "Wilbarger", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "79247", "FIPS": "48487", "County": "Wilbarger", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "78561", "FIPS": "48489", "County": "Willacy", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78569", "FIPS": "48489", "County": "Willacy", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78580", "FIPS": "48489", "County": "Willacy", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78590", "FIPS": "48489", "County": "Willacy", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78594", "FIPS": "48489", "County": "Willacy", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "78598", "FIPS": "48489", "County": "Willacy", "State": "TX", "DMA": {"Code": 636, "Name": "HARLINGEN-WESLACO-BROWNSVILLE"}},
  {"Postal_Code": "76527", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "76530", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "76537", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "76573", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "76574", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "76578", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78613", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78615", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78626", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78627", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78628", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78630", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78634", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78641", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78642", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78646", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78664", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78673", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78674", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78680", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78681", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78682", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78683", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78717", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78729", "FIPS": "48491", "County": "Williamson", "State": "TX", "DMA": {"Code": 635, "Name": "AUSTIN"}},
  {"Postal_Code": "78114", "FIPS": "48493", "County": "Wilson", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78121", "FIPS": "48493", "County": "Wilson", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78143", "FIPS": "48493", "County": "Wilson", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78147", "FIPS": "48493", "County": "Wilson", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78160", "FIPS": "48493", "County": "Wilson", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78161", "FIPS": "48493", "County": "Wilson", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "79745", "FIPS": "48495", "County": "Winkler", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "79789", "FIPS": "48495", "County": "Winkler", "State": "TX", "DMA": {"Code": 633, "Name": "ODESSA-MIDLAND"}},
  {"Postal_Code": "76023", "FIPS": "48497", "County": "Wise", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76071", "FIPS": "48497", "County": "Wise", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76073", "FIPS": "48497", "County": "Wise", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76078", "FIPS": "48497", "County": "Wise", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76225", "FIPS": "48497", "County": "Wise", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76234", "FIPS": "48497", "County": "Wise", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76246", "FIPS": "48497", "County": "Wise", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76267", "FIPS": "48497", "County": "Wise", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76426", "FIPS": "48497", "County": "Wise", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "76431", "FIPS": "48497", "County": "Wise", "State": "TX", "DMA": {"Code": 623, "Name": "DALLAS-FORT WORTH"}},
  {"Postal_Code": "75410", "FIPS": "48499", "County": "Wood", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75444", "FIPS": "48499", "County": "Wood", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75494", "FIPS": "48499", "County": "Wood", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75497", "FIPS": "48499", "County": "Wood", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75765", "FIPS": "48499", "County": "Wood", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75773", "FIPS": "48499", "County": "Wood", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "75783", "FIPS": "48499", "County": "Wood", "State": "TX", "DMA": {"Code": 709, "Name": "TYLER-LONGVIEW (LFKN&NCGD  )"}},
  {"Postal_Code": "79323", "FIPS": "48501", "County": "Yoakum", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79355", "FIPS": "48501", "County": "Yoakum", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "79376", "FIPS": "48501", "County": "Yoakum", "State": "TX", "DMA": {"Code": 651, "Name": "LUBBOCK"}},
  {"Postal_Code": "76372", "FIPS": "48503", "County": "Young", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76374", "FIPS": "48503", "County": "Young", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76450", "FIPS": "48503", "County": "Young", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76460", "FIPS": "48503", "County": "Young", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "76481", "FIPS": "48503", "County": "Young", "State": "TX", "DMA": {"Code": 627, "Name": "WICHITA FALLS-LAWTON"}},
  {"Postal_Code": "78067", "FIPS": "48505", "County": "Zapata", "State": "TX", "DMA": {"Code": 749, "Name": "LAREDO"}},
  {"Postal_Code": "78076", "FIPS": "48505", "County": "Zapata", "State": "TX", "DMA": {"Code": 749, "Name": "LAREDO"}},
  {"Postal_Code": "78564", "FIPS": "48505", "County": "Zapata", "State": "TX", "DMA": {"Code": 749, "Name": "LAREDO"}},
  {"Postal_Code": "78829", "FIPS": "48507", "County": "Zavala", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78839", "FIPS": "48507", "County": "Zavala", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "78872", "FIPS": "48507", "County": "Zavala", "State": "TX", "DMA": {"Code": 641, "Name": "SAN ANTONIO"}},
  {"Postal_Code": "84713", "FIPS": "49001", "County": "Beaver", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84731", "FIPS": "49001", "County": "Beaver", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84751", "FIPS": "49001", "County": "Beaver", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84752", "FIPS": "49001", "County": "Beaver", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84301", "FIPS": "49003", "County": "Box Elder", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84302", "FIPS": "49003", "County": "Box Elder", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84306", "FIPS": "49003", "County": "Box Elder", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84307", "FIPS": "49003", "County": "Box Elder", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84309", "FIPS": "49003", "County": "Box Elder", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84311", "FIPS": "49003", "County": "Box Elder", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84312", "FIPS": "49003", "County": "Box Elder", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84313", "FIPS": "49003", "County": "Box Elder", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84314", "FIPS": "49003", "County": "Box Elder", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84316", "FIPS": "49003", "County": "Box Elder", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84324", "FIPS": "49003", "County": "Box Elder", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84329", "FIPS": "49003", "County": "Box Elder", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84330", "FIPS": "49003", "County": "Box Elder", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84331", "FIPS": "49003", "County": "Box Elder", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84334", "FIPS": "49003", "County": "Box Elder", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84336", "FIPS": "49003", "County": "Box Elder", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84337", "FIPS": "49003", "County": "Box Elder", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84340", "FIPS": "49003", "County": "Box Elder", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84304", "FIPS": "49005", "County": "Cache", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84305", "FIPS": "49005", "County": "Cache", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84308", "FIPS": "49005", "County": "Cache", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84318", "FIPS": "49005", "County": "Cache", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84319", "FIPS": "49005", "County": "Cache", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84320", "FIPS": "49005", "County": "Cache", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84321", "FIPS": "49005", "County": "Cache", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84322", "FIPS": "49005", "County": "Cache", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84323", "FIPS": "49005", "County": "Cache", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84325", "FIPS": "49005", "County": "Cache", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84326", "FIPS": "49005", "County": "Cache", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84327", "FIPS": "49005", "County": "Cache", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84328", "FIPS": "49005", "County": "Cache", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84332", "FIPS": "49005", "County": "Cache", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84333", "FIPS": "49005", "County": "Cache", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84335", "FIPS": "49005", "County": "Cache", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84338", "FIPS": "49005", "County": "Cache", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84339", "FIPS": "49005", "County": "Cache", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84341", "FIPS": "49005", "County": "Cache", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84501", "FIPS": "49007", "County": "Carbon", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84520", "FIPS": "49007", "County": "Carbon", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84526", "FIPS": "49007", "County": "Carbon", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84529", "FIPS": "49007", "County": "Carbon", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84539", "FIPS": "49007", "County": "Carbon", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84542", "FIPS": "49007", "County": "Carbon", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84023", "FIPS": "49009", "County": "Daggett", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84046", "FIPS": "49009", "County": "Daggett", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84010", "FIPS": "49011", "County": "Davis", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84011", "FIPS": "49011", "County": "Davis", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84014", "FIPS": "49011", "County": "Davis", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84015", "FIPS": "49011", "County": "Davis", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84016", "FIPS": "49011", "County": "Davis", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84025", "FIPS": "49011", "County": "Davis", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84037", "FIPS": "49011", "County": "Davis", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84040", "FIPS": "49011", "County": "Davis", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84041", "FIPS": "49011", "County": "Davis", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84054", "FIPS": "49011", "County": "Davis", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84056", "FIPS": "49011", "County": "Davis", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84075", "FIPS": "49011", "County": "Davis", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84087", "FIPS": "49011", "County": "Davis", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84089", "FIPS": "49011", "County": "Davis", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84001", "FIPS": "49013", "County": "Duchesne", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84002", "FIPS": "49013", "County": "Duchesne", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84007", "FIPS": "49013", "County": "Duchesne", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84021", "FIPS": "49013", "County": "Duchesne", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84027", "FIPS": "49013", "County": "Duchesne", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84031", "FIPS": "49013", "County": "Duchesne", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84051", "FIPS": "49013", "County": "Duchesne", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84052", "FIPS": "49013", "County": "Duchesne", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84053", "FIPS": "49013", "County": "Duchesne", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84066", "FIPS": "49013", "County": "Duchesne", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84072", "FIPS": "49013", "County": "Duchesne", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84073", "FIPS": "49013", "County": "Duchesne", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84513", "FIPS": "49015", "County": "Emery", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84516", "FIPS": "49015", "County": "Emery", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84518", "FIPS": "49015", "County": "Emery", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84521", "FIPS": "49015", "County": "Emery", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84522", "FIPS": "49015", "County": "Emery", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84523", "FIPS": "49015", "County": "Emery", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84525", "FIPS": "49015", "County": "Emery", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84528", "FIPS": "49015", "County": "Emery", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84537", "FIPS": "49015", "County": "Emery", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84712", "FIPS": "49017", "County": "Garfield", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84716", "FIPS": "49017", "County": "Garfield", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84718", "FIPS": "49017", "County": "Garfield", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84726", "FIPS": "49017", "County": "Garfield", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84735", "FIPS": "49017", "County": "Garfield", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84736", "FIPS": "49017", "County": "Garfield", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84759", "FIPS": "49017", "County": "Garfield", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84764", "FIPS": "49017", "County": "Garfield", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84776", "FIPS": "49017", "County": "Garfield", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84515", "FIPS": "49019", "County": "Grand", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84532", "FIPS": "49019", "County": "Grand", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84540", "FIPS": "49019", "County": "Grand", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84714", "FIPS": "49021", "County": "Iron", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84719", "FIPS": "49021", "County": "Iron", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84720", "FIPS": "49021", "County": "Iron", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84721", "FIPS": "49021", "County": "Iron", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84742", "FIPS": "49021", "County": "Iron", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84753", "FIPS": "49021", "County": "Iron", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84756", "FIPS": "49021", "County": "Iron", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84760", "FIPS": "49021", "County": "Iron", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84761", "FIPS": "49021", "County": "Iron", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84772", "FIPS": "49021", "County": "Iron", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84628", "FIPS": "49023", "County": "Juab", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84639", "FIPS": "49023", "County": "Juab", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84645", "FIPS": "49023", "County": "Juab", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84648", "FIPS": "49023", "County": "Juab", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84710", "FIPS": "49025", "County": "Kane", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84729", "FIPS": "49025", "County": "Kane", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84741", "FIPS": "49025", "County": "Kane", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84755", "FIPS": "49025", "County": "Kane", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84758", "FIPS": "49025", "County": "Kane", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84762", "FIPS": "49025", "County": "Kane", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84624", "FIPS": "49027", "County": "Millard", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84631", "FIPS": "49027", "County": "Millard", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84635", "FIPS": "49027", "County": "Millard", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84636", "FIPS": "49027", "County": "Millard", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84637", "FIPS": "49027", "County": "Millard", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84638", "FIPS": "49027", "County": "Millard", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84640", "FIPS": "49027", "County": "Millard", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84644", "FIPS": "49027", "County": "Millard", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84649", "FIPS": "49027", "County": "Millard", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84656", "FIPS": "49027", "County": "Millard", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84728", "FIPS": "49027", "County": "Millard", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84018", "FIPS": "49029", "County": "Morgan", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84050", "FIPS": "49029", "County": "Morgan", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84723", "FIPS": "49031", "County": "Piute", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84732", "FIPS": "49031", "County": "Piute", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84740", "FIPS": "49031", "County": "Piute", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84743", "FIPS": "49031", "County": "Piute", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84750", "FIPS": "49031", "County": "Piute", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84028", "FIPS": "49033", "County": "Rich", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84038", "FIPS": "49033", "County": "Rich", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84064", "FIPS": "49033", "County": "Rich", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84086", "FIPS": "49033", "County": "Rich", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84006", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84020", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84044", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84047", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84065", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84070", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84084", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84088", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84090", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84091", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84092", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84093", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84094", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84095", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84096", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84101", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84102", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84103", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84104", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84105", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84106", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84107", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84108", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84109", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84110", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84111", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84112", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84113", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84114", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84115", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84116", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84117", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84118", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84119", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84120", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84121", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84122", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84123", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84124", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84125", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84126", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84127", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84128", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84130", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84131", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84132", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84133", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84134", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84136", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84138", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84139", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84141", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84143", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84144", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84145", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84147", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84148", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84150", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84151", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84152", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84157", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84158", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84165", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84170", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84171", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84180", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84184", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84189", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84190", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84199", "FIPS": "49035", "County": "Salt Lake", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84510", "FIPS": "49037", "County": "San Juan", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84511", "FIPS": "49037", "County": "San Juan", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84512", "FIPS": "49037", "County": "San Juan", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84530", "FIPS": "49037", "County": "San Juan", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84531", "FIPS": "49037", "County": "San Juan", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84533", "FIPS": "49037", "County": "San Juan", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84534", "FIPS": "49037", "County": "San Juan", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84535", "FIPS": "49037", "County": "San Juan", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84536", "FIPS": "49037", "County": "San Juan", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84621", "FIPS": "49039", "County": "Sanpete", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84622", "FIPS": "49039", "County": "Sanpete", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84623", "FIPS": "49039", "County": "Sanpete", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84627", "FIPS": "49039", "County": "Sanpete", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84629", "FIPS": "49039", "County": "Sanpete", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84630", "FIPS": "49039", "County": "Sanpete", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84632", "FIPS": "49039", "County": "Sanpete", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84634", "FIPS": "49039", "County": "Sanpete", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84642", "FIPS": "49039", "County": "Sanpete", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84643", "FIPS": "49039", "County": "Sanpete", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84646", "FIPS": "49039", "County": "Sanpete", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84647", "FIPS": "49039", "County": "Sanpete", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84662", "FIPS": "49039", "County": "Sanpete", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84665", "FIPS": "49039", "County": "Sanpete", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84667", "FIPS": "49039", "County": "Sanpete", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84620", "FIPS": "49041", "County": "Sevier", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84652", "FIPS": "49041", "County": "Sevier", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84654", "FIPS": "49041", "County": "Sevier", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84657", "FIPS": "49041", "County": "Sevier", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84701", "FIPS": "49041", "County": "Sevier", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84711", "FIPS": "49041", "County": "Sevier", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84724", "FIPS": "49041", "County": "Sevier", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84730", "FIPS": "49041", "County": "Sevier", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84739", "FIPS": "49041", "County": "Sevier", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84744", "FIPS": "49041", "County": "Sevier", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84754", "FIPS": "49041", "County": "Sevier", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84766", "FIPS": "49041", "County": "Sevier", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84017", "FIPS": "49043", "County": "Summit", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84024", "FIPS": "49043", "County": "Summit", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84033", "FIPS": "49043", "County": "Summit", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84036", "FIPS": "49043", "County": "Summit", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84055", "FIPS": "49043", "County": "Summit", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84060", "FIPS": "49043", "County": "Summit", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84061", "FIPS": "49043", "County": "Summit", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84068", "FIPS": "49043", "County": "Summit", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84098", "FIPS": "49043", "County": "Summit", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84022", "FIPS": "49045", "County": "Tooele", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84029", "FIPS": "49045", "County": "Tooele", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84034", "FIPS": "49045", "County": "Tooele", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84069", "FIPS": "49045", "County": "Tooele", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84071", "FIPS": "49045", "County": "Tooele", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84074", "FIPS": "49045", "County": "Tooele", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84080", "FIPS": "49045", "County": "Tooele", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84083", "FIPS": "49045", "County": "Tooele", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84008", "FIPS": "49047", "County": "Uintah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84026", "FIPS": "49047", "County": "Uintah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84035", "FIPS": "49047", "County": "Uintah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84039", "FIPS": "49047", "County": "Uintah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84063", "FIPS": "49047", "County": "Uintah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84076", "FIPS": "49047", "County": "Uintah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84078", "FIPS": "49047", "County": "Uintah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84079", "FIPS": "49047", "County": "Uintah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84085", "FIPS": "49047", "County": "Uintah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84003", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84004", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84005", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84013", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84042", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84043", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84045", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84057", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84058", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84059", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84062", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84097", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84601", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84602", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84603", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84604", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84605", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84606", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84626", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84633", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84651", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84653", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84655", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84660", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84663", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84664", "FIPS": "49049", "County": "Utah", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84032", "FIPS": "49051", "County": "Wasatch", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84049", "FIPS": "49051", "County": "Wasatch", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84082", "FIPS": "49051", "County": "Wasatch", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84722", "FIPS": "49053", "County": "Washington", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84725", "FIPS": "49053", "County": "Washington", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84733", "FIPS": "49053", "County": "Washington", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84737", "FIPS": "49053", "County": "Washington", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84738", "FIPS": "49053", "County": "Washington", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84745", "FIPS": "49053", "County": "Washington", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84746", "FIPS": "49053", "County": "Washington", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84757", "FIPS": "49053", "County": "Washington", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84763", "FIPS": "49053", "County": "Washington", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84765", "FIPS": "49053", "County": "Washington", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84767", "FIPS": "49053", "County": "Washington", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84770", "FIPS": "49053", "County": "Washington", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84771", "FIPS": "49053", "County": "Washington", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84774", "FIPS": "49053", "County": "Washington", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84779", "FIPS": "49053", "County": "Washington", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84780", "FIPS": "49053", "County": "Washington", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84781", "FIPS": "49053", "County": "Washington", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84782", "FIPS": "49053", "County": "Washington", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84783", "FIPS": "49053", "County": "Washington", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84784", "FIPS": "49053", "County": "Washington", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84790", "FIPS": "49053", "County": "Washington", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84791", "FIPS": "49053", "County": "Washington", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84715", "FIPS": "49055", "County": "Wayne", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84734", "FIPS": "49055", "County": "Wayne", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84747", "FIPS": "49055", "County": "Wayne", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84749", "FIPS": "49055", "County": "Wayne", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84773", "FIPS": "49055", "County": "Wayne", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84775", "FIPS": "49055", "County": "Wayne", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84067", "FIPS": "49057", "County": "Weber", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84201", "FIPS": "49057", "County": "Weber", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84244", "FIPS": "49057", "County": "Weber", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84310", "FIPS": "49057", "County": "Weber", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84315", "FIPS": "49057", "County": "Weber", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84317", "FIPS": "49057", "County": "Weber", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84401", "FIPS": "49057", "County": "Weber", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84402", "FIPS": "49057", "County": "Weber", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84403", "FIPS": "49057", "County": "Weber", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84404", "FIPS": "49057", "County": "Weber", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84405", "FIPS": "49057", "County": "Weber", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84407", "FIPS": "49057", "County": "Weber", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84408", "FIPS": "49057", "County": "Weber", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84409", "FIPS": "49057", "County": "Weber", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84412", "FIPS": "49057", "County": "Weber", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84414", "FIPS": "49057", "County": "Weber", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "84415", "FIPS": "49057", "County": "Weber", "State": "UT", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "05443", "FIPS": "50001", "County": "Addison", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05456", "FIPS": "50001", "County": "Addison", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05469", "FIPS": "50001", "County": "Addison", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05472", "FIPS": "50001", "County": "Addison", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05473", "FIPS": "50001", "County": "Addison", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05487", "FIPS": "50001", "County": "Addison", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05491", "FIPS": "50001", "County": "Addison", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05669", "FIPS": "50001", "County": "Addison", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05734", "FIPS": "50001", "County": "Addison", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05740", "FIPS": "50001", "County": "Addison", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05747", "FIPS": "50001", "County": "Addison", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05748", "FIPS": "50001", "County": "Addison", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05753", "FIPS": "50001", "County": "Addison", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05760", "FIPS": "50001", "County": "Addison", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05766", "FIPS": "50001", "County": "Addison", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05769", "FIPS": "50001", "County": "Addison", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05770", "FIPS": "50001", "County": "Addison", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05778", "FIPS": "50001", "County": "Addison", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05152", "FIPS": "50003", "County": "Bennington", "State": "VT", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "05201", "FIPS": "50003", "County": "Bennington", "State": "VT", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "05250", "FIPS": "50003", "County": "Bennington", "State": "VT", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "05251", "FIPS": "50003", "County": "Bennington", "State": "VT", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "05252", "FIPS": "50003", "County": "Bennington", "State": "VT", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "05253", "FIPS": "50003", "County": "Bennington", "State": "VT", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "05254", "FIPS": "50003", "County": "Bennington", "State": "VT", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "05255", "FIPS": "50003", "County": "Bennington", "State": "VT", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "05257", "FIPS": "50003", "County": "Bennington", "State": "VT", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "05260", "FIPS": "50003", "County": "Bennington", "State": "VT", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "05261", "FIPS": "50003", "County": "Bennington", "State": "VT", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "05262", "FIPS": "50003", "County": "Bennington", "State": "VT", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "05340", "FIPS": "50003", "County": "Bennington", "State": "VT", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "05350", "FIPS": "50003", "County": "Bennington", "State": "VT", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "05352", "FIPS": "50003", "County": "Bennington", "State": "VT", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "05768", "FIPS": "50003", "County": "Bennington", "State": "VT", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "05776", "FIPS": "50003", "County": "Bennington", "State": "VT", "DMA": {"Code": 532, "Name": "ALBANY-SCHENECTADY-TROY"}},
  {"Postal_Code": "05042", "FIPS": "50005", "County": "Caledonia", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05046", "FIPS": "50005", "County": "Caledonia", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05050", "FIPS": "50005", "County": "Caledonia", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05069", "FIPS": "50005", "County": "Caledonia", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05819", "FIPS": "50005", "County": "Caledonia", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05821", "FIPS": "50005", "County": "Caledonia", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05828", "FIPS": "50005", "County": "Caledonia", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05832", "FIPS": "50005", "County": "Caledonia", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05836", "FIPS": "50005", "County": "Caledonia", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05838", "FIPS": "50005", "County": "Caledonia", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05843", "FIPS": "50005", "County": "Caledonia", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05848", "FIPS": "50005", "County": "Caledonia", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05849", "FIPS": "50005", "County": "Caledonia", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05850", "FIPS": "50005", "County": "Caledonia", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05851", "FIPS": "50005", "County": "Caledonia", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05861", "FIPS": "50005", "County": "Caledonia", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05862", "FIPS": "50005", "County": "Caledonia", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05863", "FIPS": "50005", "County": "Caledonia", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05866", "FIPS": "50005", "County": "Caledonia", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05867", "FIPS": "50005", "County": "Caledonia", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05871", "FIPS": "50005", "County": "Caledonia", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05873", "FIPS": "50005", "County": "Caledonia", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05401", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05402", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05403", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05404", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05405", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05406", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05407", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05408", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05439", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05444", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05445", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05446", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05449", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05451", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05452", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05453", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05454", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05461", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05462", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05465", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05466", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05468", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05477", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05482", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05489", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05490", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05494", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05495", "FIPS": "50007", "County": "Chittenden", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05824", "FIPS": "50009", "County": "Essex", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05837", "FIPS": "50009", "County": "Essex", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05840", "FIPS": "50009", "County": "Essex", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05846", "FIPS": "50009", "County": "Essex", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05858", "FIPS": "50009", "County": "Essex", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05901", "FIPS": "50009", "County": "Essex", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05902", "FIPS": "50009", "County": "Essex", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05903", "FIPS": "50009", "County": "Essex", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05904", "FIPS": "50009", "County": "Essex", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05905", "FIPS": "50009", "County": "Essex", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05906", "FIPS": "50009", "County": "Essex", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05907", "FIPS": "50009", "County": "Essex", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05441", "FIPS": "50011", "County": "Franklin", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05447", "FIPS": "50011", "County": "Franklin", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05448", "FIPS": "50011", "County": "Franklin", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05450", "FIPS": "50011", "County": "Franklin", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05455", "FIPS": "50011", "County": "Franklin", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05457", "FIPS": "50011", "County": "Franklin", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05459", "FIPS": "50011", "County": "Franklin", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05460", "FIPS": "50011", "County": "Franklin", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05470", "FIPS": "50011", "County": "Franklin", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05471", "FIPS": "50011", "County": "Franklin", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05476", "FIPS": "50011", "County": "Franklin", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05478", "FIPS": "50011", "County": "Franklin", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05479", "FIPS": "50011", "County": "Franklin", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05481", "FIPS": "50011", "County": "Franklin", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05483", "FIPS": "50011", "County": "Franklin", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05485", "FIPS": "50011", "County": "Franklin", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05488", "FIPS": "50011", "County": "Franklin", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05440", "FIPS": "50013", "County": "Grand Isle", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05458", "FIPS": "50013", "County": "Grand Isle", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05463", "FIPS": "50013", "County": "Grand Isle", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05474", "FIPS": "50013", "County": "Grand Isle", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05486", "FIPS": "50013", "County": "Grand Isle", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05442", "FIPS": "50015", "County": "Lamoille", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05464", "FIPS": "50015", "County": "Lamoille", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05492", "FIPS": "50015", "County": "Lamoille", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05652", "FIPS": "50015", "County": "Lamoille", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05653", "FIPS": "50015", "County": "Lamoille", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05655", "FIPS": "50015", "County": "Lamoille", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05656", "FIPS": "50015", "County": "Lamoille", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05657", "FIPS": "50015", "County": "Lamoille", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05661", "FIPS": "50015", "County": "Lamoille", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05662", "FIPS": "50015", "County": "Lamoille", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05665", "FIPS": "50015", "County": "Lamoille", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05672", "FIPS": "50015", "County": "Lamoille", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05680", "FIPS": "50015", "County": "Lamoille", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05033", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05036", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05038", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05039", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05040", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05041", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05043", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05045", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05051", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05054", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05058", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05060", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05061", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05070", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05072", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05074", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05075", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05076", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05077", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05079", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05081", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05083", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05085", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05086", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05675", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05679", "FIPS": "50017", "County": "Orange", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05820", "FIPS": "50019", "County": "Orleans", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05822", "FIPS": "50019", "County": "Orleans", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05823", "FIPS": "50019", "County": "Orleans", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05825", "FIPS": "50019", "County": "Orleans", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05826", "FIPS": "50019", "County": "Orleans", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05827", "FIPS": "50019", "County": "Orleans", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05829", "FIPS": "50019", "County": "Orleans", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05830", "FIPS": "50019", "County": "Orleans", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05833", "FIPS": "50019", "County": "Orleans", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05839", "FIPS": "50019", "County": "Orleans", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05841", "FIPS": "50019", "County": "Orleans", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05842", "FIPS": "50019", "County": "Orleans", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05845", "FIPS": "50019", "County": "Orleans", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05847", "FIPS": "50019", "County": "Orleans", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05853", "FIPS": "50019", "County": "Orleans", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05855", "FIPS": "50019", "County": "Orleans", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05857", "FIPS": "50019", "County": "Orleans", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05859", "FIPS": "50019", "County": "Orleans", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05860", "FIPS": "50019", "County": "Orleans", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05868", "FIPS": "50019", "County": "Orleans", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05872", "FIPS": "50019", "County": "Orleans", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05874", "FIPS": "50019", "County": "Orleans", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05875", "FIPS": "50019", "County": "Orleans", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05701", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05702", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05730", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05731", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05732", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05733", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05735", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05736", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05737", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05738", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05739", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05741", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05742", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05743", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05744", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05745", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05750", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05751", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05757", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05758", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05759", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05761", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05762", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05763", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05764", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05765", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05773", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05774", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05775", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05777", "FIPS": "50021", "County": "Rutland", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05601", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05602", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05603", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05604", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05609", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05620", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05633", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05640", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05641", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05647", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05648", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05649", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05650", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05651", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05654", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05658", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05660", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05663", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05664", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05666", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05667", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05670", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05671", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05673", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05674", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05676", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05677", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05678", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05681", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05682", "FIPS": "50023", "County": "Washington", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05101", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05141", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05146", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05148", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05154", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05155", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05158", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05159", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05301", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05302", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05303", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05304", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05341", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05342", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05343", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05344", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05345", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05346", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05351", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05353", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05354", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05355", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05356", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05357", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05358", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05359", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05360", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05361", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05362", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05363", "FIPS": "50025", "County": "Windham", "State": "VT", "DMA": {"Code": 506, "Name": "BOSTON"}},
  {"Postal_Code": "05001", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05009", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05030", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05031", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05032", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05034", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05035", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05037", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05047", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05048", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05049", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05052", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05053", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05055", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05056", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05059", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05062", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05065", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05067", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05068", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05071", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05073", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05084", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05088", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05089", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05091", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05142", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05143", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05144", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05149", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05150", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05151", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05153", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05156", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05161", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05746", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05767", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "05772", "FIPS": "50027", "County": "Windsor", "State": "VT", "DMA": {"Code": 523, "Name": "BURLINGTON-PLATTSBURGH"}},
  {"Postal_Code": "23301", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23302", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23303", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23306", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23308", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23336", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23337", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23341", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23345", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23356", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23357", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23358", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23359", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23389", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23395", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23396", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23399", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23401", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23404", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23407", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23409", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23410", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23412", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23414", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23415", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23416", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23417", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23418", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23420", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23421", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23422", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23423", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23426", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23427", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23440", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23441", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23442", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23480", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23483", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23488", "FIPS": "51001", "County": "Accomack", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "22901", "FIPS": "51003", "County": "Albemarle", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22909", "FIPS": "51003", "County": "Albemarle", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22911", "FIPS": "51003", "County": "Albemarle", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22924", "FIPS": "51003", "County": "Albemarle", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22931", "FIPS": "51003", "County": "Albemarle", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22932", "FIPS": "51003", "County": "Albemarle", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22936", "FIPS": "51003", "County": "Albemarle", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22937", "FIPS": "51003", "County": "Albemarle", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22940", "FIPS": "51003", "County": "Albemarle", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22943", "FIPS": "51003", "County": "Albemarle", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22945", "FIPS": "51003", "County": "Albemarle", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22946", "FIPS": "51003", "County": "Albemarle", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22947", "FIPS": "51003", "County": "Albemarle", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22959", "FIPS": "51003", "County": "Albemarle", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22987", "FIPS": "51003", "County": "Albemarle", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "24562", "FIPS": "51003", "County": "Albemarle", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "24590", "FIPS": "51003", "County": "Albemarle", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "24422", "FIPS": "51005", "County": "Alleghany", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24448", "FIPS": "51005", "County": "Alleghany", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24457", "FIPS": "51005", "County": "Alleghany", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24474", "FIPS": "51005", "County": "Alleghany", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23002", "FIPS": "51007", "County": "Amelia", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23083", "FIPS": "51007", "County": "Amelia", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23105", "FIPS": "51007", "County": "Amelia", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "24521", "FIPS": "51009", "County": "Amherst", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24533", "FIPS": "51009", "County": "Amherst", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24572", "FIPS": "51009", "County": "Amherst", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24574", "FIPS": "51009", "County": "Amherst", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24595", "FIPS": "51009", "County": "Amherst", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23939", "FIPS": "51011", "County": "Appomattox", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23958", "FIPS": "51011", "County": "Appomattox", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24522", "FIPS": "51011", "County": "Appomattox", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24593", "FIPS": "51011", "County": "Appomattox", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "22201", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22202", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22203", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22204", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22205", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22206", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22207", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22209", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22210", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22211", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22212", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22213", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22214", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22215", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22216", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22217", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22218", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22219", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22222", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22223", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22225", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22226", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22227", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22229", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22230", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22234", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22240", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22241", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22242", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22243", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22244", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22245", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22246", "FIPS": "51013", "County": "Arlington", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22843", "FIPS": "51015", "County": "Augusta", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "22939", "FIPS": "51015", "County": "Augusta", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "22952", "FIPS": "51015", "County": "Augusta", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "24411", "FIPS": "51015", "County": "Augusta", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "24421", "FIPS": "51015", "County": "Augusta", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "24430", "FIPS": "51015", "County": "Augusta", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "24431", "FIPS": "51015", "County": "Augusta", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "24432", "FIPS": "51015", "County": "Augusta", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "24437", "FIPS": "51015", "County": "Augusta", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "24440", "FIPS": "51015", "County": "Augusta", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "24459", "FIPS": "51015", "County": "Augusta", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "24463", "FIPS": "51015", "County": "Augusta", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "24467", "FIPS": "51015", "County": "Augusta", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "24469", "FIPS": "51015", "County": "Augusta", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "24476", "FIPS": "51015", "County": "Augusta", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "24477", "FIPS": "51015", "County": "Augusta", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "24479", "FIPS": "51015", "County": "Augusta", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "24482", "FIPS": "51015", "County": "Augusta", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "24485", "FIPS": "51015", "County": "Augusta", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "24486", "FIPS": "51015", "County": "Augusta", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "24412", "FIPS": "51017", "County": "Bath", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24445", "FIPS": "51017", "County": "Bath", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24460", "FIPS": "51017", "County": "Bath", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24484", "FIPS": "51017", "County": "Bath", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24487", "FIPS": "51017", "County": "Bath", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24095", "FIPS": "51019", "County": "Bedford", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24104", "FIPS": "51019", "County": "Bedford", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24121", "FIPS": "51019", "County": "Bedford", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24122", "FIPS": "51019", "County": "Bedford", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24174", "FIPS": "51019", "County": "Bedford", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24178", "FIPS": "51019", "County": "Bedford", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24526", "FIPS": "51019", "County": "Bedford", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24536", "FIPS": "51019", "County": "Bedford", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24551", "FIPS": "51019", "County": "Bedford", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24556", "FIPS": "51019", "County": "Bedford", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24570", "FIPS": "51019", "County": "Bedford", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24314", "FIPS": "51021", "County": "Bland", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24315", "FIPS": "51021", "County": "Bland", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24318", "FIPS": "51021", "County": "Bland", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24366", "FIPS": "51021", "County": "Bland", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24050", "FIPS": "51023", "County": "Botetourt", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24064", "FIPS": "51023", "County": "Botetourt", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24066", "FIPS": "51023", "County": "Botetourt", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24077", "FIPS": "51023", "County": "Botetourt", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24083", "FIPS": "51023", "County": "Botetourt", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24085", "FIPS": "51023", "County": "Botetourt", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24090", "FIPS": "51023", "County": "Botetourt", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24130", "FIPS": "51023", "County": "Botetourt", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24175", "FIPS": "51023", "County": "Botetourt", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24438", "FIPS": "51023", "County": "Botetourt", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23821", "FIPS": "51025", "County": "Brunswick", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23843", "FIPS": "51025", "County": "Brunswick", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23845", "FIPS": "51025", "County": "Brunswick", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23856", "FIPS": "51025", "County": "Brunswick", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23857", "FIPS": "51025", "County": "Brunswick", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23868", "FIPS": "51025", "County": "Brunswick", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23873", "FIPS": "51025", "County": "Brunswick", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23876", "FIPS": "51025", "County": "Brunswick", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23887", "FIPS": "51025", "County": "Brunswick", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23889", "FIPS": "51025", "County": "Brunswick", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23893", "FIPS": "51025", "County": "Brunswick", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23920", "FIPS": "51025", "County": "Brunswick", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "24239", "FIPS": "51027", "County": "Buchanan", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24603", "FIPS": "51027", "County": "Buchanan", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24614", "FIPS": "51027", "County": "Buchanan", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24620", "FIPS": "51027", "County": "Buchanan", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24624", "FIPS": "51027", "County": "Buchanan", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24627", "FIPS": "51027", "County": "Buchanan", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24628", "FIPS": "51027", "County": "Buchanan", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24631", "FIPS": "51027", "County": "Buchanan", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24634", "FIPS": "51027", "County": "Buchanan", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24639", "FIPS": "51027", "County": "Buchanan", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24646", "FIPS": "51027", "County": "Buchanan", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24647", "FIPS": "51027", "County": "Buchanan", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24656", "FIPS": "51027", "County": "Buchanan", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24657", "FIPS": "51027", "County": "Buchanan", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24658", "FIPS": "51027", "County": "Buchanan", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "23004", "FIPS": "51029", "County": "Buckingham", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23123", "FIPS": "51029", "County": "Buckingham", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23921", "FIPS": "51029", "County": "Buckingham", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23936", "FIPS": "51029", "County": "Buckingham", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "24517", "FIPS": "51031", "County": "Campbell", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24528", "FIPS": "51031", "County": "Campbell", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24538", "FIPS": "51031", "County": "Campbell", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24550", "FIPS": "51031", "County": "Campbell", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24554", "FIPS": "51031", "County": "Campbell", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24571", "FIPS": "51031", "County": "Campbell", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24576", "FIPS": "51031", "County": "Campbell", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24588", "FIPS": "51031", "County": "Campbell", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "22427", "FIPS": "51033", "County": "Caroline", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22428", "FIPS": "51033", "County": "Caroline", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22446", "FIPS": "51033", "County": "Caroline", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22501", "FIPS": "51033", "County": "Caroline", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22514", "FIPS": "51033", "County": "Caroline", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22535", "FIPS": "51033", "County": "Caroline", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22538", "FIPS": "51033", "County": "Caroline", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22546", "FIPS": "51033", "County": "Caroline", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22552", "FIPS": "51033", "County": "Caroline", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22580", "FIPS": "51033", "County": "Caroline", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "24317", "FIPS": "51035", "County": "Carroll", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24325", "FIPS": "51035", "County": "Carroll", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24328", "FIPS": "51035", "County": "Carroll", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24343", "FIPS": "51035", "County": "Carroll", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24351", "FIPS": "51035", "County": "Carroll", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24352", "FIPS": "51035", "County": "Carroll", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24381", "FIPS": "51035", "County": "Carroll", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23030", "FIPS": "51036", "County": "Charles City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23147", "FIPS": "51036", "County": "Charles City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23923", "FIPS": "51037", "County": "Charlotte", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23934", "FIPS": "51037", "County": "Charlotte", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23937", "FIPS": "51037", "County": "Charlotte", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23947", "FIPS": "51037", "County": "Charlotte", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23959", "FIPS": "51037", "County": "Charlotte", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23962", "FIPS": "51037", "County": "Charlotte", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23963", "FIPS": "51037", "County": "Charlotte", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23964", "FIPS": "51037", "County": "Charlotte", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23967", "FIPS": "51037", "County": "Charlotte", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23976", "FIPS": "51037", "County": "Charlotte", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23112", "FIPS": "51041", "County": "Chesterfield", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23113", "FIPS": "51041", "County": "Chesterfield", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23114", "FIPS": "51041", "County": "Chesterfield", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23120", "FIPS": "51041", "County": "Chesterfield", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23234", "FIPS": "51041", "County": "Chesterfield", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23235", "FIPS": "51041", "County": "Chesterfield", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23236", "FIPS": "51041", "County": "Chesterfield", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23237", "FIPS": "51041", "County": "Chesterfield", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23297", "FIPS": "51041", "County": "Chesterfield", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23831", "FIPS": "51041", "County": "Chesterfield", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23832", "FIPS": "51041", "County": "Chesterfield", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23836", "FIPS": "51041", "County": "Chesterfield", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23838", "FIPS": "51041", "County": "Chesterfield", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "20130", "FIPS": "51043", "County": "Clarke", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20135", "FIPS": "51043", "County": "Clarke", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22611", "FIPS": "51043", "County": "Clarke", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22620", "FIPS": "51043", "County": "Clarke", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22646", "FIPS": "51043", "County": "Clarke", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22663", "FIPS": "51043", "County": "Clarke", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "24127", "FIPS": "51045", "County": "Craig", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24131", "FIPS": "51045", "County": "Craig", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "20106", "FIPS": "51047", "County": "Culpeper", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22701", "FIPS": "51047", "County": "Culpeper", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22713", "FIPS": "51047", "County": "Culpeper", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22714", "FIPS": "51047", "County": "Culpeper", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22718", "FIPS": "51047", "County": "Culpeper", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22724", "FIPS": "51047", "County": "Culpeper", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22726", "FIPS": "51047", "County": "Culpeper", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22729", "FIPS": "51047", "County": "Culpeper", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22733", "FIPS": "51047", "County": "Culpeper", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22735", "FIPS": "51047", "County": "Culpeper", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22736", "FIPS": "51047", "County": "Culpeper", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22737", "FIPS": "51047", "County": "Culpeper", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22741", "FIPS": "51047", "County": "Culpeper", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22746", "FIPS": "51047", "County": "Culpeper", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "23027", "FIPS": "51049", "County": "Cumberland", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23040", "FIPS": "51049", "County": "Cumberland", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "24217", "FIPS": "51051", "County": "Dickenson", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24220", "FIPS": "51051", "County": "Dickenson", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24226", "FIPS": "51051", "County": "Dickenson", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24228", "FIPS": "51051", "County": "Dickenson", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24256", "FIPS": "51051", "County": "Dickenson", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24269", "FIPS": "51051", "County": "Dickenson", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24272", "FIPS": "51051", "County": "Dickenson", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24607", "FIPS": "51051", "County": "Dickenson", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "23822", "FIPS": "51053", "County": "Dinwiddie", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23830", "FIPS": "51053", "County": "Dinwiddie", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23833", "FIPS": "51053", "County": "Dinwiddie", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23840", "FIPS": "51053", "County": "Dinwiddie", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23841", "FIPS": "51053", "County": "Dinwiddie", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23850", "FIPS": "51053", "County": "Dinwiddie", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23872", "FIPS": "51053", "County": "Dinwiddie", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23885", "FIPS": "51053", "County": "Dinwiddie", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23894", "FIPS": "51053", "County": "Dinwiddie", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22436", "FIPS": "51057", "County": "Essex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22437", "FIPS": "51057", "County": "Essex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22438", "FIPS": "51057", "County": "Essex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22454", "FIPS": "51057", "County": "Essex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22476", "FIPS": "51057", "County": "Essex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22504", "FIPS": "51057", "County": "Essex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22509", "FIPS": "51057", "County": "Essex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22560", "FIPS": "51057", "County": "Essex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23115", "FIPS": "51057", "County": "Essex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "20120", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20121", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20122", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20124", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20151", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20153", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20170", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20171", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20172", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20190", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20191", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20192", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20193", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20194", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20195", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20196", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22003", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22009", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22015", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22027", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22031", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22032", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22033", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22034", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22035", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22036", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22037", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22039", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22041", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22042", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22043", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22044", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22047", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22060", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22066", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22067", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22079", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22081", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22082", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22092", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22095", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22096", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22101", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22102", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22103", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22106", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22107", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22108", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22109", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22116", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22118", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22119", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22120", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22121", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22122", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22124", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22150", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22151", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22152", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22153", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22156", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22158", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22159", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22160", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22161", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22180", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22181", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22182", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22183", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22184", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22185", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22199", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22303", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22306", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22307", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22308", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22309", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22310", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22312", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22315", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22321", "FIPS": "51059", "County": "Fairfax", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20115", "FIPS": "51061", "County": "Fauquier", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20116", "FIPS": "51061", "County": "Fauquier", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20119", "FIPS": "51061", "County": "Fauquier", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20128", "FIPS": "51061", "County": "Fauquier", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20137", "FIPS": "51061", "County": "Fauquier", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20138", "FIPS": "51061", "County": "Fauquier", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20139", "FIPS": "51061", "County": "Fauquier", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20140", "FIPS": "51061", "County": "Fauquier", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20144", "FIPS": "51061", "County": "Fauquier", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20184", "FIPS": "51061", "County": "Fauquier", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20185", "FIPS": "51061", "County": "Fauquier", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20186", "FIPS": "51061", "County": "Fauquier", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20187", "FIPS": "51061", "County": "Fauquier", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20188", "FIPS": "51061", "County": "Fauquier", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20198", "FIPS": "51061", "County": "Fauquier", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22639", "FIPS": "51061", "County": "Fauquier", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22643", "FIPS": "51061", "County": "Fauquier", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22712", "FIPS": "51061", "County": "Fauquier", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22720", "FIPS": "51061", "County": "Fauquier", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22728", "FIPS": "51061", "County": "Fauquier", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22734", "FIPS": "51061", "County": "Fauquier", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22739", "FIPS": "51061", "County": "Fauquier", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22742", "FIPS": "51061", "County": "Fauquier", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "24072", "FIPS": "51063", "County": "Floyd", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24079", "FIPS": "51063", "County": "Floyd", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24091", "FIPS": "51063", "County": "Floyd", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24105", "FIPS": "51063", "County": "Floyd", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24380", "FIPS": "51063", "County": "Floyd", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "22963", "FIPS": "51065", "County": "Fluvanna", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22974", "FIPS": "51065", "County": "Fluvanna", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "23022", "FIPS": "51065", "County": "Fluvanna", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "23055", "FIPS": "51065", "County": "Fluvanna", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "23084", "FIPS": "51065", "County": "Fluvanna", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "24065", "FIPS": "51067", "County": "Franklin", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24067", "FIPS": "51067", "County": "Franklin", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24088", "FIPS": "51067", "County": "Franklin", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24092", "FIPS": "51067", "County": "Franklin", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24101", "FIPS": "51067", "County": "Franklin", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24102", "FIPS": "51067", "County": "Franklin", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24137", "FIPS": "51067", "County": "Franklin", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24146", "FIPS": "51067", "County": "Franklin", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24151", "FIPS": "51067", "County": "Franklin", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24176", "FIPS": "51067", "County": "Franklin", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24184", "FIPS": "51067", "County": "Franklin", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "22602", "FIPS": "51069", "County": "Frederick", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22603", "FIPS": "51069", "County": "Frederick", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22622", "FIPS": "51069", "County": "Frederick", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22624", "FIPS": "51069", "County": "Frederick", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22625", "FIPS": "51069", "County": "Frederick", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22637", "FIPS": "51069", "County": "Frederick", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22638", "FIPS": "51069", "County": "Frederick", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22645", "FIPS": "51069", "County": "Frederick", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22654", "FIPS": "51069", "County": "Frederick", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22655", "FIPS": "51069", "County": "Frederick", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22656", "FIPS": "51069", "County": "Frederick", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "24086", "FIPS": "51071", "County": "Giles", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24093", "FIPS": "51071", "County": "Giles", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24124", "FIPS": "51071", "County": "Giles", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24128", "FIPS": "51071", "County": "Giles", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24134", "FIPS": "51071", "County": "Giles", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24136", "FIPS": "51071", "County": "Giles", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24147", "FIPS": "51071", "County": "Giles", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24150", "FIPS": "51071", "County": "Giles", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24167", "FIPS": "51071", "County": "Giles", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23001", "FIPS": "51073", "County": "Gloucester", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23003", "FIPS": "51073", "County": "Gloucester", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23018", "FIPS": "51073", "County": "Gloucester", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23050", "FIPS": "51073", "County": "Gloucester", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23061", "FIPS": "51073", "County": "Gloucester", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23062", "FIPS": "51073", "County": "Gloucester", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23072", "FIPS": "51073", "County": "Gloucester", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23107", "FIPS": "51073", "County": "Gloucester", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23131", "FIPS": "51073", "County": "Gloucester", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23154", "FIPS": "51073", "County": "Gloucester", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23155", "FIPS": "51073", "County": "Gloucester", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23178", "FIPS": "51073", "County": "Gloucester", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23183", "FIPS": "51073", "County": "Gloucester", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23184", "FIPS": "51073", "County": "Gloucester", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23190", "FIPS": "51073", "County": "Gloucester", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23014", "FIPS": "51075", "County": "Goochland", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23038", "FIPS": "51075", "County": "Goochland", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23039", "FIPS": "51075", "County": "Goochland", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23063", "FIPS": "51075", "County": "Goochland", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23065", "FIPS": "51075", "County": "Goochland", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23067", "FIPS": "51075", "County": "Goochland", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23102", "FIPS": "51075", "County": "Goochland", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23103", "FIPS": "51075", "County": "Goochland", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23129", "FIPS": "51075", "County": "Goochland", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23153", "FIPS": "51075", "County": "Goochland", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23160", "FIPS": "51075", "County": "Goochland", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "24292", "FIPS": "51077", "County": "Grayson", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24326", "FIPS": "51077", "County": "Grayson", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24330", "FIPS": "51077", "County": "Grayson", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24348", "FIPS": "51077", "County": "Grayson", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24363", "FIPS": "51077", "County": "Grayson", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24378", "FIPS": "51077", "County": "Grayson", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "22935", "FIPS": "51079", "County": "Greene", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22965", "FIPS": "51079", "County": "Greene", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22968", "FIPS": "51079", "County": "Greene", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22973", "FIPS": "51079", "County": "Greene", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "23867", "FIPS": "51081", "County": "Greensville", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23870", "FIPS": "51081", "County": "Greensville", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23879", "FIPS": "51081", "County": "Greensville", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "24520", "FIPS": "51083", "County": "Halifax", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24534", "FIPS": "51083", "County": "Halifax", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24535", "FIPS": "51083", "County": "Halifax", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24539", "FIPS": "51083", "County": "Halifax", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24558", "FIPS": "51083", "County": "Halifax", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24577", "FIPS": "51083", "County": "Halifax", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24589", "FIPS": "51083", "County": "Halifax", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24592", "FIPS": "51083", "County": "Halifax", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24597", "FIPS": "51083", "County": "Halifax", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24598", "FIPS": "51083", "County": "Halifax", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23005", "FIPS": "51085", "County": "Hanover", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23015", "FIPS": "51085", "County": "Hanover", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23047", "FIPS": "51085", "County": "Hanover", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23069", "FIPS": "51085", "County": "Hanover", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23111", "FIPS": "51085", "County": "Hanover", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23116", "FIPS": "51085", "County": "Hanover", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23146", "FIPS": "51085", "County": "Hanover", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23162", "FIPS": "51085", "County": "Hanover", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23192", "FIPS": "51085", "County": "Hanover", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23058", "FIPS": "51087", "County": "Henrico", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23059", "FIPS": "51087", "County": "Henrico", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23060", "FIPS": "51087", "County": "Henrico", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23075", "FIPS": "51087", "County": "Henrico", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23150", "FIPS": "51087", "County": "Henrico", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23226", "FIPS": "51087", "County": "Henrico", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23227", "FIPS": "51087", "County": "Henrico", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23228", "FIPS": "51087", "County": "Henrico", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23229", "FIPS": "51087", "County": "Henrico", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23230", "FIPS": "51087", "County": "Henrico", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23231", "FIPS": "51087", "County": "Henrico", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23233", "FIPS": "51087", "County": "Henrico", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23238", "FIPS": "51087", "County": "Henrico", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23242", "FIPS": "51087", "County": "Henrico", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23250", "FIPS": "51087", "County": "Henrico", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23255", "FIPS": "51087", "County": "Henrico", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23288", "FIPS": "51087", "County": "Henrico", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23294", "FIPS": "51087", "County": "Henrico", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "24054", "FIPS": "51089", "County": "Henry", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24055", "FIPS": "51089", "County": "Henry", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24078", "FIPS": "51089", "County": "Henry", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24089", "FIPS": "51089", "County": "Henry", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24148", "FIPS": "51089", "County": "Henry", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24165", "FIPS": "51089", "County": "Henry", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24168", "FIPS": "51089", "County": "Henry", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24413", "FIPS": "51091", "County": "Highland", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24433", "FIPS": "51091", "County": "Highland", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24442", "FIPS": "51091", "County": "Highland", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24458", "FIPS": "51091", "County": "Highland", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24465", "FIPS": "51091", "County": "Highland", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24468", "FIPS": "51091", "County": "Highland", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23304", "FIPS": "51093", "County": "Isle of Wight", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23314", "FIPS": "51093", "County": "Isle of Wight", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23315", "FIPS": "51093", "County": "Isle of Wight", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23397", "FIPS": "51093", "County": "Isle of Wight", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23424", "FIPS": "51093", "County": "Isle of Wight", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23430", "FIPS": "51093", "County": "Isle of Wight", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23431", "FIPS": "51093", "County": "Isle of Wight", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23487", "FIPS": "51093", "County": "Isle of Wight", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23898", "FIPS": "51093", "County": "Isle of Wight", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23081", "FIPS": "51095", "County": "James City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23127", "FIPS": "51095", "County": "James City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23168", "FIPS": "51095", "County": "James City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23185", "FIPS": "51095", "County": "James City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23188", "FIPS": "51095", "County": "James City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23023", "FIPS": "51097", "County": "King and Queen", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23085", "FIPS": "51097", "County": "King and Queen", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23091", "FIPS": "51097", "County": "King and Queen", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23108", "FIPS": "51097", "County": "King and Queen", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23110", "FIPS": "51097", "County": "King and Queen", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23126", "FIPS": "51097", "County": "King and Queen", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23148", "FIPS": "51097", "County": "King and Queen", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23156", "FIPS": "51097", "County": "King and Queen", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23161", "FIPS": "51097", "County": "King and Queen", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23177", "FIPS": "51097", "County": "King and Queen", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22448", "FIPS": "51099", "County": "King George", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22451", "FIPS": "51099", "County": "King George", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22481", "FIPS": "51099", "County": "King George", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22485", "FIPS": "51099", "County": "King George", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22526", "FIPS": "51099", "County": "King George", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22544", "FIPS": "51099", "County": "King George", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22547", "FIPS": "51099", "County": "King George", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "23009", "FIPS": "51101", "County": "King William", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23086", "FIPS": "51101", "County": "King William", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23106", "FIPS": "51101", "County": "King William", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23181", "FIPS": "51101", "County": "King William", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22480", "FIPS": "51103", "County": "Lancaster", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22482", "FIPS": "51103", "County": "Lancaster", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22503", "FIPS": "51103", "County": "Lancaster", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22507", "FIPS": "51103", "County": "Lancaster", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22513", "FIPS": "51103", "County": "Lancaster", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22517", "FIPS": "51103", "County": "Lancaster", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22523", "FIPS": "51103", "County": "Lancaster", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22528", "FIPS": "51103", "County": "Lancaster", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22576", "FIPS": "51103", "County": "Lancaster", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22578", "FIPS": "51103", "County": "Lancaster", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "24218", "FIPS": "51105", "County": "Lee", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24221", "FIPS": "51105", "County": "Lee", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24243", "FIPS": "51105", "County": "Lee", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24248", "FIPS": "51105", "County": "Lee", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24263", "FIPS": "51105", "County": "Lee", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24265", "FIPS": "51105", "County": "Lee", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24277", "FIPS": "51105", "County": "Lee", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24281", "FIPS": "51105", "County": "Lee", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24282", "FIPS": "51105", "County": "Lee", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "20101", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20102", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20103", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20104", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20105", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20107", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20117", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20118", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20129", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20131", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20132", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20134", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20141", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20142", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20146", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20147", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20148", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20149", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20152", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20158", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20159", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20160", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20163", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20164", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20165", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20166", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20167", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20175", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20176", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20177", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20178", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20180", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20189", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20197", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20199", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22093", "FIPS": "51107", "County": "Loudoun", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "23024", "FIPS": "51109", "County": "Louisa", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23093", "FIPS": "51109", "County": "Louisa", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23117", "FIPS": "51109", "County": "Louisa", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23170", "FIPS": "51109", "County": "Louisa", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23938", "FIPS": "51111", "County": "Lunenburg", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23941", "FIPS": "51111", "County": "Lunenburg", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23944", "FIPS": "51111", "County": "Lunenburg", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23952", "FIPS": "51111", "County": "Lunenburg", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23974", "FIPS": "51111", "County": "Lunenburg", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22709", "FIPS": "51113", "County": "Madison", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22711", "FIPS": "51113", "County": "Madison", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22715", "FIPS": "51113", "County": "Madison", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22719", "FIPS": "51113", "County": "Madison", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22721", "FIPS": "51113", "County": "Madison", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22722", "FIPS": "51113", "County": "Madison", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22723", "FIPS": "51113", "County": "Madison", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22725", "FIPS": "51113", "County": "Madison", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22727", "FIPS": "51113", "County": "Madison", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22730", "FIPS": "51113", "County": "Madison", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22731", "FIPS": "51113", "County": "Madison", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22732", "FIPS": "51113", "County": "Madison", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22738", "FIPS": "51113", "County": "Madison", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22743", "FIPS": "51113", "County": "Madison", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22748", "FIPS": "51113", "County": "Madison", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22948", "FIPS": "51113", "County": "Madison", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22989", "FIPS": "51113", "County": "Madison", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "23021", "FIPS": "51115", "County": "Mathews", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23025", "FIPS": "51115", "County": "Mathews", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23035", "FIPS": "51115", "County": "Mathews", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23045", "FIPS": "51115", "County": "Mathews", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23056", "FIPS": "51115", "County": "Mathews", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23064", "FIPS": "51115", "County": "Mathews", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23066", "FIPS": "51115", "County": "Mathews", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23068", "FIPS": "51115", "County": "Mathews", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23076", "FIPS": "51115", "County": "Mathews", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23109", "FIPS": "51115", "County": "Mathews", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23119", "FIPS": "51115", "County": "Mathews", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23125", "FIPS": "51115", "County": "Mathews", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23128", "FIPS": "51115", "County": "Mathews", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23130", "FIPS": "51115", "County": "Mathews", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23138", "FIPS": "51115", "County": "Mathews", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23163", "FIPS": "51115", "County": "Mathews", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23915", "FIPS": "51117", "County": "Mecklenburg", "State": "VA", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "23917", "FIPS": "51117", "County": "Mecklenburg", "State": "VA", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "23919", "FIPS": "51117", "County": "Mecklenburg", "State": "VA", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "23924", "FIPS": "51117", "County": "Mecklenburg", "State": "VA", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "23927", "FIPS": "51117", "County": "Mecklenburg", "State": "VA", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "23950", "FIPS": "51117", "County": "Mecklenburg", "State": "VA", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "23968", "FIPS": "51117", "County": "Mecklenburg", "State": "VA", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "23970", "FIPS": "51117", "County": "Mecklenburg", "State": "VA", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "24529", "FIPS": "51117", "County": "Mecklenburg", "State": "VA", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "24580", "FIPS": "51117", "County": "Mecklenburg", "State": "VA", "DMA": {"Code": 560, "Name": "RALEIGH-DURHAM"}},
  {"Postal_Code": "23031", "FIPS": "51119", "County": "Middlesex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23032", "FIPS": "51119", "County": "Middlesex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23043", "FIPS": "51119", "County": "Middlesex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23070", "FIPS": "51119", "County": "Middlesex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23071", "FIPS": "51119", "County": "Middlesex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23079", "FIPS": "51119", "County": "Middlesex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23092", "FIPS": "51119", "County": "Middlesex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23149", "FIPS": "51119", "County": "Middlesex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23169", "FIPS": "51119", "County": "Middlesex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23175", "FIPS": "51119", "County": "Middlesex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23176", "FIPS": "51119", "County": "Middlesex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23180", "FIPS": "51119", "County": "Middlesex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "24023", "FIPS": "51121", "County": "Montgomery", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24060", "FIPS": "51121", "County": "Montgomery", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24061", "FIPS": "51121", "County": "Montgomery", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24062", "FIPS": "51121", "County": "Montgomery", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24063", "FIPS": "51121", "County": "Montgomery", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24068", "FIPS": "51121", "County": "Montgomery", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24073", "FIPS": "51121", "County": "Montgomery", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24087", "FIPS": "51121", "County": "Montgomery", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24111", "FIPS": "51121", "County": "Montgomery", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24138", "FIPS": "51121", "County": "Montgomery", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24149", "FIPS": "51121", "County": "Montgomery", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24162", "FIPS": "51121", "County": "Montgomery", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "22920", "FIPS": "51125", "County": "Nelson", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "22922", "FIPS": "51125", "County": "Nelson", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "22938", "FIPS": "51125", "County": "Nelson", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "22949", "FIPS": "51125", "County": "Nelson", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "22958", "FIPS": "51125", "County": "Nelson", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "22964", "FIPS": "51125", "County": "Nelson", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "22967", "FIPS": "51125", "County": "Nelson", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "22969", "FIPS": "51125", "County": "Nelson", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "22971", "FIPS": "51125", "County": "Nelson", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "22976", "FIPS": "51125", "County": "Nelson", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24464", "FIPS": "51125", "County": "Nelson", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24553", "FIPS": "51125", "County": "Nelson", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24581", "FIPS": "51125", "County": "Nelson", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24599", "FIPS": "51125", "County": "Nelson", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23011", "FIPS": "51127", "County": "New Kent", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23089", "FIPS": "51127", "County": "New Kent", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23124", "FIPS": "51127", "County": "New Kent", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23140", "FIPS": "51127", "County": "New Kent", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23141", "FIPS": "51127", "County": "New Kent", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23307", "FIPS": "51131", "County": "Northampton", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23310", "FIPS": "51131", "County": "Northampton", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23313", "FIPS": "51131", "County": "Northampton", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23316", "FIPS": "51131", "County": "Northampton", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23347", "FIPS": "51131", "County": "Northampton", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23350", "FIPS": "51131", "County": "Northampton", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23354", "FIPS": "51131", "County": "Northampton", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23398", "FIPS": "51131", "County": "Northampton", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23405", "FIPS": "51131", "County": "Northampton", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23408", "FIPS": "51131", "County": "Northampton", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23413", "FIPS": "51131", "County": "Northampton", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23419", "FIPS": "51131", "County": "Northampton", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23429", "FIPS": "51131", "County": "Northampton", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23443", "FIPS": "51131", "County": "Northampton", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23482", "FIPS": "51131", "County": "Northampton", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23486", "FIPS": "51131", "County": "Northampton", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "22432", "FIPS": "51133", "County": "Northumberland", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22435", "FIPS": "51133", "County": "Northumberland", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22456", "FIPS": "51133", "County": "Northumberland", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22473", "FIPS": "51133", "County": "Northumberland", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22511", "FIPS": "51133", "County": "Northumberland", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22530", "FIPS": "51133", "County": "Northumberland", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22539", "FIPS": "51133", "County": "Northumberland", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22579", "FIPS": "51133", "County": "Northumberland", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23824", "FIPS": "51135", "County": "Nottoway", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23825", "FIPS": "51135", "County": "Nottoway", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23922", "FIPS": "51135", "County": "Nottoway", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23930", "FIPS": "51135", "County": "Nottoway", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23955", "FIPS": "51135", "County": "Nottoway", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22433", "FIPS": "51137", "County": "Orange", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22508", "FIPS": "51137", "County": "Orange", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22542", "FIPS": "51137", "County": "Orange", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22567", "FIPS": "51137", "County": "Orange", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22923", "FIPS": "51137", "County": "Orange", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22942", "FIPS": "51137", "County": "Orange", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22957", "FIPS": "51137", "County": "Orange", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22960", "FIPS": "51137", "County": "Orange", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22972", "FIPS": "51137", "County": "Orange", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22650", "FIPS": "51139", "County": "Page", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22835", "FIPS": "51139", "County": "Page", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22849", "FIPS": "51139", "County": "Page", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22851", "FIPS": "51139", "County": "Page", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "24053", "FIPS": "51141", "County": "Patrick", "State": "VA", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "24076", "FIPS": "51141", "County": "Patrick", "State": "VA", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "24082", "FIPS": "51141", "County": "Patrick", "State": "VA", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "24120", "FIPS": "51141", "County": "Patrick", "State": "VA", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "24133", "FIPS": "51141", "County": "Patrick", "State": "VA", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "24171", "FIPS": "51141", "County": "Patrick", "State": "VA", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "24177", "FIPS": "51141", "County": "Patrick", "State": "VA", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "24185", "FIPS": "51141", "County": "Patrick", "State": "VA", "DMA": {"Code": 518, "Name": "GREENSBORO-HIGH POINT-WINSTON SALEM"}},
  {"Postal_Code": "24069", "FIPS": "51143", "County": "Pittsylvania", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24139", "FIPS": "51143", "County": "Pittsylvania", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24161", "FIPS": "51143", "County": "Pittsylvania", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24527", "FIPS": "51143", "County": "Pittsylvania", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24530", "FIPS": "51143", "County": "Pittsylvania", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24531", "FIPS": "51143", "County": "Pittsylvania", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24549", "FIPS": "51143", "County": "Pittsylvania", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24557", "FIPS": "51143", "County": "Pittsylvania", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24563", "FIPS": "51143", "County": "Pittsylvania", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24565", "FIPS": "51143", "County": "Pittsylvania", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24566", "FIPS": "51143", "County": "Pittsylvania", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24569", "FIPS": "51143", "County": "Pittsylvania", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24586", "FIPS": "51143", "County": "Pittsylvania", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24594", "FIPS": "51143", "County": "Pittsylvania", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23101", "FIPS": "51145", "County": "Powhatan", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23139", "FIPS": "51145", "County": "Powhatan", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23901", "FIPS": "51147", "County": "Prince Edward", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23909", "FIPS": "51147", "County": "Prince Edward", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23942", "FIPS": "51147", "County": "Prince Edward", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23943", "FIPS": "51147", "County": "Prince Edward", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23954", "FIPS": "51147", "County": "Prince Edward", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23960", "FIPS": "51147", "County": "Prince Edward", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23966", "FIPS": "51147", "County": "Prince Edward", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23801", "FIPS": "51149", "County": "Prince George", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23842", "FIPS": "51149", "County": "Prince George", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23875", "FIPS": "51149", "County": "Prince George", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "20109", "FIPS": "51153", "County": "Prince William", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20111", "FIPS": "51153", "County": "Prince William", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20112", "FIPS": "51153", "County": "Prince William", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20136", "FIPS": "51153", "County": "Prince William", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20143", "FIPS": "51153", "County": "Prince William", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20155", "FIPS": "51153", "County": "Prince William", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20156", "FIPS": "51153", "County": "Prince William", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20168", "FIPS": "51153", "County": "Prince William", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20169", "FIPS": "51153", "County": "Prince William", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20181", "FIPS": "51153", "County": "Prince William", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20182", "FIPS": "51153", "County": "Prince William", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22025", "FIPS": "51153", "County": "Prince William", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22026", "FIPS": "51153", "County": "Prince William", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22125", "FIPS": "51153", "County": "Prince William", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22134", "FIPS": "51153", "County": "Prince William", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22172", "FIPS": "51153", "County": "Prince William", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22191", "FIPS": "51153", "County": "Prince William", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22192", "FIPS": "51153", "County": "Prince William", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22193", "FIPS": "51153", "County": "Prince William", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22194", "FIPS": "51153", "County": "Prince William", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22195", "FIPS": "51153", "County": "Prince William", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "24058", "FIPS": "51155", "County": "Pulaski", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24084", "FIPS": "51155", "County": "Pulaski", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24126", "FIPS": "51155", "County": "Pulaski", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24129", "FIPS": "51155", "County": "Pulaski", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24132", "FIPS": "51155", "County": "Pulaski", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24301", "FIPS": "51155", "County": "Pulaski", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24324", "FIPS": "51155", "County": "Pulaski", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24347", "FIPS": "51155", "County": "Pulaski", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "22623", "FIPS": "51157", "County": "Rappahannock", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22627", "FIPS": "51157", "County": "Rappahannock", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22640", "FIPS": "51157", "County": "Rappahannock", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22716", "FIPS": "51157", "County": "Rappahannock", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22740", "FIPS": "51157", "County": "Rappahannock", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22747", "FIPS": "51157", "County": "Rappahannock", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22749", "FIPS": "51157", "County": "Rappahannock", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22460", "FIPS": "51159", "County": "Richmond", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22472", "FIPS": "51159", "County": "Richmond", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22548", "FIPS": "51159", "County": "Richmond", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22570", "FIPS": "51159", "County": "Richmond", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22572", "FIPS": "51159", "County": "Richmond", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "24018", "FIPS": "51161", "County": "Roanoke", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24019", "FIPS": "51161", "County": "Roanoke", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24020", "FIPS": "51161", "County": "Roanoke", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24059", "FIPS": "51161", "County": "Roanoke", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24070", "FIPS": "51161", "County": "Roanoke", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24179", "FIPS": "51161", "County": "Roanoke", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24415", "FIPS": "51163", "County": "Rockbridge", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24435", "FIPS": "51163", "County": "Rockbridge", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24439", "FIPS": "51163", "County": "Rockbridge", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24472", "FIPS": "51163", "County": "Rockbridge", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24473", "FIPS": "51163", "County": "Rockbridge", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24483", "FIPS": "51163", "County": "Rockbridge", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24555", "FIPS": "51163", "County": "Rockbridge", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24578", "FIPS": "51163", "County": "Rockbridge", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24579", "FIPS": "51163", "County": "Rockbridge", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "22811", "FIPS": "51165", "County": "Rockingham", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "22812", "FIPS": "51165", "County": "Rockingham", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "22815", "FIPS": "51165", "County": "Rockingham", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "22820", "FIPS": "51165", "County": "Rockingham", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "22821", "FIPS": "51165", "County": "Rockingham", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "22827", "FIPS": "51165", "County": "Rockingham", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "22830", "FIPS": "51165", "County": "Rockingham", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "22831", "FIPS": "51165", "County": "Rockingham", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "22832", "FIPS": "51165", "County": "Rockingham", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "22833", "FIPS": "51165", "County": "Rockingham", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "22834", "FIPS": "51165", "County": "Rockingham", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "22840", "FIPS": "51165", "County": "Rockingham", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "22841", "FIPS": "51165", "County": "Rockingham", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "22846", "FIPS": "51165", "County": "Rockingham", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "22848", "FIPS": "51165", "County": "Rockingham", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "22850", "FIPS": "51165", "County": "Rockingham", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "22853", "FIPS": "51165", "County": "Rockingham", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "24441", "FIPS": "51165", "County": "Rockingham", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "24471", "FIPS": "51165", "County": "Rockingham", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "24224", "FIPS": "51167", "County": "Russell", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24225", "FIPS": "51167", "County": "Russell", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24237", "FIPS": "51167", "County": "Russell", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24260", "FIPS": "51167", "County": "Russell", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24266", "FIPS": "51167", "County": "Russell", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24280", "FIPS": "51167", "County": "Russell", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24649", "FIPS": "51167", "County": "Russell", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24244", "FIPS": "51169", "County": "Scott", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24245", "FIPS": "51169", "County": "Scott", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24250", "FIPS": "51169", "County": "Scott", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24251", "FIPS": "51169", "County": "Scott", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24258", "FIPS": "51169", "County": "Scott", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24271", "FIPS": "51169", "County": "Scott", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24290", "FIPS": "51169", "County": "Scott", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "22626", "FIPS": "51171", "County": "Shenandoah", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22641", "FIPS": "51171", "County": "Shenandoah", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22644", "FIPS": "51171", "County": "Shenandoah", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22652", "FIPS": "51171", "County": "Shenandoah", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22657", "FIPS": "51171", "County": "Shenandoah", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22660", "FIPS": "51171", "County": "Shenandoah", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22664", "FIPS": "51171", "County": "Shenandoah", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22810", "FIPS": "51171", "County": "Shenandoah", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22824", "FIPS": "51171", "County": "Shenandoah", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22842", "FIPS": "51171", "County": "Shenandoah", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22844", "FIPS": "51171", "County": "Shenandoah", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22845", "FIPS": "51171", "County": "Shenandoah", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22847", "FIPS": "51171", "County": "Shenandoah", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "24311", "FIPS": "51173", "County": "Smyth", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24319", "FIPS": "51173", "County": "Smyth", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24354", "FIPS": "51173", "County": "Smyth", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24370", "FIPS": "51173", "County": "Smyth", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24375", "FIPS": "51173", "County": "Smyth", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "23827", "FIPS": "51175", "County": "Southampton", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23828", "FIPS": "51175", "County": "Southampton", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23829", "FIPS": "51175", "County": "Southampton", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23837", "FIPS": "51175", "County": "Southampton", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23844", "FIPS": "51175", "County": "Southampton", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23866", "FIPS": "51175", "County": "Southampton", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23874", "FIPS": "51175", "County": "Southampton", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23878", "FIPS": "51175", "County": "Southampton", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "22407", "FIPS": "51177", "County": "Spotsylvania", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22408", "FIPS": "51177", "County": "Spotsylvania", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22534", "FIPS": "51177", "County": "Spotsylvania", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22553", "FIPS": "51177", "County": "Spotsylvania", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22565", "FIPS": "51177", "County": "Spotsylvania", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22135", "FIPS": "51179", "County": "Stafford", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22403", "FIPS": "51179", "County": "Stafford", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22405", "FIPS": "51179", "County": "Stafford", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22406", "FIPS": "51179", "County": "Stafford", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22412", "FIPS": "51179", "County": "Stafford", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22430", "FIPS": "51179", "County": "Stafford", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22463", "FIPS": "51179", "County": "Stafford", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22471", "FIPS": "51179", "County": "Stafford", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22545", "FIPS": "51179", "County": "Stafford", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22554", "FIPS": "51179", "County": "Stafford", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22555", "FIPS": "51179", "County": "Stafford", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22556", "FIPS": "51179", "County": "Stafford", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "23839", "FIPS": "51181", "County": "Surry", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23846", "FIPS": "51181", "County": "Surry", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23881", "FIPS": "51181", "County": "Surry", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23883", "FIPS": "51181", "County": "Surry", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23899", "FIPS": "51181", "County": "Surry", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23882", "FIPS": "51183", "County": "Sussex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23884", "FIPS": "51183", "County": "Sussex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23888", "FIPS": "51183", "County": "Sussex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23890", "FIPS": "51183", "County": "Sussex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23891", "FIPS": "51183", "County": "Sussex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23897", "FIPS": "51183", "County": "Sussex", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "24316", "FIPS": "51185", "County": "Tazewell", "State": "VA", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24377", "FIPS": "51185", "County": "Tazewell", "State": "VA", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24601", "FIPS": "51185", "County": "Tazewell", "State": "VA", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24602", "FIPS": "51185", "County": "Tazewell", "State": "VA", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24604", "FIPS": "51185", "County": "Tazewell", "State": "VA", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24605", "FIPS": "51185", "County": "Tazewell", "State": "VA", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24606", "FIPS": "51185", "County": "Tazewell", "State": "VA", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24608", "FIPS": "51185", "County": "Tazewell", "State": "VA", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24609", "FIPS": "51185", "County": "Tazewell", "State": "VA", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24612", "FIPS": "51185", "County": "Tazewell", "State": "VA", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24613", "FIPS": "51185", "County": "Tazewell", "State": "VA", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24619", "FIPS": "51185", "County": "Tazewell", "State": "VA", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24622", "FIPS": "51185", "County": "Tazewell", "State": "VA", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24630", "FIPS": "51185", "County": "Tazewell", "State": "VA", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24635", "FIPS": "51185", "County": "Tazewell", "State": "VA", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24637", "FIPS": "51185", "County": "Tazewell", "State": "VA", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24640", "FIPS": "51185", "County": "Tazewell", "State": "VA", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24641", "FIPS": "51185", "County": "Tazewell", "State": "VA", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24651", "FIPS": "51185", "County": "Tazewell", "State": "VA", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "22610", "FIPS": "51187", "County": "Warren", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22630", "FIPS": "51187", "County": "Warren", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22642", "FIPS": "51187", "County": "Warren", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22649", "FIPS": "51187", "County": "Warren", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "24202", "FIPS": "51191", "County": "Washington", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24210", "FIPS": "51191", "County": "Washington", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24211", "FIPS": "51191", "County": "Washington", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24212", "FIPS": "51191", "County": "Washington", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24236", "FIPS": "51191", "County": "Washington", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24270", "FIPS": "51191", "County": "Washington", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24327", "FIPS": "51191", "County": "Washington", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24340", "FIPS": "51191", "County": "Washington", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24361", "FIPS": "51191", "County": "Washington", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "22442", "FIPS": "51193", "County": "Westmoreland", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22443", "FIPS": "51193", "County": "Westmoreland", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22469", "FIPS": "51193", "County": "Westmoreland", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22488", "FIPS": "51193", "County": "Westmoreland", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22520", "FIPS": "51193", "County": "Westmoreland", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22524", "FIPS": "51193", "County": "Westmoreland", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22529", "FIPS": "51193", "County": "Westmoreland", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22558", "FIPS": "51193", "County": "Westmoreland", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22577", "FIPS": "51193", "County": "Westmoreland", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22581", "FIPS": "51193", "County": "Westmoreland", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "24215", "FIPS": "51195", "County": "Wise", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24216", "FIPS": "51195", "County": "Wise", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24219", "FIPS": "51195", "County": "Wise", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24230", "FIPS": "51195", "County": "Wise", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24246", "FIPS": "51195", "County": "Wise", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24279", "FIPS": "51195", "County": "Wise", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24283", "FIPS": "51195", "County": "Wise", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24293", "FIPS": "51195", "County": "Wise", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24312", "FIPS": "51197", "County": "Wythe", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24313", "FIPS": "51197", "County": "Wythe", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24322", "FIPS": "51197", "County": "Wythe", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24323", "FIPS": "51197", "County": "Wythe", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24350", "FIPS": "51197", "County": "Wythe", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24360", "FIPS": "51197", "County": "Wythe", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24368", "FIPS": "51197", "County": "Wythe", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24374", "FIPS": "51197", "County": "Wythe", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24382", "FIPS": "51197", "County": "Wythe", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23090", "FIPS": "51199", "County": "York", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23665", "FIPS": "51199", "County": "York", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23690", "FIPS": "51199", "County": "York", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23691", "FIPS": "51199", "County": "York", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23692", "FIPS": "51199", "County": "York", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23693", "FIPS": "51199", "County": "York", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23694", "FIPS": "51199", "County": "York", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23696", "FIPS": "51199", "County": "York", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "22301", "FIPS": "51510", "County": "Alexandria City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22302", "FIPS": "51510", "County": "Alexandria City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22304", "FIPS": "51510", "County": "Alexandria City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22305", "FIPS": "51510", "County": "Alexandria City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22311", "FIPS": "51510", "County": "Alexandria City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22313", "FIPS": "51510", "County": "Alexandria City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22314", "FIPS": "51510", "County": "Alexandria City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22320", "FIPS": "51510", "County": "Alexandria City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22331", "FIPS": "51510", "County": "Alexandria City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22332", "FIPS": "51510", "County": "Alexandria City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22333", "FIPS": "51510", "County": "Alexandria City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22334", "FIPS": "51510", "County": "Alexandria City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22336", "FIPS": "51510", "County": "Alexandria City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "24523", "FIPS": "51515", "County": "Bedford City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24201", "FIPS": "51520", "County": "Bristol", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24203", "FIPS": "51520", "County": "Bristol", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24209", "FIPS": "51520", "County": "Bristol", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "24416", "FIPS": "51530", "County": "Buena Vista City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "22902", "FIPS": "51540", "County": "Charlottesville City", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22903", "FIPS": "51540", "County": "Charlottesville City", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22904", "FIPS": "51540", "County": "Charlottesville City", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22905", "FIPS": "51540", "County": "Charlottesville City", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22906", "FIPS": "51540", "County": "Charlottesville City", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22907", "FIPS": "51540", "County": "Charlottesville City", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22908", "FIPS": "51540", "County": "Charlottesville City", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "22910", "FIPS": "51540", "County": "Charlottesville City", "State": "VA", "DMA": {"Code": 584, "Name": "CHARLOTTESVILLE"}},
  {"Postal_Code": "23320", "FIPS": "51550", "County": "Chesapeake City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23321", "FIPS": "51550", "County": "Chesapeake City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23322", "FIPS": "51550", "County": "Chesapeake City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23323", "FIPS": "51550", "County": "Chesapeake City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23324", "FIPS": "51550", "County": "Chesapeake City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23325", "FIPS": "51550", "County": "Chesapeake City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23326", "FIPS": "51550", "County": "Chesapeake City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23327", "FIPS": "51550", "County": "Chesapeake City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23328", "FIPS": "51550", "County": "Chesapeake City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23834", "FIPS": "51570", "County": "Colonial Heights City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "24426", "FIPS": "51580", "County": "Covington City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24540", "FIPS": "51590", "County": "Danville City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24541", "FIPS": "51590", "County": "Danville City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24543", "FIPS": "51590", "County": "Danville City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24544", "FIPS": "51590", "County": "Danville City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23847", "FIPS": "51595", "County": "Emporia City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "22030", "FIPS": "51600", "County": "Fairfax City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22038", "FIPS": "51600", "County": "Fairfax City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22040", "FIPS": "51610", "County": "Falls Church City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22046", "FIPS": "51610", "County": "Falls Church City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "23851", "FIPS": "51620", "County": "Franklin City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "22401", "FIPS": "51630", "County": "Fredericksburg City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22402", "FIPS": "51630", "County": "Fredericksburg City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22404", "FIPS": "51630", "County": "Fredericksburg City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "24333", "FIPS": "51640", "County": "Galax City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23630", "FIPS": "51650", "County": "Hampton City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23651", "FIPS": "51650", "County": "Hampton City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23661", "FIPS": "51650", "County": "Hampton City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23663", "FIPS": "51650", "County": "Hampton City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23664", "FIPS": "51650", "County": "Hampton City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23666", "FIPS": "51650", "County": "Hampton City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23667", "FIPS": "51650", "County": "Hampton City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23668", "FIPS": "51650", "County": "Hampton City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23669", "FIPS": "51650", "County": "Hampton City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23670", "FIPS": "51650", "County": "Hampton City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23681", "FIPS": "51650", "County": "Hampton City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "22801", "FIPS": "51660", "County": "Harrisonburg City", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "22802", "FIPS": "51660", "County": "Harrisonburg City", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "22803", "FIPS": "51660", "County": "Harrisonburg City", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "22807", "FIPS": "51660", "County": "Harrisonburg City", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "23860", "FIPS": "51670", "County": "Hopewell City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "24450", "FIPS": "51678", "County": "Lexington City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24501", "FIPS": "51680", "County": "Lynchburg City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24502", "FIPS": "51680", "County": "Lynchburg City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24503", "FIPS": "51680", "County": "Lynchburg City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24504", "FIPS": "51680", "County": "Lynchburg City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24505", "FIPS": "51680", "County": "Lynchburg City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24506", "FIPS": "51680", "County": "Lynchburg City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24512", "FIPS": "51680", "County": "Lynchburg City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24513", "FIPS": "51680", "County": "Lynchburg City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24514", "FIPS": "51680", "County": "Lynchburg City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24515", "FIPS": "51680", "County": "Lynchburg City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "20108", "FIPS": "51683", "County": "Manassas City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20110", "FIPS": "51683", "County": "Manassas City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "20113", "FIPS": "51685", "County": "Manassas Park City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "24112", "FIPS": "51690", "County": "Martinsville City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24113", "FIPS": "51690", "County": "Martinsville City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24114", "FIPS": "51690", "County": "Martinsville City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24115", "FIPS": "51690", "County": "Martinsville City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23601", "FIPS": "51700", "County": "Newport News City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23602", "FIPS": "51700", "County": "Newport News City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23603", "FIPS": "51700", "County": "Newport News City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23604", "FIPS": "51700", "County": "Newport News City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23605", "FIPS": "51700", "County": "Newport News City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23606", "FIPS": "51700", "County": "Newport News City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23607", "FIPS": "51700", "County": "Newport News City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23608", "FIPS": "51700", "County": "Newport News City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23609", "FIPS": "51700", "County": "Newport News City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23612", "FIPS": "51700", "County": "Newport News City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23628", "FIPS": "51700", "County": "Newport News City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23501", "FIPS": "51710", "County": "Norfolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23502", "FIPS": "51710", "County": "Norfolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23503", "FIPS": "51710", "County": "Norfolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23504", "FIPS": "51710", "County": "Norfolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23505", "FIPS": "51710", "County": "Norfolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23506", "FIPS": "51710", "County": "Norfolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23507", "FIPS": "51710", "County": "Norfolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23508", "FIPS": "51710", "County": "Norfolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23509", "FIPS": "51710", "County": "Norfolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23510", "FIPS": "51710", "County": "Norfolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23511", "FIPS": "51710", "County": "Norfolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23512", "FIPS": "51710", "County": "Norfolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23513", "FIPS": "51710", "County": "Norfolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23514", "FIPS": "51710", "County": "Norfolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23515", "FIPS": "51710", "County": "Norfolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23517", "FIPS": "51710", "County": "Norfolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23518", "FIPS": "51710", "County": "Norfolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23519", "FIPS": "51710", "County": "Norfolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23520", "FIPS": "51710", "County": "Norfolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23521", "FIPS": "51710", "County": "Norfolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23523", "FIPS": "51710", "County": "Norfolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23529", "FIPS": "51710", "County": "Norfolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23541", "FIPS": "51710", "County": "Norfolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23551", "FIPS": "51710", "County": "Norfolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "24273", "FIPS": "51720", "County": "Norton City", "State": "VA", "DMA": {"Code": 531, "Name": "TRI-CITIES, TN-VA"}},
  {"Postal_Code": "23803", "FIPS": "51730", "County": "Petersburg City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23804", "FIPS": "51730", "County": "Petersburg City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23805", "FIPS": "51730", "County": "Petersburg City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23806", "FIPS": "51730", "County": "Petersburg City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23662", "FIPS": "51735", "County": "Poquoson City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23701", "FIPS": "51740", "County": "Portsmouth City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23702", "FIPS": "51740", "County": "Portsmouth City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23703", "FIPS": "51740", "County": "Portsmouth City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23704", "FIPS": "51740", "County": "Portsmouth City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23705", "FIPS": "51740", "County": "Portsmouth City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23707", "FIPS": "51740", "County": "Portsmouth City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23708", "FIPS": "51740", "County": "Portsmouth City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23709", "FIPS": "51740", "County": "Portsmouth City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "24141", "FIPS": "51750", "County": "Radford City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24142", "FIPS": "51750", "County": "Radford City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24143", "FIPS": "51750", "County": "Radford City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "23173", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23218", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23219", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23220", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23221", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23222", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23223", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23224", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23225", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23232", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23240", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23241", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23249", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23260", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23261", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23269", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23273", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23274", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23276", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23278", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23279", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23282", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23284", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23285", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23286", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23289", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23290", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23291", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23292", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23293", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23295", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "23298", "FIPS": "51760", "County": "Richmond City", "State": "VA", "DMA": {"Code": 556, "Name": "RICHMOND-PETERSBURG"}},
  {"Postal_Code": "24001", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24002", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24003", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24004", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24005", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24006", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24007", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24008", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24009", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24010", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24011", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24012", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24013", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24014", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24015", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24016", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24017", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24022", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24024", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24025", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24026", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24027", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24028", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24029", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24030", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24031", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24032", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24033", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24034", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24035", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24036", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24037", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24038", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24040", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24042", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24043", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24044", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24045", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24048", "FIPS": "51770", "County": "Roanoke City", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24153", "FIPS": "51775", "County": "Salem", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24155", "FIPS": "51775", "County": "Salem", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24157", "FIPS": "51775", "County": "Salem", "State": "VA", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24401", "FIPS": "51790", "County": "Staunton City", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "24402", "FIPS": "51790", "County": "Staunton City", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "23432", "FIPS": "51800", "County": "Suffolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23433", "FIPS": "51800", "County": "Suffolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23434", "FIPS": "51800", "County": "Suffolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23435", "FIPS": "51800", "County": "Suffolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23436", "FIPS": "51800", "County": "Suffolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23437", "FIPS": "51800", "County": "Suffolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23438", "FIPS": "51800", "County": "Suffolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23439", "FIPS": "51800", "County": "Suffolk City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23450", "FIPS": "51810", "County": "Virginia Beach City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23451", "FIPS": "51810", "County": "Virginia Beach City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23452", "FIPS": "51810", "County": "Virginia Beach City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23453", "FIPS": "51810", "County": "Virginia Beach City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23454", "FIPS": "51810", "County": "Virginia Beach City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23455", "FIPS": "51810", "County": "Virginia Beach City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23456", "FIPS": "51810", "County": "Virginia Beach City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23457", "FIPS": "51810", "County": "Virginia Beach City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23458", "FIPS": "51810", "County": "Virginia Beach City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23459", "FIPS": "51810", "County": "Virginia Beach City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23460", "FIPS": "51810", "County": "Virginia Beach City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23461", "FIPS": "51810", "County": "Virginia Beach City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23462", "FIPS": "51810", "County": "Virginia Beach City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23463", "FIPS": "51810", "County": "Virginia Beach City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23464", "FIPS": "51810", "County": "Virginia Beach City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23465", "FIPS": "51810", "County": "Virginia Beach City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23466", "FIPS": "51810", "County": "Virginia Beach City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23467", "FIPS": "51810", "County": "Virginia Beach City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23471", "FIPS": "51810", "County": "Virginia Beach City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23479", "FIPS": "51810", "County": "Virginia Beach City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "22980", "FIPS": "51820", "County": "Waynesboro City", "State": "VA", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "23186", "FIPS": "51830", "County": "Williamsburg City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "23187", "FIPS": "51830", "County": "Williamsburg City", "State": "VA", "DMA": {"Code": 544, "Name": "NORFOLK-PORTSMOUTH-NEWPORT NEWS"}},
  {"Postal_Code": "22601", "FIPS": "51840", "County": "Winchester City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "22604", "FIPS": "51840", "County": "Winchester City", "State": "VA", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "99105", "FIPS": "53001", "County": "Adams", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99169", "FIPS": "53001", "County": "Adams", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99341", "FIPS": "53001", "County": "Adams", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99344", "FIPS": "53001", "County": "Adams", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99371", "FIPS": "53001", "County": "Adams", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99401", "FIPS": "53003", "County": "Asotin", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99402", "FIPS": "53003", "County": "Asotin", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99403", "FIPS": "53003", "County": "Asotin", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99320", "FIPS": "53005", "County": "Benton", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "99336", "FIPS": "53005", "County": "Benton", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "99337", "FIPS": "53005", "County": "Benton", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "99338", "FIPS": "53005", "County": "Benton", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "99345", "FIPS": "53005", "County": "Benton", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "99346", "FIPS": "53005", "County": "Benton", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "99350", "FIPS": "53005", "County": "Benton", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "99352", "FIPS": "53005", "County": "Benton", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "99353", "FIPS": "53005", "County": "Benton", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "99354", "FIPS": "53005", "County": "Benton", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98801", "FIPS": "53007", "County": "Chelan", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98807", "FIPS": "53007", "County": "Chelan", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98811", "FIPS": "53007", "County": "Chelan", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98815", "FIPS": "53007", "County": "Chelan", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98816", "FIPS": "53007", "County": "Chelan", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98817", "FIPS": "53007", "County": "Chelan", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98821", "FIPS": "53007", "County": "Chelan", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98822", "FIPS": "53007", "County": "Chelan", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98826", "FIPS": "53007", "County": "Chelan", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98828", "FIPS": "53007", "County": "Chelan", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98831", "FIPS": "53007", "County": "Chelan", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98836", "FIPS": "53007", "County": "Chelan", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98847", "FIPS": "53007", "County": "Chelan", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98852", "FIPS": "53007", "County": "Chelan", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98305", "FIPS": "53009", "County": "Clallam", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98324", "FIPS": "53009", "County": "Clallam", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98326", "FIPS": "53009", "County": "Clallam", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98331", "FIPS": "53009", "County": "Clallam", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98343", "FIPS": "53009", "County": "Clallam", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98350", "FIPS": "53009", "County": "Clallam", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98357", "FIPS": "53009", "County": "Clallam", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98362", "FIPS": "53009", "County": "Clallam", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98363", "FIPS": "53009", "County": "Clallam", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98381", "FIPS": "53009", "County": "Clallam", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98382", "FIPS": "53009", "County": "Clallam", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98601", "FIPS": "53011", "County": "Clark", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98604", "FIPS": "53011", "County": "Clark", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98606", "FIPS": "53011", "County": "Clark", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98607", "FIPS": "53011", "County": "Clark", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98622", "FIPS": "53011", "County": "Clark", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98629", "FIPS": "53011", "County": "Clark", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98642", "FIPS": "53011", "County": "Clark", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98660", "FIPS": "53011", "County": "Clark", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98661", "FIPS": "53011", "County": "Clark", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98662", "FIPS": "53011", "County": "Clark", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98663", "FIPS": "53011", "County": "Clark", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98664", "FIPS": "53011", "County": "Clark", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98665", "FIPS": "53011", "County": "Clark", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98666", "FIPS": "53011", "County": "Clark", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98667", "FIPS": "53011", "County": "Clark", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98668", "FIPS": "53011", "County": "Clark", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98671", "FIPS": "53011", "County": "Clark", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98675", "FIPS": "53011", "County": "Clark", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98682", "FIPS": "53011", "County": "Clark", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98683", "FIPS": "53011", "County": "Clark", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98684", "FIPS": "53011", "County": "Clark", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98685", "FIPS": "53011", "County": "Clark", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98686", "FIPS": "53011", "County": "Clark", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98687", "FIPS": "53011", "County": "Clark", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "99328", "FIPS": "53013", "County": "Columbia", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99359", "FIPS": "53013", "County": "Columbia", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98581", "FIPS": "53015", "County": "Cowlitz", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98603", "FIPS": "53015", "County": "Cowlitz", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98609", "FIPS": "53015", "County": "Cowlitz", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98611", "FIPS": "53015", "County": "Cowlitz", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98616", "FIPS": "53015", "County": "Cowlitz", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98625", "FIPS": "53015", "County": "Cowlitz", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98626", "FIPS": "53015", "County": "Cowlitz", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98632", "FIPS": "53015", "County": "Cowlitz", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98645", "FIPS": "53015", "County": "Cowlitz", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98649", "FIPS": "53015", "County": "Cowlitz", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98674", "FIPS": "53015", "County": "Cowlitz", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98802", "FIPS": "53017", "County": "Douglas", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98813", "FIPS": "53017", "County": "Douglas", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98830", "FIPS": "53017", "County": "Douglas", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98843", "FIPS": "53017", "County": "Douglas", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98845", "FIPS": "53017", "County": "Douglas", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98850", "FIPS": "53017", "County": "Douglas", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98858", "FIPS": "53017", "County": "Douglas", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "99107", "FIPS": "53019", "County": "Ferry", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99118", "FIPS": "53019", "County": "Ferry", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99121", "FIPS": "53019", "County": "Ferry", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99138", "FIPS": "53019", "County": "Ferry", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99140", "FIPS": "53019", "County": "Ferry", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99146", "FIPS": "53019", "County": "Ferry", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99150", "FIPS": "53019", "County": "Ferry", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99160", "FIPS": "53019", "County": "Ferry", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99166", "FIPS": "53019", "County": "Ferry", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99301", "FIPS": "53021", "County": "Franklin", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "99302", "FIPS": "53021", "County": "Franklin", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "99326", "FIPS": "53021", "County": "Franklin", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "99330", "FIPS": "53021", "County": "Franklin", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "99335", "FIPS": "53021", "County": "Franklin", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "99343", "FIPS": "53021", "County": "Franklin", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "99347", "FIPS": "53023", "County": "Garfield", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98823", "FIPS": "53025", "County": "Grant", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98824", "FIPS": "53025", "County": "Grant", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98832", "FIPS": "53025", "County": "Grant", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98837", "FIPS": "53025", "County": "Grant", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98848", "FIPS": "53025", "County": "Grant", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98851", "FIPS": "53025", "County": "Grant", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98853", "FIPS": "53025", "County": "Grant", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98857", "FIPS": "53025", "County": "Grant", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98860", "FIPS": "53025", "County": "Grant", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99115", "FIPS": "53025", "County": "Grant", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99123", "FIPS": "53025", "County": "Grant", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99133", "FIPS": "53025", "County": "Grant", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99135", "FIPS": "53025", "County": "Grant", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99321", "FIPS": "53025", "County": "Grant", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99349", "FIPS": "53025", "County": "Grant", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99357", "FIPS": "53025", "County": "Grant", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98520", "FIPS": "53027", "County": "Grays Harbor", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98526", "FIPS": "53027", "County": "Grays Harbor", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98535", "FIPS": "53027", "County": "Grays Harbor", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98536", "FIPS": "53027", "County": "Grays Harbor", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98537", "FIPS": "53027", "County": "Grays Harbor", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98541", "FIPS": "53027", "County": "Grays Harbor", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98547", "FIPS": "53027", "County": "Grays Harbor", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98550", "FIPS": "53027", "County": "Grays Harbor", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98552", "FIPS": "53027", "County": "Grays Harbor", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98557", "FIPS": "53027", "County": "Grays Harbor", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98559", "FIPS": "53027", "County": "Grays Harbor", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98562", "FIPS": "53027", "County": "Grays Harbor", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98563", "FIPS": "53027", "County": "Grays Harbor", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98566", "FIPS": "53027", "County": "Grays Harbor", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98568", "FIPS": "53027", "County": "Grays Harbor", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98569", "FIPS": "53027", "County": "Grays Harbor", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98571", "FIPS": "53027", "County": "Grays Harbor", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98575", "FIPS": "53027", "County": "Grays Harbor", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98583", "FIPS": "53027", "County": "Grays Harbor", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98587", "FIPS": "53027", "County": "Grays Harbor", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98595", "FIPS": "53027", "County": "Grays Harbor", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98236", "FIPS": "53029", "County": "Island", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98239", "FIPS": "53029", "County": "Island", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98249", "FIPS": "53029", "County": "Island", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98253", "FIPS": "53029", "County": "Island", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98260", "FIPS": "53029", "County": "Island", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98277", "FIPS": "53029", "County": "Island", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98278", "FIPS": "53029", "County": "Island", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98282", "FIPS": "53029", "County": "Island", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98320", "FIPS": "53031", "County": "Jefferson", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98325", "FIPS": "53031", "County": "Jefferson", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98339", "FIPS": "53031", "County": "Jefferson", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98358", "FIPS": "53031", "County": "Jefferson", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98365", "FIPS": "53031", "County": "Jefferson", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98368", "FIPS": "53031", "County": "Jefferson", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98376", "FIPS": "53031", "County": "Jefferson", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98001", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98002", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98003", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98004", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98005", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98006", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98007", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98008", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98009", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98010", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98011", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98013", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98014", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98015", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98019", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98022", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98023", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98024", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98025", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98027", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98028", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98029", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98030", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98031", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98032", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98033", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98034", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98035", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98038", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98039", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98040", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98041", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98042", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98045", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98047", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98050", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98051", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98052", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98053", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98054", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98055", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98056", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98057", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98058", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98059", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98062", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98063", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98064", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98065", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98070", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98071", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98072", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98073", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98074", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98075", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98077", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98083", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98089", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98092", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98093", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98101", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98102", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98103", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98104", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98105", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98106", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98107", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98108", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98109", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98111", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98112", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98113", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98114", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98115", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98116", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98117", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98118", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98119", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98121", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98122", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98124", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98125", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98126", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98127", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98129", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98131", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98132", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98133", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98134", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98136", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98138", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98139", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98141", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98144", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98145", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98146", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98148", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98151", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98154", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98155", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98158", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98160", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98161", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98164", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98165", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98166", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98168", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98170", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98171", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98174", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98175", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98177", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98178", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98181", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98184", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98185", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98188", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98190", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98191", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98194", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98195", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98198", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98199", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98224", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98288", "FIPS": "53033", "County": "King", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98061", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98110", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98310", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98311", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98312", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98314", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98315", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98322", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98337", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98340", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98342", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98345", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98346", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98353", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98359", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98364", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98366", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98367", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98370", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98378", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98380", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98383", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98384", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98386", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98392", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98393", "FIPS": "53035", "County": "Kitsap", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98068", "FIPS": "53037", "County": "Kittitas", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98922", "FIPS": "53037", "County": "Kittitas", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98925", "FIPS": "53037", "County": "Kittitas", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98926", "FIPS": "53037", "County": "Kittitas", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98934", "FIPS": "53037", "County": "Kittitas", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98940", "FIPS": "53037", "County": "Kittitas", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98941", "FIPS": "53037", "County": "Kittitas", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98943", "FIPS": "53037", "County": "Kittitas", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98946", "FIPS": "53037", "County": "Kittitas", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98950", "FIPS": "53037", "County": "Kittitas", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98602", "FIPS": "53039", "County": "Klickitat", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98605", "FIPS": "53039", "County": "Klickitat", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98613", "FIPS": "53039", "County": "Klickitat", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98617", "FIPS": "53039", "County": "Klickitat", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98619", "FIPS": "53039", "County": "Klickitat", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98620", "FIPS": "53039", "County": "Klickitat", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98623", "FIPS": "53039", "County": "Klickitat", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98628", "FIPS": "53039", "County": "Klickitat", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98635", "FIPS": "53039", "County": "Klickitat", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98650", "FIPS": "53039", "County": "Klickitat", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98670", "FIPS": "53039", "County": "Klickitat", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98672", "FIPS": "53039", "County": "Klickitat", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98673", "FIPS": "53039", "County": "Klickitat", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "99322", "FIPS": "53039", "County": "Klickitat", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "99356", "FIPS": "53039", "County": "Klickitat", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98336", "FIPS": "53041", "County": "Lewis", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98355", "FIPS": "53041", "County": "Lewis", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98356", "FIPS": "53041", "County": "Lewis", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98361", "FIPS": "53041", "County": "Lewis", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98377", "FIPS": "53041", "County": "Lewis", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98522", "FIPS": "53041", "County": "Lewis", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98531", "FIPS": "53041", "County": "Lewis", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98532", "FIPS": "53041", "County": "Lewis", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98533", "FIPS": "53041", "County": "Lewis", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98538", "FIPS": "53041", "County": "Lewis", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98539", "FIPS": "53041", "County": "Lewis", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98542", "FIPS": "53041", "County": "Lewis", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98544", "FIPS": "53041", "County": "Lewis", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98564", "FIPS": "53041", "County": "Lewis", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98565", "FIPS": "53041", "County": "Lewis", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98570", "FIPS": "53041", "County": "Lewis", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98572", "FIPS": "53041", "County": "Lewis", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98582", "FIPS": "53041", "County": "Lewis", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98585", "FIPS": "53041", "County": "Lewis", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98591", "FIPS": "53041", "County": "Lewis", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98593", "FIPS": "53041", "County": "Lewis", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98596", "FIPS": "53041", "County": "Lewis", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "99008", "FIPS": "53043", "County": "Lincoln", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99029", "FIPS": "53043", "County": "Lincoln", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99032", "FIPS": "53043", "County": "Lincoln", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99103", "FIPS": "53043", "County": "Lincoln", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99117", "FIPS": "53043", "County": "Lincoln", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99122", "FIPS": "53043", "County": "Lincoln", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99134", "FIPS": "53043", "County": "Lincoln", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99144", "FIPS": "53043", "County": "Lincoln", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99147", "FIPS": "53043", "County": "Lincoln", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99154", "FIPS": "53043", "County": "Lincoln", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99159", "FIPS": "53043", "County": "Lincoln", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99185", "FIPS": "53043", "County": "Lincoln", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98524", "FIPS": "53045", "County": "Mason", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98528", "FIPS": "53045", "County": "Mason", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98546", "FIPS": "53045", "County": "Mason", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98548", "FIPS": "53045", "County": "Mason", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98555", "FIPS": "53045", "County": "Mason", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98560", "FIPS": "53045", "County": "Mason", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98584", "FIPS": "53045", "County": "Mason", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98588", "FIPS": "53045", "County": "Mason", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98592", "FIPS": "53045", "County": "Mason", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98812", "FIPS": "53047", "County": "Okanogan", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98814", "FIPS": "53047", "County": "Okanogan", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98819", "FIPS": "53047", "County": "Okanogan", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98827", "FIPS": "53047", "County": "Okanogan", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98829", "FIPS": "53047", "County": "Okanogan", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98833", "FIPS": "53047", "County": "Okanogan", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98834", "FIPS": "53047", "County": "Okanogan", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98840", "FIPS": "53047", "County": "Okanogan", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98841", "FIPS": "53047", "County": "Okanogan", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98844", "FIPS": "53047", "County": "Okanogan", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98846", "FIPS": "53047", "County": "Okanogan", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98849", "FIPS": "53047", "County": "Okanogan", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98855", "FIPS": "53047", "County": "Okanogan", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98856", "FIPS": "53047", "County": "Okanogan", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98859", "FIPS": "53047", "County": "Okanogan", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98862", "FIPS": "53047", "County": "Okanogan", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99116", "FIPS": "53047", "County": "Okanogan", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99124", "FIPS": "53047", "County": "Okanogan", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99155", "FIPS": "53047", "County": "Okanogan", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98527", "FIPS": "53049", "County": "Pacific", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98554", "FIPS": "53049", "County": "Pacific", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98561", "FIPS": "53049", "County": "Pacific", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98577", "FIPS": "53049", "County": "Pacific", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98586", "FIPS": "53049", "County": "Pacific", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98590", "FIPS": "53049", "County": "Pacific", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98614", "FIPS": "53049", "County": "Pacific", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98624", "FIPS": "53049", "County": "Pacific", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98631", "FIPS": "53049", "County": "Pacific", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98637", "FIPS": "53049", "County": "Pacific", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98638", "FIPS": "53049", "County": "Pacific", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98640", "FIPS": "53049", "County": "Pacific", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98641", "FIPS": "53049", "County": "Pacific", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98644", "FIPS": "53049", "County": "Pacific", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "99119", "FIPS": "53051", "County": "Pend Oreille", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99139", "FIPS": "53051", "County": "Pend Oreille", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99152", "FIPS": "53051", "County": "Pend Oreille", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99153", "FIPS": "53051", "County": "Pend Oreille", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99156", "FIPS": "53051", "County": "Pend Oreille", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99180", "FIPS": "53051", "County": "Pend Oreille", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98303", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98304", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98321", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98323", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98327", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98328", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98329", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98330", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98332", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98333", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98335", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98338", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98344", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98348", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98349", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98351", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98352", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98354", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98360", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98371", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98372", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98373", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98374", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98375", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98385", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98387", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98388", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98390", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98391", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98394", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98395", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98396", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98397", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98398", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98401", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98402", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98403", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98404", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98405", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98406", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98407", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98408", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98409", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98411", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98412", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98413", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98415", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98416", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98418", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98421", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98422", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98424", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98430", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98431", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98433", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98438", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98439", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98442", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98443", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98444", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98445", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98446", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98447", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98450", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98455", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98460", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98464", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98465", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98466", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98467", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98471", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98477", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98481", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98492", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98493", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98497", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98498", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98499", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98558", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98580", "FIPS": "53053", "County": "Pierce", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98222", "FIPS": "53055", "County": "San Juan", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98243", "FIPS": "53055", "County": "San Juan", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98245", "FIPS": "53055", "County": "San Juan", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98250", "FIPS": "53055", "County": "San Juan", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98261", "FIPS": "53055", "County": "San Juan", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98279", "FIPS": "53055", "County": "San Juan", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98280", "FIPS": "53055", "County": "San Juan", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98286", "FIPS": "53055", "County": "San Juan", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98297", "FIPS": "53055", "County": "San Juan", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98221", "FIPS": "53057", "County": "Skagit", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98232", "FIPS": "53057", "County": "Skagit", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98233", "FIPS": "53057", "County": "Skagit", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98235", "FIPS": "53057", "County": "Skagit", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98237", "FIPS": "53057", "County": "Skagit", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98238", "FIPS": "53057", "County": "Skagit", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98255", "FIPS": "53057", "County": "Skagit", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98257", "FIPS": "53057", "County": "Skagit", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98263", "FIPS": "53057", "County": "Skagit", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98267", "FIPS": "53057", "County": "Skagit", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98273", "FIPS": "53057", "County": "Skagit", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98274", "FIPS": "53057", "County": "Skagit", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98283", "FIPS": "53057", "County": "Skagit", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98284", "FIPS": "53057", "County": "Skagit", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98610", "FIPS": "53059", "County": "Skamania", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98639", "FIPS": "53059", "County": "Skamania", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98648", "FIPS": "53059", "County": "Skamania", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98651", "FIPS": "53059", "County": "Skamania", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98012", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98020", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98021", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98026", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98036", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98037", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98043", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98046", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98082", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98087", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98201", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98203", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98204", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98205", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98206", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98207", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98208", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98213", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98223", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98241", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98251", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98252", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98256", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98258", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98259", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98270", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98271", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98272", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98275", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98287", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98290", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98291", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98292", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98293", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98294", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98296", "FIPS": "53061", "County": "Snohomish", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "99001", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99003", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99004", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99005", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99006", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99009", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99011", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99012", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99014", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99016", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99018", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99019", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99020", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99021", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99022", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99023", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99025", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99026", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99027", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99030", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99031", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99036", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99037", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99039", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99201", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99202", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99203", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99204", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99205", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99206", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99207", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99208", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99209", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99210", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99211", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99212", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99213", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99214", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99215", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99216", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99217", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99218", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99219", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99220", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99223", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99224", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99228", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99251", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99252", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99256", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99258", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99260", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99299", "FIPS": "53063", "County": "Spokane", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99013", "FIPS": "53065", "County": "Stevens", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99034", "FIPS": "53065", "County": "Stevens", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99040", "FIPS": "53065", "County": "Stevens", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99101", "FIPS": "53065", "County": "Stevens", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99109", "FIPS": "53065", "County": "Stevens", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99110", "FIPS": "53065", "County": "Stevens", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99114", "FIPS": "53065", "County": "Stevens", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99126", "FIPS": "53065", "County": "Stevens", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99129", "FIPS": "53065", "County": "Stevens", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99131", "FIPS": "53065", "County": "Stevens", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99137", "FIPS": "53065", "County": "Stevens", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99141", "FIPS": "53065", "County": "Stevens", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99148", "FIPS": "53065", "County": "Stevens", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99151", "FIPS": "53065", "County": "Stevens", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99157", "FIPS": "53065", "County": "Stevens", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99167", "FIPS": "53065", "County": "Stevens", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99173", "FIPS": "53065", "County": "Stevens", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99181", "FIPS": "53065", "County": "Stevens", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98501", "FIPS": "53067", "County": "Thurston", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98502", "FIPS": "53067", "County": "Thurston", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98503", "FIPS": "53067", "County": "Thurston", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98504", "FIPS": "53067", "County": "Thurston", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98505", "FIPS": "53067", "County": "Thurston", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98506", "FIPS": "53067", "County": "Thurston", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98507", "FIPS": "53067", "County": "Thurston", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98508", "FIPS": "53067", "County": "Thurston", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98509", "FIPS": "53067", "County": "Thurston", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98511", "FIPS": "53067", "County": "Thurston", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98512", "FIPS": "53067", "County": "Thurston", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98513", "FIPS": "53067", "County": "Thurston", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98516", "FIPS": "53067", "County": "Thurston", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98530", "FIPS": "53067", "County": "Thurston", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98540", "FIPS": "53067", "County": "Thurston", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98556", "FIPS": "53067", "County": "Thurston", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98576", "FIPS": "53067", "County": "Thurston", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98579", "FIPS": "53067", "County": "Thurston", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98589", "FIPS": "53067", "County": "Thurston", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98597", "FIPS": "53067", "County": "Thurston", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98599", "FIPS": "53067", "County": "Thurston", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98612", "FIPS": "53069", "County": "Wahkiakum", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98621", "FIPS": "53069", "County": "Wahkiakum", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98643", "FIPS": "53069", "County": "Wahkiakum", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "98647", "FIPS": "53069", "County": "Wahkiakum", "State": "WA", "DMA": {"Code": 820, "Name": "PORTLAND, OR"}},
  {"Postal_Code": "99323", "FIPS": "53071", "County": "Walla Walla", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "99324", "FIPS": "53071", "County": "Walla Walla", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "99329", "FIPS": "53071", "County": "Walla Walla", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "99348", "FIPS": "53071", "County": "Walla Walla", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "99360", "FIPS": "53071", "County": "Walla Walla", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "99361", "FIPS": "53071", "County": "Walla Walla", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "99362", "FIPS": "53071", "County": "Walla Walla", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "99363", "FIPS": "53071", "County": "Walla Walla", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98220", "FIPS": "53073", "County": "Whatcom", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98225", "FIPS": "53073", "County": "Whatcom", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98226", "FIPS": "53073", "County": "Whatcom", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98227", "FIPS": "53073", "County": "Whatcom", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98228", "FIPS": "53073", "County": "Whatcom", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98229", "FIPS": "53073", "County": "Whatcom", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98230", "FIPS": "53073", "County": "Whatcom", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98231", "FIPS": "53073", "County": "Whatcom", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98240", "FIPS": "53073", "County": "Whatcom", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98244", "FIPS": "53073", "County": "Whatcom", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98247", "FIPS": "53073", "County": "Whatcom", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98248", "FIPS": "53073", "County": "Whatcom", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98262", "FIPS": "53073", "County": "Whatcom", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98264", "FIPS": "53073", "County": "Whatcom", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98266", "FIPS": "53073", "County": "Whatcom", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98276", "FIPS": "53073", "County": "Whatcom", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98281", "FIPS": "53073", "County": "Whatcom", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "98295", "FIPS": "53073", "County": "Whatcom", "State": "WA", "DMA": {"Code": 819, "Name": "SEATTLE-TACOMA"}},
  {"Postal_Code": "99017", "FIPS": "53075", "County": "Whitman", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99033", "FIPS": "53075", "County": "Whitman", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99102", "FIPS": "53075", "County": "Whitman", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99104", "FIPS": "53075", "County": "Whitman", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99111", "FIPS": "53075", "County": "Whitman", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99113", "FIPS": "53075", "County": "Whitman", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99125", "FIPS": "53075", "County": "Whitman", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99128", "FIPS": "53075", "County": "Whitman", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99130", "FIPS": "53075", "County": "Whitman", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99136", "FIPS": "53075", "County": "Whitman", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99143", "FIPS": "53075", "County": "Whitman", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99149", "FIPS": "53075", "County": "Whitman", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99158", "FIPS": "53075", "County": "Whitman", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99161", "FIPS": "53075", "County": "Whitman", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99163", "FIPS": "53075", "County": "Whitman", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99164", "FIPS": "53075", "County": "Whitman", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99165", "FIPS": "53075", "County": "Whitman", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99170", "FIPS": "53075", "County": "Whitman", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99171", "FIPS": "53075", "County": "Whitman", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99174", "FIPS": "53075", "County": "Whitman", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99176", "FIPS": "53075", "County": "Whitman", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99179", "FIPS": "53075", "County": "Whitman", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "99333", "FIPS": "53075", "County": "Whitman", "State": "WA", "DMA": {"Code": 881, "Name": "SPOKANE"}},
  {"Postal_Code": "98901", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98902", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98903", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98904", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98907", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98908", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98909", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98920", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98921", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98923", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98929", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98930", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98932", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98933", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98935", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98936", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98937", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98938", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98939", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98942", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98944", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98947", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98948", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98951", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98952", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "98953", "FIPS": "53077", "County": "Yakima", "State": "WA", "DMA": {"Code": 810, "Name": "YAKIMA-PASCO-RCHLND-KNNWCK"}},
  {"Postal_Code": "26238", "FIPS": "54001", "County": "Barbour", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26250", "FIPS": "54001", "County": "Barbour", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26275", "FIPS": "54001", "County": "Barbour", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26349", "FIPS": "54001", "County": "Barbour", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26405", "FIPS": "54001", "County": "Barbour", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26416", "FIPS": "54001", "County": "Barbour", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "25401", "FIPS": "54003", "County": "Berkeley", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25402", "FIPS": "54003", "County": "Berkeley", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25403", "FIPS": "54003", "County": "Berkeley", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25404", "FIPS": "54003", "County": "Berkeley", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25405", "FIPS": "54003", "County": "Berkeley", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25413", "FIPS": "54003", "County": "Berkeley", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25419", "FIPS": "54003", "County": "Berkeley", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25420", "FIPS": "54003", "County": "Berkeley", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25421", "FIPS": "54003", "County": "Berkeley", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25427", "FIPS": "54003", "County": "Berkeley", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25428", "FIPS": "54003", "County": "Berkeley", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25429", "FIPS": "54003", "County": "Berkeley", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25440", "FIPS": "54003", "County": "Berkeley", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25009", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25021", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25024", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25028", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25049", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25051", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25053", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25081", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25093", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25108", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25114", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25130", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25142", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25148", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25149", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25154", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25165", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25169", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25181", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25193", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25203", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25204", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25205", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25206", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25208", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25209", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25529", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25572", "FIPS": "54005", "County": "Boone", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26335", "FIPS": "54007", "County": "Braxton", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26601", "FIPS": "54007", "County": "Braxton", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26615", "FIPS": "54007", "County": "Braxton", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26619", "FIPS": "54007", "County": "Braxton", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26621", "FIPS": "54007", "County": "Braxton", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26623", "FIPS": "54007", "County": "Braxton", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26624", "FIPS": "54007", "County": "Braxton", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26627", "FIPS": "54007", "County": "Braxton", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26629", "FIPS": "54007", "County": "Braxton", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26631", "FIPS": "54007", "County": "Braxton", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26030", "FIPS": "54009", "County": "Brooke", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26032", "FIPS": "54009", "County": "Brooke", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26035", "FIPS": "54009", "County": "Brooke", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26037", "FIPS": "54009", "County": "Brooke", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26058", "FIPS": "54009", "County": "Brooke", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26070", "FIPS": "54009", "County": "Brooke", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26075", "FIPS": "54009", "County": "Brooke", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "25504", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25510", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25537", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25541", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25545", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25559", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25701", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25702", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25703", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25705", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25706", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25707", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25708", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25709", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25710", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25711", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25712", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25713", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25714", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25715", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25716", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25717", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25718", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25719", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25720", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25721", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25722", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25723", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25724", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25725", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25726", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25727", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25728", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25729", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25755", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25770", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25771", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25772", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25773", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25774", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25775", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25776", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25777", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25778", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25779", "FIPS": "54011", "County": "Cabell", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25234", "FIPS": "54013", "County": "Calhoun", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25235", "FIPS": "54013", "County": "Calhoun", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25261", "FIPS": "54013", "County": "Calhoun", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25268", "FIPS": "54013", "County": "Calhoun", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26136", "FIPS": "54013", "County": "Calhoun", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26137", "FIPS": "54013", "County": "Calhoun", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26147", "FIPS": "54013", "County": "Calhoun", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26151", "FIPS": "54013", "County": "Calhoun", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26152", "FIPS": "54013", "County": "Calhoun", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25019", "FIPS": "54015", "County": "Clay", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25030", "FIPS": "54015", "County": "Clay", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25043", "FIPS": "54015", "County": "Clay", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25063", "FIPS": "54015", "County": "Clay", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25088", "FIPS": "54015", "County": "Clay", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25111", "FIPS": "54015", "County": "Clay", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25113", "FIPS": "54015", "County": "Clay", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25125", "FIPS": "54015", "County": "Clay", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25133", "FIPS": "54015", "County": "Clay", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25141", "FIPS": "54015", "County": "Clay", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25164", "FIPS": "54015", "County": "Clay", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25211", "FIPS": "54015", "County": "Clay", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25285", "FIPS": "54015", "County": "Clay", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26617", "FIPS": "54015", "County": "Clay", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26339", "FIPS": "54017", "County": "Doddridge", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26411", "FIPS": "54017", "County": "Doddridge", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26436", "FIPS": "54017", "County": "Doddridge", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26456", "FIPS": "54017", "County": "Doddridge", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "25002", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25031", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25036", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25040", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25057", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25085", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25090", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25115", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25118", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25119", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25136", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25139", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25152", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25161", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25173", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25185", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25186", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25812", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25831", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25833", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25837", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25840", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25846", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25854", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25855", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25862", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25864", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25866", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25868", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25879", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25880", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25901", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25904", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25907", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25914", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25917", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25936", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25938", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25942", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25976", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25986", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "26680", "FIPS": "54019", "County": "Fayette", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25267", "FIPS": "54021", "County": "Gilmer", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26342", "FIPS": "54021", "County": "Gilmer", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26351", "FIPS": "54021", "County": "Gilmer", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26384", "FIPS": "54021", "County": "Gilmer", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26430", "FIPS": "54021", "County": "Gilmer", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26443", "FIPS": "54021", "County": "Gilmer", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26611", "FIPS": "54021", "County": "Gilmer", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26636", "FIPS": "54021", "County": "Gilmer", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26638", "FIPS": "54021", "County": "Gilmer", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26707", "FIPS": "54023", "County": "Grant", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26720", "FIPS": "54023", "County": "Grant", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26731", "FIPS": "54023", "County": "Grant", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26739", "FIPS": "54023", "County": "Grant", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26833", "FIPS": "54023", "County": "Grant", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26847", "FIPS": "54023", "County": "Grant", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26855", "FIPS": "54023", "County": "Grant", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "24901", "FIPS": "54025", "County": "Greenbrier", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24902", "FIPS": "54025", "County": "Greenbrier", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24910", "FIPS": "54025", "County": "Greenbrier", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24916", "FIPS": "54025", "County": "Greenbrier", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24925", "FIPS": "54025", "County": "Greenbrier", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24931", "FIPS": "54025", "County": "Greenbrier", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24938", "FIPS": "54025", "County": "Greenbrier", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24943", "FIPS": "54025", "County": "Greenbrier", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24957", "FIPS": "54025", "County": "Greenbrier", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24961", "FIPS": "54025", "County": "Greenbrier", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24966", "FIPS": "54025", "County": "Greenbrier", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24970", "FIPS": "54025", "County": "Greenbrier", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24977", "FIPS": "54025", "County": "Greenbrier", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24986", "FIPS": "54025", "County": "Greenbrier", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24991", "FIPS": "54025", "County": "Greenbrier", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25958", "FIPS": "54025", "County": "Greenbrier", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25962", "FIPS": "54025", "County": "Greenbrier", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25972", "FIPS": "54025", "County": "Greenbrier", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25981", "FIPS": "54025", "County": "Greenbrier", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25984", "FIPS": "54025", "County": "Greenbrier", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25431", "FIPS": "54027", "County": "Hampshire", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25437", "FIPS": "54027", "County": "Hampshire", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25444", "FIPS": "54027", "County": "Hampshire", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26704", "FIPS": "54027", "County": "Hampshire", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26711", "FIPS": "54027", "County": "Hampshire", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26714", "FIPS": "54027", "County": "Hampshire", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26722", "FIPS": "54027", "County": "Hampshire", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26755", "FIPS": "54027", "County": "Hampshire", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26757", "FIPS": "54027", "County": "Hampshire", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26761", "FIPS": "54027", "County": "Hampshire", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26763", "FIPS": "54027", "County": "Hampshire", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26808", "FIPS": "54027", "County": "Hampshire", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26817", "FIPS": "54027", "County": "Hampshire", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26823", "FIPS": "54027", "County": "Hampshire", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26852", "FIPS": "54027", "County": "Hampshire", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26865", "FIPS": "54027", "County": "Hampshire", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26034", "FIPS": "54029", "County": "Hancock", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26047", "FIPS": "54029", "County": "Hancock", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26050", "FIPS": "54029", "County": "Hancock", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26056", "FIPS": "54029", "County": "Hancock", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26062", "FIPS": "54029", "County": "Hancock", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26801", "FIPS": "54031", "County": "Hardy", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26810", "FIPS": "54031", "County": "Hardy", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26812", "FIPS": "54031", "County": "Hardy", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26818", "FIPS": "54031", "County": "Hardy", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26836", "FIPS": "54031", "County": "Hardy", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26838", "FIPS": "54031", "County": "Hardy", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26845", "FIPS": "54031", "County": "Hardy", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26851", "FIPS": "54031", "County": "Hardy", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26301", "FIPS": "54033", "County": "Harrison", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26302", "FIPS": "54033", "County": "Harrison", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26306", "FIPS": "54033", "County": "Harrison", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26323", "FIPS": "54033", "County": "Harrison", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26330", "FIPS": "54033", "County": "Harrison", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26361", "FIPS": "54033", "County": "Harrison", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26366", "FIPS": "54033", "County": "Harrison", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26369", "FIPS": "54033", "County": "Harrison", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26385", "FIPS": "54033", "County": "Harrison", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26386", "FIPS": "54033", "County": "Harrison", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26404", "FIPS": "54033", "County": "Harrison", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26408", "FIPS": "54033", "County": "Harrison", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26422", "FIPS": "54033", "County": "Harrison", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26426", "FIPS": "54033", "County": "Harrison", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26431", "FIPS": "54033", "County": "Harrison", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26438", "FIPS": "54033", "County": "Harrison", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26448", "FIPS": "54033", "County": "Harrison", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26451", "FIPS": "54033", "County": "Harrison", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26461", "FIPS": "54033", "County": "Harrison", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26463", "FIPS": "54033", "County": "Harrison", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26568", "FIPS": "54033", "County": "Harrison", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "25231", "FIPS": "54035", "County": "Jackson", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25239", "FIPS": "54035", "County": "Jackson", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25241", "FIPS": "54035", "County": "Jackson", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25244", "FIPS": "54035", "County": "Jackson", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25245", "FIPS": "54035", "County": "Jackson", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25248", "FIPS": "54035", "County": "Jackson", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25252", "FIPS": "54035", "County": "Jackson", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25262", "FIPS": "54035", "County": "Jackson", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25271", "FIPS": "54035", "County": "Jackson", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25275", "FIPS": "54035", "County": "Jackson", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26164", "FIPS": "54035", "County": "Jackson", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25410", "FIPS": "54037", "County": "Jefferson", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25414", "FIPS": "54037", "County": "Jefferson", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25423", "FIPS": "54037", "County": "Jefferson", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25425", "FIPS": "54037", "County": "Jefferson", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25430", "FIPS": "54037", "County": "Jefferson", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25432", "FIPS": "54037", "County": "Jefferson", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25438", "FIPS": "54037", "County": "Jefferson", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25441", "FIPS": "54037", "County": "Jefferson", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25442", "FIPS": "54037", "County": "Jefferson", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25443", "FIPS": "54037", "County": "Jefferson", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25446", "FIPS": "54037", "County": "Jefferson", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25003", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25015", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25025", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25026", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25035", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25039", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25045", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25054", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25061", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25064", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25067", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25071", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25075", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25079", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25083", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25086", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25102", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25103", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25107", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25110", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25112", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25126", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25132", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25134", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25143", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25156", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25160", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25162", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25177", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25201", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25202", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25214", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25301", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25302", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25303", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25304", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25305", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25306", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25309", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25311", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25312", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25313", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25314", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25315", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25317", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25320", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25321", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25322", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25323", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25324", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25325", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25326", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25327", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25328", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25329", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25330", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25331", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25332", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25333", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25334", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25335", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25336", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25337", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25338", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25339", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25350", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25356", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25357", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25358", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25360", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25361", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25362", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25364", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25365", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25375", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25387", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25389", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25392", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25396", "FIPS": "54039", "County": "Kanawha", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26321", "FIPS": "54041", "County": "Lewis", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26338", "FIPS": "54041", "County": "Lewis", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26343", "FIPS": "54041", "County": "Lewis", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26372", "FIPS": "54041", "County": "Lewis", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26376", "FIPS": "54041", "County": "Lewis", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26378", "FIPS": "54041", "County": "Lewis", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26412", "FIPS": "54041", "County": "Lewis", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26447", "FIPS": "54041", "County": "Lewis", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26452", "FIPS": "54041", "County": "Lewis", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "25501", "FIPS": "54043", "County": "Lincoln", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25506", "FIPS": "54043", "County": "Lincoln", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25521", "FIPS": "54043", "County": "Lincoln", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25523", "FIPS": "54043", "County": "Lincoln", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25524", "FIPS": "54043", "County": "Lincoln", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25540", "FIPS": "54043", "County": "Lincoln", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25544", "FIPS": "54043", "County": "Lincoln", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25557", "FIPS": "54043", "County": "Lincoln", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25564", "FIPS": "54043", "County": "Lincoln", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25565", "FIPS": "54043", "County": "Lincoln", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25567", "FIPS": "54043", "County": "Lincoln", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25571", "FIPS": "54043", "County": "Lincoln", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25573", "FIPS": "54043", "County": "Lincoln", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25022", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25047", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25076", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25121", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25183", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25505", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25508", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25547", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25601", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25606", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25607", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25611", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25612", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25614", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25617", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25624", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25625", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25628", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25630", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25632", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25634", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25635", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25637", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25638", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25639", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25644", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25646", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25647", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25649", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25652", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25653", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25654", "FIPS": "54045", "County": "Logan", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "24801", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24808", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24811", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24813", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24815", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24816", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24817", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24824", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24826", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24828", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24829", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24830", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24831", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24836", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24842", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24843", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24844", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24846", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24848", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24850", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24853", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24855", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24861", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24862", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24866", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24868", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24871", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24872", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24873", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24878", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24879", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24881", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24884", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24887", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24888", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24892", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24894", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24895", "FIPS": "54047", "County": "McDowell", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "26554", "FIPS": "54049", "County": "Marion", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26555", "FIPS": "54049", "County": "Marion", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26559", "FIPS": "54049", "County": "Marion", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26560", "FIPS": "54049", "County": "Marion", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26563", "FIPS": "54049", "County": "Marion", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26566", "FIPS": "54049", "County": "Marion", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26570", "FIPS": "54049", "County": "Marion", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26571", "FIPS": "54049", "County": "Marion", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26572", "FIPS": "54049", "County": "Marion", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26574", "FIPS": "54049", "County": "Marion", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26576", "FIPS": "54049", "County": "Marion", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26578", "FIPS": "54049", "County": "Marion", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26582", "FIPS": "54049", "County": "Marion", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26585", "FIPS": "54049", "County": "Marion", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26586", "FIPS": "54049", "County": "Marion", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26587", "FIPS": "54049", "County": "Marion", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26588", "FIPS": "54049", "County": "Marion", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26591", "FIPS": "54049", "County": "Marion", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26031", "FIPS": "54051", "County": "Marshall", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26033", "FIPS": "54051", "County": "Marshall", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26036", "FIPS": "54051", "County": "Marshall", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26038", "FIPS": "54051", "County": "Marshall", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26039", "FIPS": "54051", "County": "Marshall", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26040", "FIPS": "54051", "County": "Marshall", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26041", "FIPS": "54051", "County": "Marshall", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26055", "FIPS": "54051", "County": "Marshall", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "25106", "FIPS": "54053", "County": "Mason", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25123", "FIPS": "54053", "County": "Mason", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25187", "FIPS": "54053", "County": "Mason", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25247", "FIPS": "54053", "County": "Mason", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25253", "FIPS": "54053", "County": "Mason", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25260", "FIPS": "54053", "County": "Mason", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25264", "FIPS": "54053", "County": "Mason", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25265", "FIPS": "54053", "County": "Mason", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25287", "FIPS": "54053", "County": "Mason", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25502", "FIPS": "54053", "County": "Mason", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25503", "FIPS": "54053", "County": "Mason", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25515", "FIPS": "54053", "County": "Mason", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25520", "FIPS": "54053", "County": "Mason", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25550", "FIPS": "54053", "County": "Mason", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "24701", "FIPS": "54055", "County": "Mercer", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24712", "FIPS": "54055", "County": "Mercer", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24714", "FIPS": "54055", "County": "Mercer", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24715", "FIPS": "54055", "County": "Mercer", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24724", "FIPS": "54055", "County": "Mercer", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24729", "FIPS": "54055", "County": "Mercer", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24731", "FIPS": "54055", "County": "Mercer", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24732", "FIPS": "54055", "County": "Mercer", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24733", "FIPS": "54055", "County": "Mercer", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24736", "FIPS": "54055", "County": "Mercer", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24737", "FIPS": "54055", "County": "Mercer", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24738", "FIPS": "54055", "County": "Mercer", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24739", "FIPS": "54055", "County": "Mercer", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24740", "FIPS": "54055", "County": "Mercer", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24747", "FIPS": "54055", "County": "Mercer", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24751", "FIPS": "54055", "County": "Mercer", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25820", "FIPS": "54055", "County": "Mercer", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25841", "FIPS": "54055", "County": "Mercer", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25922", "FIPS": "54055", "County": "Mercer", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25971", "FIPS": "54055", "County": "Mercer", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "26710", "FIPS": "54057", "County": "Mineral", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26717", "FIPS": "54057", "County": "Mineral", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26719", "FIPS": "54057", "County": "Mineral", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26726", "FIPS": "54057", "County": "Mineral", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26743", "FIPS": "54057", "County": "Mineral", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26750", "FIPS": "54057", "County": "Mineral", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26753", "FIPS": "54057", "County": "Mineral", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "26767", "FIPS": "54057", "County": "Mineral", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "24851", "FIPS": "54059", "County": "Mingo", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25608", "FIPS": "54059", "County": "Mingo", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25621", "FIPS": "54059", "County": "Mingo", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25650", "FIPS": "54059", "County": "Mingo", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25651", "FIPS": "54059", "County": "Mingo", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25661", "FIPS": "54059", "County": "Mingo", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25665", "FIPS": "54059", "County": "Mingo", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25666", "FIPS": "54059", "County": "Mingo", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25667", "FIPS": "54059", "County": "Mingo", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25670", "FIPS": "54059", "County": "Mingo", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25671", "FIPS": "54059", "County": "Mingo", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25672", "FIPS": "54059", "County": "Mingo", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25674", "FIPS": "54059", "County": "Mingo", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25676", "FIPS": "54059", "County": "Mingo", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25678", "FIPS": "54059", "County": "Mingo", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25685", "FIPS": "54059", "County": "Mingo", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25686", "FIPS": "54059", "County": "Mingo", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25688", "FIPS": "54059", "County": "Mingo", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25690", "FIPS": "54059", "County": "Mingo", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25691", "FIPS": "54059", "County": "Mingo", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25692", "FIPS": "54059", "County": "Mingo", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25696", "FIPS": "54059", "County": "Mingo", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25697", "FIPS": "54059", "County": "Mingo", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26501", "FIPS": "54061", "County": "Monongalia", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26502", "FIPS": "54061", "County": "Monongalia", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26504", "FIPS": "54061", "County": "Monongalia", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26505", "FIPS": "54061", "County": "Monongalia", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26506", "FIPS": "54061", "County": "Monongalia", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26507", "FIPS": "54061", "County": "Monongalia", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26508", "FIPS": "54061", "County": "Monongalia", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26521", "FIPS": "54061", "County": "Monongalia", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26527", "FIPS": "54061", "County": "Monongalia", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26531", "FIPS": "54061", "County": "Monongalia", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26534", "FIPS": "54061", "County": "Monongalia", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26541", "FIPS": "54061", "County": "Monongalia", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26543", "FIPS": "54061", "County": "Monongalia", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26544", "FIPS": "54061", "County": "Monongalia", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26546", "FIPS": "54061", "County": "Monongalia", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26590", "FIPS": "54061", "County": "Monongalia", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "24918", "FIPS": "54063", "County": "Monroe", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24941", "FIPS": "54063", "County": "Monroe", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24945", "FIPS": "54063", "County": "Monroe", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24951", "FIPS": "54063", "County": "Monroe", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24963", "FIPS": "54063", "County": "Monroe", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24974", "FIPS": "54063", "County": "Monroe", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24976", "FIPS": "54063", "County": "Monroe", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24983", "FIPS": "54063", "County": "Monroe", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24984", "FIPS": "54063", "County": "Monroe", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24985", "FIPS": "54063", "County": "Monroe", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24993", "FIPS": "54063", "County": "Monroe", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25411", "FIPS": "54065", "County": "Morgan", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25422", "FIPS": "54065", "County": "Morgan", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25434", "FIPS": "54065", "County": "Morgan", "State": "WV", "DMA": {"Code": 511, "Name": "WASHINGTON, DC-HAGRSTWN"}},
  {"Postal_Code": "25059", "FIPS": "54067", "County": "Nicholas", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26202", "FIPS": "54067", "County": "Nicholas", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26205", "FIPS": "54067", "County": "Nicholas", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26261", "FIPS": "54067", "County": "Nicholas", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26610", "FIPS": "54067", "County": "Nicholas", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26651", "FIPS": "54067", "County": "Nicholas", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26656", "FIPS": "54067", "County": "Nicholas", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26660", "FIPS": "54067", "County": "Nicholas", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26662", "FIPS": "54067", "County": "Nicholas", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26667", "FIPS": "54067", "County": "Nicholas", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26671", "FIPS": "54067", "County": "Nicholas", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26675", "FIPS": "54067", "County": "Nicholas", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26676", "FIPS": "54067", "County": "Nicholas", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26678", "FIPS": "54067", "County": "Nicholas", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26679", "FIPS": "54067", "County": "Nicholas", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26681", "FIPS": "54067", "County": "Nicholas", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26684", "FIPS": "54067", "County": "Nicholas", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26690", "FIPS": "54067", "County": "Nicholas", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26691", "FIPS": "54067", "County": "Nicholas", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26003", "FIPS": "54069", "County": "Ohio", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26059", "FIPS": "54069", "County": "Ohio", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26060", "FIPS": "54069", "County": "Ohio", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26074", "FIPS": "54069", "County": "Ohio", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26802", "FIPS": "54071", "County": "Pendleton", "State": "WV", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "26804", "FIPS": "54071", "County": "Pendleton", "State": "WV", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "26807", "FIPS": "54071", "County": "Pendleton", "State": "WV", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "26814", "FIPS": "54071", "County": "Pendleton", "State": "WV", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "26815", "FIPS": "54071", "County": "Pendleton", "State": "WV", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "26866", "FIPS": "54071", "County": "Pendleton", "State": "WV", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "26884", "FIPS": "54071", "County": "Pendleton", "State": "WV", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "26886", "FIPS": "54071", "County": "Pendleton", "State": "WV", "DMA": {"Code": 569, "Name": "HARRISONBURG"}},
  {"Postal_Code": "26134", "FIPS": "54073", "County": "Pleasants", "State": "WV", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "26170", "FIPS": "54073", "County": "Pleasants", "State": "WV", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "24915", "FIPS": "54075", "County": "Pocahontas", "State": "WV", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24920", "FIPS": "54075", "County": "Pocahontas", "State": "WV", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24924", "FIPS": "54075", "County": "Pocahontas", "State": "WV", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24927", "FIPS": "54075", "County": "Pocahontas", "State": "WV", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24934", "FIPS": "54075", "County": "Pocahontas", "State": "WV", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24944", "FIPS": "54075", "County": "Pocahontas", "State": "WV", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24946", "FIPS": "54075", "County": "Pocahontas", "State": "WV", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "24954", "FIPS": "54075", "County": "Pocahontas", "State": "WV", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "26209", "FIPS": "54075", "County": "Pocahontas", "State": "WV", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "26264", "FIPS": "54075", "County": "Pocahontas", "State": "WV", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "26291", "FIPS": "54075", "County": "Pocahontas", "State": "WV", "DMA": {"Code": 573, "Name": "ROANOKE-LYNCHBURG"}},
  {"Postal_Code": "26374", "FIPS": "54077", "County": "Preston", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26410", "FIPS": "54077", "County": "Preston", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26425", "FIPS": "54077", "County": "Preston", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26444", "FIPS": "54077", "County": "Preston", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26519", "FIPS": "54077", "County": "Preston", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26520", "FIPS": "54077", "County": "Preston", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26524", "FIPS": "54077", "County": "Preston", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26525", "FIPS": "54077", "County": "Preston", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26537", "FIPS": "54077", "County": "Preston", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26542", "FIPS": "54077", "County": "Preston", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26547", "FIPS": "54077", "County": "Preston", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26705", "FIPS": "54077", "County": "Preston", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26716", "FIPS": "54077", "County": "Preston", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "26764", "FIPS": "54077", "County": "Preston", "State": "WV", "DMA": {"Code": 508, "Name": "PITTSBURGH"}},
  {"Postal_Code": "25011", "FIPS": "54079", "County": "Putnam", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25033", "FIPS": "54079", "County": "Putnam", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25070", "FIPS": "54079", "County": "Putnam", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25082", "FIPS": "54079", "County": "Putnam", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25109", "FIPS": "54079", "County": "Putnam", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25124", "FIPS": "54079", "County": "Putnam", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25159", "FIPS": "54079", "County": "Putnam", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25168", "FIPS": "54079", "County": "Putnam", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25213", "FIPS": "54079", "County": "Putnam", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25526", "FIPS": "54079", "County": "Putnam", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25560", "FIPS": "54079", "County": "Putnam", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25569", "FIPS": "54079", "County": "Putnam", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25007", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25008", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25044", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25048", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25060", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25062", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25140", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25174", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25180", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25801", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25802", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25813", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25816", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25817", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25818", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25823", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25825", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25827", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25832", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25836", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25839", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25843", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25844", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25849", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25851", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25853", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25857", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25860", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25865", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25871", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25873", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25878", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25902", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25906", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25908", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25909", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25911", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25915", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25918", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25919", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25920", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25921", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25926", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25927", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25932", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25989", "FIPS": "54081", "County": "Raleigh", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "26224", "FIPS": "54083", "County": "Randolph", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26230", "FIPS": "54083", "County": "Randolph", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26241", "FIPS": "54083", "County": "Randolph", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26253", "FIPS": "54083", "County": "Randolph", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26254", "FIPS": "54083", "County": "Randolph", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26257", "FIPS": "54083", "County": "Randolph", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26259", "FIPS": "54083", "County": "Randolph", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26263", "FIPS": "54083", "County": "Randolph", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26267", "FIPS": "54083", "County": "Randolph", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26268", "FIPS": "54083", "County": "Randolph", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26270", "FIPS": "54083", "County": "Randolph", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26273", "FIPS": "54083", "County": "Randolph", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26276", "FIPS": "54083", "County": "Randolph", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26278", "FIPS": "54083", "County": "Randolph", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26280", "FIPS": "54083", "County": "Randolph", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26282", "FIPS": "54083", "County": "Randolph", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26283", "FIPS": "54083", "County": "Randolph", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26285", "FIPS": "54083", "County": "Randolph", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26293", "FIPS": "54083", "County": "Randolph", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26294", "FIPS": "54083", "County": "Randolph", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26296", "FIPS": "54083", "County": "Randolph", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26148", "FIPS": "54085", "County": "Ritchie", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26161", "FIPS": "54085", "County": "Ritchie", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26178", "FIPS": "54085", "County": "Ritchie", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26325", "FIPS": "54085", "County": "Ritchie", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26327", "FIPS": "54085", "County": "Ritchie", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26337", "FIPS": "54085", "County": "Ritchie", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26346", "FIPS": "54085", "County": "Ritchie", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26362", "FIPS": "54085", "County": "Ritchie", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26415", "FIPS": "54085", "County": "Ritchie", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26421", "FIPS": "54085", "County": "Ritchie", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "25005", "FIPS": "54087", "County": "Roane", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25243", "FIPS": "54087", "County": "Roane", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25251", "FIPS": "54087", "County": "Roane", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25259", "FIPS": "54087", "County": "Roane", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25266", "FIPS": "54087", "County": "Roane", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25270", "FIPS": "54087", "County": "Roane", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25276", "FIPS": "54087", "County": "Roane", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25286", "FIPS": "54087", "County": "Roane", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "24935", "FIPS": "54089", "County": "Summers", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24962", "FIPS": "54089", "County": "Summers", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24981", "FIPS": "54089", "County": "Summers", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25951", "FIPS": "54089", "County": "Summers", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25965", "FIPS": "54089", "County": "Summers", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25966", "FIPS": "54089", "County": "Summers", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25969", "FIPS": "54089", "County": "Summers", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25977", "FIPS": "54089", "County": "Summers", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25978", "FIPS": "54089", "County": "Summers", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25979", "FIPS": "54089", "County": "Summers", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25985", "FIPS": "54089", "County": "Summers", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "26347", "FIPS": "54091", "County": "Taylor", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26354", "FIPS": "54091", "County": "Taylor", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26424", "FIPS": "54091", "County": "Taylor", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26435", "FIPS": "54091", "County": "Taylor", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26440", "FIPS": "54091", "County": "Taylor", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26260", "FIPS": "54093", "County": "Tucker", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26269", "FIPS": "54093", "County": "Tucker", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26271", "FIPS": "54093", "County": "Tucker", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26287", "FIPS": "54093", "County": "Tucker", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26289", "FIPS": "54093", "County": "Tucker", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26292", "FIPS": "54093", "County": "Tucker", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26146", "FIPS": "54095", "County": "Tyler", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26149", "FIPS": "54095", "County": "Tyler", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26175", "FIPS": "54095", "County": "Tyler", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26320", "FIPS": "54095", "County": "Tyler", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26434", "FIPS": "54095", "County": "Tyler", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26201", "FIPS": "54097", "County": "Upshur", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26210", "FIPS": "54097", "County": "Upshur", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26215", "FIPS": "54097", "County": "Upshur", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26218", "FIPS": "54097", "County": "Upshur", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26219", "FIPS": "54097", "County": "Upshur", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26228", "FIPS": "54097", "County": "Upshur", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26229", "FIPS": "54097", "County": "Upshur", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26234", "FIPS": "54097", "County": "Upshur", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26236", "FIPS": "54097", "County": "Upshur", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26237", "FIPS": "54097", "County": "Upshur", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "25507", "FIPS": "54099", "County": "Wayne", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25511", "FIPS": "54099", "County": "Wayne", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25512", "FIPS": "54099", "County": "Wayne", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25514", "FIPS": "54099", "County": "Wayne", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25517", "FIPS": "54099", "County": "Wayne", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25530", "FIPS": "54099", "County": "Wayne", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25534", "FIPS": "54099", "County": "Wayne", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25535", "FIPS": "54099", "County": "Wayne", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25555", "FIPS": "54099", "County": "Wayne", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25562", "FIPS": "54099", "County": "Wayne", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25570", "FIPS": "54099", "County": "Wayne", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25669", "FIPS": "54099", "County": "Wayne", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25699", "FIPS": "54099", "County": "Wayne", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "25704", "FIPS": "54099", "County": "Wayne", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26203", "FIPS": "54101", "County": "Webster", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26206", "FIPS": "54101", "County": "Webster", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26208", "FIPS": "54101", "County": "Webster", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26217", "FIPS": "54101", "County": "Webster", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26222", "FIPS": "54101", "County": "Webster", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26266", "FIPS": "54101", "County": "Webster", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26288", "FIPS": "54101", "County": "Webster", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26298", "FIPS": "54101", "County": "Webster", "State": "WV", "DMA": {"Code": 598, "Name": "CLARKSBURG-WESTON"}},
  {"Postal_Code": "26155", "FIPS": "54103", "County": "Wetzel", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26159", "FIPS": "54103", "County": "Wetzel", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26162", "FIPS": "54103", "County": "Wetzel", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26167", "FIPS": "54103", "County": "Wetzel", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26186", "FIPS": "54103", "County": "Wetzel", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26348", "FIPS": "54103", "County": "Wetzel", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26377", "FIPS": "54103", "County": "Wetzel", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26419", "FIPS": "54103", "County": "Wetzel", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26437", "FIPS": "54103", "County": "Wetzel", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26561", "FIPS": "54103", "County": "Wetzel", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26562", "FIPS": "54103", "County": "Wetzel", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26575", "FIPS": "54103", "County": "Wetzel", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26581", "FIPS": "54103", "County": "Wetzel", "State": "WV", "DMA": {"Code": 554, "Name": "WHEELING-STEUBENVILLE"}},
  {"Postal_Code": "26138", "FIPS": "54105", "County": "Wirt", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26141", "FIPS": "54105", "County": "Wirt", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26143", "FIPS": "54105", "County": "Wirt", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26160", "FIPS": "54105", "County": "Wirt", "State": "WV", "DMA": {"Code": 564, "Name": "CHARLESTON-HUNTINGTON"}},
  {"Postal_Code": "26101", "FIPS": "54107", "County": "Wood", "State": "WV", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "26102", "FIPS": "54107", "County": "Wood", "State": "WV", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "26103", "FIPS": "54107", "County": "Wood", "State": "WV", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "26104", "FIPS": "54107", "County": "Wood", "State": "WV", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "26105", "FIPS": "54107", "County": "Wood", "State": "WV", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "26106", "FIPS": "54107", "County": "Wood", "State": "WV", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "26120", "FIPS": "54107", "County": "Wood", "State": "WV", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "26121", "FIPS": "54107", "County": "Wood", "State": "WV", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "26133", "FIPS": "54107", "County": "Wood", "State": "WV", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "26142", "FIPS": "54107", "County": "Wood", "State": "WV", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "26150", "FIPS": "54107", "County": "Wood", "State": "WV", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "26169", "FIPS": "54107", "County": "Wood", "State": "WV", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "26180", "FIPS": "54107", "County": "Wood", "State": "WV", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "26181", "FIPS": "54107", "County": "Wood", "State": "WV", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "26184", "FIPS": "54107", "County": "Wood", "State": "WV", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "26187", "FIPS": "54107", "County": "Wood", "State": "WV", "DMA": {"Code": 597, "Name": "PARKERSBURG"}},
  {"Postal_Code": "24716", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24719", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24726", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24818", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24822", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24823", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24827", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24834", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24839", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24845", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24847", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24849", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24854", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24857", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24859", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24860", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24867", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24869", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24870", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24874", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24880", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24882", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "24898", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25810", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25811", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25826", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25845", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25848", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25870", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25875", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25876", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25882", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25913", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25916", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25928", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "25943", "FIPS": "54109", "County": "Wyoming", "State": "WV", "DMA": {"Code": 559, "Name": "BLUEFIELD-BECKLEY-OAK HILL"}},
  {"Postal_Code": "53910", "FIPS": "55001", "County": "Adams", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "53927", "FIPS": "55001", "County": "Adams", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "53934", "FIPS": "55001", "County": "Adams", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "53936", "FIPS": "55001", "County": "Adams", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54613", "FIPS": "55001", "County": "Adams", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54514", "FIPS": "55003", "County": "Ashland", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54517", "FIPS": "55003", "County": "Ashland", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54527", "FIPS": "55003", "County": "Ashland", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54546", "FIPS": "55003", "County": "Ashland", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54806", "FIPS": "55003", "County": "Ashland", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54846", "FIPS": "55003", "County": "Ashland", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54850", "FIPS": "55003", "County": "Ashland", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54855", "FIPS": "55003", "County": "Ashland", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54861", "FIPS": "55003", "County": "Ashland", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54728", "FIPS": "55005", "County": "Barron", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54733", "FIPS": "55005", "County": "Barron", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54762", "FIPS": "55005", "County": "Barron", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54805", "FIPS": "55005", "County": "Barron", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54812", "FIPS": "55005", "County": "Barron", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54813", "FIPS": "55005", "County": "Barron", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54818", "FIPS": "55005", "County": "Barron", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54822", "FIPS": "55005", "County": "Barron", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54826", "FIPS": "55005", "County": "Barron", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54829", "FIPS": "55005", "County": "Barron", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54841", "FIPS": "55005", "County": "Barron", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54857", "FIPS": "55005", "County": "Barron", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54868", "FIPS": "55005", "County": "Barron", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54889", "FIPS": "55005", "County": "Barron", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54814", "FIPS": "55007", "County": "Bayfield", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54816", "FIPS": "55007", "County": "Bayfield", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54821", "FIPS": "55007", "County": "Bayfield", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54827", "FIPS": "55007", "County": "Bayfield", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54832", "FIPS": "55007", "County": "Bayfield", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54839", "FIPS": "55007", "County": "Bayfield", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54844", "FIPS": "55007", "County": "Bayfield", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54847", "FIPS": "55007", "County": "Bayfield", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54856", "FIPS": "55007", "County": "Bayfield", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54865", "FIPS": "55007", "County": "Bayfield", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54891", "FIPS": "55007", "County": "Bayfield", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54115", "FIPS": "55009", "County": "Brown", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54126", "FIPS": "55009", "County": "Brown", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54155", "FIPS": "55009", "County": "Brown", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54162", "FIPS": "55009", "County": "Brown", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54173", "FIPS": "55009", "County": "Brown", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54180", "FIPS": "55009", "County": "Brown", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54208", "FIPS": "55009", "County": "Brown", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54229", "FIPS": "55009", "County": "Brown", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54301", "FIPS": "55009", "County": "Brown", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54302", "FIPS": "55009", "County": "Brown", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54303", "FIPS": "55009", "County": "Brown", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54304", "FIPS": "55009", "County": "Brown", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54305", "FIPS": "55009", "County": "Brown", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54306", "FIPS": "55009", "County": "Brown", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54307", "FIPS": "55009", "County": "Brown", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54308", "FIPS": "55009", "County": "Brown", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54311", "FIPS": "55009", "County": "Brown", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54313", "FIPS": "55009", "County": "Brown", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54324", "FIPS": "55009", "County": "Brown", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54344", "FIPS": "55009", "County": "Brown", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54610", "FIPS": "55011", "County": "Buffalo", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54622", "FIPS": "55011", "County": "Buffalo", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54629", "FIPS": "55011", "County": "Buffalo", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54743", "FIPS": "55011", "County": "Buffalo", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54755", "FIPS": "55011", "County": "Buffalo", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54756", "FIPS": "55011", "County": "Buffalo", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54830", "FIPS": "55013", "County": "Burnett", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54840", "FIPS": "55013", "County": "Burnett", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54845", "FIPS": "55013", "County": "Burnett", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54872", "FIPS": "55013", "County": "Burnett", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54893", "FIPS": "55013", "County": "Burnett", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "53014", "FIPS": "55015", "County": "Calumet", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "53061", "FIPS": "55015", "County": "Calumet", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "53062", "FIPS": "55015", "County": "Calumet", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "53088", "FIPS": "55015", "County": "Calumet", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54110", "FIPS": "55015", "County": "Calumet", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54123", "FIPS": "55015", "County": "Calumet", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54129", "FIPS": "55015", "County": "Calumet", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54160", "FIPS": "55015", "County": "Calumet", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54169", "FIPS": "55015", "County": "Calumet", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54724", "FIPS": "55017", "County": "Chippewa", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54726", "FIPS": "55017", "County": "Chippewa", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54727", "FIPS": "55017", "County": "Chippewa", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54729", "FIPS": "55017", "County": "Chippewa", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54732", "FIPS": "55017", "County": "Chippewa", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54745", "FIPS": "55017", "County": "Chippewa", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54748", "FIPS": "55017", "County": "Chippewa", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54757", "FIPS": "55017", "County": "Chippewa", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54768", "FIPS": "55017", "County": "Chippewa", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54774", "FIPS": "55017", "County": "Chippewa", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54405", "FIPS": "55019", "County": "Clark", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54420", "FIPS": "55019", "County": "Clark", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54421", "FIPS": "55019", "County": "Clark", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54422", "FIPS": "55019", "County": "Clark", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54425", "FIPS": "55019", "County": "Clark", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54436", "FIPS": "55019", "County": "Clark", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54437", "FIPS": "55019", "County": "Clark", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54446", "FIPS": "55019", "County": "Clark", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54456", "FIPS": "55019", "County": "Clark", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54460", "FIPS": "55019", "County": "Clark", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54493", "FIPS": "55019", "County": "Clark", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54498", "FIPS": "55019", "County": "Clark", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54746", "FIPS": "55019", "County": "Clark", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54771", "FIPS": "55019", "County": "Clark", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "53555", "FIPS": "55021", "County": "Columbia", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53901", "FIPS": "55021", "County": "Columbia", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53911", "FIPS": "55021", "County": "Columbia", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53923", "FIPS": "55021", "County": "Columbia", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53925", "FIPS": "55021", "County": "Columbia", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53928", "FIPS": "55021", "County": "Columbia", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53932", "FIPS": "55021", "County": "Columbia", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53935", "FIPS": "55021", "County": "Columbia", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53954", "FIPS": "55021", "County": "Columbia", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53955", "FIPS": "55021", "County": "Columbia", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53957", "FIPS": "55021", "County": "Columbia", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53960", "FIPS": "55021", "County": "Columbia", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53965", "FIPS": "55021", "County": "Columbia", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53969", "FIPS": "55021", "County": "Columbia", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53821", "FIPS": "55023", "County": "Crawford", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "53826", "FIPS": "55023", "County": "Crawford", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54626", "FIPS": "55023", "County": "Crawford", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54628", "FIPS": "55023", "County": "Crawford", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54631", "FIPS": "55023", "County": "Crawford", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54640", "FIPS": "55023", "County": "Crawford", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54645", "FIPS": "55023", "County": "Crawford", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54654", "FIPS": "55023", "County": "Crawford", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54655", "FIPS": "55023", "County": "Crawford", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54657", "FIPS": "55023", "County": "Crawford", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "53508", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53515", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53517", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53523", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53527", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53528", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53529", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53531", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53532", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53558", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53559", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53560", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53562", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53571", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53572", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53575", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53589", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53590", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53593", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53596", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53597", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53598", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53701", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53702", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53703", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53704", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53705", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53706", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53707", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53708", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53711", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53713", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53714", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53715", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53716", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53717", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53718", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53719", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53725", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53726", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53744", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53774", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53777", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53778", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53779", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53782", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53783", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53784", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53785", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53786", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53788", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53789", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53790", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53791", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53792", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53793", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53794", "FIPS": "55025", "County": "Dane", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53003", "FIPS": "55027", "County": "Dodge", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53006", "FIPS": "55027", "County": "Dodge", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53016", "FIPS": "55027", "County": "Dodge", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53032", "FIPS": "55027", "County": "Dodge", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53034", "FIPS": "55027", "County": "Dodge", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53035", "FIPS": "55027", "County": "Dodge", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53039", "FIPS": "55027", "County": "Dodge", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53047", "FIPS": "55027", "County": "Dodge", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53048", "FIPS": "55027", "County": "Dodge", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53050", "FIPS": "55027", "County": "Dodge", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53059", "FIPS": "55027", "County": "Dodge", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53078", "FIPS": "55027", "County": "Dodge", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53091", "FIPS": "55027", "County": "Dodge", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53098", "FIPS": "55027", "County": "Dodge", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53099", "FIPS": "55027", "County": "Dodge", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53557", "FIPS": "55027", "County": "Dodge", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53579", "FIPS": "55027", "County": "Dodge", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53916", "FIPS": "55027", "County": "Dodge", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53922", "FIPS": "55027", "County": "Dodge", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53933", "FIPS": "55027", "County": "Dodge", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53956", "FIPS": "55027", "County": "Dodge", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53963", "FIPS": "55027", "County": "Dodge", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "54202", "FIPS": "55029", "County": "Door", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54204", "FIPS": "55029", "County": "Door", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54209", "FIPS": "55029", "County": "Door", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54210", "FIPS": "55029", "County": "Door", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54211", "FIPS": "55029", "County": "Door", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54212", "FIPS": "55029", "County": "Door", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54213", "FIPS": "55029", "County": "Door", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54226", "FIPS": "55029", "County": "Door", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54234", "FIPS": "55029", "County": "Door", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54235", "FIPS": "55029", "County": "Door", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54246", "FIPS": "55029", "County": "Door", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54820", "FIPS": "55031", "County": "Douglas", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54836", "FIPS": "55031", "County": "Douglas", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54838", "FIPS": "55031", "County": "Douglas", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54842", "FIPS": "55031", "County": "Douglas", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54849", "FIPS": "55031", "County": "Douglas", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54854", "FIPS": "55031", "County": "Douglas", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54864", "FIPS": "55031", "County": "Douglas", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54873", "FIPS": "55031", "County": "Douglas", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54874", "FIPS": "55031", "County": "Douglas", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54880", "FIPS": "55031", "County": "Douglas", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54890", "FIPS": "55031", "County": "Douglas", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54725", "FIPS": "55033", "County": "Dunn", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54730", "FIPS": "55033", "County": "Dunn", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54734", "FIPS": "55033", "County": "Dunn", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54735", "FIPS": "55033", "County": "Dunn", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54737", "FIPS": "55033", "County": "Dunn", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54739", "FIPS": "55033", "County": "Dunn", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54749", "FIPS": "55033", "County": "Dunn", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54751", "FIPS": "55033", "County": "Dunn", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54763", "FIPS": "55033", "County": "Dunn", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54764", "FIPS": "55033", "County": "Dunn", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54765", "FIPS": "55033", "County": "Dunn", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54772", "FIPS": "55033", "County": "Dunn", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54701", "FIPS": "55035", "County": "Eau Claire", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54702", "FIPS": "55035", "County": "Eau Claire", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54703", "FIPS": "55035", "County": "Eau Claire", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54720", "FIPS": "55035", "County": "Eau Claire", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54722", "FIPS": "55035", "County": "Eau Claire", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54741", "FIPS": "55035", "County": "Eau Claire", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54742", "FIPS": "55035", "County": "Eau Claire", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54120", "FIPS": "55037", "County": "Florence", "State": "WI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "54121", "FIPS": "55037", "County": "Florence", "State": "WI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "54542", "FIPS": "55037", "County": "Florence", "State": "WI", "DMA": {"Code": 553, "Name": "MARQUETTE"}},
  {"Postal_Code": "53010", "FIPS": "55039", "County": "Fond du Lac", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "53019", "FIPS": "55039", "County": "Fond du Lac", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "53049", "FIPS": "55039", "County": "Fond du Lac", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "53057", "FIPS": "55039", "County": "Fond du Lac", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "53065", "FIPS": "55039", "County": "Fond du Lac", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "53079", "FIPS": "55039", "County": "Fond du Lac", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "53919", "FIPS": "55039", "County": "Fond du Lac", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "53931", "FIPS": "55039", "County": "Fond du Lac", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54932", "FIPS": "55039", "County": "Fond du Lac", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54935", "FIPS": "55039", "County": "Fond du Lac", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54936", "FIPS": "55039", "County": "Fond du Lac", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54937", "FIPS": "55039", "County": "Fond du Lac", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54971", "FIPS": "55039", "County": "Fond du Lac", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54974", "FIPS": "55039", "County": "Fond du Lac", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54979", "FIPS": "55039", "County": "Fond du Lac", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54103", "FIPS": "55041", "County": "Forest", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54511", "FIPS": "55041", "County": "Forest", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54520", "FIPS": "55041", "County": "Forest", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54541", "FIPS": "55041", "County": "Forest", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54566", "FIPS": "55041", "County": "Forest", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "53518", "FIPS": "55043", "County": "Grant", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53554", "FIPS": "55043", "County": "Grant", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53569", "FIPS": "55043", "County": "Grant", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53573", "FIPS": "55043", "County": "Grant", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53801", "FIPS": "55043", "County": "Grant", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53802", "FIPS": "55043", "County": "Grant", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53804", "FIPS": "55043", "County": "Grant", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53805", "FIPS": "55043", "County": "Grant", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53806", "FIPS": "55043", "County": "Grant", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53807", "FIPS": "55043", "County": "Grant", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53808", "FIPS": "55043", "County": "Grant", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53809", "FIPS": "55043", "County": "Grant", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53810", "FIPS": "55043", "County": "Grant", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53811", "FIPS": "55043", "County": "Grant", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53812", "FIPS": "55043", "County": "Grant", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53813", "FIPS": "55043", "County": "Grant", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53816", "FIPS": "55043", "County": "Grant", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53817", "FIPS": "55043", "County": "Grant", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53818", "FIPS": "55043", "County": "Grant", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53820", "FIPS": "55043", "County": "Grant", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53824", "FIPS": "55043", "County": "Grant", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53825", "FIPS": "55043", "County": "Grant", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53827", "FIPS": "55043", "County": "Grant", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53502", "FIPS": "55045", "County": "Green", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53520", "FIPS": "55045", "County": "Green", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53521", "FIPS": "55045", "County": "Green", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53522", "FIPS": "55045", "County": "Green", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53550", "FIPS": "55045", "County": "Green", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53566", "FIPS": "55045", "County": "Green", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53570", "FIPS": "55045", "County": "Green", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53574", "FIPS": "55045", "County": "Green", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53926", "FIPS": "55047", "County": "Green Lake", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "53939", "FIPS": "55047", "County": "Green Lake", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "53946", "FIPS": "55047", "County": "Green Lake", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "53947", "FIPS": "55047", "County": "Green Lake", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54923", "FIPS": "55047", "County": "Green Lake", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54941", "FIPS": "55047", "County": "Green Lake", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54968", "FIPS": "55047", "County": "Green Lake", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "53503", "FIPS": "55049", "County": "Iowa", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53506", "FIPS": "55049", "County": "Iowa", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53507", "FIPS": "55049", "County": "Iowa", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53526", "FIPS": "55049", "County": "Iowa", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53533", "FIPS": "55049", "County": "Iowa", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53535", "FIPS": "55049", "County": "Iowa", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53543", "FIPS": "55049", "County": "Iowa", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53544", "FIPS": "55049", "County": "Iowa", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53553", "FIPS": "55049", "County": "Iowa", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53565", "FIPS": "55049", "County": "Iowa", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53580", "FIPS": "55049", "County": "Iowa", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53582", "FIPS": "55049", "County": "Iowa", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53595", "FIPS": "55049", "County": "Iowa", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "54525", "FIPS": "55051", "County": "Iron", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54534", "FIPS": "55051", "County": "Iron", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54536", "FIPS": "55051", "County": "Iron", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54547", "FIPS": "55051", "County": "Iron", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54550", "FIPS": "55051", "County": "Iron", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54559", "FIPS": "55051", "County": "Iron", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54565", "FIPS": "55051", "County": "Iron", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54611", "FIPS": "55053", "County": "Jackson", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54615", "FIPS": "55053", "County": "Jackson", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54635", "FIPS": "55053", "County": "Jackson", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54642", "FIPS": "55053", "County": "Jackson", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54643", "FIPS": "55053", "County": "Jackson", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54659", "FIPS": "55053", "County": "Jackson", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54754", "FIPS": "55053", "County": "Jackson", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "53036", "FIPS": "55055", "County": "Jefferson", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53038", "FIPS": "55055", "County": "Jefferson", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53094", "FIPS": "55055", "County": "Jefferson", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53137", "FIPS": "55055", "County": "Jefferson", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53156", "FIPS": "55055", "County": "Jefferson", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53178", "FIPS": "55055", "County": "Jefferson", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53538", "FIPS": "55055", "County": "Jefferson", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53549", "FIPS": "55055", "County": "Jefferson", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53551", "FIPS": "55055", "County": "Jefferson", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53594", "FIPS": "55055", "County": "Jefferson", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53929", "FIPS": "55057", "County": "Juneau", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53944", "FIPS": "55057", "County": "Juneau", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53948", "FIPS": "55057", "County": "Juneau", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53950", "FIPS": "55057", "County": "Juneau", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53962", "FIPS": "55057", "County": "Juneau", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53968", "FIPS": "55057", "County": "Juneau", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "54618", "FIPS": "55057", "County": "Juneau", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "54637", "FIPS": "55057", "County": "Juneau", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "54641", "FIPS": "55057", "County": "Juneau", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "54646", "FIPS": "55057", "County": "Juneau", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53101", "FIPS": "55059", "County": "Kenosha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53102", "FIPS": "55059", "County": "Kenosha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53104", "FIPS": "55059", "County": "Kenosha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53109", "FIPS": "55059", "County": "Kenosha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53140", "FIPS": "55059", "County": "Kenosha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53141", "FIPS": "55059", "County": "Kenosha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53142", "FIPS": "55059", "County": "Kenosha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53143", "FIPS": "55059", "County": "Kenosha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53144", "FIPS": "55059", "County": "Kenosha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53152", "FIPS": "55059", "County": "Kenosha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53158", "FIPS": "55059", "County": "Kenosha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53159", "FIPS": "55059", "County": "Kenosha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53168", "FIPS": "55059", "County": "Kenosha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53170", "FIPS": "55059", "County": "Kenosha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53171", "FIPS": "55059", "County": "Kenosha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53179", "FIPS": "55059", "County": "Kenosha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53181", "FIPS": "55059", "County": "Kenosha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53192", "FIPS": "55059", "County": "Kenosha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53194", "FIPS": "55059", "County": "Kenosha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53199", "FIPS": "55059", "County": "Kenosha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "54201", "FIPS": "55061", "County": "Kewaunee", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54205", "FIPS": "55061", "County": "Kewaunee", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54216", "FIPS": "55061", "County": "Kewaunee", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54217", "FIPS": "55061", "County": "Kewaunee", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54601", "FIPS": "55063", "County": "La Crosse", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54602", "FIPS": "55063", "County": "La Crosse", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54603", "FIPS": "55063", "County": "La Crosse", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54614", "FIPS": "55063", "County": "La Crosse", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54636", "FIPS": "55063", "County": "La Crosse", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54644", "FIPS": "55063", "County": "La Crosse", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54650", "FIPS": "55063", "County": "La Crosse", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54653", "FIPS": "55063", "County": "La Crosse", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54669", "FIPS": "55063", "County": "La Crosse", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "53504", "FIPS": "55065", "County": "Lafayette", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53510", "FIPS": "55065", "County": "Lafayette", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53516", "FIPS": "55065", "County": "Lafayette", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53530", "FIPS": "55065", "County": "Lafayette", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53541", "FIPS": "55065", "County": "Lafayette", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53586", "FIPS": "55065", "County": "Lafayette", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53587", "FIPS": "55065", "County": "Lafayette", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53599", "FIPS": "55065", "County": "Lafayette", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53803", "FIPS": "55065", "County": "Lafayette", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "54409", "FIPS": "55067", "County": "Langlade", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54418", "FIPS": "55067", "County": "Langlade", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54424", "FIPS": "55067", "County": "Langlade", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54428", "FIPS": "55067", "County": "Langlade", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54430", "FIPS": "55067", "County": "Langlade", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54462", "FIPS": "55067", "County": "Langlade", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54464", "FIPS": "55067", "County": "Langlade", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54465", "FIPS": "55067", "County": "Langlade", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54485", "FIPS": "55067", "County": "Langlade", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54491", "FIPS": "55067", "County": "Langlade", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54435", "FIPS": "55069", "County": "Lincoln", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54442", "FIPS": "55069", "County": "Lincoln", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54452", "FIPS": "55069", "County": "Lincoln", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54487", "FIPS": "55069", "County": "Lincoln", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54532", "FIPS": "55069", "County": "Lincoln", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "53015", "FIPS": "55071", "County": "Manitowoc", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "53042", "FIPS": "55071", "County": "Manitowoc", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "53063", "FIPS": "55071", "County": "Manitowoc", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54207", "FIPS": "55071", "County": "Manitowoc", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54214", "FIPS": "55071", "County": "Manitowoc", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54215", "FIPS": "55071", "County": "Manitowoc", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54220", "FIPS": "55071", "County": "Manitowoc", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54221", "FIPS": "55071", "County": "Manitowoc", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54227", "FIPS": "55071", "County": "Manitowoc", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54228", "FIPS": "55071", "County": "Manitowoc", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54230", "FIPS": "55071", "County": "Manitowoc", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54232", "FIPS": "55071", "County": "Manitowoc", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54240", "FIPS": "55071", "County": "Manitowoc", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54241", "FIPS": "55071", "County": "Manitowoc", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54245", "FIPS": "55071", "County": "Manitowoc", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54247", "FIPS": "55071", "County": "Manitowoc", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54401", "FIPS": "55073", "County": "Marathon", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54402", "FIPS": "55073", "County": "Marathon", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54403", "FIPS": "55073", "County": "Marathon", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54408", "FIPS": "55073", "County": "Marathon", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54411", "FIPS": "55073", "County": "Marathon", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54417", "FIPS": "55073", "County": "Marathon", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54426", "FIPS": "55073", "County": "Marathon", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54427", "FIPS": "55073", "County": "Marathon", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54429", "FIPS": "55073", "County": "Marathon", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54432", "FIPS": "55073", "County": "Marathon", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54440", "FIPS": "55073", "County": "Marathon", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54448", "FIPS": "55073", "County": "Marathon", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54455", "FIPS": "55073", "County": "Marathon", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54471", "FIPS": "55073", "County": "Marathon", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54474", "FIPS": "55073", "County": "Marathon", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54476", "FIPS": "55073", "County": "Marathon", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54479", "FIPS": "55073", "County": "Marathon", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54484", "FIPS": "55073", "County": "Marathon", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54488", "FIPS": "55073", "County": "Marathon", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54102", "FIPS": "55075", "County": "Marinette", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54104", "FIPS": "55075", "County": "Marinette", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54112", "FIPS": "55075", "County": "Marinette", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54114", "FIPS": "55075", "County": "Marinette", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54119", "FIPS": "55075", "County": "Marinette", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54125", "FIPS": "55075", "County": "Marinette", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54143", "FIPS": "55075", "County": "Marinette", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54151", "FIPS": "55075", "County": "Marinette", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54156", "FIPS": "55075", "County": "Marinette", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54157", "FIPS": "55075", "County": "Marinette", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54159", "FIPS": "55075", "County": "Marinette", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54161", "FIPS": "55075", "County": "Marinette", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54177", "FIPS": "55075", "County": "Marinette", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "53920", "FIPS": "55077", "County": "Marquette", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53930", "FIPS": "55077", "County": "Marquette", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53949", "FIPS": "55077", "County": "Marquette", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53952", "FIPS": "55077", "County": "Marquette", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53953", "FIPS": "55077", "County": "Marquette", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53964", "FIPS": "55077", "County": "Marquette", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "54960", "FIPS": "55077", "County": "Marquette", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "54135", "FIPS": "55078", "County": "Menominee", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54150", "FIPS": "55078", "County": "Menominee", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "53110", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53129", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53130", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53132", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53154", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53172", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53201", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53202", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53203", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53204", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53205", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53206", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53207", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53208", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53209", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53210", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53211", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53212", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53213", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53214", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53215", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53216", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53217", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53218", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53219", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53220", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53221", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53222", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53223", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53224", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53225", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53226", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53227", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53228", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53233", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53234", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53235", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53237", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53244", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53259", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53263", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53267", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53268", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53274", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53278", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53288", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53290", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53293", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53295", "FIPS": "55079", "County": "Milwaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "54619", "FIPS": "55081", "County": "Monroe", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54620", "FIPS": "55081", "County": "Monroe", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54638", "FIPS": "55081", "County": "Monroe", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54648", "FIPS": "55081", "County": "Monroe", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54649", "FIPS": "55081", "County": "Monroe", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54656", "FIPS": "55081", "County": "Monroe", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54660", "FIPS": "55081", "County": "Monroe", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54662", "FIPS": "55081", "County": "Monroe", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54666", "FIPS": "55081", "County": "Monroe", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54670", "FIPS": "55081", "County": "Monroe", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54101", "FIPS": "55083", "County": "Oconto", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54124", "FIPS": "55083", "County": "Oconto", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54138", "FIPS": "55083", "County": "Oconto", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54139", "FIPS": "55083", "County": "Oconto", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54141", "FIPS": "55083", "County": "Oconto", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54149", "FIPS": "55083", "County": "Oconto", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54153", "FIPS": "55083", "County": "Oconto", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54154", "FIPS": "55083", "County": "Oconto", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54171", "FIPS": "55083", "County": "Oconto", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54174", "FIPS": "55083", "County": "Oconto", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54175", "FIPS": "55083", "County": "Oconto", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54463", "FIPS": "55085", "County": "Oneida", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54501", "FIPS": "55085", "County": "Oneida", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54529", "FIPS": "55085", "County": "Oneida", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54531", "FIPS": "55085", "County": "Oneida", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54539", "FIPS": "55085", "County": "Oneida", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54543", "FIPS": "55085", "County": "Oneida", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54548", "FIPS": "55085", "County": "Oneida", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54562", "FIPS": "55085", "County": "Oneida", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54564", "FIPS": "55085", "County": "Oneida", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54568", "FIPS": "55085", "County": "Oneida", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54106", "FIPS": "55087", "County": "Outagamie", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54113", "FIPS": "55087", "County": "Outagamie", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54130", "FIPS": "55087", "County": "Outagamie", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54131", "FIPS": "55087", "County": "Outagamie", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54136", "FIPS": "55087", "County": "Outagamie", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54140", "FIPS": "55087", "County": "Outagamie", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54152", "FIPS": "55087", "County": "Outagamie", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54165", "FIPS": "55087", "County": "Outagamie", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54170", "FIPS": "55087", "County": "Outagamie", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54911", "FIPS": "55087", "County": "Outagamie", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54912", "FIPS": "55087", "County": "Outagamie", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54913", "FIPS": "55087", "County": "Outagamie", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54914", "FIPS": "55087", "County": "Outagamie", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54915", "FIPS": "55087", "County": "Outagamie", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54919", "FIPS": "55087", "County": "Outagamie", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54922", "FIPS": "55087", "County": "Outagamie", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54931", "FIPS": "55087", "County": "Outagamie", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54942", "FIPS": "55087", "County": "Outagamie", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54944", "FIPS": "55087", "County": "Outagamie", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "53004", "FIPS": "55089", "County": "Ozaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53012", "FIPS": "55089", "County": "Ozaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53021", "FIPS": "55089", "County": "Ozaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53024", "FIPS": "55089", "County": "Ozaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53074", "FIPS": "55089", "County": "Ozaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53080", "FIPS": "55089", "County": "Ozaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53092", "FIPS": "55089", "County": "Ozaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53097", "FIPS": "55089", "County": "Ozaukee", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "54721", "FIPS": "55091", "County": "Pepin", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54736", "FIPS": "55091", "County": "Pepin", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54759", "FIPS": "55091", "County": "Pepin", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54769", "FIPS": "55091", "County": "Pepin", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54003", "FIPS": "55093", "County": "Pierce", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54010", "FIPS": "55093", "County": "Pierce", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54011", "FIPS": "55093", "County": "Pierce", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54014", "FIPS": "55093", "County": "Pierce", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54021", "FIPS": "55093", "County": "Pierce", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54022", "FIPS": "55093", "County": "Pierce", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54723", "FIPS": "55093", "County": "Pierce", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54740", "FIPS": "55093", "County": "Pierce", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54750", "FIPS": "55093", "County": "Pierce", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54761", "FIPS": "55093", "County": "Pierce", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54767", "FIPS": "55093", "County": "Pierce", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54001", "FIPS": "55095", "County": "Polk", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54004", "FIPS": "55095", "County": "Polk", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54005", "FIPS": "55095", "County": "Polk", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54006", "FIPS": "55095", "County": "Polk", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54009", "FIPS": "55095", "County": "Polk", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54020", "FIPS": "55095", "County": "Polk", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54024", "FIPS": "55095", "County": "Polk", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54026", "FIPS": "55095", "County": "Polk", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54810", "FIPS": "55095", "County": "Polk", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54824", "FIPS": "55095", "County": "Polk", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54837", "FIPS": "55095", "County": "Polk", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54853", "FIPS": "55095", "County": "Polk", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54858", "FIPS": "55095", "County": "Polk", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54406", "FIPS": "55097", "County": "Portage", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54407", "FIPS": "55097", "County": "Portage", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54423", "FIPS": "55097", "County": "Portage", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54443", "FIPS": "55097", "County": "Portage", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54458", "FIPS": "55097", "County": "Portage", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54467", "FIPS": "55097", "County": "Portage", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54473", "FIPS": "55097", "County": "Portage", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54481", "FIPS": "55097", "County": "Portage", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54492", "FIPS": "55097", "County": "Portage", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54909", "FIPS": "55097", "County": "Portage", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54921", "FIPS": "55097", "County": "Portage", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54459", "FIPS": "55099", "County": "Price", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54513", "FIPS": "55099", "County": "Price", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54515", "FIPS": "55099", "County": "Price", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54524", "FIPS": "55099", "County": "Price", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54537", "FIPS": "55099", "County": "Price", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54552", "FIPS": "55099", "County": "Price", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54555", "FIPS": "55099", "County": "Price", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54556", "FIPS": "55099", "County": "Price", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "53105", "FIPS": "55101", "County": "Racine", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53108", "FIPS": "55101", "County": "Racine", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53126", "FIPS": "55101", "County": "Racine", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53139", "FIPS": "55101", "County": "Racine", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53167", "FIPS": "55101", "County": "Racine", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53177", "FIPS": "55101", "County": "Racine", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53182", "FIPS": "55101", "County": "Racine", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53185", "FIPS": "55101", "County": "Racine", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53401", "FIPS": "55101", "County": "Racine", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53402", "FIPS": "55101", "County": "Racine", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53403", "FIPS": "55101", "County": "Racine", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53404", "FIPS": "55101", "County": "Racine", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53405", "FIPS": "55101", "County": "Racine", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53406", "FIPS": "55101", "County": "Racine", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53407", "FIPS": "55101", "County": "Racine", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53408", "FIPS": "55101", "County": "Racine", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53490", "FIPS": "55101", "County": "Racine", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53540", "FIPS": "55103", "County": "Richland", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53556", "FIPS": "55103", "County": "Richland", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53581", "FIPS": "55103", "County": "Richland", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53584", "FIPS": "55103", "County": "Richland", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53924", "FIPS": "55103", "County": "Richland", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53501", "FIPS": "55105", "County": "Rock", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53505", "FIPS": "55105", "County": "Rock", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53511", "FIPS": "55105", "County": "Rock", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53512", "FIPS": "55105", "County": "Rock", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53525", "FIPS": "55105", "County": "Rock", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53534", "FIPS": "55105", "County": "Rock", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53536", "FIPS": "55105", "County": "Rock", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53537", "FIPS": "55105", "County": "Rock", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53542", "FIPS": "55105", "County": "Rock", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53545", "FIPS": "55105", "County": "Rock", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53546", "FIPS": "55105", "County": "Rock", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53547", "FIPS": "55105", "County": "Rock", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53548", "FIPS": "55105", "County": "Rock", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53563", "FIPS": "55105", "County": "Rock", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53576", "FIPS": "55105", "County": "Rock", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "54526", "FIPS": "55107", "County": "Rusk", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54530", "FIPS": "55107", "County": "Rusk", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54563", "FIPS": "55107", "County": "Rusk", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54731", "FIPS": "55107", "County": "Rusk", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54766", "FIPS": "55107", "County": "Rusk", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54819", "FIPS": "55107", "County": "Rusk", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54848", "FIPS": "55107", "County": "Rusk", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54895", "FIPS": "55107", "County": "Rusk", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54002", "FIPS": "55109", "County": "Saint Croix", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54007", "FIPS": "55109", "County": "Saint Croix", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54013", "FIPS": "55109", "County": "Saint Croix", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54015", "FIPS": "55109", "County": "Saint Croix", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54016", "FIPS": "55109", "County": "Saint Croix", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54017", "FIPS": "55109", "County": "Saint Croix", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54023", "FIPS": "55109", "County": "Saint Croix", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54025", "FIPS": "55109", "County": "Saint Croix", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54027", "FIPS": "55109", "County": "Saint Croix", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54028", "FIPS": "55109", "County": "Saint Croix", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54082", "FIPS": "55109", "County": "Saint Croix", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "53561", "FIPS": "55111", "County": "Sauk", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53577", "FIPS": "55111", "County": "Sauk", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53578", "FIPS": "55111", "County": "Sauk", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53583", "FIPS": "55111", "County": "Sauk", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53588", "FIPS": "55111", "County": "Sauk", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53913", "FIPS": "55111", "County": "Sauk", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53937", "FIPS": "55111", "County": "Sauk", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53940", "FIPS": "55111", "County": "Sauk", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53941", "FIPS": "55111", "County": "Sauk", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53942", "FIPS": "55111", "County": "Sauk", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53943", "FIPS": "55111", "County": "Sauk", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53951", "FIPS": "55111", "County": "Sauk", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53958", "FIPS": "55111", "County": "Sauk", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53959", "FIPS": "55111", "County": "Sauk", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "53961", "FIPS": "55111", "County": "Sauk", "State": "WI", "DMA": {"Code": 669, "Name": "MADISON"}},
  {"Postal_Code": "54828", "FIPS": "55113", "County": "Sawyer", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54834", "FIPS": "55113", "County": "Sawyer", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54835", "FIPS": "55113", "County": "Sawyer", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54843", "FIPS": "55113", "County": "Sawyer", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54862", "FIPS": "55113", "County": "Sawyer", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54867", "FIPS": "55113", "County": "Sawyer", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54876", "FIPS": "55113", "County": "Sawyer", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54896", "FIPS": "55113", "County": "Sawyer", "State": "WI", "DMA": {"Code": 676, "Name": "DULUTH-SUPERIOR"}},
  {"Postal_Code": "54107", "FIPS": "55115", "County": "Shawano", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54111", "FIPS": "55115", "County": "Shawano", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54127", "FIPS": "55115", "County": "Shawano", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54128", "FIPS": "55115", "County": "Shawano", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54137", "FIPS": "55115", "County": "Shawano", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54166", "FIPS": "55115", "County": "Shawano", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54182", "FIPS": "55115", "County": "Shawano", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54414", "FIPS": "55115", "County": "Shawano", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54416", "FIPS": "55115", "County": "Shawano", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54450", "FIPS": "55115", "County": "Shawano", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54486", "FIPS": "55115", "County": "Shawano", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54499", "FIPS": "55115", "County": "Shawano", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54928", "FIPS": "55115", "County": "Shawano", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54948", "FIPS": "55115", "County": "Shawano", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54978", "FIPS": "55115", "County": "Shawano", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "53001", "FIPS": "55117", "County": "Sheboygan", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53011", "FIPS": "55117", "County": "Sheboygan", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53013", "FIPS": "55117", "County": "Sheboygan", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53020", "FIPS": "55117", "County": "Sheboygan", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53023", "FIPS": "55117", "County": "Sheboygan", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53026", "FIPS": "55117", "County": "Sheboygan", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53031", "FIPS": "55117", "County": "Sheboygan", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53044", "FIPS": "55117", "County": "Sheboygan", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53070", "FIPS": "55117", "County": "Sheboygan", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53073", "FIPS": "55117", "County": "Sheboygan", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53075", "FIPS": "55117", "County": "Sheboygan", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53081", "FIPS": "55117", "County": "Sheboygan", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53082", "FIPS": "55117", "County": "Sheboygan", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53083", "FIPS": "55117", "County": "Sheboygan", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53085", "FIPS": "55117", "County": "Sheboygan", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53093", "FIPS": "55117", "County": "Sheboygan", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "54433", "FIPS": "55119", "County": "Taylor", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54434", "FIPS": "55119", "County": "Taylor", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54439", "FIPS": "55119", "County": "Taylor", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54447", "FIPS": "55119", "County": "Taylor", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54451", "FIPS": "55119", "County": "Taylor", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54470", "FIPS": "55119", "County": "Taylor", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54480", "FIPS": "55119", "County": "Taylor", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54490", "FIPS": "55119", "County": "Taylor", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54612", "FIPS": "55121", "County": "Trempealeau", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54616", "FIPS": "55121", "County": "Trempealeau", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54625", "FIPS": "55121", "County": "Trempealeau", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54627", "FIPS": "55121", "County": "Trempealeau", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54630", "FIPS": "55121", "County": "Trempealeau", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54661", "FIPS": "55121", "County": "Trempealeau", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54738", "FIPS": "55121", "County": "Trempealeau", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54747", "FIPS": "55121", "County": "Trempealeau", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54758", "FIPS": "55121", "County": "Trempealeau", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54760", "FIPS": "55121", "County": "Trempealeau", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54770", "FIPS": "55121", "County": "Trempealeau", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54773", "FIPS": "55121", "County": "Trempealeau", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54621", "FIPS": "55123", "County": "Vernon", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54623", "FIPS": "55123", "County": "Vernon", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54624", "FIPS": "55123", "County": "Vernon", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54632", "FIPS": "55123", "County": "Vernon", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54634", "FIPS": "55123", "County": "Vernon", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54639", "FIPS": "55123", "County": "Vernon", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54651", "FIPS": "55123", "County": "Vernon", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54652", "FIPS": "55123", "County": "Vernon", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54658", "FIPS": "55123", "County": "Vernon", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54664", "FIPS": "55123", "County": "Vernon", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54665", "FIPS": "55123", "County": "Vernon", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54667", "FIPS": "55123", "County": "Vernon", "State": "WI", "DMA": {"Code": 702, "Name": "LA CROSSE-EAU CLAIRE"}},
  {"Postal_Code": "54512", "FIPS": "55125", "County": "Vilas", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54519", "FIPS": "55125", "County": "Vilas", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54521", "FIPS": "55125", "County": "Vilas", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54538", "FIPS": "55125", "County": "Vilas", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54540", "FIPS": "55125", "County": "Vilas", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54545", "FIPS": "55125", "County": "Vilas", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54554", "FIPS": "55125", "County": "Vilas", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54557", "FIPS": "55125", "County": "Vilas", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54558", "FIPS": "55125", "County": "Vilas", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54560", "FIPS": "55125", "County": "Vilas", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54561", "FIPS": "55125", "County": "Vilas", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "53114", "FIPS": "55127", "County": "Walworth", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53115", "FIPS": "55127", "County": "Walworth", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53120", "FIPS": "55127", "County": "Walworth", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53121", "FIPS": "55127", "County": "Walworth", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53125", "FIPS": "55127", "County": "Walworth", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53128", "FIPS": "55127", "County": "Walworth", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53138", "FIPS": "55127", "County": "Walworth", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53147", "FIPS": "55127", "County": "Walworth", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53148", "FIPS": "55127", "County": "Walworth", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53157", "FIPS": "55127", "County": "Walworth", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53176", "FIPS": "55127", "County": "Walworth", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53184", "FIPS": "55127", "County": "Walworth", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53190", "FIPS": "55127", "County": "Walworth", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53191", "FIPS": "55127", "County": "Walworth", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53195", "FIPS": "55127", "County": "Walworth", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53585", "FIPS": "55127", "County": "Walworth", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "54801", "FIPS": "55129", "County": "Washburn", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54817", "FIPS": "55129", "County": "Washburn", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54859", "FIPS": "55129", "County": "Washburn", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54870", "FIPS": "55129", "County": "Washburn", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54871", "FIPS": "55129", "County": "Washburn", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54875", "FIPS": "55129", "County": "Washburn", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "54888", "FIPS": "55129", "County": "Washburn", "State": "WI", "DMA": {"Code": 613, "Name": "MINNEAPOLIS-SAINT PAUL"}},
  {"Postal_Code": "53002", "FIPS": "55131", "County": "Washington", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53017", "FIPS": "55131", "County": "Washington", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53022", "FIPS": "55131", "County": "Washington", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53027", "FIPS": "55131", "County": "Washington", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53033", "FIPS": "55131", "County": "Washington", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53037", "FIPS": "55131", "County": "Washington", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53040", "FIPS": "55131", "County": "Washington", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53060", "FIPS": "55131", "County": "Washington", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53076", "FIPS": "55131", "County": "Washington", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53086", "FIPS": "55131", "County": "Washington", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53090", "FIPS": "55131", "County": "Washington", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53095", "FIPS": "55131", "County": "Washington", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53005", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53007", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53008", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53018", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53029", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53045", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53046", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53051", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53052", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53056", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53058", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53064", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53066", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53069", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53072", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53089", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53103", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53118", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53119", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53122", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53127", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53146", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53149", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53150", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53151", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53153", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53183", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53186", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53187", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53188", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "53189", "FIPS": "55133", "County": "Waukesha", "State": "WI", "DMA": {"Code": 617, "Name": "MILWAUKEE"}},
  {"Postal_Code": "54926", "FIPS": "55135", "County": "Waupaca", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54929", "FIPS": "55135", "County": "Waupaca", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54933", "FIPS": "55135", "County": "Waupaca", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54940", "FIPS": "55135", "County": "Waupaca", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54945", "FIPS": "55135", "County": "Waupaca", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54946", "FIPS": "55135", "County": "Waupaca", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54949", "FIPS": "55135", "County": "Waupaca", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54950", "FIPS": "55135", "County": "Waupaca", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54961", "FIPS": "55135", "County": "Waupaca", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54962", "FIPS": "55135", "County": "Waupaca", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54969", "FIPS": "55135", "County": "Waupaca", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54977", "FIPS": "55135", "County": "Waupaca", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54981", "FIPS": "55135", "County": "Waupaca", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54983", "FIPS": "55135", "County": "Waupaca", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54990", "FIPS": "55135", "County": "Waupaca", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54930", "FIPS": "55137", "County": "Waushara", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54943", "FIPS": "55137", "County": "Waushara", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54965", "FIPS": "55137", "County": "Waushara", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54966", "FIPS": "55137", "County": "Waushara", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54967", "FIPS": "55137", "County": "Waushara", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54970", "FIPS": "55137", "County": "Waushara", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54976", "FIPS": "55137", "County": "Waushara", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54982", "FIPS": "55137", "County": "Waushara", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54984", "FIPS": "55137", "County": "Waushara", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54901", "FIPS": "55139", "County": "Winnebago", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54902", "FIPS": "55139", "County": "Winnebago", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54903", "FIPS": "55139", "County": "Winnebago", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54904", "FIPS": "55139", "County": "Winnebago", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54906", "FIPS": "55139", "County": "Winnebago", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54927", "FIPS": "55139", "County": "Winnebago", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54934", "FIPS": "55139", "County": "Winnebago", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54947", "FIPS": "55139", "County": "Winnebago", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54952", "FIPS": "55139", "County": "Winnebago", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54956", "FIPS": "55139", "County": "Winnebago", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54957", "FIPS": "55139", "County": "Winnebago", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54963", "FIPS": "55139", "County": "Winnebago", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54964", "FIPS": "55139", "County": "Winnebago", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54980", "FIPS": "55139", "County": "Winnebago", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54985", "FIPS": "55139", "County": "Winnebago", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54986", "FIPS": "55139", "County": "Winnebago", "State": "WI", "DMA": {"Code": 658, "Name": "GREEN BAY-APPLETON"}},
  {"Postal_Code": "54404", "FIPS": "55141", "County": "Wood", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54410", "FIPS": "55141", "County": "Wood", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54412", "FIPS": "55141", "County": "Wood", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54413", "FIPS": "55141", "County": "Wood", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54415", "FIPS": "55141", "County": "Wood", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54441", "FIPS": "55141", "County": "Wood", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54449", "FIPS": "55141", "County": "Wood", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54454", "FIPS": "55141", "County": "Wood", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54457", "FIPS": "55141", "County": "Wood", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54466", "FIPS": "55141", "County": "Wood", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54469", "FIPS": "55141", "County": "Wood", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54472", "FIPS": "55141", "County": "Wood", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54475", "FIPS": "55141", "County": "Wood", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54489", "FIPS": "55141", "County": "Wood", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54494", "FIPS": "55141", "County": "Wood", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "54495", "FIPS": "55141", "County": "Wood", "State": "WI", "DMA": {"Code": 705, "Name": "WAUSAU-RHINELANDER"}},
  {"Postal_Code": "82051", "FIPS": "56001", "County": "Albany", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82052", "FIPS": "56001", "County": "Albany", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82055", "FIPS": "56001", "County": "Albany", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82058", "FIPS": "56001", "County": "Albany", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82063", "FIPS": "56001", "County": "Albany", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82070", "FIPS": "56001", "County": "Albany", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82071", "FIPS": "56001", "County": "Albany", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82072", "FIPS": "56001", "County": "Albany", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82073", "FIPS": "56001", "County": "Albany", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82083", "FIPS": "56001", "County": "Albany", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82084", "FIPS": "56001", "County": "Albany", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82410", "FIPS": "56003", "County": "Big Horn", "State": "WY", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "82411", "FIPS": "56003", "County": "Big Horn", "State": "WY", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "82412", "FIPS": "56003", "County": "Big Horn", "State": "WY", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "82420", "FIPS": "56003", "County": "Big Horn", "State": "WY", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "82421", "FIPS": "56003", "County": "Big Horn", "State": "WY", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "82422", "FIPS": "56003", "County": "Big Horn", "State": "WY", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "82426", "FIPS": "56003", "County": "Big Horn", "State": "WY", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "82428", "FIPS": "56003", "County": "Big Horn", "State": "WY", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "82431", "FIPS": "56003", "County": "Big Horn", "State": "WY", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "82432", "FIPS": "56003", "County": "Big Horn", "State": "WY", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "82434", "FIPS": "56003", "County": "Big Horn", "State": "WY", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "82441", "FIPS": "56003", "County": "Big Horn", "State": "WY", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "82716", "FIPS": "56005", "County": "Campbell", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82717", "FIPS": "56005", "County": "Campbell", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82718", "FIPS": "56005", "County": "Campbell", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82725", "FIPS": "56005", "County": "Campbell", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82727", "FIPS": "56005", "County": "Campbell", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82731", "FIPS": "56005", "County": "Campbell", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82732", "FIPS": "56005", "County": "Campbell", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82301", "FIPS": "56007", "County": "Carbon", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82321", "FIPS": "56007", "County": "Carbon", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82323", "FIPS": "56007", "County": "Carbon", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82324", "FIPS": "56007", "County": "Carbon", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82325", "FIPS": "56007", "County": "Carbon", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82327", "FIPS": "56007", "County": "Carbon", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82329", "FIPS": "56007", "County": "Carbon", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82331", "FIPS": "56007", "County": "Carbon", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82332", "FIPS": "56007", "County": "Carbon", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82334", "FIPS": "56007", "County": "Carbon", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82335", "FIPS": "56007", "County": "Carbon", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82615", "FIPS": "56007", "County": "Carbon", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82224", "FIPS": "56009", "County": "Converse", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82229", "FIPS": "56009", "County": "Converse", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82633", "FIPS": "56009", "County": "Converse", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82637", "FIPS": "56009", "County": "Converse", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82710", "FIPS": "56011", "County": "Crook", "State": "WY", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "82711", "FIPS": "56011", "County": "Crook", "State": "WY", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "82712", "FIPS": "56011", "County": "Crook", "State": "WY", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "82714", "FIPS": "56011", "County": "Crook", "State": "WY", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "82720", "FIPS": "56011", "County": "Crook", "State": "WY", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "82721", "FIPS": "56011", "County": "Crook", "State": "WY", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "82729", "FIPS": "56011", "County": "Crook", "State": "WY", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "82310", "FIPS": "56013", "County": "Fremont", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82501", "FIPS": "56013", "County": "Fremont", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82510", "FIPS": "56013", "County": "Fremont", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82512", "FIPS": "56013", "County": "Fremont", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82513", "FIPS": "56013", "County": "Fremont", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82514", "FIPS": "56013", "County": "Fremont", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82515", "FIPS": "56013", "County": "Fremont", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82516", "FIPS": "56013", "County": "Fremont", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82520", "FIPS": "56013", "County": "Fremont", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82523", "FIPS": "56013", "County": "Fremont", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82524", "FIPS": "56013", "County": "Fremont", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82642", "FIPS": "56013", "County": "Fremont", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82649", "FIPS": "56013", "County": "Fremont", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82212", "FIPS": "56015", "County": "Goshen", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82217", "FIPS": "56015", "County": "Goshen", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82218", "FIPS": "56015", "County": "Goshen", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82219", "FIPS": "56015", "County": "Goshen", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82221", "FIPS": "56015", "County": "Goshen", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82223", "FIPS": "56015", "County": "Goshen", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82240", "FIPS": "56015", "County": "Goshen", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82243", "FIPS": "56015", "County": "Goshen", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82244", "FIPS": "56015", "County": "Goshen", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82430", "FIPS": "56017", "County": "Hot Springs", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82443", "FIPS": "56017", "County": "Hot Springs", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82639", "FIPS": "56019", "County": "Johnson", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82640", "FIPS": "56019", "County": "Johnson", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82834", "FIPS": "56019", "County": "Johnson", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82840", "FIPS": "56019", "County": "Johnson", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82001", "FIPS": "56021", "County": "Laramie", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82002", "FIPS": "56021", "County": "Laramie", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82003", "FIPS": "56021", "County": "Laramie", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82005", "FIPS": "56021", "County": "Laramie", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82006", "FIPS": "56021", "County": "Laramie", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82007", "FIPS": "56021", "County": "Laramie", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82008", "FIPS": "56021", "County": "Laramie", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82009", "FIPS": "56021", "County": "Laramie", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82010", "FIPS": "56021", "County": "Laramie", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82050", "FIPS": "56021", "County": "Laramie", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82053", "FIPS": "56021", "County": "Laramie", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82054", "FIPS": "56021", "County": "Laramie", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82059", "FIPS": "56021", "County": "Laramie", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82060", "FIPS": "56021", "County": "Laramie", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82061", "FIPS": "56021", "County": "Laramie", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82081", "FIPS": "56021", "County": "Laramie", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "82082", "FIPS": "56021", "County": "Laramie", "State": "WY", "DMA": {"Code": 759, "Name": "CHEYENNE-SCOTTSBLUFF"}},
  {"Postal_Code": "83101", "FIPS": "56023", "County": "Lincoln", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83110", "FIPS": "56023", "County": "Lincoln", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83111", "FIPS": "56023", "County": "Lincoln", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83112", "FIPS": "56023", "County": "Lincoln", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83114", "FIPS": "56023", "County": "Lincoln", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83116", "FIPS": "56023", "County": "Lincoln", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83118", "FIPS": "56023", "County": "Lincoln", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83119", "FIPS": "56023", "County": "Lincoln", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83120", "FIPS": "56023", "County": "Lincoln", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83121", "FIPS": "56023", "County": "Lincoln", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83122", "FIPS": "56023", "County": "Lincoln", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83123", "FIPS": "56023", "County": "Lincoln", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83124", "FIPS": "56023", "County": "Lincoln", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83126", "FIPS": "56023", "County": "Lincoln", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83127", "FIPS": "56023", "County": "Lincoln", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83128", "FIPS": "56023", "County": "Lincoln", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "82601", "FIPS": "56025", "County": "Natrona", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82602", "FIPS": "56025", "County": "Natrona", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82604", "FIPS": "56025", "County": "Natrona", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82605", "FIPS": "56025", "County": "Natrona", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82609", "FIPS": "56025", "County": "Natrona", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82620", "FIPS": "56025", "County": "Natrona", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82630", "FIPS": "56025", "County": "Natrona", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82635", "FIPS": "56025", "County": "Natrona", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82636", "FIPS": "56025", "County": "Natrona", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82638", "FIPS": "56025", "County": "Natrona", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82643", "FIPS": "56025", "County": "Natrona", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82644", "FIPS": "56025", "County": "Natrona", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82646", "FIPS": "56025", "County": "Natrona", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82648", "FIPS": "56025", "County": "Natrona", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82222", "FIPS": "56027", "County": "Niobrara", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82225", "FIPS": "56027", "County": "Niobrara", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82227", "FIPS": "56027", "County": "Niobrara", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82242", "FIPS": "56027", "County": "Niobrara", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82190", "FIPS": "56029", "County": "Park", "State": "WY", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "82414", "FIPS": "56029", "County": "Park", "State": "WY", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "82423", "FIPS": "56029", "County": "Park", "State": "WY", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "82433", "FIPS": "56029", "County": "Park", "State": "WY", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "82435", "FIPS": "56029", "County": "Park", "State": "WY", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "82440", "FIPS": "56029", "County": "Park", "State": "WY", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "82450", "FIPS": "56029", "County": "Park", "State": "WY", "DMA": {"Code": 756, "Name": "BILLINGS"}},
  {"Postal_Code": "82201", "FIPS": "56031", "County": "Platte", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82210", "FIPS": "56031", "County": "Platte", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82213", "FIPS": "56031", "County": "Platte", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82214", "FIPS": "56031", "County": "Platte", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82215", "FIPS": "56031", "County": "Platte", "State": "WY", "DMA": {"Code": 751, "Name": "DENVER"}},
  {"Postal_Code": "82801", "FIPS": "56033", "County": "Sheridan", "State": "WY", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "82831", "FIPS": "56033", "County": "Sheridan", "State": "WY", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "82832", "FIPS": "56033", "County": "Sheridan", "State": "WY", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "82833", "FIPS": "56033", "County": "Sheridan", "State": "WY", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "82835", "FIPS": "56033", "County": "Sheridan", "State": "WY", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "82836", "FIPS": "56033", "County": "Sheridan", "State": "WY", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "82837", "FIPS": "56033", "County": "Sheridan", "State": "WY", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "82838", "FIPS": "56033", "County": "Sheridan", "State": "WY", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "82839", "FIPS": "56033", "County": "Sheridan", "State": "WY", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "82842", "FIPS": "56033", "County": "Sheridan", "State": "WY", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "82844", "FIPS": "56033", "County": "Sheridan", "State": "WY", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "82845", "FIPS": "56033", "County": "Sheridan", "State": "WY", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "82922", "FIPS": "56035", "County": "Sublette", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "82923", "FIPS": "56035", "County": "Sublette", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "82925", "FIPS": "56035", "County": "Sublette", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "82941", "FIPS": "56035", "County": "Sublette", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83113", "FIPS": "56035", "County": "Sublette", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83115", "FIPS": "56035", "County": "Sublette", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "82322", "FIPS": "56037", "County": "Sweetwater", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "82336", "FIPS": "56037", "County": "Sweetwater", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "82901", "FIPS": "56037", "County": "Sweetwater", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "82902", "FIPS": "56037", "County": "Sweetwater", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "82929", "FIPS": "56037", "County": "Sweetwater", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "82932", "FIPS": "56037", "County": "Sweetwater", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "82934", "FIPS": "56037", "County": "Sweetwater", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "82935", "FIPS": "56037", "County": "Sweetwater", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "82938", "FIPS": "56037", "County": "Sweetwater", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "82942", "FIPS": "56037", "County": "Sweetwater", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "82943", "FIPS": "56037", "County": "Sweetwater", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "82945", "FIPS": "56037", "County": "Sweetwater", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "83001", "FIPS": "56039", "County": "Teton", "State": "WY", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83002", "FIPS": "56039", "County": "Teton", "State": "WY", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83011", "FIPS": "56039", "County": "Teton", "State": "WY", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83012", "FIPS": "56039", "County": "Teton", "State": "WY", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83013", "FIPS": "56039", "County": "Teton", "State": "WY", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83014", "FIPS": "56039", "County": "Teton", "State": "WY", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83025", "FIPS": "56039", "County": "Teton", "State": "WY", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "83414", "FIPS": "56039", "County": "Teton", "State": "WY", "DMA": {"Code": 758, "Name": "IDAHO FALLS-POCATELLO"}},
  {"Postal_Code": "82930", "FIPS": "56041", "County": "Uinta", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "82931", "FIPS": "56041", "County": "Uinta", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "82933", "FIPS": "56041", "County": "Uinta", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "82936", "FIPS": "56041", "County": "Uinta", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "82937", "FIPS": "56041", "County": "Uinta", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "82939", "FIPS": "56041", "County": "Uinta", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "82944", "FIPS": "56041", "County": "Uinta", "State": "WY", "DMA": {"Code": 770, "Name": "SALT LAKE CITY"}},
  {"Postal_Code": "82401", "FIPS": "56043", "County": "Washakie", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82442", "FIPS": "56043", "County": "Washakie", "State": "WY", "DMA": {"Code": 767, "Name": "CASPER-RIVERTON"}},
  {"Postal_Code": "82701", "FIPS": "56045", "County": "Weston", "State": "WY", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "82715", "FIPS": "56045", "County": "Weston", "State": "WY", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "82723", "FIPS": "56045", "County": "Weston", "State": "WY", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
  {"Postal_Code": "82730", "FIPS": "56045", "County": "Weston", "State": "WY", "DMA": {"Code": 764, "Name": "RAPID CITY"}},
]
